#!/usr/local/bin/perl

# PURPOSE: This script reads and analyzes the IsoFax
#	   accounting_log, and prints the information to standard
#	   output.  If desired, the information may be redirected
#	   at the command line to a seperate file. 
#
# SYNTAX:  Analyze.pl accounting_log ddmmyyyy ddmmyyyy {-s, -t, -n}
#
# USAGE:
#	   "Analyze.pl" is this script name, and "accounting_log"
#	   is the name of the accounting log.  Two "ddmmyyyy"s are
#	   the staring date and ending date, respectively.  The
#	   date should consist of NUMBERS ONLY, and no metacharacters.
#
#	   The arguments in the curly braces are optional.
#	   Select only one:
#		-s	This sorts the output by sender.
#		-t	This does the same as, but prints total
#			pages and total duration(secs).
#		-n	This sorts the output by phone number.
#	   If these options are ommited, by default, the output
#	   is sorted by date.
#
#	NOTE TO USER:
#	 The names of the months in following line (@months) may need
#	 to be modified depending on the language that your operating
#	 system is set to. Make sure that all of the quotation marks
# 	 are still there though!

	@months = ("Jan","Feb","Mar","Apr","May","Jun",
	   "Jul","Aug","Sep","Oct","Nov","Dec");

################################################################################
#			THE CODE: (drumroll please...)

&file_name();

&select_format($ARGV[3]);

system("rm ${path}/tempsort") || die "\n";


##########################################################################
############################   F O R M A T S   ###########################
##########################################################################
##################
format S_TOP = 
		  ***** A C C O U N T I N G   L O G *****
   Sender         Date     Time    Number Dialed   Pgs  Sec    CSI      Host
------------  -----------  -----  ---------------  --- ----- --------  -------
.
format S_MAIN =
@<<<<<<<<<<<  @||||||||||  @<<<<  @<<<<<<<<<<<<<< @||| @|||| @<<<<<<<  @<<<<<<
$email,$date,$time,$phone,$pages,$duration,$csi,$host
.
format S_SEC = 
		      Totals for @|||||||||||:    @||| @||||
$lastusr, $totpages, $totduration

.
format SEC_TOP =
.
##################
format N_TOP = 
		  ***** A C C O U N T I N G   L O G *****
 Number Dialed      Date      Time   Pgs  Sec    CSI       Sender       Host
---------------  -----------  -----  --- ----- --------  ------------  -------
.
format N_MAIN =
@<<<<<<<<<<<<<<  @||||||||||  @<<<<  @|| @|||| @<<<<<<<  @<<<<<<<<<<<  @<<<<<<
$phone,$date,$time,$pages,$duration,$csi,$email,$host
.
##################
format OTHR_TOP = 
		  ***** A C C O U N T I N G   L O G *****
    Date     Time   Number Dialed   Pgs  Sec    CSI       Sender       Host
-----------  -----  --------------- --- ----- --------  ------------  -------
.
format OTHR_MAIN =
@||||||||||  @<<<<  @<<<<<<<<<<<<<< @|| @|||| @<<<<<<<  @<<<<<<<<<<<  @<<<<<<
$date,$time,$phone,$pages,$duration,$csi,$email,$host
.
##################
##########################################################################
#######################    S U B - R O U T I N E S    ####################
##########################################################################
# Selects which format to write with
sub select_format
{
	if ((@_[0] eq "-s") || (@_[0] eq "-S"))
	{
		select(STDOUT);
		$^ = 'S_TOP';
		$~ = 'S_MAIN';
		&parse_s(); 
	}
	elsif ((@_[0] eq "-t") || (@_[0] eq "-T"))
	{
		select(STDOUT);
		$^ = 'S_TOP';
		$~ = 'S_MAIN';
		&parse_s_tot(); 
	}
	elsif ((@_[0] eq "-n") || (@_[0] eq "-N"))
	{
		select(STDOUT);
		$^ = 'N_TOP';
		$~ = 'N_MAIN';
		&parse_n();
	}
	else
	{
		select(STDOUT);
		$^ = 'OTHR_TOP';
		$~ = 'OTHR_MAIN';
		&parse_d();
	}
}
##########################################################################
# 
sub sort_s
{
	$n_line = 0;
	while (<>)
	{
		$trash = unpack"a1",$_;

		if ($trash == " ")
		{
		   $date = unpack"a11",$_;
		   ($trash, $date, $trash) = split(/\s/,$date,3);
		   ($trash,$trash,$time,$uid,$host,$email,$phone,$pages,
$duration,$subj,$recipient,$csi,$billing,$tracking) = split(/\s/);
		}
		else
		{
		   ($date,$time,$uid,$host,$email,$phone,$pages,
$duration,$subj,$recipient,$csi,$billing,$tracking) = split(/\s/);
		}

		#print "$date\n";
		@by_send[$n_line] = join(' ',$email,$date,$time,$uid,$host,
			    $phone,$pages,$duration,$subj,$recipient,$csi,
						      $billing,$tracking);
		$n_line++;
	}
	@by_send = sort(@by_send);

	foreach $x (0..$#by_send)
	{
	  #print "$x\n";
	  print TEMP @by_send[$x],"\n";
	}
}

##########################################################################
# 
sub sort_n
{
	$n_line = 0;
	while (<>)
	{
		$trash = unpack"a1",$_;

		if ($trash == " ")
		{
		   $date = unpack"a11",$_;
		   ($trash, $date, $trash) = split(/\s/,$date,3);
		   ($trash,$trash,$time,$uid,$host,$email,$phone,$pages,
$duration,$subj,$recipient,$csi,$billing,$tracking) = split(/\s/);
		}
		else
		{
		   ($date,$time,$uid,$host,$email,$phone,$pages,
$duration,$subj,$recipient,$csi,$billing,$tracking) = split(/\s/);
		}

		@by_num[$n_line] = join(' ',$phone,$date,$time,$uid,$host,
			    $email,$pages,$duration,$subj,$recipient,$csi,
						      $billing,$tracking);
		$n_line++;
	}

	@by_num = sort (@by_num);

	foreach $x (0..$#by_num)
	{
	  print TEMP @by_num[$x],"\n";
	}
}

##########################################################################
# 
sub parse_s_tot
{
	$totpages = 0;
	$totduration = 0;
	$firstrun = "abc";
	open(TEMP,"+>${path}/tempsort");
	$start = &parsedateI(@ARGV[1]);
	$finish = &parsedateI(@ARGV[2]);
 	@ARGV = splice(@ARGV,0,1);
	&sort_s();
	close(TEMP);
	$ARGV[0] = "${path}/tempsort";

	while (<>)
	{
		($email,$date,$time,$uid,$host,$phone,$pages,$duration,$subj,
			   $recipient,$csi,$billing,$tracking) = split(/\s/);
		($date, $trash) = split(/\s/,$date,2);

		if ($firstrun eq "abc")
		{
		  $lastusr = $email;
		  $firstrun = "";
		}

		$found = parsedateII($date);

		if (($email gt $lastusr) && ($found >= $start) && ($found <= $finish))
		{
		  select(STDOUT);
		  $~ = 'S_SEC';
		  $^ = 'SEC_TOP';
		  write STDOUT;
		  #print "new\n";
		  #print "$totpages\n";
		  #print "$totduration\n";

		  select(STDOUT);
		  $^ = 'S_TOP';
		  $~ = 'S_MAIN';
		  $totpages = 0;
		  $totduration = 0;
		}

		if (($found >= $start) && ($found <= $finish))		
		{		
		  write STDOUT;
		  #print "$lastusr\n";
		  $totpages += $pages;
		  $totduration += $duration;
		  $lastusr = $email
		}		
	}
	select(STDOUT);
	$~ = 'S_SEC';
	$^ = 'SEC_TOP';
	write STDOUT;
}

##########################################################################
# 
sub parse_s
{
	open(TEMP,"+>${path}/tempsort");
	$start = &parsedateI(@ARGV[1]);
	$finish = &parsedateI(@ARGV[2]);
 	@ARGV = splice(@ARGV,0,1);
	&sort_s();
	close(TEMP);
	$ARGV[0] = "${path}/tempsort";
 
	while (<>)
	{
		($email,$date,$time,$uid,$host,$phone,$pages,$duration,$subj,
			   $recipient,$csi,$billing,$tracking) = split(/\s/);
		($date, $trash) = split(/\s/,$date,2);
		#print "$date\n";
		
		$found = parsedateII($date);
		write STDOUT if (($found >= $start) && ($found <= $finish));
	}
}

##########################################################################
# 
sub parse_n
{
	open(TEMP,"+>${path}/tempsort");
	$start = &parsedateI(@ARGV[1]);
	$finish = &parsedateI(@ARGV[2]);
 	@ARGV = splice(@ARGV,0,1);
	&sort_n();
	close(TEMP);
	$ARGV[0] = "${path}/tempsort";
 
	while (<>)
	{
		($phone,$date,$time,$uid,$host,$email,$pages,$duration,$subj,
			   $recipient,$csi,$billing,$tracking) = split(/\s/);
		($date, $trash) = split(/\s/,$date,2);
		#print "$date\n";
		
		$found = parsedateII($date);
		write STDOUT if (($found >= $start) && ($found <= $finish));
	}
}

##########################################################################
# Extracts the file path & base from the file given in @ARGV[0].
sub file_name
{
	$path = dirname(@ARGV[0]);
	$scriptpath = dirname($0);
	$base = basename(@ARGV[0], "");
	return($base, $scriptpath, $path);
}

####################################
# parse through the accounting_log
sub parse_d
{
	$start = &parsedateI(@ARGV[1]);
	$finish = &parsedateI(@ARGV[2]);
 	@ARGV = splice(@ARGV,0,1);
 
	while (<>)
	{
		$trash = unpack"a1",$_;

		if ($trash == " ")
		{
		   # print "a\n";
		   $date = unpack"a11",$_;
		   ($trash, $date, $trash) = split(/\s/,$date,3);
		   ($trash,$trash,$time,$uid,$host,$email,$phone,$pages,
$duration,$subj,$recipient,$csi,$billing,$tracking) = split(/\s/);
		}
		else
		{
		   # print "b\n";
		   ($date,$time,$uid,$host,$email,$phone,$pages,
$duration,$subj,$recipient,$csi,$billing,$tracking) = split(/\s/);
		}
		# print "$date\n";
		
		$found = parsedateII($date);
		if (($found >= $start) && ($found <= $finish))
		{
		  write STDOUT;
		  $exist =  1;
		}

		if ($found > $finish)
		{
		  print "No log entries were found for those dates.\n" if (! $exist);
		  last;
		}
	}
	

}

####################################################
# Converts ddmmyyyy format to seconds since 1/1/70
sub parsedateI
{
	if (length(@_[0]) == 8)
	{
	   ($dd,$mm,$yyyy) = unpack"a2 a2 a4",@_[0];
	   ($yyyy, $yy) = unpack"a2 a2",$yyyy;
	   $mm--;
	   $secdate = timelocal("0","0","0",$dd,$mm,$yy);
	   return($secdate);
	}
	else
	{
	   die("The given date was not valid:I\n");
	}   
}

#########################################################
# Converts dd-Mon-yy(yy) format to seconds since 1/1/70
sub parsedateII
{
	#print "@_[0]\n";
	if ((length(@_[0]) == 11) || (length(@_[0]) == 10))
	{
	   ($dd,$mm,$yyyy) = split(/-/,@_[0]);
	   ($yyyy, $yy) = unpack"a2 a2",$yyyy;
	}
	elsif  ((length(@_[0]) == 9) || (length(@_[0]) == 8))
	{
	   ($dd,$mm,$yy) = split(/-/,@_[0]);
	}
	else
	{
	   die("The accounting log date was not valid:II\n");
	}

	for ($mm) {
	 if	(/@months[11]/) {$mm = 11}
  	 elsif	(/@months[10]/) {$mm = 10}
  	 elsif	(/@months[9]/) {$mm = 9}
   	 elsif	(/@months[8]/) {$mm = 8}
   	 elsif	(/@months[7]/) {$mm = 7}
   	 elsif	(/@months[6]/) {$mm = 6}
  	 elsif	(/@months[5]/) {$mm = 5}
   	 elsif	(/@months[4]/) {$mm = 4}
   	 elsif	(/@months[3]/) {$mm = 3}
   	 elsif	(/@months[2]/) {$mm = 2}
   	 elsif	(/@months[1]/) {$mm = 1}
   	 else		       {$mm = 0}
	}

	$secdate = timelocal("0","0","0", $dd,$mm,$yy);
	return($secdate);
}

################################################################################
#############******************************************************#############
#############* E X T R A    L I B R A R I E S   &   M O D U L E S *#############
#############******************************************************#############
################* These are from packages that came with perl. *################
################************************************************################


	    #####From the Basename.pm module included with perl#####

#   fileparse_set_fstype() - specify OS-based rules used in future
#                            calls to routines in this package
#
#   Currently recognized values: VMS, MSDOS, MacOS
#       Any other name uses Unix-style rules

sub fileparse_set_fstype {
  my($old) = $Fileparse_fstype;
  $Fileparse_fstype = $_[0] if $_[0];
  $old;
}

#   fileparse() - parse file specification
#
#   calling sequence:
#     ($filename,$prefix,$tail) = &basename_pat($filespec,@excludelist);
#     where  $filespec    is the file specification to be parsed, and
#            @excludelist is a list of patterns which should be removed
#                         from the end of $filename.
#            $filename    is the part of $filespec after $prefix (i.e. the
#                         name of the file).  The elements of @excludelist
#                         are compared to $filename, and if an  
#            $prefix     is the path portion $filespec, up to and including
#                        the end of the last directory name
#            $tail        any characters removed from $filename because they
#                         matched an element of @excludelist.
#
#   fileparse() first removes the directory specification from $filespec,
#   according to the syntax of the OS (code is provided below to handle
#   VMS, Unix, MSDOS and MacOS; you can pick the one you want using
#   fileparse_set_fstype(), or you can accept the default, which is
#   based on the information in the builtin variable $^O).  It then compares
#   each element of @excludelist to $filename, and if that element is a
#   suffix of $filename, it is removed from $filename and prepended to
#   $tail.  By specifying the elements of @excludelist in the right order,
#   you can 'nibble back' $filename to extract the portion of interest
#   to you.
#
#   For example, on a system running Unix,
#   ($base,$path,$type) = fileparse('/virgil/aeneid/draft.book7',
#                                       '\.book\d+');
#   would yield $base == 'draft',
#               $path == '/virgil/aeneid/'  (note trailing slash)
#               $tail == '.book7'.
#   Similarly, on a system running VMS,
#   ($name,$dir,$type) = fileparse('Doc_Root:[Help]Rhetoric.Rnh','\..*');
#   would yield $name == 'Rhetoric';
#               $dir == 'Doc_Root:[Help]', and
#               $type == '.Rnh'.
#
#   Version 2.2  13-Oct-1994  Charles Bailey  bailey@genetics.upenn.edu 


sub fileparse {
  my($fullname,@suffices) = @_;
  my($fstype) = $Fileparse_fstype;
  my($dirpath,$tail,$suffix);

  if ($fstype =~ /^VMS/i) {
    if ($fullname =~ m#/#) { $fstype = '' }  # We're doing Unix emulation
    else {
      ($dirpath,$basename) = ($fullname =~ /(.*[:>\]])?(.*)/);
      $dirpath = $ENV{'DEFAULT'} unless $dirpath;
    }
  }
  if ($fstype =~ /^MSDOS/i) {
    ($dirpath,$basename) = ($fullname =~ /(.*\\)?(.*)/);
    $dirpath = '.\\' unless $dirpath;
  }
  elsif ($fstype =~ /^MAC/i) {
    ($dirpath,$basename) = ($fullname =~ /(.*:)?(.*)/);
  }
  elsif ($fstype !~ /^VMS/i) {  # default to Unix
    ($dirpath,$basename) = ($fullname =~ m#(.*/)?(.*)#);
    $dirpath = './' unless $dirpath;
  }

  if (@suffices) {
    $tail = '';
    foreach $suffix (@suffices) {
      if ($basename =~ /($suffix)$/) {
        $tail = $1 . $tail;
        $basename = $`;
      }
    }
  }

  wantarray ? ($basename,$dirpath,$tail) : $basename;

}


#   basename() - returns first element of list returned by fileparse()

sub basename {
  my($name) = shift;
  (fileparse($name, map("\Q$_\E",@_)))[0];
}
  

#    dirname() - returns device and directory portion of file specification
#        Behavior matches that of Unix dirname(1) exactly for Unix and MSDOS
#        filespecs except for names ending with a separator, e.g., "/xx/yy/".
#        This differs from the second element of the list returned
#        by fileparse() in that the trailing '/' (Unix) or '\' (MSDOS) (and
#        the last directory name if the filespec ends in a '/' or '\'), is lost.

sub dirname {
    my($basename,$dirname) = fileparse($_[0]);
    my($fstype) = $Fileparse_fstype;

    if ($fstype =~ /VMS/i) { 
        if ($_[0] =~ m#/#) { $fstype = '' }
        else { return $dirname }
    }
    if ($fstype =~ /MacOS/i) { return $dirname }
    elsif ($fstype =~ /MSDOS/i) { 
        if ( $dirname =~ /:\\$/) { return $dirname }
        chop $dirname;
        $dirname =~ s:[^\\]+$:: unless $basename;
        $dirname = '.' unless $dirname;
    }
    else { 
        if ( $dirname eq '/') { return $dirname }
        chop $dirname;
        $dirname =~ s:[^/]+$:: unless $basename;
        $dirname = '.' unless $dirname;
    }

    $dirname;
}
################################################################################


	    #####From the timelocal.pl package included with perl#####

;# These routines are quite efficient and yet are always guaranteed to agree
;# with localtime() and gmtime().  We manage this by caching the start times
;# of any months we've seen before.  If we know the start time of the month,
;# we can always calculate any time within the month.  The start times
;# themselves are guessed by successive approximation starting at the
;# current time, since most dates seen in practice are close to the
;# current date.  Unlike algorithms that do a binary search (calling gmtime
;# once for each bit of the time value, resulting in 32 calls), this algorithm
;# calls it at most 6 times, and usually only once or twice.  If you hit
;# the month cache, of course, it doesn't call it at all.

;# timelocal is implemented using the same cache.  We just assume that we're
;# translating a GMT time, and then fudge it when we're done for the timezone
;# and daylight savings arguments.  The timezone is determined by examining
;# the result of localtime(0) when the package is initialized.  The daylight
;# savings offset is currently assumed to be one hour.

;# Both routines return -1 if the integer limit is hit. I.e. for dates
;# after the 1st of January, 2038 on most machines.


sub timegm {
    package timelocal;
    #########CONFIG
    local($[) = 0;
    @epoch = localtime(0);
    $tzmin = $epoch[2] * 60 + $epoch[1];	# minutes east of GMT
    if ($tzmin > 0) {
	$tzmin = 24 * 60 - $tzmin;		# minutes west of GMT
	$tzmin -= 24 * 60 if $epoch[5] == 70;	# account for the date line
    }
    $SEC = 1;
    $MIN = 60 * $SEC;
    $HR = 60 * $MIN;
    $DAYS = 24 * $HR;
    #print "$DAYS\n";
    $YearFix = ((gmtime(946684800))[5] == 100) ? 100 : 0;
    1;
    ##########ENDCONFIG
    $ym = pack(C2, @_[5,4]);
    $cheat = $cheat{$ym} || &cheat;
    return -1 if $cheat<0;
    $cheat + $_[0] * $SEC + $_[1] * $MIN + $_[2] * $HR + ($_[3]-1) * $DAYS;
}

sub timelocal {
    package timelocal;
    #########CONFIG
    local($[) = 0;
    @epoch = localtime(0);
    $tzmin = $epoch[2] * 60 + $epoch[1];	# minutes east of GMT
    if ($tzmin > 0) {
	$tzmin = 24 * 60 - $tzmin;		# minutes west of GMT
	$tzmin -= 24 * 60 if $epoch[5] == 70;	# account for the date line
    }
    $SEC = 1;
    $MIN = 60 * $SEC;
    $HR = 60 * $MIN;
    $DAYS = 24 * $HR;
    #print "$DAYS\n";
    $YearFix = ((gmtime(946684800))[5] == 100) ? 100 : 0;
    1;
    ##########ENDCONFIG
    $time = &main'timegm + $tzmin*$MIN;
    return -1 if $cheat<0;
    @test = localtime($time);
    $time -= $HR if $test[2] != $_[2];
    $time;
}

package timelocal;
sub cheat {
    #########CONFIG
    local($[) = 0;
    @epoch = localtime(0);
    $tzmin = $epoch[2] * 60 + $epoch[1];	# minutes east of GMT
    if ($tzmin > 0) {
	$tzmin = 24 * 60 - $tzmin;		# minutes west of GMT
	$tzmin -= 24 * 60 if $epoch[5] == 70;	# account for the date line
    }
    $SEC = 1;
    $MIN = 60 * $SEC;
    $HR = 60 * $MIN;
    $DAYS = 24 * $HR;
    #print "$DAYS\n";
    $YearFix = ((gmtime(946684800))[5] == 100) ? 100 : 0;
    1;
    ##########ENDCONFIG
    $year = $_[5];
    $month = $_[4];
    die "Month out of range 0..11 in timelocal.pl\n" 
	if $month > 11 || $month < 0;
    die "Day out of range 1..31 in timelocal.pl\n" 
	if $_[3] > 31 || $_[3] < 1;
    die "Hour out of range 0..23 in timelocal.pl\n"
	if $_[2] > 23 || $_[2] < 0;
    die "Minute out of range 0..59 in timelocal.pl\n"
	if $_[1] > 59 || $_[1] < 0;
    die "Second out of range 0..59 in timelocal.pl\n"
	if $_[0] > 59 || $_[0] < 0;
    $guess = $^T;
    @g = gmtime($guess);
    $year += $YearFix if $year < $epoch[5];
    $lastguess = "";
    while ($diff = $year - @g[5]) {
	$guess += ($diff * (363 * $DAYS));
	@g = gmtime($guess);
	if (($thisguess = "@g") eq $lastguess){
	    return -1; #date beyond this machine's integer limit
	}
	$lastguess = $thisguess;
	#print @g[5], "...", $year, "\n";
    }
    while ($diff = $month - $g[4]) {
	$guess += $diff * (27 * $DAYS);
	@g = gmtime($guess);
	if (($thisguess = "@g") eq $lastguess){
	    return -1; #date beyond this machine's integer limit
	}
	$lastguess = $thisguess;
    }
    @gfake = gmtime($guess-1); #still being sceptic
    if ("@gfake" eq $lastguess){
	return -1; #date beyond this machine's integer limit
    }
    $g[3]--;
    $guess -= $g[0] * $SEC + $g[1] * $MIN + $g[2] * $HR + $g[3] * $DAYS;
    $cheat{$ym} = $guess;
}

__END__
