/***********************************************************************
 * FILENAME:         acclib.h
 *
 * CURRENT VERSION:  %version: lgca~20 %
 * DATE MODIFIED:    %date_modified: Fri Jul 15 13:15:16 2005 %
 *
 * PURPOSE:
 *   Header file for ACC API
 *
 * Copyright (c) 1999-2004 Brooktrout Technology, Inc. All Rights Reserved.
 *
 * This software is the property of Brooktrout Technology, Inc.
 * Any reproduction, disclosure, or distribution of this software is
 * expressly prohibited unless first agreed to in writing by Brooktrout
 * Technology, Inc.
 ***********************************************************************/

#ifndef _acclib_h
#define _acclib_h

//----------------------------------------------------------------------
//
// Edit History:
//
//  06-Dec-99  FJN  Created
//  22-Aug-00  FJN  Insert args_dsp_role structure
//  03-Jan-01  FJN  Insert version number declarations
//  03-Jan-27  RTW  Increment build number for SDK v3.2
//  27-Apr-04  JEB  Revise versioning
//  14-May-04  JEB  Change version to 1.6.0 for SDK v3.3.0
//
//----------------------------------------------------------------------

#include "../../bfv.api/inc/sdk_version.h"

// define version
#define	ACC_API_MAJOR_VERSION		5
#define	ACC_API_MIDDLE_VERSION		1
#define	ACC_API_MINOR_VERSION		0
#define	ACC_API_BUILD_NUMBER		SDK_BUILD_NUMBER

#define	ACC_API_COPYRIGHT_STR		"Copyright (c) 1999-2004 Brooktrout Technology"

/*  Create version string in the format "1.2.3B4"
    (major.middle.minorBbuild) */
#define ACC_API_VERSION2(a,b,c,d)	#a "." #b "." #c "B" #d
#define ACC_API_VERSION1(a,b,c,d)	ACC_API_VERSION2(a,b,c,d)
#define ACC_API_VERSION			ACC_API_VERSION1(ACC_API_MAJOR_VERSION, ACC_API_MIDDLE_VERSION, ACC_API_MINOR_VERSION, SDK_BUILD_NUMBER)


// Maintain backward compatability. Deprecated, do not use for future development.
#define CONFAPI_VER_NUM2(a,b,c,d)	a ## b ## c ## d
#define CONFAPI_VER_NUM1(a,b,c,d)	CONFAPI_VER_NUM2(a,b,c,d)
#define CONFAPI_VER_NUM			CONFAPI_VER_NUM1(ACC_API_MAJOR_VERSION,ACC_API_MIDDLE_VERSION,0,ACC_API_MINOR_VERSION)

#define CONFAPI_VERSION2(a,b,c,d)	#a "." #b "." #c #d
#define CONFAPI_VERSION1(a,b,c,d)	CONFAPI_VERSION2(a,b,c,d)
#define CONFAPI_VERSION			CONFAPI_VERSION1(ACC_API_MAJOR_VERSION,ACC_API_MIDDLE_VERSION,0,ACC_API_MINOR_VERSION)

// The following definitions conform to the way the Bfv API defines them.
#define	CONFAPI_V1			ACC_API_MAJOR_VERSION
#define	CONFAPI_V2			ACC_API_MIDDLE_VERSION
#define	CONFAPI_V3			0
#define	CONFAPI_V4			ACC_API_MINOR_VERSION
#define	CONFAPI_COPYRIGHT_STR		ACC_API_COPYRIGHT_STR



// define constants
#define	VOLUME_DEFAULT	0x80000000
#define	INITIAL_AGC_MAX	0
#define	INITIAL_MUTE	FALSE
#define	CONFID_MAX		0x7FFF

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE (!FALSE)
#endif

// TBD
typedef void *PORT;

// Debug trace level codes
typedef enum confTraceLevelCodes
{
	CONF_TRACE_LEVEL_NONE,
	CONF_TRACE_LEVEL_LOW,
	CONF_TRACE_LEVEL_MEDIUM,
	CONF_TRACE_LEVEL_HIGH
} CONF_TRACE_LEVEL, *PCONF_TRACE_LEVEL;

// Debug mask bits

#define	CONF_TRACE_MASK_ADMIN	0x0001
#define	CONF_TRACE_MASK_CONF	0x0002
#define	CONF_TRACE_MASK_PARTY	0x0004

// args_conf_admin is used by add BfvConferenceAdmin* functions
typedef struct args_conf_admin
{
	// The following are used as inputs to BfvConferenceAdminInitialize
	// and BfvConferenceAdminTerminate
	int rendezvousKey;	// A unique value that represents the "address" to
						// use to contact that conference administrator.
						// This value depends on the underlying inter-process
						// communications mechanism. For example, it might
						// be a software module ID if the administrator is
						// a software BOSTON module, it could be an IPC key
						// if System V style IPC queues are used, etc. If set
						// to 0, then the system default is used.

	// The following are used as inputs to BfvConferenceAdminInitialize
	// and BfvConferenceAdminModify and outputs from BfvConferenceAdminQuery
	int minID;			// The smallest conference ID that will be assigned
						// by the administrator
	int maxID;			// The largest conference ID that will be assigned
						// by the administrator

	// The following are used as outputs from BfvConferenceAdminQuery
	int lastIssuedID;	// The most recent ID assigned by the conference admin
	int numIssuedIDs;	// The total number of IDs issued since initialization
	int numConferences;	// The current number of outstanding issued IDs

	// The following is used as input to BfvConferenceAdminGetById
	int reserveFlag;	// Set to non-0 to reserve supplied ConfID if available
						// Set to 0 if no reservation required for unallocated
						// confIDs
	unsigned confID;	// The conference ID

	// Debug trace parameters
	CONF_TRACE_LEVEL debugTraceLevel;	// debug trace verbosity
	unsigned debugTraceMask;			// debug trace focus
	void (*debugTraceHandler)(char *);	// debug trace callback
										// null implies local printf

	// The following is an output from any function that uses this struct
	RES res;			// Standard result structure
} ARGS_CONF_ADMIN, *PARGS_CONF_ADMIN;

// Conference party participation modes
typedef enum confPartyModes
{
	CONF_MODE_DEFAULT = 0,		// Same as CONF_MODE_FULL_DUPLEX
	CONF_MODE_FULL_DUPLEX,		// Listen and talk participation
	CONF_MODE_LISTEN_ONLY,		// Listen only
	CONF_MODE_TALK_ONLY,		// Talk only
	CONF_MODE_DISCONNECT		// Neither listen nor talk participation
} CONF_PARTY_MODES, *PCONF_PARTY_MODES;

// Conference types
typedef enum confPartyTypes
{
	CONF_TYPE_DEFAULT = 0,		// Same as CONF_TYPE_BOARD
	CONF_TYPE_BOARD,			// Conference formed on local hardware module
	CONF_TYPE_NETWORK			// Conference formed in a remote network
								// (not supported in release 1.2)
} CONF_PARTY_TYPE, *PCONF_PARTY_TYPE;

// Noise limit parameter types
typedef enum confNoiseLimits
{
	CONF_NOISE_LIMIT_DEFAULT = 0,	// Same as CONF_NOISE_LIMIT_DISABLE
	CONF_NOISE_LIMIT_DISABLE,	// Disable noise limit processing
	CONF_NOISE_LIMIT_ENABLE		// Enable noise limit processing
} CONF_NOISE_LIMIT, *PCONF_NOISE_LIMIT;

// DSP channel resource roles
typedef enum confChannelRole
{
	CONF_CHANNEL_ROLE_TERMINAL,
	CONF_CHANNEL_ROLE_NODE
} CONF_CHANNEL_ROLE, *PCONF_CHANNEL_ROLE;

// Conference comfort noise types
typedef enum confComfortNoise
{
	CONF_COMFORT_NOISE_DEFAULT=0,
	CONF_COMFORT_NOISE_NONE,
	CONF_COMFORT_NOISE_NORMAL,
	CONF_COMFORT_NOISE_UNIFORM
} CONF_COMFORT_NOISE, *PCONF_COMFORT_NOISE;

// Party role/monitor types
typedef enum confPartyRole
{
	CONF_PARTY_ROLE_DEFAULT = 0,
	CONF_PARTY_ROLE_NORMAL,
	CONF_PARTY_ROLE_COACH,
	CONF_PARTY_ROLE_PUPIL
} CONF_PARTY_ROLE, *PCONF_PARTY_ROLE;

// Resource handling
typedef enum resourceOperations
{
	FREE = 0,					// Free the resource
	ALLOCATE,					// Allocate the resource
	CHANGE						// Change resource configuration and allocation
} RESOURCE_OPERATIONS, *PRESOURCE_OPERATIONS;

// args_resource used for argument passing to the user provided resourceHandler functions
typedef struct args_resource
{
#if 0 // now calling argument to resource handler
	BTLINE *board_lp;				// Handle to the board for use with switching commands
#endif
	PORT *resource;					// Handle to a resource
	enum resourceOperations operation; // What to do with respect to the resource
	unsigned confID;				// conference ID
	unsigned callRef;				// party ID to connect/disconnect/change
									// zero implies conference resource

	// conference resource arguments
	int minReserveSize;				// minimum number of parties to conference

	// party resource arguments
	enum confPartyModes mode;		// Participation mode
	enum confPartyTypes type;		// Participation type
	enum confNoiseLimits noiseLimit; // Specifies whether or not to use noise limiting
	enum confComfortNoise comfortNoise;
	enum confPartyRole partyRole;
	int agcMax;						// AGC setting dB
	int mute;						// mute (boolean)
	int volume;						// Volume (in dB)
	int clampKeys;					// boolean to enable disable DTMF clamping

	// output status
	RES res;
} ARGS_RESOURCE, *PARGS_RESOURCE;

// Party handle types
typedef enum confPartyHandleTypes
{
	CONF_HANDLE_TYPE_NETWORK = 0,	// party is network port
									// Handle is a BTLINE*
	CONF_HANDLE_TYPE_PCM,			// party is a PCM bus resource
									// Handle is a PORT*
	CONF_HANDLE_TYPE_CHANNEL		// party is a DSP channel
									// Handle is a BTLINE*
} CONF_PARTY_HANDLE_TYPES, *PCONF_PARTY_HANDLE_TYPES;

// Conference party object
typedef struct _CONF_PARTY
{
	struct _CONF_PARTY *next;		// Link to next CONF_PARTY or NULL
	struct _CONF_PARTY *prev;		// Link to previous CONF_PARTY or NULL
	unsigned callRef;				// Reference ID for the party

	BTLINE *board_lp;				// Handle to the board for use with
									// switching commands
	void *handle;					// Pointer to handle for source party object
	enum confPartyHandleTypes handleType; // Type of the above handle
	enum confPartyModes mode;		// Participation mode
	enum confPartyTypes type;		// Participation type
	enum confNoiseLimits noiseLimit; // Specifies whether or not to use noise
	enum confPartyRole role;
									// limiting
	int agcMax;						// AGC setting dB
	int mute;						// mute (boolean)
	int volume;						// Volume (in dB)
} CONF_PARTY, *PCONF_PARTY;

// args_conference used for argument passing to various BfvConference functions
typedef struct args_conference
{
	int rendezvousKey;	// A unique value that represents the "address" to
						// use to contact that conference administrator.
						// This value depends on the underlying inter-process
						// communications mechanism. For example, it might
						// be a software module ID if the administrator is
						// a software BOSTON module, it could be an IPC key
						// if System V style IPC queues are used, etc. If set
						// to 0, then the system default is used.

	// The following initial_* members are used to establish the properties
	// for a new party when it is added. These are used as inputs only with
	// BfvConferenceBegin and BfvConferenceModify and outputs from
	// BfvConferenceQuery
	enum confPartyModes initialMode;	// Default mode
	enum confPartyTypes initialType;	// Default participation type
	enum confNoiseLimits initialNoiseLimit; // Default noise limit setting
	enum confComfortNoise comfortNoise;	// comfort noise setting for all parties
	int initialAgcMax;					// Default AGC max in dB
	int initialMute;					// Default mute (boolean)
	int initialVolume;					// Default volume (in dB)

	// The following members are used as inputs only with BfvConferenceBegin
	// and BfvConferenceModify and outputs from BfvConferenceQuery
	int clampKeys;		// boolean to enable disable DTMF clamping
	int (*resourceHandler)(BTLINE*, struct args_resource *args); // A pointer to a user
						// supplied callback function that will allocate
						// resources to the conference on demand.
						// Set to NULL if the API is to manage all resource
						// allocation.
	int minReserveSize;	// Min number of members that conference must support.
						// Set to 0 for no minimum resource reservation.
						// Set to the minimum number of parties that the
						// conference will need to support. Setting this value
						// will cause the minimum number of resources to be
						// pre-allocated to the conference (possibly using
						// resourceHandler). An error will occur if sufficient
						// resources are not available. When used with
						// BfvConferenceModify, will cause additional resources
						// to be allocated or unused resources to be freed
						// depending on the current resource use and whether
						// the new value is greater than or less than the old
						// value. An error will be returned if sufficient
						// resources cannot be allocated.
	int maxTalkers;		// Max members that the conference will allow. An error
						// will be returned if BfvConferencePartyAdd is
						// attempted on a conference that already has
						// maxTalkers talking participants.

	// The following is used as an input or output to BfvConferenceBegin and
	// output from BfvConferenceQuery
	unsigned confID;	// The conference ID. If set to 0, a unique ID will be
						// assigned. If non-zero the supplied value will be
						// retained.

	// The following is used as outputs from BfvConferenceQuery
	int numMembers;		// Total number of participants in conference (all
						// modes)
	int numReserved;	// Number of reserved participant slots (includes those
						// in use)
	int numTalkerOnly;	// Number of talk-only mode participants
	int numListenOnly;	// Number of listen-only mode participants
	int numFullDuplex;	// Number of full-duplex participants
	int numDisconnected; // Number of disconnected participants
	CONF_PARTY *partiesHead; // pointer to array of numMembers CONF_PARTY

	// The following is an output from any function that uses this struct
	RES res;			// Standard result structure
} ARGS_CONFERENCE, PARGS_CONFERENCE;

// Conference object
typedef struct _BTCONF
{
	// The contents of this structure are to be defined by the design. Since
	// the application should never directly access the members of this
	// structure there is no need to define them as part of a functional
	// specification or interface specification.
	int rendezvousKey;	// The rendezvousKey that was used to create the
						// conference
	unsigned confID;	// The conference ID

	// For perspective and not definition, it is reasonable to infer that this
	// structure will contain copies of parameters specified in
	// BfvConferenceBegin and BfvConferenceModify. Additionally, it will likely
	// contain a linked list of all parties in the conference. Other state
	// and statistics information may also be stored in this structure.
	enum confPartyModes initialMode;	// Default mode
	enum confPartyTypes initialType;	// Default participation type
	enum confNoiseLimits initialNoiseLimit; // Default noise limit setting
	enum confComfortNoise comfortNoise;	// conference wide comfort noise setting
	int initialAgcMax;					// Default AGC max in dB
	int initialMute;					// Default mute (boolean)
	int initialVolume;					// Default volume (in dB)
	int clampKeys;						// boolean to enable disable DTMF clamping
	int (*resourceHandler)(BTLINE*, struct args_resource *args); // Pointer to user func
	int minReserveSize;	// Min number of members that conference must support
	int maxTalkers;		// Max members that the conference will allow

	int numDisconnected;
	int numFullDuplex;
	int numListenOnly;
	int numMembers;
	int numReserved;
	int numTalkerOnly;
} BTCONF, *PBTCONF;

// args_conf_party used for argument passing to various BfvConferenceParty*
// functions
typedef struct args_conf_party
{
	unsigned confID;						// parent conference ID

	// The following is input and output to BfvConferencePartyAdd
	// It is also sued as input to all other BfvConferenceParty* functions
	unsigned callRef;						// The reference ID for the party

	// The following members are inputs to BfvConferencePartyAdd
	void *handle;							// Pointer to a handle for the
											// source party object
	enum confPartyHandleTypes handleType;	// The type of the above handle

	// The following members are inputs to BfvConferencePartyAdd,
	// BfvConferencePartyModify and outputs from BfvConferencePartyQuery
	enum confPartyModes mode;				// Participation mode
	enum confPartyTypes type;				// Participation type
	enum confNoiseLimits noiseLimit;		// Specifies whether or not to use noise limiting
	enum confPartyRole role;				// Party role
	int agcMax;								// AGC setting dB
	int mute;								// mute (boolean)
	int volume;								// Volume (in dB)

	// The following is an output from any function that uses this struct
	RES res;								// Standard result structure
} ARGS_CONF_PARTY, *PARGS_CONF_PARTY;

// dsp role
typedef struct args_dsp_role
{
	unsigned confID;						// conference ID
	int operation;							// ALLOCATE, CHANGE, FREE
	CONF_CHANNEL_ROLE role;					// CONF_CHANNEL_ROLE_TERMINAL, CONF_CHANNEL_ROLE_NODE
	int mode;								// CONF_MODE_FULL_DUPLEX...
	int volume;								// volume (in dB)
	int mute;								// mute (boolean)
	int clampKeys;							// clamp keys (boolean)
	enum confComfortNoise comfortNoise;		// comfort noise
	enum confPartyRole partyRole;			// party role
	RES res;								// standard result structure
} ARGS_DSP_ROLE, *PARGS_DSP_ROLE;

// RES.line_status codes
typedef enum confErrorCodes
{
	CONF_ERROR_BAD_PARAMETERS = 1000,
	CONF_ERROR_BAD_ADMIN_INIT,
	CONF_ERROR_ADMIN_EXISTS,
	CONF_ERROR_ADMIN_MISSING,
	CONF_ERROR_ID_MISSING_RESERVED,
	CONF_ERROR_ID_MISSING_NOT_RESERVED,
	CONF_ERROR_INSUFFICIENT_RESOURCES,
	CONF_ERROR_ID_IN_USE,
	CONF_ERROR_RESOURCE_HANDLER_ERROR,
	CONF_ERROR_INVALID_CONFERENCE,
	CONF_ERROR_CALLREF_IN_USE,
	CONF_ERROR_RELEASING_RESOURCES,
	CONF_ERROR_INVALID_CALLREF,
	CONF_ERROR_SYSTEM,
	CONF_ERROR_TALKER_LIMIT,
	CONF_ERROR_ID_NOT_BEGUN
} CONF_ERROR_CODES, *PCONF_ERROR_CODES;

// function prototypes
int BfvConferenceAdminInitialize(struct args_conf_admin *);
int BfvConferenceAdminQuery(struct args_conf_admin *);
int BfvConferenceAdminModify(struct args_conf_admin *);
int BfvConferenceAdminTerminate(struct args_conf_admin *);
int BfvConferenceAdminGetById(struct args_conf_admin *);
int BfvConferenceBegin(struct args_conference *);
int BfvConferenceEnd(struct args_conference *);
int BfvConferenceQuery(struct args_conference *);
void BfvConferenceQueryFree(PCONF_PARTY);
int BfvConferenceModify(struct args_conference *);
int BfvConferencePartyAdd(BTLINE *, struct args_conf_party *);
int BfvConferencePartyDrop(BTLINE *, struct args_conf_party *);
int BfvConferencePartyQuery(BTLINE *, struct args_conf_party *);
int BfvConferencePartyModify(BTLINE *, struct args_conf_party *);
int BfvConferenceDspRole(BTLINE *, struct args_dsp_role *);
void BfvConferenceDebugFuncSet(void (*)(char *));
void BfvConferenceDebugModeSet(unsigned, unsigned);

#endif		// include once
