// %A%
//----------------------------------------------------------------------
// Source: xwndproc.c
//
// Copyright (c) 1999-2000 Brooktrout Technology, Needham, MA
// All rights reserved
// 
// Edit History:
//
// 06-Dec-99  FJN  Created
//----------------------------------------------------------------------

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#ifdef linux
#include <pthread.h>
#define thread_t pthread_t
#else
#include <thread.h>
#endif
#include <semaphore.h>

//----------------------------------------------------------------------
// miscellaneous declarations
//----------------------------------------------------------------------
#define	UNIT_MAX	128

#define	DEFAULT_BORDER_WIDTH	1
#define	ADMIN_MAX				12
#define	PROMPT_X_SIZE			30
#define	UNIT_STATUS_WIDTH		128

#define	SEM_SHARE_LOCAL			0
#define	SEM_SHARE_GLOBAL		1

//----------------------------------------------------------------------
// window configuration declarations
//----------------------------------------------------------------------

#define	MAIN_X_SIZE				600
#define	MAIN_Y_SIZE				700
#define	MAIN_X_ORIGIN			10
#define	MAIN_Y_ORIGIN			10

#define	UNIT_X_SIZE				40
#define	UNIT_Y_SIZE				20
#define	UNIT_X_ORIGIN			(10 + PROMPT_X_SIZE)
#define	UNIT_Y_ORIGIN			(MAIN_Y_SIZE - UNIT_Y_SIZE - 10)

#define	DTMF_X_SIZE				80
#define	DTMF_Y_SIZE				20
#define	DTMF_X_ORIGIN			(PROMPT_X_SIZE + UNIT_X_ORIGIN + UNIT_X_SIZE + 10)
#define	DTMF_Y_ORIGIN			(MAIN_Y_SIZE - DTMF_Y_SIZE - 10)

#define	ORIGINATE_X_SIZE		80
#define	ORIGINATE_Y_SIZE		20
#define	ORIGINATE_X_ORIGIN		(DTMF_X_ORIGIN + DTMF_X_SIZE + 10)
#define	ORIGINATE_Y_ORIGIN		(MAIN_Y_SIZE - ORIGINATE_Y_SIZE - 10)

#define	TONE_X_SIZE				80
#define	TONE_Y_SIZE				20
#define	TONE_X_ORIGIN			(ORIGINATE_X_ORIGIN + ORIGINATE_X_SIZE + 10)
#define	TONE_Y_ORIGIN			(MAIN_Y_SIZE - TONE_Y_SIZE - 10)

#define	HANGUP_X_SIZE			80
#define	HANGUP_Y_SIZE			20
#define	HANGUP_X_ORIGIN			(TONE_X_ORIGIN + TONE_X_SIZE + 10)
#define	HANGUP_Y_ORIGIN			(MAIN_Y_SIZE - HANGUP_Y_SIZE - 10)

#define	EXIT_X_SIZE				80
#define	EXIT_Y_SIZE				20
#define	EXIT_X_ORIGIN			(HANGUP_X_ORIGIN + HANGUP_X_SIZE + 10)
#define	EXIT_Y_ORIGIN			(MAIN_Y_SIZE - EXIT_Y_SIZE - 10)

//----------------------------------------------------------------------
// external prototypes
//----------------------------------------------------------------------

void CallOriginate(unsigned, char *);
void CallPlayTones(unsigned, char *);
void CallTerminate(unsigned);

extern unsigned unitNum;

//----------------------------------------------------------------------
// static variables
//----------------------------------------------------------------------

static Display *hDisplay;
static Window mainWnd;
static Window unitWnd;
static Window dtmfWnd;
static Window originateWnd;
static Window toneWnd;
static Window hangupWnd;
static Window exitWnd;

static char szUnitStatus[UNIT_MAX+ADMIN_MAX][UNIT_STATUS_WIDTH];
static int iLineSpace;

static char szAppName[] = "ConferenceDemo";

//----------------------------------------------------------------------
// Procedure XWndStatusUpdate
//----------------------------------------------------------------------
void XWndStatusUpdate(unsigned unit, char *pMsg)
{
#ifdef DEBUG
	printf("%d : %s\n", unit, pMsg);
#endif
	strncpy(szUnitStatus[unit], pMsg, UNIT_STATUS_WIDTH);
	szUnitStatus[unit][UNIT_STATUS_WIDTH-1] = 0;

	XClearArea(hDisplay, mainWnd, 0, iLineSpace * unit, MAIN_X_SIZE, iLineSpace, True);
	XFlush(hDisplay);
}

//----------------------------------------------------------------------
// Procedure XWndMsgBox
//----------------------------------------------------------------------
void XWndMsgBox(char *pMsg)
{
	register i;

	for (i = unitNum + 1; i < unitNum + ADMIN_MAX - 1; i++)
		strcpy(szUnitStatus[i], szUnitStatus[i+1]);

	strncpy(szUnitStatus[unitNum+ADMIN_MAX-1], pMsg, UNIT_STATUS_WIDTH);
	szUnitStatus[unitNum+ADMIN_MAX-1][UNIT_STATUS_WIDTH-1] = 0;

	XClearArea(hDisplay, mainWnd, 0, iLineSpace * unitNum, MAIN_X_SIZE, iLineSpace * ADMIN_MAX, True);
	XFlush(hDisplay);
}

//----------------------------------------------------------------------
// Procedure XWndProc
//----------------------------------------------------------------------
void *XWndProc(void *pSem)
{
	register i;
	char *p;
	char *display_name = NULL;
	char szText[128];
	char szUnit[80];
	char szDtmf[80];
	unsigned long mfgpix;
	unsigned long mbgpix;
	unsigned uAdminNdx;
	thread_t ThreadID;
	int fDone = 0;
	int iStatus;
	int iCount;
	int iLen;
	int iWidth;
	unsigned unit;

	char unitMsg[] = "Unit";
	char dtmfMsg[] = "DTMF";
	char originateMsg[] = "Originate";
	char toneMsg[] = "Tone";
	char hangupMsg[] = "Hangup";
	char exitMsg[] = "Exit";

	GC mainGC;
	GC unitGC;
	GC dtmfGC;
	GC originateGC;
	GC toneGC;
	GC hangupGC;
	GC exitGC;

	Colormap defaultColorMap;
	KeySym key;
	XColor color;
	XComposeStatus cs;
	XEvent theEvent;
	XFontStruct *fontStruct;
	XGCValues gcv;
	XSetWindowAttributes xswa;
	XSizeHints xsh;
	XWMHints xwmh;

	szUnit[0] = 0;
	szDtmf[0] = 0;

	for (unit = 0; unit < unitNum; unit++)
		strcpy(szUnitStatus[unit], "Uninitialized");

	//---------------------------------------------------------------
	// initialize for multiple threads
	//---------------------------------------------------------------
	if (XInitThreads())
		printf("XInitThreads successful\n");
	else
	{
		printf("XInitThreads failed\n");
		exit(1);
	}

	//---------------------------------------------------------------
	// access display
	//---------------------------------------------------------------
	if ((hDisplay = XOpenDisplay(display_name)) == NULL)
	{
		printf("XOpenDisplay failure %s\n", XDisplayName(display_name));
		exit(1);
	}

	//---------------------------------------------------------------
	// setup fonts
	//---------------------------------------------------------------
	if ((fontStruct = XLoadQueryFont(hDisplay, "fixed")) == NULL)
	{
		printf("XLoadQueryFont failure %s\n", DisplayString(hDisplay));
		exit(1);
	}
	iLineSpace = fontStruct->ascent + fontStruct->descent + 2;

	//---------------------------------------------------------------
	// setup colors
	//---------------------------------------------------------------
	defaultColorMap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));

	if (XParseColor(hDisplay, defaultColorMap, "white", &color) == 0 ||
		XAllocColor(hDisplay, defaultColorMap, &color) == 0)
		mfgpix = BlackPixel(hDisplay, DefaultScreen(hDisplay));
	else
		mfgpix = color.pixel;

	if (XParseColor(hDisplay, defaultColorMap, "blue", &color) == 0 ||
		XAllocColor(hDisplay, defaultColorMap, &color) == 0)
		mbgpix = WhitePixel(hDisplay, DefaultScreen(hDisplay));
	else
		mbgpix = color.pixel;

	//---------------------------------------------------------------
	// setup main window
	//---------------------------------------------------------------
	mainWnd = XCreateSimpleWindow(
			hDisplay,
			DefaultRootWindow(hDisplay),
			MAIN_X_ORIGIN,
			MAIN_Y_ORIGIN,
			MAIN_X_SIZE,
			MAIN_Y_SIZE,
			DEFAULT_BORDER_WIDTH,
			mfgpix,
			mbgpix);

	memset(&xsh, 0, sizeof(XSizeHints));
	xsh.flags = PPosition | PSize | PMinSize;
	xsh.min_width = xsh.width = MAIN_X_SIZE;
	xsh.min_height = xsh.height = MAIN_Y_SIZE;
	xsh.x = MAIN_X_ORIGIN;
	xsh.y = MAIN_Y_ORIGIN;
	XSetStandardProperties(hDisplay, mainWnd, szAppName, szAppName, None, NULL, 0, &xsh);

	memset(&xwmh, 0, sizeof(XWMHints));
	xwmh.flags = InputHint | StateHint;
	xwmh.input = True;
	xwmh.initial_state = NormalState;
	XSetWMHints(hDisplay, mainWnd, &xwmh);

	memset(&gcv, 0, sizeof(XGCValues));
	gcv.font = fontStruct->fid;
	gcv.foreground = mfgpix;
	gcv.background = mbgpix;
	mainGC = XCreateGC(hDisplay, mainWnd, GCFont | GCForeground | GCBackground, &gcv);

	memset(&xswa, 0, sizeof(XSetWindowAttributes));
	xswa.colormap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));
	xswa.bit_gravity = CenterGravity;
	XChangeWindowAttributes(hDisplay, mainWnd, CWColormap | CWBitGravity, &xswa);

	XSelectInput(hDisplay, mainWnd, ExposureMask);

	XMapWindow(hDisplay, mainWnd);

	//---------------------------------------------------------------
	// setup unit child window
	//---------------------------------------------------------------
	unitWnd = XCreateSimpleWindow(
			hDisplay,
			mainWnd,
			UNIT_X_ORIGIN,
			UNIT_Y_ORIGIN,
			UNIT_X_SIZE,
			UNIT_Y_SIZE,
			DEFAULT_BORDER_WIDTH,
			mfgpix,
			mbgpix);

	memset(&xwmh, 0, sizeof(XWMHints));
	xwmh.flags = InputHint | StateHint;
	xwmh.input = True;
	xwmh.initial_state = NormalState;
	XSetWMHints(hDisplay, unitWnd, &xwmh);

	memset(&gcv, 0, sizeof(XGCValues));
	gcv.font = fontStruct->fid;
	gcv.foreground = mfgpix;
	gcv.background = mbgpix;
	unitGC = XCreateGC(hDisplay, unitWnd, GCFont | GCForeground | GCBackground, &gcv);

	memset(&xswa, 0, sizeof(XSetWindowAttributes));
	xswa.colormap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));
	xswa.bit_gravity = CenterGravity;
	XChangeWindowAttributes(hDisplay, unitWnd, CWColormap | CWBitGravity, &xswa);

	XSelectInput(hDisplay, unitWnd, ExposureMask | KeyPressMask);

	XMapWindow(hDisplay, unitWnd);

	//---------------------------------------------------------------
	// setup dtmf child window
	//---------------------------------------------------------------
	dtmfWnd = XCreateSimpleWindow(
			hDisplay,
			mainWnd,
			DTMF_X_ORIGIN,
			DTMF_Y_ORIGIN,
			DTMF_X_SIZE,
			DTMF_Y_SIZE,
			DEFAULT_BORDER_WIDTH,
			mfgpix,
			mbgpix);

	memset(&xwmh, 0, sizeof(XWMHints));
	xwmh.flags = InputHint | StateHint;
	xwmh.input = True;
	xwmh.initial_state = NormalState;
	XSetWMHints(hDisplay, dtmfWnd, &xwmh);

	memset(&gcv, 0, sizeof(XGCValues));
	gcv.font = fontStruct->fid;
	gcv.foreground = mfgpix;
	gcv.background = mbgpix;
	dtmfGC = XCreateGC(hDisplay, dtmfWnd, GCFont | GCForeground | GCBackground, &gcv);

	memset(&xswa, 0, sizeof(XSetWindowAttributes));
	xswa.colormap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));
	xswa.bit_gravity = CenterGravity;
	XChangeWindowAttributes(hDisplay, dtmfWnd, CWColormap | CWBitGravity, &xswa);

	XSelectInput(hDisplay, dtmfWnd, ExposureMask | KeyPressMask);

	XMapWindow(hDisplay, dtmfWnd);

	//---------------------------------------------------------------
	// setup originate child window
	//---------------------------------------------------------------
	originateWnd = XCreateSimpleWindow(
			hDisplay,
			mainWnd,
			ORIGINATE_X_ORIGIN,
			ORIGINATE_Y_ORIGIN,
			ORIGINATE_X_SIZE,
			ORIGINATE_Y_SIZE,
			DEFAULT_BORDER_WIDTH,
			mfgpix,
			mbgpix);

	memset(&xwmh, 0, sizeof(XWMHints));
	xwmh.flags = InputHint | StateHint;
	xwmh.input = False;
	xwmh.initial_state = NormalState;
	XSetWMHints(hDisplay, originateWnd, &xwmh);

	memset(&gcv, 0, sizeof(XGCValues));
	gcv.font = fontStruct->fid;
	gcv.foreground = mfgpix;
	gcv.background = mbgpix;
	originateGC = XCreateGC(hDisplay, originateWnd, GCFont | GCForeground | GCBackground, &gcv);

	memset(&xswa, 0, sizeof(XSetWindowAttributes));
	xswa.colormap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));
	xswa.bit_gravity = CenterGravity;
	XChangeWindowAttributes(hDisplay, originateWnd, CWColormap | CWBitGravity, &xswa);

	XSelectInput(hDisplay, originateWnd, ExposureMask | ButtonPressMask);

	XMapWindow(hDisplay, originateWnd);

	//---------------------------------------------------------------
	// setup tone child window
	//---------------------------------------------------------------
	toneWnd = XCreateSimpleWindow(
			hDisplay,
			mainWnd,
			TONE_X_ORIGIN,
			TONE_Y_ORIGIN,
			TONE_X_SIZE,
			TONE_Y_SIZE,
			DEFAULT_BORDER_WIDTH,
			mfgpix,
			mbgpix);

	memset(&xwmh, 0, sizeof(XWMHints));
	xwmh.flags = InputHint | StateHint;
	xwmh.input = False;
	xwmh.initial_state = NormalState;
	XSetWMHints(hDisplay, toneWnd, &xwmh);

	memset(&gcv, 0, sizeof(XGCValues));
	gcv.font = fontStruct->fid;
	gcv.foreground = mfgpix;
	gcv.background = mbgpix;
	toneGC = XCreateGC(hDisplay, toneWnd, GCFont | GCForeground | GCBackground, &gcv);

	memset(&xswa, 0, sizeof(XSetWindowAttributes));
	xswa.colormap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));
	xswa.bit_gravity = CenterGravity;
	XChangeWindowAttributes(hDisplay, toneWnd, CWColormap | CWBitGravity, &xswa);

	XSelectInput(hDisplay, toneWnd, ExposureMask | ButtonPressMask);

	XMapWindow(hDisplay, toneWnd);

	//---------------------------------------------------------------
	// setup hangup child window
	//---------------------------------------------------------------
	hangupWnd = XCreateSimpleWindow(
			hDisplay,
			mainWnd,
			HANGUP_X_ORIGIN,
			HANGUP_Y_ORIGIN,
			HANGUP_X_SIZE,
			HANGUP_Y_SIZE,
			DEFAULT_BORDER_WIDTH,
			mfgpix,
			mbgpix);

	memset(&xwmh, 0, sizeof(XWMHints));
	xwmh.flags = InputHint | StateHint;
	xwmh.input = False;
	xwmh.initial_state = NormalState;
	XSetWMHints(hDisplay, hangupWnd, &xwmh);

	memset(&gcv, 0, sizeof(XGCValues));
	gcv.font = fontStruct->fid;
	gcv.foreground = mfgpix;
	gcv.background = mbgpix;
	hangupGC = XCreateGC(hDisplay, hangupWnd, GCFont | GCForeground | GCBackground, &gcv);

	memset(&xswa, 0, sizeof(XSetWindowAttributes));
	xswa.colormap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));
	xswa.bit_gravity = CenterGravity;
	XChangeWindowAttributes(hDisplay, hangupWnd, CWColormap | CWBitGravity, &xswa);

	XSelectInput(hDisplay, hangupWnd, ExposureMask | ButtonPressMask);

	XMapWindow(hDisplay, hangupWnd);

	//---------------------------------------------------------------
	// setup exit child window
	//---------------------------------------------------------------
	exitWnd = XCreateSimpleWindow(
			hDisplay,
			mainWnd,
			EXIT_X_ORIGIN,
			EXIT_Y_ORIGIN,
			EXIT_X_SIZE,
			EXIT_Y_SIZE,
			DEFAULT_BORDER_WIDTH,
			mfgpix,
			mbgpix);

	memset(&xwmh, 0, sizeof(XWMHints));
	xwmh.flags = InputHint | StateHint;
	xwmh.input = False;
	xwmh.initial_state = NormalState;
	XSetWMHints(hDisplay, exitWnd, &xwmh);

	memset(&gcv, 0, sizeof(XGCValues));
	gcv.font = fontStruct->fid;
	gcv.foreground = mfgpix;
	gcv.background = mbgpix;
	exitGC = XCreateGC(hDisplay, exitWnd, GCFont | GCForeground | GCBackground, &gcv);

	memset(&xswa, 0, sizeof(XSetWindowAttributes));
	xswa.colormap = DefaultColormap(hDisplay, DefaultScreen(hDisplay));
	xswa.bit_gravity = CenterGravity;
	XChangeWindowAttributes(hDisplay, exitWnd, CWColormap | CWBitGravity, &xswa);

	XSelectInput(hDisplay, exitWnd, ExposureMask | ButtonPressMask);

	XMapWindow(hDisplay, exitWnd);

	//---------------------------------------------------------------
	// signal parent that initialization is complete
	//---------------------------------------------------------------
	sem_post((sem_t *) pSem);

	//---------------------------------------------------------------
	// process xwindows events
	//---------------------------------------------------------------
	while (!fDone)
	{
		XNextEvent(hDisplay, &theEvent);

		//---------------------------------------------------------------
		// process main window events
		//---------------------------------------------------------------
		if (theEvent.xany.window == mainWnd)
			switch (theEvent.type)
			{
			case Expose :
				if (theEvent.xexpose.count == 0)
				{
					XDrawString(hDisplay, mainWnd, mainGC, UNIT_X_ORIGIN - PROMPT_X_SIZE, UNIT_Y_ORIGIN + 14, unitMsg, strlen(unitMsg));
					XDrawString(hDisplay, mainWnd, mainGC, DTMF_X_ORIGIN - PROMPT_X_SIZE, DTMF_Y_ORIGIN + 14, dtmfMsg, strlen(dtmfMsg));

					for (unit = 0; unit < unitNum; unit++)
					{
						sprintf(szText, "%2u/%2u : %s", unit, unit + unitNum, szUnitStatus[unit]);
						XDrawString( hDisplay, mainWnd, mainGC, 10, iLineSpace * (unit + 1) - fontStruct->descent, szText, strlen(szText));
					}
					for (unit = unitNum; unit < unitNum + ADMIN_MAX; unit++)
						XDrawString(hDisplay, mainWnd, mainGC, 10, iLineSpace * (unit + 1) - fontStruct->descent, szUnitStatus[unit], strlen(szUnitStatus[unit]));
				}
				break;
#if 0
			case FocusIn :
				if (theEvent.xfocus.mode == NotifyNormal &&
					theEvent.xfocus.detail == NotifyNonlinear)
					XSetInputFocus(hDisplay, unitWnd, RevertToParent, CurrentTime);
				break;
#endif
			}

		//---------------------------------------------------------------
		// process unit window events
		//---------------------------------------------------------------
		else if (theEvent.xany.window == unitWnd)
			switch (theEvent.type)
			{
			case Expose :
				if (theEvent.xexpose.count == 0)
					XDrawString(hDisplay, unitWnd, unitGC, 10, 14, szUnit, strlen(szUnit));
				break;

			case KeyPress :
				iCount = XLookupString(&theEvent.xkey, szText, sizeof(szText), &key, &cs);
				szText[iCount] = 0;
#ifdef DEBUG_VERBOSE
				printf("unit KeyPress type %u state %u keycode %u szText %s\n",
					theEvent.xkey.type,
					theEvent.xkey.state,
					theEvent.xkey.keycode,
					szText);
#endif
				for (p = szText; *p; p++)
					if (*p == '\b')
						if (strlen(szUnit))
							szUnit[strlen(szUnit)-1] = 0;
						else;
					else if (isdigit(*p))
					{
						iLen = strlen(szUnit);
						szUnit[iLen++] = *p;
						szUnit[iLen] = 0;
					}
				XClearArea(hDisplay, unitWnd, 0, 0, 0, 0, True);
				break;
			}

		//---------------------------------------------------------------
		// process dtmf window events
		//---------------------------------------------------------------
		else if (theEvent.xany.window == dtmfWnd)
			switch (theEvent.type)
			{
			case Expose :
				if (theEvent.xexpose.count == 0)
					XDrawString(hDisplay, dtmfWnd, dtmfGC, 10, 14, szDtmf, strlen(szDtmf));
				break;

			case KeyPress :
				iCount = XLookupString(&theEvent.xkey, szText, sizeof(szText), &key, &cs);
				szText[iCount] = 0;
#ifdef DEBUG_VERBOSE
				printf("dtmf KeyPress type %u state %u keycode %u szText %s\n",
					theEvent.xkey.type,
					theEvent.xkey.state,
					theEvent.xkey.keycode,
					szText);
#endif
				for (p = szText; *p; p++)
					if (*p == '\b')
						if (strlen(szDtmf))
							szDtmf[strlen(szDtmf)-1] = 0;
						else;
					else if (isprint(*p))
					{
						iLen = strlen(szDtmf);
						szDtmf[iLen++] = *p;
						szDtmf[iLen] = 0;
					}
				XClearArea(hDisplay, dtmfWnd, 0, 0, 0, 0, True);
				break;
			}

		//---------------------------------------------------------------
		// process originate window events
		//---------------------------------------------------------------
		else if (theEvent.xany.window == originateWnd)
			switch (theEvent.type)
			{
			case Expose :
				if (theEvent.xexpose.count == 0)
				{
					iWidth = XTextWidth(fontStruct, originateMsg, strlen(originateMsg));
					XDrawString(hDisplay, originateWnd, originateGC, (ORIGINATE_X_SIZE - iWidth) / 2, 14, originateMsg, strlen(originateMsg));
				}
				break;

			case ButtonPress :
				sscanf(szUnit, "%u", &unit);
				printf("CallOriginate%u szDtmf %s\n", unit, szDtmf);
				CallOriginate(unit, szDtmf);
				break;
			}

		//---------------------------------------------------------------
		// process tone window events
		//---------------------------------------------------------------
		else if (theEvent.xany.window == toneWnd)
			switch (theEvent.type)
			{
			case Expose :
				if (theEvent.xexpose.count == 0)
				{
					iWidth = XTextWidth(fontStruct, toneMsg, strlen(toneMsg));
					XDrawString(hDisplay, toneWnd, toneGC, (TONE_X_SIZE - iWidth) / 2, 14, toneMsg, strlen(toneMsg));
				}
				break;

			case ButtonPress :
				sscanf(szUnit, "%u", &unit);
				printf("CallPlayTones%u szDtmf %s\n", unit, szDtmf);
				CallPlayTones(unit, szDtmf);
				break;
			}

		//---------------------------------------------------------------
		// process hangup window events
		//---------------------------------------------------------------
		else if (theEvent.xany.window == hangupWnd)
			switch (theEvent.type)
			{
			case Expose :
				if (theEvent.xexpose.count == 0)
				{
					iWidth = XTextWidth(fontStruct, hangupMsg, strlen(hangupMsg));
					XDrawString(hDisplay, hangupWnd, hangupGC, (HANGUP_X_SIZE - iWidth) / 2, 14, hangupMsg, strlen(hangupMsg));
				}
				break;

			case ButtonPress :
				sscanf(szUnit, "%u", &unit);
				printf("CallTerminate%u\n", unit);
				CallTerminate(unit);
				break;
			}

		//---------------------------------------------------------------
		// process exit window events
		//---------------------------------------------------------------
		else if (theEvent.xany.window == exitWnd)
			switch (theEvent.type)
			{
			case Expose :
				if (theEvent.xexpose.count == 0)
				{
					iWidth = XTextWidth(fontStruct, exitMsg, strlen(exitMsg));
					XDrawString(hDisplay, exitWnd, exitGC, (EXIT_X_SIZE - iWidth) / 2, 14, exitMsg, strlen(exitMsg));
				}
				break;

			case ButtonPress :
				fDone = 1;
				break;
			}
	}

	XFreeGC(hDisplay, mainGC);
	XFreeGC(hDisplay, exitGC);
	XDestroyWindow(hDisplay, mainWnd);
	XCloseDisplay(hDisplay);
	exit(0);
}
