/*
 * csend.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
Csend uses low-level, noninfopkt, raw-data, fax-sending
routines to send facsimiles. It allows sending a single fax page either
in standard fax format from an MH/MSB G3 data file or in an
enhanced fax format (e.g. JPEG, JBIG) from an EFF data file. T.30
holdup is used to check on the receivers capabilities before deciding
which type of file to send.
***************************************************************************/

#include <string.h>
#include "btlib.h"

void process_results BT_ARGS((BTLINE *lp,FAX_RES *fres,long duration));
void process_aoc BT_ARGS((BTLINE *lp));
void process_call_info BT_ARGS((CALL_RES *call_res));
int holdup BT_ARGS((BTLINE *lp,char *arg,int section));

typedef struct {
    unsigned char switchToG3;
    struct args_fax args_fax;
} MyHoldupArg;

char *gEffFileName = "c1.jpg";
char *gG3FileName = "eagle.301";
unsigned gEFFOpts = BT_EFF_JPEG_ENABLE|BT_EFF_JPEG_FULLCOLOR;
int done;

void
usage ()
{
    printf("Usage: csend [options] <phone number>\n");
    printf("\t-u unitno\n");
    printf("\t-v turn on API debug mode\n");
    printf("\t-c EFF file (color or other enhanced), default: c1.jpg\n");
    printf("\t-g G3 file (MH), default: eagle.301\n");
    printf("\t-e Enhanced Fax Format (EFF) options, default: 3\n");
    printf("\t OR together the following hex values:\n");
    printf("\t   1=JPEG Enable\n");
    printf("\t   2=Full Color\n");
    printf("\t   4=Default tables\n");
    printf("\t   8=12bit (vs 8bit)\n");
    printf("\t   10=No Subsampling\n");
    printf("\t   20=Custom Illuminant\n");
    printf("\t   40=Custom Gamut\n");
    printf("\t   100=JBIG\n");
    printf("\t   200=JBIG L0\n");
    printf("\t   400=Lossless Color\n");
    printf("\t   800,1000,2000=MRC\n");
    printf("\t   4000=Plane Interleave\n");
    printf("\t   8000=Page Length Strip\n");
    exit(1);
}

int
main (int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp = NULL;
    void dowork();
    struct args_line_admin args;
    struct args_debug_mode args_debug;
    BTERR bterr;
    char *snum;
    int unit;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "u:vc:g:e:s:")) != EOF)
    {
        switch (c)
        {
          case 'u':
            unit = atoi (optarg);
            break;
          case 'v':
            BT_ZERO(args_debug);
            args_debug.mode = DEBUG_ALL;
            args_debug.options = API_DBG_MODE_SET;
            BfvDebugModeSetAdv(&args_debug);
            break;
          case 'c':
            gEffFileName = optarg;
            break;
          case 'g':
            gG3FileName = optarg;
            break;
          case 'e':
            if (sscanf(optarg,"%x",&gEFFOpts) != 1)
                usage();
            break;
          default:
            usage();
            break;
        }
    }
    
    if (optind == argc)
        usage();

    snum = argv[optind++];
    
    if (optind < argc)
        usage();

    BT_ZERO(args);
    args.unit = unit;
    lp = BfvLineAttach(&args);
    if (lp == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        printf ("BfvLineAttach: %s\n",bterr.long_msg);
        exit (1);
    }
    
    dowork(lp, snum);
    
    return(0);
}

int
optionsCompatible(BTLINE *lp, unsigned inNeededOptions)
{
    int theOptionsAreCompatible = 1;
    unsigned tmp;
    
    if ((inNeededOptions & BT_EFF_JPEG_ENABLE) &&
        (!((LINE_DIS_DTC(lp))->JPEG_Coding)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JPEG_FULLCOLOR) &&
        (!((LINE_DIS_DTC(lp))->JPEG_FullColor)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JPEG_12BIT) &&
        (!((LINE_DIS_DTC(lp))->JPEG_12Bit)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JPEG_NOSUBSAMPLING) &&
        (!((LINE_DIS_DTC(lp))->JPEG_NoSubSampling)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JPEG_CUSTOM_ILLUMINANT) &&
        (!((LINE_DIS_DTC(lp))->JPEG_CustomIlluminant)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JPEG_CUSTOM_GAMUT) &&
        (!((LINE_DIS_DTC(lp))->JPEG_CustomGamut)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JPEG_CUSTOM_GAMUT) &&
        (!((LINE_DIS_DTC(lp))->JPEG_CustomGamut)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JBIG) &&
        (!((LINE_DIS_DTC(lp))->JBIG_Coding)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_JBIG_L0) &&
        (!((LINE_DIS_DTC(lp))->JBIG_L0)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_LOSSLESS) &&
        (!((LINE_DIS_DTC(lp))->Lossless_Mode)))
        theOptionsAreCompatible = 0;

    tmp = (inNeededOptions & BT_EFF_MRC_MASK) >> 7;
    if ((tmp & (LINE_DIS_DTC(lp))->Lossless_Mode) != tmp)
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_PLANE_INTERLEAVE) &&
        (!((LINE_DIS_DTC(lp))->Plane_Interleave_Mode)))
        theOptionsAreCompatible = 0;

    if ((inNeededOptions & BT_EFF_PAGE_LENGTH_STRIP) &&
        (!((LINE_DIS_DTC(lp))->MRC_Page_Length_Strips)))
        theOptionsAreCompatible = 0;

    return(theOptionsAreCompatible);
}

int
holdup(BTLINE *lp, char *arg, int section)
{
    if (section == T30H_SECTION_XMIT_DCS)
    {
        /* NOTE: we will use the arg to pass the eff_options needed for the
           color doc, if these options are not supported by called terminal,
           set a flag in the arg so the rest of the sending code will know to
           switch to the G3 doc. */
        MyHoldupArg *myHoldupArg = (MyHoldupArg *)arg;
        BTERR bterr;

        if (!(optionsCompatible
              (lp,(unsigned)myHoldupArg->args_fax.eff_page_type)))
        {
            myHoldupArg->switchToG3 = 1;
            
            /* Reset the EFF options to disabled, remote
               doesn't support our options */
            myHoldupArg->args_fax.eff_page_type = BT_EFF_DISABLE;
        }
        
        /* Make sure the channel knows what type we want to send.
           NOTE: The format command isn't given to the channel
           if holdup is set until this function is called again
           from the holdup. */
        BfvFaxBeginSendRaw(lp, &myHoldupArg->args_fax);
        if (myHoldupArg->args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&myHoldupArg->args_fax.res,&bterr);
            printf ("BfvFaxBeginSendRaw: %s\n",bterr.long_msg);
            return(1);
        }
    }
    
    /* Always let it continue; just switch to G3 if not EFF-capable. */
    return(0);
}

void
dowork(BTLINE *lp, char *snum)
{
    BTERR bterr;
    FAX_RES fres;
    long start_time;
    struct args_line_admin args_admin;
    struct args_telephone args_tel;
    struct args_fax args_fax;
    MyHoldupArg myHoldupArg;
    
    BT_ZERO(fres);
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    if (BfvLineReset (lp, &args_admin) < 0)
    {
        fprintf (stderr, "Reset failed, status %lX.\n",
                 args_admin.reset_status);
        goto err_1;
    }
    
    if (!LINE_HAS_CAP(lp,CAP_FAX_XMIT))
    {
        fprintf (stderr, "unit %d does not support fax\n", LINE_UNIT_NUM(lp));
        goto err_1;
    }
    
    /*    check to see if the firmware supports color faxing */
    if (!LINE_HAS_CAP(lp,CAP_FAX_JPEG_PAGES))
    {
        fprintf(stderr, "unit %d does not support color fax\n",
                LINE_UNIT_NUM(lp));
        goto err_1;
    }
    
    /* Set up fax result structure */
    LINE_FAX_RES(lp) = &fres;
    
    /*
     * Send step 1:
     *      Call BfvLineOriginateCall to establish the connection
     */
    printf( "BfvLineOriginateCall\n" );
    BT_ZERO(args_tel);
    args_tel.phonenum = snum;
    args_tel.call_protocol_code = CALL_PROTOCOL_FAX;
    args_tel.func = NULL;
    BfvLineOriginateCall (lp, &args_tel);
    if (args_tel.res.status != BT_STATUS_OK || 
        LINE_STATE(lp) != LINE_STATE_CONNECTED)
    {
        BfvErrorMessage(lp,&args_tel.res,&bterr);
        printf ("BfvLineOriginateCall: %s\n",bterr.long_msg);
        goto err_2;
    }
    
    /* Start timing after we detect Fax answer tone */
    start_time = time(NULL);
    
    /*
     * Send step 2
     *      Setup the T.30 Holdup function
     *      we pass a pointer to myHoldupArg so the holdup function will have a
     *      way to communicate
     */
    BT_ZERO(args_fax);
    args_fax.holdup_mode = T30_HOLDUP_DCS;    /* holdup before DCS is sent */
    args_fax.func = holdup;                   /* call this function */
    args_fax.arg = (char *)&myHoldupArg;
    BfvFaxT30Holdup(lp, &args_fax);           /* install the holdup */
    if (args_fax.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_fax.res,&bterr);
        printf ("BfvFaxT30Holdup: %s\n",bterr.long_msg);
        goto err;
    }
    
    myHoldupArg.switchToG3 = 0;              /* assume EFF */
    BT_ZERO(myHoldupArg.args_fax);
    myHoldupArg.args_fax.resolution = RES_200H_200V;
    myHoldupArg.args_fax.width = WIDTH_A4;
    myHoldupArg.args_fax.bft_type = BFT_NONE;
    myHoldupArg.args_fax.eff_page_type = gEFFOpts;

    /*
     * Send step 3:
     *      Call BfvFaxBeginSendRaw to setup the fax for transmission
     */
    printf( "BfvFaxBeginSendRaw\n" );
    BT_ZERO(args_fax);
    args_fax.resolution = RES_200H_200V;
    args_fax.width = WIDTH_A4;
    args_fax.bft_type = BFT_NONE;
    args_fax.eff_page_type = gEFFOpts;
    BfvFaxBeginSendRaw(lp, &args_fax);
    if (args_fax.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_fax.res,&bterr);
        printf ("BfvFaxBeginSendRaw: %s\n",bterr.long_msg);
        goto err;
    }
    
    /*
     * Send step 4:
     *      Call BfvFaxGetRemoteInfo and BfvFaxWaitForTraining to talk to
     *      the remote fax and exchange info, the holdup function
     *      will gets after you call BfvFaxWaitForTraining()
     */
    printf( "BfvFaxGetRemoteInfo\n" );
    BT_ZERO(args_fax);
    BfvFaxGetRemoteInfo(lp, &args_fax);
    if (args_fax.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_fax.res,&bterr);
        printf ("BfvFaxGetRemoteInfo: %s\n",bterr.long_msg);
        goto err;
    }
    else
        printf ("Remote ID: '%s'\n", args_fax.remote_info.remote_id );
    printf( "BfvFaxWaitForTraining\n" );
    BT_ZERO(args_fax);
    BfvFaxWaitForTraining(lp, &args_fax);
    if (args_fax.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_fax.res,&bterr);
        printf ("BfvFaxWaitForTraining: %s\n",bterr.long_msg);
        goto err;
    }
    
    /* Send step 4.1
     *      Before sending the file, check the results of the holdup
     *      Send EFF or G3 ?
     */
    BT_ZERO(args_fax);
    if (myHoldupArg.switchToG3)
    {
        printf( "BfvFaxSendFile '%s'\n", gG3FileName );
        args_fax.fname = gG3FileName;
        args_fax.fmt = DATA_G3;
    }
    else
    {
        printf( "BfvFaxSendFile '%s'\n", gEffFileName );
        args_fax.fname = gEffFileName;
    }
    
    /*
     * Send step 5:
     *      Call BfvFaxSendFile to send the data
     *
     *      At this point you could call BfvFaxSendData to send buffers
     *      of data received from elsewhere.
     */  
    BfvFaxSendFile(lp, &args_fax);
    if (args_fax.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_fax.res,&bterr);
        printf ("BfvFaxSendFile: %s\n",bterr.long_msg);
        goto err;
    }
    
    /*
     * Send step 6:
     *      Call BfvFaxEndOfDocument to finish up after all pages,
     *      wait for hangup.
     */
    printf( "BfvFaxEndOfDocument\n" );
    BT_ZERO(args_fax);
    if (BfvFaxEndOfDocument(lp, &args_fax) < 0)
    {
        BfvErrorMessage(lp,&args_fax.res,&bterr);
        printf ("BfvFaxEndOfDocument: %s\n",bterr.long_msg);
        goto err;
    }
    
    printf( "Send Done\n" );
    
err:
    process_results(lp,&fres,time(NULL) - start_time);
    
err_2:
    BT_ZERO(args_tel);
    BfvLineTerminateCall (lp, &args_tel);

    process_aoc(lp);

err_1:
    BT_ZERO(args_admin);
    BfvLineDetach (lp, &args_admin);
}

char *
conf_vals(int conf)
{
    switch (conf)
    {
      case FSK_MCF_1:
      case FSK_MCF_2:
        return("MCF");
      case FSK_RTN_1:
      case FSK_RTN_2:
        return("RTN");
      case FSK_RTP_1:
      case FSK_RTP_2:
        return("RTP");
      case FSK_PIN_1:
      case FSK_PIN_2:
        return("PIN");
      case FSK_PIP_1:
      case FSK_PIP_2:
        return("PIP");
      default:
        return("?");
    }
}

void
process_results(BTLINE *lp, FAX_RES *fres, long duration)
{
    static char *bft_names[] =
        {NULL,"T.434","BTM",NULL,"DTM",NULL,NULL,NULL,"EDIFACT"};
    static int bit_rates[] =
        {2400,4800,7200,9600,7200,9600,12000,14400,12000,14400,
         2400,4800,7200,9600,12000,14400,16800,19200,
         21600,24000,26400,28800,31200,33600};
    static int ecm_vals[] = {0,256,64};
    static char *fmt_names[2][2] = {{"MH","MR"},{"MMR","MMR"}};
    static char *res_names[] =
        {"200Hx100V(Norm)",
         "200Hx200V(Fine)",
         "200Hx400V",
         "300Hx300V",
         "400Hx400V",
         "600Hx600V", 
         "1200Hx1200V",
         "300Hx600V",
         "400Hx800V",
         "600Hx1200V",
         "100Hx100V",
        };
    static char *width_names[] = {"A4","B4","A3"};
    PAGE_RES *pageres;
    PAGE_RES *temp;
    int jj;
    
    /* loop down the page res list */
    pageres = fres->reslist_head;
    jj = 1;
    while (pageres)
    {
        printf("Page %2d:  ", jj);
        if (pageres->bft_type)
            printf("BFT type %s  ",bft_names[pageres->bft_type]);
        else
        {
            if (pageres->eff_page_type)
            {
                printf("EFF Page: ");
                if (pageres->eff_page_type & BT_EFF_JPEG_ENABLE)
                {
                    printf("JPEG(");
                    if (pageres->eff_page_type & BT_EFF_JPEG_FULLCOLOR)
                        printf("full color, ");
                    else
                        printf("grayscale, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_DEFAULT_TABLES)
                        printf("default tbls, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_12BIT)
                        printf("12bit, ");
                    else
                        printf("8bit, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_NOSUBSAMPLING)
                        printf("1:1:1, ");
                    else
                        printf("4:1:1, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_CUSTOM_ILLUMINANT)
                        printf("cust illum, ");
                    else
                        printf("std illum, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_CUSTOM_GAMUT)
                        printf("cust gamut");
                    else
                        printf("std gamut");
                    printf(")\n          ");
                }
                if (pageres->eff_page_type & BT_EFF_JBIG)
                {
                    printf("JBIG(");
                    if (pageres->eff_page_type & BT_EFF_JBIG_L0)
                        printf("L0");
                    else
                        printf("std");
                    printf(") ");
                }
                if (pageres->eff_page_type & BT_EFF_LOSSLESS)
                    printf("LOSSLESS COLOR ");
                if (pageres->eff_page_type & BT_EFF_PLANE_INTERLEAVE)
                    printf("PLANE_INTERLEAVE ");
                if (pageres->eff_page_type & BT_EFF_MRC_MASK)
                    printf("MRC(%X) ",
                           (pageres->eff_page_type & BT_EFF_MRC_MASK) >> 4);
                if (pageres->eff_page_type & BT_EFF_PAGE_LENGTH_STRIP)
                    printf("PAGE_LEN_STRIP ");
            }
            else
            {
                if (pageres->direction)
                    printf("ASCII %d  ",pageres->ascii_bytes);
                printf("BdLn %d  TotLn %d  ",
                       pageres->bad_lines, pageres->total_lines);
            }
            printf("Res %s  ",res_names[pageres->resolution]);
            printf("Width %s  ",width_names[pageres->width]);
        }
        printf("Bps %u\n",bit_rates[pageres->bit_rate]);
        
        printf("          ConfVal %s  SigLev %04X  LineNoise %04X",
               conf_vals(pageres->confirm_value),
               pageres->sig_level,pageres->line_noise);
        if (!pageres->direction)
            printf("  SigQual %04X",pageres->sig_quality);
        printf("\n");
        
        temp = pageres;
        pageres = pageres->next;
#ifdef MILL_DLL
        _dll_free(temp);
#else
        free(temp);
#endif
        jj++;
    }
    printf("Duration %ld secs  ",duration);
    printf("ECM %d  ",ecm_vals[LINE_DCS(lp)->Error_Correction_Mode]);
    printf("Line Format %s",
           fmt_names[LINE_DCS(lp)->MMR][LINE_DCS(lp)->TwoD_Coding]);
    printf("\n");
}

void
process_aoc(BTLINE *lp)
{
    AOC_INFO *aoc = LINE_AOC_INFO(lp);
        
    if (aoc == NULL || aoc->status == BT_AOC_UNAVAILABLE)
        printf("Charge information unavailable.\n");
    else if (aoc->status == BT_AOC_FREE)
        printf("Charge information: FREE.\n");
    else if (aoc->status == BT_AOC_UNITS)
        printf("Charge information: Unit type %d, value %ld\n",
               aoc->charge_info,aoc->charge);
    else if (aoc->status == BT_AOC_CURRENCY)
    {
        static char *currency_names[] = {
            "1/1000","1/100","1/10","1","10","100","1000" };
#define NUM_CURRENCIES (sizeof(currency_names) / sizeof(*currency_names))
        char *cur_name;
            
        cur_name = (0 <= aoc->charge_info &&
                    aoc->charge_info < NUM_CURRENCIES) ?
            currency_names[aoc->charge_info] :
            "Invalid currency";
        printf("Charge information: Currency multiplier %s (%d), value %ld\n",
               cur_name,aoc->charge_info,aoc->charge);
    }
    else
        printf("Charge information: Bad status value %d\n",
               aoc->status);
}

void
process_call_info(CALL_RES *call_res)
{
    if (strlen(call_res->called_party_number))
        printf("Called party number: \"%s\"\n",
            call_res->called_party_number);
    if (strlen(call_res->called_party_subaddress))
        printf("Called party subaddress: \"%s\"\n",
           call_res->called_party_subaddress);
    if (strlen(call_res->calling_party_number))
        printf("Calling party number: \"%s\"\n",
           call_res->calling_party_number);
    if (strlen(call_res->calling_party_subaddress))
        printf("Calling party subaddress: \"%s\"\n",
           call_res->calling_party_subaddress);
    if (strlen(call_res->redir_number))
        printf("Redirection number: \"%s\", reason %X\n",
           call_res->redir_number, call_res->redir_reason);
}
