/*
 * decode.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The decode program reads a specified infopkt stream file and lists the
individual infopkts of which the stream consists.
***************************************************************************/

#include <string.h>
#include "btlib.h"

static void putnice BT_ARGS ((int));
static char *data_fmt_name BT_ARGS ((unsigned));
static char *resolution_name BT_ARGS ((unsigned));
static char *width_name BT_ARGS ((unsigned));
static char *speech_sample_rate_name BT_ARGS ((unsigned));
static char *coding_format_name BT_ARGS ((unsigned));
static char *speech_bps_name BT_ARGS ((unsigned));
static char *speech_afe_name BT_ARGS ((unsigned));
static char *speech_data_fmt_name BT_ARGS ((unsigned));
static char *fax_bps_name BT_ARGS ((unsigned));
static char *fax_scan_time_name BT_ARGS ((unsigned));
static char *fax_mod_name BT_ARGS ((unsigned));
static char *fax_compr_name BT_ARGS ((unsigned));

void
usage ()
{
    fprintf (stderr, "usage: decode [-f] file\n");
    fprintf (stderr, "       -f = follow indirect infopkts\n");
    exit (1);
}

int
main (int argc, char **argv)
{
    int c;
    extern int optind;
    struct infopkt_stream *ips;
    struct infopkt *ip;
    struct g3strippkt *g3p;
    struct asciistrippkt *asciis;
    struct asciipagepkt *asciip;
    struct pageparampkt *pparm;
    struct cvsdparampkt *cvsdp;
    struct t30parampkt *t30p;
    struct docpkt *docp;
    struct SPI *spip;
    struct eospkt *eosp;
    struct promptpkt *promptp;
    struct prompt_phrase *phrases;
    struct faxhdrpkt *hdrpkt;
    struct bftpkt *bftpkt;
    struct overlaypkt *overlaypkt;
    struct spacepkt *spacepkt;
    struct effpagepkt *effpage;
    struct args_debug_mode args_debug;
    char *name;
    long offset;
    int i;
    struct asciipkt *ap;
    struct annotpkt *anp;
    int n;
    char *datap;
    int follow_flag = 0;
    struct args_infopkt args;
    BTERR bterr;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "f")) != EOF)
    {
        switch (c)
        {
          case 'f':
            follow_flag = 1;
            break;
          default:
            usage ();
        }
    }
    
    if (argc < optind)
        usage ();

    name = argv[optind++];
    
    if (argc != optind)
        usage ();

    BT_ZERO(args_debug);
    args_debug.mode = DEBUG_ALL;
    args_debug.options = API_DBG_MODE_SET;
    BfvDebugModeSetAdv(&args_debug);
    
    BT_ZERO(args);
    args.fname = name;
    args.fmode = "r";
    if ((ips = BfvInfopktOpen (&args)) == NULL)
    {
        BfvErrorMessage(NULL,&args.res,&bterr);
        fprintf (stderr, "BfvInfopktOpen: %s\n",bterr.long_msg);
        exit (1);
    }
    
    args.ips = ips;
    while (1)
    {
        offset = BfvInfopktFtell (&args);
        args.i_mode = (follow_flag) ? INDIR_MODE_FOLLOW : INDIR_MODE_RETURN;
        ip = BfvInfopktGet (&args);
        
        if (ip == NULL)
        {
            if (args.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(NULL,&args.res,&bterr);
                printf ("BfvInfopktGet: %s\n",bterr.long_msg);
            }
            break;
        }
        
        printf ("%ld(%d): ", offset, ip->length);
        switch (ip->type)
        {
          case INFOPKT_G3:
            printf ("g3");
            break;
          case INFOPKT_BTG3:
            printf ("btg3");
            break;
          case INFOPKT_ASCII:
            printf ("ascii ");
            if (ip->length < sizeof (struct asciipkt))
            {
                printf ("packet too short");
            }
            else
            {
                ap = (struct asciipkt *)ip;
                n = ap->hdr.length - sizeof (struct asciipkt);
                datap = (char *)ip + sizeof (struct asciipkt);
                putchar ('"');
                for (i = 0; i < n && i < 20 && datap[i]; i++)
                    putnice (datap[i]);
                if (i < n)
                    printf ("...");
                putchar ('"');
            }
            break;
          case INFOPKT_CVSD:
            printf ("cvsd");
            break;
          case INFOPKT_SPEECH:
            printf ("speech");
            break;
          case INFOPKT_G3_STRIP_PARAMETERS:
            g3p = (struct g3strippkt *)ip;
            printf("g3 strip params: resolution %d (%s), width %d (%s),\n",
                   g3p->resolution, resolution_name(g3p->resolution),
                   g3p->width, width_name(g3p->width));
            printf("\tdata_fmt %d (%s)",
                   g3p->data_fmt, data_fmt_name(g3p->data_fmt));
            break;
          case INFOPKT_ASCII_STRIP_PARAMETERS:
            asciis = (struct asciistrippkt *)ip;
            printf("ascii strip params: resolution %d (%s), width %d (%s),\n",
                   asciis->resolution, resolution_name(asciis->resolution),
                   asciis->width,width_name(asciis->width));
            printf("\teof char %X, font num %d, left_margin %d, ",
                   asciis->eof_char, asciis->font_no,
                   asciis->left_margin);
            printf("right margin %d, line spacing %d",
                   asciis->right_margin, asciis->line_spacing);
            break;
          case INFOPKT_ASCII_PAGE_PARAMETERS:
            asciip = (struct asciipagepkt *)ip;
            printf("ascii page params: top margin %d, bottom %d, ",
                   asciip->top_margin, asciip->bottom_margin);
            printf("length %d, pad flag %d", asciip->length,
                   asciip->pad_flag);
            break;
          case INFOPKT_PAGE_PARAMETERS:
            pparm = (struct pageparampkt *)ip;
            printf("page params: top margin %d, bottom %d, length %d, ascii pad %d,\n",
                   pparm->top_margin, pparm->bottom_margin,
                   pparm->length, pparm->ascii_pad);
            printf("\timage pad %d, image break %d, image margin %d",
                   pparm->image_pad, pparm->image_break, pparm->image_margin);
            break;
          case INFOPKT_CVSD_PARAMETERS:
            cvsdp = (struct cvsdparampkt *)ip;
            printf("cvsd param: rate %d, flags %d",
                   cvsdp->rate,cvsdp->flags);
            break;
          case INFOPKT_T30_PARAMETERS:
        {
            unsigned mod_type, line_compr, coding_type;
                
            t30p = (struct t30parampkt *)ip;
#define SIZE_WITH_MT (sizeof(struct infopkt) + 4 * sizeof(unsigned char))
            /* For compatibility with 3.7 and earlier */
            if (ip->length < SIZE_WITH_MT)
                mod_type = MODULATION_GENERIC;
            else
                mod_type = t30p->modulation_type;
#define SIZE_WITH_LC (sizeof(struct infopkt) + 5 * sizeof(unsigned char))
            /* For compatibility with 4.0 and earlier */
            if (ip->length < SIZE_WITH_LC)
            {
                line_compr = LINECOMPR_ANY;
                /* Location in structure moved, ensure value is 0 */
                coding_type = 0;
            }
            else
            {
                line_compr = t30p->line_compression;
                coding_type = t30p->coding_type;
            }       
            printf("T30 param: bit rate %d (%s), scan time %d (%s),\n",
                   t30p->bit_rate, fax_bps_name(t30p->bit_rate),
                   t30p->scan_time, fax_scan_time_name(t30p->scan_time));
            printf("\tmodulation %d (%s), line compression %d (%s),\n",
                   mod_type, fax_mod_name(mod_type),
                   line_compr, fax_compr_name(line_compr));
            printf("\tcoding type (unused) %d",
                   coding_type);
            break;
        }
          case INFOPKT_DOCUMENT_PARAMETERS:
            docp = (struct docpkt *)ip;
            printf("doc params: resolution %d (%s), width %d (%s),\n",
                   docp->resolution, resolution_name(docp->resolution),
                   docp->hor_width, width_name(docp->hor_width));
            printf("\tlength (unused) %d",
                   docp->vert_length);
            break;
          case INFOPKT_SPEECH_PARAMETERS:
            spip = (struct SPI *)ip;
            printf("speech params: format %u (%s), compr samp rate %u (%s),\n",
                   spip->coding_format,coding_format_name(spip->coding_format),
                   spip->sample_rate,
                   speech_sample_rate_name(spip->sample_rate));
            /* For compatibility with 3.6 and earlier */
            if (ip->length < sizeof(*spip))
                spip->agc = 0xff;
            printf("\tbits per samp %u (%s), AFE samp rate %u (%s), data format %X (%s),\n",
                   spip->bits_per_sample,
                   speech_bps_name(spip->bits_per_sample),
                   spip->afe_rate,speech_afe_name(spip->afe_rate),
                   spip->data_fmt,speech_data_fmt_name(spip->data_fmt));
            printf("\tagc %X",
                   spip->agc);
            break;
          case INFOPKT_INDIR_INFOPKT:
            printf ("indir ");
            if (ip->length < sizeof (struct indirinfopkt))
                printf ("packet too short");
            else
            {
                n = ip->length - sizeof (struct indirinfopkt);
                datap = (char *)ip+sizeof(struct indirinfopkt);
                putchar ('"');
                for (i = 0; i < n && datap[i]; i++)
                    putchar (datap[i]);
                putchar ('"');
            }
            break;
          case INFOPKT_INDIR_ASCII:
            printf ("indirascii");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_INDIR_G3:
            printf ("indirg3");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_INDIR_BTG3:
            printf ("indirbtg3");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_INDIR_TIFF:
            printf ("indirtiff");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_INDIR_DCX:
            printf ("indirdcx");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_INDIR_CVSD:
            printf ("indircvsd");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_INDIR_SPEECH:
            printf ("indirspeech");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_INDIR_WAVE:
            printf ("indirwave");
            printf(" filename %s", (char *)ip+sizeof(struct infopkt));
            break;
          case INFOPKT_BEGINNING_OF_PAGE:
            printf ("bop");
            break;
          case INFOPKT_END_OF_SPEECH:
            eosp = (struct eospkt *)ip;
            /* For compatibility with 3.2 and earlier */
            if (ip->length == sizeof(struct infopkt))
                eosp->mode = 0;
            printf ("eos: mode %u",eosp->mode);
            break;
          case INFOPKT_PROMPT_MAP:
            promptp = (struct promptpkt *)ip;
            phrases = (struct prompt_phrase *)
                (((char *)ip) + sizeof(struct promptpkt));
            printf("prompt map: num_phrases %u\n",promptp->num_phrases);
            for (i = 0; i < (int)promptp->num_phrases; i++)
            {
                if (i)
                    printf("\n");
                printf("            size %08X offset %08X",
                       phrases[i].phrase_size,phrases[i].offset);
            }
            break;
          case INFOPKT_ANNOTATION:
            printf ("annotation ");
            if (ip->length < sizeof (struct annotpkt))
            {
                printf ("packet too short");
                break;
            }
            else
            {
                anp = (struct annotpkt *)ip;
                n = anp->hdr.length - sizeof (struct annotpkt);
                datap = (char *)ip + sizeof (struct annotpkt);
                putchar ('"');
                for (i = 0; i < n && i < 50 && datap[i]; i++)
                    putnice (datap[i]);
                if (i < n)
                    printf ("...");
                putchar ('"');
            }
            break;
          case INFOPKT_FAX_HDR:
            hdrpkt = (struct faxhdrpkt *)ip;
            printf("fax hdr: placement %u, insertion mode %u\n\t\"",
                   hdrpkt->placement,hdrpkt->insert_mode);
            n = hdrpkt->hdr.length - sizeof (struct faxhdrpkt);
            datap = (char *)ip + sizeof (struct faxhdrpkt);
            for (i = 0; i < n && i < 66 && datap[i]; i++)
                putnice (datap[i]);
            if (i < n)
                printf ("...");
            putchar ('"');
            break;
          case INFOPKT_BFT:
            bftpkt = (struct bftpkt *)ip;
            switch (bftpkt->bft_type)
            {
              case BFT_T434:
                datap = "T.434 BFT";
                break;
              case BFT_BTM:
                datap = "BTM";
                break;
              case BFT_DTM:
                datap = "DTM";
                break;
              case BFT_EDIFACT:
                datap = "Edifact";
                break;
              default:
                datap = "Unknown";
                break;
            }
            printf("BFT: bft_type %d (%s)",bftpkt->bft_type,datap);
            break;
          case INFOPKT_OVERLAY:
            overlaypkt = (struct overlaypkt *)ip;
            printf("overlay: overlay_number %u",overlaypkt->overlay_number);
            break;
          case INFOPKT_SPACE:
            spacepkt = (struct spacepkt *)ip;
            switch (spacepkt->units)
            {
              case UNITS_INCHES_TENTHS:
                datap = "1/10 inches";
                break;
              case UNITS_SCAN_LINES:
                datap = "Scan lines";
                break;
              default:
                datap = "Unknown";
                break;
            }
            printf("space: spacing %d, units %u (%s)",
                   spacepkt->spacing,spacepkt->units,datap);
            break;
          case INFOPKT_EFF_PAGE_PARAMETERS:
            effpage = (struct effpagepkt *)ip;
            printf("EFF page params: eff_page_type %X",
                   effpage->eff_page_type);
            break;
          case INFOPKT_USER0:
          case INFOPKT_USER1:
          case INFOPKT_USER2:
          case INFOPKT_USER3:
          case INFOPKT_USER4:
          case INFOPKT_USER5:
          case INFOPKT_USER6:
          case INFOPKT_USER7:
          case INFOPKT_USER8:
          case INFOPKT_USER9:
            printf("user %d ...",ip->type - INFOPKT_USER0);
            break;
          default:
            printf ("unknown type %d", ip->type);
            break;
        }
        printf ("\n");
    }
    BfvInfopktClose(&args);
    
    return(0);
}

static void
putnice (int c)
{
    if (c & 0x80)
    {
        printf ("M-");
        c &= 0x7f;
    }
    if (c == 0177)
    {
        printf ("^?");
        return;
    }
    if (c < ' ')
    {
        putchar ('^');
        c += '@';
    }
    putchar (c);
}

static char *
data_fmt_name (unsigned fmt)
{
    switch (fmt)
    {
      case FMT_MH_UNALIGN_MSB:
        return("MH_UNALIGN_MSB");
      case FMT_MH_UNALIGN_LSB:
        return("MH_UNALIGN_LSB");
      case FMT_MH_ALIGN_MSB:
        return("MH_ALIGN_MSB");
      case FMT_MH_ALIGN_LSB:
        return("MH_ALIGN_LSB");
      case FMT_MR_UNALIGN_MSB:
        return("MR_UNALIGN_MSB");
      case FMT_MR_UNALIGN_LSB:
        return("MR_UNALIGN_LSB");
      case FMT_MR_ALIGN_MSB:
        return("MR_ALIGN_MSB");
      case FMT_MR_ALIGN_LSB:
        return("MR_ALIGN_LSB");
      case FMT_MMR_UNALIGN_MSB:
        return("MMR_UNALIGN_MSB");
      case FMT_MMR_UNALIGN_LSB:
        return("MMR_UNALIGN_LSB");
      case FMT_MMR_ALIGN_MSB:
        return("MMR_ALIGN_MSB");
      case FMT_MMR_ALIGN_LSB:
        return("MMR_ALIGN_LSB");
      case FMT_ASCII:
        return("ASCII");
      case FMT_PCX_BILEVEL:
        return("PCX_BILEVEL");
      default:
        return("unknown");
    }
}

static char *
resolution_name (unsigned res)
{
    switch (res)
    {
      case RES_200H_100V:
        return("200Hx100V(normal)");
      case RES_200H_200V:
        return("200Hx200V(fine)");
      case RES_200H_400V:
        return("200Hx400V");
      case RES_300H_300V:
        return("300Hx300V");
      case RES_400H_400V:
        return("400Hx400V");
      case RES_600H_600V:
        return("600Hx600V");
      case RES_1200H_1200V:
        return("1200Hx1200V");
      case RES_300H_600V:
        return("300Hx600V");
      case RES_400H_800V:
        return("400Hx800V");
      case RES_600H_1200V:
        return("600Hx1200V");
      case RES_100H_100V:
        return("100Hx100V");
      default:
        return("unknown");
    }
}

static char *
width_name (unsigned width)
{
    switch (width)
    {
      case WIDTH_A4:
        return("A4");
      case WIDTH_B4:
        return("B4");
      case WIDTH_A3:
        return("A3");
      default:
        return("unknown");
    }
}

static char *
speech_sample_rate_name (unsigned rate)
{
    switch (rate)
    {
      case RATE_6000:
        return("6000");
      case RATE_8000:
        return("8000");
      case RATE_20000:
        return("20000");
      case RATE_24000:
        return("24000");
      case RATE_28000:
        return("28000");
      case RATE_32000:
        return("32000");
      case RATE_16000:
        return("16000");
      case RATE_9600:
        return("9600");
      case RATE_11000:
        return("11000");
      case RATE_5300:
        return("5300");
      case RATE_6300:
        return("6300");
      case RATE_13000:
        return("13000");
      case RATE_12300:
        return("12300");      
      default:
        return("unknown");
    }
}

static char *
coding_format_name (unsigned fmt)
{
    switch (fmt)
    {
      case CODE_CVSD:
        return("CVSD");
      case CODE_ADPCM:
        return("ADPCM");
      case CODE_PCM_ULAW:
        return("PCM_ULAW");
      case CODE_PCM_ALAW:
        return("PCM_ALAW");
      case CODE_LINEAR:
        return("LINEAR");
      case CODE_SUBBAND:
        return("SUBBAND");
      case CODE_OKI_ADPCM:
        return("OKI_ADPCM");
      case CODE_G723_1:
        return("G723_1");
      case CODE_G729_A:
        return("G729_A");
      case CODE_SX7300:
        return("SX7300");
      case CODE_SX9600:
        return("SX9600");
      case CODE_RHET_ADPCM:
        return("RHET_ADPCM");
      case CODE_MS_GSM:
        return("MS_GSM");
      case CODE_DVI_ADPCM:
        return("DVI_ADPCM");
      case CODE_GSM_610:
        return("GSM_610");
      case CODE_GSM_660:
        return("GSM_660");
      case CODE_RAWDATA:
        return("RAWDATA");
      case CODE_G726:
        return("G726");
      default:
        return("unknown");
    }
}

static char *
speech_bps_name (unsigned bps)
{
    switch (bps)
    {
      case BITS_1:
        return("1");
      case BITS_2:
        return("2");
      case BITS_3:
        return("3");
      case BITS_4:
        return("4");
      case BITS_5:
        return("5");
      case BITS_8:
        return("8");
      case BITS_12:
        return("12");
      case BITS_16:
        return("16");
      case BITS_24:
        return("24");
      default:
        return("unknown");
    }
}

static char *
speech_afe_name (unsigned afe_rate)
{
    switch (afe_rate)
    {
      case AFE_8000:
        return("8000");
      case AFE_9600:
        return("9600");
      default:
        return("unknown");
    }
}

static char *
speech_data_fmt_name (unsigned data_fmt)
{
    switch (data_fmt)
    {
      case SPCH_MSB:
        return("MSB");
      case SPCH_LSB:
        return("LSB");
      default:
        return("unknown");
    }
}

static char *
fax_bps_name (unsigned bps)
{
    switch (bps)
    {
      case BITRATE_33600:
        return("33600");
      case BITRATE_31200:
        return("31200");
      case BITRATE_28800:
        return("28800");
      case BITRATE_26400:
        return("26400");
      case BITRATE_24000:
        return("24000");
      case BITRATE_21600:
        return("21600");
      case BITRATE_19200:
        return("19200");
      case BITRATE_16800:
        return("16800");
      case BITRATE_14400:
        return("14400");
      case BITRATE_12000:
        return("12000");
      case BITRATE_9600:
        return("9600");
      case BITRATE_7200:
        return("7200");
      case BITRATE_4800:
        return("4800");
      case BITRATE_2400:
        return("2400");
      case BITRATE_GENERIC:
        return("Generic");
      default:
        return("unknown");
    }
}

static char *
fax_scan_time_name (unsigned scan_time)
{
    switch (scan_time)
    {
      case SCANTIME_0:
        return("0");
      case SCANTIME_5:
        return("5");
      case SCANTIME_10:
        return("10");
      case SCANTIME_20:
        return("20");
      case SCANTIME_40:
        return("40");
      default:
        return("unknown");
    }
}

static char *
fax_mod_name (unsigned mod)
{
    switch (mod)
    {
      case MODULATION_V27:
        return("V27");
      case MODULATION_V29:
        return("V29");
      case MODULATION_V33:
        return("V33");
      case MODULATION_V17:
        return("V17");
      case MODULATION_V34:
        return("V34");
      case MODULATION_GENERIC:
        return("Generic");
      default:
        return("unknown");
    }
}

static char *
fax_compr_name (unsigned compr)
{
    switch (compr)
    {
      case LINECOMPR_ANY:
        return("Any");
      case LINECOMPR_MH:
        return("MH only");
      case LINECOMPR_MR_MH:
        return("MR/MH");
      case LINECOMPR_MMR_MR_MH:
        return("MMR/MR/MH");
      default:
        return("unknown");
    }
}
