/*
 * dstrip.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The dstrip program writes out individual PCX pages from a DCX file.
The first page is written to g3data.301, the second to g3data.302, and
so on, until all of the pages are written.
***************************************************************************/

#include <string.h>
#include <ctype.h>
#if defined(unix) || defined(M_XENIX) || defined(_AIX)
#include <unistd.h>
#endif
#include "btlib.h"

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

void usage()
{
    fprintf(stderr, "Usage: dstrip [-o <output_base>] <dcx_filename>\n");
    fprintf(stderr, "\t-o = use supplied output filename base\n");
    exit(1);
}

unsigned offset_table[1024];

int main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    char *nami, namo[512];
    int pagecnt = 1;
    FILE *infp,*outfp;
    int n;
    unsigned char buf[1024];
    static char output_base[512] = "g3data";
    unsigned page_offset;
    unsigned data_size;
    unsigned *ltmp;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "o:")) != EOF)
    {
        switch (c)
        {
          case 'o':
            strcpy(output_base,optarg);
            break;
          default:
            usage ();
        }
    }
    
    nami = argv[optind++];
    
    if (optind != argc)
        usage ();

#ifndef USES_FAT_FILESYSTEM
    if ((infp = fopen(nami, "r")) == NULL)
#else
    if ((infp = fopen(nami, "rb")) == NULL)
#endif
    {
        printf("error opening input DCX file\n");
        exit(1);
    }
        
    if (fread((char *)&data_size,sizeof(data_size),1,infp) != 1 ||
        data_size != DCX_ID)
    {
        printf("Invalid DCX file.\n");
        exit(1);
    }
        
    /* Read in entire offset table - at most 1024 entries */
    /* It's not an error if we can't read them all, since the file might
       be small. */
    if (fread((char *)offset_table,sizeof(unsigned),1024,infp) < (unsigned)1)
    {
        printf("Error reading page offsets from DCX file\n");
        exit(1);
    }
        
    sprintf(namo, "%s.3%02d", output_base, pagecnt);
#ifndef USES_FAT_FILESYSTEM
    if ((outfp = fopen(namo, "w")) == NULL)
#else
    if ((outfp = fopen(namo, "wb")) == NULL)
#endif
    {
        printf("error opening output file\n");
        exit(1);
    }
            
    for (;;)
    {
        if ((page_offset = offset_table[pagecnt - 1]) == 0)
            break;

        if (pagecnt != 1)
        {
            fclose(outfp);
            sprintf(namo, "%s.3%02d", output_base, pagecnt);
#ifndef USES_FAT_FILESYSTEM
            if ((outfp = fopen(namo, "w")) == NULL)
#else
            if ((outfp = fopen(namo, "wb")) == NULL)
#endif
            {
                printf("error opening output file\n");
                exit(1);
            }
        }
        pagecnt++;
                
        /* The size of each page is impossible to determine. Instead, we will
           scan the page offsets for the next higher than the current one. */
        data_size = 0x7fffffff;
        for (ltmp = offset_table; *ltmp; ltmp++)
        {
            if (*ltmp > page_offset && *ltmp < data_size)
                data_size = *ltmp;
        }
        if (data_size == 0x7fffffff)
        {
            if (fseek(infp,0L,SEEK_END))
            {
                printf("Error reading going to eof in DCX file\n");
                exit(1);
            }
            data_size = ftell(infp);
        }
        data_size -= page_offset;
                
        if (fseek(infp,(long)page_offset,SEEK_SET))
        {
            printf("Error reading going to page offset in DCX file\n");
            exit(1);
        }
                
        while (data_size > 0)
        {
            n = sizeof(buf);
            if (data_size < (unsigned)n)
                n = data_size;
            n = fread((char *)buf,1,n,infp);
            if (n <= 0)
            {
                printf("Error reading image data from DCX file, EOF.\n");
                exit(1);
            }
            data_size -= n;
            if (fwrite((char *)buf,n,1,outfp) != 1)
            {
                printf("Error writing output file\n");
                exit(1);
            }
        }
    }
            
    fclose(infp);
    fclose(outfp);
            
    return(0);
}
