/*
 * faxll.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The faxll program uses the low-level non-infopkt raw data fax
sending and receiving routines to send or receive facsimiles. This
application uses the BfvFaxSendFile function. It also uses the user function
feature of BfvLineOriginateCall to print call progress values.
***************************************************************************/

#include <string.h>
#include "btlib.h"

#define BOOL unsigned
#define TRUE    1
#define FALSE   0

int done;
BOOL send_mode = FALSE, recv_mode = FALSE;
unsigned current_type = DATA_G3;
int next_res = RES_200H_100V, next_bft = BFT_NONE;
int add_header = 0;
BOOL pass_through_mode = FALSE;
unsigned long next_eff = BT_EFF_DISABLE;
int hdr_overlay = 0;
int doing_regular_fax = 1;
int sending_next_page = 0;

void process_results BT_ARGS((BTLINE *lp,FAX_RES *fres,long duration));
void process_aoc BT_ARGS((BTLINE *lp));
void process_call_info BT_ARGS((CALL_RES *call_res));
#ifdef MSDOS
void nothing(void);
#endif
int cp_func BT_ARGS((BTLINE *lp,char *arg));

void
usage()
{
#if 0
    fprintf(stderr, "usage: faxll [-u <unitno>] [-v] [-h] [-H] [-f <fontfile>] -s <phoneno>\n");
#endif
    fprintf(stderr, "usage: faxll [-u <unitno>] [-v] [-h] [-H] -s <phoneno>\n");
#if 0
    fprintf(stderr, "\t\t[-g] [-F] [-a] [-p] <file> [-b] [-B #] [-E #] [-P] ...\n");
#endif
    fprintf(stderr, "\t\t[-F] [-E #] [-g] [-a] [-p] <file> [-b] ...\n");
#if 0
    fprintf(stderr, "or:    faxll [-u <unitno>] [-v] [-p] [-P] -r <g3file> ...\n");
#endif
    fprintf(stderr, "or:    faxll [-u <unitno>] [-v] [-p] -r <g3file> ...\n");
    fprintf(stderr, "\tu = unit number (fax card)\n");
    fprintf(stderr, "\ts = send mode\n");
    fprintf(stderr, "\tr = receive mode\n");
    fprintf(stderr, "\tv = turn on API debug mode\n");
    fprintf(stderr, "\th/H = insert/overlay a page header\n");
#if 0
    fprintf(stderr, "\tf = download font\n");
#endif
    fprintf(stderr, "\tg = following files are raw g3 data (default)\n");
    fprintf(stderr, "\ta = following files are raw ASCII text\n");
    fprintf(stderr, "\tp = following files are raw PCX data/receive PCX data\n");
    fprintf(stderr, "\tF = next page is fine resolution, otherwise normal\n");
    fprintf(stderr, "\tb = beginning of page\n");
#if 0
    fprintf(stderr, "\tB = beginning of BFT page of specified type (1=T.434, 2=BTM, 4=DTM)\n");
#endif
    fprintf(stderr, "\tE = beginning of Enhanced Fax Format page with specified options\n");
    fprintf(stderr, "\t    OR together the following hex values:\n");
    fprintf(stderr, "\t\t1=JPEG enabled, 2=Full Color, 4=Default Tables,\n");
    fprintf(stderr, "\t\t8=12 Bit, 10=No Subsampling, 20=Custom Illuminant,\n");
    fprintf(stderr, "\t\t40=Custom Gamut, 100=JBIG enabled, 200=L0 option,\n");
    fprintf(stderr, "\t\t400=Lossless Color, 800,1000,2000=MRC, 4000=Plane Interleave,\n");
    fprintf(stderr, "\t\t8000=Page Length Strip\n");
#if 0
    fprintf(stderr, "\tP = use G3 pass-through mode\n");
#endif
    fprintf(stderr, "\twhen sending, g3 and ascii files can be mixed");
    fprintf(stderr, "(-g f1 f2 -a f3 -b -g f4)\n" );
    fprintf(stderr, "\twhen receiving, each file name in the list receives a page of\n");
    fprintf(stderr, "\t\tg3 data (so there must be enough names)\n" );
    exit( 1 );
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp = NULL;
    int unit;
    char *snum;
    char *font = NULL;
    void dowork();
    struct args_line_admin args;
    struct args_debug_mode args_debug;
    BTERR bterr;
    
    BT_API_SET_VER();
    
    unit = 0;
    snum = ",1234";
    
#ifndef linux
    while ( (c = getopt( argc, argv, "u:rs:gavFhHB:E:Pp" )) != EOF )
    {
#else
    /* Linux getopt bug has forced the need for this kludge */
    for (optind = 1; optind < argc; optind++)
    {
        if (argv[optind] == NULL)
            usage();
        if (argv[optind][0] != '-')
            break;
        c = argv[optind][1];
        if (c == 'u' || c == 'f' || c == 's' || c == 'B' || c == 'E')
        {
            if (++optind == argc)
                usage();
            optarg = argv[optind];
        }
#endif
        switch ( c )
        {
          case 'u':
            unit = atoi( optarg );
            break;
#if 0
          case 'f':
            font = optarg;
            break;
#endif
          case 's':
            snum = optarg;
            send_mode = TRUE;
            break;
          case 'r':
            recv_mode = TRUE;
            break;
          case 'g':
            current_type = DATA_G3;
            break;
          case 'a':
            current_type = DATA_ASCII;
            break;
          case 'F':
            next_res = RES_200H_200V;
            break;
          case 'v':
            BT_ZERO(args_debug);
            args_debug.mode = DEBUG_ALL;
            args_debug.options = API_DBG_MODE_SET;
            BfvDebugModeSetAdv(&args_debug);
            break;
          case 'H':
            hdr_overlay = 1;
            /* fall through */
          case 'h':
            add_header = 1;
            break;
          case 'B':
            next_bft = atoi(optarg);
            doing_regular_fax = 0;
            break;
          case 'p':
            current_type = FMT_PCX_BILEVEL;
            break;
          case 'E':
            if (sscanf(optarg,"%x",&next_eff) != 1)
                usage();
            if (next_eff & BT_EFF_JPEG_ENABLE)
                /* JPEG square resolution only */
                next_res = RES_200H_200V;
            doing_regular_fax = 0;
            break;
          case 'P':
            pass_through_mode = TRUE;
            break;
          default:
            usage();
        }
    }
    
    if ((!send_mode && !recv_mode) || optind == argc || (current_type == DATA_ASCII && recv_mode))
        usage();

    BT_ZERO(args);
    args.unit = unit;
    lp = BfvLineAttach(&args);
    if (lp == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        printf ("BfvLineAttach: %s\n",bterr.long_msg);
        exit (1);
    }
    
    dowork(lp,snum,font,argc,argv);
    
    return(0);
}

void
dowork(BTLINE *lp, char *snum, char *font, int argc, char **argv)
{
    extern int optind;
    extern char *optarg;
    BTERR bterr;
    FAX_RES fres;
    long start_time;
    struct args_line_admin args_admin;
    struct args_telephone args_tel;
    struct args_fax args_fax;
    struct args_fax_strip_params args_fax_strip;
    int saved_res;
    
    BT_ZERO(fres);
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    if (BfvLineReset (lp, &args_admin) < 0)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset: %s: status %lX.\n",
                 bterr.long_msg,args_admin.reset_status);
        goto err_1;
    }

    if (!LINE_HAS_CAP(lp,CAP_FAX_XMIT))
    {
        fprintf (stderr, "unit %d does not support fax\n", LINE_UNIT_NUM(lp));
        goto err_1;
    }
    
    /* Set up fax result structure */
    LINE_FAX_RES(lp) = &fres;
    
    /*
     *   SSSS  EEEEE  N   N  DDDD
     *  S      E      NN  N  D   D
     *   SSS   EEEE   N N N  D   D
     *      S  E      N  NN  D   D
     *  SSSS   EEEEE  N   N  DDDD
     */
    if ( send_mode )
    {
#if 0
        if ( font != NULL )
        {
            BT_ZERO(args_fax);
            args_fax.fname = font;
            args_fax.font_no = 0;
            BfvFaxDownloadFont(lp, &args_fax);
            if (args_fax.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fax.res,&bterr);
                printf ("BfvFaxDownLoadFont: %s\n",bterr.long_msg);
                goto err_1;
            }
        }
#endif
        /*
         * Send step 1:
         *      Call BfvLineOriginateCall to establish the connection
         */
        printf( "BfvLineOriginateCall\n" );
        BT_ZERO(args_tel);
        args_tel.phonenum = snum;
        args_tel.call_protocol_code = CALL_PROTOCOL_FAX;
        args_tel.func = cp_func;
        BfvLineOriginateCall (lp, &args_tel);
        if (args_tel.res.status != BT_STATUS_OK || 
            LINE_STATE(lp) != LINE_STATE_CONNECTED)
        {
            BfvErrorMessage(lp,&args_tel.res,&bterr);
            printf ("BfvLineOriginateCall: %s\n",bterr.long_msg);
            goto err_2;
        }
        
        /* Start timing after we detect Fax answer tone */
        start_time = time(NULL);
        
        /*
         * Send step 2:
         *      Call BfvFaxBeginSendRaw to setup the fax for transmission
         */
        printf( "BfvFaxBeginSendRaw\n" );
        BT_ZERO(args_fax);
        args_fax.resolution = next_res;
        args_fax.width = WIDTH_A4;
        args_fax.bft_type = next_bft;
        args_fax.eff_page_type = next_eff;
        if (pass_through_mode)
            args_fax.g3_pt_enable = 1;
        BfvFaxBeginSendRaw(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxBeginSendRaw: %s\n",bterr.long_msg);
            goto err;
        }
        saved_res = next_res;
            
        /*
         * Send step 3:
         *      Call BfvFaxGetRemoteInfo and BfvFaxWaitForTraining to talk to
         *      the remote fax and exchange info
         */
        printf( "BfvFaxGetRemoteInfo\n" );
        BT_ZERO(args_fax);
        BfvFaxGetRemoteInfo(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxGetRemoteInfo: %s\n",bterr.long_msg);
            goto err;
        }
        else
            printf ("Remote ID: '%s'\n", args_fax.remote_info.remote_id );

        if (add_header)
        {
            char hdr_buf[128];
            
            if (next_eff != BT_EFF_DISABLE || pass_through_mode)
            {
                fprintf(stderr,
                        "Headers are not allowed with Enhanced Fax Formats or pass through mode\n");
                exit(1);
            }
            
            sprintf(hdr_buf,
                    "Fax sent to '%s' on %%A %%m/%%d/%%y at %%I:%%M:%%S%%p, page %%P",
                    args_fax.remote_info.remote_id);
            BT_ZERO(args_fax);
            args_fax.placement = HDR_HEADER;
            args_fax.insert_mode =
                (hdr_overlay) ? HDR_MODE_OVERLAY : HDR_MODE_INSERT;
            args_fax.label = hdr_buf;
            BfvFaxHeader(lp,&args_fax);
            if (args_fax.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fax.res,&bterr);
                printf ("BfvFaxHeader: %s\n",bterr.long_msg);
                goto err;
            }
            
            next_res = RES_200H_100V;
            next_bft = BFT_NONE;
            next_eff = BT_EFF_DISABLE;
        }
            
        printf( "BfvFaxWaitForTraining\n" );
        BT_ZERO(args_fax);
        BfvFaxWaitForTraining(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxWaitForTraining: %s\n",bterr.long_msg);
            goto err;
        }
        
        for (; optind < argc; optind++ )
        {
            /*
             * Send step 4:
             *      Call BfvFaxSendFile to send the data
             *
             *      At this point you could call BfvFaxSendData to send buffers
             *      of data received from elsewhere.
             */  
            if ( argv[ optind ][0] != '-' )
            {
                if (sending_next_page)
                {
                    printf( "BfvFaxNextPageRaw\n" );
                    BT_ZERO(args_fax);
                    args_fax.resolution = next_res;
                    args_fax.width = WIDTH_A4;
                    args_fax.bft_type = next_bft;
                    args_fax.eff_page_type = next_eff;
                    BfvFaxNextPageRaw(lp, &args_fax);
                    if (args_fax.res.status != BT_STATUS_OK)
                    {
                        BfvErrorMessage(lp,&args_fax.res,&bterr);
                        printf ("BfvFaxNextPageRaw: %s\n",bterr.long_msg);
                        goto err;
                    }
                    saved_res = next_res;
                    sending_next_page = 0;
                }

                if (doing_regular_fax)
                {
                    printf( "BfvFaxStripParams\n");
                    BT_ZERO(args_fax_strip);
                    args_fax_strip.fmt = current_type;
                    args_fax_strip.resolution = saved_res;
                    args_fax_strip.width = WIDTH_A4;
                    BfvFaxStripParams(lp, &args_fax_strip);
                    if (args_fax_strip.res.status != BT_STATUS_OK)
                    {
                        BfvErrorMessage(lp,&args_fax_strip.res,&bterr);
                        printf ("BfvFaxStripParams: %s\n",bterr.long_msg);
                        goto err;
                    }
                }
                else
                    doing_regular_fax = 1;

                printf( "BfvFaxSendFile '%s'\n", argv[ optind ] );
                BT_ZERO(args_fax);
                args_fax.fname = argv[optind];
                args_fax.fmt = current_type;
                BfvFaxSendFile(lp, &args_fax);
                if (args_fax.res.status != BT_STATUS_OK)
                {
                    BfvErrorMessage(lp,&args_fax.res,&bterr);
                    printf ("BfvFaxSendFile: %s\n",bterr.long_msg);
                    goto err;
                }
                next_res = RES_200H_100V;
                next_bft = BFT_NONE;
                next_eff = BT_EFF_DISABLE;
                /*
                 * Send step 5:
                 *      Call BfvFaxNextPageRaw if requested on command line
                 *      to end a page
                 */
            }
            else
            {
                int c = argv[optind][1];

                if (c == 'B' || c == 'E')
                {
                    if (++optind == argc)
                        usage();
                    optarg = argv[optind];
                    current_type = DATA_G3;
                }

                switch (c)
                {
                  case 'E':
                    if (sscanf(optarg,"%x",&next_eff) != 1)
                        usage();
                    if (next_eff & BT_EFF_JPEG_ENABLE)
                        /* JPEG square resolution only */
                        next_res = RES_200H_200V;
                    doing_regular_fax = 0;
                    /* fall through */
                  case 'B':
                    if (argv[ optind ][1] == 'B')
                    {
                        next_bft = atoi(optarg);
                        doing_regular_fax = 0;
                    }
                    /* fall through */
                  case 'b':
                    sending_next_page = 1;
                    break;
                  case 'g':
                    current_type = DATA_G3;
                    break;
                  case 'a':
                    current_type = DATA_ASCII;
                    break;
                  case 'p':
                    current_type = FMT_PCX_BILEVEL;
                    break;
                  case 'F':
                    next_res = RES_200H_200V;
                    break;
                }
            }
        }
        
        /*
         * Send step 6:
         *      Call BfvFaxEndOfDocument to finish up after all pages,
         *      wait for hangup.
         */
        printf( "BfvFaxEndOfDocument\n" );
        BT_ZERO(args_fax);
        if (BfvFaxEndOfDocument(lp, &args_fax) < 0)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxEndOfDocument: %s\n",bterr.long_msg);
            goto err;
        }
        
        printf( "Done\n" );
    }
    
    /*
     *  RRRR   EEEEE   CCCC  V   V 
     *  R   R  E      C      V   V
     *  RRRR   EEEE   C      V   V
     *  R  R   E      C       V V
     *  R   R  EEEEE   CCCC    V
     */
    if ( recv_mode )
    {
        /*
         * Receive step 1:
         *      Wait for an incoming call
         */
        printf( "BfvLineWaitForCall (ringing or DID digits)\n" );
        BT_ZERO(args_tel);
        BfvLineWaitForCall (lp, &args_tel);
        if (args_tel.res.status != BT_STATUS_OK || 
            args_tel.res.line_status != WAIT_FOR_CALL_OK)
        {
            BfvErrorMessage(lp,&args_tel.res,&bterr);
            printf ("BfvLineWaitForCall: %s\n",bterr.long_msg);
            goto err_1;
        }
        process_call_info(&args_tel.call_res);
        
        /* Start timing after we detect ringing */
        start_time = time(NULL);
        
        /*
         * Receive step 2:
         *      The call is here, pick it up
         */
        printf( "BfvLineAnswer\n" );
        BT_ZERO(args_tel);
        BfvLineAnswer (lp, &args_tel);
        if (args_tel.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_tel.res,&bterr);
            printf ("BfvLineAnswer: %s\n",bterr.long_msg);
            goto err_1;
        }
        
        /* Set the receive format. Not normally needed, but required
           if receiving in a non-standard format, such as PCX. */
        BT_ZERO(args_fax);
        args_fax.fmt = current_type;
        BfvFaxSetReceiveFmt(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxSetReceiveFmt: %s\n",bterr.long_msg);
            goto err;
        }
        
        /*
         * Receive step 3:
         *      Start receiving sequence
         */
        printf( "BfvFaxBeginReceive\n" );
        BT_ZERO(args_fax);
        if (pass_through_mode)
            args_fax.g3_pt_enable = 1;
        if (BfvFaxBeginReceive(lp, &args_fax) < 0)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxBeginReceive: %s\n",bterr.long_msg);
            goto err;
        }
        
        /*
         * Receive step 4:
         *      Call BfvFaxGetRemoteInfo and BfvFaxWaitForTraining to talk to
         *      the remote fax and exchange info
         */
        printf( "BfvFaxGetRemoteInfo\n" );
        BT_ZERO(args_fax);
        BfvFaxGetRemoteInfo(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxGetRemoteInfo: %s\n",bterr.long_msg);
            goto err;
        }
        else
            printf ("Remote ID: '%s'\n", args_fax.remote_info.remote_id );

        printf( "BfvFaxWaitForTraining\n" );
        BT_ZERO(args_fax);
        BfvFaxWaitForTraining(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxWaitForTraining: %s\n",bterr.long_msg);
            goto err;
        }
        
        do
        {
            /*
             * Receive step 5:
             *      Receive the page of fax data into a file
             *
             *      At this point you could call BfvFaxReceiveData to receive
             *      buffers of data to write to wherever.
             */
            printf( "BfvFaxReceiveFile '%s'\n", argv[ optind ] );
            BT_ZERO(args_fax);
            args_fax.fname = argv[optind];
            args_fax.expect_another = 0;
            BfvFaxReceiveFile(lp, &args_fax);
            if (args_fax.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fax.res,&bterr);
                printf ("BfvFaxReceiveFile: %s\n",bterr.long_msg);
                goto err;
            }
            optind++;
            if (args_fax.expect_another && optind == argc)
            {
                printf( "out of receive file names\n" );
                goto err;
            }
        }
        while (args_fax.expect_another);
        /*
         * Receive step 6:
         *      Wait for completion using BfvFaxEndReception.
         */
        printf( "BfvFaxEndReception\n" );
        BT_ZERO(args_fax);
        if (BfvFaxEndReception(lp, &args_fax) < 0)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxEndReception: %s\n",bterr.long_msg);
            goto err;
        }
        
        printf( "Done\n" );
    }
    
err:
    process_results(lp,&fres,time(NULL) - start_time);
    
err_2:
    BT_ZERO(args_tel);
    BfvLineTerminateCall (lp, &args_tel);

    process_aoc(lp);

err_1:
    BT_ZERO(args_admin);
    BfvLineDetach (lp, &args_admin);
    
    done = 1;
}

int cp_func (BTLINE *lp, char *arg)
{
    struct args_telephone args;
    
    BT_ZERO(args);
    while (BfvDataCP(lp,&args) == 0)
    {
        switch (args.type)
        {
          case CP_TYPE_LOW:
            printf("cp_func: Low  CP, value %02X\n",args.data);
            break;
          case CP_TYPE_HIGH:
            printf("cp_func: High CP, value %02X\n",args.data);
            break;
          case CP_TYPE_BOARD_DET:
            printf("cp_func: Line card-determined CP, value %02X\n",args.data);
            break;
          case CP_TYPE_CED:
            printf("cp_func: Fax answer tone detected.\n");
            break;
        }
    }
    return(0);
}

char *
conf_vals(int conf)
{
    switch (conf)
    {
      case FSK_MCF_1:
      case FSK_MCF_2:
        return("MCF");
      case FSK_RTN_1:
      case FSK_RTN_2:
        return("RTN");
      case FSK_RTP_1:
      case FSK_RTP_2:
        return("RTP");
      case FSK_PIN_1:
      case FSK_PIN_2:
        return("PIN");
      case FSK_PIP_1:
      case FSK_PIP_2:
        return("PIP");
      default:
        return("?");
    }
}

void
process_results(BTLINE *lp, FAX_RES *fres, long duration)
{
    static char *bft_names[] =
        {NULL,"T.434","BTM",NULL,"DTM",NULL,NULL,NULL,"EDIFACT"};
    static int bit_rates[] =
        {2400,4800,7200,9600,7200,9600,12000,14400,12000,14400,
         2400,4800,7200,9600,12000,14400,16800,19200,
         21600,24000,26400,28800,31200,33600};
    static int ecm_vals[] = {0,256,64};
    static char *fmt_names[2][2] = {{"MH","MR"},{"MMR","MMR"}};
    static char *res_names[] =
        {"200Hx100V(Norm)",
         "200Hx200V(Fine)",
         "200Hx400V",
         "300Hx300V",
         "400Hx400V",
         "600Hx600V", 
         "1200Hx1200V",
         "300Hx600V",
         "400Hx800V",
         "600Hx1200V",
         "100Hx100V",
        };
    static char *width_names[] = {"A4","B4","A3"};
    PAGE_RES *pageres;
    PAGE_RES *temp;
    int jj;
    
    /* loop down the page res list */
    pageres = fres->reslist_head;
    jj = 1;
    while (pageres)
    {
        printf("Page %2d:  ", jj);
        if (pageres->bft_type)
            printf("BFT type %s  ",bft_names[pageres->bft_type]);
        else
        {
            if (pageres->eff_page_type)
            {
                printf("EFF Page: ");
                if (pageres->eff_page_type & BT_EFF_JPEG_ENABLE)
                {
                    printf("JPEG(");
                    if (pageres->eff_page_type & BT_EFF_JPEG_FULLCOLOR)
                        printf("full color, ");
                    else
                        printf("grayscale, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_DEFAULT_TABLES)
                        printf("default tbls, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_12BIT)
                        printf("12bit, ");
                    else
                        printf("8bit, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_NOSUBSAMPLING)
                        printf("1:1:1, ");
                    else
                        printf("4:1:1, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_CUSTOM_ILLUMINANT)
                        printf("cust illum, ");
                    else
                        printf("std illum, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_CUSTOM_GAMUT)
                        printf("cust gamut");
                    else
                        printf("std gamut");
                    printf(")\n          ");
                }
                if (pageres->eff_page_type & BT_EFF_JBIG)
                {
                    printf("JBIG(");
                    if (pageres->eff_page_type & BT_EFF_JBIG_L0)
                        printf("L0");
                    else
                        printf("std");
                    printf(") ");
                }
                if (pageres->eff_page_type & BT_EFF_LOSSLESS)
                    printf("LOSSLESS COLOR ");
                if (pageres->eff_page_type & BT_EFF_PLANE_INTERLEAVE)
                    printf("PLANE_INTERLEAVE ");
                if (pageres->eff_page_type & BT_EFF_MRC_MASK)
                    printf("MRC(%X) ",
                           (pageres->eff_page_type & BT_EFF_MRC_MASK) >> 4);
                if (pageres->eff_page_type & BT_EFF_PAGE_LENGTH_STRIP)
                    printf("PAGE_LEN_STRIP ");
            }
            else
            {
                if (pageres->direction)
                    printf("ASCII %d  ",pageres->ascii_bytes);
                printf("BdLn %d  TotLn %d  ",
                       pageres->bad_lines, pageres->total_lines);
            }
            printf("Res %s  ",res_names[pageres->resolution]);
            printf("Width %s  ",width_names[pageres->width]);
        }
        printf("Bps %u\n",bit_rates[pageres->bit_rate]);
        
        printf("          ConfVal %s  SigLev %04X  LineNoise %04X",
               conf_vals(pageres->confirm_value),
               pageres->sig_level,pageres->line_noise);
        if (!pageres->direction)
            printf("  SigQual %04X",pageres->sig_quality);
        printf("\n");
        
        temp = pageres;
        pageres = pageres->next;
#ifdef MILL_DLL
        _dll_free(temp);
#else
        free(temp);
#endif
        jj++;
    }
    printf("Duration %ld secs  ",duration);
    printf("ECM %d  ",ecm_vals[LINE_DCS(lp)->Error_Correction_Mode]);
    printf("Line Format %s",
           fmt_names[LINE_DCS(lp)->MMR][LINE_DCS(lp)->TwoD_Coding]);
    printf("\n");
}

void
process_aoc(BTLINE *lp)
{
    AOC_INFO *aoc = LINE_AOC_INFO(lp);
        
    if (aoc == NULL || aoc->status == BT_AOC_UNAVAILABLE)
        printf("Charge information unavailable.\n");
    else if (aoc->status == BT_AOC_FREE)
        printf("Charge information: FREE.\n");
    else if (aoc->status == BT_AOC_UNITS)
        printf("Charge information: Unit type %d, value %ld\n",
               aoc->charge_info,aoc->charge);
    else if (aoc->status == BT_AOC_CURRENCY)
    {
        static char *currency_names[] = {
            "1/1000","1/100","1/10","1","10","100","1000" };
#define NUM_CURRENCIES (sizeof(currency_names) / sizeof(*currency_names))
        char *cur_name;
            
        cur_name = (0 <= aoc->charge_info &&
                    aoc->charge_info < NUM_CURRENCIES) ?
            currency_names[aoc->charge_info] :
            "Invalid currency";
        printf("Charge information: Currency multiplier %s (%d), value %ld\n",
               cur_name,aoc->charge_info,aoc->charge);
    }
    else
        printf("Charge information: Bad status value %d\n",
               aoc->status);
}

void
process_call_info(CALL_RES *call_res)
{
    if (strlen(call_res->called_party_number))
        printf("Called party number: \"%s\"\n",
            call_res->called_party_number);
    if (strlen(call_res->called_party_subaddress))
        printf("Called party subaddress: \"%s\"\n",
           call_res->called_party_subaddress);
    if (strlen(call_res->calling_party_number))
        printf("Calling party number: \"%s\"\n",
           call_res->calling_party_number);
    if (strlen(call_res->calling_party_subaddress))
        printf("Calling party subaddress: \"%s\"\n",
           call_res->calling_party_subaddress);
    if (strlen(call_res->redir_number))
        printf("Redirection number: \"%s\", reason %X\n",
           call_res->redir_number, call_res->redir_reason);
}
