/*
 * faxpml.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The faxpml program uses the medium level infopkt file fax polling
routines to send and/or receive facsimiles. It performs ordinary
sending and receiving functions and all polling variations. Users can
enter the local id at the command line.
***************************************************************************/

#include <string.h>
#include "btlib.h"

void process_results BT_ARGS((BTLINE *lp,FAX_RES *fres,long duration));
void process_aoc BT_ARGS((BTLINE *lp));
void process_call_info BT_ARGS((CALL_RES *call_res));

void
usage ()
{
    fprintf (stderr, "usage: faxpml [options]\n");
    fprintf (stderr, "  -c num      dial given phone number\n");
    fprintf (stderr, "  -a          answer\n");
    fprintf (stderr, "  -s infopktfile  file to send, if permitted\n");
    fprintf (stderr, "  -r infopktfile  file to receive, if permitted\n");
    fprintf (stderr, "  -u unitnum  use specified unit number\n");
    fprintf (stderr, "  -L          loop for testing\n");
    fprintf (stderr, " one of -c or -a is required\n");
    fprintf (stderr, " one or both of -s and -r is required\n");
    exit (1);
}

int cflag;
int aflag;
char *snum;
int unit;
int Lflag;
int done;

int
main (int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp;
    char *sname = NULL, *rname = NULL;
    void dowork();
    struct args_line_admin args;
    struct args_debug_mode args_debug;
    BTERR bterr;
    
    BT_API_SET_VER();
    BT_ZERO(args_debug);
    args_debug.mode = DEBUG_ALL;
    args_debug.options = API_DBG_MODE_SET;
    BfvDebugModeSetAdv(&args_debug);
    
    while ((c = getopt (argc, argv, "c:au:s:r:L")) != EOF)
    {
        switch (c)
        {
          case 'L':
            Lflag = 1;
            break;
          case 'u':
            unit = atoi (optarg);
            break;
          case 'c':
            cflag = 1;
            snum = optarg;
            break;
          case 'a':
            aflag = 1;
            break;
          case 's':
            sname = optarg;
            break;
          case 'r':
            rname = optarg;
            break;
          default:
            usage ();
        }
    }
    
    if (optind != argc)
        usage ();

    if (cflag + aflag != 1)
        usage ();

    if (sname == NULL && rname == NULL)
        usage ();

    BT_ZERO(args);
    args.unit = unit;
    if ((lp = BfvLineAttach (&args)) == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        fprintf (stderr,"BfvLineAttach: %s\n",bterr.long_msg);
        exit (1);
    }
    
    dowork (lp,sname,rname);
    
    return(0);
}

void
dowork(BTLINE *lp, char *sname, char *rname)
{
    struct infopkt_stream *s_ips = NULL, *r_ips = NULL;
    BTERR bterr;
    FAX_RES fax_res;
    long start_time;
#define SENDING 0
#define RECEIVING 1
    int cur_dir;
    struct args_line_admin args_admin;
    struct args_telephone args_tel;
    struct args_fax args_fax;
    struct args_infopkt args_infopkt;
    
loop:
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    if (BfvLineReset (lp, &args_admin) < 0)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset: %s: status %lX.\n",
                 bterr.long_msg,args_admin.reset_status);
        exit(1);
    }
    
    if (!LINE_HAS_CAP(lp,CAP_FAX_XMIT))
    {
        fprintf (stderr, "unit %d does not support fax\n", LINE_UNIT_NUM(lp));
        exit(1);
    }
    
    if (sname)
    {
        BT_ZERO(args_infopkt);
        args_infopkt.fname = sname;
        args_infopkt.fmode = "r";
        if ((s_ips = BfvInfopktOpen (&args_infopkt)) == NULL)
        {
            BfvErrorMessage(NULL,&args_infopkt.res,&bterr);
            fprintf (stderr, "BfvInfopktOpen %s: %s\n",sname,bterr.long_msg);
            exit(1);
        }
    }
    
    if (rname)
    {
        BT_ZERO(args_infopkt);
        args_infopkt.fname = rname;
        args_infopkt.fmode = "w";
        if ((r_ips = BfvInfopktOpen (&args_infopkt)) == NULL)
        {
            BfvErrorMessage(NULL,&args_infopkt.res,&bterr);
            fprintf (stderr, "BfvInfopktOpen %s: %s\n",rname,bterr.long_msg);
            exit(1);
        }
    }
    
    BT_ZERO(fax_res);
    LINE_FAX_RES(lp) = &fax_res;
    
    if (aflag)
    {
        BT_ZERO(args_tel);
        BfvLineWaitForCall (lp, &args_tel);
        if (args_tel.res.status != BT_STATUS_OK || 
            args_tel.res.line_status != WAIT_FOR_CALL_OK)
        {
            BfvErrorMessage(lp,&args_tel.res,&bterr);
            printf ("BfvLineWaitForCall: %s\n",bterr.long_msg);
            goto got_err_1;
        }
        process_call_info(&args_tel.call_res);
        
        /* Start timing after we detect ringing */
        start_time = time(NULL);
        
        BT_ZERO(args_tel);
        BfvLineAnswer (lp, &args_tel);
        if (args_tel.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_tel.res,&bterr);
            printf ("BfvLineAnswer: %s\n",bterr.long_msg);
            goto got_err_1;
        }
        
        BT_ZERO(args_fax);
        args_fax.local_id = "answering";
        BfvFaxSetLocalId(lp, &args_fax);
        
        cur_dir = RECEIVING;
        args_fax.oa_flag = BT_ANSWER;
        args_fax.s_ips = s_ips;
        args_fax.r_ips = r_ips;
        BfvFaxBegin(lp, &args_fax);
    }
    else
    {
        BT_ZERO(args_tel);
        args_tel.phonenum = snum;
        args_tel.call_protocol_code = CALL_PROTOCOL_FAX;
        BfvLineOriginateCall (lp, &args_tel);
        if (args_tel.res.status != BT_STATUS_OK || 
            LINE_STATE(lp) != LINE_STATE_CONNECTED)
        {
            BfvErrorMessage(lp,&args_tel.res,&bterr);
            printf ("BfvLineOriginateCall: %s\n",bterr.long_msg);
            goto got_err_2;
        }
        
        /* Start timing after we detect Fax answer tone */
        start_time = time(NULL);
        
        BT_ZERO(args_fax);
        args_fax.local_id = "dialing";
        BfvFaxSetLocalId(lp, &args_fax);
        
        cur_dir = SENDING;
        args_fax.oa_flag = BT_ORIGINATE;
        args_fax.s_ips = s_ips;
        args_fax.r_ips = r_ips;
        BfvFaxBegin(lp, &args_fax);
    }
    
    if (args_fax.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_fax.res,&bterr);
        printf ("BfvFaxBegin: %s\n",bterr.long_msg);
        goto got_err;
    }
    
    do
    {
        BT_ZERO(args_fax);
        BfvFaxGetRemoteInfo(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxGetRemoteInfo: %s\n",bterr.long_msg);
            goto got_err;
        }
        BT_ZERO(args_fax);
        BfvFaxWaitForTraining(lp, &args_fax);
        if (args_fax.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(lp,&args_fax.res,&bterr);
            printf ("BfvFaxWaitForTraining: %s\n",bterr.long_msg);
            goto got_err;
        }
        if (LINE_STATE(lp) == LINE_STATE_TURNAROUND)
        {
            cur_dir = !cur_dir;
            continue;
        }
        
        switch (cur_dir)
        {
          case SENDING:
            for (;;)
            {
                BT_ZERO(args_fax);
                args_fax.s_ips = s_ips;
                if (BfvFaxNextPage (lp, &args_fax) <= 0)
                    break;
                BT_ZERO(args_fax);
                args_fax.s_ips = s_ips;
                BfvFaxSendPage (lp, &args_fax);
                if (args_fax.res.status != BT_STATUS_OK)
                {
                    BfvErrorMessage(lp,&args_fax.res,&bterr);
                    printf ("BfvFaxSendPage: %s\n",bterr.long_msg);
                    goto got_err;
                }
            }
            if (args_fax.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fax.res,&bterr);
                printf ("BfvFaxNextPage: %s\n",bterr.long_msg);
                goto got_err;
            }
            BT_ZERO(args_fax);
            BfvFaxEndOfDocument (lp, &args_fax);
            if (args_fax.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fax.res,&bterr);
                printf ("BfvFaxEndOfDocument: %s\n",bterr.long_msg);
                goto got_err;
            }
            break;
          case RECEIVING:
            BT_ZERO(args_fax);
            args_fax.r_ips = r_ips;
            BfvFaxReceivePages(lp, &args_fax);
            if (args_fax.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fax.res,&bterr);
                printf ("BfvFaxReceivePages: %s\n",bterr.long_msg);
                goto got_err;
            }
            break;
        }
        cur_dir = !cur_dir;
    }
    while (LINE_STATE(lp) == LINE_STATE_TURNAROUND);
    
got_err:
    process_results(lp,&fax_res,time(NULL) - start_time);
    
got_err_2:
    BT_ZERO(args_tel);
    BfvLineTerminateCall (lp, &args_tel);

    process_aoc(lp);

got_err_1:
    if (s_ips)
    {
        BT_ZERO(args_infopkt);
        args_infopkt.ips = s_ips;
        BfvInfopktClose (&args_infopkt);
    }
    if (r_ips)
    {
        BT_ZERO(args_infopkt);
        args_infopkt.ips = r_ips;
        BfvInfopktClose (&args_infopkt);
    }
    
    if (Lflag)
        goto loop;

    BT_ZERO(args_admin);
    BfvLineDetach (lp, &args_admin);
    
    exit(0);
}

char *
conf_vals(int conf)
{
    switch (conf)
    {
      case FSK_MCF_1:
      case FSK_MCF_2:
        return("MCF");
      case FSK_RTN_1:
      case FSK_RTN_2:
        return("RTN");
      case FSK_RTP_1:
      case FSK_RTP_2:
        return("RTP");
      case FSK_PIN_1:
      case FSK_PIN_2:
        return("PIN");
      case FSK_PIP_1:
      case FSK_PIP_2:
        return("PIP");
      default:
        return("?");
    }
}

void
process_results(BTLINE *lp, FAX_RES *fres, long duration)
{
    static char *bft_names[] =
        {NULL,"T.434","BTM",NULL,"DTM",NULL,NULL,NULL,"EDIFACT"};
    static int bit_rates[] =
        {2400,4800,7200,9600,7200,9600,12000,14400,12000,14400,
         2400,4800,7200,9600,12000,14400,16800,19200,
         21600,24000,26400,28800,31200,33600};
    static int ecm_vals[] = {0,256,64};
    static char *fmt_names[2][2] = {{"MH","MR"},{"MMR","MMR"}};
    static char *res_names[] =
        {"200Hx100V(Norm)",
         "200Hx200V(Fine)",
         "200Hx400V",
         "300Hx300V",
         "400Hx400V",
         "600Hx600V", 
         "1200Hx1200V",
         "300Hx600V",
         "400Hx800V",
         "600Hx1200V",
         "100Hx100V",
        };
    static char *width_names[] = {"A4","B4","A3"};
    PAGE_RES *pageres;
    PAGE_RES *temp;
    int jj;
    
    /* loop down the page res list */
    pageres = fres->reslist_head;
    jj = 1;
    while (pageres)
    {
        printf("Page %2d:  ", jj);
        if (pageres->bft_type)
            printf("BFT type %s  ",bft_names[pageres->bft_type]);
        else
        {
            if (pageres->eff_page_type)
            {
                printf("EFF Page: ");
                if (pageres->eff_page_type & BT_EFF_JPEG_ENABLE)
                {
                    printf("JPEG(");
                    if (pageres->eff_page_type & BT_EFF_JPEG_FULLCOLOR)
                        printf("full color, ");
                    else
                        printf("grayscale, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_DEFAULT_TABLES)
                        printf("default tbls, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_12BIT)
                        printf("12bit, ");
                    else
                        printf("8bit, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_NOSUBSAMPLING)
                        printf("1:1:1, ");
                    else
                        printf("4:1:1, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_CUSTOM_ILLUMINANT)
                        printf("cust illum, ");
                    else
                        printf("std illum, ");
                    if (pageres->eff_page_type & BT_EFF_JPEG_CUSTOM_GAMUT)
                        printf("cust gamut");
                    else
                        printf("std gamut");
                    printf(")\n          ");
                }
                if (pageres->eff_page_type & BT_EFF_JBIG)
                {
                    printf("JBIG(");
                    if (pageres->eff_page_type & BT_EFF_JBIG_L0)
                        printf("L0");
                    else
                        printf("std");
                    printf(") ");
                }
                if (pageres->eff_page_type & BT_EFF_LOSSLESS)
                    printf("LOSSLESS COLOR ");
                if (pageres->eff_page_type & BT_EFF_PLANE_INTERLEAVE)
                    printf("PLANE_INTERLEAVE ");
                if (pageres->eff_page_type & BT_EFF_MRC_MASK)
                    printf("MRC(%X) ",
                           (pageres->eff_page_type & BT_EFF_MRC_MASK) >> 4);
                if (pageres->eff_page_type & BT_EFF_PAGE_LENGTH_STRIP)
                    printf("PAGE_LEN_STRIP ");
            }
            else
            {
                if (pageres->direction)
                    printf("ASCII %d  ",pageres->ascii_bytes);
                printf("BdLn %d  TotLn %d  ",
                       pageres->bad_lines, pageres->total_lines);
            }
            printf("Res %s  ",res_names[pageres->resolution]);
            printf("Width %s  ",width_names[pageres->width]);
        }
        printf("Bps %u\n",bit_rates[pageres->bit_rate]);
        
        printf("          ConfVal %s  SigLev %04X  LineNoise %04X",
               conf_vals(pageres->confirm_value),
               pageres->sig_level,pageres->line_noise);
        if (!pageres->direction)
            printf("  SigQual %04X",pageres->sig_quality);
        printf("\n");
        
        temp = pageres;
        pageres = pageres->next;
#ifdef MILL_DLL
        _dll_free(temp);
#else
        free(temp);
#endif
        jj++;
    }
    printf("Duration %ld secs  ",duration);
    printf("ECM %d  ",ecm_vals[LINE_DCS(lp)->Error_Correction_Mode]);
    printf("Line Format %s",
           fmt_names[LINE_DCS(lp)->MMR][LINE_DCS(lp)->TwoD_Coding]);
    printf("\n");
}

void
process_aoc(BTLINE *lp)
{
    AOC_INFO *aoc = LINE_AOC_INFO(lp);
        
    if (aoc == NULL || aoc->status == BT_AOC_UNAVAILABLE)
        printf("Charge information unavailable.\n");
    else if (aoc->status == BT_AOC_FREE)
        printf("Charge information: FREE.\n");
    else if (aoc->status == BT_AOC_UNITS)
        printf("Charge information: Unit type %d, value %ld\n",
               aoc->charge_info,aoc->charge);
    else if (aoc->status == BT_AOC_CURRENCY)
    {
        static char *currency_names[] = {
            "1/1000","1/100","1/10","1","10","100","1000" };
#define NUM_CURRENCIES (sizeof(currency_names) / sizeof(*currency_names))
        char *cur_name;
            
        cur_name = (0 <= aoc->charge_info &&
                    aoc->charge_info < NUM_CURRENCIES) ?
            currency_names[aoc->charge_info] :
            "Invalid currency";
        printf("Charge information: Currency multiplier %s (%d), value %ld\n",
               cur_name,aoc->charge_info,aoc->charge);
    }
    else
        printf("Charge information: Bad status value %d\n",
               aoc->status);
}

void
process_call_info(CALL_RES *call_res)
{
    if (strlen(call_res->called_party_number))
        printf("Called party number: \"%s\"\n",
            call_res->called_party_number);
    if (strlen(call_res->called_party_subaddress))
        printf("Called party subaddress: \"%s\"\n",
           call_res->called_party_subaddress);
    if (strlen(call_res->calling_party_number))
        printf("Calling party number: \"%s\"\n",
           call_res->calling_party_number);
    if (strlen(call_res->calling_party_subaddress))
        printf("Calling party subaddress: \"%s\"\n",
           call_res->calling_party_subaddress);
    if (strlen(call_res->redir_number))
        printf("Redirection number: \"%s\", reason %X\n",
           call_res->redir_number, call_res->redir_reason);
}
