/*
 * mkdcx.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The mkdcx program creates a DCX file from a collection of raw PCX
files.
***************************************************************************/

#include <string.h>
#if defined(unix) || defined(M_XENIX) || defined(_AIX)
#include <unistd.h>
#endif
#include "btlib.h"

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

int total_pages;

void usage()
{
    fprintf(stderr, "Usage: mkdcx -o <dcx_file> <pcxfilename> ...\n");
    exit(1);
}

unsigned offset_table[1024];

int 
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    char *infile, *outfile;
    FILE *infp,*outfp;
    unsigned ltmp;
    int cur_page = 0;
    unsigned char buffer[1024];
    int n;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "o:")) != EOF)
    {
        switch (c)
        {
          case 'o':
            outfile = optarg;
            break;
          default:
            usage ();
        }
    }
    
    if ((total_pages = argc - optind) <= 0)
        usage ();

    if (total_pages > 1023)
    {
        printf("Too many pages specified.\n");
        exit(1);
    }
    
    if (outfile == NULL)
        usage ();

#ifdef USES_FAT_FILESYSTEM
    outfp = fopen(outfile,"wb+");
#else
    outfp = fopen(outfile,"w+");
#endif
    if (outfp == NULL)
    {
        fprintf (stderr, "Can't create %s\n", outfile);
        exit (1);
    }
    
    ltmp = DCX_ID;
    /* Write the DCX ID and preliminary offset table */
    if (fwrite((char *)&ltmp,sizeof(ltmp),1,outfp) != 1 ||
        fwrite((char *)offset_table,sizeof(unsigned),total_pages + 1,outfp) !=
        (unsigned)total_pages + 1)
    {
        fprintf(stderr,"Error writing output file\n");
        exit(1);
    }
    
    while (optind < argc)
    {
        infile = argv[optind++];
#ifdef USES_FAT_FILESYSTEM
        if ((infp = fopen(infile,"rb")) == NULL)
#else
        if ((infp = fopen(infile,"r")) == NULL)
#endif
        {
            fprintf(stderr,"Can't read %s\n",infile);
            break;
        }
        
        offset_table[cur_page++] = ftell(outfp);
        
        while ((n = fread((char *)buffer,1,sizeof(buffer),infp)) > 0)
        {
            if (fwrite((char *)buffer,n,1,outfp) != 1)
            {
                fprintf(stderr,"Error writing output file\n");
                exit(1);
            }
        }

        fclose(infp);
    }
    
    /* Update the offset table */
    if (fseek(outfp,4L,SEEK_SET) ||
        fwrite((char *)offset_table,sizeof(unsigned),total_pages + 1,outfp) !=
        (unsigned)total_pages + 1)
    {
        fprintf(stderr,"Error writing output file\n");
        exit(1);
    }
    
    fclose(outfp);
    
    return(0);
}
