/*
 * mkinfopk.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The mkinfopk program builds an infopkt stream file.
***************************************************************************/

#include <string.h>
#include <ctype.h>
#include <memory.h>
#include "btlib.h"

void
usage ()
{
    printf("usage: mkinfopk -o output_fname [-i input_fname] {infopkt_type arg} ...\n");
    printf("  For the following types, a filename arg is required:\n");
    printf("    ascii, g3, btg3, speech, indir[infopkt], indirascii, indirg3,\n");
    printf("    indirbtg3, indirspeech, indirtiff, indirdcx, indirwave\n");
    printf("  For the following types, if the arg is @, you will be\n");
    printf("  prompted for parameter values, otherwise default parameter\n");
    printf("  values will be used:\n");
    printf("     doc, bop, g3_strip, ascii_strip, page, t30, eos, spi, fax_hdr,\n");
    printf("     bft, overlay, space, eff\n");
    printf("  For the following type, either a filename arg or @ is required:\n");
    printf("     annot\n");
    printf("  If -i is used, remaining arguments will be read from a file.\n");
    
    exit (1);
}

int cvsd_used;
int ascpp_used;
void prompt_param BT_ARGS((char *vname, unsigned short *pval, char *fmt));
void prompt_param_int BT_ARGS((char *vname, unsigned *pval, char *fmt));
int process BT_ARGS((struct infopkt_stream *,char *,FILE *));
void put_pkt BT_ARGS((struct args_infopkt *args));

int
main (int argc, char **argv)
{
    char cmd_line[1024];
    char *args_array[50];
    int c;
    extern char *optarg;
    extern int optind;
    struct infopkt_stream *ips;
    FILE *fp;
    char *output_fname = NULL;
    char *input_fname = NULL;
    char *name;
    char *type;
    int n, ret;
    char buf[1024];
    struct indirinfopkt *indirp;
    struct indirasciipkt *indirasc;
    struct indirg3pkt *indirg3;
    struct indirbtg3pkt *indirbtg3;
    struct indirtiffpkt *indirtiff;
    struct indirdcxpkt *indirdcx;
    struct indircvsdpkt *indircvsd;
    struct g3strippkt *g3s;
    struct asciistrippkt *asciis;
    struct asciipagepkt *asciip;
    struct pageparampkt *pparm;
    struct cvsdparampkt *cvsdp;
    struct t30parampkt *t30p;
    struct boppkt *bopp;
    struct docpkt *docp;
    struct eospkt *eosp;
    struct indirspeechpkt *indirspeech;
    struct indirwavepkt *indirwave;
    struct SPI *spip;
    struct annotpkt *anp;
    struct faxhdrpkt *hdrpkt;
    struct bftpkt *bftpkt;
    struct overlaypkt *overlaypkt;
    struct spacepkt *spacepkt;
    struct effpagepkt *effpage;
    struct args_infopkt args;
    struct args_debug_mode args_debug;
    BTERR bterr;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "o:i:")) != EOF)
    {
        switch (c)
        {
          case 'o':
            output_fname = optarg;
            break;
          case 'i':
            input_fname = optarg;
            break;
          default:
            usage ();
        }
    }
    
    if (input_fname == NULL && optind >= argc)
        usage ();

    if (output_fname == NULL)
        usage ();

    BT_ZERO(args_debug);
    args_debug.mode = DEBUG_ALL;
    args_debug.options = API_DBG_MODE_SET;
    BfvDebugModeSetAdv(&args_debug);
    
    BT_ZERO(args);
    args.fname = output_fname;
    args.fmode = "w";
    if ((ips = BfvInfopktOpen (&args)) == NULL)
    {
        BfvErrorMessage(NULL,&args.res,&bterr);
        fprintf (stderr, "BfvInfopktOpen: %s\n",bterr.long_msg);
        exit (1);
    }
    
    if (input_fname)
    {
        char *tmp;
        
        if (optind < argc)
        {
            fprintf(stderr,"Additional arguments must not be specified if input_fname is used.\n");
            exit(1);
        }
        
        fp = fopen(input_fname,"r");
        if (fp == NULL)
        {
            fprintf(stderr,"Error opening input file %s\n",input_fname);
            exit(1);
        }
        if (fgets(cmd_line,sizeof(cmd_line),fp) == NULL)
        {
            fprintf(stderr,"Error reading input file.\n");
            exit(1);
        }
        fclose(fp);
        
        argc = 0;
        argv = args_array;
        optind = 0;
        tmp = cmd_line;
        for (;;)
        {
            while (*tmp && isspace(*tmp))
                tmp++;
            if (!*tmp)
                break;
            if (argc >= (sizeof(args_array) / sizeof(*args_array)))
            {
                fprintf(stderr,"Too many commands in input file.\n");
                exit(1);
            }
            argv[argc++] = tmp;
            while (*tmp && !isspace(*tmp))
                tmp++;
            if (!*tmp)
                break;
            *tmp = 0;
            tmp++;
        }
        argv[argc] = NULL;
    }
    
    while (optind + 1 < argc)
    {
        type = argv[optind++];
        name = argv[optind++];
        
        if (strcmp (type, "indirinfopkt") == 0 ||
            strcmp (type, "indir") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirinfopkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirp = (struct indirinfopkt *)buf;
            indirp->hdr.type = INFOPKT_INDIR_INFOPKT;
            indirp->hdr.length = sizeof (struct indirinfopkt) + n;
            strcpy (buf + sizeof (struct indirinfopkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirp;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indirascii") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirasciipkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirasc = (struct indirasciipkt *)buf;
            indirasc->hdr.type = INFOPKT_INDIR_ASCII;
            indirasc->hdr.length = sizeof (struct indirasciipkt) + n;
            strcpy (buf + sizeof (struct indirasciipkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirasc;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indirg3") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirg3pkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirg3 = (struct indirg3pkt *)buf;
            indirg3->hdr.type = INFOPKT_INDIR_G3;
            indirg3->hdr.length = sizeof (struct indirg3pkt) + n;
            strcpy (buf + sizeof (struct indirg3pkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirg3;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indirbtg3") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirbtg3pkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirbtg3 = (struct indirbtg3pkt *)buf;
            indirbtg3->hdr.type = INFOPKT_INDIR_BTG3;
            indirbtg3->hdr.length = sizeof (struct indirbtg3pkt) + n;
            strcpy (buf + sizeof (struct indirbtg3pkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirbtg3;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indirtiff") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirtiffpkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirtiff = (struct indirtiffpkt *)buf;
            indirtiff->hdr.type = INFOPKT_INDIR_TIFF;
            indirtiff->hdr.length = sizeof (struct indirtiffpkt) + n;
            strcpy (buf + sizeof (struct indirtiffpkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirtiff;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indirdcx") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirdcxpkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirdcx = (struct indirdcxpkt *)buf;
            indirdcx->hdr.type = INFOPKT_INDIR_DCX;
            indirdcx->hdr.length = sizeof (struct indirdcxpkt) + n;
            strcpy (buf + sizeof (struct indirdcxpkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirdcx;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indircvsd") == 0)
        {
            cvsd_used = 1;
            n = strlen (name) + 1;
            if (sizeof (struct indircvsdpkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indircvsd = (struct indircvsdpkt *)buf;
            indircvsd->hdr.type = INFOPKT_INDIR_CVSD;
            indircvsd->hdr.length = sizeof (struct indircvsdpkt) + n;
            strcpy (buf + sizeof (struct indircvsdpkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indircvsd;
            put_pkt (&args);
            continue;
        }
        else if (strcmp (type, "doc") == 0)
        {
            docp = (struct docpkt *)buf;
            docp->hdr.type = INFOPKT_DOCUMENT_PARAMETERS;
            docp->hdr.length = sizeof (struct docpkt);
            docp->resolution = RES_200H_100V;
            docp->hor_width = WIDTH_A4;
            docp->vert_length = 0;
            if (!strcmp(name,"@"))
            {
                unsigned short tmp;
                
                printf("Please enter Document Parameters\n");
                tmp = docp->resolution;
                printf("\tResolution (0=200Hx100V(normal),1=200Hx200V(fine),2=200Hx400V,\n");
                printf("\t  3=300Hx300V,4=400Hx400V,5=600Hx600V,6=1200Hx1200V,7=300Hx600V,\n");
                prompt_param("  8=400Hx800V,9=600Hx1200V,10=100Hx100V)",
                             &tmp,"%u");
                docp->resolution = (unsigned char)tmp;
                tmp = docp->hor_width;
                prompt_param("Width (0=A4,1=B4,2=A3)",&tmp,"%u");
                docp->hor_width = (unsigned char)tmp;
                tmp = docp->vert_length;
                prompt_param("Vertical length (unused)",&tmp,"%u");
                docp->vert_length = (unsigned char)tmp;
            }
            args.ips = ips;
            args.ip = (struct infopkt *)docp;
            put_pkt (&args);
            continue;
        }
        else if (strcmp (type, "bop") == 0)
        {
            bopp = (struct boppkt *)buf;
            bopp->hdr.type = INFOPKT_BEGINNING_OF_PAGE;
            bopp->hdr.length = sizeof (struct boppkt);
            /* Parameter not used for bop */
            args.ips = ips;
            args.ip = (struct infopkt *)bopp;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "g3_strip"))
        {
            g3s = (struct g3strippkt *)buf;
            g3s->hdr.type = INFOPKT_G3_STRIP_PARAMETERS;
            g3s->hdr.length = sizeof (struct g3strippkt); 
            g3s->resolution = RES_200H_100V;
            g3s->width = WIDTH_A4;
            g3s->data_fmt = FMT_MH_UNALIGN_MSB;
            if (!strcmp(name,"@"))
            {
                printf("Please enter G3 Strip Parameters\n");
                printf("\tResolution (0=200Hx100V(normal),1=200Hx200V(fine),2=200Hx400V,\n");
                printf("\t  3=300Hx300V,4=400Hx400V,5=600Hx600V,6=1200Hx1200V,7=300Hx600V,\n");
                prompt_param("  8=400Hx800V,9=600Hx1200V,10=100Hx100V)",
                             &g3s->resolution,"%u");
                prompt_param("Width (0=A4,1=B4,2=A3)",&g3s->width,"%u");
                prompt_param("Data Format (0=MH_UNAL_MSB,1=MH_UNAL_LSB,2=MH_AL_MSB,3=MH_AL_LSB,\n\t  4=MR_UNAL_MSB,5=MR_UNAL_LSB,6=MR_AL_MSB,7=MR_AL_LSB,\n\t  16=MMR_UNAL_MSB,17=MMR_UNAL_LSB,18=MMR_AL_MSB,\n\t  19=MMR_AL_LSB,9=PCX"
                             ,&g3s->data_fmt,"%u");
            }
            args.ips = ips;
            args.ip = (struct infopkt *)g3s;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "ascii_strip"))
        {
            asciis = (struct asciistrippkt *)buf;
            asciis->hdr.type = INFOPKT_ASCII_STRIP_PARAMETERS;
            asciis->hdr.length = sizeof (struct asciistrippkt);
            asciis->resolution = RES_200H_100V;
            asciis->width = WIDTH_A4;
            asciis->eof_char = 0x1a;
            asciis->font_no = 0;
            asciis->left_margin = 5;
            asciis->right_margin = 0;
            asciis->line_spacing = 2;
            if (!strcmp(name,"@"))
            {
                printf("Please enter Ascii Strip Parameters\n");
                printf("\tResolution (0=200Hx100V(normal),1=200Hx200V(fine),2=200Hx400V,\n");
                printf("\t  3=300Hx300V,4=400Hx400V,5=600Hx600V,6=1200Hx1200V,7=300Hx600V,\n");
                prompt_param("  8=400Hx800V,9=600Hx1200V,10=100Hx100V)",
                             &asciis->resolution,"%u");
                prompt_param("Width (0=A4,1=B4,2=A3)",&asciis->width,"%u");
                prompt_param("EOF character (hex)",&asciis->eof_char,"%X");
                prompt_param("Font number",&asciis->font_no,"%u");
                prompt_param("Left margin (0.1\" units)",
                             &asciis->left_margin,"%u");
                prompt_param("Right margin (0.1\" units)",
                             &asciis->right_margin,"%u");
                prompt_param("Line spacing (0.1\" units)",
                             &asciis->line_spacing,"%u");
            }
            args.ips = ips;
            args.ip = (struct infopkt *)asciis;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "ascii_page"))
        {
            ascpp_used = 1;
            asciip = (struct asciipagepkt *)buf;
            asciip->hdr.type = INFOPKT_ASCII_PAGE_PARAMETERS;
            asciip->hdr.length = sizeof (struct asciipagepkt);
            asciip->top_margin = 3;
            asciip->bottom_margin = 3;
            asciip->length = 1143;
            asciip->pad_flag = 0;
            if (!strcmp(name,"@"))
            {
                printf("Please enter Ascii Page Parameters\n");
                prompt_param("Top margin (0.1\" units)",
                             &asciip->top_margin,"%u");
                prompt_param("Bottom margin (0.1\" units)",
                             &asciip->bottom_margin,"%u");
                prompt_param("Length (normal res. lines)",
                             &asciip->length,"%u");
                prompt_param("Pad flag (0=pad,1=no pad)",
                             &asciip->pad_flag,"%u");
            }
            args.ips = ips;
            args.ip = (struct infopkt *)asciip;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "page"))
        {
            pparm = (struct pageparampkt *)buf;
            pparm->hdr.type = INFOPKT_PAGE_PARAMETERS;
            pparm->hdr.length = sizeof (struct pageparampkt);
            pparm->top_margin = 3;
            pparm->bottom_margin = 3;
            pparm->length = 1143;
            pparm->ascii_pad = 0;
            pparm->image_pad = 0;
            pparm->image_break = 0;
            pparm->image_margin = 0;
            if (!strcmp(name,"@"))
            {
                printf("Please enter Page Parameters\n");
                prompt_param("Top margin (0.1\" units)",
                             &pparm->top_margin,"%u");
                prompt_param("Bottom margin (0.1\" units)",
                             &pparm->bottom_margin,"%u");
                prompt_param("Length (normal res. lines)",&pparm->length,"%u");
                prompt_param("Ascii pad (0=pad,1=no pad)",
                             &pparm->ascii_pad,"%u");
                prompt_param("Image pad (0=no pad,1=pad)",
                             &pparm->image_pad,"%u");
                prompt_param("Image break (0=no break,1=break)",
                             &pparm->image_break,"%u");
                prompt_param("Image margin (0=no margins,1=use margins)",
                             &pparm->image_margin,"%u");
            }
            args.ips = ips;
            args.ip = (struct infopkt *)pparm;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "cvsd_param"))
        {
            cvsd_used = 1;
            cvsdp = (struct cvsdparampkt *)buf;
            cvsdp->hdr.type = INFOPKT_CVSD_PARAMETERS;
            cvsdp->hdr.length = sizeof (struct cvsdparampkt);
            cvsdp->flags = 0;
            cvsdp->rate = 20000;
            if (!strcmp(name,"@"))
            {
                printf("Please enter CVSD Parameters\n");
                prompt_param("Rate (bps)",&cvsdp->rate,"%u");
                prompt_param("Flags (unused)",&cvsdp->flags,"%u");
            }
            args.ips = ips;
            args.ip = (struct infopkt *)cvsdp;
            put_pkt (&args);
            continue;
        }
        else if (strcmp (type, "eos") == 0)
        {
            eosp = (struct eospkt *)buf;
            eosp->hdr.type = INFOPKT_END_OF_SPEECH;
            eosp->hdr.length = sizeof (struct eospkt);
            eosp->mode = 0;
            if (!strcmp(name,"@"))
            {
                printf("Please enter EOS parameters\n");
                prompt_param("Mode (0=terminate,1=do not terminate)",
                             &eosp->mode,"%u");
            }
            args.ips = ips;
            args.ip = (struct infopkt *)eosp;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "t30"))
        {
            t30p = (struct t30parampkt *)buf;
            t30p->hdr.type = INFOPKT_T30_PARAMETERS;
            t30p->hdr.length = sizeof (struct t30parampkt);
            t30p->bit_rate = BITRATE_GENERIC;
            t30p->scan_time = SCANTIME_0;
            t30p->modulation_type = MODULATION_GENERIC;
            t30p->line_compression = LINECOMPR_ANY;
            t30p->coding_type = 0;
            if (!strcmp(name,"@"))
            {
                unsigned short tmp;
                
                printf("Please enter T30 Parameters\n");
                tmp = t30p->bit_rate;
                prompt_param("Bit rate (0=2400,2=4800,1=7200,3=9600,4=12000,5=14400,\n\t  6=16800,7=19200,8=21600,9=24000,10=26400,11=28800,12=31200,\n\t  13=33600,255=Generic)",
                             &tmp,"%u");
                t30p->bit_rate = (unsigned char)tmp;
                tmp = t30p->scan_time;
                prompt_param("Scan time (7=0ms,1=5ms,2=10ms,0=20ms,4=40ms)",
                             &tmp,"%u");
                t30p->scan_time = (unsigned char)tmp;
                tmp = t30p->modulation_type;
                prompt_param("Modulation type (0=Generic,1=V.27,2=V.29,3=V.33,4=V.17,5=V.34)",
                             &tmp,"%u");
                t30p->modulation_type = (unsigned char)tmp;
                tmp = t30p->line_compression;
                prompt_param("Line compression (0=Any,1=MH only,2=MR/MH,3=MMR/MR/MH)",
                             &tmp,"%u");
                t30p->line_compression = (unsigned char)tmp;
                tmp = t30p->coding_type;
                prompt_param("Coding type (unused)",&tmp,"%u");
                t30p->coding_type = (unsigned char)tmp;
            }
            args.ips = ips;
            args.ip = (struct infopkt *)t30p;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indirspeech") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirspeechpkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirspeech = (struct indirspeechpkt *)buf;
            indirspeech->hdr.type = INFOPKT_INDIR_SPEECH;
            indirspeech->hdr.length = sizeof (struct indirspeechpkt) + n;
            strcpy (buf + sizeof (struct indirspeechpkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirspeech;
            put_pkt (&args);
            continue;
        }
        if (strcmp (type, "indirwave") == 0)
        {
            n = strlen (name) + 1;
            if (sizeof (struct indirwavepkt) + n + 1 > sizeof(buf))
            {
                fprintf (stderr, "indir name too long: %s\n",
                         name);
                continue;
            }
            indirwave = (struct indirwavepkt *)buf;
            indirwave->hdr.type = INFOPKT_INDIR_WAVE;
            indirwave->hdr.length = sizeof (struct indirspeechpkt) + n;
            strcpy (buf + sizeof (struct indirwavepkt), name);
            args.ips = ips;
            args.ip = (struct infopkt *)indirwave;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "spi"))
        {
            spip = (struct SPI *)buf;
            spip->hdr.type = INFOPKT_SPEECH_PARAMETERS;
            spip->hdr.length = sizeof (struct SPI);
            spip->coding_format = CODE_ADPCM;
            spip->sample_rate = RATE_8000;
            spip->bits_per_sample = BITS_4;
            spip->afe_rate = AFE_8000;
            spip->data_fmt = SPCH_MSB;
            spip->agc = 0xff;
            if (!strcmp(name,"@"))
            {
                unsigned short tmp;
                
                printf("Please enter SPEECH Parameters\n");
                tmp = spip->coding_format;
                prompt_param("Coding format (0=CVSD,1=ADPCM,2=u-law PCM,3=A-law PCM,\n\t  7=G723-1,8=G729-A,9=SX7300,10=SX9600,17=G726,\n\t  14=GSM 6.10,15=GSM 6.60,16=Raw data,4=Linear)",
                             &tmp,"%u");
                spip->coding_format = (unsigned char)tmp;
                prompt_param("Compressed sample rate (9=5300,0=6000,10=6300,1=8000,7=9600,8=11000,\n\t  12=12300,11=13000,6=16000,2=20000,3=24000,4=28000,\n\t  5=32000)",
                             &spip->sample_rate,"%u");
                tmp = spip->bits_per_sample;
                prompt_param("Bits per sample (0=1,7=2,1=3,2=4,8=5,3=8,4=12,5=16,6=24)",
                             &tmp,"%u");
                spip->bits_per_sample = (unsigned char)tmp;
                tmp = spip->afe_rate;
                prompt_param("AFE sample rate (0=8000)",&tmp,"%u");
                spip->afe_rate = (unsigned char)tmp;
                tmp = spip->data_fmt;
                prompt_param("Data format (hex)",&tmp,"%X");
                spip->data_fmt = (unsigned char)tmp;
                tmp = spip->agc;
                prompt_param("AGC value (hex, 0xff = none)",&tmp,"%X");
                spip->agc = (unsigned char)tmp;
            }
            args.ips = ips;
            args.ip = (struct infopkt *)spip;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "annot"))
        {
            if (!strcmp(name,"@"))
            {
                char linebuf[512];
                
                anp = (struct annotpkt *)buf;
                anp->hdr.type = INFOPKT_ANNOTATION;
                
                printf("Please enter ASCII Annotation, '.' alone to end\n");
                
                n = sizeof(struct annotpkt);
                while (fgets(linebuf,sizeof(linebuf),stdin) &&
                       strcmp(linebuf,".") && strcmp(linebuf,".\n"))
                {
                    if (n + strlen(linebuf) >= sizeof(buf))
                    {
                        printf("Annotation too large.\n");
                        break;
                    }
                    strcpy(&buf[n],linebuf);
                    n += strlen(linebuf);
                }
                
                anp->hdr.length = (unsigned short)n;
                args.ips = ips;
                args.ip = (struct infopkt *)anp;
                put_pkt (&args);
                
                continue;
            }
        }
        else if (!strcmp(type, "fax_hdr"))
        {
            char linebuf[512];
            
            hdrpkt = (struct faxhdrpkt *)buf;
            hdrpkt->hdr.type = INFOPKT_FAX_HDR;
            hdrpkt->placement = HDR_HEADER;
            hdrpkt->insert_mode = HDR_MODE_INSERT;
            if (!strcmp(name,"@"))
            {
                unsigned short tmp;
                
                printf("Please enter Fax Header Parameters\n");
                tmp = hdrpkt->placement;
                prompt_param("Header Placement (0=Header,1=Footer)",&tmp,"%u");
                hdrpkt->placement = (unsigned char)tmp;
                tmp = hdrpkt->insert_mode;
                prompt_param("Insertion Mode (1=Overlay,2=Replace,3=Insert,0=disable)",
                             &tmp,"%u");
                hdrpkt->insert_mode = (unsigned char)tmp;
            }
            
            printf("Please enter ASCII header, hit return when completed\n");
            n = sizeof(struct faxhdrpkt);
            if (fgets(linebuf,sizeof(linebuf),stdin))
            {
                if (linebuf[strlen(linebuf) - 1] == '\n')
                    linebuf[strlen(linebuf) - 1] = '\0';
                strcpy(&buf[n],linebuf);
                n += strlen(linebuf) + 1;
            }
            
            hdrpkt->hdr.length = (unsigned short)n;
            args.ips = ips;
            args.ip = (struct infopkt *)hdrpkt;
            put_pkt (&args);
            
            continue;
        }
        else if (!strcmp(type, "bft"))
        {
            bftpkt = (struct bftpkt *)buf;
            bftpkt->hdr.type = INFOPKT_BFT;
            bftpkt->hdr.length = sizeof (struct bftpkt);
            bftpkt->bft_type = BFT_T434;
            if (!strcmp(name,"@"))
            {
                unsigned short tmp;
                
                printf("Please enter BFT Parameters\n");
                tmp = bftpkt->bft_type;
                prompt_param("Bft type (1=T.434,2=BTM,4=DTM)",&tmp,"%u");
                bftpkt->bft_type = (unsigned char)tmp;
            }
            args.ips = ips;
            args.ip = (struct infopkt *)bftpkt;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "overlay"))
        {
            overlaypkt = (struct overlaypkt *)buf;
            overlaypkt->hdr.type = INFOPKT_OVERLAY;
            overlaypkt->hdr.length = sizeof (struct overlaypkt);
            overlaypkt->overlay_number = 0;
            if (!strcmp(name,"@"))
            {
                unsigned short tmp;
                
                printf("Please enter Overlay Parameters\n");
                tmp = overlaypkt->overlay_number;
                prompt_param("Overlay number",&tmp,"%d");
                overlaypkt->overlay_number = (char)tmp;
            }
            args.ips = ips;
            args.ip = (struct infopkt *)overlaypkt;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "space"))
        {
            spacepkt = (struct spacepkt *)buf;
            spacepkt->hdr.type = INFOPKT_SPACE;
            spacepkt->hdr.length = sizeof (struct spacepkt);
            spacepkt->spacing = 0;
            spacepkt->units = 0;
            if (!strcmp(name,"@"))
            {
                unsigned short tmp;
                
                printf("Please enter Vertical Space Parameters\n");
                tmp = spacepkt->spacing;
                prompt_param("Amount of space (negative for backwards)",
                             &tmp,"%d");
                spacepkt->spacing = tmp;
                tmp = spacepkt->units;
                prompt_param("Units (0=1/10 inch,1=scan lines)",&tmp,"%u");
                spacepkt->units = (unsigned char)tmp;
            }
            args.ips = ips;
            args.ip = (struct infopkt *)spacepkt;
            put_pkt (&args);
            continue;
        }
        else if (!strcmp(type, "eff"))
        {
            effpage = (struct effpagepkt *)buf;
            effpage->hdr.type = INFOPKT_EFF_PAGE_PARAMETERS;
            effpage->hdr.length = sizeof (struct effpagepkt);
            effpage->eff_page_type = 0;
            if (!strcmp(name,"@"))
            {
                printf("Please enter EFF page parameters\n");
                prompt_param_int("EFF page type (hex, OR together, 1=JPEG Enable,2=Full color,\n\t  4=Dflt Huffman,8=12 bit,10=No subsamp,20=Cust Illum,40=Cust Gamut,\n\t  100=JBIG,200=L0,400=Lossless,800=Plane Intrlv,\n\t  1000-4000=MRC,8000=Page Len",
                                 &effpage->eff_page_type,"%X");
            }
            args.ips = ips;
            args.ip = (struct infopkt *)effpage;
            put_pkt (&args);
            continue;
        }
        
        if (strcmp(type,"g3") && strcmp(type,"btg3") &&
            strcmp(type,"ascii") && strcmp(type,"btcvsd") && 
            strcmp(type,"cvsd") && strcmp(type,"speech") &&
            strcmp(type,"annotation"))
        {
            fprintf (stderr, "unknown type: %s\n", type);
            break;
        }
        
        if (strcmp (name, "-") == 0)
            ret = process (ips, type, stdin);
        else
        {
#ifndef USES_FAT_FILESYSTEM
            fp = fopen (name, "r");
#else
            if (!strcmp (type, "ascii"))
                fp = fopen (name, "r");
            else
                fp = fopen (name, "rb");
#endif
            if (fp == NULL)
                fprintf (stderr, "can't open %s\n", name);
            else
            {
                ret = process (ips, type, fp);
                fclose (fp);
            }
        }
        if (ret)
            break;
    }
    
    if (optind != argc)
        fprintf (stderr, "warning: no file for final type: %s\n",
                 argv[optind]);
    args.ips = ips;
    BfvInfopktClose(&args);
    
    if (cvsd_used)
        printf ("Note: SPEECH and SPI infopkts are preferred to CVSD.\n");
    if (ascpp_used)
        printf ("Note: PAGE infopkts are preferred to ASCII_PAGE.\n");

    return(0);
}

int
process (struct infopkt_stream *ips, char *type, FILE *f)
{
    char buf[1024];
    int n, avail;
    struct infopkt *ip;
    char *data;
    int header_length;
    struct args_infopkt args;
    
    memset(buf,0,sizeof(buf));
    ip = (struct infopkt *)buf;
    
    if (strcmp (type, "g3") == 0)
    {
        header_length = sizeof (struct g3pkt);
        ip->type = INFOPKT_G3;
    }
    else if (strcmp (type, "btg3") == 0)
    {
        header_length = sizeof (struct btg3pkt);
        ip->type = INFOPKT_BTG3;
    }
    else if (strcmp (type, "ascii") == 0)
    {
        header_length = sizeof (struct asciipkt);
        ip->type = INFOPKT_ASCII;
    }
    else if (strcmp (type, "btcvsd") == 0)
    {
        int a, b;
        cvsd_used = 1;
        header_length = sizeof (struct cvsdpkt);
        ip->type = INFOPKT_CVSD;
        a = getc (f);
        b = getc (f);
        ((struct cvsdpkt *)ip)->rate = (b << 8) + a;
    }
    else if (strcmp (type, "cvsd") == 0)
    {
        cvsd_used = 1;
        header_length = sizeof (struct cvsdpkt);
        ip->type = INFOPKT_CVSD;
        /* this will go away, the rate is in a cvsdparampkt */
        ((struct cvsdpkt *)ip)->rate = 20000;
    }
    else if (strcmp (type, "speech") == 0)
    {
        header_length = sizeof (struct speechpkt);
        ip->type = INFOPKT_SPEECH;
    }
    else if (strcmp (type, "annotation") == 0)
    {
        header_length = sizeof (struct annotpkt);
        ip->type = INFOPKT_ANNOTATION;
    }
    
    data = buf + header_length;
    avail = sizeof(buf) - header_length;
    while ((n = fread (data, 1, avail, f)) > 0)
    {
        ip->length = header_length + n;
        BT_ZERO(args);
        args.ips = ips;
        args.ip = ip;
        put_pkt (&args);
        /* Only the first infopkt is btg3; the rest are normal g3 */
        if (ip->type == INFOPKT_BTG3)
            ip->type = INFOPKT_G3;
    }
    return(0);
}

void 
prompt_param(char *vname, unsigned short *pval, char *fmt)
{
    char temp[300];
    int tmp_val;
    
    printf("\t%s [",vname);
    printf(fmt,*pval);
    printf("]: ");
    fflush(stdout);
    fgets(temp,sizeof(temp),stdin);
    if (sscanf(temp,fmt,&tmp_val) > 0)
        *pval = (unsigned short)tmp_val;
}

void 
prompt_param_int(char *vname, unsigned *pval, char *fmt)
{
    char temp[300];
    
    printf("\t%s [",vname);
    printf(fmt,*pval);
    printf("]: ");
    fflush(stdout);
    fgets(temp,sizeof(temp),stdin);
    sscanf(temp,fmt,pval);
}

void 
put_pkt(struct args_infopkt *args)
{
    BTERR bterr;
    
    if (BfvInfopktPut (args) < 0)
    {
        BfvErrorMessage(NULL,&args->res,&bterr);
        fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
        exit(1);
    }
}
