/*
 * mkprompt.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The mkprompt program creates or updates a prompt file from infopkt
files that contain individual phrases. When updating, you can specify
an existing phrase number or a new phrase number.
***************************************************************************/

#include <string.h>
#include <memory.h>
#if defined(unix) || defined(M_XENIX) || defined(_AIX)
#include <unistd.h>
#endif
#include "btlib.h"

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

#define TMP_FILE "prompt.tmp"

#if defined(__OS2__) && !defined(__TURBOC__)
#define unlink _unlink
#endif

void
usage ()
{
    printf("usage: mkprompt prompt_file [phrase_ipkt] ...\n");
    printf("   or  mkprompt -u phrase_num prompt_file phrase_ipkt\n");
    exit (1);
}

int
main (argc, argv)
int argc;
char **argv;
{
    int c;
    extern char *optarg;
    extern int optind;
    struct infopkt_stream *ips,*out_ips;
    int i;
    int u_flag = 0;
    int update_phrase;
    char *prompt_file;
    char *phrase_name;
    struct infopkt_stream *phrase_ips;
    struct infopkt *ip;
    struct promptpkt *pr;
    struct prompt_phrase *phrases;
    long map_offset = 0;
    long off_1,off_2;
    int phrase_num = 0;
    char *output;
    long offset_diff;
    static struct eospkt eosp =
        {{INFOPKT_END_OF_SPEECH,sizeof(struct eospkt)},1};
    struct args_infopkt args;
    struct args_debug_mode args_debug;
    BTERR bterr;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "u:")) != EOF)
    {
        switch (c)
        {
          case 'u':
            u_flag = 1;
            update_phrase = atol(optarg);
            if (update_phrase >= MAX_PHRASES_PER_MAP)
            {
                fprintf(stderr,
                        "mkprompt: Cannot handle phrase numbers larger than %d.\n",
                        MAX_PHRASES_PER_MAP - 1);
                exit(1);
            }
            break;
          default:
            usage ();
        }
    }
    
    if (optind >= argc)
        usage ();

    prompt_file = argv[optind++];
    
    if (optind >= argc)
        usage ();

    if (u_flag)
    {
        if (argc - optind != 1)
            usage ();
        output = TMP_FILE;
    }
    else
        output = prompt_file;

    BT_ZERO(args_debug);
    args_debug.mode = DEBUG_ALL;
    args_debug.options = API_DBG_MODE_SET;
    BfvDebugModeSetAdv(&args_debug);
    
    BT_ZERO(args);
    args.fname = output;
    args.fmode = "w+";
    if ((out_ips = BfvInfopktOpen (&args)) == NULL)
    {
        BfvErrorMessage(NULL,&args.res,&bterr);
        fprintf (stderr, "BfvInfopktOpen %s: %s\n",output,bterr.long_msg);
        exit (1);
    }
    
    if (u_flag)
    {
        /* If we are updating, read existing map */
        
        args.fname = prompt_file;
        args.fmode = "r+";
        if ((ips = BfvInfopktOpen (&args)) == NULL)
        {
            BfvErrorMessage(NULL,&args.res,&bterr);
            fprintf (stderr, "BfvInfopktOpen %s: %s\n",
                     prompt_file,bterr.long_msg);
            exit (1);
        }
        
        /* Look for prompt map, copying beginning of file */
        for (;;)
        {
            args.ips = ips;
            args.i_mode = INDIR_MODE_RETURN;
            if ((ip = BfvInfopktGet(&args)) == NULL)
            {
                BfvErrorMessage(NULL,&args.res,&bterr);
                fprintf (stderr, "BfvInfopktGet: %s\n",bterr.long_msg);
                exit(1);
            }
            
            if (ip->type == INFOPKT_PROMPT_MAP)
                break;

            if (ip->type < (unsigned short)INFOPKT_USER0 ||
                ip->type > (unsigned short)INFOPKT_USER9)
            {
                fprintf(stderr,"Prompt file error\n");
                exit(1);
            }
            
            args.ips = out_ips;
            args.ip = ip;
            if (BfvInfopktPut(&args) < 0)
            {
                BfvErrorMessage(NULL,&args.res,&bterr);
                fprintf (stderr,"BfvInfopktPut: %s\n",bterr.long_msg);
                exit(1);
            }
        }
        
        /* We found a map */
        args.ips = ips;
        map_offset = BfvInfopktFtell(&args) - ip->length;
        c = ((struct promptpkt *)ip)->num_phrases;
        if (update_phrase >= c)
            c = update_phrase + 1;
        pr = (struct promptpkt *)
            malloc(sizeof(struct promptpkt) +
                   c * sizeof(struct prompt_phrase));
        if (pr == NULL)
        {
            fprintf(stderr,"Malloc error\n");
            exit(1);
        }
        phrases = (struct prompt_phrase *)&pr[1];
        memcpy((char *)pr,(char *)ip,
               sizeof(struct promptpkt) + c * sizeof(struct prompt_phrase));
        pr->hdr.length =
            sizeof(struct promptpkt) + c * sizeof(struct prompt_phrase);
        offset_diff = (c - pr->num_phrases) * sizeof(struct prompt_phrase);
        if (offset_diff > 0)
            memset((char *)&phrases[pr->num_phrases],0,(unsigned)offset_diff);
        /* Write out a temporary updated one */
        args.ips = out_ips;
        args.ip = (struct infopkt *)pr;
        if (BfvInfopktPut(&args) < 0)
        {
            BfvErrorMessage(NULL,&args.res,&bterr);
            fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
            exit(1);
        }
        
        /* Copy file up to the update phrase, or the end if it was a new
           prompt */
        for (;;)
        {
            if (phrases[update_phrase].offset != 0)
            {
                args.ips = ips;
                if (BfvInfopktFtell(&args) >=
                    (long)phrases[update_phrase].offset)
                    break;
            }
            args.ips = ips;
            args.i_mode = INDIR_MODE_RETURN;
            if ((ip = BfvInfopktGet(&args)) == NULL)
            {
                if (args.res.status != BT_STATUS_OK)
                {
                    BfvErrorMessage(NULL,&args.res,&bterr);
                    fprintf (stderr, "BfvInfopktGet: %s\n",bterr.long_msg);
                    exit(1);
                }
                break;
            }
            
            args.ips = out_ips;
            args.ip = ip;
            if (BfvInfopktPut(&args) < 0)
            {
                BfvErrorMessage(NULL,&args.res,&bterr);
                fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
                exit(1);
            }
        }
        
        /* Copy the rest of the file after the update phrase */
        if (phrases[update_phrase].offset)
        {
            args.ips = ips;
            args.offset = phrases[update_phrase].offset +
                phrases[update_phrase].phrase_size;
            args.origin = SEEK_SET;
            BfvInfopktFseek(&args);
            for (;;)
            {
                args.ips = ips;
                args.i_mode = INDIR_MODE_RETURN;
                if ((ip = BfvInfopktGet(&args)) == NULL)
                {
                    if (args.res.status != BT_STATUS_OK)
                    {
                        BfvErrorMessage(NULL,&args.res,&bterr);
                        fprintf (stderr, "BfvInfopktGet: %s\n",bterr.long_msg);
                        exit(1);
                    }
                    break;
                }
                args.ips = out_ips;
                args.ip = ip;
                if (BfvInfopktPut(&args) < 0)
                {
                    BfvErrorMessage(NULL,&args.res,&bterr);
                    fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
                    exit(1);
                }
            }
        }
        
        args.ips = ips;
        BfvInfopktClose(&args);
        
        /* Update offsets for other phrases */
        for (i = 0; i < (int)pr->num_phrases; i++)
        {
            if (i == update_phrase || phrases[i].offset == 0)
                continue;
            phrases[i].offset += (unsigned)offset_diff;
            if (phrases[i].offset > phrases[update_phrase].offset)
                phrases[i].offset -=
                    (unsigned)phrases[update_phrase].phrase_size;
        }
        pr->num_phrases = c;
        phrase_num = update_phrase;
    }
    else
    {
        /* If we are creating, write temporary new map */
        c = argc - optind;
        if (c > MAX_PHRASES_PER_MAP)
        {
            fprintf(stderr,
                    "mkprompt: Cannot handle more than %d phrases.\n",
                    MAX_PHRASES_PER_MAP);
            exit(1);
        }
        
        pr = (struct promptpkt *)
            malloc(sizeof(struct promptpkt) +
                   c * sizeof(struct prompt_phrase));
        if (pr == NULL)
        {
            fprintf(stderr,"Malloc error\n");
            exit(1);
        }
        pr->hdr.type = INFOPKT_PROMPT_MAP;
        pr->num_phrases = c;
        pr->hdr.length = sizeof(struct promptpkt) +
            pr->num_phrases * sizeof(struct prompt_phrase);
        
        phrases = (struct prompt_phrase *)&pr[1];
        memset((char *)phrases,0,
               (unsigned)(pr->num_phrases * sizeof(struct prompt_phrase)));
        
        args.ips = out_ips;
        args.ip = (struct infopkt *)pr;
        if (BfvInfopktPut(&args) < 0)
        {
            BfvErrorMessage(NULL,&args.res,&bterr);
            fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
            exit(1);
        }
    }
    
    while (optind < argc)
    {
        phrase_name = argv[optind++];
        args.ips = out_ips;
        off_1 = BfvInfopktFtell(&args);
        args.fname = phrase_name;
        args.fmode = "r";
        if ((phrase_ips = BfvInfopktOpen (&args)) == NULL)
        {
            BfvErrorMessage(NULL,&args.res,&bterr);
            fprintf (stderr, "BfvInfopktOpen %s: %s\n",
                     phrase_name,bterr.long_msg);
            exit (1);
        }
        for (;;)
        {
            args.ips = phrase_ips;
            args.i_mode = INDIR_MODE_RETURN;
            if ((ip = BfvInfopktGet(&args)) == NULL)
            {
                if (args.res.status != BT_STATUS_OK)
                {
                    BfvErrorMessage(NULL,&args.res,&bterr);
                    fprintf (stderr, "BfvInfopktGet: %s\n",bterr.long_msg);
                    exit(1);
                }
                break;
            }
            args.ips = out_ips;
            args.ip = ip;
            if (BfvInfopktPut(&args) < 0)
            {
                BfvErrorMessage(NULL,&args.res,&bterr);
                fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
                exit(1);
            }
        }
        
        args.ips = phrase_ips;
        BfvInfopktClose(&args);
        /* Add in an EOS */
        args.ips = out_ips;
        args.ip = (struct infopkt *)&eosp;
        if (BfvInfopktPut(&args) < 0)
        {
            BfvErrorMessage(NULL,&args.res,&bterr);
            fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
            exit(1);
        }
        
        args.ips = out_ips;
        off_2 = BfvInfopktFtell(&args);
        phrases[phrase_num].offset = (unsigned)off_1;
        phrases[phrase_num].phrase_size = (unsigned)(off_2 - off_1);
        phrase_num++;
    }
    
    /* Update the map infopkt at the beginning */
    args.ips = out_ips;
    args.offset = map_offset;
    args.origin = SEEK_SET;
    args.ip = (struct infopkt *)pr;
    if (BfvInfopktFseek(&args))
    {
        fprintf(stderr,"File write error\n");
        exit(1);
    }
    if (BfvInfopktPut(&args) < 0)
    {
        BfvErrorMessage(NULL,&args.res,&bterr);
        fprintf (stderr, "BfvInfopktPut: %s\n",bterr.long_msg);
        exit(1);
    }
    
    args.ips = out_ips;
    BfvInfopktClose(&args);
    
    if (u_flag)
    {
        unlink(prompt_file);
#if defined(unix) || defined(M_XENIX)
        if (link(TMP_FILE,prompt_file) < 0 || unlink(TMP_FILE) < 0)
#else
        if (rename(TMP_FILE,prompt_file))
#endif
        {
            fprintf(stderr,"Couldn't rename %s to %s\n",TMP_FILE,prompt_file);
            exit(1);
        }
    }
    
    printf("Prompt file '%s' created\n",prompt_file);
    
    return(0);
}
