/*
 * shoparam.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The shoparam program displays the contents of the line structure
and the parameter values set in a user configuration file and the
read-only parameters taken from the country configuration file.
***************************************************************************/

/* For Windows only:
   If an application uses a DLL version of the Bfv API library, under some
   conditions the application and the Bfv API library may link to different
   versions or instances of the compiler-supplied C runtime (CRT) libraries.
   This creates incompatibilities between file pointers or allocated memory
   among the application and the Bfv API library.

   Under these conditions, the application must use the Bfv API supplied _dll_
   versions of the file I/O and memory allocation functions if the pointer or
   memory is passed to/from the application and the library.

   The Bfv API DLL libraries links with MSVCRT.DLL. When using the Visual
   Studio mechanism to compile for debugging, applications may be linked with
   MSVCRTD.DLL. When using Visual Studio.NET, applications link with
   MSVCR71.DLL. When using the DLLS version of the Bfv API library, it also
   links with MVCRT.DLL, but it is a different instance of the CRT library.
*/

#include <malloc.h>
#include <string.h>
#include "btlib.h"

void
usage ()
{
    fprintf (stderr, "usage: shoparam <user config file>\n");
    exit (1);
}

int main(int argc, char **argv)
{
    BTLINE *lp;
    int dowork();
    struct args_line_admin args;
    BTERR bterr;
    
    BT_API_SET_VER();
    
    if (argc != 2)
        usage();
    
    BT_ZERO(args);
    args.unit = -1;
    if ((lp = BfvLineAttach (&args)) == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        printf ("BfvLineAttach: %s\n",bterr.long_msg);
        exit(1);
    }
    
    dowork(lp,argv);
    
    BfvLineDetach (lp, &args);
    
    return(0);
}

dowork(BTLINE *lp, char **argv)
{
    struct args_line_admin args_admin;
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = argv[1];
#ifdef BROOKTROUT_MILLENNIAL
    args_admin.skip_teleph = 1;
#endif
    if (BfvLineReset (lp, &args_admin) < 0)
    {
        fprintf (stderr, "Reset failed, status %lX.\n",
                 args_admin.reset_status);
        return(0);
    }
    
    if (args_admin.reset_status & RST_USR_CONFIG_UNREADABLE)
        printf("Warning: User configuration file unreadable.\n");
    if (args_admin.reset_status & RST_USR_CONFIG_BADFORMAT)
        printf("Warning: User configuration file has bad format.\n");
    if (args_admin.reset_status & RST_CPARM_UNREADABLE)
        printf("Warning: BT_CPARM file unreadable.\n");
    if (args_admin.reset_status & RST_COUNTRY_NOTFOUND)
        printf("Warning: Country specified in the user config file not found.\n");

#ifdef MILL_DLL
    BfvLineDumpStructure (lp, _dll_stdout());
#else
    BfvLineDumpStructure (lp, stdout);
#endif
    
    return(0);
}
