/*
 * summation.c
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The voice program uses the speech-infopkt-file recording and playing
routines to record and play speech. Recording begins when you start
speaking. Recording continues for a maximum of ten seconds or the
time specified in the -n option.
***************************************************************************/

#include <string.h>
#include <ctype.h>
#include "btlib.h"

int rcrd_func BT_ARGS((BTLINE *lp,char *arg));
void process_aoc BT_ARGS((BTLINE *lp));
void process_call_info BT_ARGS((CALL_RES *call_res));

void
usage ()
{
    fprintf (stderr, "usage: summation [options] infopktfile\n");
    fprintf (stderr, "or     summation -g unitnum\n");
    fprintf (stderr, "  -u unitnum  recording unit number\n");
    fprintf (stderr, "  -s unitnum  summation unit number\n");
    fprintf (stderr, "  -n secs     specify recording time\n");
    fprintf (stderr, "  -f          specify record coding format (dflt:pcm_ulaw)\n");
    fprintf (stderr, "use the number of the format, or one of:\n");
    fprintf (stderr, "   adpcm[32,24],pcm_ulaw[64,48,88],pcm_alaw[64,48,88],\n");
    fprintf (stderr, "   g723-1[-53,-63],g729-a,sx7300,sx9600,gsm_610,rawdata,\n");
    fprintf (stderr, "   g726[-40,-32,-24,-16],linear[128]\n");
    fprintf (stderr, "'#' causes immediate termination of play/record.\n");
    fprintf (stderr, "  -g unitnum  list summation group value for unit number\n");
    fprintf (stderr, "-u and -s must be given, or -g.\n");
    exit (1);
}

int unit = -1;
int sum_unit = -1;
int format_specified = 0;
long the_timeout = 10000L;
int group_flag = 0;

unsigned coding_fmt;
struct coding_info {
    char name[30];
    unsigned fmt;
    unsigned rate;
    unsigned bits_per_samp;
} fmt_list[] = {
    { {"pcm_ulaw"}, CODE_PCM_ULAW, RATE_8000, BITS_8 },
    { {"pcm_ulaw64"}, CODE_PCM_ULAW, RATE_8000, BITS_8 },
    { {"pcm_ulaw48"}, CODE_PCM_ULAW, RATE_6000, BITS_8 },
    { {"pcm_ulaw88"}, CODE_PCM_ULAW, RATE_11000, BITS_8 },
    { {"pcm_alaw"}, CODE_PCM_ALAW, RATE_8000, BITS_8 },
    { {"pcm_alaw64"}, CODE_PCM_ALAW, RATE_8000, BITS_8 },
    { {"pcm_alaw48"}, CODE_PCM_ALAW, RATE_6000, BITS_8 },
    { {"pcm_alaw88"}, CODE_PCM_ALAW, RATE_11000, BITS_8 },
    { {"adpcm"}, CODE_ADPCM, RATE_8000, BITS_4 },
    { {"adpcm32"}, CODE_ADPCM, RATE_8000, BITS_4 },
    { {"adpcm24"}, CODE_ADPCM, RATE_6000, BITS_4 },
    { {"g723-1"}, CODE_G723_1, RATE_5300, BITS_4 },
    { {"g723-1-53"}, CODE_G723_1, RATE_5300, BITS_4 },
    { {"g723-1-63"}, CODE_G723_1, RATE_6300, BITS_4 },
    { {"g729-a"}, CODE_G729_A, RATE_8000, BITS_4 },
    { {"sx7300"}, CODE_SX7300, RATE_8000, BITS_4 },
    { {"sx9600"}, CODE_SX9600, RATE_8000, BITS_4 },
    { {"gsm_610"}, CODE_GSM_610, RATE_13000, BITS_8 },
    { {"rawdata"}, CODE_RAWDATA, RATE_8000, BITS_8 },
    { {"g726"}, CODE_G726, RATE_8000, BITS_5 },
    { {"g726-40"}, CODE_G726, RATE_8000, BITS_5 },
    { {"g726-32"}, CODE_G726, RATE_8000, BITS_4 },
    { {"g726-24"}, CODE_G726, RATE_8000, BITS_3 },
    { {"g726-16"}, CODE_G726, RATE_8000, BITS_2 },
    { {"linear"}, CODE_LINEAR, RATE_8000, BITS_16 },
    { {"linear128"}, CODE_LINEAR, RATE_8000, BITS_16 },
    /* Not supported or compatibility only */
    { {"cvsd"}, CODE_CVSD, RATE_20000, BITS_1 },
    { {"subband"}, CODE_SUBBAND, RATE_8000, BITS_3 },
    { {"oki_adpcm"}, CODE_OKI_ADPCM, RATE_8000, BITS_4 },
    { {"rhet_adpcm"}, CODE_RHET_ADPCM, RATE_8000, BITS_4 },
    { {"rhet_adpcm32"}, CODE_RHET_ADPCM, RATE_8000, BITS_4 },
    { {"rhet_adpcm24"}, CODE_RHET_ADPCM, RATE_6000, BITS_4 },
    { {"dvi_adpcm"}, CODE_DVI_ADPCM, RATE_8000, BITS_4 },
    { {"dvi_adpcm32"}, CODE_DVI_ADPCM, RATE_8000, BITS_4 },
    { {"dvi_adpcm24"}, CODE_DVI_ADPCM, RATE_6000, BITS_4 },
    { {"ms_gsm"}, CODE_MS_GSM, RATE_8000, BITS_4 },
    { {"gsm_660"}, CODE_GSM_660, RATE_12300, BITS_8 },
};
#define NUM_FMTS (sizeof(fmt_list) / sizeof(*fmt_list))
/* Extra braces around strings above required to avoid QNX 2 compiler bug */


void dowork BT_ARGS ((BTLINE *lp1, BTLINE *lp2, char *name));
void dowork1 BT_ARGS ((BTLINE *lp1, BTLINE *lp2, char *name));

unsigned
format_string_lookup(char *fmtstr)
{
    unsigned i;
    
    for (i = 0; i < NUM_FMTS; i++)
    {
        if (!strcmp(fmtstr,fmt_list[i].name))
            break;
    }
    if (i >= NUM_FMTS)
    {
        fprintf(stderr,"Unknown format '%s'\n",fmtstr);
        exit(1);
    }
    return(i);
}


int
main (int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp1, *lp2;
    char *name;
    int i;
    unsigned tmp;
    struct args_line_admin args;
    struct args_debug_mode args_debug;
    BTERR bterr;
    
    BT_API_SET_VER();
    BT_ZERO(args_debug);
    args_debug.mode = DEBUG_ALL;
    args_debug.options = API_DBG_MODE_SET;
    BfvDebugModeSetAdv(&args_debug);
    
    while ((c = getopt (argc, argv, "u:s:f:n:g:")) != EOF)
    {
        switch (c)
        {
          case 'u':
            unit = atoi (optarg);
            break;
          case 's':
            sum_unit = atoi (optarg);
            break;
          case 'f':
            if (isdigit(optarg[0]))
            {
                tmp = atoi(optarg);
                for (i = 0; i < NUM_FMTS; i++)
                {
                    if (fmt_list[i].fmt == tmp)
                    {
                        coding_fmt = i;
                        break;
                    }
                }
                if (i >= NUM_FMTS)
                {
                    fprintf(stderr,"Invalid format %d\n",tmp);
                    exit(1);
                }
            }
            else
                coding_fmt = format_string_lookup(optarg);
            format_specified = 1;
            break;
          case 'n':
            the_timeout = 1000L * atoi(optarg);
            break;
          case 'g':
            unit = atoi (optarg);
            group_flag = 1;
            break;
          default:
            usage ();
        }
    }
    
    if (group_flag == 0)
    {
        if (optind >= argc)
            usage ();

        name = argv[optind++];
    }
    
    if (optind != argc)
        usage ();

    if ((unit < 0 || sum_unit < 0) && group_flag == 0)
        usage ();

    if (unit == sum_unit)
    {
        fprintf (stderr,"Channels for -u and -s must be different.\n");
        exit(1);
    }

    BT_ZERO(args);
    args.unit = unit;
    if ((lp1 = BfvLineAttach (&args)) == NULL)
    {
        BfvErrorMessage(NULL,&args.res,&bterr);
        fprintf (stderr,"BfvLineAttach for %u: %s\n",
                 unit,bterr.long_msg);
        exit (1);
    }
    
    if (!group_flag)
    {
        BT_ZERO(args);
        args.unit = sum_unit;
        if ((lp2 = BfvLineAttach (&args)) == NULL)
        {
            BfvErrorMessage(NULL,&args.res,&bterr);
            fprintf (stderr,"BfvLineAttach for %u: %s\n",
                     sum_unit,bterr.long_msg);
            exit (1);
        }
    }

    dowork1 (lp1, lp2, name);
    
    return(0);
}

void
dowork1 (BTLINE *lp1, BTLINE *lp2, char *name)
{
    struct args_line_admin args_admin;
    
    if (!format_specified)
    {
        coding_fmt = format_string_lookup("pcm_ulaw");
    }
    
    dowork (lp1, lp2, name);

    BT_ZERO(args_admin);
    BfvLineDetach (lp1, &args_admin);
    BT_ZERO(args_admin);
    BfvLineDetach (lp2, &args_admin);
}

void
dowork (BTLINE *lp1, BTLINE *lp2, char *name)
{
    struct infopkt_stream *ips;
    BTERR bterr;
    struct args_line_admin args_admin;
    struct args_telephone args_tel;
    struct args_speech args_speech;
    struct args_infopkt args_infopkt;
    struct args_tone args_tone;
    unsigned group1, group2;
    int offh1 = 0, offh2 = 0;
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    if (BfvLineReset (lp1, &args_admin) < 0)
    {
        BfvErrorMessage(lp1,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset for %u: %s: status %lX.\n",
                 LINE_UNIT_NUM(lp1),
                 bterr.long_msg,args_admin.reset_status);
        exit(1);
    }
    
    /* Check for digital configuration error */
    if (args_admin.reset_status & RST_DIG_CONFIG_UNREADABLE)
    {
        fprintf (stderr, "Telephony/Digital configuration file not found.\n");
        exit(1);
    }
    
    if (!LINE_HAS_CAP(lp1,CAP_SPEECH_RECORD))
    {
        fprintf (stderr, "unit %d does not support speech recording\n",
                 LINE_UNIT_NUM(lp1));
        exit(1);
    }
    
    if (!LINE_HAS_CAP(lp1,CAP_SPEECH_SUMMATION))
    {
        fprintf (stderr, "unit %d does not support speech summation\n",
                 LINE_UNIT_NUM(lp1));
        exit(1);
    }
    
    BT_ZERO(args_speech);
    BfvSpeechQuerySummationGroup(lp1, &args_speech);
    if (args_speech.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp1,&args_speech.res,&bterr);
        fprintf (stderr, "BfvSpeechQuerySummationGroup for %u: %s\n",
                 LINE_UNIT_NUM(lp1),
                 bterr.long_msg);
        exit(1);
    }
    group1 = args_speech.summation_group;

    if (group_flag)
    {
        printf("Group for channel %u = %08X\n",
               LINE_UNIT_NUM(lp1),group1);
        exit(0);
    }

    if (group1 == 0)
    {
        fprintf (stderr, "Error - Summation group 0 for channel %u.\n",
                 LINE_UNIT_NUM(lp1));
        exit(1);
    }


    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    if (BfvLineReset (lp2, &args_admin) < 0)
    {
        BfvErrorMessage(lp2,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset for %u: %s: status %lX.\n",
                 LINE_UNIT_NUM(lp2),
                 bterr.long_msg,args_admin.reset_status);
        exit(1);
    }
    
    /* Check for digital configuration error */
    if (args_admin.reset_status & RST_DIG_CONFIG_UNREADABLE)
    {
        fprintf (stderr, "Telephony/Digital configuration file not found.\n");
        exit(1);
    }
    
    if (!LINE_HAS_CAP(lp2,CAP_SPEECH_RECORD))
    {
        fprintf (stderr, "unit %d does not support speech recording\n",
                 LINE_UNIT_NUM(lp2));
        exit(1);
    }
    
    if (!LINE_HAS_CAP(lp2,CAP_SPEECH_SUMMATION))
    {
        fprintf (stderr, "unit %d does not support speech summation\n",
                 LINE_UNIT_NUM(lp2));
        exit(1);
    }
    
    BT_ZERO(args_speech);
    BfvSpeechQuerySummationGroup(lp2, &args_speech);
    if (args_speech.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp2,&args_speech.res,&bterr);
        fprintf (stderr, "BfvSpeechQuerySummationGroup for %u: %s\n",
                 LINE_UNIT_NUM(lp2),
                 bterr.long_msg);
        exit(1);
    }
    group2 = args_speech.summation_group;

    if (group2 == 0)
    {
        fprintf (stderr, "Error - Summation group 0 for channel %u.\n",
                 LINE_UNIT_NUM(lp2));
        exit(1);
    }


    if (group1 != group2)
    {
        fprintf
            (stderr,
             "Error - Summation groups do not match: %08X for %u, %08X for %u.\n",
             group1, LINE_UNIT_NUM(lp1),
             group2, LINE_UNIT_NUM(lp2));
        exit(1);
    }

    BT_ZERO(args_infopkt);
    args_infopkt.fname = name;
    args_infopkt.fmode = "w";
    if ((ips = BfvInfopktOpen (&args_infopkt)) == NULL)
    {
        BfvErrorMessage(NULL,&args_infopkt.res,&bterr);
        printf ("BfvInfopktOpen: %s\n",bterr.long_msg);
        exit(1);
    }
    
    /* Wait for a call on the first channel */
    BT_ZERO(args_tel);
    BfvLineWaitForCall (lp1, &args_tel);
    if (args_tel.res.status != BT_STATUS_OK || 
        args_tel.res.line_status != WAIT_FOR_CALL_OK)
    {
        BfvErrorMessage(lp1,&args_tel.res,&bterr);
        printf ("BfvLineWaitForCall for channel %u: %s\n",
                LINE_UNIT_NUM(lp1),
                bterr.long_msg);
        goto got_err_1;
    }
    process_call_info(&args_tel.call_res);
        
    BT_ZERO(args_tel);
    BfvLineAnswer (lp1, &args_tel);
    if (args_tel.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp1,&args_tel.res,&bterr);
        printf ("BfvLineAnswer for channel %u: %s\n",
                LINE_UNIT_NUM(lp1),
                bterr.long_msg);
        goto got_err_1;
    }
    offh1 = 1;
    
    /* Wait for a call on the second channel */
    BT_ZERO(args_tel);
    BfvLineWaitForCall (lp2, &args_tel);
    if (args_tel.res.status != BT_STATUS_OK || 
        args_tel.res.line_status != WAIT_FOR_CALL_OK)
    {
        BfvErrorMessage(lp2,&args_tel.res,&bterr);
        printf ("BfvLineWaitForCall for channel %u: %s\n",
                LINE_UNIT_NUM(lp2),
                bterr.long_msg);
        goto got_err_1;
    }
    process_call_info(&args_tel.call_res);
        
    BT_ZERO(args_tel);
    BfvLineAnswer (lp2, &args_tel);
    if (args_tel.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp2,&args_tel.res,&bterr);
        printf ("BfvLineAnswer for channel %u: %s\n",
                LINE_UNIT_NUM(lp2),
                bterr.long_msg);
        goto got_err_1;
    }
    offh2 = 1;
    
    BT_ZERO(args_tone);
    args_tone.decode_flag = DTMF_12TONE; 
    BfvToneDetectEnable(lp1, &args_tone);
    
    /* Clear previous CP data */
    BT_ZERO(args_tel);
    while (BfvDataCP(lp1,&args_tel) == 0);
    
    /* Enable call progress, and allow BfvToneGet to check for CP. 
       Can be done only in a restricted mode. */
    BT_ZERO(args_tel);
    args_tel.call_protocol_code = CALL_PROTOCOL_VOICE_NO_RAW;
    args_tel.call_mode = BT_ORIGINATE;
    if (BfvLineCallProgressEnable(lp1,&args_tel) < 0)
    {
        BfvErrorMessage(lp1,&args_tel.res,&bterr);
        printf ("BfvLineCallProgressEnable for channel %u: %s\n",
                LINE_UNIT_NUM(lp1),
                bterr.long_msg);
        goto got_err;
    }
    
    BT_ZERO(args_tone);
    args_tone.decode_flag = DTMF_12TONE; 
    BfvToneDetectEnable(lp2, &args_tone);
    
    /* Clear previous CP data */
    BT_ZERO(args_tel);
    while (BfvDataCP(lp2,&args_tel) == 0);
    
    /* Enable call progress, and allow BfvToneGet to check for CP. 
       Can be done only in a restricted mode. */
    BT_ZERO(args_tel);
    args_tel.call_protocol_code = CALL_PROTOCOL_VOICE_NO_RAW;
    args_tel.call_mode = BT_ORIGINATE;
    if (BfvLineCallProgressEnable(lp2,&args_tel) < 0)
    {
        BfvErrorMessage(lp2,&args_tel.res,&bterr);
        printf ("BfvLineCallProgressEnable for channel %u: %s\n",
                LINE_UNIT_NUM(lp2),
                bterr.long_msg);
        goto got_err;
    }
    
    /* record for 10 seconds. Uses specified format, bps value, bit per
       sample value, with AFE 8000samp/sec, MSB data. Will do silence
       removal, no silence timeout. */
    BT_ZERO(args_speech);
    args_speech.ips = ips;
    args_speech.coding_fmt = fmt_list[coding_fmt].fmt;
    args_speech.rate = fmt_list[coding_fmt].rate;
    args_speech.bits_per_samp = fmt_list[coding_fmt].bits_per_samp;
    args_speech.afe_rate = AFE_8000;
    args_speech.data_fmt = SPCH_MSB;
    args_speech.timeout = the_timeout;
    args_speech.silence_timeout = 0L;
    args_speech.s_compr = 0;
    args_speech.func = rcrd_func;
    args_speech.arg = (char *)lp2;
    args_speech.lp_summed = lp2;
    BfvSpeechRecord (lp1, &args_speech);
    
    if (args_speech.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp1,&args_speech.res,&bterr);
        printf ("BfvSpeechRecord: %s\n",bterr.long_msg);
        goto got_err;
    }
    
    BT_ZERO(args_tel);
    BfvLineCallProgressDisable(lp1,&args_tel);

    BT_ZERO(args_tone);
    BfvToneDetectDisable(lp1,&args_tone);
    
    BT_ZERO(args_tel);
    BfvLineCallProgressDisable(lp2,&args_tel);

    BT_ZERO(args_tone);
    BfvToneDetectDisable(lp2,&args_tone);
    
got_err:
    if (offh1)
    {
        BT_ZERO(args_tel);
        BfvLineTerminateCall (lp1, &args_tel);
    }
    if (offh2)
    {
        BT_ZERO(args_tel);
        BfvLineTerminateCall (lp2, &args_tel);
    }

    process_aoc(lp1);
    process_aoc(lp2);

got_err_1:
    args_infopkt.ips = ips;
    BfvInfopktClose (&args_infopkt);
}

/* # = Abort, 1 = Gain Up, 2 = Gain Down, 3 = Speed Up, 4 = Speed Down. */
int
rcrd_func (BTLINE *lp1, char *arg)
{
    int tmp;
    struct args_tone args_tone;
    struct args_telephone args_tel;
    BTLINE *lp2 = (BTLINE *)arg;
    
    BT_ZERO(args_tone);
    args_tone.tone_cp = 1;
    switch (tmp = BfvToneGet(lp1,&args_tone))
    {
      case '#':
        /* Stop playing/recording */
        return(1);
      case 1:
        /* CP available */
        BT_ZERO(args_tel);
        if (BfvDataCP(lp1,&args_tel))
        {
            printf("CP data reported available, but is not!\n");
            break;
        }
        if (args_tel.type != CP_TYPE_BOARD_DET)
        {
            printf("Unexpected CP type %u found.\n",args_tel.type);
            break;
        }
        switch (args_tel.data)
        {
          case CS_BUSY1:
          case CS_DIALTON:
            /* Stop on busy/reorder or dialtone */
            return(1);
          case CS_SILENCE:
          case CS_CNGDETCT:
            break;
          default:
            printf("Unexpected CP data value %02X found.\n",args_tel.data);
            break;
        }
        break;
    }
    if (args_tone.res.status != BT_STATUS_OK &&
        args_tone.res.status != BT_STATUS_TIMEOUT)
    {
        BTERR bterr;
        
        BfvErrorMessage(lp1,&args_tone.res,&bterr);
        printf ("BfvToneGet for channel %u: %s\n",
                LINE_UNIT_NUM(lp1),
                bterr.long_msg);
        return(1);
    }

    BT_ZERO(args_tone);
    args_tone.tone_cp = 1;
    switch (tmp = BfvToneGet(lp2,&args_tone))
    {
      case '#':
        /* Stop playing/recording */
        return(1);
      case 1:
        /* CP available */
        BT_ZERO(args_tel);
        if (BfvDataCP(lp2,&args_tel))
        {
            printf("CP data reported available, but is not!\n");
            break;
        }
        if (args_tel.type != CP_TYPE_BOARD_DET)
        {
            printf("Unexpected CP type %u found.\n",args_tel.type);
            break;
        }
        switch (args_tel.data)
        {
          case CS_BUSY1:
          case CS_DIALTON:
            /* Stop on busy/reorder or dialtone */
            return(1);
          case CS_SILENCE:
          case CS_CNGDETCT:
            break;
          default:
            printf("Unexpected CP data value %02X found.\n",args_tel.data);
            break;
        }
        break;
    }
    if (args_tone.res.status != BT_STATUS_OK &&
        args_tone.res.status != BT_STATUS_TIMEOUT)
    {
        BTERR bterr;
        
        BfvErrorMessage(lp2,&args_tone.res,&bterr);
        printf ("BfvToneGet for channel %u: %s\n",
                LINE_UNIT_NUM(lp2),
                bterr.long_msg);
        return(1);
    }

    return (0); /* Keep playing */
}

void
process_aoc(BTLINE *lp)
{
    AOC_INFO *aoc = LINE_AOC_INFO(lp);
        
    if (aoc == NULL || aoc->status == BT_AOC_UNAVAILABLE)
        printf("Charge information unavailable.\n");
    else if (aoc->status == BT_AOC_FREE)
        printf("Charge information: FREE.\n");
    else if (aoc->status == BT_AOC_UNITS)
        printf("Charge information: Unit type %d, value %ld\n",
               aoc->charge_info,aoc->charge);
    else if (aoc->status == BT_AOC_CURRENCY)
    {
        static char *currency_names[] = {
            "1/1000","1/100","1/10","1","10","100","1000" };
#define NUM_CURRENCIES (sizeof(currency_names) / sizeof(*currency_names))
        char *cur_name;
            
        cur_name = (0 <= aoc->charge_info &&
                    aoc->charge_info < NUM_CURRENCIES) ?
            currency_names[aoc->charge_info] :
            "Invalid currency";
        printf("Charge information: Currency multiplier %s (%d), value %ld\n",
               cur_name,aoc->charge_info,aoc->charge);
    }
    else
        printf("Charge information: Bad status value %d\n",
               aoc->status);
}

void
process_call_info(CALL_RES *call_res)
{
    if (strlen(call_res->called_party_number))
        printf("Called party number: \"%s\"\n",
            call_res->called_party_number);
    if (strlen(call_res->called_party_subaddress))
        printf("Called party subaddress: \"%s\"\n",
           call_res->called_party_subaddress);
    if (strlen(call_res->calling_party_number))
        printf("Calling party number: \"%s\"\n",
           call_res->calling_party_number);
    if (strlen(call_res->calling_party_subaddress))
        printf("Calling party subaddress: \"%s\"\n",
           call_res->calling_party_subaddress);
    if (strlen(call_res->redir_number))
        printf("Redirection number: \"%s\", reason %X\n",
           call_res->redir_number, call_res->redir_reason);
}
