/*
 * tiffdump.c -- displays the contents of a TIFF Class F file.
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/**************************************************************************
The tiffdump program displays the contents of a TIFF-F file. It
displays each of the image file directory entries.
***************************************************************************/

/* For Windows only:
   If an application uses a DLL version of the Bfv API library, under some
   conditions the application and the Bfv API library may link to different
   versions or instances of the compiler-supplied C runtime (CRT) libraries.
   This creates incompatibilities between file pointers or allocated memory
   among the application and the Bfv API library.

   Under these conditions, the application must use the Bfv API supplied _dll_
   versions of the file I/O and memory allocation functions if the pointer or
   memory is passed to/from the application and the library.

   The Bfv API DLL libraries links with MSVCRT.DLL. When using the Visual
   Studio mechanism to compile for debugging, applications may be linked with
   MSVCRTD.DLL. When using Visual Studio.NET, applications link with
   MSVCR71.DLL. When using the DLLS version of the Bfv API library, it also
   links with MVCRT.DLL, but it is a different instance of the CRT library.
*/

#include <string.h>
#if defined(unix) || defined(M_XENIX) || defined(_AIX)
#include <unistd.h>
#endif
#include "btlib.h"

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

int debug_opt;
/* Some of the required fields have defaults, and so their actual appearance
   in a file is optional. This list does not include those with defaults. */
int required_fields[] = {
    TAG_NEWSUBFILETYPE, TAG_IMAGEWIDTH, TAG_IMAGELENGTH, TAG_STRIPOFFSETS,
    TAG_ROWSPERSTRIP, TAG_STRIPBYTECOUNTS, TAG_XRESOLUTION, TAG_YRESOLUTION,
    TAG_COMPRESSION, TAG_PHOTOMETRICINTERP, TAG_T4OPTIONS, TAG_T6OPTIONS,
    TAG_PAGENUMBER,
};
#define NUM_REQUIRED (sizeof(required_fields) / sizeof(*required_fields))
int r_f_appears[NUM_REQUIRED];
int page_num;
TFILE *tp;
int t_rd_long BT_ARGS((TFILE *tp,unsigned *longp));
int saved_x_res = 200;
int saved_y_res = 100;
int saved_compression = 3;
int saved_width = 1728;

char *
get_tag_name(unsigned tag)
{
    char *tag_name;
    
    switch (tag)
    {
      case TAG_NEWSUBFILETYPE:
        tag_name = "NEWSUBFILETYPE";
        break;
      case TAG_SUBFILETYPE:
        tag_name = "SUBFILETYPE";
        break;
      case TAG_IMAGEWIDTH:
        tag_name = "IMAGEWIDTH";
        break;
      case TAG_IMAGELENGTH:
        tag_name = "IMAGELENGTH";
        break;
      case TAG_BITSPERSAMPLE:
        tag_name = "BITSPERSAMPLE";
        break;
      case TAG_COMPRESSION:
        tag_name = "COMPRESSION";
        break;
      case TAG_PHOTOMETRICINTERP:
        tag_name = "PHOTOMETRICINTERP";
        break;
      case TAG_THRESHHOLDING:
        tag_name = "THRESHHOLDING";
        break;
      case TAG_CELLWIDTH:
        tag_name = "CELLWIDTH";
        break;
      case TAG_CELLLENGTH:
        tag_name = "CELLLENGTH";
        break;
      case TAG_FILLORDER:
        tag_name = "FILLORDER";
        break;
      case TAG_DOCUMENTNAME:
        tag_name = "DOCUMENTNAME";
        break;
      case TAG_IMAGEDESCRIPTION:
        tag_name = "IMAGEDESCRIPTION";
        break;
      case TAG_MAKE:
        tag_name = "MAKE";
        break;
      case TAG_MODEL:
        tag_name = "MODEL";
        break;
      case TAG_STRIPOFFSETS:
        tag_name = "STRIPOFFSETS";
        break;
      case TAG_SAMPLESPERPIXEL:
        tag_name = "SAMPLESPERPIXEL";
        break;
      case TAG_ROWSPERSTRIP:
        tag_name = "ROWSPERSTRIP";
        break;
      case TAG_STRIPBYTECOUNTS:
        tag_name = "STRIPBYTECOUNTS";
        break;
      case TAG_ORIENTATION:
        tag_name = "ORIENTATION";
        break;
      case TAG_XRESOLUTION:
        tag_name = "XRESOLUTION";
        break;
      case TAG_YRESOLUTION:
        tag_name = "YRESOLUTION";
        break;
      case TAG_PLANARCONFIG:
        tag_name = "PLANARCONFIG";
        break;
      case TAG_PAGENAME:
        tag_name = "PAGENAME";
        break;
      case TAG_XPOSITION:
        tag_name = "XPOSITION";
        break;
      case TAG_YPOSITION:
        tag_name = "YPOSITION";
        break;
      case TAG_FREEOFFSETS:
        tag_name = "FREEOFFSETS";
        break;
      case TAG_FREEBYTECOUNTS:
        tag_name = "FREEBYTECOUNTS";
        break;
      case TAG_GRAYRESPONSEUNIT:
        tag_name = "GRAYRESPONSEUNIT";
        break;
      case TAG_GRAYRESPONSECURVE:
        tag_name = "GRAYRESPONSECURVE";
        break;
      case TAG_T4OPTIONS:
        tag_name = "T4OPTIONS";
        break;
      case TAG_T6OPTIONS:
        tag_name = "T6OPTIONS";
        break;
      case TAG_RESOLUTIONUNIT:
        tag_name = "RESOLUTIONUNIT";
        break;
      case TAG_PAGENUMBER:
        tag_name = "PAGENUMBER";
        break;
      case TAG_TRANSFERFUNCTION:
        tag_name = "TRANSFERFUNCTION";
        break;
      case TAG_SOFTWARE:
        tag_name = "SOFTWARE";
        break;
      case TAG_DATETIME:
        tag_name = "DATETIME";
        break;
      case TAG_ARTIST:
        tag_name = "ARTIST";
        break;
      case TAG_HOSTCOMPUTER:
        tag_name = "HOSTCOMPUTER";
        break;
      case TAG_PREDICTOR:
        tag_name = "PREDICTOR";
        break;
      case TAG_WHITEPOINT:
        tag_name = "WHITEPOINT";
        break;
      case TAG_PRIMARYCHROM:
        tag_name = "PRIMARYCHROMATICITIES";
        break;
      case TAG_COLORMAP:
        tag_name = "COLORMAP";
        break;
      case TAG_HALFTONEHINTS:
        tag_name = "HALFTONEHINTS";
        break;
      case TAG_TILEWIDTH:
        tag_name = "TILEWIDTH";
        break;
      case TAG_TILELENGTH:
        tag_name = "TILELENGTH";
        break;
      case TAG_TILEOFFSETS:
        tag_name = "TILEOFFSETS";
        break;
      case TAG_TILEBYTECOUNTS:
        tag_name = "TILEBYTECOUNTS";
        break;
      case TAG_BADFAXLINES:
        tag_name = "BADFAXLINES";
        break;
      case TAG_CLEANFAXDATA:
        tag_name = "CLEANFAXDATA";
        break;
      case TAG_CONSECBADFAXLINES:
        tag_name = "CONSECBADFAXLINES";
        break;
      case TAG_INKSET:
        tag_name = "INKSET";
        break;
      case TAG_INKNAMES:
        tag_name = "INKNAMES";
        break;
      case TAG_NUMBEROFINKS:
        tag_name = "NUMBEROFINKS";
        break;
      case TAG_DOTRANGE:
        tag_name = "DOTRANGE";
        break;
      case TAG_TARGETPRINTER:
        tag_name = "TARGETPRINTER";
        break;
      case TAG_EXTRASAMPLES:
        tag_name = "EXTRASAMPLES";
        break;
      case TAG_SAMPLEFORMAT:
        tag_name = "SAMPLEFORMAT";
        break;
      case TAG_SMINSAMPLEVALUE:
        tag_name = "SMINSAMPLEVALUE";
        break;
      case TAG_SMAXSAMPLEVALUE:
        tag_name = "SMAXSAMPLEVALUE";
        break;
      case TAG_TRANSFERRANGE:
        tag_name = "TRANSFERRANGE";
        break;
      case TAG_JPEGPROC:
        tag_name = "JPEGPROC";
        break;
      case TAG_JPEGINTCHGFMT:
        tag_name = "JPEGINTERCHANGEFORMAT";
        break;
      case TAG_JPEGINTCHGFMTLEN:
        tag_name = "JPEGINTERCHANGEFORMATLENGTH";
        break;
      case TAG_JPEGRESTARTINTERVAL:
        tag_name = "JPEGRESTARTINTERVAL";
        break;
      case TAG_JPEGLLPREDICTORS:
        tag_name = "JPEGLOSSLESSPREDICTORS";
        break;
      case TAG_JPEGPOINTTRANSFORMS:
        tag_name = "JPEGPOINTTRANSFORMS";
        break;
      case TAG_JPEGQTABLES:
        tag_name = "JPEGQTABLES";
        break;
      case TAG_JPEGDCTABLES:
        tag_name = "JPEGDCTABLES";
        break;
      case TAG_JPEGACTABLES:
        tag_name = "JPEGACTABLES";
        break;
      case TAG_YCBCRCOEFFICIENTS:
        tag_name = "YCBCRCOEFFICIENTS";
        break;
      case TAG_YCBCRSUBSAMPLING:
        tag_name = "YCBCRSUBSAMPLING";
        break;
      case TAG_YCBCRPOSITIONING:
        tag_name = "YCBCRPOSITIONING";
        break;
      case TAG_REFERENCEBLACKWHITE:
        tag_name = "REFERENCEBLACKWHITE";
        break;
      case TAG_COPYRIGHT:
        tag_name = "COPYRIGHT";
        break;
      default:
        tag_name = "(unknown)";
        break;
    }
    return(tag_name);
}

void 
check_requirement(unsigned tag, int appears, unsigned val)
{
    int bad = 0;
    int tolerate = 0;
    int tol_missing = 0;
    unsigned ltmp;
    unsigned ltmp1, ltmp2;
    int res1,res2;
    
    /* Certain tags only required depending on the values of other tags */
    if (tag == TAG_T4OPTIONS && saved_compression == 4 ||
        tag == TAG_T6OPTIONS && saved_compression != 4)
        return;

    if (appears)
    {
        switch (tag)
        {
          case TAG_NEWSUBFILETYPE:
            if (val != 2)
                bad = 1;
            break;
          case TAG_IMAGEWIDTH:
            if (val != 1728 && val != 2048 && val != 2432 &&
                val != 2592 && val != 3072 && val != 3648 &&
                val != 3456 && val != 4096 && val != 4864 &&
                val != 5184 && val != 6144 && val != 7296 &&
                val != 10368 && val != 12288 && val != 14592 &&
                val != 864 && val != 1024 && val != 1216)
                bad = 1;
            saved_width = (bad) ? 0 : val;
            break;
          case TAG_BITSPERSAMPLE:
            if (val != 1)
                bad = 1;
            break;
          case TAG_COMPRESSION:
            if (val != 3 && val != 4)
                bad = 1;
            saved_compression = val;
            break;
          case TAG_PHOTOMETRICINTERP:
            if (val != 0 && val != 1)
                bad = 1;
            break;
          case TAG_FILLORDER:
            if (val != 1 && val != 2)
                bad = 1;
            break;
          case TAG_SAMPLESPERPIXEL:
            if (val != 1)
                bad = 1;
            break;
          case TAG_XRESOLUTION:
#ifdef MILL_DLL
            ltmp = _dll_ftell(TIFF_FP(tp));
            if (_dll_fseek(TIFF_FP(tp),(long)val,SEEK_SET))
#else
            ltmp = ftell(TIFF_FP(tp));
            if (fseek(TIFF_FP(tp),(long)val,SEEK_SET))
#endif
            {
                printf("*** Error seeking to offset %X\n",val);
                exit(1);
            }
            if (t_rd_long(tp,&ltmp1) || t_rd_long(tp,&ltmp2))
            {
                printf("*** Error reading resolution from offset %X\n",val);
                exit(1);
            }
#ifdef MILL_DLL
            if (_dll_fseek(TIFF_FP(tp),(long)ltmp,SEEK_SET))
#else
            if (fseek(TIFF_FP(tp),(long)ltmp,SEEK_SET))
#endif
            {
                printf("*** Error seeking to offset %X\n",ltmp);
                exit(1);
            }
            val = ltmp1;
            printf("\t%d / %d = ",ltmp1,ltmp2);
            if (ltmp2 == 0)
            {
                ltmp = 0;
                bad = 1;
                printf("zero denominator\n");
                break;
            }
            if ((ltmp1 % ltmp2) != 0)
            {
                bad = 1;
                printf("non-integer, truncated to ");
            }
            if ((ltmp = ltmp1 / ltmp2) != 204 && ltmp != 300 && ltmp != 400 &&
                ltmp != 200 && ltmp != 408 && ltmp != 600 && ltmp != 1200 &&
                ltmp != 100 &&
                ltmp != 77)
                bad = 1;
            printf("%d\n",ltmp);
            saved_x_res = ltmp;
            break;
          case TAG_YRESOLUTION:
#ifdef MILL_DLL
            ltmp = _dll_ftell(TIFF_FP(tp));
            if (_dll_fseek(TIFF_FP(tp),(long)val,SEEK_SET))
#else
            ltmp = ftell(TIFF_FP(tp));
            if (fseek(TIFF_FP(tp),(long)val,SEEK_SET))
#endif
            {
                printf("*** Error seeking to offset %X\n",val);
                exit(1);
            }
            if (t_rd_long(tp,&ltmp1) || t_rd_long(tp,&ltmp2))
            {
                printf("*** Error reading resolution from offset %X\n",val);
                exit(1);
            }
#ifdef MILL_DLL
            if (_dll_fseek(TIFF_FP(tp),(long)ltmp,SEEK_SET))
#else
            if (fseek(TIFF_FP(tp),(long)ltmp,SEEK_SET))
#endif
            {
                printf("*** Error seeking to offset %X\n",ltmp);
                exit(1);
            }
            val = ltmp1;
            printf("\t%d / %d = ",ltmp1,ltmp2);
            if (ltmp2 == 0)
            {
                ltmp = 0;
                bad = 1;
                printf("zero denominator\n");
                break;
            }
            if ((ltmp1 % ltmp2) != 0)
            {
                /* Allow for 38.5 cm */
                if (!(ltmp1 / ltmp2 == 38 && ((2 * ltmp1) % ltmp2) == 0))
                    bad = 1;
                printf("non-integer, truncated to ");
            }
            if ((ltmp = (ltmp1 / ltmp2)) != 98 && ltmp != 196 &&
                ltmp != 300 && ltmp != 400 && ltmp != 100 && ltmp != 200 &&
                ltmp != 391 && ltmp != 600 && ltmp != 1200 && ltmp != 800 &&
                ltmp != 100 &&
                ltmp != 77 && ltmp != 38)
                bad = 1;
            printf("%d\n",ltmp);
            saved_y_res = ltmp;
            break;
          case TAG_T4OPTIONS:
            if (val & ~0x5)
                bad = 1;
            break;
          case TAG_T6OPTIONS:
            if (val != 0)
                bad = 1;
            break;
          case TAG_RESOLUTIONUNIT:
            if (val != 2 && val != 3)
                bad = 1;
            break;
          case TAG_PAGENUMBER:
            if ((int)(val & 0xffff) != page_num)
                bad = 1;
            /* On first page only, upper short must be count of pages,
               which is non-zero. */
            if (page_num == 0 && ((val >> 16) & 0xffff) == 0)
                bad = 1;
            break;
          default:
            return;
        }
        if (bad)
        {
            printf("*** Required tag %s (%d) has bad value %08X",
                   get_tag_name(tag),tag,val);
            if (tag == TAG_XRESOLUTION || tag == TAG_YRESOLUTION)
                printf("/%08X (%d)",ltmp2,ltmp);
            else
                printf(" (%u)",val);
            printf(".\n");
        }
    }
    else
        printf("*** Required tag %s (%d) missing.\n",get_tag_name(tag),tag);
    if (!appears || bad)
    {
        printf("\tValid value(s) are ");
        switch (tag)
        {
          case TAG_NEWSUBFILETYPE:
            printf("2.\n");
            if (bad)
                tolerate = 1;
            else
                tol_missing = 1;
            break;
          case TAG_IMAGEWIDTH:
            printf("1728,2048,2432/200dpi; 2592,3072,3648/300dpi; 3456,4096,4864/400dpi.\n");
            printf("5184,6144,7296/600dpi; 10368,12288,14592/1200dpi; 864,1024,1216/100dpi.\n");
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_IMAGELENGTH:
            printf("length of image.\n");
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_BITSPERSAMPLE:
            printf("1.\n");
            if (bad)
                tolerate = 1;
            else
                tol_missing = 1;
            break;
          case TAG_COMPRESSION:
            printf("3 and 4.\n");
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_PHOTOMETRICINTERP:
            printf("0 and 1\n");
            if (bad)
                tolerate = 1;
            else
                tol_missing = 1;
            break;
          case TAG_FILLORDER:
            printf("1 and 2.\n");
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_STRIPOFFSETS:
            printf("offset of each strip.\n");
            break;
          case TAG_SAMPLESPERPIXEL:
            printf("1.\n");
            if (bad)
                tolerate = 1;
            else
                tol_missing = 1;
            break;
          case TAG_ROWSPERSTRIP:
            printf("number of rows in each strip.\n");
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_STRIPBYTECOUNTS:
            printf("number of bytes in each strip.\n");
            break;
          case TAG_XRESOLUTION:
            printf("fractions representing 200, 204, 300, 400, 408, 600, 1200,\n");
            printf("    or 100 (or 77 cm).\n");
            if (bad &&
                (194 <= ltmp && ltmp <= 214 ||
                 290 <= ltmp && ltmp <= 310 ||
                 390 <= ltmp && ltmp <= 410 ||
                 590 <= ltmp && ltmp <= 610 ||
                 1190 <= ltmp && ltmp <= 1210 ||
                 90 <= ltmp && ltmp <= 110 ||
                 75 <= ltmp && ltmp <= 79)) /* 77 for cm */
                tolerate = 1;
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_YRESOLUTION:
            printf("fractions representing 98, 100, 196, 200, 300, 391, 400, 600, 800,\n");
            printf("    1200, or 100 (or 38.5, 77 cm).\n");
            if (bad &&
                (88 <= ltmp && ltmp <= 108 ||
                 186 <= ltmp && ltmp <= 206 ||
                 290 <= ltmp && ltmp <= 310 ||
                 390 <= ltmp && ltmp <= 410 ||
                 37 <= ltmp && ltmp <= 40 || /* 38.5 and 77 for cm */
                 75 <= ltmp && ltmp <= 79))
                tolerate = 1;
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_T4OPTIONS:
            printf("0, 1, 4, or 5.\n");
            if (bad)
                tolerate = 1;
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_T6OPTIONS:
            printf("0.\n");
            if (bad)
                tolerate = 1;
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_RESOLUTIONUNIT:
            printf("2 or 3.\n");
            if (!appears)
                tol_missing = 1;
            break;
          case TAG_PAGENUMBER:
            printf("lower 16 bits %04X",page_num);
            if (page_num == 0)
                printf(", upper 16 bits non-zero");
            printf(".\n");
            if (bad)
                tolerate = 1;
            else
                tol_missing = 1;
            break;
          default:
            return;
        }
        if (tolerate)
            printf("\tBrooktrout functions will tolerate this bad value.\n");
        if (tol_missing)
            printf("\tBrooktrout functions will tolerate the lack of this tag.\n");
    }
    /* Special case: If X resolution, checked stored width to make sure
       it is valid. The value must be one of the 9 valid ones. */
    if (tag == TAG_XRESOLUTION && saved_width)
    {
        if (!appears)
            ltmp = 204;
        if ((194 <= ltmp && ltmp <= 214 && saved_width != 1728 &&
             saved_width != 2048 && saved_width != 2432) ||
            (290 <= ltmp && ltmp <= 310 && saved_width != 2592 &&
             saved_width != 3072 && saved_width != 3648) ||
            (390 <= ltmp && ltmp <= 410 && saved_width != 3456 &&
             saved_width != 4096 && saved_width != 4864) ||
            (590 <= ltmp && ltmp <= 610 && saved_width != 5184 &&
             saved_width != 6144 && saved_width != 7296) ||
            (1190 <= ltmp && ltmp <= 1210 && saved_width != 10368 &&
             saved_width != 12288 && saved_width != 14592) ||
            (90 <= ltmp && ltmp <= 110 && saved_width != 864 &&
             saved_width != 1024 && saved_width != 1216))
        {
            printf("*** Previous tag %s (%d) has bad value %08X (%u).\n",
                   get_tag_name(TAG_IMAGEWIDTH),TAG_IMAGEWIDTH,
                   saved_width,saved_width);
            printf("\tValid value(s) are 1728,2048,2432/200dpi; 2592,3072,3648/300dpi; 3456,4096,4864/400dpi.\n");
            printf("    5184,6144,7296/600dpi; 10368,12288,14592/1200dpi; 864,1024,1216/100dpi.\n");
        }
    }
    
    if (tag == TAG_RESOLUTIONUNIT && !bad)
    {
        if (appears && val == TIFF_RES_UNIT_CM)
        {
            if (!(saved_x_res == 77 &&
                  (saved_y_res == 38 || saved_y_res == 77)))
                bad = 1;
        }
        else
        {
            res1 = ((saved_x_res + 20) / 100) * 100;
            res2 = ((saved_y_res + 10) / 100) * 100;
            
            if (!(res1 == 200 && res2 == 100 ||
                  res1 == 200 && res2 == 200 ||
                  res1 == 200 && res2 == 400 ||
                  res1 == 300 && res2 == 300 ||
                  res1 == 400 && res2 == 400 ||
                  res1 == 600 && res2 == 600 ||
                  res1 == 1200 && res2 == 1200 ||
                  res1 == 300 && res2 == 600 ||
                  res1 == 400 && res2 == 800 ||
                  res1 == 600 && res2 == 1200 ||
                  res1 == 100 && res2 == 100))
                bad = 1;
        }
        if (bad)
        {
            printf("Invalid resolution combination %d x %d\n",
                   saved_x_res, saved_y_res);
        }
    }
}

void
usage()
{
    printf("Usage: tiffdump [-d] tiff_file\n");
    exit(0);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    int tmp;
    int dump_func BT_ARGS((TFILE *tp,struct ifd_field *ifd,char *arg));
    struct args_tiff args;
    struct args_debug_mode args_debug;
    BTERR bterr;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "d")) != EOF)
    {
        switch (c)
        {
          case 'd':
            debug_opt = 1;
            break;
          default:
            usage();
        }
    }
    
    if (argc - optind != 1)
        usage();

    BT_ZERO(args_debug);
    args_debug.mode = DEBUG_ALL;
    args_debug.options = API_DBG_MODE_SET;
    BfvDebugModeSetAdv(&args_debug);
    
    BT_ZERO(args);
    args.fname = argv[optind];
    args.fmode = "r";
    tp = BfvTiffOpen(&args);
    if (tp == NULL)
    {
        BfvErrorMessage(NULL,&args.res,&bterr);
        fprintf (stderr, "BfvTiffOpen: %s\n",bterr.long_msg);
        exit(1);
    }
    
    printf("Tiff file %s opened. Bytes revd=%d.\n",
           argv[optind],tp->bytes_reversed);
    
    for (;;)
    {
        if (debug_opt)
        {
            unsigned ltmp;
            
            memset(r_f_appears,0,sizeof(r_f_appears));
            
            /* Makes use of an internal API field to locate the offset
               of the offset of the next IFD entry */
#ifdef MILL_DLL
            if (_dll_fseek(TIFF_FP(tp),(long)tp->cur_offset_offset,SEEK_SET))
#else
            if (fseek(TIFF_FP(tp),(long)tp->cur_offset_offset,SEEK_SET))
#endif
            {
                printf("*** Error seeking to offset %X\n",
                       tp->cur_offset_offset);
                exit(1);
            }
            t_rd_long(tp,&ltmp);
            if ((ltmp & 1) != 0)
            {
                printf("*** Invalid odd IFD offset value %08X.\n",ltmp);
                printf("\tBrooktrout functions will tolerate this bad value.\n");
            }
        }
        args.tp = tp;
        args.func = dump_func;
        tmp = BfvTiffReadIFD(&args);
        if (tmp == 0)
            break;
        if (tmp < 0)
        {
            BfvErrorMessage(NULL,&args.res,&bterr);
            printf ("BfvTiffReadIFD: %s\n",bterr.long_msg);
        }
        if (debug_opt)
        {
            for (c = 0; c < NUM_REQUIRED; c++)
            {
                if (r_f_appears[c] == 0)
                    check_requirement(required_fields[c],0,0);
            }
        }
        if (tmp <= 0)
            break;
        printf("Page info done.\n");
        page_num++;
    }
    
    args.tp = tp;
    BfvTiffClose(&args);
    
    return(0);
}

int
dump_func(TFILE *tp, struct ifd_field *ifd, char *arg)
{
    unsigned ltmp;
    static int sizes[] = { 0, 1, 1, 2, 4, 8, 1, 1, 2, 4, 8, 4, 8 };
    static char *ft_names[] = {
        "BYTE", "ASCII", "SHORT", "LONG", "RATNL", "SBYTE", "UNDEF",
        "SSHORT", "SLONG", "SRATNL", "FLOAT", "DOUBLE",
    };  
    int tmp;
    unsigned char buf[1024];
    unsigned buf_size;
    char *tag_name,*ft_name;
    unsigned converted_offset;
    
    if (ifd == NULL)
        return(0);

    tag_name = get_tag_name(ifd->tag);
    
    if (tmp = (ifd->field_type < FT_BYTE || ifd->field_type > FT_DOUBLE))
        ft_name = "???";
    else
        ft_name = ft_names[ifd->field_type - 1];

    switch (ifd->field_type)
    {
      case FT_BYTE:
        converted_offset = (unsigned)((unsigned char)ifd->offset);
        break;
      case FT_SHORT:
        converted_offset = (unsigned)((unsigned short)ifd->offset);
        break;
      case FT_LONG:
      default:
        converted_offset = ifd->offset;
        break;
    }

    printf("Tag %-17s (%u), field type %-6s(%u), count %2d, offset %08X\n",
           tag_name,ifd->tag,ft_name,ifd->field_type,ifd->count,
           converted_offset);
    if (tmp == 0 && sizes[ifd->field_type] * ifd->count > 4)
    {
#ifdef MILL_DLL
        ltmp = _dll_ftell(TIFF_FP(tp));
        if (_dll_fseek(TIFF_FP(tp),(long)ifd->offset,SEEK_SET))
#else
        ltmp = ftell(TIFF_FP(tp));
        if (fseek(TIFF_FP(tp),(long)ifd->offset,SEEK_SET))
#endif
        {
            printf("*** Error seeking to offset %X\n",ifd->offset);
            exit(1);
        }
        buf_size = sizes[ifd->field_type] * ifd->count;
        if (buf_size > sizeof(buf) - 1)
            buf_size = sizeof(buf) - 1;
#ifdef MILL_DLL
        if (_dll_fread((char *)buf,buf_size,1,TIFF_FP(tp)) != 1)
#else
        if (fread((char *)buf,buf_size,1,TIFF_FP(tp)) != 1)
#endif
        {
            printf("*** Error reading %d bytes from offset %X\n",
                   sizes[ifd->field_type] * ifd->count,ifd->offset);
            exit(1);
        }
        putchar('\t');
        for (tmp = 0; tmp < (int)(sizes[ifd->field_type] * ifd->count); tmp++)
            printf("%02X ",buf[tmp]);
        if (buf_size == sizeof(buf) - 1)
            printf("...");
        putchar('\n');
        if (ifd->field_type == FT_ASCII)
        {
            buf[ifd->count] = 0;
            printf("\t\"%s\"\n",buf);
        }
#ifdef MILL_DLL
        if (_dll_fseek(TIFF_FP(tp),(long)ltmp,SEEK_SET))
#else
        if (fseek(TIFF_FP(tp),(long)ltmp,SEEK_SET))
#endif
        {
            printf("*** Error seeking to offset %X\n",ltmp);
            exit(1);
        }
        
        if (debug_opt && (ifd->offset & 1) != 0)
        {
            printf("*** Invalid odd IFD entry offset value %08X.\n",
                   ifd->offset);
            printf("\tBrooktrout functions will tolerate this bad value.\n");
        }
    }
    
    if (debug_opt)
    {
        for (tmp = 0; tmp < NUM_REQUIRED; tmp++)
        {
            if (required_fields[tmp] == ifd->tag)
            {
                r_f_appears[tmp] = 1;
                break;
            }
        }
        check_requirement(ifd->tag,1,converted_offset);
    }
    
    return(0);
}

int
t_rd_long(TFILE *tp, unsigned *longp)
{
    unsigned ltmp;
    
#ifdef MILL_DLL
    if (_dll_fread((char *)&ltmp,sizeof(ltmp),1,TIFF_FP(tp)) != 1)
#else
    if (fread((char *)&ltmp,sizeof(ltmp),1,TIFF_FP(tp)) != 1)
#endif
        return(1);
#ifndef BT_BIG_ENDIAN
    if (tp->bytes_reversed)
#else
    if (!tp->bytes_reversed)
#endif
    {
        ((unsigned char *)longp)[0] = ((unsigned char *)&ltmp)[3];
        ((unsigned char *)longp)[1] = ((unsigned char *)&ltmp)[2];
        ((unsigned char *)longp)[2] = ((unsigned char *)&ltmp)[1];
        ((unsigned char *)longp)[3] = ((unsigned char *)&ltmp)[0];
    }
    else
        *longp = ltmp;
    return(0);
}
