/* connlist.c */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
The connlist program lists currently established call switching
connections. 

Full-duplex connections are always reported as a pair of
simplex transmit-only connections, where the source and destination
endpoints are swapped for the two halves of the full-duplex
connection.
***************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#if defined(WINNT) || defined(__OS2__) || defined(__QNX__) || defined(WIN95)
#include <stdlib.h>
#endif
#include <errno.h>
#include <time.h>
#include "btlib.h"

unsigned module = 0x2;
int exit_code = 0;

void dowork(BTLINE *lp);

void
usage ()
{
    fprintf (stderr, "usage: connlist [-m mod] [-v]\n");
    fprintf (stderr, "  -m mod  Use specified module (dflt %X)\n",
             module);
    fprintf (stderr, "  -v  Enable API debug mode output\n");
    exit (1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp;
    BTERR bterr;
    struct args_line_admin args;
    struct args_addr_info args_info;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "m:v")) != EOF)
    {
        switch (c)
        {
          case 'm':
            if (sscanf(optarg,"%x",&module) != 1)
                usage();
            break;
          case 'v':
            BfvDebugModeSet(DEBUG_ALL);
            break;
          default:
            usage ();
        }
    }

    if (optind != argc)
        usage ();
    BT_ZERO(args_info);
    args_info.mod_num = module;
    BfvModuleInfo(NULL,&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(NULL,&args_info.res,&bterr);
        fprintf (stderr,"BfvModuleInfo: %s\n",bterr.long_msg);
        exit(1);
    }
    if (!args_info.exists)
    {
        fprintf(stderr,"Specified module (%X) doesn't exist.\n",module);
        exit(1);
    }
    if (!args_info.hardware)
    {
        fprintf(stderr,"Specified module (%X) is not a hardware module.\n",
                module);
        exit(1);
    }
    
    BT_ZERO(args);
    args.dest_addr.mm_bModule = module;
    args.dest_addr.mm_bChannel = 1;
    args.present = 1;
    if ((lp = BfvSessionAttach(&args)) == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        fprintf (stderr,"BfvSessionAttach: %s\n",bterr.long_msg);
        exit(1);
    }
       
    dowork(lp);
    
    BT_ZERO(args);
    BfvSessionDetach(lp,&args);
    
    return(exit_code);
}

void print_info(unsigned port_class,
                unsigned port_unit,
                unsigned stream,
                unsigned slot,
                unsigned port_type,
                struct transport_options *opt)
{
    static char *port_classes[] = {
        "CHANNEL",
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        "UDPTL",
        NULL,
        "BUS",
        "NETWORK",
        NULL,
        NULL,
        NULL,
        "RTP",
    };
#define NUM_PORT_CLASSES (sizeof(port_classes) / sizeof(*port_classes))
    char *port_class_s;
    char port_class_tmp_buf[20];
    
    static char *port_types[] = {
        "CHANNEL",
        NULL,
        "MVIP",
        "PEB",
        "SCBUS",
        "H100",
        "T1",
        "E1",
        "PRI_T1",
        "PRI_E1",
        "AT_MODEM",
        "BRI",
        "ANALOG_LOOP_ST",
        "ANALOG_DID",
    };
#define NUM_PORT_TYPES (sizeof(port_types) / sizeof(*port_types))
    char *port_type_s;
    char port_type_tmp_buf[20];

    if (port_class >= NUM_PORT_CLASSES ||
        (port_class_s = port_classes[port_class]) == NULL)
    {
        sprintf(port_class_tmp_buf,"?%X",port_class);
        port_class_s = port_class_tmp_buf;
    }
    printf("Class %7s, ",port_class_s);
        
    switch (port_class)
    {
      default:
        /* For everything except the IP cases */
        if (port_type >= NUM_PORT_TYPES ||
            (port_type_s = port_types[port_type]) == NULL)
        {
            sprintf(port_type_tmp_buf,"?%X",port_type);
            port_type_s = port_type_tmp_buf;
        }
        printf("unit %3d, stream %3d, slot %3d, type %10s\n",
               port_unit,stream,slot,port_type_s);
        break;
      case CALL_SW_PORT_UDPTL_DEF:
        printf("local_ip %u.%u.%u.%u, local_port %u, remote_ip %u.%u.%u.%u, remote_port %u\n",
               opt->UDPTLopts.localAddr.sin_addr.S_un.S_un_b.s_b1,
               opt->UDPTLopts.localAddr.sin_addr.S_un.S_un_b.s_b2,
               opt->UDPTLopts.localAddr.sin_addr.S_un.S_un_b.s_b3,
               opt->UDPTLopts.localAddr.sin_addr.S_un.S_un_b.s_b4,
               opt->UDPTLopts.localAddr.sin_port,
               opt->UDPTLopts.remoteAddr.sin_addr.S_un.S_un_b.s_b1,
               opt->UDPTLopts.remoteAddr.sin_addr.S_un.S_un_b.s_b2,
               opt->UDPTLopts.remoteAddr.sin_addr.S_un.S_un_b.s_b3,
               opt->UDPTLopts.remoteAddr.sin_addr.S_un.S_un_b.s_b4,
               opt->UDPTLopts.remoteAddr.sin_port);
#if 0
        printf("              t38_version %d\n",
               opt->UDPTLopts.t38_version);
        printf("              t38_max_bit_rate %d\n",
               opt->UDPTLopts.t38_max_bit_rate);
        printf("              t38_fax_fill_bit_removal %d\n",
               opt->UDPTLopts.t38_fax_fill_bit_removal);
        printf("              t38_fax_transcoding_MMR %d\n",
               opt->UDPTLopts.t38_fax_transcoding_MMR);
        printf("              t38_fax_transcoding_JBIG %d\n",
               opt->UDPTLopts.t38_fax_transcoding_JBIG);
        printf("              t38_fax_rate_management %d\n",
               opt->UDPTLopts.t38_fax_rate_management);
        printf("              t38_fax_max_buffer %d\n",
               opt->UDPTLopts.t38_fax_max_buffer);
        printf("              t38_fax_max_datagram_recv %d\n",
               opt->UDPTLopts.t38_fax_max_datagram_recv);
        printf("              t38_fax_udp_EC %d\n",
               opt->UDPTLopts.t38_fax_udp_EC);
        printf("              t38_fax_max_datagram_send %d\n",
               opt->UDPTLopts.t38_fax_max_datagram_send);
        printf("              t38_UDPTL_redundancy_depth_control %d\n",
               opt->UDPTLopts.t38_UDPTL_redundancy_depth_control);
        printf("              t38_UDPTL_redundancy_depth_image %d\n",
               opt->UDPTLopts.t38_UDPTL_redundancy_depth_image);
        printf("              t38_t30_fast_notify %d\n",
               opt->UDPTLopts.t38_t30_fast_notify);
#endif
        break;
      case CALL_SW_PORT_RTP_DEF:
        printf("local_RTP_ip %u.%u.%u.%u, local_RTP_port %u, remote_RTP_ip %u.%u.%u.%u, remote_RTP_port %u, local_RTCP_ip %u.%u.%u.%u, local_RTCP_port %u, remote_RTCP_ip %u.%u.%u.%u, remote_RTCP_port %u\n",
               opt->RTPopts.localRTPAddr.sin_addr.S_un.S_un_b.s_b1,
               opt->RTPopts.localRTPAddr.sin_addr.S_un.S_un_b.s_b2,
               opt->RTPopts.localRTPAddr.sin_addr.S_un.S_un_b.s_b3,
               opt->RTPopts.localRTPAddr.sin_addr.S_un.S_un_b.s_b4,
               opt->RTPopts.localRTPAddr.sin_port,
               opt->RTPopts.remoteRTPAddr.sin_addr.S_un.S_un_b.s_b1,
               opt->RTPopts.remoteRTPAddr.sin_addr.S_un.S_un_b.s_b2,
               opt->RTPopts.remoteRTPAddr.sin_addr.S_un.S_un_b.s_b3,
               opt->RTPopts.remoteRTPAddr.sin_addr.S_un.S_un_b.s_b4,
               opt->RTPopts.remoteRTPAddr.sin_port,
               opt->RTPopts.localRTCPAddr.sin_addr.S_un.S_un_b.s_b1,
               opt->RTPopts.localRTCPAddr.sin_addr.S_un.S_un_b.s_b2,
               opt->RTPopts.localRTCPAddr.sin_addr.S_un.S_un_b.s_b3,
               opt->RTPopts.localRTCPAddr.sin_addr.S_un.S_un_b.s_b4,
               opt->RTPopts.localRTCPAddr.sin_port,
               opt->RTPopts.remoteRTCPAddr.sin_addr.S_un.S_un_b.s_b1,
               opt->RTPopts.remoteRTCPAddr.sin_addr.S_un.S_un_b.s_b2,
               opt->RTPopts.remoteRTCPAddr.sin_addr.S_un.S_un_b.s_b3,
               opt->RTPopts.remoteRTCPAddr.sin_addr.S_un.S_un_b.s_b4,
               opt->RTPopts.remoteRTCPAddr.sin_port);
        break;
    }
}


void
dowork(BTLINE *lp)
{
    struct args_line_admin args_admin;
    struct args_tel_ctrl_call_sw args_tel;
    BTERR bterr;
    int ret;
    
    static char *conn_modes[] = {
        NULL,
        "TRANSMIT_ONLY",
        "RECEIVE_ONLY",
        "FULL_DUPLEX",
        NULL,
        NULL,
        NULL,
        "DUPL_SIG",
        NULL,
        "SDINV_TRANSMIT_ONLY",
        "SDINV_RECEIVE_ONLY",
        "SDINV_FULL_DUPLEX",
        NULL,
        NULL,
        NULL,
        "SDINV_DUPL_SIG",
    };
#define NUM_CONN_MODES (sizeof(conn_modes) / sizeof(*conn_modes))
    char *conn_mode;
    char conn_mode_tmp_buf[20];
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    args_admin.skip_teleph = 1;
    ret = BfvLineReset (lp, &args_admin);
    if (ret < 0)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset: %s: status %lX.\n",
                 bterr.long_msg,args_admin.reset_status);
        exit(1);
    }
    
    BT_ZERO(args_tel);
    while ((ret = BfvCallSWGetConns(lp,&args_tel)) > 0)
    {
        if (args_tel.conn_mode >= NUM_CONN_MODES ||
            (conn_mode = conn_modes[args_tel.conn_mode]) == NULL)
        {
            sprintf(conn_mode_tmp_buf,"?%X",args_tel.conn_mode);
            conn_mode = conn_mode_tmp_buf;
        }
        printf("Conn mode: %19s\n",conn_mode);
        
        printf("    Source:   ");
        print_info(args_tel.src_port_class,
                   args_tel.src_port_unit,
                   args_tel.src_stream,
                   args_tel.src_slot,
                   args_tel.src_port_type,
                   &args_tel.srcOptions);

        printf("      Dest:   ");
        print_info(args_tel.dest_port_class,
                   args_tel.dest_port_unit,
                   args_tel.dest_stream,
                   args_tel.dest_slot,
                   args_tel.dest_port_type,
                   &args_tel.destOptions);
    }
    if (ret < 0)
    {
        BfvErrorMessage(lp,&args_tel.res,&bterr);
        printf ("BfvCallSWGetConns: %s\n",bterr.long_msg);
        exit(1);
    }
}
