/* deact.c */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
The deact program deactivates a hardware module, marking it as
dead.
***************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#if defined(WINNT) || defined(__OS2__) || defined(__QNX__) || defined(WIN95)
#include <stdlib.h>
#endif
#include <errno.h>
#include <time.h>
#include "btlib.h"

void dowork(BTLINE *lp);

void
usage ()
{
    fprintf (stderr, "usage: deact [-a] [-s] mod\n");
    fprintf (stderr, "-a = Deactivate all modules on the same board\n");
    fprintf (stderr, "-s = Use cPCI slot number instead of module number\n");
    exit (1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTERR bterr;
    struct args_addr_info args_info;
    unsigned module;
    int do_all = 0, use_slot = 0;
    
    BT_API_SET_VER();
    /*    BfvDebugModeSet(DEBUG_ALL); */
    
    while ((c = getopt (argc, argv, "as")) != EOF)
    {
        switch (c)
        {
          case 'a':
            do_all = 1;
            break;
          case 's':
            use_slot = 1;
            break;
          default:
            usage ();
        }
    }
    
    if (optind != argc - 1)
        usage();

    if (sscanf(argv[optind],"%x",&module) != 1)
        usage();

    if (!use_slot)
    {
        BT_ZERO(args_info);
        args_info.mod_num = module;
        BfvModuleInfo(NULL,&args_info);
        if (args_info.res.status != BT_STATUS_OK)
        {
            BfvErrorMessage(NULL,&args_info.res,&bterr);
            fprintf (stderr,"BfvModuleInfo: %s\n",bterr.long_msg);
            exit(1);
        }
        if (!args_info.exists)
        {
            fprintf(stderr,"Specified module (%X) doesn't exist.\n",module);
            exit(1);
        }
        if (!args_info.hardware)
        {
            fprintf(stderr,"Specified module (%X) is not a hardware module.\n",
                    module);
            exit(1);
        }
    }
    
    BT_ZERO(args_info);
    args_info.deact_mode = (do_all) ? DEACT_MODE_ALL : DEACT_MODE_SINGLE;
    if (use_slot)
        args_info.slot_num = module;
    else
        args_info.mod_num = module;
    BfvModuleDeactivate(&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(NULL,&args_info.res,&bterr);
        fprintf (stderr,"BfvModuleDeactivate: %s\n",bterr.long_msg);
        exit (1);
    }
    
    if (do_all)
        printf("All modules on board with module %X deactivated.\n",module);
    else if (use_slot)
        printf("All modules on board in slot %d deactivated.\n",module);
    else
        printf("Module %X deactivated.\n",module);

    return(0);
}
