/* feature.c */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
The feature program manipulates feature set data.
It can query or download feature set data.
***************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#if defined(WINNT) || defined(__OS2__) || defined(__QNX__) || defined(WIN95)
#include <stdlib.h>
#endif
#include <errno.h>
#include <time.h>
#include "btlib.h"

unsigned module = 0x2;
int action;
char *passwd;
char *feature_file;
int raw_mode = 0;
int exit_code = 0;

void dowork(BTLINE *lp);

void
usage ()
{
    fprintf (stderr, "usage: feature [-m mod] [-v] <action>\n");
    fprintf (stderr, "  -m mod  Apply action to specified module (dflt %X)\n",
             module);
    fprintf (stderr, "  -v      Enable API debug mode\n");
    fprintf (stderr, "Possible actions:\n");
    fprintf (stderr, "  -q = Query loaded feature set\n");
    fprintf (stderr, "  -d feature_file = Download ASCII feature file\n");
    fprintf (stderr, "      -b = perform download using binary file\n");
    exit (1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp = NULL;
    BTERR bterr;
    struct args_line_admin args;
    struct args_addr_info args_info;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "m:qd:bv")) != EOF)
    {
        switch (c)
        {
          case 'm':
            if (sscanf(optarg,"%x",&module) != 1)
                usage();
            break;
          case 'q':
            action = c;
            break;
          case 'd':
            action = c;
            feature_file = optarg;
            break;
          case 'b':
            raw_mode = 1;
            break;
          case 'v':
            BfvDebugModeSet(DEBUG_ALL);
            break;
          default:
            usage ();
        }
    }
    
    if (action == 0 || optind != argc)
        usage ();
    BT_ZERO(args_info);
    args_info.mod_num = module;
    BfvModuleInfo(NULL,&args_info);
    if (!args_info.exists)
    {
        fprintf(stderr,"Specified module (%X) doesn't exist.\n",module);
        exit(1);
    }
    if (!args_info.hardware)
    {
        fprintf(stderr,"Specified module (%X) is not a hardware module.\n",
                module);
        exit(1);
    }
    
    BT_ZERO(args);
    args.dest_addr.mm_bModule = module;
    args.dest_addr.mm_bChannel = 1;
    args.present = 1;
    args.unique = 0;
    if ((lp = BfvSessionAttach(&args)) == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        fprintf (stderr,"BfvSessionAttach: %s\n",bterr.long_msg);
        exit(1);
    }
       
    dowork(lp);
    
    return(exit_code);
}

void
dowork(BTLINE *lp)
{
    struct args_line_admin args_admin;
    struct args_feature_set args_fs;
    BTERR bterr;
    int ret;
    FILE *fp;
    unsigned char buf[512];
    unsigned utmp;
    unsigned bufct;
    int data_found;
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    args_admin.skip_teleph = 1;
    if (BfvLineReset (lp, &args_admin) < 0)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset: %s: status %lX.\n",
                 bterr.long_msg,args_admin.reset_status);
        exit(1);
    }
    
    switch (action)
    {
      case 'q':
        for (;;)
        {
            BT_ZERO(args_fs);
            BfvFeatureSetQuery(lp,&args_fs);
            if (args_fs.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fs.res,&bterr);
                fprintf (stderr,"BfvFeatureSetQuery: %s\n",bterr.long_msg);
                exit(1);
            }
            if (args_fs.key[0] == 0)
                return;

            switch (args_fs.type)
            {
              case FEATURE_BOOLEAN:
                printf("Feature: %s = %c\n",
                       args_fs.key, (args_fs.value_int) ? 'T' : 'F');
                break;
              case FEATURE_INTEGER:
                printf("Feature: %s = %d\n",args_fs.key, args_fs.value_int);
                break;
              case FEATURE_STRING:
                printf("Feature: %s = %s\n",
                       args_fs.key, args_fs.output_value_string);
                break;
              default:
                printf("Feature error, key %s unknown type %d\n",
                       args_fs.key, args_fs.type);
                exit(1);
            }
        }
        break;
      case 'd':
        if (raw_mode)
        {
            BT_ZERO(args_fs);
            args_fs.fname = feature_file;
            ret = BfvFeatureSetDownload(lp,&args_fs);
            if (args_fs.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fs.res,&bterr);
                fprintf (stderr,"BfvFeatureSetDownload: %s\n",bterr.long_msg);
                exit(1);
            }
        }
        else
        {
            if ((fp = fopen(feature_file,"r")) == NULL)
            {
                fprintf(stderr,"Error opening feature file %s\n",
                        feature_file);
                exit(1);
            }
            while (fgets((char *)buf, sizeof(buf), fp))
            {
                if (strstr((char *)buf,"License Data:"))
                    break;
            }
            data_found = 0;
            bufct = 0;
            while (fscanf(fp,"%x",&utmp) == 1)
            {
                data_found = 1;
                buf[bufct++] = (unsigned char)utmp;
                if (bufct >= sizeof(buf))
                {
                    BT_ZERO(args_fs);
                    args_fs.buf = buf;
                    args_fs.size = bufct;
                    ret = BfvFeatureSetDownloadData(lp,&args_fs);
                    if (args_fs.res.status != BT_STATUS_OK)
                    {
                        BfvErrorMessage(lp,&args_fs.res,&bterr);
                        fprintf (stderr,"BfvFeatureSetDownloadData: %s\n",
                                 bterr.long_msg);
                        exit(1);
                    }
                    bufct = 0;
                }
            }
            if (!data_found)
            {
                fprintf(stderr,"No license data found in feature file.\n");
                exit(1);
            }
            if (bufct)
            {
                BT_ZERO(args_fs);
                args_fs.buf = buf;
                args_fs.size = bufct;
                ret = BfvFeatureSetDownloadData(lp,&args_fs);
                if (args_fs.res.status != BT_STATUS_OK)
                {
                    BfvErrorMessage(lp,&args_fs.res,&bterr);
                    fprintf (stderr,"BfvFeatureSetDownloadData: %s\n",
                             bterr.long_msg);
                    exit(1);
                }
            }
            BT_ZERO(args_fs);
            ret = BfvFeatureSetDownloadData(lp,&args_fs);
            if (args_fs.res.status != BT_STATUS_OK)
            {
                BfvErrorMessage(lp,&args_fs.res,&bterr);
                fprintf (stderr,"BfvFeatureSetDownloadData: %s\n",
                         bterr.long_msg);
                exit(1);
            }
        }
        printf("Done, result value %d.\n",ret);
        if (ret != 0)
            exit_code = 1;
        break;
    }
}
