/* firm.c */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
The firm program is used to download firmware.
***************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#if defined(WINNT) || defined(__OS2__) || defined(__QNX__) || defined(WIN95)
#include <stdlib.h>
#endif
#include <errno.h>
#include <time.h>
#include "btlib.h"

unsigned module = 0x2;
int firm_type = -1;
unsigned firm_dest = 0xffffffff;
int exit_code = 0;
unsigned config_spec = 0;

void dowork(BTLINE *lp,char *name);

void
usage ()
{
    fprintf (stderr, "usage: firm [-m mod] [-c conf_spec] -t type firmware_file\n");
    fprintf (stderr, "  -m mod  Download to specified module (dflt %X).\n",
             module);
    fprintf (stderr, "  -t type Type of firmware downloaded.\n");
    fprintf (stderr, "          0 = DSP_LOADER    1 = DSP_APP\n");
    fprintf (stderr, "          2 = PROC_APP      3 = PROC_FLASH\n");
    fprintf (stderr, "  -c conf_spec Configuration specification value (PROC_APP only).\n");
    exit(1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    char *name;
    BTLINE *lp;
    BTERR bterr;
    struct args_line_admin args;
    struct args_addr_info args_info;
    
    BT_API_SET_VER();
    BfvDebugModeSet(DEBUG_ALL);
    
    while ((c = getopt (argc, argv, "m:t:d:c:")) != EOF)
    {
        switch (c)
        {
          case 'm':
            if (sscanf(optarg,"%x",&module) != 1)
                usage();
            break;
          case 't':
            firm_type = atoi (optarg);
            break;
          case 'd':
            if (sscanf(optarg,"%x",&firm_dest) != 1)
                usage();
            break;
          case 'c':
            config_spec = atoi (optarg);
            break;
          default:
            usage ();
        }
    }
    
    if (optind >= argc)
        usage ();

    name = argv[optind++];
    
    if (optind != argc)
        usage ();

    if (firm_type < 0)
        usage();

    BT_ZERO(args_info);
    args_info.mod_num = module;
    BfvModuleInfo(NULL,&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(NULL,&args_info.res,&bterr);
        fprintf (stderr,"BfvModuleInfo: %s\n",bterr.long_msg);
        exit(1);
    }
    if (!args_info.exists)
    {
        fprintf(stderr,"Specified module (%X) doesn't exist.\n",module);
        exit(1);
    }
    if (!args_info.hardware)
    {
        fprintf(stderr,"Specified module (%X) is not a hardware module.\n",
                module);
        exit(1);
    }
    
    BT_ZERO(args);
    args.dest_addr.mm_bModule = module;
    args.dest_addr.mm_bChannel = 1;
    args.present = 0;
    args.unique = 1;
    if ((lp = BfvSessionAttach(&args)) == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        fprintf (stderr,"BfvSessionAttach: %s\n",bterr.long_msg);
        exit (1);
    }
       
    dowork(lp,name);
    
    BT_ZERO(args);
    BfvSessionDetach(lp,&args);
    
    return(exit_code);
}

void
dowork(BTLINE *lp,char *name)
{
    struct args_line_admin args_admin;
    struct args_download args_download;
    BTERR bterr;
    int ret;
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    args_admin.no_init = 2;
    args_admin.skip_teleph = 1;
    if (BfvLineReset (lp, &args_admin) < 0)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset: %s: status %lX.\n",
                 bterr.long_msg,args_admin.reset_status);
        /* Even if BfvLineReset gets an error, keep going if it's
           a control processor download. Otherwise abort. */
        if (firm_type == 2)
            fprintf (stderr, "Continuing regardless of reset error.\n");
        else
            exit(1);
    }
    
    
    BT_ZERO(args_download);
    args_download.fname = name;
    if (firm_type >= 0)
    {
        args_download.specify_type = 1;
        args_download.dl_type = firm_type;
        args_download.dl_dest = firm_dest;
        if (firm_type == 2)
            args_download.config_spec_value = config_spec;
    }
    ret = BfvFirmwareDownload(lp,&args_download);
    if (args_download.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_download.res,&bterr);
        fprintf (stderr,"BfvFirmwareDownload: %s\n",bterr.long_msg);
    }
    
    printf("Done, result value %d.\n",ret);
    if (ret != 0)
        exit_code = 1;
}
