/* font.c */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
The font program downloads ASCII fonts for fax transmission and
reports on font download status.
***************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#if defined(WINNT) || defined(__OS2__) || defined(__QNX__) || defined(WIN95)
#include <stdlib.h>
#endif
#include <errno.h>
#include <time.h>
#include "btlib.h"

unsigned module = 0x2;
int exit_code = 0;
int qflag, dflag;

void dowork(BTLINE *lp);

void
usage ()
{
    fprintf (stderr, "usage: font [-m mod] [-q] [-d] [-v]\n");
    fprintf (stderr, "  -m mod  Use specified module (dflt %X)\n",
             module);
    fprintf (stderr, "  -q = Report download status of fonts\n");
    fprintf (stderr, "  -d = Download fonts as specified in btcall.cfg\n");
    fprintf (stderr, "  -v = Turn on API debug mode\n");
    exit (1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp;
    BTERR bterr;
    struct args_line_admin args;
    struct args_addr_info args_info;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "m:qdv")) != EOF)
    {
        switch (c)
        {
          case 'm':
            if (sscanf(optarg,"%x",&module) != 1)
                usage();
            break;
          case 'q':
            qflag = 1;
            break;
          case 'd':
            dflag = 1;
            break;
          case 'v':
            BfvDebugModeSet(DEBUG_ALL);
            break;
          default:
            usage ();
        }
    }
    
    if (optind != argc)
        usage ();

    if (qflag + dflag != 1)
        usage();

    BT_ZERO(args_info);
    args_info.mod_num = module;
    BfvModuleInfo(NULL,&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(NULL,&args_info.res,&bterr);
        fprintf (stderr,"BfvModuleInfo: %s\n",bterr.long_msg);
        exit(1);
    }
    if (!args_info.exists)
    {
        fprintf(stderr,"Specified module (%X) doesn't exist.\n",module);
        exit(1);
    }
    if (!args_info.hardware)
    {
        fprintf(stderr,"Specified module (%X) is not a hardware module.\n",
                module);
        exit(1);
    }
    
    BT_ZERO(args);
    args.dest_addr.mm_bModule = module;
    args.dest_addr.mm_bChannel = 1;
    args.present = 1;
    if ((lp = BfvSessionAttach(&args)) == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        fprintf (stderr,"BfvSessionAttach: %s\n",bterr.long_msg);
        exit (1);
    }
    
    BT_ZERO(args_info);
    args_info.facility = MILL_FACILITY_FAX;
    BfvCheckFacility(lp,&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_info.res,&bterr);
        fprintf (stderr,"BfvCheckFacility: %s\n",bterr.long_msg);
        exit (1);
    }
    if (!args_info.exists)
    {
        fprintf(stderr,
                "Fax facility does not exist for channel 1 on module %X\n",
                module);
        exit(1);
    }
    
    dowork(lp);
    
    BT_ZERO(args);
    BfvSessionDetach(lp,&args);
    
    return(exit_code);
}

void
dowork(BTLINE *lp)
{
    struct args_line_admin args_admin;
    BTERR bterr;
    int ret;
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    args_admin.skip_teleph = 1;
    if (dflag)
        args_admin.mill_load_fonts = 1;
    ret = BfvLineReset (lp, &args_admin);
    if (ret < 0)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset: %s: status %lX.\n",
                 bterr.long_msg,args_admin.reset_status);
    }
    
    if (args_admin.reset_status & RST_FONT_UNREADABLE)
        fprintf (stderr, "A font file specified could not be opened/read.\n");
    if (args_admin.reset_status & RST_FONT_DOWNLOADED)
        fprintf (stderr, "A font specified was already downloaded.\n");
    if (args_admin.reset_status & RST_FONT_DLOAD_ERR)
        fprintf (stderr, "An error occurred during font download.\n");

    if (ret < 0)
        exit (1);

    if (qflag)
    {
        int i, any_dl = 0;
        
        printf("Fonts downloaded:  \n");
        for (i = 0; i < MAX_FONTS - 1; i++)
            if (LINE_FONT_DOWNLOADED(lp,i))
            {
                printf("%d ",i);
                any_dl = 1;
            }
        if (LINE_FONT_DOWNLOADED(lp,255))
        {
            printf("%d ",255);
            any_dl = 1;
        }
        if (!any_dl)
            printf("None\n");
        else
            printf("\n");
    }
}
