/* modinfo.c */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
The modinfo program lists information about active hardware and
software modules maintained within the driver.
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "btlib.h"

void show_mod_info(unsigned mod_num,int notify_exist);

int show_config_info;
int show_pci_info;
int show_slot_info;
int show_hw_res_info;
int show_hw_info;
int show_priv_info;

void
usage ()
{
    fprintf (stderr, "usage: modinfo [-p] [-c] [-s] [-h] [-H] [-a] [mod]\n");
    fprintf (stderr, "  -p = list PCI configuration information\n");
    fprintf (stderr, "  -c = list firmware config options\n");
    fprintf (stderr, "  -s = list cPCI slot and CPU information\n");
    fprintf (stderr, "  -h = list hardware resource info\n");
    fprintf (stderr, "  -H = list hardware info reported by FW\n");
    fprintf (stderr, "  -a = all previous options\n");
    exit (1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    unsigned module = 0xff;
    
    BT_API_SET_VER();
#if 0
    BfvDebugModeSet(DEBUG_ALL);
#endif
    
    while ((c = getopt (argc, argv, "pcshHaP")) != EOF)
    {
        switch (c)
        {
          case 'p':
            show_pci_info = 1;
            break;
          case 'c':
            show_config_info = 1;
            break;
          case 's':
            show_slot_info = 1;
            break;
          case 'h':
            show_hw_res_info = 1;
            break;
          case 'H':
            show_hw_info = 1;
            break;
          case 'a':
            show_pci_info = 1;
            show_config_info = 1;
            show_slot_info = 1;
            show_hw_res_info = 1;
            show_hw_info = 1;
            break;
          case 'P':
            show_priv_info = 1;
            break;
          default:
            usage ();
        }
    }
    
    if (optind < argc)
    {
        if (sscanf(argv[optind++],"%x",&module) != 1)
            usage();
        
        if (optind != argc)
            usage ();
    }
    
    if (module != 0xff)
    {
        show_mod_info(module,1);
        exit(0);
    }
    
    for (module = 0; module <= M_ADDR_MAX_COMPONENT_VALUE; module++)
        show_mod_info(module,0);

    return(0);
}

void
show_mod_info(unsigned mod_num,int notify_exist)
{
    BTERR bterr;
    struct args_addr_info args_info;
    unsigned config_opt;
    
    BT_ZERO(args_info);
    args_info.mod_num = mod_num;
    BfvModuleInfo(NULL,&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(NULL,&args_info.res,&bterr);
        fprintf (stderr,"BfvModuleInfo: %s\n",bterr.long_msg);
        exit (1);
    }
    
    if (!args_info.exists)
    {
        if (notify_exist)
            printf("%02X: Module doesn't exist.\n",mod_num);
        return;
    }
    
    printf("%02X: ",mod_num);
    
    if (args_info.hardware)
        printf("hw");
    else
        printf("not hw");

    if (args_info.hardware && args_info.num_channels == 0)
        printf("; *DEAD*");
    else
    {
        printf("; %3u chan",args_info.num_channels);
        if (args_info.num_channels != 1)
            printf("s");

        if (!args_info.hardware)
        {
            printf(".\n");
            return;
        }
        
        printf("; ordinals ");
        if (!args_info.num_ordinals)
            printf("   none");
        else
            printf("%3u-%3u",
                   args_info.ordinal_start,
                   args_info.ordinal_start + args_info.num_ordinals - 1);
    }
    
    printf("; bus type ");
    switch (args_info.bus_type)
    {
      case MILL_BUS_ISA:
        printf("ISA");
        break;
      case MILL_BUS_PCI:
        printf("PCI");
        break;
      case MILL_BUS_VIRT_MOD:
        printf("VIRT");
        break;
      default:
        printf("%d unknown", args_info.bus_type);
        break;
    }
    
    printf("; hw type %04X",args_info.hw_type);
    
    printf("; hw ID %X",args_info.hw_id);
    
    if (args_info.bus_type == MILL_BUS_PCI && show_pci_info)
    {
        printf(";\n    PCI vend %04X; dev %04X; subsys vend %04X; subsys ID %04X; interf type %02X",
               args_info.pci_vend_id,
               args_info.pci_dev_id,
               args_info.pci_sub_vend_id,
               args_info.pci_sub_id,
               args_info.pci_interf_type);
    }
    
    printf(".\n");
    
    /* Warn if the ordinal channel mapping is skewed. */
    if (args_info.hardware && args_info.num_channels > 0 &&
        args_info.num_ordinals > 0 && args_info.num_ordinals_init > 0)
    {
        if (args_info.num_channels - 1 > args_info.num_ordinals_init)
        {
            printf("    ** NOTICE: Number of channels too large for reserved ordinal range.\n");
            printf("    ** %u channels will be inaccessable as ordinal channels.\n",
                   args_info.num_channels - 1 - args_info.num_ordinals_init);
        }
        else if (args_info.num_channels - 1 < args_info.num_ordinals_init)
        {
            printf("    ** NOTICE: Number of channels smaller than reserved ordinal range.\n");
            printf("    ** There may be a gap of %u ordinal channel values not used by channels.\n",
                   args_info.num_ordinals_init - (args_info.num_channels - 1));
        }
    }

    if (args_info.bus_type == MILL_BUS_PCI && show_slot_info)
        printf("    cPCI Slot %d, CPU number %d.\n",
               MOD_BOARD_SLOT(mod_num),MOD_CPU_NUM(mod_num));

    if (args_info.bus_type == MILL_BUS_PCI && show_hw_res_info)
    {
        printf("    Runtime %08X len %X; Mem %08X len %X;\n",
               args_info.pci_runtime_start,
               args_info.pci_runtime_size,
               args_info.pci_mem_start,
               args_info.pci_mem_size);
        printf("    ModCtrl %08X len %X; IO %08X len %X; Irq %d.\n",
               args_info.pci_modctrl_start,
               args_info.pci_modctrl_size,
               args_info.pci_io_start,
               args_info.pci_io_size,
               args_info.pci_irq);
    }
    
    if (args_info.hardware && show_hw_info)
        printf("    Hardware info: %X.\n",args_info.hw_info);
    
    if (show_priv_info)
    {
        int i;
        
        printf("    ");
        for (i = 0; i < sizeof(args_info.private_info); i++)
            printf("%02X ",args_info.private_info[i]);
        printf("\n");
    }
    
    /* This option reuses args_info, so should come last. */
    if (show_config_info)
    {
        for (config_opt = 0; ; config_opt++)
        {
            BT_ZERO(args_info);
            args_info.mod_num = mod_num;
            args_info.config_spec_num = config_opt;
            if (BfvModuleConfigSpecsGet(NULL,&args_info) < 1)
                break;
            printf("    Config spec %d = \"%s\"\n",
                   args_info.config_spec_value,
                   args_info.config_spec_desc);
        }
    }
}
