/* sr140_disabling.c */
/* Copyright (c) 1989-2005 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
Checks whether a channel or module is a virtual module.
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "btlib.h"

void show_mod_info(unsigned mod_num,int unit);

void
usage ()
{
    fprintf (stderr, "usage: sr140_disabling [-u unitnum] [-m mod]\n");
    fprintf (stderr, " one of -u or -m is required\n");
    exit (1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    unsigned module = 0xff;
    int unit = -1;
    BTERR bterr;
    struct args_line_admin args_admin;
    BTLINE *lp;
    
    BT_API_SET_VER();
#if 0
    BfvDebugModeSet(DEBUG_ALL);
#endif
    
    while ((c = getopt (argc, argv, "u:m:")) != EOF)
    {
        switch (c)
        {
          case 'u':
            if (module != 0xff)
                usage();
            unit = atoi (optarg);
            break;
          case 'm':
            if (unit > 0)
                usage();
            if (sscanf(optarg,"%x",&module) != 1)
                usage();
            break;
          default:
            usage ();
        }
    }
    
    if (optind != argc)
        usage ();

    if (unit < 0 && module == 0xff)
        usage ();

    /* If a module is specified, we can use the value directly to
       get module information. */
    if (module != 0xff)
    {
        show_mod_info(module,-1);
        exit(0);
    }

    /* If a channel number is specified, attach to the channel.
       We use BfvSessionAttach so that our check here will not
       have a side effect on any other use of the channel that may
       be present. */
    BT_ZERO(args_admin);
    args_admin.dest_addr.mm_bChannel = unit & 0xff;
    args_admin.dest_addr.mm_bFacility = (unit >> 8) & 0xff;
    args_admin.present = 1;
    if ((lp = BfvSessionAttach(&args_admin)) == NULL)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr,"BfvSessionAttach: %s\n",bterr.long_msg);
        exit (1);
    }

    /* Determine the module number from the line pointer, and get
       module information. */
    show_mod_info(LINE_DEST_ADDR(lp).mm_bModule,unit);

    BT_ZERO(args_admin);
    BfvSessionDetach (lp, &args_admin);

    return(0);
}

void show_mod_info(unsigned mod_num,int unit)
{
    BTERR bterr;
    struct args_addr_info args_info;
    
    /* Call BfvModuleInfo to determine information about the
       module in use. */
    BT_ZERO(args_info);
    args_info.mod_num = mod_num;
    BfvModuleInfo(NULL,&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(NULL,&args_info.res,&bterr);
        fprintf (stderr,"BfvModuleInfo: %s\n",bterr.long_msg);
        exit (1);
    }
    
    if (unit >= 0)
        printf("U%u: ",unit);

    /* Look at the BfvModuleInfo output to determine the module type. */
    if (!args_info.exists)
        printf("M%X: Module doesn't exist.\n",mod_num);
    else if (args_info.hardware)
    {
        if (args_info.bus_type == MILL_BUS_VIRT_MOD)
            printf("M%X: Module is virtual.\n",mod_num);
        else
            printf("M%X: Module is hardware.\n",mod_num);
    }
    else
        printf("M%X: Module is software.\n",mod_num);

    /* At this point, an application could decide to not allow use of
       this channel or module. */
}
