/* telsave.c */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/**************************************************************************
The telsave program writes telephony parameters to a boards Non-Volatile
RAM (NVRAM).
***************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#if defined(WINNT) || defined(__OS2__) || defined(__QNX__) || defined(WIN95)
#include <stdlib.h>
#endif
#include <errno.h>
#include <time.h>
#include "btlib.h"

unsigned module = 0x2;
int exit_code = 0;

void dowork(BTLINE *lp);

void
usage ()
{
    fprintf (stderr, "usage: telsave [-m mod] [-v] -s\n");
    fprintf (stderr, "  -m mod  Use specified module (dflt %X)\n",
             module);
    fprintf (stderr, "  -v      Verbose mode\n");
    fprintf (stderr, "  -s      Save telephony data\n");
    exit (1);
}

int
main(int argc, char **argv)
{
    int c;
    extern int optind;
    extern char *optarg;
    BTLINE *lp;
    BTERR bterr;
    struct args_line_admin args;
    struct args_addr_info args_info;
    int do_it = 0;
    
    BT_API_SET_VER();
    
    while ((c = getopt (argc, argv, "m:sv")) != EOF)
    {
        switch (c)
        {
          case 'm':
            if (sscanf(optarg,"%x",&module) != 1)
                usage();
            break;
          case 's':
            do_it = 1;
            break;
          case 'v':
            BfvDebugModeSet(DEBUG_ALL);
            break;
          default:
            usage ();
        }
    }
    
    if (optind != argc)
        usage ();

    if (!do_it)
        usage ();

    BT_ZERO(args_info);
    args_info.mod_num = module;
    BfvModuleInfo(NULL,&args_info);
    if (args_info.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(NULL,&args_info.res,&bterr);
        fprintf (stderr,"BfvModuleInfo: %s\n",bterr.long_msg);
        exit(1);
    }
    if (!args_info.exists)
    {
        fprintf(stderr,"Specified module (%X) doesn't exist.\n",module);
        exit(1);
    }
    if (!args_info.hardware)
    {
        fprintf(stderr,"Specified module (%X) is not a hardware module.\n",
                module);
        exit(1);
    }
    
    BT_ZERO(args);
    args.dest_addr.mm_bModule = module;
    args.dest_addr.mm_bChannel = 1;
    args.present = 1;
    if ((lp = BfvSessionAttach(&args)) == NULL)
    {
        BfvErrorMessage(lp,&args.res,&bterr);
        fprintf (stderr,"BfvSessionAttach: %s\n",bterr.long_msg);
        exit (1);
    }
       
    dowork(lp);
    
    BT_ZERO(args);
    BfvSessionDetach(lp,&args);
    
    return(exit_code);
}

void
dowork(BTLINE *lp)
{
    struct args_line_admin args_admin;
    struct args_tel_ctrl_call_sw args_tel;
    BTERR bterr;
    int ret;
    
    BT_ZERO(args_admin);
    args_admin.config_file_name = "btcall.cfg";
    ret = BfvLineReset (lp, &args_admin);
    if (ret < 0)
    {
        BfvErrorMessage(lp,&args_admin.res,&bterr);
        fprintf (stderr, "BfvLineReset: %s: status %lX.\n",
                 bterr.long_msg,args_admin.reset_status);
        exit(1);
    }
    
    BT_ZERO(args_tel);
    BfvTelephSave(lp,&args_tel);
    if (args_tel.res.status != BT_STATUS_OK)
    {
        BfvErrorMessage(lp,&args_tel.res,&bterr);
        fprintf (stderr, "BfvTelephSave: %s\n",bterr.long_msg);
        exit(1);
    }
}
