/*
 * api_sys.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */


/* Contains various system-dependent includes or defines which are used
   by applications (and possibly the API source). */

#ifndef _API_SYS_H
#define _API_SYS_H


/* Interface version number of the Bfv API. Whenever any argument structures
   change, this must be incremented. */
#define MILL_API_INTRFC_NUM 24


#if (defined(linux) || defined(_AIX)) && defined(BFAX_THREADS)
/* If used, this header file must be first. */
#include <pthread.h>
#endif

#include <stdio.h>

/* Some platforms may not have this file. May need to conditionally
   include if not available on all ANSI platforms. */
#include <stdlib.h>

/* Support for variable argument lists. May need to conditionally
   include and include a different one. */
#include <stdarg.h>

#if defined(WINNT) || defined(WIN95)
#include <windows.h>
#define _mill_sleep(n) Sleep((n) * 1000)
#elif defined(__OS2__)
#define INCL_BASE 1
#include <os2.h>
#define _mill_sleep(n) DosSleep((n) * 1000L)
#else
#include <unistd.h>
#define _mill_sleep sleep
#define _MILL_USING_REAL_SLEEP
#endif

/* For use with access; some platforms don't define. */
#ifndef R_OK
#define R_OK 4
#define F_OK 0
#endif

/* For use with fseek; some platforms don't define. */
#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

/* Define the macros used for declaring functions and variables which
   depend on the platform. */
#if defined(WINNT) || defined(WIN95)
#if defined(MILL_SOURCE_COMPILE) && defined(M_DLL_COMP)
#define M_APIE1 __declspec(dllexport)
#define M_APIE2
#define M_CB1
#define M_CB2
#define M_APIG1 __declspec(dllexport)
#define M_APIG2
#elif defined(MILL_DLL)
#define M_APIE1 __declspec(dllimport)
#define M_APIE2
#define M_CB1
#define M_CB2
#define M_APIG1 __declspec(dllimport)
#define M_APIG2
#else /* not SOURCE_COMPILE with DLL compilation or DLL app linking */
#define M_APIE1
#define M_APIE2
#define M_CB1
#define M_CB2
#define M_APIG1
#define M_APIG2
#endif
#else /* Not WINNT or WIN95 */
#define M_APIE1
#define M_APIE2
#define M_CB1
#define M_CB2
#define M_APIG1
#define M_APIG2
#endif

#if defined(__OS2__)
#define MILL_LIB_BAD_VARS
#endif

#if defined(WINNT) || defined(__OS2__) || defined(WIN95)
#define MILL_PLAT_HAS_DLLS
#endif

#if defined(__OS2__)
#define MILL_DLL_VARS_USE_PTRS
#endif

#if defined(WINNT) || defined(__OS2__) || defined(WIN95)
#define MILL_API_SETS_PRIO
#endif

#if defined(WINNT) || defined(__OS2__) || defined(WIN95)
#define MILL_NEEDS_GETOPT
#endif

/* Default endian-ness */
#define MILL_LITTLE_ENDIAN

#ifdef linux
/* Drivers use <asm/byteorder.h> to define __LITTLE_ENDIAN or __BIG_ENDIAN, but
   these are both already defined by <endian.h>, included by other files. */
/* Start by checking for __BYTE_ORDER. If it doesn't exist, then
   explicitly include <endian.h>. */
#undef MILL_LITTLE_ENDIAN
#undef MILL_BIG_ENDIAN
#ifdef __BYTE_ORDER
#if __BYTE_ORDER == __LITTLE_ENDIAN
#define MILL_LITTLE_ENDIAN
#endif
#if __BYTE_ORDER == __BIG_ENDIAN
#define MILL_BIG_ENDIAN
#endif
#else
#undef __LITTLE_ENDIAN
#undef __BIG_ENDIAN
#include <endian.h>
#ifdef __LITTLE_ENDIAN
#define MILL_LITTLE_ENDIAN
#endif
#ifdef __BIG_ENDIAN
#define MILL_BIG_ENDIAN
#endif
#endif
#endif


#if (defined(SOLARIS) && defined(sparc)) || defined(_AIX)
#undef MILL_LITTLE_ENDIAN
#define MILL_BIG_ENDIAN
#endif

#ifdef MILL_BIG_ENDIAN
#define BT_BIG_ENDIAN
#endif
#ifdef MILL_LITTLE_ENDIAN
#define BT_LITTLE_ENDIAN
#endif


/* Some systems uses different names for these string functions */
#if defined(SOLARIS) || defined(linux)
#ifndef stricmp
#define stricmp strcasecmp
#endif
#ifndef strnicmp
#define strnicmp strncasecmp
#endif
#endif

/* Some versions of UnixWare have str(n)casecmp, some have nothing */
#if defined(UNIXWARE)
#define stricmp my_strcasecmp
#define strnicmp my_strncasecmp
#endif

/* Portable way to cast pointers to integers */
#ifdef SOLARIS
#define MILL_PTR_INT_TYPE uintptr_t
#else
#define MILL_PTR_INT_TYPE unsigned long
#endif


#endif
