/*
 * btmib.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

#ifndef BT_MIB_H
#define BT_MIB_H

#pragma pack(1)

#define ENET_ADDRESS_SIZE 6
typedef unsigned char ENet_Address[ENET_ADDRESS_SIZE];

#define IPv4_ADDRESS_SIZE 4
typedef unsigned char IPv4_Address[IPv4_ADDRESS_SIZE];


typedef struct
{
#define IF_MIB_RCV_TABLE_SIZE 16
    ENet_Address ifRcvAddressTable[IF_MIB_RCV_TABLE_SIZE];
    ENet_Address ifPhysAddress;
    unsigned ifHighSpeed;
    unsigned ifHCInOctets;
    unsigned ifHCInUcastPkts;
    unsigned ifHCInDiscards;
    unsigned ifHCInErrors;
    unsigned ifInUnknownProtos;
    unsigned ifHCOutOctets;
    unsigned ifInUcastPkts;
    unsigned ifInMulticastPkts;
    unsigned ifInBroadcast;
    unsigned ifOutUcastPkts;
    unsigned ifOutMulticastPkts;
    unsigned ifOutBroadcastPkts;
} IF_MIB;


typedef struct
{
    unsigned dot3StatsExcessiveCollisions;
    unsigned dot3StatsLateCollisions;
    unsigned dot3StatsInternalMacTransmitErrors;
    unsigned dot3StatsCarrierSenseErrors;
    unsigned dot3StatsDeferredTransmissions;
    unsigned dot3StatsSingleCollisionFrames;
    unsigned dot3StatsFCSErrors;
    unsigned dot3StatsAlignmentErrors;
    unsigned dot3StatsInternalMacReceiveErrors;
    unsigned dot3StatsXTransmitGoodFrames;
    unsigned dot3StatsXTransmitFlowControls;
    unsigned dot3StatsXReceiveGoodFrames;
    unsigned dot3StatsXReceiveResourceErrors;
    unsigned dot3StatsXReceiveShortFrameErrors;
    unsigned dot3StatsXReceiveFlowControls;
    unsigned dot3StatsSQETestErrors;
    unsigned dot3StatsFrameTooLongs;
    unsigned dot3StatsEtherChipSet;
} ENet_MIB;


typedef struct
{
    IPv4_Address ipAdEntAddr;
    IPv4_Address ipAdEntNetMask;
    unsigned short ipAdEntBcastAddr;
    unsigned short ipAdEntReasmMaxSize;
} IP_MIB_Addr_Table;


typedef struct
{
    unsigned char ipForwarding;
    unsigned char ipDefaultTTL;
    unsigned ipInReceives;
    unsigned ipInHdrErrors;
    unsigned ipInAddrErrors;
    unsigned ipForwDatagrams;
    unsigned ipUnknownProtos;
    unsigned ipInDiscards;
    unsigned ipInDelivers;
    unsigned ipOutRequests;
    unsigned ipOutDiscards;
    unsigned ipOutNoRoutes;
    unsigned ipReasmTimeout;
    unsigned ipReasmReqds;
    unsigned ipReasmOKs;
    unsigned ipReasmFails;
    unsigned ipFragOKs;
    unsigned ipFragFails;
    unsigned ipFragCreates;
    unsigned ipRoutingDiscards;
#define IP_MIB_ADDR_TABLE_SIZE 16
    IP_MIB_Addr_Table ipAddrTable[IP_MIB_ADDR_TABLE_SIZE];
} IP_MIB;


typedef struct
{
    unsigned icmpInMsgs;
    unsigned icmpInErrors;
    unsigned icmpInDestUnreachs;
    unsigned icmpInTimeExcds;
    unsigned icmpInParmProbs;
    unsigned icmpInSrcQuenchs;
    unsigned icmpInRedirects;
    unsigned icmpInEchos;
    unsigned icmpInEchoReps;
    unsigned icmpInTimestamps;
    unsigned icmpInTimestampReps;
    unsigned icmpInAddrMasks;
    unsigned icmpInAddrMaskReps;
    unsigned icmpOutMsgs;
    unsigned icmpOutErrors;
    unsigned icmpOutDestUnreachs;
    unsigned icmpOutTimeExcds;
    unsigned icmpOutParmProbs;
    unsigned icmpOutSrcQuenchs;
    unsigned icmpOutRedirects;
    unsigned icmpOutEchos;
    unsigned icmpOutEchoReps;
    unsigned icmpOutTimestamps;
    unsigned icmpOutTimestampReps;
    unsigned icmpOutAddrMasks;
    unsigned icmpOutAddrMaskReps;
} ICMP_MIB;


typedef struct
{
    IPv4_Address ipRouteDest;
    int ipRouteMetric1;
    int ipRouteMetric2;
    int ipRouteMetric3;
    int ipRouteMetric4;
    IPv4_Address ipRouteNextHop;
    int ipRouteType;
    int ipRouteProto;
    int ipRouteAge;
    IPv4_Address ipRouteMask;
    int ipRouteMetric5;
} IPRouting_MIB;


typedef struct
{
    ENet_Address atPhysAddress;
    IPv4_Address atNetAddress;
} ARP_MIB_Addr_Table;

typedef struct
{
#define ARP_MIB_ADDR_TABLE_SIZE 60
    ARP_MIB_Addr_Table arpAddrTable[ARP_MIB_ADDR_TABLE_SIZE];
} ARP_MIB;


typedef struct
{
    unsigned udpInDatagrams;
    unsigned udpNoPorts;
    unsigned udpInErrors;
    unsigned udpOutDatagrams;
} UDP_MIB;

#pragma pack()

#endif
