/*
 * btnet.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

#ifndef BT_NET_H
#define BT_NET_H

#include "btmib.h"

#pragma pack(1)

struct bt_in_addr
{
    union
    {
        struct {
            unsigned char s_b1, s_b2, s_b3, s_b4;
        } S_un_b;
        struct {
            unsigned short s_w1, s_w2;
        } S_un_w;
        unsigned int S_addr;
        unsigned long S_junk;     // To maintain union/structure size
    } S_un;
};

struct bt_sockaddr_in
{
    short sin_family;
    unsigned short sin_port;
    struct bt_in_addr sin_addr;
    char sin_zero[8];
};

#define BT_MAC_ADDR_LEN         6
#define BT_IP_ADDR_LEN          4

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_network_config {
    RES res;
    char *configFile;
    unsigned configType;
    unsigned unit;
    unsigned ARPflush;
    unsigned set_trunkType;
    unsigned trunkType;
    unsigned set_trunkMember1;
    unsigned trunkMember1;
    unsigned set_trunkMember2;
    unsigned trunkMember2;
    unsigned set_enableDHCP;
    unsigned enableDHCP;
    unsigned set_localAddr;
    struct bt_sockaddr_in localAddr;
    unsigned set_localNetmask;
    struct bt_sockaddr_in localNetmask;
    unsigned set_gateway1;
    struct bt_sockaddr_in gateway1;
    unsigned set_serviceType;
    unsigned serviceType;
    unsigned serviceTypeHint;
    unsigned set_speed;
    unsigned speed;
    unsigned set_duplex;
    unsigned duplex;
    unsigned set_flowControl;
    unsigned flowControl;
    unsigned set_ARPTimeout;
    unsigned ARPTimeout;
    unsigned char MAC[BT_MAC_ADDR_LEN];
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_network_query {
    RES res;
    unsigned unit;
    unsigned report_mask;
    IF_MIB intrf;
    ENet_MIB ethernet;
    IP_MIB ip;
    ICMP_MIB icmp;
    IPRouting_MIB routing;
    ARP_MIB arp;
    UDP_MIB udp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/


#pragma pack()

#endif


#ifdef MILL_DO_PROTOS

B_APIE1 int B_APIE2 MillNetworkConfigSet(MILL_LINE *lp,
                                         struct args_network_config *args);
B_APIE1 int B_APIE2 MillNetworkConfigGet(MILL_LINE *lp,
                                         struct args_network_config *args);
B_APIE1 int B_APIE2 MillNetworkConfigSave(MILL_LINE *lp,
                                          struct args_network_config *args);
B_APIE1 int B_APIE2 MillNetworkQuery(MILL_LINE *lp,
                                     struct args_network_query *args);

#endif
