/*
 * caps.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

#ifndef _caps_h_
#define _caps_h_

/* Capabilities. These include hardware features of the channel such as
   phone interface, and firmware features of the channel such as whether
   speech recording is supported. */

/* To add a new capability, you may add the name anywhere in the following
   list. The value of the new capability should be set equal to the
   old value of CAP_TERMINATOR, and CAP_TERMINATOR should be increased
   by 1.
*/


/* This value should be one more than the highest value in use. It is
   also the value that the next one to be added should use. */
#define CAP_TERMINATOR 95

enum caps {
    /* Phone interface */
    /* Loop start interface */
    CAP_PHIF_ANALOG_LOOP_START = 0,
    /* DID interface */
    CAP_PHIF_ANALOG_DID = 1,
    /* MVIP interface */
    CAP_PHIF_MVIP = 2,
    /* PEB interface */
    CAP_PHIF_PEB = 3,
    /* MVIP or PEB interface. Centennial compatibility. */
    CAP_PHIF_MVIP_PEB = 4,
    /* ISDN BRI interface */
    CAP_PHIF_ISDN_BRI = 5,
    /* SCSA interface */
    CAP_PHIF_SCSA = 6,
    /* H100 interface */
    CAP_PHIF_H100 = 7,
    /* T1 interface */
    CAP_PHIF_T1 = 8,
    /* E1 interface */
    CAP_PHIF_E1 = 9,
    /* PRI-E1 interface */
    CAP_PHIF_PRI_E1 = 10,
    /* PRI-T1 interface */
    CAP_PHIF_PRI_T1 = 11,
    /* AT-MODEM interface */
    CAP_PHIF_AT_MODEM = 12,
    /* Can configure telephony attributes dynamically */
    CAP_DYN_TELEPH_CONFIG = 13,

    /* Fax */
    /* Fax transmit/receive */
    CAP_FAX_XMIT = 14,
    /* ECM capability */
    CAP_FAX_ECM = 15,
    /* Can handle B4 and A3 widths, in addition to A4 */
    CAP_FAX_WIDTH = 16,
    /* Can do "fax pad", per X.38 specs */
    CAP_FAX_PAD = 17,
    /* Can transmit at 14400 and 12000 bps */
    CAP_FAX_SPEED_144 = 18,
    /* Can transmit/receive on the phone line in MMR format */
    CAP_FAX_MMR_XMIT = 19,
    /* Can transmit/receive files stored in MMR format */
    CAP_FAX_MMR_FMT = 20,
    /* Can transmit/receive on the phone line in MR format and
       transmit/receive files stored in MR format. Centennial compatibility. */
    CAP_FAX_MR = 21,
    /* Can transmit/receive on the phone line in MR format */
    CAP_FAX_MR_XMIT = 22,
    /* Can transmit/receive files stored in MR format */
    CAP_FAX_MR_FMT = 23,
    /* BFT transmit/receive */
    CAP_BFT = 24,
    /* Can transmit/receive files stored in PCX format */
    CAP_FAX_PCX_FMT = 25,
    /* Native ASCII conversion */
    CAP_FAX_ASCII_NATIVE = 26,
    /* Can transmit/receive the SUB, PWD, and SEP FSK signals */
    CAP_SUBPWDSEP = 27,
    /* Supports fax page overlays */
    CAP_FAX_OVERLAY = 28,
    /* Can force receive storage resolution */
    CAP_FAX_FORCE_RES = 29,
    /* Handles high resolution faxes, 200x400, 300x300, 400x400 */
    CAP_FAX_2x4_3x3_4x4 = 30,
    /* Handles other resolutions: 100x100, 300x600, 400x800,
       600x600, 600x1200, 1200x1200 */
    CAP_FAX_OTHER_RES_COMBOS = 31,
    /* Can add headers in overlay mode */
    CAP_FAX_HEADER_OVERLAY = 32,
    /* G3 pass-through mode */
    CAP_FAX_G3_PT = 33,
    /* JPEG pages (in pass-through style) */
    CAP_FAX_JPEG_PAGES = 34,
    /* JBIG pages (in pass-through style) */
    CAP_FAX_JBIG_PAGES = 35,
    /* JPEG format natively, for strips and storage formats */
    CAP_FAX_JPEG_NATIVE = 36,
    /* JBIG format natively, for strips and storage formats */
    CAP_FAX_JBIG_NATIVE = 37,
    /* T.43 Lossless color */
    CAP_FAX_LOSSLESS_COLOR = 38,
    /* Independent local time for each channel */
    CAP_FAX_LOCAL_TIME = 39,
    /* T.30 holdup */
    CAP_FAX_T30_HOLDUP = 40,
    /* V.34 */
    CAP_FAX_V34 = 41,
    /* T.38 */
    CAP_FAX_T38 = 42,

    /* Speech */
    CAP_SPEECH_PLAY = 43,
    /* Speech recording */
    CAP_SPEECH_RECORD = 44,
    /* Simultaneous speech playback and recording */
    CAP_SPEECH_SIMUL_PLAY_RECORD = 45,
    /* Support conference */
    CAP_SPEECH_CONFERENCE = 84,
    /* Speech recording summation */
    CAP_SPEECH_SUMMATION = 94,
    /* Can play/record in Linear format */
    CAP_SPEECH_LINEAR = 46,
    /* Can play/record in CVSD format */
    CAP_SPEECH_CVSD = 47,
    /* Can play/record in ADPCM format */
    CAP_SPEECH_ADPCM = 48,
    /* Can play/record in u-law PCM format */
    CAP_SPEECH_UPCM = 49,
    /* Can play/record in OKI ADPCM format, obsolete */
    CAP_SPEECH_OKI_ADPCM = 50,
    /* Can play/record in A-law PCM format */
    CAP_SPEECH_APCM = 51,
    /* Can play/record in Subband format */
    CAP_SPEECH_SUBBAND = 52,
    /* Can play/record in G723-1 format */
    CAP_SPEECH_G723_1 = 53,
    /* Can play/record in G729-A format */
    CAP_SPEECH_G729_A = 54,
    /* Can play/record in SX7300 format */
    CAP_SPEECH_SX7300 = 55,
    /* Can play/record in SX9600 format */
    CAP_SPEECH_SX9600 = 56,
    /* Can play/record in Rhetorex ADPCM format */
    CAP_SPEECH_RHET_ADPCM = 57,
    /* Can play/record in MS GSM format */
    CAP_SPEECH_MS_GSM = 58,
    /* Can play/record in DVI ADPCM format */
    CAP_SPEECH_DVI_ADPCM = 59,
    /* Can play/record in GSM 6.10 format */
    CAP_SPEECH_GSM_610 = 60,
    /* Can play/record in GSM 6.60 format */
    CAP_SPEECH_GSM_660 = 61,
    /* Can play/record using raw data format */
    CAP_SPEECH_RAWDATA = 62,
    /* Can play/record using G726 format */
    CAP_SPEECH_G726 = 63,
    /* Can play using CNID_FSK format */
    CAP_CNID_FSK = 64,

    /* Signal detection */
    /* Can detect DTMF tones */
    CAP_SIG_DET_DTMF = 85,
    /* Can detect MF tones */
    CAP_SIG_DET_MF = 65,
    /* Supports PCPM parameters */
    CAP_SIG_DET_PCPM = 66,
    /* Supports call progress */
    CAP_SIG_DET_CALL_PROGRESS = 86,

    /* Signal generation */
    /* Can generate DTMF tones */
    CAP_SIG_GEN_DTMF = 93,
    /* Can generate MF tones */
    CAP_SIG_GEN_MF = 67,
    /* Can generate call progress signals */
    CAP_SIG_GEN_CP = 68,
    /* Can generate advanced call progress signals */
    CAP_SIG_GEN_CP_ADV = 69,

    /* Call switching */
    CAP_CALL_SWITCHING = 87,

    /* Network */
    CAP_NETWORK_CONFIG = 70,
    CAP_NETWORK_QUERY = 71,

    /* Packetization */
    CAP_PKT_QOS = 72,

    /* Capabilities used internally by the API library */
    CAP_FIRM_FLASH = 73,
    CAP_FIRM_DYN_CTRL_PROC_APP = 74,
    CAP_FIRM_DYN_DSP_LDR = 75,
    CAP_FIRM_DYN_DSP_APP = 76,
    CAP_FIRM_VIRT_MOD = 88,
    CAP_CRIT_SEC = 77,
    CAP_CRIT_SEC_RESP = 78,
    CAP_SPEECH_FORMAT_INDEP = 79,
    CAP_NVRAM_INITIALIZE = 80,
    CAP_FTRL_DOWNLOAD = 81,
    CAP_FTRL_QUERY = 82,
    CAP_TELE_CTRL_PARAMS_SAVE = 83,
    CAP_TELE_CTRL_PARAMS_RESET = 89,
    CAP_TELE_CTRL_PARAMS_GETINFO = 90,
    CAP_MODULE_LEVEL_ACCESS = 91,
    CAP_BSMI_LEVEL_ACCESS = 92
};


#define LINE_HAS_CAP(lp,cap) MillLineCapCheck(lp,cap)

/* The number of capabilities value currently used. The CAP_TERMINATOR value
   isn't actually used, so its value is the number of actual capabilities */
#define CAP_SIZE        CAP_TERMINATOR

/* Number of bytes needed for capabilities info */
#define NUM_CAP_BYTES ((CAP_SIZE + 7) / 8)

#endif
