/***********************************************************************
 * FILENAME:         component_version.h
 *
 * CURRENT VERSION:  %version: 5 %
 * DATE MODIFIED:    %date_modified: Tue May 04 15:17:43 2004 %
 *
 * CREATION DATE:    Wed Apr 21 18:49:38 2004
 *
 * PURPOSE:
 *   This file contains the version information for one SDK component.
 *
 * USAGE NOTE:
 *   Before including this file, you must define:
 *      #define COMPONENT_NAME  <component_name>   (no double quotes)
 *      #define <component_name>_MAJOR_VERSION  <number>
 *      #define <component_name>_MIDDLE_VERSION <number>
 *      #define <component_name>_MINOR_VERSION  <number>
 *
 *   Example:
 *      #define COMPONENT_NAME  BSMI
 *      #define BSMI_MAJOR_VERSION  2
 *      #define BSMI_MIDDLE_VERSION 0
 *      #define BSMI_MINOR_VERSION  5
 *
 * Copyright (c) 2004 Brooktrout Technology, Inc. All Rights Reserved.
 *
 * This software is the property of Brooktrout Technology, Inc.
 * Any reproduction, disclosure, or distribution of this software is
 * expressly prohibited unless first agreed to in writing by Brooktrout
 * Technology, Inc.
 ***********************************************************************/

#ifndef _component_version_h
#define _component_version_h


#include "sdk_version.h"


/* Create <component_name>_MAJOR_VERSION, <component_name>_MIDDLE_VERSION,
          <component_name>_MINOR_VERSION */

#define C_MAJ_VER2(b)   b##_MAJOR_VERSION
#define C_MAJ_VER1(a)   C_MAJ_VER2(a)
#define C_MAJ_VER       C_MAJ_VER1(COMPONENT_NAME)

#define C_MID_VER2(b)   b##_MIDDLE_VERSION
#define C_MID_VER1(a)   C_MID_VER2(a)
#define C_MID_VER       C_MID_VER1(COMPONENT_NAME)

#define C_MIN_VER2(b)   b##_MINOR_VERSION
#define C_MIN_VER1(a)   C_MIN_VER2(a)
#define C_MIN_VER       C_MIN_VER1(COMPONENT_NAME)

/* Create string for COMPONENT_NAME */
#define VER_STRINGIZE2(b)       #b
#define VER_STRINGIZE1(a)       VER_STRINGIZE2(a)
#define COMPONENT_NAME_STR      VER_STRINGIZE1(COMPONENT_NAME)


/*  Create version string in the format "1.2.3B4"
    (major.middle.minorBbuild) */
#define C_VERSION2(a,b,c,d)     #a "." #b "." #c "B" #d
#define C_VERSION1(a,b,c,d)     C_VERSION2(a,b,c,d)
#define COMPONENT_VERSION       C_VERSION1(C_MAJ_VER, C_MID_VER, C_MIN_VER, SDK_BUILD_NUMBER)

/*  Create version string in the format "1.2.3B4.5"
    (major.middle.minorBbuild.autonumber) */
#define C_VERSION_AUTO2(a,b,c,d,e)      #a "." #b "." #c "B" #d "." #e
#define C_VERSION_AUTO1(a,b,c,d,e)      C_VERSION_AUTO2(a,b,c,d,e)
#define COMPONENT_VERSION_AUTO          C_VERSION_AUTO1(C_MAJ_VER, C_MID_VER, C_MIN_VER, SDK_BUILD_NUMBER, SDK_AUTO_NUMBER)


/*  Version string for the component: */
static char sdk_component_version_str[] = "$(@)SDKV:" SDK_VERSION_AUTO "@-@COM:" COMPONENT_NAME_STR "@-@CV:" COMPONENT_VERSION_AUTO "@-@DT:" __DATE__ "@-@TM:" __TIME__ "@-@BRKT(@)$";


#endif          /* include once */
