/*
 * dcs.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

#ifndef _dcs_h_
#define _dcs_h_

/* Pack structures used by both API and API applications, in case applications
   compiled with non-standard packing on cmd line. */
#pragma pack(1)

/************************************************************************/
/*                                                                      */
/* The following values are located in the line structure for each      */
/* channel.  The general purpose of the values is to report the         */
/* capabilities of the remote machine as well as store the current line */
/* configuration.  This allows a user level application to quickly      */
/* access the current line values as well as the line capabilities      */
/* allowing for easy monitoring of the line status.                     */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Structure : dis_dtc_info                                             */
/* Overview  : This structure contains information about the receiving  */
/* fax machine.  If the called machine is the receiver, then the        */
/* structure contains the Digital Identification Signal (DIS).  If the  */
/* the calling machine is the receiver (polling) then the structure     */
/* contains the Digital Transmit Command (DTC) information.  In both    */
/* cases the structure format is the same.                              */
/* Information Field Values :                                           */
/*      Transmitter_t4  : Non-zero for transmitter-T.4 operation        */
/*      Receiver_t4     : Non-zero for receiver-T.4 operation           */
/*      Data_Signalling_Rate (Bit Rate Capabilities) :                  */
/*              0 : 2400 bit/s only     (V.27 ter fallback mode)        */
/*              1 : 4800 or 2400 bit/s  (V.27 ter)                      */
/*              2 : 9600 or 7200 bit/s  (V.29)                          */
/*              3 : 9600, 7200, 4800 or 2400    (V.27 ter and V.29)     */
/*              4 : 14400, 12000, 9600, 7200, 4800 or 2400              */
/*                  (V.27 ter, V.29, and V.33)                          */
/*              5 : 14400, 12000, 9600, 7200, 4800 or 2400              */
/*                  (V.27 ter, V.29, V.33, and V.17)                    */
/*              6 : 33600, 31200,..., or 2400                           */
/*                  (V.34)                                              */
/*      Resolution (capability):                                        */
/*              0 : 200H x 100V (Normal Resolution)                     */
/*                  (98 dpi=3.85 ln/mm vert, 204 dpi=8 pel/mm hor)      */
/*              1 : 200H x 200V (Fine Resolution)                       */
/*                  (196 dpi=7.7 ln/mm vert, 204 dpi=8 pel/mm hor)      */
/*                  or 200x200 dpi, depending on Resolution_Unit        */
/*      Resolution_200H_400V (capability):                              */
/*              1 : 200 x 400 Resolution                                */
/*                  (392 dpi=15.4 ln/mm vert, 204 dpi=8 pel/mm hor)     */
/*      Resolution_300H_300V (capability):                              */
/*              1 : 300x300 dpi                                         */
/*      Resolution_400H_400H (capability):                              */
/*              1 : 392 dpi=15.4 ln/mm vert, 408 dpi=16 pel/mm hor      */
/*                  or 400x400 dpi, depending on Resolution_Unit        */
/*      Resolution_600H_600V (capability):                              */
/*              1 : 600x600 dpi                                         */
/*      Resolution_1200H_1200V (capability):                            */
/*              1 : 1200x1200 dpi                                       */
/*      Resolution_300H_600V (capability):                              */
/*              1 : 300x600 dpi                                         */
/*      Resolution_400H_800V (capability):                              */
/*              1 : 400x800 dpi                                         */
/*      Resolution_600H_1200V (capability):                             */
/*              1 : 600x1200 dpi                                        */
/*      ColorRes_100x100 (capability):                                  */
/*              1 : 100x100 dpi for color/grayscale                     */
/*      ColorRes_300x300_400x400 (capability):                          */
/*              1 : 300x300 or 400x400 dpi for color/grayscale          */
/*      Resolution_Unit preferred:                                      */
/*              0 : Metric-based                                        */
/*              1 : Inch-based                                          */
/*              2 : Both                                                */
/*      TwoD_Coding :                                                   */
/*              0 : 1-D Modified Huffman coding                         */
/*              1 : 2-D Modified Read coding                            */
/*      MMR (capability):                                               */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Recording_Width (Capabilities) :                                */
/*              0 : 1728 pixels in 215 mm (A4)                          */
/*              1 : 1728 pixels in 215 mm and                           */
/*                  2048 pixels in 255 mm (B4)                          */
/*              2 : 1728 pixels in 215 mm and                           */
/*                  2048 pixels in 255 mm and                           */
/*                  2432 pixels in 303 mm (A3)                          */
/*              3 : 1216 pixels in 151 mm                               */
/*              4 :  864 pixels in 107 mm                               */
/*              5 : 1728 pixels in 151 mm                               */
/*              6 : 1728 pixels in 107 mm                               */
/*      Max_Recording_Length (Capabilities) :                           */
/*              0 : A4 (297 mm)                                         */
/*              1 : Unlimited                                           */
/*              2 : A4 (297 mm) and B4 (364 mm)                         */
/*              3 : Invalid                                             */
/*      Min_Scan_Line_Time (Capabilities) :                             */
/*              0 : 0 ms at 3.85 I/mm; T(7.7) = T(3.85)                 */
/*              1 : 5 ms at 3.85 I/mm; T(7.7) = T(3.85)                 */
/*              2 : 10 ms at 3.85 I/mm; T(7.7) = 1/2 T(3.85)            */
/*              3 : 10 ms at 3.85 I/mm; T(7.7) = T(3.85)                */
/*              4 : 20 ms at 3.85 I/mm; T(7.7) = 1/2 T(3.85)            */
/*              5 : 20 ms at 3.85 I/mm; T(7.7) = T(3.85)                */
/*              6 : 40 ms at 3.85 I/mm; T(7.7) = 1/2 T(3.85)            */
/*              7 : 40 ms at 3.85 I/mm; T(7.7) = T(3.85)                */
/*      Min_Scan_Higher (Capabilities) :                                */
/*              0 : T(15.4) = T(7.7)                                    */
/*              1 : T(15.4) = 1/2 T(7.7)                                */
/*      Handshake_2400_bit : Set when non-zero; cleared when zero.      */
/*      Uncompressed_Mode  : Set when non-zero; cleared when zero.      */
/*      Error_Correction_Mode (capability):                             */
/*              0 : disable                                             */
/*              1 : enable, with 256 byte frames                        */
/*              2 : enable, with 64 byte frames                         */
/*      Error_Limiting_Mode (capability):                               */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Selective_Polling (capability):                                 */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Subaddressing_Capability:                                       */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Password_Capability:                                            */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Data_File_Capable :                                             */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Binary_File_Transfer :                                          */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Document_Transfer_Mode :                                        */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Edifact_Transfer :                                              */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Basic_Transfer_Mode :                                           */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Character_File_Capable :                                        */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Character_Mode :                                                */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Mixed_Mode :                                                    */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Processable_Mode_26 :                                           */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      JPEG_Coding: Support for JPEG coding (Color Fax - Bit 68)       */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      JPEG_FullColor: CIELAB full color space (Bit 69)                */
/*              0 : called terminal has grayscale or b&w grayscale only */
/*              1 : called terminal has full color capability           */
/*      JPEG_12Bit: 12 Bit color per pel/component (Bit 71)             */
/*              0 : can only accept 8 Bits per pel/component            */
/*              1 : can also accept 12 Bits per per/component           */
/*      JPEG_NoSubSampling, subsampling (73)                            */
/*              0 : called terminal expects 4:1:1 subsampling ratio     */
/*              1 : accepts as an option no subsampling in the          */
/*                  chrominance components (1:1:1)                      */
/*      JPEG_CustumIlluminant:  (Bit 74)                                */
/*              0 : uses CIE standard illuminant D50                    */
/*              1 : can also accept custom illuminants, the definition  */
/*                  of which is embedded in JPEG syntax                 */
/*      JPEG_CustomGamut: (Bit 75)                                      */
/*              0 : uses default gamut range                            */
/*              1 : called terminal can also accept custom gamut ranges */
/*                  the definition of which is embedded into the JPEG   */
/*                  syntax                                              */
/*                                                                      */
/************************************************************************/
                
struct dis_dtc_info {
    unsigned char sent; /* Non-0 if sent, 0 if received */
    unsigned char Transmitter_t4;
    unsigned char Receiver_t4;
    unsigned char Data_Signalling_Rate;
    unsigned char Resolution;
    unsigned char TwoD_Coding;
    unsigned char Recording_Width;
    unsigned char Max_Recording_Length;
    unsigned char Min_Scan_Line_Time;
    
    /* The following fields are only set if the extend bit is on. */
    unsigned char Handshake_2400_bit;
    unsigned char Uncompressed_Mode;
    unsigned char Error_Correction_Mode;
    unsigned char Error_Limiting_Mode;
    unsigned char MMR;
    unsigned char Resolution_200H_400V;
    unsigned char Resolution_300H_300V;
    unsigned char Resolution_400H_400V;
    unsigned char Resolution_Unit;
    unsigned char Min_Scan_Higher;
    unsigned char Selective_Polling;
    unsigned char Subaddressing_Capability;
    unsigned char Password_Capability;
    unsigned char Data_File_Capable;
    unsigned char Binary_File_Transfer;
    unsigned char Document_Transfer_Mode;
    unsigned char Edifact_Transfer;
    unsigned char Basic_Transfer_Mode;
    unsigned char Character_File_Capable;
    unsigned char Character_Mode;
    unsigned char Mixed_Mode;
    unsigned char Processable_Mode_26;
    unsigned char Store_And_Fwd_Internet_Fax;   /* 1 T.37 simple mode */
                                                /* 2 reserved */
    unsigned char Real_Time_Internet_Fax;       /* 3 T.38 */
                                                /* 4 reserved */
                                                /* 5 reserved */
    unsigned char V8_Capability;                /* 6 */
    unsigned char Num_Preferred_Octets;         /* 7 0==256, 1==64 */
                                                /* 8 reserved */
    
                                                /* 33 invalid */
    unsigned char Mult_Selective_Polling;       /* 34 */
    unsigned char Polled_SubAddress;            /* 35 */
    /* 36 T.43 lossless color/grayscale */
    unsigned char Lossless_Mode;
    /* 37 T.43 plane or strip interleaving */
    unsigned char Plane_Interleave_Mode;
    unsigned char ADPCM;                        /* 38 G.726 */
                                                /* 39 - reserved */
    
    unsigned char Digital_Network;              /* 66 */
    unsigned char FullDuplex;                   /* 67 */
    unsigned char JPEG_Coding;                  /* 68 */
    unsigned char JPEG_FullColor;               /* 69 */
                                                /* 70 always 0 in DIS */
    unsigned char JPEG_12Bit;                   /* 71 */
                                                /* 72 extend */
    unsigned char JPEG_NoSubSampling;           /* 73 */
    unsigned char JPEG_CustomIlluminant;        /* 74 */
    unsigned char JPEG_CustomGamut;             /* 75 */
    unsigned char NA_Letter;                    /* 76 */
    unsigned char NA_Legal;                     /* 77 */
    unsigned char JBIG_Coding;                  /* 78 T.85 basic */
    unsigned char JBIG_L0;                      /* 79 T.85 optional L0 */
                                                /* 80 extend */
    unsigned char HKM_Key_Mgt;                  /* 81 */
    unsigned char RSA_Key_Mgt;                  /* 82 */
    unsigned char Override_Mode;                /* 83 */
    unsigned char HFX40_Cipher;                 /* 84 */
                                                /* 85 reserved */
                                                /* 86 reserved */
    unsigned char HFX40_Hashing;                /* 87 */
                                                /* 88 extend */
                                                /* 89 reserved */
                                                /* 90 reserved */
                                                /* 91 reserved */
    /* (92-94) T.44 Mixed Raster Content Mode */
    unsigned char MRC_Mode;
    /* 95 T.44 Page length max strip size */
    unsigned char MRC_Page_Length_Strips;
                                                /* 96 extend */
    unsigned char ColorRes_300x300_400x400;     /* 97 */
    unsigned char ColorRes_100x100;             /* 98 */
    unsigned char PhaseC_BFT_Negotiations;      /* 99 */
                                                /* 100 reserved */
    /* 101 Internet Selective Polling address */
    unsigned char ISP;
    /* 102 Internet Routing Address */
    unsigned char IRA;                          
                                                /* 103 reserved */
                                                /* 104 extend */
    unsigned char Resolution_600H_600V;         /* 105 */
    unsigned char Resolution_1200H_1200V;       /* 106 */
    unsigned char Resolution_300H_600V;         /* 107 b&w only */
    unsigned char Resolution_400H_800V;         /* 108 b&w only */
    unsigned char Resolution_600H_1200V;        /* 109 b&w only */
                                                /* 110 reserved */
                                                /* 111 reserved */
                                                /* 112 extend */
};


/************************************************************************/
/*                                                                      */
/* Structure : dcs_info                                                 */
/* Overview  : This structure contains information about the sending    */
/* fax machine (transmitter).  The Digital Command Signal (DCS) is the  */
/* digital command response to the standard capabilities identified by  */
/* a DIS or DTC signal.  The DCS reflects what one may consider the     */
/* the current line configuration, and may vary during the duration     */
/* of the call (specifically when phase B phasing/training fails).      */ 
/* Information Field Values :                                           */
/*      Receiver_t4     : Non-zero for receiver-T.4 operation           */
/*      Data_Signalling_Rate :                                          */
/*              0 : 2400 bit/s  (V.27)                                  */
/*              1 : 4800 bit/s  (V.27)                                  */
/*              2 : 7200 bit/s  (V.29)                                  */
/*              3 : 9600 bit/s  (V.29)                                  */
/*              4 : 7200 bit/s  (V.17)                                  */
/*              5 : 9600 bit/s  (V.17)                                  */
/*              6 :12000 bit/s  (V.17)                                  */
/*              7 :14400 bit/s  (V.17)                                  */
/*              8 :12000 bit/s  (V.33)                                  */
/*              9 :14400 bit/s  (V.33)                                  */
/*              10 : 2400 bit/s (V.34)                                  */
/*              11 : 4800 bit/s (V.34)                                  */
/*              12 : 7200 bit/s (V.34)                                  */
/*              13 : 9600 bit/s (V.34)                                  */
/*              14 :12000 bit/s (V.34)                                  */
/*              15 :14400 bit/s (V.34)                                  */
/*              16 :16800 bit/s (V.34)                                  */
/*              17 :19200 bit/s (V.34)                                  */
/*              18 :21600 bit/s (V.34)                                  */
/*              19 :24000 bit/s (V.34)                                  */
/*              20 :26400 bit/s (V.34)                                  */
/*              21 :28800 bit/s (V.34)                                  */
/*              22 :31200 bit/s (V.34)                                  */
/*              23 :33600 bit/s (V.34)                                  */
/*      Resolution :                                                    */
/*              0 : 200H x 100V (Normal) Resolution                     */
/*                  (98 dpi=3.85 ln/mm vert, 204 dpi=8 pel/mm hor)      */
/*              1 : 200H x 200V (Fine) Resolution                       */
/*                  (196 dpi=7.7 ln/mm vert, 204 dpi=8 pel/mm hor)      */
/*                  or 200x200 dpi, depending on Resolution_Unit        */
/*              2 : 200H x 400V Resolution                              */
/*                  (392 dpi=15.4 ln/mm vert, 204 dpi=8 pel/mm hor)     */
/*              3 : 300H x 300V Resolution                              */
/*              4 : 400H x 400V Resolution                              */
/*                  392 dpi=15.4 ln/mm vert, 408 dpi=16 pel/mm hor      */
/*                  or 400x400 dpi, depending on Resolution_Unit        */
/*              5 : 600H x 600V Resolution                              */
/*              6 : 1200H x 1200V Resolution                            */
/*              7 : 300H x 600V Resolution                              */
/*              8 : 400H x 800V Resolution                              */
/*              9 : 600H x 1200V Resolution                             */
/*              10 : 100H x 100V Resolution                             */
/*      Resolution_Unit :                                               */
/*              0 : Metric-based                                        */
/*              1 : Inch-based                                          */
/*      TwoD_Coding :                                                   */
/*              0 : 1-D Modified Huffman coding                         */
/*              1 : 2-D Modified Read coding                            */
/*      MMR :                                                           */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Recording_Width :                                               */
/*              0 : 1728 pixels in 215 mm (A4)                          */
/*              1 : 2048 pixels in 255 mm (B4)                          */
/*              2 : 2432 pixels in 303 mm (A3)                          */
/*              3 : 1216 pixels in 151 mm                               */
/*              4 :  864 pixels in 107 mm                               */
/*      Max_Recording_Length :                                          */
/*              0 : A4 (297 mm)                                         */
/*              1 : Unlimited                                           */
/*              2 : B4 (364 mm)                                         */
/*              3 : Invalid                                             */
/*      Min_Scan_Line_Time :                                            */
/*              0 :  0 ms                                               */ 
/*              1 :  5 ms                                               */
/*              3 : 10 ms                                               */
/*              5 : 20 ms                                               */
/*              7 : 40 ms                                               */
/*      Note : Min_Scan_Line_Time is based on a resolution              */
/*      setting of T(3.85) == T(7.7).                                   */
/*                                                                      */
/*      Handshake_2400_bit : Set when non-zero; cleared when zero.      */
/*      Uncompressed_Mode  : Set when non-zero; cleared when zero.      */
/*      Error_Correction_Mode :                                         */
/*              0 : disable                                             */
/*              1 : enable, with 256 byte frames                        */
/*              2 : enable, with 64 byte frames                         */
/*      Error_Limiting_Mode :                                           */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Binary_File_Transfer :                                          */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Document_Transfer_Mode :                                        */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Edifact_Transfer :                                              */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Basic_Transfer_Mode :                                           */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Character_Mode :                                                */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Mixed_Mode :                                                    */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      Processable_Mode_26 :                                           */
/*              0 : disable                                             */
/*              1 : enable                                              */
/*      JPEG_Coding: (Bit 68)                                           */
/*              0 : disable - image will not be coded using JPEG        */
/*              1 : enable - image will be coded using JPEG             */
/*      JPEG_FullColor: (Bit 69)                                        */
/*              0 : grayscale - only CIELAB L* (lightness) will be sent */
/*              1 : full color image in CIELAB representation           */
/*      JPEG_Default_Tables: (Bit 70)                                   */
/*              0 : Not default huffman tables                          */
/*              1 : Default Huffman tables (iff 8 bits per pel)         */
/*      JPEG_12Bit: (Bit 71)                                            */
/*              0 : data imaged at 8 Bits per pel/component             */
/*              1 : data imaged at 12 bits per pel/component            */
/*      JPEG_NoSubSampling: (73)                                        */
/*              0 : 4:1:1 subsampling ratio used in image               */
/*              1 : 1:1:1 no subsampling                                */
/*      JPEG_CustomIlluminant:  (Bit 74)                                */
/*              0 : uses CIE standard illuminant D50                    */
/*              1 : a custom illuminant is used, the specification      */
/*                  of which is embedded in JPEG syntax                 */
/*      JPEG_CustomGamut: (Bit 75)                                      */
/*              0 : uses default gamut range                            */
/*              1 : a custom gamut is used, the specification of which  */
/*                  is embedded into the JPEG syntax                    */
/*                                                                      */
/************************************************************************/

struct dcs_info {
    unsigned char sent; /* Non-0 if sent, 0 if received */
    unsigned char Receiver_t4;
    unsigned char Data_Signalling_Rate;
    unsigned char Resolution;
    unsigned char TwoD_Coding;
    unsigned char Recording_Width;
    unsigned char Max_Recording_Length;
    unsigned char Min_Scan_Line_Time;
    
    /* The following fields are only set if the extend bit is on. */
    unsigned char Handshake_2400_bit;
    unsigned char Uncompressed_Mode;
    unsigned char Error_Correction_Mode;
    unsigned char Error_Limiting_Mode;
    unsigned char MMR;
    unsigned char Resolution_Unit;
    unsigned char Binary_File_Transfer;
    unsigned char Document_Transfer_Mode;
    unsigned char Edifact_Transfer;
    unsigned char Basic_Transfer_Mode;
    unsigned char Character_Mode;
    unsigned char Mixed_Mode;
    unsigned char Processable_Mode_26;
    unsigned char Store_And_Fwd_Internet_Fax;   /* 1 T.37 simple mode */
                                                /* 2 reserved */
    unsigned char Real_Time_Internet_Fax;       /* 3 T.38 */
                                                /* 4 reserved */
                                                /* 5 reserved */
                                                /* 6 invalid */
                                                /* 7 invalid */
                                                /* 8 reserved */

                                                /* 33 invalid */
                                                /* 34 set to "0" */
                                                /* 35 set to "0" */
    /* 36 T.43 lossless color/grayscale */
    unsigned char Lossless_Mode;
    /* 37 T.43 plane or strip interleaving */ 
    unsigned char Plane_Interleave_Mode;
    unsigned char ADPCM;                        /* 38 G.726 */
                                                /* 39 reserved */
    
    unsigned char Digital_Network;              /* 66 */
    unsigned char FullDuplex;                   /* 67 */
    unsigned char JPEG_Coding;                  /* 68 */
    unsigned char JPEG_FullColor;               /* 69 */
    unsigned char JPEG_Default_Tables;          /* 70 */
    unsigned char JPEG_12Bit;                   /* 71 */
                                                /* 72 extend */
    unsigned char JPEG_NoSubSampling;           /* 73 */
    unsigned char JPEG_CustomIlluminant;        /* 74 */
    unsigned char JPEG_CustomGamut;             /* 75 */
    unsigned char NA_Letter;                    /* 76 */
    unsigned char NA_Legal;                     /* 77 */
    unsigned char JBIG_Coding;                  /* 78 T.85 basic */
    unsigned char JBIG_L0;                      /* 79 T.85 optional L0 */
                                                /* 80 extend */
    unsigned char HKM_Key_Mgt;                  /* 81 */
    unsigned char RSA_Key_Mgt;                  /* 82 */
    unsigned char Override_Mode;                /* 83 */
    unsigned char HFX40_Cipher;                 /* 84 */
                                                /* 85 reserved */
                                                /* 86 reserved */
    unsigned char HFX40_Hashing;                /* 87 */
                                                /* 88 extend */
                                                /* 89 reserved */
                                                /* 90 reserved */
                                                /* 91 reserved */
    /* (92-94) T.44 Mixed Raster Content Mode */
    unsigned char MRC_Mode;
    /* 95 T.44 Page length max strip size */
    unsigned char MRC_Page_Length_Strips;
                                                /* 96 extend */
    /* 97 300H_300V or 400H_400V valid resolutions for color */
    /* 98 100H_100V color resolution */
    unsigned char PhaseC_BFT_Negotiations;      /* 99 */
                                                /* 100 set to "0" */
                                                /* 101 set to "0" */
    /* 102 Internet Routing Address */
    unsigned char IRA;
                                                /* 103 reserved */
                                                /* 104 extend */
    /* 105 600H_600V b&w or color */
    /* 106 1200H_1200V b&w or color */
                                                /* 107 300H_600V b&w only */
                                                /* 108 400H_800V b&w only */
                                                /* 109 600H_1200V b&w only */
                                                /* 110 reserved */
                                                /* 111 reserved */
                                                /* 112 extend */
};

#pragma pack()

#endif
