/***********************************************************************
 * FILENAME:         ecclib.h
 *
 * CURRENT VERSION:  %version: 133 %
 * DATE MODIFIED:    %date_modified: Thu Jun 30 15:34:55 2005 %
 *
 * PURPOSE:
 *   This is the header file for Enhanced Call Control (ECC).
 *
 * Copyright (c) 1997 - 2004 Brooktrout Technology, Inc. All Rights Reserved.
 *
 * This software is the property of Brooktrout Technology, Inc.
 * Any reproduction, disclosure, or distribution of this software is
 * expressly prohibited unless first agreed to in writing by Brooktrout
 * Technology, Inc.
 ***********************************************************************/

#ifndef _ecclib_h

#include "btlib.h"

#ifndef _ecclib_h
#define _ecclib_h


/* Version strings for ECC */
#define ECC_VER_NUM                     000
#define ECC_VERSION                     SDK_VER_COMPLETE
#define ECC_V1                          SDK_MAJOR_VERSION
#define ECC_V2                          SDK_MIDDLE_VERSION
#define ECC_V3                          SDK_MINOR_VERSION
#define ECC_BUILD_NUM                   SDK_BUILD_NUMBER
#define ECC_VER_DATE                    SDK_VER_COMPLETE

#define _ECC_MAX_B_CHANNELS             32      /* number of b-channels */
#define _ECC_MAX_D_CHANNELS             9       /* number of d-channels */
#define _ECC_MAX_PORTS                  9       /* maximum number of ports */
#define _ECC_MAX_BOARDS                 14      /* maximum number of boards */
#define _ECC_MAX_ORDINALS               1344    /* 14 boards * 96 boston channels. */
#define _ECC_MAX_ORDINAL_PER_MODULE     60
#define _ECC_MAX_MODULES                128
#define _ECC_MAX_HOST_MODULES           9
#define _ECC_MAX_ANI_LENGTH             15
#define _ECC_MAX_IE_LENGTH              96

#define ECC_MAX_MSN                     10

#define _ECC_MAX_ETHERNET_PORTS         2       /* maximum number of ethernet ports */
#define _ECC_MAX_HOST_CC_ENTRIES        9       /* maximum number of host call control entries */
#define _ECC_MAX_IP_ADDRESS             32      /* maximum IP address string */

#define BfvCallAccept MillCallAccept
#define BfvCallCtrlClose MillCallCtrlClose
#define BfvCallCtrlInit MillCallCtrlInit
#define BfvCallDisconnect MillCallDisconnect
#define BfvCallReconfigureHostModule MillCallReconfigureHostModule
#define BfvCallReject MillCallReject
#define BfvCallRingDetect MillCallRingDetect
#define BfvCallSendAlerting MillCallSendAlerting
#define BfvCallSetup   MillCallSetup
#define BfvCallStatus MillISDNStatus
#define BfvCallWaitForAccept MillCallWaitForAccept
#define BfvCallWaitForAlerting MillCallWaitForAlerting
#define BfvCallWaitForComplete MillCallWaitForComplete
#define BfvCallWaitForRelease MillCallWaitForRelease
#define BfvCallWaitForSetup MillCallWaitForSetup
#define BfvDebugModeSetISDN MillDebugModeSetISDN
#define BfvISDNStatus MillISDNStatus
#define BfvLineCCProtocolGet MillLineCCProtocolGet

#define BfvLineTransferCapabilityQuery MillLineTransferCapabilityQuery
#define BfvLineTransfer MillLineTransfer
#define BfvLineTransferComplete MillLineTransferComplete
#define BfvLineTransferCancel MillLineTransferCancel
#define BfvCallHold MillCallHold
#define BfvCallWaitForHold MillCallWaitForHold
#define BfvCallRetrieve MillCallRetrieve
#define BfvCallWaitForRetrieve MillCallWaitForRetrieve
#define BfvCallTransferComplete MillCallTransferComplete
#define BfvCallWaitTransferComplete MillCallWaitTransferComplete

#define BfvCallDivert MillCallDivert
#define BfvCallWaitForDivert MillCallWaitForDivert
#define BfvLineDivert MillLineDivert

/* Define Bfv high level function names both with and without the
   ISDN suffix. */
#define BfvLineAnswer MillLineAnswerISDN
#define BfvLineDialString MillLineDialStringISDN
#define BfvLineOrigCallDB MillLineOrigCallDBISDN
#define BfvLineOriginateCall MillLineOriginateCallISDN
#define BfvLineTerminateCall MillLineTerminateCallISDN
#define BfvLineWaitForCall MillLineWaitForCallISDN
#define BfvLoopCurrentDetectDisable MillLoopCurrentDetectDisableISDN
#define BfvLoopCurrentDetectEnable MillLoopCurrentDetectEnableISDN

#define BfvLineAnswerISDN MillLineAnswerISDN
#define BfvLineDialStringISDN MillLineDialStringISDN
#define BfvLineOrigCallDBISDN MillLineOrigCallDBISDN
#define BfvLineOriginateCallISDN MillLineOriginateCallISDN
#define BfvLineTerminateCallISDN MillLineTerminateCallISDN
#define BfvLineWaitForCallISDN MillLineWaitForCallISDN
#define BfvLoopCurrentDetectDisableISDN MillLoopCurrentDetectDisableISDN
#define BfvLoopCurrentDetectEnableISDN MillLoopCurrentDetectEnableISDN
#define BfvEnableExceptionLogging MillEnableExceptionLogging
#define BfvDisableExceptionLogging MillDisableExceptionLogging


/* ecc_signalling_mode values for BfvCallCtrlInit arguments */
#define ECC_SIGNALLING_MODE_ROBBED_BIT  0
#define ECC_SIGNALLING_MODE_ISDN        1

/* B-Channel states */
#define BST_IDLE                    101
#define BST_DIAL                    102
#define BST_DIAL_COMPLETE           103
#define BST_ALERTING                104
#define BST_WAIT_FOR_CALL           107
#define BST_CALL_DETECTED           108
#define BST_CONNECTED               109
#define BST_CLEAR_CALL              110
#define BST_DOWN                    115

/* Transfer types */
#define LINE_XFER_NONE                  ECC_XFER_NONE              
#define LINE_XFER_SINGLE                ECC_XFER_SINGLE
#define LINE_XFER_TWO_CHAN              ECC_XFER_TWO_CHAN
#define LINE_XFER_ALL                   ECC_XFER_ALL
#define LINE_XFER_TWO_CHAN_NEEDS_NAILUP ECC_XFER_TWO_CHAN_NEEDS_NAILUP

/* Divert types */
#define DIVERT_NONE                 ECC_DIVERT_NONE
#define DIVERT_UNCONDITIONAL        ECC_DIVERT_UNCONDITIONAL
#define DIVERT_BUSY                 ECC_DIVERT_BUSY
#define DIVERT_NO_RESPONSE          ECC_DIVERT_NO_RESPONSE

/* Name Identification charater set */
#define NAME_CHAR_SET_UNKNOWN          -1
#define NAME_CHAR_SET_NOT_INCLUDED      0 /*charater set not included and will not be sent*/
#define NAME_CHAR_SET_ISO8859_1         1 /*charater set as specified in International Standard ISO 8859-1 */
#define NAME_CHAR_SET_ISO8859_2         3 /*charater set as specified in International Standard ISO 8859-2 */
#define NAME_CHAR_SET_ISO8859_3         4 /*charater set as specified in International Standard ISO 8859-3 */
#define NAME_CHAR_SET_ISO8859_4         5 /*charater set as specified in International Standard ISO 8859-4 */
#define NAME_CHAR_SET_ISO8859_5         6 /*charater set as specified in International Standard ISO 8859-5 */
#define NAME_CHAR_SET_ISO8859_7         7 /*charater set as specified in International Standard ISO 8859-7 */
#define NAME_CHAR_SET_ISO10646_BMP      8 /*charater set as specified in International Standard ISO 10646-1 and ITU-T Rec X.680 */
#define NAME_CHAR_SET_ISO10646_UTF      9 /*charater set as specified in International Standard ISO 10646-1 UTF-8-STRING as deffined in Annex R */

#ifndef _MAX_PATH
#define _MAX_PATH                   256  /* max. length of full pathname */
#endif

#ifndef IISDN_H
/* from iisdn.h only include if iisdn.h has not been included */
#define IISDNnumtUNKNOWN                0       /* Unknown type of number */
#define IISDNnumtINTERNATIONAL          1       /* International number */
#define IISDNnumtNATIONAL               2       /* National number */
#define IISDNnumtSUBSCRIBER             4       /* Subsriber number */
#define IISDNnumtABBREVIATED            6       /* Abbreviated number */

#define IISDNnumpUNKNOWN                0       /* Unknown numbering plan */
#define IISDNnumpISDN                   1       /* Recommendation E.164/E.163 */
#define IISDNnumpPRIVATE                9       /* Private numbering plan */
#define IISDNnumpTELEPHONY              2       /* Telephony numbering plan */

#define IISDNpresALLOWED                0       /* Presentation is allowed */
#define IISDNpresRESTRICTED             1       /* Presentation is restricted */
#define IISDNpresNUM_NOT_AVAIL          2       /* Number not avail. due to interworking */

#define IISDNscrUSER_NOT_SCREENED       0       /* User provided, not screened */
#define IISDNscrUSER_VER_PASSED         1       /* User provided, verified and passed */
#define IISDNscrUSER_VER_FAILED         2       /* User provided, verified and failed */
#define IISDNscrNETWORK_PROVIDED        3       /* Network provided */

#define IISDNdirUSER_SIDE               0       /* DTE (the customer premise equipment) */
#define IISDNdirNETWORK_SIDE            1       /* DCE (the network) */

#define IISDNvarJATE_REDIAL_3IN3_MINS   0       /* JATE Redial method 3 in 3 minutes    */
#define IISDNvarJATE_REDIAL_15X         1       /* JATE Redial method 15 times          */
#define IISDNvarJATE_REDIAL_NO_RESTRICT 2       /* JATE no redial restrictions          */
#endif /* IISDN_H */

// addition clearing causes that are not in iisdn.h
#define CLEARcausNO_DIALTONE            1000
#define CLEARcausINTERNAL_DIAL_ERROR    1001
#define CLEARcausNO_LOOPCURRENT         1002
#define CLEARcausJATE_REJECT            1003

#define ECC_MAX_DIGIT_STR MAX_DID
#define ECC_MAX_NAME_STR MAX_NAME_STR

#pragma pack(1)

typedef struct _information
{
    unsigned char called_party[ ECC_MAX_DIGIT_STR ];
    unsigned char calling_party[ ECC_MAX_DIGIT_STR ];
    int message;
    unsigned char aoc_information[ ECC_MAX_DIGIT_STR ];
    int aoc_set;
    unsigned long call_type;
    unsigned char redir_number[ ECC_MAX_DIGIT_STR ];
    int redir_reason;
    unsigned char called_party_subaddress[ ECC_MAX_DIGIT_STR ];
    unsigned char calling_party_subaddress[ ECC_MAX_DIGIT_STR ];
    unsigned char board_port_string[ ECC_MAX_DIGIT_STR ];
    unsigned char orig_called_num[ ECC_MAX_DIGIT_STR ];
    unsigned char calling_party_presentation;
    unsigned char calling_party_screening;
    unsigned char name_ident[ ECC_MAX_NAME_STR ];
    int name_char_set;
    int redir_counter;
    int redir_orig_reason;
    unsigned char connected_party[ ECC_MAX_DIGIT_STR ];
    unsigned char connected_num_type;
    unsigned char connected_num_plan;
    unsigned char connected_num_presentation;
    unsigned char connected_num_screening;
} INFORMATION;


typedef enum TRxBoardType {
    NOT_PRESENT = 0,
    SOMBRERO_BD = 1, 
    HERMES_BD, 
    CAB_BRI_BD, 
    CAB_ANALOG_BD,
    COLOMA_BD,
    TIGGER_BD,
    CAB_BRI_BD_WITH_H100, 
    CAB_ANALOG_BD_WITH_H100,
    VIRTUAL_BD
} TRxBoardType;

/* valid port configurations */
typedef enum TRxPortConfig {
    PORT_CONFIG_INACTIVE                   = 0,
    PORT_CONFIG_MVIP                       = 1,
    PORT_CONFIG_H1X0                       = 2,
    PORT_CONFIG_T1_ISDN                    = 3,
    PORT_CONFIG_T1_ROBBED_BIT              = 4,
    PORT_CONFIG_E1_ISDN                    = 5,
    PORT_CONFIG_E1_CAS                     = 6,
    PORT_CONFIG_E1_CAS_R2                  = 7,
    PORT_CONFIG_E1_DPNSS                   = 8,
    PORT_CONFIG_BRI                        = 9,
    PORT_CONFIG_ANALOG                     = 10,
    PORT_CONFIG_HOST_MODULE_1               = 12,
    PORT_CONFIG_HOST_MODULE_2               = 13,
    PORT_CONFIG_HOST_MODULE_3               = 14,
    PORT_CONFIG_HOST_MODULE_4               = 15,
    PORT_CONFIG_HOST_MODULE_5               = 16,
    PORT_CONFIG_HOST_MODULE_6               = 17,
    PORT_CONFIG_HOST_MODULE_7               = 18,
    PORT_CONFIG_HOST_MODULE_8               = 19,
    PORT_CONFIG_HOST_MODULE_9               = 20,
    PORT_CONFIG_E1_QSIG                     = 21,
    PORT_CONFIG_T1_QSIG                     = 22,
    PORT_CONFIG_BRI_QSIG                    = 23,
    PORT_CONFIG_UNKNOWN                     = 24
} TRxPortConfig;


typedef enum TRxEmulation {
    USER_EMULATION = IISDNdirUSER_SIDE,
    NETWORK_EMULATION = IISDNdirNETWORK_SIDE
} TRxEmulation;

typedef enum TRxT1LineBuildOut {
    T1_0_133_FT     = TELE_CTRL_0_133_DEF,
    T1_133_266_FT   = TELE_CTRL_133_266_DEF,
    T1_266_399_FT   = TELE_CTRL_266_399_DEF,
    T1_399_533_FT   = TELE_CTRL_399_533_DEF,
    T1_533_655_FT   = TELE_CTRL_533_655_DEF,
    T1_NEG_7_5DB    = TELE_CTRL_NEG_7_5DB_DEF,
    T1_NEG_15DB     = TELE_CTRL_NEG_15DB_DEF,
    T1_NEG_22_5DB   = TELE_CTRL_NEG_22_5DB_DEF
} TRxT1LineBuildOut;

typedef enum TRxE1LineImpedance {
    E1_75_OHM   = TELE_CTRL_75_OHM_DEF,
    E1_120_OHM  = TELE_CTRL_120_OHM_DEF
} TRxE1LineImpedance;

typedef struct TRxInactiveInfo {
    int reserved;
} TRxInactiveInfo;

typedef enum TRxFraming {
    FRAMING_D4  = TELE_CTRL_DSX1D4_DEF,
    FRAMING_ESF = TELE_CTRL_DSX1EF_DEF
} TRxFraming;

typedef enum TRxT1LineCoding {
    T1_LINE_CODING_JBZS     = TELE_CTRL_DSX1JBZS_DEF,
    T1_LINE_CODING_B8ZS     = TELE_CTRL_DSX1B8ZS_DEF,
    T1_LINE_CODING_ZBTSI    = TELE_CTRL_DSX1ZBTSI_DEF,
    T1_LINE_CODING_AMI      = TELE_CTRL_DSX1AMI_DEF
} TRxT1LineCoding;

typedef enum TRxE1LineCoding {
    E1_LINE_CODING_HDB3 = TELE_CTRL_DSX1HDB3_DEF,
    E1_LINE_CODING_AMI  = TELE_CTRL_DSX1AMI_DEF
} TRxE1LineCoding;

typedef enum TRxECCCallType {
    ECC_CALL_TYPE_DEFAULT       = 0, /* Use default from config file */
    ECC_CALL_TYPE_VOICE         = 1, /* Voice call */
    ECC_CALL_TYPE_MODEM         = 2,  /* 3.1 kHz audio */
    ECC_CALL_TYPE_AUTO          = 99, /* try 3.1 kHz audio and then voice */
    ECC_CALL_TYPE_AUTO_REDIAL   = 100 /* for internal use only */
} TRxECCCallType;

typedef enum TRxNumberingType {
    ECC_NUM_TYPE_UNKNOWN        = IISDNnumtUNKNOWN,
    ECC_NUM_TYPE_INTERNATIONAL  = IISDNnumtINTERNATIONAL,
    ECC_NUM_TYPE_NATIONAL       = IISDNnumtNATIONAL,
    ECC_NUM_TYPE_SUBSCRIBER     = IISDNnumtSUBSCRIBER,
    ECC_NUM_TYPE_ABBREVIATED    = IISDNnumtABBREVIATED,
    ECC_NUM_TYPE_LEV_2_REGION   = 1,
    ECC_NUM_TYPE_LEV_1_REGION   = 2,
    ECC_NUM_TYPE_PISN_SPEC      = 3,
    ECC_NUM_TYPE_LEV_0_REGION   = 4
} TRxNumberingType;

typedef enum TRxNumberingPlan {
    ECC_NUM_PLAN_UNKNOWN    = IISDNnumpUNKNOWN,
    ECC_NUM_PLAN_ISDN       = IISDNnumpISDN,
    ECC_NUM_PLAN_TELEPHONY  = IISDNnumpTELEPHONY,
    ECC_NUM_PLAN_PRIVATE    = IISDNnumpPRIVATE
} TRxNumberingPlan;

typedef enum TRxPresentation {
    ECC_PRES_APP_DEFINED    = 0xFF,
    ECC_PRES_ALLOWED        = IISDNpresALLOWED,
    ECC_PRES_RESTRICTED     = IISDNpresRESTRICTED,
    ECC_PRES_NUM_NOT_AVAIL  = IISDNpresNUM_NOT_AVAIL
} TRxPresentation;

typedef enum TRxScreening {
    ECC_SCRN_APP_DEFINED                = 0xFF,
    ECC_SCRN_USER_NOT_SCREENED          = IISDNscrUSER_NOT_SCREENED,
    ECC_SCRN_USER_VERIFICATION_PASSED   = IISDNscrUSER_VER_PASSED,
    ECC_SCRN_USER_VERIFICATION_FAILED   = IISDNscrUSER_VER_FAILED,
    ECC_SCRN_NETWORK_PROVIDED           = IISDNscrNETWORK_PROVIDED
} TRxScreening;

typedef enum TransferVariant {
    XFER_VARIANT_NONE = 0,       //INVALID_PROTOCOL
    XFER_VARIANT_ETSI_EXP,       //Q931_PROTOCOL
    XFER_VARIANT_ETSI_IMP_EXP,   //Q931_XFER_IMP_EXP_LINK
    XFER_VARIANT_DPNSS,          //DPNSS_PROTOCOL
    XFER_VARIANT_NTT_P2P,        //NTT_P2P_PROTOCOL
    XFER_VARIANT_NTT_P2MP,       //NTT_P2MP_PROTOCOL
    XFER_VARIANT_TBCT,           //NATL_ISDN
    XFER_VARIANT_RLT,            //RLT_PROTOCOL
    XFER_VARIANT_HOOK_FLASH,     //HOOK_FLASH_PROTOCOL
    XFER_VARIANT_QSIG            //QSIG_PROTOCOL
} TransferVariant;

typedef struct TRxT1RobbedBitInfo {
    enum TRxT1LineBuildOut line_build_out;
    enum TRxFraming framing; /* d4, esf ect.*/
    enum TRxT1LineCoding line_coding;
    char cas_protocol_file[_MAX_PATH];
    unsigned long max_did_digits;
    unsigned long did_timeout;
    unsigned long did_offset;
    unsigned long reject_incomplete_did;
    unsigned long flash_hook_dur;  /* flash hook duration in 10ms units */
    unsigned long require_answer_signal;
    TransferVariant transfer_variant;
    char          caller_id;   /* 0 = disabled, 1 = enabled */
    unsigned char num_rings;   /* 0 to 255 rings for loop and ground start*/
    unsigned long loop_reversal_for_connect;
    unsigned long loop_reversal_for_disconnect;
} TRxT1RobbedBitInfo;

typedef struct TRxT1ISDNInfo {
    enum TRxT1LineBuildOut line_build_out;
    enum TRxT1LineCoding line_coding;
    unsigned long protocol;
    enum TRxEmulation emulation;
    unsigned long switch_type;
    unsigned long max_did_digits;
    unsigned long did_timeout;
    unsigned long reject_incomplete_did;
    unsigned long did_offset;
    unsigned long disable_conn_ack;
    unsigned long wait_for_conn_ack;
    unsigned long wait_for_bchannel_status;
    char default_caller_id[_ECC_MAX_ANI_LENGTH];
    enum TRxNumberingType   numbering_type;
    enum TRxNumberingPlan   numbering_plan;
    enum TRxScreening       screening;
    enum TRxPresentation    presentation;
    enum TRxECCCallType     call_type;
    unsigned long NSF;
    unsigned long sabme;
    TransferVariant transfer_variant;
} TRxT1ISDNInfo;

typedef struct TRxE1ISDNInfo {
    enum TRxE1LineImpedance line_impedance;
    enum TRxE1LineCoding line_coding;
    unsigned long crc_enabled;
    unsigned long protocol;
    enum TRxEmulation emulation;
    unsigned long switch_type;
    unsigned long max_did_digits;
    unsigned long did_timeout;
    unsigned long reject_incomplete_did;
    unsigned long did_offset;
    unsigned long send_dialcomplete;
    unsigned long disable_conn_ack;
    unsigned long wait_for_conn_ack;
    char default_caller_id[_ECC_MAX_ANI_LENGTH];
    enum TRxNumberingType   numbering_type;
    enum TRxNumberingPlan   numbering_plan;
    enum TRxScreening       screening;
    enum TRxPresentation    presentation;
    enum TRxECCCallType     call_type;
    unsigned long sabme;
    TransferVariant transfer_variant;
    unsigned char max_overlapped_digits;
    unsigned long disable_call_proceed;
    unsigned long send_restart;
} TRxE1ISDNInfo;

typedef struct TRxE1DPNSSInfo {
    enum TRxE1LineImpedance line_impedance;
    enum TRxE1LineCoding line_coding;
    unsigned long crc_enabled;
    unsigned long max_did_digits;
    unsigned long did_timeout;
    unsigned long reject_incomplete_did;
    unsigned long did_offset;
    unsigned long send_dialcomplete;
    char default_caller_id[_ECC_MAX_ANI_LENGTH];
    unsigned long layer2_designation;
    unsigned long layer3_designation;
    TransferVariant transfer_variant;
    unsigned char max_overlapped_digits;
} TRxE1DPNSSInfo;

typedef struct T1QSIGLineParms
{
    enum TRxT1LineBuildOut line_build_out;
    enum TRxT1LineCoding line_coding;
}T1QSIGLineParms;

typedef struct E1QSIGLineParms
{
    enum TRxE1LineImpedance line_impedance;
    unsigned long crc_enabled;
    enum TRxE1LineCoding line_coding;
}E1QSIGLineParms;

//Both T1 and E1 use the same structure
typedef struct TRxQSIGInfo {
    union line_parms {
        T1QSIGLineParms T1QSIGLineParameters;
        E1QSIGLineParms E1QSIGLineParameters;
    } line_parameters;
    unsigned long max_did_digits;
    unsigned long did_timeout;
    unsigned long reject_incomplete_did; //check if this is valid
    unsigned long did_offset;
    unsigned long send_dialcomplete;
    unsigned long disable_conn_ack; //will also not wait for connack
    char default_caller_id[_ECC_MAX_ANI_LENGTH];
    unsigned long collision_priority;
    unsigned long emulation;
    enum TRxNumberingType   numbering_type;
    enum TRxNumberingPlan   numbering_plan;
    enum TRxScreening       screening;
    enum TRxPresentation    presentation;
    enum TRxECCCallType     call_type;
    unsigned long sabme;
    TransferVariant transfer_variant;
    unsigned char max_overlapped_digits;
    unsigned long enable_call_diversion;
    unsigned long disable_alerting;
    unsigned long request_aoc;
    unsigned long disable_call_proceed;
    unsigned long call_diversion_completion_timer;
    unsigned long qsig_support;
} TRxQSIGInfo;

typedef struct TRxE1CASR2Info {
    enum TRxE1LineImpedance line_impedance;
    enum TRxE1LineCoding line_coding;
    unsigned long crc_enabled;
    unsigned long max_did;
    char default_caller_id[_ECC_MAX_ANI_LENGTH];
    char r2_protocol_file[_MAX_PATH];
} TRxE1CASR2Info;

typedef struct TRxE1CASInfo {
    enum TRxE1LineImpedance line_impedance;
    enum TRxE1LineCoding line_coding;
    unsigned long crc_enabled;
    char cas_protocol_file[_MAX_PATH];
    unsigned long require_answer_signal;
    unsigned long max_did_digits;
    unsigned long did_timeout;
    unsigned long did_offset;
    unsigned long reject_incomplete_did;
    unsigned long flash_hook_dur;  /* flash hook duration in 10ms units */
    TransferVariant transfer_variant;
    char          caller_id;   /* 0 = disabled, 1 = enabled */
    unsigned char num_rings;   /* 0 to 255 rings for loop and ground start */
    unsigned long loop_reversal_for_connect;
    unsigned long loop_reversal_for_disconnect;
} TRxE1CASInfo;

typedef struct TRxAnalogInfo {
    char            input_gain;  /* +6 to -6 dB */
    char            output_gain; /* +6 to -6 dB */
    char            caller_id;   /* 0 = disabled, 1 = enabled */
    unsigned char   num_rings;   /* 0 to 255 rings */
    unsigned long   flash_hook_dur;  /* flash hook duration in 10ms units */
    /* pause before dialing when no dial initiation character is provided */
    unsigned long   missing_wait;
    char            analog_protocol_file[_MAX_PATH];
    char            country_parameter_file[_MAX_PATH];
    unsigned long   max_did_digits;
    TransferVariant transfer_variant;
    unsigned long   loop_reversal_for_connect;
    unsigned long   loop_reversal_for_disconnect;
} TRxAnalogInfo;

typedef enum TRxBRIMode {
    BRI_AUTO_DETECT = 0,
    BRI_POINT_2_POINT = 1,
    BRI_POINT_2_MULTI_POINT = 2
} TRxBRIMode;

typedef struct TRxBRIInfo {
    enum TRxEmulation emulation;
    unsigned long max_did_digits;
    unsigned long did_timeout;
    unsigned long reject_incomplete_did;
    unsigned long did_offset;
    unsigned long send_dialcomplete;
    unsigned long disable_conn_ack;
    unsigned long wait_for_conn_ack;
    char chan0_default_caller_id[_ECC_MAX_ANI_LENGTH];
    char chan1_default_caller_id[_ECC_MAX_ANI_LENGTH];
    char chan0_default_caller_sub_address[_ECC_MAX_ANI_LENGTH];
    char chan1_default_caller_sub_address[_ECC_MAX_ANI_LENGTH];
    char msn[ECC_MAX_MSN][_ECC_MAX_ANI_LENGTH];
    enum TRxNumberingType   numbering_type;
    enum TRxNumberingPlan   numbering_plan;
    enum TRxScreening       screening;
    enum TRxPresentation    presentation;
    enum TRxECCCallType     call_type;
    /* 1 for point 2 multi-point, 0 for point 2 point */
    enum TRxBRIMode         mode;
    unsigned long preferred;
    unsigned long redirect_as_calling_party;
    char spid[_ECC_MAX_ANI_LENGTH];
    TransferVariant transfer_variant;
    unsigned char max_overlapped_digits;
    unsigned long disable_call_proceed;
} TRxBRIInfo;

typedef struct TRxMVIPInfo {
    unsigned long sec8k;
} TRxMVIPInfo;

typedef enum TRxH1X0MasterDrive {
    MASTER_DRIVE_A = TELE_CTRL_MASTER_DRIVE_A_DEF,
    MASTER_DRIVE_B = TELE_CTRL_MASTER_DRIVE_B_DEF
} TRxH1X0MasterDrive;

typedef enum TRx1X0SlaveSource {
    SLAVE_A = TELE_CTRL_SLAVE_A_DEF,
    SLAVE_B = TELE_CTRL_SLAVE_B_DEF,
    SLAVE_A_NO_FALLBACK = TELE_CTRL_SLAVE_A_NO_FALL_DEF,
    SLAVE_B_NO_FALLBACK = TELE_CTRL_SLAVE_B_NO_FALL_DEF
} TRx1X0SlaveSource;

typedef enum TRx1X0MasterRef {
    MASTER_REF_A = TELE_CTRL_MASTER_REF_A_DEF,
    MASTER_REF_B = TELE_CTRL_MASTER_REF_B_DEF,
    MASTER_REF_C = TELE_CTRL_MASTER_REF_C_DEF,
    MASTER_REF_D = TELE_CTRL_MASTER_REF_D_DEF,
    MASTER_REF_E = TELE_CTRL_MASTER_REF_E_DEF,
    MASTER_REF_F = TELE_CTRL_MASTER_REF_F_DEF,
    MASTER_REF_G = TELE_CTRL_MASTER_REF_G_DEF,
    MASTER_REF_H = TELE_CTRL_MASTER_REF_H_DEF,
    /* not a valid option for master_ref */
    MASTER_NO_FALLBACK = TELE_CTRL_MASTER_NO_FALLBACK_DEF,
    MASTER_REF_NETREF1 = TELE_CTRL_MASTER_REF_NETREF1_DEF,
    MASTER_REF_NETREF2 = TELE_CTRL_MASTER_REF_NETREF2_DEF,
    MASTER_REF_INTERNAL = TELE_CTRL_MASTER_REF_INTERNAL_DEF,
    MASTER_REF_CLOCK_A = TELE_CTRL_MASTER_REF_A_BUS_DEF,
    MASTER_REF_CLOCK_B = TELE_CTRL_MASTER_REF_B_BUS_DEF
} TRx1X0MasterRef;

typedef struct TRxH1X0Info {
    unsigned long bus_speed;
    enum TRxH1X0MasterDrive master_drive;
    enum TRx1X0SlaveSource slave_source;
    enum TRx1X0MasterRef master_ref_fallback;
} TRxH1X0Info;

typedef enum TRxLineEncoding
{
    A_LAW  = TELE_CTRL_A_LAW_DEF,
    MU_LAW = TELE_CTRL_MU_LAW_DEF
} TRxLineEncoding;

typedef struct TRxPortInfo
{
    enum TRxPortConfig port_config;
    unsigned long fractional_start_channel;
    unsigned long fractional_channel_count;
    unsigned long wait_for_service_timeout;
    unsigned long capabilities;
    union TRx_config_types {
        TRxInactiveInfo    InactiveInfo;
        TRxMVIPInfo        MVIPInfo;
        TRxH1X0Info        H1X0Info;
        TRxT1RobbedBitInfo T1RobbedBitInfo;
        TRxT1ISDNInfo      T1ISDNInfo;
        TRxE1ISDNInfo      E1ISDNInfo;
        TRxE1DPNSSInfo     E1DPNSSInfo;
        TRxQSIGInfo        QSIGInfo;
        TRxE1CASInfo       E1CASInfo;
        TRxE1CASR2Info     E1CASR2Info;
        TRxBRIInfo         BRIInfo;
        TRxAnalogInfo      AnalogInfo;
    } config;
} TRxPortInfo;

typedef enum TRxClockMode {
    CLOCK_SLAVE,
    CLOCK_MASTER,
    CLOCK_SECONDARY_MASTER
} TRxClockMode ;

typedef enum TRxClockSource {
    CLOCK_INTERNAL,
    CLOCK_NETA,
    CLOCK_NETB,
    CLOCK_NETC,
    CLOCK_NETD,
    CLOCK_NETE,
    CLOCK_NETF,
    CLOCK_NETG,
    CLOCK_NETH,
    CLOCK_NETREF1,
    CLOCK_NETREF2,
    CLOCK_CLOCKA,
    CLOCK_CLOCKB
} TRxClockSource ;

typedef enum TRxClockCompatibility {
    CLOCK_COMPATIBILITY_NONE,
    CLOCK_COMPATIBILITY_MVIP,
    CLOCK_COMPATIBILITY_SCSA
} TRxClockCompatibility;

typedef struct TRxClockInfo
{
    enum TRxClockMode clock_mode;
    enum TRxClockSource clock_source;
    enum TRxClockCompatibility clock_compatibility;
} TRxClockInfo;

typedef enum TRxJATERestrictions {
    RESTRICTION_NONE = IISDNvarJATE_REDIAL_NO_RESTRICT,
    RESTRICTION_15_TIMES = IISDNvarJATE_REDIAL_15X,
    RESTRICTION_3_IN_3_MINS = IISDNvarJATE_REDIAL_3IN3_MINS
} TRxJATERestrictions;

// Corresponds with IISDN_MAX_EMERG_NUMS
#define ECC_MAX_EMERG_NUMS                  10
// Corresponds with IISDN_MAX_DIGITS
#define ECC_MAX_DIGITS                      24
// Default value for the amount of JATE numbers to track
#define ECC_DEFAULT_JATE_TRACKED_NUMBERS    200

typedef struct TRxJATESettings
{
    TRxJATERestrictions redial_restriction;
    char emergency_number[ECC_MAX_EMERG_NUMS][ECC_MAX_DIGITS];
} TRxJATESettings;

// Settings for the ethernet_speed field.
typedef enum TRxEthernetSpeed {
    ECC_ETHERNET_SPEED_AUTO   = TELE_CTRL_SPEED_AUTO_DEF, /* auto detect */
    ECC_ETHERNET_SPEED_10MBS  = TELE_CTRL_SPEED_10M_DEF,  /* 10 Mbs */
    ECC_ETHERNET_SPEED_100MBS = TELE_CTRL_SPEED_100M_DEF  /* 100 Mbs */
} TRxEthernetSpeed;

// Settings for the ethernet_duplex field.
typedef enum TRxEthernetDuplex {
    ECC_ETHERNET_DUPLEX_HALF = TELE_CTRL_DUPLEX_HALF_DEF,  /* half duplex operation */
    ECC_ETHERNET_DUPLEX_FULL = TELE_CTRL_DUPLEX_FULL_DEF  /* full duplex operation NOT SUPPORTED */
} TRxEthernetDuplex;

// Settings for the ethernet_flow_control.
typedef enum TRxEthernetFlowConrol {
    ECC_ETHERNET_FLOW_CONTROL_AUTO = TELE_CTRL_FULLDUP_AUTO_DEF,      /* auto detect */
    ECC_ETHERNET_FLOW_CONTROL_RECEIVE_ONLY = TELE_CTRL_FULLDUP_RECEIVE_ONLY_DEF,  /* receive only flow control */
    ECC_ETHERNET_FLOW_CONTROL_TRANSMIT_ONLY = TELE_CTRL_FULLDUP_TRANSMIT_ONLY_DEF, /* transmit only flow control */
    ECC_ETHERNET_FLOW_CONTROL_BOTH = TELE_CTRL_FULLDUP_BOTH_DEF          /* both directions flow control */
} TRxEthernetFlowControl;

// Ethernet interface configuration.
typedef struct TRxEthernetInfo {
    char  dhcp;                              /* DHCP enabled or not */
    char  ip_address[_ECC_MAX_IP_ADDRESS];   /* IP address */
    char  ip_netmask[_ECC_MAX_IP_ADDRESS];   /* IP netmask mask */
    char  ip_gateway[_ECC_MAX_IP_ADDRESS];   /* IP gateway address */
    char  ip_broadcast[_ECC_MAX_IP_ADDRESS]; /* IP broardcast address */
    int   ip_arp_timeout;                    /* IP ARP protocol timeout */
    int   media_port_min;                    /* min media port number */
    int   media_port_max;                    /* max media port number */
    int   ethernet_speed;                    /* ethernet speed (10/100) */
    int   ethernet_duplex;                   /* duplex operation (full/half) */
    int   ethernet_flow_control;             /* ethernet flow control */
} TRxEthernetInfo;

// Host call control configuration.
typedef struct TRxHostCCInfo {
    int  host_module_num;                    /* host module identifier */
    int  num_channels;                       /* number of channels to configure */
    unsigned long did_offset;
} TRxHostCCInfo;

typedef struct TRxModuleInfo
{
    unsigned long module;        /* the module that this configuration is for */
    TRxPortInfo port_info[_ECC_MAX_PORTS];
    TRxClockInfo clock_info;
    enum TRxBoardType board_type;
    enum TRxLineEncoding line_encoding; /* 1 = mu-law, 0 = a-law */
    unsigned long auto_connect;  /* when set nails up dsp and phone lines */
    unsigned long num_ports;
    unsigned long exists;
    char btcall[_MAX_PATH];
    char connections[_MAX_PATH];
    unsigned long set_api;       /* 0 for ECC, 1 for BSMI */
    TRxJATESettings jate;
    unsigned long num_ethernet_ports;
    TRxEthernetInfo ethernet_info[_ECC_MAX_ETHERNET_PORTS + 1];
    unsigned long num_hostcc_entries;
    TRxHostCCInfo hostcc_info[_ECC_MAX_PORTS + 1];
    unsigned num_channels;
    char vb_firm[_MAX_PATH];
    unsigned long static_ring_detect_enable;
} TRxModuleInfo;

typedef enum TRxT38BitRate {
    ECC_T38_BIT_RATE_2400  = CALL_SW_BIT_RATE_2400_DEF,
    ECC_T38_BIT_RATE_4800  = CALL_SW_BIT_RATE_4800_DEF,
    ECC_T38_BIT_RATE_7200  = CALL_SW_BIT_RATE_7200_DEF,
    ECC_T38_BIT_RATE_9600  = CALL_SW_BIT_RATE_9600_DEF,
    ECC_T38_BIT_RATE_12000 = CALL_SW_BIT_RATE_12000_DEF,
    ECC_T38_BIT_RATE_14400 = CALL_SW_BIT_RATE_14400_DEF
} TRxT38BitRate;

typedef enum TRxT38FaxRateManagement
{
    ECC_T38_FAX_RATE_MANAGEMENT_TRANSFERRED_TCF  = CALL_SW_TRANSFERRED_TCF_DEF,
    ECC_T38_FAX_RATE_MANAGEMENT_LOCAL_TCF = CALL_SW_LOCAL_TCF_DEF
} TRxT38FaxRateManagement;

typedef enum TRxT38FaxUdpEC
{
    ECC_T38_FAX_UDP_EC_REDUNDANCY  = CALL_SW_REDUNDANCY_DEF,
    ECC_T38_FAX_UDP_EC_FEC = CALL_SW_FEC_DEF
} TRxT38FaxUdpEC;

// T38 parameter information
typedef struct TRxT38Info {
    int t38_fax_version;
    enum TRxT38BitRate t38_max_bit_rate;
    int t38_fax_fill_bit_removal;
    int t38_fax_transcoding_MMR;
    int t38_fax_transcoding_JBIG;
    enum TRxT38FaxRateManagement t38_fax_rate_management;
    int t38_fax_max_buffer;
    int t38_fax_max_datagram_recv;
    enum TRxT38FaxUdpEC t38_fax_udp_EC;
    int t38_fax_max_datagram_send;
    int t38_UDPTL_redundancy_depth_control;
    int t38_UDPTL_redundancy_depth_image;
    int t38_t30_fast_notify;
    int media_renegotiate_delay_inbound;  /* delay before media renegotiation (inbound call)*/
    int media_renegotiate_delay_outbound; /* delay before media renegotiation (outbound call)*/
} TRxT38Info;

// Host module configuration key type.
typedef enum TRxKeyType {
    TRX_KEY_STRING = 0,                      /* key type string */
    TRX_KEY_INT,                             /* key type integer */
    TRX_KEY_UNSIGNED_INT                     /* key type unsigned integer */
} TRxKeyType;

// Host module configuration key value.
typedef union TRxKeyValue {
    char          string_value[_MAX_PATH];   /* string value (nul terminated) */
    int           int_value;                 /* integer value */
    unsigned int  uint_value;                /* unsigned integer value */
} TRxKeyValue;

// Host module configuration key information.
typedef struct TRxKeyInfo {
    char               key_name[_MAX_PATH];  /* key name */
    enum TRxKeyType    key_type;             /* key type */
    union TRxKeyValue  key_value;            /* key value */
} TRxKeyInfo;

// Host module information.
typedef struct TRxHostModuleInfo {
    unsigned long   module;                  /* the module that this configuration is for */
    unsigned long   exists;                  /* module actually exists */
    int             inbound_call_timeout;    /* timeout before terminating inbound call */
    int             reserved1;
    int             reserved2;
    long            ip_stack_trace_level;    /* IP stack trace level */
    char            library_path[_MAX_PATH]; /* library pathname */
    char            library_enabled;         /* library enabled or disabled */
    char            media_use_unique_port;   /* use same of different ports for audio and image */
    int             num_keys;                /* number of configuration key pairs */
    TRxKeyInfo *    key;                     /* configuration key pairs */
    TRxT38Info      t38_info;                /* T38 fax parameters */
} TRxHostModuleInfo;

typedef enum ECC_TRACE_LEVEL {
    ecc_trace_none = 0,
    ecc_trace_error,
    ecc_trace_warning,
    ecc_trace_basic,
    ecc_trace_verbose,
    ecc_trace_level_max
} ecc_trace_level;

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_cc
{
    int mode;
    int call_protocol_code;
    int cause;
    long timeout;
    const char *calling_party;
    char orig_called_num[ECC_MAX_DIGIT_STR];
    unsigned char calling_party_presentation;
    unsigned char calling_party_screening;
    enum TRxECCCallType call_type;
    int call_mode;
    const char *phonenum;
    int state;
    RES res;
    CALL_RES cres;
    M_CB1 int (* M_CB2 func)(MILL_LINE *lp, char *arg);
    char *arg;
    /* file name of the btcall.cfg user preference file */
    const char *btcall_file;
    /* array of config structures for CallCtrlInit */
    struct TRxModuleInfo * ecc_cfg_struct;
    int num_modules_in_cfg;    /* number of elements ecc_cfg_struct */
    int set_log_file;
    const char *log_file;      /* ECC log file name */
    /* second lp where the enquiry call will be placed for transfer*/
    MILL_LINE *lp_second_channel;
    /* flag to specify that we are making an enquiry call */
    int enquiry_call;
    int calls_on_hold;         /* number of calls on hold for this channel */
    int ie_count;
    int ie_length;
    unsigned char * ie_data;
    int cause_location;
    char name_ident[MAX_NAME_STR];
    int name_char_set;
    int disable_auto_sw_connect;
    CONNECTED_NUM connected_num;
};




#pragma pack()

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/


#endif    /* _ecclib_h */
#endif    /* _ecclib_h */



#ifdef MILL_DO_PROTOS

B_APIE1 void B_APIE2 MillCallSetup(MILL_LINE *lp, struct args_cc *args);
B_APIE1 int  B_APIE2 MillCallWaitForComplete(MILL_LINE *lp, struct args_cc *args);
B_APIE1 int  B_APIE2 MillCallWaitForSetup(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallDisconnect(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallWaitForRelease(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallRingDetect(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillISDNStatus(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallAccept(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallWaitForAccept(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallReject(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallSendAlerting(MILL_LINE *lp, struct args_cc *args);
B_APIE1 int  B_APIE2 MillCallCtrlInit(struct args_cc *args);
M_APIE1 int  M_APIE2 MillCallCtrlClose(void);
B_APIE1 int  B_APIE2 MillCallWaitForAlerting(MILL_LINE *lp, struct args_cc *args);
M_APIE1 int  M_APIE2 MillCallReconfigureHostModule(int host_module);

B_APIE1 void B_APIE2 MillCallHold(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallWaitForHold(MILL_LINE * lp, struct args_cc * args);
B_APIE1 void B_APIE2 MillCallRetrieve(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallWaitForRetrieve(MILL_LINE * lp, struct args_cc * args);
B_APIE1 void B_APIE2 MillCallTransferComplete(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallWaitTransferComplete(MILL_LINE * lp, struct args_cc * args);

B_APIE1 void B_APIE2 MillCallDivert(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallWaitForDivert(MILL_LINE * lp, struct args_cc * args);
B_APIE1 void B_APIE2 MillCallForwardDivert(MILL_LINE *lp, struct args_cc *args);
B_APIE1 void B_APIE2 MillCallWaitForForwardDivert(MILL_LINE * lp, struct args_cc * args);

B_APIE1 void B_APIE2 MillLineOriginateCallISDN(MILL_LINE *lp,struct args_telephone *tel_args);
B_APIE1 int  B_APIE2 MillLineDialStringISDN(MILL_LINE *lp,struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineOrigCallDBISDN(MILL_LINE *lp, struct args_telephone *args);
B_APIE1 void B_APIE2 MillLineWaitForCallISDN(MILL_LINE *lp,struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineAnswerISDN(MILL_LINE *lp,struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineTerminateCallISDN(MILL_LINE *lp,struct args_telephone *tel_args);
M_APIE1 void M_APIE2 MillDebugModeSetISDN(int mode);
B_APIE1 void B_APIE2 MillLoopCurrentDetectEnableISDN(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLoopCurrentDetectDisableISDN(MILL_LINE *lp, struct args_telephone *tel_args);
M_APIE1 int  M_APIE2 MillLineReset_NoEcc(MILL_LINE *lp, struct args_line_admin *args);
B_APIE1 void B_APIE2 MillLineTransferCapabilityQuery(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineTransfer(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineTransferComplete(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineTransferCancel(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineHold(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineRetrieve(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineDivert(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 void B_APIE2 MillLineForwardDivert(MILL_LINE *lp, struct args_telephone *tel_args);
B_APIE1 enum TRxPortConfig B_APIE2 MillLineCCProtocolGet(MILL_LINE *lp, struct args_telephone *tel_args);
M_APIE1 void M_APIE2 MillEnableExceptionLogging(char * logFileName);
M_APIE1 void M_APIE2 MillDisableExceptionLogging();


#endif
