/* fsk_cp.h */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

#define MAX_FSK_BUF 1023   /* Max size of FSK data storage in bytes */
#define MAX_CP_BUF  128    /* Max size of call progress storage in bytes */


/************************************************************************/
/*                                                                      */
/* Structure : fsk_cp_info                                              */
/* Overview :This structure contains user accessible buffers that are   */
/* used to log all fsk data (transmitted and received) that is reported */
/* by the board, and all call progress updates and values.  The user    */
/* can use this information as a means of tracking the status of a call */
/* and can also use the call progress HI and LOW reports as input into  */
/* a call detection algorithm if the user wants to customize call       */
/* detection to recognize application specific tones.  The format for   */
/* the FSK buffer is MSxNSy where M and N represent the number of valid */
/* bytes of data to read for the attached FSK entry, not including the  */
/* S byte, which represents whether the data was sent or received       */
/* (sent = 1, received = 0).  M and N are each a byte in length and     */
/* are not included in the count of bytes to read.                      */
/* The format of the CP buffer is every two bytes represents one sample */
/* of data with the upper byte distinguishing between call progress HI, */
/* call progress LOW and New Call Progress, and the lower byte contains */
/* the appropiate associated data.  For Call Progress HI and Call       */
/* Progress LOW the data field contains the amount of time in five      */
/* millisecond units that the phone line was high or low.  New Call     */
/* Progress has a byte association value to report call status.         */
/* The buffers are implemented as ring buffers and wrap when they are   */
/* full.  If a buffer wraps, all previous data in the buffer is         */
/* discarded and only data that is logged after the wrap occured will   */
/* be available.  This is done to preserve the chronological order of   */
/* the data in the buffer (no partial overwrites).                      */
/*                                                                      */
/* Information Field Values :                                           */
/*                                                                      */
/* fsk_data :                                                           */
/*      FSK data storage buffer - holds 'MAX_FSK_BUF' bytes plus a null */
/* cp_data :                                                            */
/*      Call Progress data storage buffer - holds 'MAX_CP_BUF' bytes    */
/*      which allows for 1/2 'MAX_CP_BUF' data samples.                 */
/*      Each Call Progress sample requires 2 bytes (type and data).     */
/* fsk_index :                                                          */
/*      Offset into the fsk buffer where new data is to be stored.      */
/* cp_index :                                                           */
/*      Offset into call progress buffer where new data is to be stored */
/* cp_get_index :                                                       */
/*      Offset into call progress buffer which which points to the next */
/*      call progress value (field and data) to report to the user.     */
/*                                                                      */
/************************************************************************/

#pragma pack(1)

struct fsk_cp_info {
    unsigned char fsk_data[MAX_FSK_BUF+1];      /* FSK data storage + Null */
    unsigned char cp_data[MAX_CP_BUF];
    int fsk_index;      
    int cp_index;
    int cp_get_index;
};

#pragma pack()
