/* lstruc.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All Rights Reserved
 */

#define TONEBUF_SIZE 50
#define MAX_TELEPH_UNITS 60
#define MAX_ERR_DATA_SIZE 800
#define MAX_ERR_MSG_SIZE 800

#define MAX_DSP_VERS 40
struct firm_ver_info {
    unsigned char v_major;
    unsigned char v_middle;
    unsigned char v_minor;
    unsigned char v_build;
    unsigned short v_auto_num;
    char v_date[20];
    char v_comment[50];
};

/* Structure which contains context information for async data stack.
   This structure comes first, followed by any local data and args
   structures. */
struct async_context_hdr {
    struct async_context_hdr *next_context;
    void *args_ptr;
    void *locals_ptr;
    int timed_out;
    int using_old_args;
    int *func_id_ptr;
    /* Cleanup items */
    void (*cleanup_func)(MILL_LINE *lp,void *args);
    unsigned *cleanup_state_ptr;
    unsigned cleanup_state_value;
    MILL_LINE **cleanup_async_lp_ptr;
};

/* Structure containing info to find lp given session address.
   This implementation just contains a linked list of lps. */
struct async_lp_info {
    struct notif_lp_info {
        MILL_LINE *lp;
        struct notif_lp_info *prev;
        struct notif_lp_info *next;
    } *notif_lp_list_head;
    struct notif_lp_info *notif_lp_list_tail;
};

/* For use with single channel communication */
struct mill_line {
    /* Driver information */
    DR_HANDLE driver_access_handle;
    struct mill_ver driver_ver;
    unsigned session_handle;
    /* Our application session address */
    MILL_ADDR session_addr;
    /* Destination information */
    int api_unit_num;
    MILL_ADDR dest_addr;
    /* Address to use for source address, usually same as session_addr */
    MILL_ADDR source_addr;

    M_CB1 void (* M_CB2 incoming_cmd_func)
         (MILL_LINE *lp,struct args_packet *args);

    void *private_user_data;
    int err_on_deact;
    int ecc_notification_done;

    /* async stack area for locals and arg structures */
    unsigned char *async_stack;

    /* **************************************** */
    /* From this point on gets cleared by reset */
    /* **************************************** */

    enum line_state state;
    unsigned char capabilities[NUM_CAP_BYTES];
    unsigned reset_status;
    struct country_phone_info phone_params;
    struct user_config_params config;
    int cur_mode;

    int disconnect;
#ifndef NO_FSKCP
    struct fsk_cp_info fsk_cp;
#endif
    int final_call_progress;
#ifndef NO_DIGITAL
    AOC_INFO aoc_info;
#endif

    int play_rec_state;
    int play_rec_stopped;
    unsigned play_rec_bytes_processed;
    unsigned speech_start_time;
    unsigned speech_timeout;
    unsigned int play_rec_rate;
    unsigned play_rec_coding_fmt;
    unsigned char play_rec_bps;
    unsigned char play_rec_afe;
    unsigned char play_rec_dfmt;
    unsigned char silence_timeout;
    int caching_in_progress;
    int cache_status;

    int tonedetect_state;
    char tonebuf[TONEBUF_SIZE];
    unsigned tonebuf_in;
    unsigned tonebuf_out;
    unsigned tonebuf_unget;

    char did_digits[MAX_DID];
    unsigned char call_type;
    INFO_RES info_res_buf;
    unsigned char nsf_nss_data_size;
    unsigned char resolution_file;
    unsigned char width_file;
    unsigned char resolution_file_rcv_force;
    unsigned char confirm;
    unsigned char confirm_value;
    unsigned char page_complete;
    struct dis_dtc_info dis_dtc;
    struct dcs_info dcs;
    unsigned int cont_page_cnt;
    FAX_RES *fax_result;
    unsigned char last_fmt;
    unsigned char cur_rcv_fmt;
    unsigned short last_pg_total;
    unsigned short last_pg_bad;
    unsigned page_ct;
    unsigned char need_eop_cmd;
    unsigned char send_alert;
    enum line_state alert_state;
    unsigned char alert_ctl;
    int ioctl_errno;
    M_CB1 void (* M_CB2 page_complete_func)
         (MILL_LINE *lp,int stage,char *arg);
    char *page_complete_arg;
    unsigned char tmp_pkt_buf[TMP_PKT_SIZE];
    unsigned char *saved_data_buf;
    unsigned saved_data_size;
    unsigned char cpga_stopped;
    unsigned char cp_det_mode;
    int cp_last_cpc;
    int cp_last_cm;
    unsigned last_error_value;
    unsigned last_error_data_size;
    unsigned char last_error_data[MAX_ERR_DATA_SIZE];
    char last_error_msg[MAX_ERR_MSG_SIZE];
    unsigned intr_overrun;
    unsigned char g3_pt_enabled;
    unsigned cur_eff_page_type;
    int page_params_set;
    unsigned page_top_margin;
    unsigned page_bot_margin;
    unsigned page_len;
    unsigned page_ascii_pad;
    unsigned page_image_pad;
    unsigned page_image_break;
    unsigned page_image_margin;
    unsigned char is_prism;
    unsigned teleph_port_types[MAX_TELEPH_UNITS];
    int ring_det_ena;

    struct firm_ver_info boot_rom_ver;
    struct firm_ver_info ctrl_proc_ver;
    struct firm_ver_info dsp_vers[MAX_DSP_VERS];
    int num_dsp_vers;

    M_CB1 void (* M_CB2 notify_func)
         (MILL_LINE *lp, unsigned notify_type, struct args_board_notify *args);
    unsigned notifications;

    M_CB1 void (* M_CB2 signaling_func)
         (MILL_LINE *lp, struct args_signaling *args);

    M_CB1 void (* M_CB2 board_test_func)
         (MILL_LINE *lp, struct args_board_test *args);

    int v34_in_use;
    int v34_speed;

    unsigned err_context;
    unsigned err_value;
    unsigned eff_pt_file;
    unsigned can_xmit_cur;
    unsigned can_rcv_cur;

    unsigned vad_state;

    int in_speech_callback;

    /* async related */
    /* current top of stack; stack itself is earlier */
    unsigned char *async_stack_cur;
    /* The current waiting context */
    struct async_context_hdr *waiting_context;
    /* Current maximum depth */
    unsigned async_max_depth;
    /* Current depth */
    unsigned async_cur_depth;
    /* Arg structure of previous call */
    void *async_old_args;
    /* Current async_lp in use */
    MILL_LINE *cur_async_lp;
    /* When set, async is not allowed. For user callback functions. */
    int disallow_async;
    /* Info on sessions which are waiting for notifications
       from this session */
    struct async_lp_info async_lp_info;

#ifndef NO_HOLDUP
    unsigned char holdup_mode;
    unsigned char rcv_mcf_fsk;
    M_CB1 int (* M_CB2 holdup_func)
         (MILL_LINE *lp,char *arg,int section);
    char *holdup_arg;
#endif
#ifndef NO_BFT
    unsigned char cur_bft_file;
    unsigned char cur_bft_mode;
#endif
#ifndef NO_MODESW
    unsigned char mode_switched;
    M_CB1 int (* M_CB2 pri_req_func)(MILL_LINE *lp,char *arg);
    char *pri_req_arg;
    unsigned char cur_pri_val;
#endif
#ifndef NO_OVERLAY
    char overlay_page;
#endif
};
typedef struct mill_line MLINE;

#define _MILL_LINE_SIZE (sizeof(MLINE))
