/*
 * mill_api.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

#ifndef MILL_API_H
#define MILL_API_H

#undef MILL_DO_PROTOS

#define PKTV0_OR_V1    // Allow both packet versions 0 & 1

#include "apiver.h"
#include "api_sys.h"

typedef void MILL_LINE;

enum line_state {
    LINE_STATE_IDLE,
    LINE_STATE_RINGING,
    LINE_STATE_CONNECTED,
    LINE_STATE_RCV_INFO,
    LINE_STATE_AWAIT_TRAINING,
    LINE_STATE_FAX_MODE,
    LINE_STATE_OFF_HOOK,
    LINE_STATE_TURNAROUND,
    LINE_STATE_RESETTING,
    LINE_STATE_HOLDUP,
    LINE_STATE_NOLOOP,
    LINE_STATE_VV,
    LINE_STATE_RETAIN
};

/* Macro to clear a data structure */
#define BT_ZERO(item) memset((char *)&(item),0,sizeof(item))


/* Macro to return the state of the line */
/* The line state will be one of the values in faxcall.h */
#define LINE_STATE(lp) (*MillLineStateAccess(lp))

/* Macro to return the unit number of this channel */
#define LINE_UNIT_NUM(lp) MillLineUnitGet(lp)
/* Macros to return Millennium address information */
#define LINE_DEST_ADDR(lp) MillLineDestAddrGet(lp)
#define LINE_APP_ADDR(lp) MillLineAppAddrGet(lp)
/* Macro to access source Millennium address */
#define LINE_SRC_ADDR(lp) (*MillLineSrcAddrAccess(lp))

#define LINE_AOC_INFO(lp) MillLineAocInfoGet(lp)

/* macro to get/set FAX_RES structure for page result reporting */
#define LINE_FAX_RES(lp) (*MillLineFaxResAccess(lp))

/* macros to examine stored DCS, DIS, or DTC info decoded from FSK msgs */
#define LINE_DCS(lp) MillLineDcsGet(lp)
#define LINE_DIS_DTC(lp) MillLineDisDtcGet(lp)

/* macro to set the procedure interrupt request function and arg */
#define LINE_PRI_REQ_FUNC(lp) (*MillLinePriReqFuncAccess(lp))
#define LINE_PRI_REQ_ARG(lp) (*MillLinePriReqArgAccess(lp))
#define LINE_SET_PRI_REQ_FUNC(lp,func,arg) \
    (LINE_PRI_REQ_FUNC(lp) = (func),LINE_PRI_REQ_ARG(lp) = (arg))

/* macro to set the page complete callback function and arg */
#define LINE_PAGE_COMPLETE_FUNC(lp) (*MillLinePageComplFuncAccess(lp))
#define LINE_PAGE_COMPLETE_ARG(lp) (*MillLinePageComplArgAccess(lp))
#define LINE_SET_PAGE_COMPLETE_FUNC(lp,func,arg) \
    (LINE_PAGE_COMPLETE_FUNC(lp) = (func),LINE_PAGE_COMPLETE_ARG(lp) = (arg))

/* macro to get/set the page count */
#define LINE_PAGE_CT(lp) (*MillLinePageCtAccess(lp))

/* macro to get the vad state and bytes processed */
#define LINE_VAD_STATE(lp) MillLineVadStateGet(lp)
#define LINE_VAD_BYTES_PROCESSED(lp) MillLineVadBPGet(lp)

/* macros to get/set config structures */
#define LINE_CONFIG_STRUCT(lp) MillLineConfigStructAccess(lp)
#define LINE_PHONE_STRUCT(lp) MillLinePhoneStructAccess(lp)

/* Set alert properties */
#define LINE_ALERT_CTL(lp) (*MillLineAlertCtlAccess(lp))

/* macro to get the stored value of the last error interrupt */
#define LINE_ERROR_INTR(lp) MillLineErrorIntrGet(lp)
/* macros to get the stored error data from the last error interrupt */
#define LINE_ERROR_INTR_DATA_SIZE(lp) MillLineErrorIntrDataSizeGet(lp)
#define LINE_ERROR_INTR_DATA(lp) MillLineErrorIntrDataGet(lp)
/* macro to get the stored error message from the last error interrupt */
#define LINE_ERROR_INTR_MSG(lp) MillLineErrorIntrMsgGet(lp)

/* macro to get the stored value of the interrupt overrun status of
   the driver */
#define LINE_INTR_OVERRUN(lp) MillLineIntrOverrunGet(lp)

/* macros to get stored firmware information */
#define LINE_FIRM_MAJOR(lp) LINE_FIRM_CTRL_PROC_MAJOR(lp)
#define LINE_FIRM_MIDDLE(lp) LINE_FIRM_CTRL_PROC_MIDDLE(lp)
#define LINE_FIRM_MINOR(lp) LINE_FIRM_CTRL_PROC_MINOR(lp)
#define LINE_FIRM_DATE(lp) LINE_FIRM_CTRL_PROC_DATE(lp)

#define LINE_FIRM_BOOT_ROM_MAJOR(lp) MillFirmBootRomMajorGet(lp)
#define LINE_FIRM_BOOT_ROM_MIDDLE(lp) MillFirmBootRomMiddleGet(lp)
#define LINE_FIRM_BOOT_ROM_MINOR(lp) MillFirmBootRomMinorGet(lp)
#define LINE_FIRM_BOOT_ROM_BUILD(lp) MillFirmBootRomBuildGet(lp)
#define LINE_FIRM_BOOT_ROM_AUTO_NUM(lp) MillFirmBootRomAutoNumGet(lp)
#define LINE_FIRM_BOOT_ROM_DATE(lp) MillFirmBootRomDateGet(lp)
#define LINE_FIRM_BOOT_ROM_COMMENT(lp) MillFirmBootRomCommentGet(lp)
#define LINE_FIRM_CTRL_PROC_MAJOR(lp) MillFirmCtrlProcMajorGet(lp)
#define LINE_FIRM_CTRL_PROC_MIDDLE(lp) MillFirmCtrlProcMiddleGet(lp)
#define LINE_FIRM_CTRL_PROC_MINOR(lp) MillFirmCtrlProcMinorGet(lp)
#define LINE_FIRM_CTRL_PROC_BUILD(lp) MillFirmCtrlProcBuildGet(lp)
#define LINE_FIRM_CTRL_PROC_AUTO_NUM(lp) MillFirmCtrlProcAutoNumGet(lp)
#define LINE_FIRM_CTRL_PROC_DATE(lp) MillFirmCtrlProcDateGet(lp)
#define LINE_FIRM_CTRL_PROC_COMMENT(lp) MillFirmCtrlProcCommentGet(lp)
#define LINE_FIRM_DSP_MAJOR(lp,n) MillFirmDspMajorGet(lp,n)
#define LINE_FIRM_DSP_MIDDLE(lp,n) MillFirmDspMiddleGet(lp,n)
#define LINE_FIRM_DSP_MINOR(lp,n) MillFirmDspMinorGet(lp,n)
#define LINE_FIRM_DSP_BUILD(lp,n) MillFirmDspBuildGet(lp,n)
#define LINE_FIRM_DSP_AUTO_NUM(lp,n) MillFirmDspAutoNumGet(lp,n)
#define LINE_FIRM_DSP_DATE(lp,n) MillFirmDspDateGet(lp,n)
#define LINE_FIRM_DSP_COMMENT(lp,n) MillFirmDspCommentGet(lp,n)
#define LINE_FIRM_NUM_DSPS(lp) MillFirmNumDspsGet(lp)

#define LINE_FIRM_BITRATE(lp) ((LINE_HAS_CAP(lp,CAP_FAX_XMIT)) ? \
    ((LINE_HAS_CAP(lp,CAP_FAX_V34)) ? 33600 : 14400) : 0)
#define LINE_FIRM_TYPE(lp) 114
#define LINE_FIRM_ID(lp) ""

#define LINE_FIRM_CHECKSUM(lp) 0
#define LINE_FIRM_CHK_OK(lp) 1
#define LINE_FONT_DOWNLOADED(lp,font_no) \
        MillLineFontDownloaded(lp,font_no,NULL)
#define LINE_FONT_DOWNLOADED_ASYNC(lp,font_no,args) \
        MillLineFontDownloaded(lp,font_no,args)

#define LINE_INCOMING_CMD_FUNC(lp) (*MillLineIncomingCmdFuncAccess(lp))
#define LINE_SET_INCOMING_CMD_FUNC(lp,func) (LINE_INCOMING_CMD_FUNC(lp) = func)

#define MILL_LINE_SIZE MillSizeGet()
#define BTLINE_SIZE MILL_LINE_SIZE

#define LINE_PRIVATE_USER_DATA(lp) (*MillLinePrivateUserDataAccess(lp))

#define LINE_DRIVER_VER_LOADED(lp) (MillLineDriverVersionGet(lp))

#define LINE_FAX_T30_RCV_MCF_FSK(lp) MillLineFaxT30RcvMcfFskGet(lp)


#define COMBVER(ver,build) (ver + 100000 * build)
#define COMBINED_API_VER_NUM COMBVER(API_VER_NUM,MILL_API_BUILD_NUM)
#define COMBINED_API_INTRFC_NUM COMBVER(API_VER_NUM,MILL_API_INTRFC_NUM)
#define COMB_SDK_VER(major,middle,minor,build) \
    (((major * 100 + middle) * 100 + minor) * 1000 + build)
#ifndef MILL_LIB_BAD_VARS
#define BT_API_SET_VER() \
    (_mill_api_ver_compiled = COMBINED_API_INTRFC_NUM, \
     _mill_api_ver_bld_comp = COMBINED_API_VER_NUM, \
     _mill_api_ver_auto_comp = SDK_AUTO_NUMBER)
#define LINE_API_VER_LOADED() _mill_api_lib_ver_compiled
#define LINE_SDK_VER_LOADED() _mill_sdk_ver_compiled
#define LINE_SDK_AUTO_LOADED() _mill_sdk_auto_compiled
#else
#define BT_API_SET_VER() \
    _mill_set_api_ver_compiled(COMBINED_API_INTRFC_NUM, \
                               COMBINED_API_VER_NUM, \
                               SDK_AUTO_NUMBER)
M_APIE1 void M_APIE2
_mill_set_api_ver_compiled(intrfc_ver, int api_ver, int sdk_auto);
#define LINE_API_VER_LOADED() _mill_api_lib_ver_compiled_func()
M_APIE1 int M_APIE2 _mill_api_lib_ver_compiled_func(void);
#define LINE_SDK_VER_LOADED() _mill_sdk_ver_compiled_func()
M_APIE1 int M_APIE2 _mill_sdk_ver_compiled_func(void);
#define LINE_SDK_AUTO_LOADED() _mill_sdk_auto_compiled_func()
M_APIE1 int M_APIE2 _mill_sdk_auto_compiled_func(void);
#endif

/* Reset status condition bit values, as returned by BfvLineReset
   in the reset_status field. Those marked as fatal will cause BfvLineReset
   to return -1. */
/* Timeout occurred waiting for reset done interrupt (fatal) */
#define RST_RSTDONE_TIMEOUT             0x00000001L
/* Bad magic number (fatal) */
#define RST_MAGIC_BAD                   0x00000002L
/* The reset occurred, but the channel reported that it has no modem */
#define RST_RSTDONE_NOMODEM             0x00000004L
/* The user configuration file could not be opened/read */
#define RST_USR_CONFIG_UNREADABLE       0x00000008L
/* The user configuration file is not in the correct format */
#define RST_USR_CONFIG_BADFORMAT        0x00000010L
/* The country parameter file could not be opened/read, possible bad format */
#define RST_CPARM_UNREADABLE            0x00000020L
/* The country specified in the user configuration file could not be found */
#define RST_COUNTRY_NOTFOUND            0x00000040L
/* The firmware file specified in the user configuration file could not
   be opened/read (fatal) */
#define RST_FIRM_UNREADABLE             0x00000080L
/* An error occurred during firmware download (fatal) */
#define RST_FIRM_DLOAD_ERR              0x00000100L
/* A font file specified in the user configuration file could not be
   opened/read */
#define RST_FONT_UNREADABLE             0x00000200L
/* An error occurred during font download (fatal) */
#define RST_FONT_DLOAD_ERR              0x00000400L
/* The DAA bytes are not valid */
#define RST_DAA_BAD                     0x00000800L
/* A DID channel does not have a battery connected */
#define RST_DID_BATT_MISSING            0x00001000L
/* The hardware version that the channel reported is not valid */
#define RST_HW_VERSION_BAD              0x00002000L
/* The call control configuration file specified in the
   user configuration file could not be opened/read */
#define RST_CCTRL_CONFIG_UNREADABLE     0x00004000L
/* The call control configuration file specified in the user
   configuration file is not in the correct format (fatal) */
#define RST_CCTRL_CONFIG_BADFORMAT      0x00008000L
/* A hardware error occurred during call control configuration (fatal) */
#define RST_CCTRL_HW_ERR                0x00010000L
/* A memory allocation error occurred */
#define RST_MEM_ALLOC_ERR               0x00020000L
/* An alert occurred (fatal) */
#define RST_ALERT                       0x00040000L
/* An error interrupt or interrupt overrun occurred (fatal) */
#define RST_INT_ERR_OVRN                0x00080000L
/* The driver reported an error during reset or initialization (fatal) */
#define RST_DRIVER_ERR                  0x00100000L
/* Incorrect configuration parameters were specified, as reported by firmware,
   during call control configuration (fatal) */
#define RST_CCTRL_PARAM_ERR             0x00200000L
/* Channel reported a diagnostic error of some kind */
#define RST_DIAG_ERR                    0x00400000L
/* Maximum timeout exceeded (fatal) */
#define RST_MAX_TIMEOUT                 0x00800000L
/* Other call control configuration error (fatal) */
#define RST_CCTRL_ERR_OTHER             0x01000000L
/* Error due to channel state retention attempt. (fatal) */
#define RST_RETAIN_ERR                  0x02000000L
/* Invalid destination address for packet (fatal) */
#define RST_INVALID_DEST_ADDR           0x04000000L
/* Other packet send/receive error (fatal) */
#define RST_PACKET_SEND_RCV             0x08000000L
/* Packet content error (parse/form) (fatal) */
#define RST_PACKET_CONTENT_ERR          0x10000000L
/* Other unclassified error (fatal) */
#define RST_UNCLASSIFIED_ERR            0x20000000L
/* Firmware detected an error */
#define RST_FIRM_DETECTED_ERR           0x40000000L
/* A font file specified was already downloaded */
#define RST_FONT_DOWNLOADED             0x80000000L

/* call_protocol_code argument of BfvLineOriginateCall() and
   BfvLineCallProgressEnable() and BfvLineOrigCallDB() */
#define CALL_PROTOCOL_FAX               1
#define CALL_PROTOCOL_VOICE             2
#define CALL_PROTOCOL_RAW               3
#define CALL_PROTOCOL_FAX_NO_RAW        4
#define CALL_PROTOCOL_VOICE_NO_RAW      5

/* oa_flag argument of BfvFaxPoll() and BfvLineCallProgressEnable() */
#define BT_ANSWER               90
#define BT_ORIGINATE            99

/* BfvToneDetectEnable() flags */
#define DTMF_12TONE             12
#define DTMF_16TONE             16
#define TONE_R1                 0x11
#define TONE_R2_FORWARD         0x12
#define TONE_R2_BACKWARD        0x13
#define TONE_REPORT_END         0x80
#define TONE_REPORT_VV          0x40
#define TONE_REPORT_MASK        0x1f

#define DTMF_TONE               90
#define MONO_TONE               91

/* CALL_RES.call_type values */
#define CALL_TYPE_DID   1
#define CALL_TYPE_RING  2
#define CALL_TYPE_LOCAL 3
#define CALL_TYPE_ISDN  4

/* RES.status values */
#define BT_STATUS_OK                    0
#define BT_STATUS_ERROR                 2
#define BT_STATUS_ERROR_DIAL            3
#define BT_STATUS_ERROR_HANGUP          4
#define BT_STATUS_USER_TERMINATED       5
#define BT_STATUS_TIMEOUT               6
#define BT_STATUS_ALERT                 7
#define BT_STATUS_VV                    8
#define BT_STATUS_CONTINUING            9

#if 0
#define BT_STATUS_AUTO_TERMINATED       10
#endif

/* RES.line_status values */
/* if RES.status == BT_STATUS_ERROR_HANGUP (0 thru 256 reserved) */
#define HNG_NORMAL_XMIT         FAX_TERM_NORMAL_XMIT_DEF        /* 0 */
#define HNG_RNG_DET             FAX_TERM_RNG_DET_DEF            /* 1 */
#define HNG_ABORT               FAX_TERM_ABORT_DEF              /* 2 */
#define HNG_NO_LOOP_CURRENT     3
#define HNG_ISDN_DISCONNECT     4
#define HNG_T1_TIMEOUT          FAX_TERM_T1_TIMEOUT_DEF         /* 11 */
#define HNG_XMITB_TIMEOUT       FAX_TERM_XMITB_TIMEOUT_DEF      /* 20 */
#define HNG_XMITB_NORM          FAX_TERM_XMITB_NORM_DEF         /* 21 */
#define HNG_XMITB_MISC          FAX_TERM_XMITB_MISC_DEF         /* 22 */
#define HNG_XMITB_COMREC_VCNR   FAX_TERM_XMITB_COMREC_VCNR_DEF  /* 23 */
#define HNG_XMITB_SE            FAX_TERM_XMITB_SE_DEF           /* 24 */
#define HNG_XMITB_DCS_FTC       FAX_TERM_XMITB_DCS_FTC_DEF      /* 25 */
#define HNG_XMITB_DIS_FTC       FAX_TERM_XMITB_DIS_FTC_DEF      /* 26 */
#define HNG_XMITB_TRAINFAIL     FAX_TERM_XMITB_TRAINFAIL_DEF    /* 27 */
#define HNG_XMITB_RSPREC_VCNR   FAX_TERM_XMITB_RSPREC_VCNR_DEF  /* 28 */
#define HNG_XMITB_COMREC_DCN    FAX_TERM_XMITB_COMREC_DCN_DEF   /* 29 */
#define HNG_XMITB_RSPREC_DCN    FAX_TERM_XMITB_RSPREC_DCN_DEF   /* 30 */
#define HNG_PHASEB_INCOMPAT_FMT FAX_TERM_PHASEB_INCOMPAT_FMT_DEF/* 33 */
#define HNG_XMITB_INCOMPAT_FMT  HNG_PHASEB_INCOMPAT_FMT
#define HNG_XMITB_INVAL_DMACNT  34
#define HNG_XMITB_FTM_NOECM     FAX_TERM_XMITB_FTM_NOECM_DEF    /* 35 */
#define HNG_XMITB_INCMP_FTM     FAX_TERM_XMITB_INCMP_FTM_DEF    /* 36 */
#define HNG_XMITB_INCMP_EFF     FAX_TERM_XMITB_INCMP_EFF_DEF    /* 37 */
#define HNG_XMITB_NOEFF         FAX_TERM_XMITB_NOEFF_DEF        /* 38 */
#define HNG_XMITD_RR_NORES      FAX_TERM_XMITD_RR_NORES_DEF     /* 40 */
#define HNG_XMITD_CTC_NORES     FAX_TERM_XMITD_CTC_NORES_DEF    /* 41 */
#define HNG_XMITD_T5TO_RR       FAX_TERM_XMITD_T5TO_RR_DEF      /* 42 */
#define HNG_XMITD_NOCONT_NSTMSG FAX_TERM_XMITD_NOCONT_NSTMSG_DEF/* 43 */
#define HNG_XMITD_ERRRES_EOREOP FAX_TERM_XMITD_ERRRES_EOREOP_DEF/* 44 */
#define HNG_XMITD_RTN_DCN       FAX_TERM_XMITD_RTN_DCN_DEF      /* 45 */
#define HNG_XMITD_PPR_EOR       FAX_TERM_XMITD_PPR_EOR_DEF      /* 46 */
#define HNG_XMITD_SE            FAX_TERM_XMITD_SE_DEF           /* 51 */
#define HNG_XMITD_MPS_FTC       FAX_TERM_XMITD_MPS_FTC_DEF      /* 52 */
#define HNG_XMITD_MPS_VCNR      FAX_TERM_XMITD_MPS_VCNR_DEF     /* 53 */
#define HNG_XMITD_EOP_FTC       FAX_TERM_XMITD_EOP_FTC_DEF      /* 54 */
#define HNG_XMITD_EOP_VCNR      FAX_TERM_XMITD_EOP_VCNR_DEF     /* 55 */
#define HNG_XMITD_EOM_FTC       FAX_TERM_XMITD_EOM_FTC_DEF      /* 56 */
#define HNG_XMITD_EOM_VCNR      FAX_TERM_XMITD_EOM_VCNR_DEF     /* 57 */
#define HNG_XMITD_RSPREC_DCN    FAX_TERM_XMITD_RSPREC_DCN_DEF   /* 60 */
#define HNG_XMITD_PPSNULL_NORES FAX_TERM_XMITD_PPSNULL_NORES_DEF/* 61 */
#define HNG_XMITD_PPSMPS_NORES  FAX_TERM_XMITD_PPSMPS_NORES_DEF /* 62 */
#define HNG_XMITD_PPSEOP_NORES  FAX_TERM_XMITD_PPSEOP_NORES_DEF /* 63 */
#define HNG_XMITD_PPSEOM_NORES  FAX_TERM_XMITD_PPSEOM_NORES_DEF /* 64 */
#define HNG_XMITD_EORNULL_NORES FAX_TERM_XMITD_EORNULL_NORES_DEF/* 65 */
#define HNG_XMITD_EORMPS_NORES  FAX_TERM_XMITD_EORMPS_NORES_DEF /* 66 */
#define HNG_XMITD_EOREOP_NORES  FAX_TERM_XMITD_EOREOP_NORES_DEF /* 67 */
#define HNG_XMITD_EOREOM_NORES  FAX_TERM_XMITD_EOREOM_NORES_DEF /* 68 */
#define HNG_RCVB_TIMEOUT        FAX_TERM_RCVB_TIMEOUT_DEF       /* 70 */
#define HNG_RCVB_SE             FAX_TERM_RCVB_SE_DEF            /* 71 */
#define HNG_RCVB_MISC           FAX_TERM_RCVB_MISC_DEF          /* 72 */
#define HNG_T2_PNOTREC          FAX_TERM_T2_PNOTREC_DEF         /* 73 */
#define HNG_RCVB_T1_TIMEOUT     FAX_TERM_RCVB_T1_TIMEOUT_DEF    /* 74 */
#define HNG_NORMAL_RCV          FAX_TERM_NORMAL_RCV_DEF         /* 75 */
#define HNG_RCVB_RSPREC_DCN     FAX_TERM_RCVB_RSPREC_DCN_DEF    /* 76 */
#define HNG_T2_TIMEOUT          FAX_TERM_T2_TIMEOUT_DEF         /* 77 */
#define HNG_RCVB_INVAL_DMACNT   78
#define HNG_RCVB_FTM_NOECM      FAX_TERM_RCVB_FTM_NOECM_DEF     /* 79 */
#define HNG_RCVD_SE_VCNR        FAX_TERM_RCVD_SE_VCNR_DEF       /* 101 */
#define HNG_RCVD_COMREC_VCNR    FAX_TERM_RCVD_COMREC_VCNR_DEF   /* 102 */
#define HNG_RCVD_T3TO_NORESP    FAX_TERM_RCVD_T3TO_NORESP_DEF   /* 103 */
#define HNG_RCVD_T2TO           FAX_TERM_RCVD_T2TO_DEF          /* 104 */
#define HNG_RCVD_DCN_COMREC     FAX_TERM_RCVD_DCN_COMREC_DEF    /* 105 */
#define HNG_RCVD_COMREC_ERR     FAX_TERM_RCVD_COMREC_ERR_DEF    /* 106 */
#define HNG_RCVD_BLKCT_ERR      FAX_TERM_RCVD_BLKCT_ERR_DEF     /* 107 */
#define HNG_RCVD_PGCT_ERR       FAX_TERM_RCVD_PGCT_ERR_DEF      /* 108 */
#define HNG_RCVD_EOR            FAX_TERM_RCVD_EOR_DEF           /* 109 */
#define HNG_RCVD_RNRTO          FAX_TERM_RCVD_RNRTO_DEF         /* 110 */
#define HNG_RCVC_EOL_TIMEOUT    FAX_TERM_RCVC_EOL_TIMEOUT_DEF   /* 150 */
#define HNG_RCVC_BAD_MMR        FAX_TERM_RCVC_BAD_MMR_DEF       /* 151 */
#define HNG_INTERRUPT_ACK       240
#define HNG_COMM_FAULT          241
#define HNG_T30_HOLDUP          FAX_TERM_T30_HOLDUP_DEF         /* 242 */
#define HNG_HOLDUP_DCN          FAX_TERM_HOLDUP_DCN_DEF         /* 243 */
#define HNG_HOLDUP_DCN_NON_FPAD FAX_TERM_HOLDUP_DCN_NON_FPAD_DEF/* 244 */
/* Fake hangup values, used for other errors of similar severity level */
#define HNG_ERROR_INTERRUPT     500
#define HNG_INTERRUPT_OVERRUN   501
#define HNG_UNEXPECTED_IRSDONE  502
#define HNG_IOCTL_ERROR         503
#define HNG_OVERLAY_DLOAD_ERR   504
#define HNG_MAX_TIMEOUT         505
/* Reserved for internal use */
#define HNG_RESERVED_1          900

/* RES.line_status values */
/* if RES.status == BT_STATUS_ERROR_DIAL */
#define DIAL_OK                 257     /* Also possible for BT_STATUS_OK */
#define DIAL_NO_DIAL_TONE       258
#define DIAL_NO_LOOP_CUR        259
#define DIAL_LOCAL_IN_USE       260
#define DIAL_TRUNK_BUSY         261
#define DIAL_SLOT_BUSY          265
#define DIAL_CALL_COLLISION     266
#define DIAL_NO_WINK            267
#define DIAL_ISDN_INVAL         268
#define DIAL_JP_REDIAL_FAIL     269
#define DIAL_UNKNOWN            500
#define MAX_KNOWN_DIAL_RESULT   0xc

/* RES.line_status values */
/* if RES.status == BT_STATUS_ERROR */
#define APIERR_UNCLASSIFIED             0
/* An unclassified error */
#define APIERR_FILEIO                   1
/* A file open/read/write error */
#define APIERR_FILEFORMAT               2
/* A file is not in the correct format */
#define APIERR_BOARDCAPABILITY          3
/* The hardware or firmware does not have a required capability */
#define APIERR_NOTCONNECTED             4
/* The channel is not in a connected state, but is required to be */
#define APIERR_BADPARAMETER             5
/* An invalid parameter has been passed */
#define APIERR_MEMORY                   6
/* A memory allocation error occurred */
#define APIERR_BADSTATE                 7
/* The channel is not in a required state */
#define APIERR_TOOSOON                  8
/* Dialing was attempted too soon */
#define APIERR_BUSY                     9
/* Resource busy */
#define APIERR_DRV_OPEN_ERROR           10
/* Driver open call failed, invalid channel or driver
   not correctly installed */
#define APIERR_DRV_IOCTL_ERROR          11
/* Driver call failed */
#define APIERR_VERSION                  12
/* Incompatible driver or DLL version */
#define APIERR_INVALID_PORT             13
/* Invalid port; faxinit probably not run correctly */
#define APIERR_RINGING                  14
/* Ringing during dialing attempt */
#define APIERR_INFOPKT_NESTING          15
/* Indir infopkt nesting level too deep */
#define APIERR_MAX_TAGS                 16
/* Maximum number of TIFF tags exceeded */
#define APIERR_LOCK_FAILED              17
/* An attempt to gain a lock failed */
#define APIERR_INSUFF_BUFFER            18
/* Buffer size too small to receive data */
#define APIERR_INVALID_DEST_ADDR        19
/* Destination address not found or invalid */
#define APIERR_PACKET_CREATION          20
/* Packet or command creation error */
#define APIERR_PACKET_PARSE             21
/* Packet or command parse error */
#define APIERR_PACKET_SEND              22
/* Packet send error */
#define APIERR_PACKET_RECEIVE           23
/* Packet receive error */
#define APIERR_DATA                     24
/* DATA encountered during command processing */
#define APIERR_INVAL_BOARD_PARAM        25
/* Invalid parameter values received from board */
#define APIERR_FIRMWARE_ERR_DETECTED    26
/* Firmware detected an error */
#define APIERR_MODULE_REMOVED           27
/* Module was removed */
#define APIERR_BOARD_NO_RESPONSE        28
/* Board not responding */
#define APIERR_ASYNC_LP_ERR             29
/* Async_lp value error */
#define APIERR_ASYNC_CONTEXT_ERR        30
/* Async context error */
#define APIERR_DRV_RESOURCES            31
/* Driver out of resources */
#define APIERR_MODULE_RESET_FAILURE     32
/* Module reset failure */
#define APIERR_MODULE_I2O_FAILURE       33
/* Module I2O enable failure */
#define APIERR_MODULE_CONFIG_TIMEOUT    34
/* No valid response for module configuration */
#define APIERR_NO_TRANSPORT             35
/* No transport layer */
#define APIERR_NON_SUPPORTED_FEATURE    36
/* Non supported feature */

/* RES.line_status values */
/* if RES.status == BT_STATUS_TIMEOUT */
#define API_TO_TIME             0
#define API_TO_SILENCE          1


#if 0
/* RES.line_status values */
/* if RES.status == BT_STATUS_AUTO_TERMINATED */
#define API_AUTOTERM_VAD        1
#endif

/* final call progress results */
#define FCP_BASE                300     /* Not a CP value */
#define FCP_BUSY1               301
#define FCP_BUSY2               302
#define FCP_ROBUSY              303
#define FCP_RECALL              304
#define FCP_CONFIRM             305
#define FCP_PULSE               306
#define FCP_RING1               308
#define FCP_RING2               309
#define FCP_HUMAN               316
#define FCP_ANSWER              317
#define FCP_DIALTON             318
#define FCP_SILENCE             324
#define FCP_RNGNOANS            325
#define FCP_G2DETCT             326
#define FCP_SITINTC             327
#define FCP_QUIET               328
#define FCP_SITVACODE           329
#define FCP_SITREORD            330
#define FCP_SITNOCIR            331
#define FCP_CNGDETCT            332
#define FCP_RMTOFFHK            333
#define FCP_SPECIALCP           334
#define FCP_ANSWER_TONE_DETECT  339 /* Doesn't correspond to CS_ value */
#define FCP_UNKNOWN             340 /* Doesn't correspond to CS_ value */
#define FCP_ISDN_CALL_PROGRESS  348
#define FCP_ISDN_CALL_COLLISION 349


/* RES.line_status values for BfvLineWaitForCall() */
#define WAIT_FOR_CALL_OK                352
#define WAIT_FOR_CALL_TIMEOUT           353

/* RES.line_status values for BfvLineWaitDivertCall() */
#define DIVERT_REJECTED           300
#define DIVERT_TO_TIME            301

/* FSK values returned by BfvDataFSK() */
/* See tr.h */

/*
 * CCITT protocol tones
 */  
#define TONEID_CCITT_1  0       /* 462 Hz */
#define TONEID_CCITT_2  1       /* 1100 Hz */
#define TONEID_CCITT_3  2       /* 1650 Hz */
#define TONEID_CCITT_4  3       /* 1850 Hz */
#define TONEID_CCITT_5  4       /* 2100 Hz */

/*
 * DTMF tones
 */
#define TONEID_697_HZ   7       /* DTMF row 1 (123A) */
#define TONEID_770_HZ   13      /* DTMF row 2 (456B) */
#define TONEID_852_HZ   19      /* DTMF row 3 (789C) */
#define TONEID_941_HZ   5       /* DTMF row 4 (*0#D) */
#define TONEID_1209_HZ  8       /* DTMF col 1 (147*) */
#define TONEID_1336_HZ  10      /* DTMF col 2 (2580) */
#define TONEID_1477_HZ  12      /* DTMF col 3 (369#) */
#define TONEID_1633_HZ  6       /* DTMF col 4 (ABCD) */

/* Call progress signal generation values */
#define CP_GEN_DIALTONE 0
#define CP_GEN_RING     1
#define CP_GEN_BUSY     2
#define CP_GEN_REORDER  20
#define CP_GEN_END      0x80

/* Resolutions */
#define RES_200H_100V   0
#define RES_200H_200V   1
#define RES_200H_400V   2
#define RES_300H_300V   3
#define RES_400H_400V   4
#define RES_NORMAL      RES_200H_100V
#define RES_FINE        RES_200H_200V
#define RES_600H_600V   5
#define RES_1200H_1200V 6
#define RES_300H_600V   7
#define RES_400H_800V   8
#define RES_600H_1200V  9
#define RES_100H_100V  10
/* NOTE: non-square pixels can NOT be used for JPEG */

/* Widths */
#define WIDTH_A4        0
#define WIDTH_B4        1
#define WIDTH_A3        2

/* Standard speech rate */
#define SPEECH_NORMAL_RATE 20000

/* Fax send/receive data formats */
#define FMT_MH_UNALIGN_MSB  0x00 /* MH data,  EOLS not byte-align, MSB order */
#define FMT_MH_UNALIGN_LSB  0x01 /* MH data,  EOLS not byte-align, LSB order */
#define FMT_MH_ALIGN_MSB    0x02 /* MH data,  EOLS byte-align,     MSB order */
#define FMT_MH_ALIGN_LSB    0x03 /* MH data,  EOLS byte-align,     LSB order */
#define FMT_MR_UNALIGN_MSB  0x04 /* MR data,  EOLS not byte-align, MSB order */
#define FMT_MR_UNALIGN_LSB  0x05 /* MR data,  EOLS not byte-align, LSB order */
#define FMT_MR_ALIGN_MSB    0x06 /* MR data,  EOLS byte-align,     MSB order */
#define FMT_MR_ALIGN_LSB    0x07 /* MR data,  EOLS byte-align,     LSB order */
#define FMT_MMR_UNALIGN_MSB 0x10 /* MMR data, EOLS not byte-align, MSB order */
#define FMT_MMR_UNALIGN_LSB 0x11 /* MMR data, EOLS not byte-align, LSB order */
#define FMT_MMR_ALIGN_MSB   0x12 /* MMR data, EOLS byte-align,     MSB order */
#define FMT_MMR_ALIGN_LSB   0x13 /* MMR data, EOLS byte-align,     LSB order */
#define FMT_ASCII           0x08 /* Ascii data */
#define FMT_PCX_BILEVEL     0x09 /* PCX data */
#define FMT_BINARY          0x0a /* Binary data - reserved, not used */
#define FMT_SPACE           0x0b /* Space, used internally */
#define FMT_PASS_THROUGH_LSB 0x16 /* Pass through LSB, used internally */
#define FMT_PASS_THROUGH_MSB 0x17 /* Pass through MSB, used internally */
#define FMT_JPEG            0xf0 /* JPEG, future use */
#define FMT_JBIG            0xf1 /* JBIG, future use */
/* Old data fmt types */
#define DATA_G3         FMT_MH_UNALIGN_MSB
#define DATA_ASCII      FMT_ASCII

/* Defines for BFT types */
#define BFT_NONE        0
#define BFT_T434        1
#define BFT_BTM         2
#define BFT_DTM         4
#define BFT_EDIFACT     8


/* Bit rate definitions for BfvFaxT30Params and t30parampkt */
#define BITRATE_33600   FAX_BITRATE_33600_DEF
#define BITRATE_31200   FAX_BITRATE_31200_DEF
#define BITRATE_28800   FAX_BITRATE_28800_DEF
#define BITRATE_26400   FAX_BITRATE_26400_DEF
#define BITRATE_24000   FAX_BITRATE_24000_DEF
#define BITRATE_21600   FAX_BITRATE_21600_DEF
#define BITRATE_19200   FAX_BITRATE_19200_DEF
#define BITRATE_16800   FAX_BITRATE_16800_DEF
#define BITRATE_14400   FAX_BITRATE_14400_DEF
#define BITRATE_12000   FAX_BITRATE_12000_DEF
#define BITRATE_9600    FAX_BITRATE_9600_DEF
#define BITRATE_7200    FAX_BITRATE_7200_DEF
#define BITRATE_4800    FAX_BITRATE_4800_DEF
#define BITRATE_2400    FAX_BITRATE_2400_DEF
#define BITRATE_GENERIC FAX_BITRATE_GENERIC_DEF

/* Scan time definitions */
#define SCANTIME_0      FAX_SCAN_TIME_0_DEF
#define SCANTIME_5      FAX_SCAN_TIME_5_DEF
#define SCANTIME_10     FAX_SCAN_TIME_10_DEF
#define SCANTIME_20     FAX_SCAN_TIME_20_DEF
#define SCANTIME_40     FAX_SCAN_TIME_40_DEF

/* Modulation type definitions for BfvFaxT30Params and t30parampkt */
#define MODULATION_GENERIC      FAX_MODULATION_GENERIC_DEF
#define MODULATION_V27          FAX_MODULATION_V27_DEF
#define MODULATION_V29          FAX_MODULATION_V29_DEF
#define MODULATION_V33          FAX_MODULATION_V33_DEF
#define MODULATION_V17          FAX_MODULATION_V17_DEF
#define MODULATION_V34          FAX_MODULATION_V34_DEF

/* Line_compression definitions for BfvFaxT30Params and t30parampkt */
#define LINECOMPR_ANY           0
#define LINECOMPR_MH            1
#define LINECOMPR_MR_MH         2
#define LINECOMPR_MMR_MR_MH     3

/* Bit rate definitions for page result structures and dcs_info */
#define BITRATE_RSLT_33600_V34  23
#define BITRATE_RSLT_31200_V34  22
#define BITRATE_RSLT_28800_V34  21
#define BITRATE_RSLT_26400_V34  20
#define BITRATE_RSLT_24000_V34  19
#define BITRATE_RSLT_21600_V34  18
#define BITRATE_RSLT_19200_V34  17
#define BITRATE_RSLT_16800_V34  16
#define BITRATE_RSLT_14400_V34  15
#define BITRATE_RSLT_12000_V34  14
#define BITRATE_RSLT_9600_V34   13
#define BITRATE_RSLT_7200_V34   12
#define BITRATE_RSLT_4800_V34   11
#define BITRATE_RSLT_2400_V34   10

#define BITRATE_RSLT_14400_V33  9
#define BITRATE_RSLT_12000_V33  8
#define BITRATE_RSLT_14400_V17  7
#define BITRATE_RSLT_12000_V17  6
#define BITRATE_RSLT_9600_V17   5
#define BITRATE_RSLT_7200_V17   4
#define BITRATE_RSLT_9600_V29   3
#define BITRATE_RSLT_7200_V29   2
#define BITRATE_RSLT_4800_V27   1
#define BITRATE_RSLT_2400_V27   0

/* Fax mode switching definitions */
#define MODE_FAX                1
#define MODE_VOICE              2
#define MODE_VOICE_IMMED        3
#define MODE_PRI_REQ            4
#define MODE_PRI_REQ_IMMED      5
#define MODE_PRI_REQ_END        6

/* Speech coding formats */
#define CODE_CVSD       0
#define CODE_ADPCM      1
#define CODE_PCM_ULAW   2
#define CODE_PCM_ALAW   3
#define CODE_LINEAR     4
#define CODE_SUBBAND    5
#define CODE_OKI_ADPCM  6
#define CODE_G723_1     7
#define CODE_G729_A     8
#define CODE_SX7300     9
#define CODE_SX9600     10
#define CODE_RHET_ADPCM 11
#define CODE_MS_GSM     12
#define CODE_DVI_ADPCM  13
#define CODE_GSM_610    14
#define CODE_GSM_660    15
#define CODE_RAWDATA    16
#define CODE_G726       17
#define CODE_CNID_FSK   30

/* Speech bit rates */
#define RATE_6000       0
#define RATE_8000       1
#define RATE_20000      2
#define RATE_24000      3
#define RATE_28000      4
#define RATE_32000      5
#define RATE_16000      6
#define RATE_9600       7
#define RATE_11000      8
#define RATE_5300       9
#define RATE_6300       10
#define RATE_13000      11
#define RATE_12300      12
#define RATE_STOP       0xffff

/* Speech bits per sample */
#define BITS_1  0
#define BITS_3  1
#define BITS_4  2
#define BITS_8  3
#define BITS_12 4
#define BITS_16 5
#define BITS_24 6
#define BITS_2  7
#define BITS_5  8

/* Speech AFE sampling rates */
#define AFE_8000        0
#define AFE_9600        1

/* Speech data formats masks */
#define SPCH_MSB        0
#define SPCH_LSB        1

/* Speech modification values */
#define SPCH_MOD_GAIN_UP        2
#define SPCH_MOD_GAIN_DOWN      3
#define SPCH_MOD_RATE_UP        4
#define SPCH_MOD_RATE_DOWN      5
#define SPCH_MOD_GAIN_SET       6
#define SPCH_MOD_RATE_SET       7

/* Defines for use with PROMPT_MAP structures */
#define PROMPT_IPS(prompt_map) ((prompt_map)->ips)
#define PROMPT_NUM_PHRASES(prompt_map) ((prompt_map)->promptpkt.num_phrases)
#define PROMPT_PHRASE_OFFSET(prompt_map,phrase_num) \
    ((prompt_map)->phrases[phrase_num].offset)
#define PROMPT_PHRASE_SIZE(prompt_map,phrase_num) \
    ((prompt_map)->phrases[phrase_num].phrase_size)


/* Fax header placement constants */
#define HDR_HEADER 0
#define HDR_FOOTER 1

/* Fax header insertion mode constants */
#define HDR_MODE_DISABLE 0x00
#define HDR_MODE_OVERLAY 0x01
#define HDR_MODE_REPLACE 0x02
#define HDR_MODE_INSERT  0x03

/* Xmit & Receive mode constants */
#define XMIT_MODE_AUTO          0
#define XMIT_MODE_MANUAL        1
#define RCV_MODE_AUTO           0
#define RCV_MODE_MANUAL         1

/* T30 holdup mode constants */
#define T30_HOLDUP_NONE         0
#define T30_HOLDUP_DCS          1
#define T30_HOLDUP_RCV_MCF      2

/* T30 holdup section constants */
#define T30H_SECTION_XMIT_DCS   5
#define T30H_SECTION_RCV_DCS    6
#define T30H_SECTION_RCV_MCF    7

/* NSF/NSS constants */
#define SET_NSF_NSC     1
#define SET_NSS         2

/* Memory allocation mem_type values */
#define BTMEM_UNKNOWN           0
/* Unknown type, unknown size. */
#define BTMEM_BTLINE            1
/* A MILL_LINE structure. Size is size of an internally defined line struct. */
#define BTMEM_AVAIL             2
/* Used by BfvLinesAvail. Size is currently fixed at 100 bytes. */
#define BTMEM_FONTNAME          3
/* Used during BfvLineReset and BfvLineConfig to store font filenames.
   Size variable. */
#define BTMEM_DH                4
/* Used by dump_history, dump_history_unit, and dump_history_all.
   Size will depend on configured history size and which function is called. */
#define BTMEM_IPS               5
/* An infopkt stream pointer. Size is sizeof(struct infopkt_stream). */
#define BTMEM_IPSBUF            6
/* An infopkt buffer. Size variable based on infopkts in stream. */
#define BTMEM_IPSMEMPKT         7
/* A structure for memory infopkts. Size is sizeof(struct mem_infopkt_info). */
#define BTMEM_TFILE             8
/* A TFILE structure. Size is sizeof(TFILE). */
#define BTMEM_TIFFIFD           9
/* A buffer used to store a TIFF IFD, currently allows for 40 IFD entries.
   Size is 40 * sizeof(struct ifd_field). */
#define BTMEM_PAGERES           10
/* A PAGE_RES structure. Size is sizeof(PAGE_RES). */
#define BTMEM_PROMPTMAP         11
/* A PROMPT_MAP structure. Size is sizeof(PROMPT_MAP). */
#define BTMEM_PHRASES           12
/* A buffer for phrase information. Size depends upon the number of phrases
   in a prompt file. */
#define BTMEM_SETBUF            13
/* A buffer used for the stdio fread/fwrite functions. Size is either BUFSIZ,
   defined in stdio.h, or 12 (for an associated API strcture). */
#define BTMEM_DOS_THREAD        14
/* A buffer used internally for MSDOS thread management. Size is 24 bytes. */
#define BTMEM_DOS_STACK         15
/* An MSDOS thread stack. Size is equal to the value of _thread_stack_size. */
#define BTMEM_T434STR           16
/* A buffer allocated for a string read from a T.434 BFT file when
   calling BfvT434ParseFile. Size is the size of the string in the file. */
#define BTMEM_ASYNC_STACK       17
/* Stack for async function calls. */

/* Values for use with BfvFaxOverlay */
#define BT_OVERLAY_DOWNLOAD             1
#define BT_OVERLAY_RELEASE              2
#define BT_OVERLAY_OR                   3

/* Values for use with BfvFaxSpace and the spacepkt infopkt */
#define UNITS_INCHES_TENTHS             0
#define UNITS_SCAN_LINES                1

/* ISDN AOC status */
#define BT_AOC_UNAVAILABLE      0
#define BT_AOC_FREE             1
#define BT_AOC_UNITS            2
#define BT_AOC_CURRENCY         3

/* ISDN AOC currency */
#define BT_AOC_CUR_THOUSANDTH   0
#define BT_AOC_CUR_HUNDREDTH    1
#define BT_AOC_CUR_TENTH        2
#define BT_AOC_CUR_ONE          3
#define BT_AOC_CUR_TEN          4
#define BT_AOC_CUR_HUNDRED      5
#define BT_AOC_CUR_THOUSAND     6

/* ISDN Redirection reasons */
#define BT_REDIR_UNKNOWN        0
#define BT_REDIR_CALL_FWD_BUSY  0x1
#define BT_REDIR_CALL_FWD_NOANS 0x2
#define BT_REDIR_OOS            0x5
#define BT_REDIR_CALL_FWD_DTE   0xa
#define BT_REDIR_CALL_FWD_ALL   0xf

/* Enhanced fax format type values */
/* These values correspond directly to DIS/DCS bits 68-75 */
/* JPEG coding, JBIG coding, etc. disabled */
#define BT_EFF_DISABLE                  0x0
/* 68: JPEG coding */
#define BT_EFF_JPEG_ENABLE              0x1
/* 69: 0 grayscale, 1 Full Color */
#define BT_EFF_JPEG_FULLCOLOR           0x2
/* 70: DIS always 0, 1 in DCS means default Huffman tables used */
#define BT_EFF_JPEG_DEFAULT_TABLES      0x4
/* 71: 0 8 bit, 1 12 bit JPEG encoding */
#define BT_EFF_JPEG_12BIT               0x8
/* 73: 0 subsampling 4:1:1, 1 subsampling 1:1:1 */
#define BT_EFF_JPEG_NOSUBSAMPLING       0x10
/* 74: 0 standard CIE illuminant D50, 1 embedded in JPEG syntax */
#define BT_EFF_JPEG_CUSTOM_ILLUMINANT   0x20
/* 75: 0 default gamut, 1 gamut embedded in JPEG syntax */
#define BT_EFF_JPEG_CUSTOM_GAMUT        0x40
/* T.82/85 -- DIS/DCS bit 78 */
#define BT_EFF_JBIG                     0x0100
/* T.82/85 optional L0 mode -- DIS/DCS bit 79 */
#define BT_EFF_JBIG_L0                  0x0200
/* T.43 Lossless color and grayscale option -- DIS/DCS bit 36*/
#define BT_EFF_LOSSLESS                 0x0400
/* b4-b6 correspond to T.44 MRC mode -- DIS/DCS bits 92-94 */
#define BT_EFF_MRC_MASK                 0x3800
/* T.43 vs. stripe interleave -- DIS/DCS bit 37 */
#define BT_EFF_PLANE_INTERLEAVE         0x4000
/* T.44 page length maximum strip size -- DIS/DCS bit 95 */
#define BT_EFF_PAGE_LENGTH_STRIP        0x8000

/* DebugModeSet bits, DebugModeSetAdv mode */
#define DEBUG_NONE              0
#define DEBUG_PRINT_CMD         0x01
#define DEBUG_PRINT_INTR        0x02
#define DEBUG_MON               0x04
#define DEBUG_ERR               0x08
#define DEBUG_DEBUG             0x10
#define DEBUG_ACCULIB           0x20                      /* internal use only */
#define DEBUG_ENTRY_EXIT        0x40                      /* internal use only */
#define DEBUG_ENTRY_EXIT_ARGS   0x80                      /* internal use only */
#define DEBUG_MAX               DEBUG_ENTRY_EXIT_ARGS     /* internal use only */
#define DEBUG_ALL               -1

/* Values for BfvDebugModeSetAdv options */
#define API_DBG_EXT_CTRL          0x00000001
#define API_DBG_MODE_SET          0x00000002
#define API_DBG_FUNC_SET          0x00000004
#define API_DBG_FUNC_ENTRY_EXIT   0x00000008
#define API_DBG_CC_API_SET        0x00000010
#define API_DBG_CC_L3L4_SET       0x00000020
#define API_DBG_CC_L4L3_SET       0x00000040
#define API_DBG_CC_INT_SET        0x00000080
#define API_DBG_CC_HOST_MOD_SET   0x00000100
#define API_DBG_CC_IP_STACK_SET   0x00000200
#define API_DBG_CC_LOG_FILE_SET   0x00000400
#define API_DBG_FILE_SET          0x00000800

#define API_DBG_FUNC_OPT_ENA      0x00000001
#define API_DBG_FUNC_OPT_ARGS     0x00000002


#define DIRECT_XFER_WRITE       0
#define DIRECT_XFER_READ        1


/* Feature set defines */
#define FEATURE_BOOLEAN '#'
#define FEATURE_INTEGER '='
#define FEATURE_STRING '$'


/* Pkt/Command function send/rcv return value bits */
#define MILL_SEND_COMPL         0x1
#define MILL_SEND_ERR           0x2
#define MILL_SEND_INCOMING      0x4
#define MILL_SEND_INC_PROC      0x8
#define MILL_RCV_COMPL          0x1
#define MILL_RCV_ERR            0x2


/* Pkt/Command send function incoming_flags values */
#define MILL_SEND_INCOMING_IGNORE 0
#define MILL_SEND_INCOMING_RETURN 1
#define MILL_SEND_INCOMING_PROCESS 2
#define MILL_SEND_INCOMING_PROCESS_RETURN 3

#define MILL_PKT_PRIO_LOW 0
#define MILL_PKT_PRIO_HIGH 1

#ifndef MILL_BUS_UNKNOWN
#define MILL_BUS_UNKNOWN        0
#define MILL_BUS_ISA            1
#define MILL_BUS_PCI            2
#define MILL_BUS_VIRT_MOD       3
#endif

#define BT_MON_MODE_ENABLE              1
#define BT_MON_MODE_DISABLE             2
#define BT_MON_MODE_ONETIME             3

#define MILL_TEST_ALL_DEF 0xffffffff

#define CP_TYPE_LOW             0
#define CP_TYPE_HIGH            1
#define CP_TYPE_BOARD_DET       2
#define CP_TYPE_CED             3
#define CP_TYPE_FREQUENCY       4
#define CP_TYPE_CADENCE         5

#define ECHOC_OP_ENABLE                 1
#define ECHOC_OP_DISABLE                2
#define ECHOC_OP_RESET                  3
#define ECHOC_OP_ALT_INPUT_ENABLE       4
#define ECHOC_OP_ALT_INPUT_DISABLE      5
#define ECHOC_OP_OUTPUT_SELECT          6
#define ECHOC_OP_BDE_SETUP              7
#define ECHOC_OP_BDE_INITIATE           8
#define ECHOC_OP_BDE_RESET              9
#define ECHOC_OP_NLP_ENABLE             10
#define ECHOC_OP_NLP_DISABLE            11

#define VAD_NO_SPEECH_DETECTED  0
#define VAD_SPEECH_DETECTED     1

#define DEACT_MODE_SINGLE       0
#define DEACT_MODE_ALL          1

#define MILL_OPTIMAL_DATA_XFER_SIZE 1006


/* Async use */
#define LP_CONTINUE ((MILL_LINE *)(MILL_PTR_INT_TYPE)0xffffffff)


/* Millennium Address structure and defines */
#include "milladdr.h"


/* Should be in facility specific header file? */
#include "fsk.h"



/* Various structures used by functions and macros */
#pragma pack(1)

#define PC_FIFO_NBYTES  10
#define LP_FIFO_NBYTES  10
#define MAX_DID         256
#define MAX_NSF         256
#define MAX_NAME_STR    51
#define MAX_CONN_NUM    32

typedef struct {
    int status;
    int line_status;
} RES;

typedef struct {
    int call_type;
    char dest_id[MAX_DID];
    /* The rest are ISDN only */
#define called_party_number dest_id
    char called_party_subaddress[MAX_DID];
    char calling_party_number[MAX_DID];
    char calling_party_subaddress[MAX_DID];
    char redir_number[MAX_DID];
    int redir_reason;
    int redir_orig_reason;
    int redir_counter;
    char name_ident[MAX_NAME_STR];
    int name_char_set;
    char connected_num[MAX_CONN_NUM];
} CALL_RES;

typedef struct {
    char remote_id[21];
    unsigned char nsf_nss_frame[MAX_NSF];
    char subaddress[21];
    char password[21];
    char selective_polling[21];
} INFO_RES;

typedef struct page_res {
    int page_complete_type;
    int continue_breaks;
    unsigned ascii_bytes;
    unsigned bad_lines;
    unsigned total_lines;
    unsigned total_rcv;
    unsigned bit_rate;
    unsigned page_status;
    unsigned char pc_fifo[PC_FIFO_NBYTES];
    unsigned sig_level;
    unsigned line_noise;
    unsigned sig_quality;
    unsigned char lp_fifo[LP_FIFO_NBYTES];
    unsigned char confirm_value;
    int direction;
    int resolution;
    int width;
    unsigned bft_type;
    unsigned long eff_page_type;
    struct page_res *next;
} PAGE_RES;

typedef struct {
    int number_of_pages;
    int bad_pages;
    char remote_id[21];
    long duration;
    PAGE_RES *reslist_head;
    PAGE_RES *reslist_tail;
} FAX_RES;

typedef struct {
    int status;
    int charge_info;
    long charge;
} AOC_INFO;

typedef struct
{
    char addr[MAX_CONN_NUM];
    unsigned char connected_num_type;
    unsigned char connected_num_plan;
    unsigned char connected_num_presentation;
    unsigned char connected_num_screening;
} CONNECTED_NUM;

#define BTERR_SUPPORTED 1
typedef struct {
    char short_msg[31];
    char long_msg[129];
} BTERR;

struct cpgen_tone_info {
    unsigned freq;
    unsigned atten;
};

struct cpgen_signal_info {
    unsigned play_time;
    struct cpgen_tone_info *tone_info;
};

#pragma pack()


/* If BCONST or MCONST is defined, enable "const" declarations for function
   prototypes. */
#if defined(BCONST) || defined(MCONST)
#undef MCONST
#define MCONST const
#else
#define MCONST
#endif

#include "misc.h"
#include "caps.h"
#include "pktdef.h"
#include "dialdb.h"
#include "infopkt.h"
#include "tiff.h"
#include "t434.h"
#include "dcs.h"
#include "line.h"
#include "phone.h"
#include "btnet.h"
#include "millver.h"


/* Argument structures */
#pragma pack(1)


/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_line_admin {
    RES res;
    int unit;
    MILL_ADDR dest_addr;
    MILL_ADDR local_addr;
    int present;
    int unique;
    int reset_on_close;
    unsigned buf_size;
    unsigned vol_mon_upper_thresh;
    unsigned vol_mon_lower_thresh;
    MCONST char *config_file_name;
    MCONST char *config_file_line;
    long reset_status;
    int type;
    unsigned port;
    int mvip_peb;
    int law;
    int robbed;
    int clock_rate;
    int sig_prot;
    int mf_freq;
    int tstream;
    int tslot;
    int rstream;
    int rslot;
    int no_fonts;
    int specify_reset_type;
    int reset_type;
    int use_config_lines;
    unsigned base;
    MCONST char *local_add;
    MCONST char *local_sub;
    MCONST char *spid;
    MCONST char *calling_p_num;
    MCONST char *calling_p_sub;
    int bearer_capability;
    int bchan_pref;
    unsigned char alert_value;
    int retain;
    MILL_LINE *retain_lp;
    int mill_load_fonts;
    int skip_teleph;
    int err_on_deact;
    int no_init;
    MILL_LINE *async_lp;
    int virt_mod;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_telephone {
    RES res;
    CALL_RES call_res;
    MCONST char *phonenum;
    int call_protocol_code;
    long timeout;
    int call_mode;
    M_CB1 int (* M_CB2 func)(MILL_LINE *lp, char *arg);
    char *arg;
    unsigned char type;
    unsigned char data;
    MCONST char *db_file;
    MCONST RES *db_res;
    long wait_time;
    int reason;
    M_CB1 int (* M_CB2 db_func)(struct dialdb *dbentry,char *arg);
    char *raw_number;
    int orig_answer;
    unsigned report_cadence;
    unsigned report_freq;
    unsigned freq_report_time;
    unsigned template_number;
    unsigned prog_freq;
    unsigned prog_cadence;
    unsigned diss_mode;
    unsigned short frequencies[2];
    unsigned duration;
    int level;
    unsigned short cadence_pattern[4];
    struct {
        unsigned short freq;
        short level;
    } freq_data[3];
    int diss_only;
    unsigned cause_code;
    MILL_LINE *async_lp;
    int transfer_mode;
    int transfer_group;
    MILL_LINE *lp_second_channel;
    int supervised;
    int transfer_line_state;
    int hold_call;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_speech {
    RES res;
    struct infopkt_stream *ips;
    unsigned coding_fmt;
    unsigned rate;
    unsigned bits_per_samp;
    unsigned afe_rate;
    unsigned data_fmt;
    long timeout;
    long silence_timeout;
    int s_compr;
    M_CB1 int (* M_CB2 func)(MILL_LINE *lp, char *arg);
    char *arg;
    unsigned char *buf;
    unsigned size;
    int modification;
    int value;
    int beep;
    MCONST char *fname;
    PROMPT_MAP *prompt_map;
    unsigned phrase_number;
    unsigned agc_value;
    int use_agc;
    int playf_cont;
    unsigned bytes_processed;
    MILL_LINE *async_lp;
    unsigned prompt_id;
    unsigned prompt_size;
    unsigned sticky;
    unsigned cache_hits;
    unsigned bin_id;
    unsigned bin_size;
    unsigned bin_count;
    int echoc_op;
    int output_select;
    int vad;
    int vad_thresh;
    unsigned vad_passthru;
    unsigned bde_delay;
    int asr_mode;
    unsigned summation_group;
    MILL_LINE *lp_summed;
    unsigned min_callback;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_fax {
    RES res;
    struct infopkt_stream *s_ips;
    struct infopkt_stream *r_ips;
    MCONST char *local_id;
    FAX_RES fax_res;
    INFO_RES remote_info;
    int oa_flag;
    unsigned fmt;
    unsigned char *buf;
    unsigned size;
    int resolution;
    int resolution_negot;
    int width;
    unsigned bft_type;
    MCONST char *fname;
    int expect_another;
    int can_send;
    int can_receive;
    int font_no;
    int end_mode;
    int holdup_mode;
    M_CB1 int (* M_CB2 func)(MILL_LINE *lp, char *arg, int section);
    char *arg;
    int xmit_again;
    int nsf_or_nss;
    unsigned selector;
    int overlay_number;
    int action;
    int spacing;
    int units;
    int placement;
    int insert_mode;
    MCONST char *label;
    TFILE *s_tp;
    TFILE *r_tp;
    int combine;
    FILE *fp;
    MCONST struct ifd_field *opt_tags;
    int num_opt_tags;
    int xmit_mode;
    int rcv_mode;
    int xmit_sync;
    int use_open_file;
    long open_file_size;
    int force_res;
    int btg3;
    int force_eom;
    int compat_width;
    int g3_pt_enable;
    unsigned long eff_page_type;
    int specify_tz_offset;
    int tz_offset;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_fax_t30_params {
    RES res;
    unsigned bit_rate;
    unsigned scan_time;
    unsigned modulation_type;
    unsigned line_compression;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_fax_page_params {
    RES res;
    unsigned top_margin;
    unsigned bottom_margin;
    unsigned length;
    unsigned ascii_pad;
    unsigned image_pad;
    unsigned image_break;
    unsigned image_margin;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_fax_strip_params {
    RES res;
    unsigned fmt;
    int resolution;
    unsigned width;
    unsigned left_margin;
    unsigned right_margin;
    unsigned line_spacing;
    unsigned eof_char;
    unsigned font_no;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_tone {
    RES res;
    int digit;
    long millisecs;
    int tone_type;
    int toneid;
    int volume;
    int cp_type;
    unsigned count;
    MCONST struct cpgen_signal_info *signal_info;
    int decode_flag;
    int tone_cp;
    unsigned interdigit_time;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_download {
    RES res;
    MCONST unsigned char *buf;
    unsigned size;
    MCONST char *fname;
    int specify_type;
    unsigned dl_type;
    unsigned dl_dest;
    unsigned config_spec_value;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_dh {
    RES res;
    FILE *fp;
    int unit;
    int entries;
    int print_chan;
    int consec_seq;
    int dont_reduce_fifo;
    int continuous_output;
    unsigned module;
    unsigned channel;
    int raw_dump;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_intlimit {
    RES res;
    unsigned limit;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_direct_xfer {
    RES res;
    unsigned char *buf;
    unsigned size;
    unsigned mod_num;
    unsigned mod_addr;
    int direction;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_addr_info {
    RES res;
    unsigned mod_num;
    unsigned exists;
    unsigned num_channels;
    unsigned hardware;
    unsigned facility;
    MILL_ADDR m_addr;
    unsigned bus_type;
    unsigned hw_type;
    unsigned hw_id;
    unsigned ordinal_start;
    unsigned num_ordinals;
    unsigned config_spec_num;
    unsigned config_spec_value;
    char config_spec_desc[51];
    unsigned short pci_vend_id;
    unsigned short pci_dev_id;
    unsigned short pci_sub_vend_id;
    unsigned short pci_sub_id;
    unsigned char pci_interf_type;
    unsigned char boot_ver_major;
    unsigned char boot_ver_middle;
    unsigned char boot_ver_minor;
    unsigned char boot_ver_build;
    unsigned short boot_ver_auto_num;
    char boot_ver_date[20];
    char boot_ver_comment[50];
    unsigned slot_num;
    int deact_mode;
    unsigned pci_runtime_start;
    unsigned pci_runtime_size;
    unsigned pci_mem_start;
    unsigned pci_mem_size;
    unsigned pci_modctrl_start;
    unsigned pci_modctrl_size;
    unsigned pci_io_start;
    unsigned pci_io_size;
    unsigned pci_irq;
    unsigned char private_info[128];
    unsigned hw_info;
    unsigned num_ordinals_init;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

#define TMP_PKT_SIZE 1100
#define TMP_PKT_SIZE_LIMIT 1024
#define cmd_subcmd cmd_specifier

struct args_packet {
    RES res;
    unsigned facility;
    unsigned cmd_verb;
    unsigned cmd_specifier;
    int prio;
    int incoming_flag;
    int no_hangup;
    unsigned timeout;
    unsigned char *cmd_buf;
    unsigned tag_id;
    unsigned tag_type;
    unsigned tag_len;
    unsigned char *tag_ptr;
    unsigned tag_data_len;
    unsigned char *tag_data_ptr;
    unsigned total_tag_len;
    unsigned var_value;
    unsigned flags;
    MILL_ADDR dest_addr;
    MILL_ADDR src_addr;
    unsigned cmd_len;
    unsigned pkt_len;
    unsigned char *pkt_buf;
    unsigned stored_len;
#ifdef PKTV0_OR_V1
    int sent;
#endif
    int dont_continue;
    unsigned char *data_parse_form_ptr;
    unsigned data_parse_form_len;
    unsigned bit_len;
    unsigned int_size;
    int ignore_data;
    int fac_src;
    unsigned char *orig_pkt_buf;
    unsigned orig_pkt_len;
    int clear_all;
    int already_parsed;
    int add_dbg_output;
    va_list *p_args_v;
    unsigned num_tags;
    int use_alt_chan;
    MILL_LINE *async_lp;
    int bulk_data;
    int alloc_dest;
    int no_process;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_feature_set {
    RES res;
    MCONST unsigned char *buf;
    unsigned size;
    char *fname;
    MCONST char key[9];
    int type;
    int value_int;
    MCONST char output_value_string[128];
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct UDPTL_options
{
    struct bt_sockaddr_in remoteAddr;
    struct bt_sockaddr_in localAddr;
    int t38_version;
    int t38_max_bit_rate;
    int t38_fax_fill_bit_removal;
    int t38_fax_transcoding_MMR;
    int t38_fax_transcoding_JBIG;
    int t38_fax_rate_management;
    int t38_fax_max_buffer;
    int t38_fax_max_datagram_recv;
    int t38_fax_udp_EC;
    int t38_fax_max_datagram_send;
    int t38_UDPTL_redundancy_depth_control;
    int t38_UDPTL_redundancy_depth_image;
    int t38_t30_fast_notify;
    struct bt_sockaddr_in remoteRTCPAddr;
    struct bt_sockaddr_in localRTCPAddr;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct RTP_options
{
    struct bt_sockaddr_in remoteRTPAddr;
    struct bt_sockaddr_in localRTPAddr;
    struct bt_sockaddr_in remoteRTCPAddr;
    struct bt_sockaddr_in localRTCPAddr;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct transport_options
{
    // Used when port_class is CALL_SW_PORT_UDPTL_DEF
    struct UDPTL_options UDPTLopts;
    // Used when port_class is CALL_SW_PORT_RTP_DEF
    struct RTP_options RTPopts;
    // Options for other classes will go here in future
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_tel_ctrl_call_sw {
    RES res;
    char *config_file_name;
    unsigned port_type;
    unsigned src_port_class;
    unsigned src_port_unit;
    unsigned src_stream;
    unsigned src_slot;
    unsigned src_port_type;
    unsigned dest_port_class;
    unsigned dest_port_unit;
    unsigned dest_stream;
    unsigned dest_slot;
    unsigned conn_mode;
    unsigned dest_port_type;
    void *retain_ptr;
    unsigned set_line_encoding;
    unsigned line_encoding;
    unsigned set_signal_mode;
    unsigned signal_mode;
    unsigned set_bus_speed;
    unsigned bus_speed;
    unsigned set_sig_prot;
    unsigned sig_prot;
    unsigned set_bus_clock;
    unsigned bus_clock;
    unsigned set_bus_select;
    unsigned bus_select;
    unsigned set_mvip_sec8k;
    unsigned mvip_sec8k;
    unsigned set_ds1_line_type;
    unsigned ds1_line_type;
    unsigned set_clock_config;
    unsigned clock_config;
    unsigned set_line_loopback;
    unsigned line_loopback;
    unsigned set_fac_data_link;
    unsigned fac_data_link;
    unsigned set_t1_line_build_out;
    unsigned t1_line_build_out;
    unsigned set_line_coding;
    unsigned line_coding;
    unsigned set_h110_clock_mode;
    unsigned h110_clock_mode;
    unsigned set_h110_master_drive;
    unsigned h110_master_drive;
    unsigned set_h110_slave_source;
    unsigned h110_slave_source;
    unsigned set_h110_master_ref;
    unsigned h110_master_ref;
    unsigned set_h110_master_ref_fallbk;
    unsigned h110_master_ref_fallbk;
    unsigned set_h110_netref_drive;
    unsigned h110_netref_drive;
    unsigned set_h110_netref_source;
    unsigned h110_netref_source;
    MILL_LINE *async_lp;
    struct transport_options srcOptions;
    struct transport_options destOptions;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_board_state {
    RES res;
    unsigned state;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_board_notify {
    RES res;
    unsigned enable;
    unsigned notify_type;
    unsigned unit;
    unsigned network_alarm;
    unsigned network_error;
    unsigned board_state;
    int temp_alarm;
    int temp;
    unsigned h110_clock_status;
    unsigned link_status;
    unsigned failover_unit;
    unsigned speed;
    unsigned duplex;
    M_CB1 void (* M_CB2 func)
         (MILL_LINE *lp, unsigned notify_type, struct args_board_notify *args);
    struct args_packet *args_pkt;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_signaling {
    RES res;
    unsigned unit;
    unsigned stream;
    unsigned time_slot;
    unsigned stateOut_0;
    unsigned durationOut_0;
    unsigned stateOut_1;
    unsigned stateIn_0;
    unsigned durationIn_0;
    unsigned stateIn_1;
    unsigned millisecs;
    M_CB1 void (* M_CB2 func)
         (MILL_LINE *lp, struct args_signaling *args);
    unsigned mon_mode;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_temperature {
    RES res;
    int current_temp;
    int temp_thresh;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_board_test {
    RES res;
    unsigned iterations;
    unsigned test_mask;
    M_CB1 void (* M_CB2 func)
         (MILL_LINE *lp, struct args_board_test *args);
    unsigned status;
    unsigned err_info;
    MILL_LINE *async_lp;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_debug_mode {
    RES res;
    unsigned options;
    M_CB1 void (* M_CB2 func) (char *msg);
    int mode;
    char *fname1;
    char *fname2;
    unsigned file_limit;
    int unit;
    MILL_ADDR dest_addr;
    MILL_ADDR local_addr;
    int cc_trace_level;
    char *cc_trace_file_name;
    int func_dbg_opts;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/



#ifndef MILL_MAX_TIMEOUT
//#define MILL_MAX_TIMEOUT 0x7fffffff
#define MILL_MAX_TIMEOUT 0xffffffff
#endif


#pragma pack()


#define MillLineDetach MillSessionDetach

/* Prototypes for functions which don't have internal/external. */
M_APIE1 int M_APIE2 MillDebugModeSet(int mode);
M_APIE1 void M_APIE2 MillDebugFuncSet(M_CB1 void (* M_CB2 func) (char *msg));
M_APIE1 void M_APIE2 MillMemAllocFuncsSet
(M_CB1 void * (* M_CB2 alloc_func_ptr)
 (unsigned size,int channel,int mem_type),
 M_CB1 void (* M_CB2 free_func_ptr)
 (void *ptr,int channel,int mem_type));

#include "lineacc.h"

 /* These names will have to change, or will conflict with TR114 API */
#ifdef MILL_PLAT_HAS_DLLS
 /* Functions for use with DLLs */
M_APIE1 FILE * M_APIE2 _mdll_stdout();
M_APIE1 FILE * M_APIE2 _mdll_stdin();
M_APIE1 FILE * M_APIE2 _mdll_stderr();
M_APIE1 FILE * M_APIE2 _mdll_fopen (char *fname, char *mode);
M_APIE1 int M_APIE2 _mdll_fclose (FILE *fp);
M_APIE1 int M_APIE2 _mdll_fseek (FILE *fp, long offset, int pos);
M_APIE1 long M_APIE2 _mdll_ftell (FILE *fp);
M_APIE1 unsigned M_APIE2 _mdll_fread(void *ptr, int nitems, unsigned size,
                                     FILE *fp);
M_APIE1 unsigned M_APIE2 _mdll_fwrite (void *ptr, int nitems, unsigned size,
                                       FILE *fp);
M_APIE1 void * M_APIE2 _mdll_malloc(unsigned size);
M_APIE1 void M_APIE2 _mdll_free(void *ptr);
#endif

#ifdef MILL_DLL
/* MILL_DLL is defined by applications when linking with DLL versions of the
   BT API library in certain environments. */
#ifdef MILL_DLL_VARS_USE_PTRS
#define _mill_optind (*_p_mill_optind)
#define _mill_optarg (*_p_mill_optarg)
#define _mill_opterr (*_p_mill_opterr)
#define _mill_priority (*_p_mill_priority)
#define _mill_api_ver_compiled (*_p_mill_api_ver_compiled)
#endif
#endif

/* Globals for all API platforms */
/* If there were any, they would be declared here using
   the MILL_APIG* declarations. */

extern M_APIG1 int M_APIG2 _mill_api_ver_compiled;
extern M_APIG1 int M_APIG2 _mill_api_ver_bld_comp;
extern M_APIG1 int M_APIG2 _mill_api_ver_auto_comp;
extern M_APIG1 int M_APIG2 _mill_api_lib_ver_compiled;
extern M_APIG1 int M_APIG2 _mill_sdk_ver_compiled;
extern M_APIG1 int M_APIG2 _mill_sdk_auto_compiled;

#ifdef MILL_API_SETS_PRIO
extern M_APIG1 int M_APIG2 _mill_priority;
#endif

#ifdef MILL_NEEDS_GETOPT
/* Sometimes provided by compiler, sometimes by BT API */
/* Defines to insure that getopt doesn't conflict with other libraries */
#ifndef NO_MILL_GETOPT
M_APIE1 int M_APIE2 _mill_getopt (int args,char **argv,char *optstring);
extern M_APIG1 int M_APIG2 _mill_optind;
extern M_APIG1 char M_APIG2 *_mill_optarg;
extern M_APIG1 int M_APIG2 _mill_opterr;
#define getopt _mill_getopt
#define optind _mill_optind
#define optarg _mill_optarg
#define opterr _mill_opterr
#endif
#endif

#ifdef PKTV0_OR_V1
extern M_APIG1 unsigned M_APIG2 _mill_pkt_version_exp;
#endif

M_APIE1 void M_APIE2 _mill_lbolt_reset (void);

#ifndef BOARD_TYPE_UNKNOWN
#define BOARD_TYPE_UNKNOWN      0
#define BOARD_TYPE_FAX          1
#define BOARD_TYPE_VOICE        2
#define BOARD_TYPE_TR114        3
#define BOARD_TYPE_TRUFAX       4
#define BOARD_TYPE_BOSTON       BOARD_TYPE_TR114
#endif

#define MOD_BOARD_SLOT(mod_num) \
    ((mod_num >= 0xf0) ? 0 : \
     (((mod_num) >= 0x20) ? (mod_num) / 8 - 3 : (mod_num) - 1))
#define MOD_CPU_NUM(mod_num) \
    ((mod_num >= 0xf0) ? 0 : \
     (((mod_num) >= 0x20) ? (mod_num) % 8 : 0))
#define LP_BOARD_SLOT(lp) MOD_BOARD_SLOT(LINE_DEST_ADDR(lp).mm_bModule)
#define LP_CPU_NUM(lp) MOD_CPU_NUM(LINE_DEST_ADDR(lp).mm_bModule)

#include "ecclib.h"


#define MILL_DO_PROTOS

#endif



#ifdef MILL_DO_PROTOS

#include "funcmap.h"

/* Function prototypes */


/* Administrative*/
B_APIE1 MILL_LINE * B_APIE2 MillLineAttach(struct args_line_admin *args);
B_APIE1 MILL_LINE * B_APIE2 MillSessionAttach(struct args_line_admin *args);
B_APIE1 void B_APIE2 MillSessionDetach(MILL_LINE *lp,
                                       struct args_line_admin *args);
B_APIE1 int B_APIE2 MillLineReset(MILL_LINE *lp, struct args_line_admin *args);
B_APIE1 int B_APIE2 MillLineConfig (MILL_LINE *lp,
                                    struct args_line_admin *args);
B_APIE1 void B_APIE2 MillLineAlert(struct args_line_admin *args);
B_APIE1 int B_APIE2 MillLineInfo (struct args_line_admin *args);
B_APIE1 char * B_APIE2 MillLinesAvail (struct args_line_admin *args);


/* Telephony Control and Call Switching */
B_APIE1 int B_APIE2 MillTelephGetInfo(MILL_LINE *lp,
                                      struct args_tel_ctrl_call_sw *args);
B_APIE1 int B_APIE2 MillCallSWGetInfo(MILL_LINE *lp,
                                      struct args_tel_ctrl_call_sw *args);
B_APIE1 int B_APIE2 MillCallSWConnect(MILL_LINE *lp,
                                      struct args_tel_ctrl_call_sw *args);
B_APIE1 int B_APIE2 MillCallSWConnectIP(MILL_LINE *lp,
                                        struct args_tel_ctrl_call_sw *args);
B_APIE1 int B_APIE2 MillCallSWClearConns(MILL_LINE *lp,
                                         struct args_tel_ctrl_call_sw *args);
B_APIE1 int B_APIE2 MillCallSWGetConns(MILL_LINE *lp,
                                       struct args_tel_ctrl_call_sw *args);
B_APIE1 void B_APIE2 MillTelephReset(MILL_LINE *lp,
                                     struct args_tel_ctrl_call_sw *args);
B_APIE1 int B_APIE2 MillTelephSave(MILL_LINE *lp,
                                   struct args_tel_ctrl_call_sw *args);


/* Speech */
B_APIE1 void B_APIE2 MillSpeechPlayData(MILL_LINE *lp,
                                        struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechPlayFile(MILL_LINE *lp,
                                        struct args_speech *args);
B_APIE1 unsigned int B_APIE2 MillSpeechRecordData(MILL_LINE *lp,
                                                  struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechRecordFile(MILL_LINE *lp,
                                          struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechPlay (MILL_LINE *lp,struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechRecord (MILL_LINE *lp,struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechModify (MILL_LINE *lp,struct args_speech *args);
B_APIE1 PROMPT_MAP * B_APIE2 MillPromptOpen (struct args_infopkt *args);
B_APIE1 int B_APIE2 MillPromptClose (struct args_infopkt *args);
B_APIE1 void B_APIE2 MillPromptPlay (MILL_LINE *lp,struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechPlayWave(MILL_LINE *lp,
                                        struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechRecordWave(MILL_LINE *lp,
                                          struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechCacheFlush(MILL_LINE *lp,
                                          struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechCacheEnable(MILL_LINE *lp,
                                           struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechCacheDisable(MILL_LINE *lp,
                                            struct args_speech *args);
B_APIE1 int B_APIE2 MillSpeechCacheConfigure(MILL_LINE *lp,
                                             struct args_speech *args);
B_APIE1 int B_APIE2 MillSpeechCacheGetInfo(MILL_LINE *lp,
                                           struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechCachePlay(MILL_LINE *lp,
                                         struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechEchoCancelControl(MILL_LINE *lp,
                                                 struct args_speech *args);
B_APIE1 void B_APIE2 MillSpeechQuerySummationGroup(MILL_LINE *lp,
                                                   struct args_speech *args);


/* Fax */
B_APIE1 void B_APIE2 MillFaxBeginSendRaw (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxGetRemoteInfo (MILL_LINE *lp,
                                           struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxWaitForTraining(MILL_LINE *lp,
                                            struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxHeader(MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxSendData (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxNextPageRaw (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxEndOfDocument (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxBeginReceive (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxEndReception (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxReceiveData (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxAbort (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxSetReceiveFmt (MILL_LINE *lp,
                                           struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxStripParams (MILL_LINE *lp,
                                         struct args_fax_strip_params *args);
B_APIE1 void B_APIE2 MillFaxPageParams (MILL_LINE *lp,
                                        struct args_fax_page_params *args);
B_APIE1 void B_APIE2 MillFaxSendFile (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxReceiveFile (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxDownloadFont(MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxDownloadFontData(MILL_LINE *lp,
                                            struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxSend (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxReceive(MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxSetLocalId (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxBeginSend (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxNextPage (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxSendPage (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxT30Params (MILL_LINE *lp,
                                       struct args_fax_t30_params *args);
B_APIE1 int B_APIE2 MillFaxOverlay (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxSpace (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxBeginSendTiff (MILL_LINE *lp,
                                           struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxNextPageTiff (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxSendPageTiff (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxSendPageDCX (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxNextPageDCX (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxReceivePage (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxReceivePages (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxRcvPageDCX (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillFaxRcvPageTiff (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxBeginRaw (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxBeginTiff(MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxBegin(MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxPoll (MILL_LINE *lp,struct args_fax *args);
B_APIE1 int B_APIE2 MillDataFSK (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxSetNSF (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxSetSubPwdSep (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxSetModeSwitch (MILL_LINE *lp,
                                           struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxCheckModeSwitch (MILL_LINE *lp,
                                             struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxT30Holdup (MILL_LINE *lp,struct args_fax *args);
B_APIE1 void B_APIE2 MillFaxTurnaround (MILL_LINE *lp,struct args_fax *args);


/* Call Progress */
B_APIE1 int B_APIE2 MillLineCallProgressEnable(MILL_LINE *lp,
                                               struct args_telephone *args);
B_APIE1 void B_APIE2 MillLineCallProgressDisable(MILL_LINE *lp,
                                                 struct args_telephone *args);
B_APIE1 int B_APIE2 MillDataCP (MILL_LINE *lp,struct args_telephone *args);
B_APIE1 void B_APIE2 MillLineCallProgressProgram(MILL_LINE *lp,
                                                 struct args_telephone *args);

/* Dialing database */
B_APIE1 int B_APIE2 MillDialDBUpdate(MILL_LINE *lp,
                                     struct args_telephone *args);
B_APIE1 int B_APIE2 MillDialDBCheck (MILL_LINE *lp,
                                     struct args_telephone *args);
B_APIE1 int B_APIE2 MillDialDBList(struct args_telephone *args);

/* Tones */
B_APIE1 void B_APIE2 MillToneDetectEnable(MILL_LINE *lp,
                                          struct args_tone *args);
B_APIE1 void B_APIE2 MillToneDetectDisable(MILL_LINE *lp,
                                           struct args_tone *args);
B_APIE1 int B_APIE2 MillToneGet(MILL_LINE *lp,struct args_tone *args);
B_APIE1 void B_APIE2 MillToneUnget(MILL_LINE *lp,struct args_tone *args);
B_APIE1 void B_APIE2 MillToneFlush (MILL_LINE *lp,struct args_tone *args);
B_APIE1 int B_APIE2 MillTonePeek (MILL_LINE *lp,struct args_tone *args);
B_APIE1 void B_APIE2 MillTonePlay (MILL_LINE *lp,struct args_tone *args);
B_APIE1 void B_APIE2 MillTonePlayBeep (MILL_LINE *lp,struct args_tone *args);
B_APIE1 void B_APIE2 MillCPGen (MILL_LINE *lp,struct args_tone *args);
B_APIE1 int B_APIE2 MillCPGenAdv (MILL_LINE *lp,struct args_tone *args);


/* Misc */
B_APIE1 void B_APIE2 MillErrorMessage (MILL_LINE *lp, RES *res,
                                       BTERR *err_msg);
B_APIE1 void B_APIE2 MillHistoryDumpModChan (struct args_dh *args);
B_APIE1 void B_APIE2 MillHistoryDumpUnit (struct args_dh *args);
B_APIE1 void B_APIE2 MillHistoryDump (MILL_LINE *lp,struct args_dh *args);
B_APIE1 void B_APIE2 MillHistoryClearModChan (struct args_dh *args);
B_APIE1 void B_APIE2 MillHistoryClearUnit (struct args_dh *args);
B_APIE1 void B_APIE2 MillHistoryClear (MILL_LINE *lp,struct args_dh *args);
B_APIE1 void B_APIE2 MillLineInterruptLimit (MILL_LINE *lp,
                                             struct args_intlimit *args);
M_APIE1 void M_APIE2 MillLineDumpStructure (MILL_LINE *lp,FILE *fp);


/* Low level, Millennium specific */
B_APIE1 int B_APIE2 MillFirmwareDownload(MILL_LINE *lp,
                                         struct args_download *args);
B_APIE1 int B_APIE2 MillFirmwareDownloadData(MILL_LINE *lp,
                                             struct args_download *args);
B_APIE1 void B_APIE2 MillModuleInfo(MILL_LINE *lp,struct args_addr_info *args);
B_APIE1 int B_APIE2 MillModuleConfigSpecsGet(MILL_LINE *lp,
                                             struct args_addr_info *args);
B_APIE1 void B_APIE2 MillCheckAddress(MILL_LINE *lp,
                                      struct args_addr_info *args);
B_APIE1 void B_APIE2 MillCheckFacility(MILL_LINE *lp,
                                       struct args_addr_info *args);
B_APIE1 int B_APIE2 MillSendPkt(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillSendPktProcess(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillSendCmd(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillSetSingleVar(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillSetVars(MILL_LINE *lp,struct args_packet *args,...);
B_APIE1 void B_APIE2 MillGetVar(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillSendSimpleCmd(MILL_LINE *lp,struct args_packet *args);
B_APIE1 void B_APIE2 MillFormExtByte(struct args_packet *args);
B_APIE1 void B_APIE2 MillFormExtShort(struct args_packet *args);
B_APIE1 void B_APIE2 MillFormNbytes(struct args_packet *args);
B_APIE1 int B_APIE2 MillFormTag(struct args_packet *args);
B_APIE1 int B_APIE2 MillFormCmd(struct args_packet *args,...);
B_APIE1 int B_APIE2 MillFormAddTag(struct args_packet *args);
B_APIE1 int B_APIE2 MillFormAddTags(struct args_packet *args,...);
B_APIE1 int B_APIE2 MillFormAddTagsV(struct args_packet *args);
B_APIE1 int B_APIE2 MillFormSendCmd(MILL_LINE *lp,
                                    struct args_packet *args,...);
B_APIE1 int B_APIE2 MillFormPktHeader(struct args_packet *args);
B_APIE1 void B_APIE2 MillClearOutputAddr(MILL_LINE *lp,
                                         struct args_packet *args);
B_APIE1 void B_APIE2 MillClearOutput(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillReceivePkt(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillProcessCmd(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillRcvProcessPkt(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillRcvCmds(MILL_LINE *lp,struct args_packet *args);
B_APIE1 int B_APIE2 MillGetCmdFromPkt(struct args_packet *args);
B_APIE1 int B_APIE2 MillParseCmd(struct args_packet *args);
B_APIE1 unsigned B_APIE2 MillParseExtByte(struct args_packet *args);
B_APIE1 unsigned B_APIE2 MillParseExtShort(struct args_packet *args);
B_APIE1 unsigned B_APIE2 MillParseNbytes(struct args_packet *args);
B_APIE1 int B_APIE2 MillParseTaggedValueInt(struct args_packet *args);
B_APIE1 int B_APIE2 MillParsePktHeader(struct args_packet *args);
B_APIE1 int B_APIE2 MillGetTaggedValues(struct args_packet *args);
B_APIE1 int B_APIE2 MillWaitCmd(MILL_LINE *lp,struct args_packet *args);
B_APIE1 void B_APIE2 MillDirectXfer(MILL_LINE *lp,
                                    struct args_direct_xfer *args);
B_APIE1 void B_APIE2 MillClearInput(MILL_LINE *lp,struct args_packet *args);


/* Feature set, Millennium specific */
B_APIE1 int B_APIE2 MillFeatureSetDownloadData(MILL_LINE *lp,
                                               struct args_feature_set *args);
B_APIE1 int B_APIE2 MillFeatureSetDownload(MILL_LINE *lp,
                                           struct args_feature_set *args);
B_APIE1 void B_APIE2 MillFeatureSetQuery(MILL_LINE *lp,
                                         struct args_feature_set *args);


/* Misc, Millennium specific */
B_APIE1 void B_APIE2 MillBoardStateGet (MILL_LINE *lp,
                                        struct args_board_state *args);
B_APIE1 void B_APIE2 MillBoardStateSet (MILL_LINE *lp,
                                        struct args_board_state *args);
B_APIE1 void B_APIE2 MillBoardNotify (MILL_LINE *lp,
                                      struct args_board_notify *args);
B_APIE1 void B_APIE2 MillModuleDeactivate(struct args_addr_info *args);
B_APIE1 int B_APIE2 MillCallSignalingStateMonitor (MILL_LINE *lp,
                                                   struct args_signaling
                                                   *args);
B_APIE1 int B_APIE2 MillCallSignalingStateSet (MILL_LINE *lp,
                                               struct args_signaling *args);
B_APIE1 void B_APIE2 MillBoardTemperatureGet (MILL_LINE *lp,
                                              struct args_temperature *args);
B_APIE1 void B_APIE2 MillBoardTemperatureThreshSet (MILL_LINE *lp,
                                                    struct
                                                    args_temperature *args);
B_APIE1 void B_APIE2 MillBoardTest (MILL_LINE *lp,
                                    struct args_board_test *args);
M_APIE1 void M_APIE2 MillDebugInitData(char *dir_name);
B_APIE1 void B_APIE2 MillDebugModeSetAdv(struct args_debug_mode *args);

/* Millennium only, async */
B_APIE1 MILL_LINE * B_APIE2 MillActivityWait(MILL_LINE *lp,
                                             struct args_packet *args);
B_APIE1 void B_APIE2 MillActivityTrigger(MILL_LINE *lp,
                                         struct args_packet *args);

#include "ecclib.h"

#include "infopkt.h"
#include "tiff.h"
#include "t434.h"
#include "btnet.h"

#undef MILL_DO_PROTOS

#endif
