/*
 * phone.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

#ifndef _phone_h_
#define _phone_h_

/* These structures are used for file I/O, so we must ensure that they appear
   the same under all OS's/compilers, so we pack it on an alignment of 1. */
#pragma pack(1)

/* Note: Initialization is done for a structure of this type. Some compilers
   have bugs with initialization of structures if packing is used and
   there are word-sized elements which do not begin on word boundaries...
   So all word sized elements must come first or be aligned. */
/* If any values in here are of size > sizeof(char), then little/big endian
   architecture becomes an issue, and conversion must be done manually. */
/* Under no circumstances remove any fields from this structure once
   the BT_CPARM file has been released. If a field becomes unused, it
   may be ignored or renamed, but for compatibility it must remain. */
struct country_phone_info {
    unsigned char country_code;         /* Brooktrout Country Code */
    unsigned char tone_len;             /* Tone length */
    unsigned char tone_inter_time;      /* Tone interdigit time */
    unsigned char pulse_inter_time;     /* Pulse interdigit time */
    unsigned char pulse_make;           /* Pulse make time */
    unsigned char pulse_break;          /* Pulse break time */
    unsigned char ring_len;             /* Ring Length */
    unsigned char dtone_len;            /* Minimum length of dialtone */
    unsigned char dtone_timeout_lowbyte; /* Dialtone timeout time low byte */
    unsigned char dtone_timeout_highbyte; /* Dialtone timeout time high byte */
    unsigned char loopcur_len;          /* Minimum time for loop current */
    unsigned char loopcur_timeout;      /* Loop current timeout time */
    unsigned char loopcur_debounce;     /* Loop current de-debounce time */
    unsigned char ring_lower;           /* Ring count lower threshold */
    unsigned char ring_upper;           /* Ring count upper threshold */
    unsigned char ring_blank;           /* Ringing blank time */
    char data_level;                    /* Data transmit level */
    char dtmf_low_level;                /* DTMF lower freq. transmit level */
    char dtmf_high_level;               /* DTMF upper freq. transmit level */
    char rec_level;                     /* Receive level */
    unsigned char enable_loop_cur;      /* Flag to monitor loop current */
    unsigned char enable_beep;          /* Enable beep interrupts */
    unsigned char pulse_line;           /* The number of pulses to send */
    unsigned char dial_tone_min;        /* Delay between dial tone + dialing */
    unsigned char drop_out_time;
    unsigned char wait_for_ced_low;
    unsigned char wait_for_ced_high;
    unsigned char internat_delay;
    unsigned char fixed_id;
    unsigned char cabs;
    unsigned char min_on_hook;
    unsigned char sil_thresh_high;
    unsigned char sil_thresh_low;
    unsigned char pulse_up_thresh;
    unsigned char pulse_low_thresh;
    unsigned char ring_on_high;
    unsigned char ring_on_low;
    unsigned char ring_off_high;
    unsigned char ring_off_low;
    unsigned char reorder_busy;
    unsigned char reorder_var;
    unsigned char busy_high_thresh;
    unsigned char busy_low_thresh;
    unsigned char busy_sum;
    unsigned char busy_ratio;
    unsigned char filt_lev;
    unsigned char agc_min_high;
    unsigned char agc_min_low;
    unsigned char blind_dial;
    unsigned char int_dial_coefficients[12];
    unsigned char cp_coefficients[12];
    unsigned char sit_1st_coefficients[12];
    unsigned char sit_2nd_coefficients[12];
    unsigned char sit_3rd_coefficients[12];
    unsigned char int_dial_coefficients_114[8];
    unsigned char cp_coefficients_114[8];
    unsigned char sit_1st_coefficients_114[8];
    unsigned char sit_2nd_coefficients_114[8];
    unsigned char sit_3rd_coefficients_114[8];
    unsigned char max_bitrate;
    unsigned char scan_time;
    unsigned char gen_ring_on;
    unsigned char gen_ring_off;
    unsigned char gen_busy_on;
    unsigned char gen_busy_off;
    unsigned char gen_reorder_on;
    unsigned char gen_reorder_off;
    unsigned char fixed_ced;
    unsigned char loop_seizure;
    unsigned char loop_max_break;
    unsigned char pulse_min_break;
    unsigned char pulse_max_make;
    unsigned char pulse_min_make;
    unsigned char max_interdigit;
    unsigned char pre_wink;
    unsigned char wink_time;
    char dtmf_thresh_other;
    unsigned char dtmf_len_min;
    unsigned char dtmf_interdigit_min;
    unsigned char t30_behavior;
    unsigned char post_wink;
    unsigned char jp_did_oh_time;
    unsigned char busy_ratio_lh;
    unsigned char max_sil_timeout;
    unsigned char dl_configuration;
    unsigned char dl_wrong_limit;
    unsigned char dl_unsucc_limit;
    unsigned char dl_sum_limit;
    unsigned char dl_max_limit;
    /* Re-align on a 2-byte boundary */
    unsigned char fill_junk1;
    unsigned short dl_wr_lmt_time;
    unsigned short dl_uns_lmt_time;
    unsigned short dl_min_wrong_first;
    unsigned short dl_min_wrong;
    unsigned short dl_min_unsucc_first;
    unsigned short dl_min_unsucc;
    unsigned char v_rec_busy_pow;
    unsigned char v_rec_busy_dur;
    unsigned char v_rec_busy_diff;
    unsigned char v_play_busy_pow;
    unsigned char v_play_busy_dur;
    unsigned char v_play_busy_int;
    unsigned char v_play_busy_num;
    unsigned char v_play_busy_rep;
    unsigned char v_rp_jpn_dial;
    unsigned char v_rp_cng_pow;
};

/* Above values are all printed by shoparam.c via MillLineDumpStructure(). */


#pragma pack()

#endif          /* _phone_h_ */
