/*
 * t434.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

#ifndef _t434_h_
#define _t434_h_

#include <time.h>

#ifdef T434_STANDALONE
#ifndef __OS2__
#define M_APIE1
#define M_APIE2
#define M_CB1
#define M_CB2
#define M_APIG1
#define M_APIG2
#else
#define M_APIE1
#define M_APIE2 _System
#define M_CB1
#define M_CB2 _System
#define M_APIG1
#define M_APIG2
#endif
#endif

#define ASN1_IDENTIFIER_UNIVERSAL       0x00
#define ASN1_IDENTIFIER_APPLICATION     0x40
#define ASN1_IDENTIFIER_CONTEXTUAL      0x80
#define ASN1_IDENTIFIER_PRIVATE         0xc0
#define ASN1_IDENTIFIER_PRIMITIVE       0x00
#define ASN1_IDENTIFIER_CONSTRUCTED     0x20
#define ASN1_IDENTIFIER_TAG             0x1f


#define ASN1_LENGTH_LONG_FLAG           0x80

#define TAG_BINARY_DATA_MESSAGE         23
#define TAG_T434_MSG \
    (TAG_BINARY_DATA_MESSAGE | \
     ASN1_IDENTIFIER_APPLICATION | ASN1_IDENTIFIER_CONSTRUCTED)

enum ASN1_Tags {
    ASN1_EndOfContents = 0,
    ASN1_Boolean = 1|ASN1_IDENTIFIER_UNIVERSAL|ASN1_IDENTIFIER_PRIMITIVE,
    ASN1_Integer,
    ASN1_BitString,
    ASN1_OctetString,
    ASN1_Null,
    ASN1_ObjectIdentifier,
    ASN1_ObjectDescriptor,
    ASN1_External,
    ASN1_Real,
    ASN1_Enumerated,
    ASN1_reserved_11,
    ASN1_reserved_12,
    ASN1_reserved_13,
    ASN1_reserved_14,
    ASN1_reserved_15,
    ASN1_Sequence,
    ASN1_Set,
    ASN1_NumericString,
    ASN1_PrintableString,
    ASN1_TeletexString,
    ASN1_VideotexString,
    ASN1_IA5String,
    ASN1_UTCTime,
    ASN1_GeneralizedTime,
    ASN1_GraphicString,
    ASN1_VisibleString,
    ASN1_GeneralString
};

#define TAG_T434_SEQ \
    (ASN1_IDENTIFIER_CONSTRUCTED | ASN1_Sequence)

#define ASN1_USER_DEFINED 0xff
#define ASN1_END_OF_SEQ 0xfe

enum T434_Tags {
    T434_filename =
        0|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_permitted_actions =
        1|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_PRIMITIVE,
    T434_contents_type =
        2|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_storage_account =
        3|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_PRIMITIVE,
    T434_dtime_creation,
    T434_dtime_last_modification,
    T434_dtime_last_read_access,
    T434_reserved_7,
    T434_identity_of_creator,
    T434_identity_of_last_modifier,
    T434_identity_of_last_reader,
    T434_reserved_11,
    T434_reserved_12,
    T434_filesize,
    T434_future_filesize,
    T434_access_control =
        15|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_legal_qualifications =
        16|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_PRIMITIVE,
    T434_private_use =
        17|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_structure =
        18|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_PRIMITIVE,
    T434_application_reference =
        19|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_machine,
    T434_operating_system =
        21|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_PRIMITIVE,
    T434_recipient =
        22|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_character_set =
        23|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_PRIMITIVE,
    T434_compression =
        24|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_environment,
    T434_pathname,
    T434_store_and_forward,
    T434_protocol_version =
        28|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_PRIMITIVE,
    T434_user_visible_string =
        29|ASN1_IDENTIFIER_CONTEXTUAL|ASN1_IDENTIFIER_CONSTRUCTED,
    T434_data_file_content,
    T434_file_retrieval
};

#define BAD_TAG 0xff

#define LEN_LEN 5
#define TAG_LEN 1

#define LENGTH_INDEFINITE 0x7fffffffL

#define EMITSTRING_BUFSIZE 512

/* Used for emitting */
#define LENGTH_HIGH_BIT 0x80000000L
#define LENGTH_NEXT_HIGH_BIT 0x40000000L

#define BT_MAX_T434_VERSION 2

/* Pack structures used by both API and API applications, in case applications
   compiled with non-standard packing on cmd line. */
#pragma pack(1)

#define GEN_TIME_DIFF_LOCAL 32000
struct gentm {
    int time_diff_mins;
    struct tm tm;
};


/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_t434 {
#ifndef T434_STANDALONE
    RES res;
#endif
    FILE *fp;
    M_CB1 int (* M_CB2 parse_func)
         (unsigned tag,unsigned subtag,long length,
          long data,char *arg);
    M_CB1 unsigned char *(* M_CB2 emit_func)
         (unsigned tag,unsigned subtag,unsigned char *buf,
          long *length,char *arg);
    char *arg;
    unsigned tag;
    unsigned t434_version;
    int raw_content;
    int tag_compat;
    int emit_year2;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

#pragma pack()


#ifdef T434_STANDALONE
#define BfvT434ParseFile MillT434ParseFile
#define BfvT434EmitFile MillT434EmitFile
#define BfvT434TagName MillT434TagName
#endif

#endif

#ifdef MILL_DO_PROTOS

B_APIE1 int B_APIE2 MillT434ParseFile (struct args_t434 *args);
B_APIE1 int B_APIE2 MillT434EmitFile (struct args_t434 *args);
B_APIE1 char * B_APIE2 MillT434TagName (struct args_t434 *args);

#endif
