/* tiff.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All Rights Reserved
 */

#ifndef _tiff_h_
#define _tiff_h_

/* stdio.h needed for tiff.h */
#ifndef EOF
#include <stdio.h>
#endif

/* Tiff tag values. Includes all that are used for Fax, plus all others
   from TIFF 6.0 spec. */
#define TAG_NEWSUBFILETYPE      254
#define TAG_SUBFILETYPE         255
#define TAG_IMAGEWIDTH          256
#define TAG_IMAGELENGTH         257
#define TAG_BITSPERSAMPLE       258
#define TAG_COMPRESSION         259
#define TAG_PHOTOMETRICINTERP   262
#define TAG_THRESHHOLDING       263
#define TAG_CELLWIDTH           264
#define TAG_CELLLENGTH          265
#define TAG_FILLORDER           266
#define TAG_DOCUMENTNAME        269
#define TAG_IMAGEDESCRIPTION    270
#define TAG_MAKE                271
#define TAG_MODEL               272
#define TAG_STRIPOFFSETS        273
#define TAG_ORIENTATION         274
#define TAG_SAMPLESPERPIXEL     277
#define TAG_ROWSPERSTRIP        278
#define TAG_STRIPBYTECOUNTS     279
#define TAG_XRESOLUTION         282
#define TAG_YRESOLUTION         283
#define TAG_PLANARCONFIG        284
#define TAG_PAGENAME            285
#define TAG_XPOSITION           286
#define TAG_YPOSITION           287
#define TAG_FREEOFFSETS         288
#define TAG_FREEBYTECOUNTS      289
#define TAG_GRAYRESPONSEUNIT    290
#define TAG_GRAYRESPONSECURVE   291
#define TAG_T4OPTIONS           292
#define TAG_GROUP3OPTIONS       292
#define TAG_T6OPTIONS           293
#define TAG_GROUP4OPTIONS       293
#define TAG_RESOLUTIONUNIT      296
#define TAG_PAGENUMBER          297
#define TAG_TRANSFERFUNCTION    301
#define TAG_SOFTWARE            305
#define TAG_DATETIME            306
#define TAG_ARTIST              315
#define TAG_HOSTCOMPUTER        316
#define TAG_PREDICTOR           317
#define TAG_WHITEPOINT          318
#define TAG_PRIMARYCHROM        319
#define TAG_COLORMAP            320
#define TAG_HALFTONEHINTS       321
#define TAG_TILEWIDTH           322
#define TAG_TILELENGTH          323
#define TAG_TILEOFFSETS         324
#define TAG_TILEBYTECOUNTS      325
#define TAG_BADFAXLINES         326
#define TAG_CLEANFAXDATA        327
#define TAG_CONSECBADFAXLINES   328
#define TAG_INKSET              332
#define TAG_INKNAMES            333
#define TAG_NUMBEROFINKS        334
#define TAG_DOTRANGE            336
#define TAG_TARGETPRINTER       337
#define TAG_EXTRASAMPLES        338
#define TAG_SAMPLEFORMAT        339
#define TAG_SMINSAMPLEVALUE     340
#define TAG_SMAXSAMPLEVALUE     341
#define TAG_TRANSFERRANGE       342
#define TAG_JPEGPROC            512
#define TAG_JPEGINTCHGFMT       513
#define TAG_JPEGINTCHGFMTLEN    514
#define TAG_JPEGRESTARTINTERVAL 515
#define TAG_JPEGLLPREDICTORS    517
#define TAG_JPEGPOINTTRANSFORMS 518
#define TAG_JPEGQTABLES         519
#define TAG_JPEGDCTABLES        520
#define TAG_JPEGACTABLES        521
#define TAG_YCBCRCOEFFICIENTS   529
#define TAG_YCBCRSUBSAMPLING    530
#define TAG_YCBCRPOSITIONING    531
#define TAG_REFERENCEBLACKWHITE 532
#define TAG_COPYRIGHT           (unsigned)33432

#define FT_BYTE         1
#define FT_ASCII        2
#define FT_SHORT        3
#define FT_LONG         4
#define FT_RATIONAL     5
#define FT_SBYTE        6
#define FT_UNDEFINED    7
#define FT_SSHORT       8
#define FT_SLONG        9
#define FT_SRATIONAL    10
#define FT_FLOAT        11
#define FT_DOUBLE       12

#define TIFF_RES_UNIT_INCHES    2
#define TIFF_RES_UNIT_CM        3

/* These structures are used for file I/O, so we must ensure that they appear
   the same under all OS's/compilers, so we pack it on an alignment of 1.
   Or structure is used by both API and API applications, so must pack
   in case applications compiled with non-standard packing on cmd line. */
#pragma pack(1)

struct ifd_field {
    unsigned short tag; /* Tag type, usually one of the TAG_... values */
    unsigned short field_type; /* Field type, one of the FT_... values */
    unsigned count;     /* Number of values of the field type */
    unsigned offset;    /* Offset for values, or the value(s) themselves */
};

typedef struct {
    FILE *fp;
    int bytes_reversed;
    unsigned cur_offset_offset;
    int page_num;
    unsigned strip_offsets;
    int offset_ft;
    unsigned strip_bytecounts;
    int count_ft;
    int num_strips;
    int fillorder;
    struct ifd_field *fields;
    int bytes_in_strip;
    unsigned cur_strip_offset;
    int width;
    int resolution;
} TFILE;


/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

struct args_tiff {
    RES res;
    MCONST char *fname;
    MCONST char *fmode;
    TFILE *tp;
    unsigned char *buf;
    unsigned size;
    MCONST struct ifd_field *ifd_field;
    MCONST struct ifd_field *horiz_ifd_field;
    MCONST struct ifd_field *vert_ifd_field;
    int resolution;
    long horiz_offset;
    long vert_offset;
    M_CB1 int (* M_CB2 func)
         (TFILE *tp, MCONST struct ifd_field *ifd_ptr, char *arg);
    char *arg;
    unsigned res_unit;
};

/* All new fields for an argument structure must be added to the end of
   the structure. Compatibility support must also be added to appropriate
   internal source files, as well.
*/

#pragma pack()


/* Macro to allow usage of the file pointer field of a TFILE * pointer. */
#define TIFF_FP(tp) ((tp)->fp)


#endif /* _tiff_h_ */


#ifdef MILL_DO_PROTOS

B_APIE1 TFILE * B_APIE2 MillTiffOpen (struct args_tiff *args);
B_APIE1 int B_APIE2 MillTiffClose (struct args_tiff *args);
B_APIE1 int B_APIE2 MillTiffReadIFD (struct args_tiff *args);
B_APIE1 int B_APIE2 MillTiffReadImage (struct args_tiff *args);
B_APIE1 int B_APIE2 MillTiffReadRes (struct args_tiff *args);
B_APIE1 int B_APIE2 MillTiffWriteIFD (struct args_tiff *args);
B_APIE1 int B_APIE2 MillTiffWriteImage (struct args_tiff *args);
B_APIE1 void B_APIE2 MillTiffWriteRes (struct args_tiff *args);

#endif
