/***********************************************************************
 * FILENAME:         vttyapp.c
 *
 * CURRENT VERSION:  %version: lgca~2 %
 * DATE MODIFIED:    %date_modified: Tue Oct 15 11:08:57 2002 %
 *
 * CREATION DATE:    Tue Oct 15 11:08:57 2002
 *
 * PURPOSE:
 *   VTTY utility.
 *  
 *
 * Copyright (c) 2002 Brooktrout Technology, Inc. All Rights Reserved.
 *
 * This software is the property of Brooktrout Technology, Inc.
 * Any reproduction, disclosure, or distribution of this software is
 * expressly prohibited unless first agreed to in writing by Brooktrout
 * Technology, Inc.
 ***********************************************************************/

#ifndef lint
/* Version string for the "what" program */
static char *_vttyapp_c = "@(#) %name: vttyapp.c %  %version: lgca~2 %  %date_modified: Tue Oct 15 11:08:57 2002 %  BRKT";
#endif

#include <signal.h>
#include <stddef.h>
#include <ctype.h>
#include <string.h>
#include "bsmilib.h"
#include "osilib.h"
#include "getopt.h"


#define VTTY_CMD_BUFF_SIZE 180
HANDLE  board;
typedef  void* (*THREAD_START_FN_T)(void*);

void RemoveLeadingSpaces( char **pCmdLine)
{
	while( **pCmdLine ==' ')
		(*pCmdLine)++;
	return;
}
void RemoveTrailingSpaces( char *cmdLine)
{
	while( cmdLine[strlen(cmdLine)-1]==' ')
		cmdLine[strlen(cmdLine)-1]='\0';
	return;
}

/* Ascii to hex conversion  */
int atox(char *str)
{
   char c;
   int val = 0;

   while ((((c = *str++) <= 'f') && (c >= 'A')) ||
		  ((c >= '0') && (c <= '9'))) 
   {
	  c &= 0x4f;
	  val = (val << 4) + ((c < 10) ? c : c - 55);
   }
   return val;
}

/* Signal Handler */
void SigHdlr( int sig)
{
	int ret = 0;
	// Allow reception of packets in pipe
	osiWait(100);
	printf("\n Signal=%d received, VTTY Quitting ....\n", sig);	
	if ( (ret=BsmiClearVtty( board)))
	{
		printf("\n BsmiClearVtty failed, Return Value=%d", ret);
		// BsmiCloseAdapter( board);
		exit(9);
	}
	/*if ((ret=BsmiCloseAdapter( board)))
	{
		printf("\n BsmiCloseAdapter failed, Return Value=%d", ret);
		exit(10);
	}
	*/
	exit(0);
}


/* Thread receiving packets from VTTY facility  */
void* VttyReader( void* board)
{
	char buffer[VT_MILL_PKT_PAYLOAD_SIZE];
	int length=0, i=0;
	while( 1)
	{
		memset( buffer, 0, sizeof(buffer));
		length =0;
		/* Read packet incoming from VTTY facility */
		if ( (i=BsmiVttyRead( board, buffer, &length)))
		{
			printf("\n VttyReader:BsmiVttyRead failed, Return Val=%d..exiting..", i);
			exit(8);
		}
		/* Print packet data on screen */
		for (i=0; i<length; i++)
			putchar( buffer[i]);
	}
}

/* VTTY - entry point */
main (argc, argv)
register char **argv;
register int argc;
{
	
	int           ret=0, len=0, modNo =2, debugMode=0;
	char          cmdLine[VTTY_CMD_BUFF_SIZE];
	char          *optionString="m:v:h?", *sptr;
	unsigned long thrd_id;
	

	// Parse all command line options.
	while (1)
	{
		ret = GetOption(argc, argv, optionString, &sptr);

		// RetVal = zero means we are finished.

		if (ret == 0)
			break;

		// Errors exit.

		if ((ret == 1) || (ret == -1))
		{
			printf("\tInvalid argument, type vttyapp -h for options...\n", ret);
			exit(1);
		}

		//	A valid option has been entered.
		switch (ret)
		{	
			case 'm':
				if (sptr == NULL)					// must have a value
					continue;
				modNo = atox(sptr);
				printf("\n Module No is = 0x%0x\n", modNo);
				break;
			case 'v':
				debugMode = 1;
				printf("\n Verbose Mode \n");
				break;
			case 'h':
			case '?':
				printf("\n\t-m <N> module number in hex(default = 2)");
				printf("\n\t-v verbose, enables Bfv API Debug mode");
				printf("\n\t-h print help\n");
				exit(0);
				break;
			default:
				break;
		}
	}
	if ( debugMode)
	{
		BfvDebugModeSet( DEBUG_ALL);
	}

	// Open adapter
	board = BsmiOpenAdapter( BSMI_BUS_NUMBER, modNo, NULL);
	if ( board == (HANDLE)INVALID_HANDLE_VALUE)
	{
		printf("\n BsmiOpenAdapter failed, ret=%d", board);
		exit(2);
	}

	/* Start interaction with VTTY facility*/
	if ( (ret=BsmiSetVtty( board)))
	{
		
		if ( -ret == ECCERR_BUSY)
		{
			/* User has choice to override working VTTY facility */
			printf("\n VTTY FACILITY is BUSY\n Do you want to override?(y/n):");
			memset( cmdLine, 0, sizeof(cmdLine));
			fgets(cmdLine, VTTY_CMD_BUFF_SIZE, stdin);
			/* replace newline character with NULL 
			if ((strchr(cmdLine,'\n')))
				*(strchr(cmdLine,'\n'))='\0';
			*/
			sptr = cmdLine;
			RemoveLeadingSpaces(&sptr);
			RemoveTrailingSpaces(sptr);
			if ( strcmp( sptr, "y\n") &&
				strcmp( sptr, "Y\n") )
			{
				// If this is 2nd VTTY app..
				// Will BsmiCloseAdapter STOP the ISDN_TX_RX for 
				// all applications ???
				// BsmiCloseAdapter( board);
				exit(3);
			}
			// Override the other session by doing a ClearVtty and
			// then SetVtty
			printf("\n Overriding the other VTTY session...\n");
			if ( (ret=BsmiClearVtty( board)))
			{
				printf("\n BsmiClearVtty failed, return=%d", ret);
				exit( 3);
			}
			if ( (ret=BsmiSetVtty( board)))
			{
				printf("\n BsmiSetVtty failed, return=%d", ret);
				exit( 3);
			}
		}
		else
		{
			printf("\n BsmiSetVtty failed, ret=%d", ret);
			//BsmiCloseAdapter( board);
			exit(3);
		}
	}
	/* Handle following signals */
	signal( SIGABRT, SigHdlr);
	signal( SIGFPE,  SigHdlr);
	signal( SIGILL,  SigHdlr);
	signal( SIGINT,  SigHdlr);
	signal( SIGSEGV, SigHdlr);
	signal( SIGTERM, SigHdlr);

	// Start reading packets from Board
	if ( osiThread_Create 
			(   NULL,        // void*  stack_start
				0,           // int   stack_size -default
				(THREAD_START_FN_T)VttyReader,  
				(void*)board,// void* arguments, 
				0,           // int   flags, 
				(unsigned long*)&thrd_id     // unsigned long* thr_id
			) )
	{
		printf("\n osiThread_Create failed");
		BsmiClearVtty( board);
		//BsmiCloseAdapter( board);
		exit (4);
	}
	
	while (1) 
	{
		memset( cmdLine, 0, sizeof(cmdLine));
		fgets(cmdLine, VTTY_CMD_BUFF_SIZE, stdin);	
		/* Check that this is not a command to quit VTTY session */
		sptr = cmdLine;
		RemoveLeadingSpaces(&sptr);
		RemoveTrailingSpaces(sptr);
		if ( (strcmp( sptr, "quit\n") == 0)||
			(strcmp( sptr, "QUIT\n") == 0))
		{
			printf("\n VTTY Quitting ....\n");
			// osiWait(100);
			if ( (ret=BsmiClearVtty( board)))
			{
				printf("\n BsmiClearVtty failed, Return Value=%d", ret);
				// BsmiCloseAdapter( board);
				exit(6);
			}
			/*
			if ((ret=BsmiCloseAdapter( board)))
			{
				printf("\n BsmiCloseAdapter failed, Return Value=%d", ret);
				exit(7);
			}
			*/
			/* quit command */
			exit(0);
		}
		len = strlen(cmdLine);
		if ( ret=BsmiVttyWrite( board, cmdLine, &len))
		{
			printf("\n BsmiVttyWrite failed, ret=%d", ret);
			BsmiClearVtty( board);
			// BsmiCloseAdapter( board);
			exit(5);
		}
	}
}

