/***********************************************************************
 * FILENAME:         bsmilib.h
 *
 * CURRENT VERSION:  %version: lgca~33 %
 * DATE MODIFIED:    %date_modified: Fri Jul 15 13:16:49 2005 %
 *
 * PURPOSE:
 *   Include file for the BSMI library
 *   This file contains the constants and structure definitions that
 *   are used throughout.
 *
 * Copyright (c) 1997-2004 Brooktrout Technology, Inc. All Rights Reserved.
 *
 * This software is the property of Brooktrout Technology, Inc.
 * Any reproduction, disclosure, or distribution of this software is
 * expressly prohibited unless first agreed to in writing by Brooktrout
 * Technology, Inc.
 ***********************************************************************/


#include <stdio.h>
#include <string.h>


#include "btlib.h"

// BSMI only needs L4_to_L3_struct and L3_to_L4_struct
#include "iisdn.h"

////////////// Constants
#define	BSMI_BUS_NUMBER 0xfe


#include "../../bfv.api/inc/sdk_version.h"

// define version
#define	BSMI_API_MAJOR_VERSION		5
#define	BSMI_API_MIDDLE_VERSION		1
#define	BSMI_API_MINOR_VERSION		0
#define	BSMI_API_BUILD_NUMBER		SDK_BUILD_NUMBER

#define	BSMI_API_COPYRIGHT_STR		"Copyright (c) 1997-2004 Brooktrout Technology"

//  Create version string in the format "1.2.3B4"
//  (major.middle.minorBbuild)
#define BSMI_API_VERSION2(a,b,c,d)	#a "." #b "." #c "B" #d
#define BSMI_API_VERSION1(a,b,c,d)	BSMI_API_VERSION2(a,b,c,d)
#define BSMI_API_VERSION		BSMI_API_VERSION1(BSMI_API_MAJOR_VERSION, BSMI_API_MIDDLE_VERSION, BSMI_API_MINOR_VERSION, SDK_BUILD_NUMBER)


// Maintain backward compatibility. Deprecated, do not use for future development:
#define BSMI_VER_MAJOR		BSMI_API_MAJOR_VERSION
#define BSMI_VER_MIDDLE		BSMI_API_MIDDLE_VERSION	
#define BSMI_VER_MINOR		BSMI_API_MINOR_VERSION
#define BSMI_VER_BUILD		SDK_BUILD_NUMBER
#define BSMI_VER_DATE		__DATE__

// Macro to create a number 'abcd' from individual components
#define BSMI_VER_NUM2(a,b,c,d)	a ## b ## c ## d
#define BSMI_VER_NUM1(a,b,c,d)	BSMI_VER_NUM2(a,b,c,d)
#define BSMI_VER_NUM		BSMI_VER_NUM1(BSMI_API_MAJOR_VERSION, BSMI_API_MIDDLE_VERSION, BSMI_API_MINOR_VERSION, SDK_BUILD_NUMBER)

// The macro BSMI_VERSION expands to a series of strings
// in the form "1.2.3B4" .
#define BSMI_VERSION2(a,b,c,d)	#a "." #b "." #c "B" #d
#define BSMI_VERSION1(a,b,c,d)	BSMI_VERSION2(a,b,c,d)
#define BSMI_VERSION		BSMI_VERSION1(BSMI_API_MAJOR_VERSION, BSMI_API_MIDDLE_VERSION, BSMI_API_MINOR_VERSION, SDK_BUILD_NUMBER)

#define BSMI_COPYRIGHT_STR	BSMI_API_COPYRIGHT_STR
// End of backward compatibility part



////////// Nai Compatibility
#ifdef BTCOMPATNAI
#define NaiOpenAdapter(x,y) BsmiOpenAdapter(x,y,NULL)
#define NaiCloseAdapter BsmiCloseAdapter
#define NaiResetAdapter(x) BsmiResetAdapter(x,NULL)
#define NaiDownLoadAdapter BsmiDownLoadAdapter
#define NaiControlRead(x,y) BsmiControlRead(x,y,NULL)
#define NaiControlWrite BsmiControlWrite
// Gemini-Added directives for compatibility with NAI VTTY functions
#define NaiSetVtty      BsmiSetVtty
#define NaiClearVtty    BsmiClearVtty
#define NaiVttyRead     BsmiVttyRead
#define NaiVttyWrite    BsmiVttyWrite
#endif


// OS Specific
#ifndef WINNT

#include <errno.h>	// To replace NT functionality of SetLastError

// The following #define should be thread safe.  This according to the
// Sun OS 5.7 man page intro(2).  It states that errno is a macro that
// is thread safe and that can appear on either side of the equal sign.
#define SetLastError(x) (errno=x)
typedef void *HANDLE; // For NT, defined in windows.h

// For NT most are defined in winerror.h, other OSs are provided here
#define INVALID_HANDLE_VALUE		-1 // For NT, defined in winbase.h
#define ERROR_SUCCESS			0L
#define ERROR_NOT_ENOUGH_MEMORY		8L
#define ERROR_BAD_COMMAND		22L
#define ERROR_NOT_SUPPORTED		50L
#define ERROR_UNEXP_NET_ERR		59L
#define ERROR_INVALID_PARAMETER		87L
#define ERROR_MOD_NOT_FOUND		126L
#define ERROR_INVALID_MODULETYPE	190L
#define ERROR_TOO_MANY_MODULES		214L
#define ERROR_OPERATION_ABORTED		995L
#define ERROR_CAN_NOT_COMPLETE		1003L
#define ERROR_IO_DEVICE			1117L
#define ERROR_CONNECTION_UNAVAIL	1201L
#define ERROR_EXTENDED_ERROR		1208L
#define ERROR_CANCELLED			1223L
#define ERROR_CONNECTION_REFUSED	1225L
#define ERROR_GRACEFUL_DISCONNECT	1226L
#define ERROR_REVISION_MISMATCH		1306L
#define ERROR_FILE_CORRUPT		1392L
#define ERROR_TIMEOUT			1460L
#define ERROR_NOT_CONNECTED		2250L
#endif	

/*     OS Indepdent defines         */
/*         4/25/01 BW               */
/* -------------------------------- */
/* An unclassified error */
#define ECCERR_UNCLASSIFIED 5000L + (long)APIERR_UNCLASSIFIED
/* A file open/read/write error */
#define ECCERR_FILEIO 5000L + (long)APIERR_FILEIO
/* A file is not in the correct format */
#define ECCERR_FILEFORMAT 5000L + (long)APIERR_FILEFORMAT
/* The hardware or firmware does not have a required capability */
#define ECCERR_BOARDCAPABILITY 5000L + (long)APIERR_BOARDCAPABILITY
/* The channel is not in a connected state, but is required to be */
#define ECCERR_NOTCONNECTED 5000L + (long)APIERR_NOTCONNECTED
/* An invalid parameter has been passed */
#define ECCERR_BADPARAMETER 5000L + (long)APIERR_BADPARAMETER
/* A memory allocation error occurred */
#define ECCERR_MEMORY 5000L + (long)APIERR_MEMORY
/* The channel is not in a required state */
#define ECCERR_BADSTATE 5000L + (long)APIERR_BADSTATE
/* Dialing was attempted too soon */
#define ECCERR_TOOSOON 5000L + (long)APIERR_TOOSOON
/* Resource busy */
#define ECCERR_BUSY 5000L + (long)APIERR_BUSY
/* Driver open call failed, invalid channel or driver
   not correctly installed */
#define ECCERR_DRV_OPEN_ERROR 5000L + (long)APIERR_DRV_OPEN_ERROR
/* Driver call failed */
#define ECCERR_DRV_IOCTL_ERROR 5000L + (long)APIERR_DRV_IOCTL_ERROR
/* Incompatible driver version */
#define ECCERR_VERSION 5000L + (long)APIERR_VERSION
/* Invalid port; faxinit probably not run correctly */
#define ECCERR_INVALID_PORT 5000L + (long)APIERR_INVALID_PORT
/* Ringing during dialing attempt */
#define ECCERR_RINGING 5000L + (long)APIERR_RINGING
/* Indir infopkt nesting level too deep */
#define ECCERR_INFOPKT_NESTING 5000L + (long)APIERR_INFOPKT_NESTING
/* Maximum number of TIFF tags exceeded */
#define ECCERR_MAX_TAGS 5000L + (long)APIERR_MAX_TAGS
/* An attempt to gain a lock failed */
#define ECCERR_LOCK_FAILED 5000L + (long)APIERR_LOCK_FAILED
/* Buffer size too small to receive data */
#define ECCERR_INSUFF_BUFFER 5000L + (long)APIERR_INSUFF_BUFFER
/* Destination address not found or invalid */
#define ECCERR_INVALID_DEST_ADDR 5000L + (long)APIERR_INVALID_DEST_ADDR
/* Packet or command creation error */
#define ECCERR_PACKET_CREATION 5000L + (long)APIERR_PACKET_CREATION
/* Packet or command parse error */
#define ECCERR_PACKET_PARSE 5000L + (long)APIERR_PACKET_PARSE
/* Packet send error */
#define ECCERR_PACKET_SEND 5000L + (long)APIERR_PACKET_SEND
/* Packet receive error */
#define ECCERR_PACKET_RECEIVE 5000L + (long)APIERR_PACKET_RECEIVE
/* DATA encountered during command processing */
#define ECCERR_DATA 5000L + (long)APIERR_DATA
/* Invalid parameter values received from board */
#define ECCERR_INVAL_BOARD_PARAM 5000L + (long)APIERR_INVAL_BOARD_PARAM
/* Firmware detected an error */
#define ECCERR_FIRMWARE_ERR_DETECTED 5000L + (long)APIERR_FIRMWARE_ERR_DETECTED

/////////// Structures

struct args_mod {
	unsigned char *list;
	int	list_size;
	int total;
};

struct args_open {
	unsigned write_buf_size;
	unsigned read_buf_size;
};

struct args_reset {
	unsigned char facil_ver_major;	// IISDN version info from module
	unsigned char facil_ver_minor;
	unsigned char facil_ver_build;
	// facil_ver_interface, Added by Apollo-4710
	unsigned char facil_ver_interface; 
	unsigned char host_ver_major;	// IISDN version when BSMI library was compiled
	unsigned char host_ver_minor;
	unsigned char host_ver_build;
	// host_ver_interface, Added by Apollo-4710
	unsigned char host_ver_interface;
	int bsmi_ver_num;				// BSMI version when BSMI library was compiled
	unsigned char no_config;
	char *config_file_name;
};

struct args_bsmi {
	unsigned timeout;
	int set_read_dump_func; 
	void (*read_dump_func)(int , struct args_packet *, unsigned char *, unsigned );
	int set_write_dump_func; 
	void (*write_dump_func)(int , struct args_packet *, unsigned char *, unsigned );
};

struct bsmiline {
	BTLINE *ctrl_write;
	BTLINE *ctrl_read;
	void (*read_dump_func)(int , struct args_packet *, unsigned char *, unsigned );
	void (*write_dump_func)(int , struct args_packet *, unsigned char *, unsigned );
};

typedef struct bsmiline BSMILINE;

struct args_alert {
	int type;
	unsigned char alert_value;
};

#define BSMI_ALERT_READ 0
#define BSMI_ALERT_WRITE 1

HANDLE
BsmiOpenAdapter(int bus_number, int slot_number, struct args_open *args);

int
BsmiCloseAdapter(HANDLE handle);

int
BsmiResetAdapter(HANDLE handle,struct args_reset *args);

int
BsmiLastErrorTranslate(RES *res);

int
BsmiModuleList(struct args_mod *args);

int
BsmiLineAlert(HANDLE board, struct args_alert *args);

int
BsmiControlRead(HANDLE handle, L3_to_L4_struct *buffer,struct args_bsmi *args);

int
BsmiControlWrite(HANDLE handle, L4_to_L3_struct *buffer);

void
BsmiDumpMessage(int direction, struct args_packet *args, unsigned char *msg, unsigned msg_len);

void
BsmiDumpFuncSet(HANDLE board, struct args_bsmi *args);

/* Following definitions added for Gemini  */
//
// VTTY definitions
// VTTY packet size
#define VT_MILL_PKT_PAYLOAD_SIZE 1006

int 
BsmiSetVtty  (HANDLE board);

int
BsmiClearVtty( HANDLE board);

int 
BsmiVttyRead ( HANDLE board, char* buffer, int* length);

int 
BsmiVttyWrite ( HANDLE board, char* buffer, int* length);

