/******************************************************************************
 * r2Brazil.h - Brazil R2 parameters
 *
 *  Description:
 *
 *      This include file contains the parameters needed to configure IISDN's
 *      R2-MFC for operation in Brazil.
 *
 *      Certain parameters are network specific and cannot be modified.
 *
 *  (C) - Copyright Brooktrout Technology Inc. (Data Technology Division) 2000
 ******************************************************************************
 */

#ifndef _R2Brazil_H_
#define _R2Brazil_H_

#define R2_MF_PARAM_NOT_USED    (0) /* clarify params not currently implemented */

static IISDN_E1_CAS_R2_DATA r2_Brazil = 
{
    // IISDN_R2_DIGITAL_LINE_SIG_PARAMS: 
    200,    /* (unsigned short) r2OutSeizeTimer (ms) */
    90,     /* (unsigned short) r2OutAnswerTimeOut (s) */  
    200,    /* (unsigned short) inboundReleaseGuardTime */
    0,      /* (unsigned short) inboundLineQualTimerIdle */  
    0,      /* (unsigned short) DebugBitMask */  

    1,      /* (unsigned char)  c_d_cas_bits ------> C=0 D=1 */
    0,      /* (unsigned char)  inboundClearbackControl ----> release guard / forced release */

    /*
     * IISDN_R2_INTERREGISTER_PARAMS:
     *
     * NOTE!: Only the R2 in-dial application parameters may be modified by the user. All
     *        others are network specific and modification would likely cause malfunction.
     */
    /*
     * R2 Indial Application Parameters: these parameters may be modified by the
     * user according to application requirements.
     */
    7,              /* (unsigned short) dnisMaxNumDigits: max # of DNIS digits required */  
    7,              /* (unsigned short) aniMaxNumDigits: max # of ANI digits required */    
    1,              /* (unsigned short) dnisNumDigitsBeforeANI */

    /*
     * R2 Inbound Timers
     */
    15,             /* (short) interForwardToneTimeOut: units in s or DISABLED */
    IISDN_INBOUND_PULSE_MIN_DELAY_ITU_MSEC, /* (short) prePulseToneDelay (s) */
    IISDN_INBOUND_PULSE_DURATION_ITU_MSEC,  /* (short) pulseToneDuration (s) */

    /*
     * R2 Oubound Timers
     */
    17,     /* (short) forwardToneMaxOnTime: units in sec or DISABLED */    
    27,     /* (short) forwardToneMaxOffTime: units in sec or DISABLED */
    17,     /* (short) forwardGroup2MaxOnTime: units in sec or DISABLED */

    /*
     * R2 Backward Protocol Params
     */
    IISDN_INDIAL_DNIS_TIME_OUT_PULSE_GROUPII_REQUEST, /* (unsigned short) dnisTimeOutAction */

    /*
     * Forward R2 Signals, use IISDN_R2F_XX's 
     */ 
    IISDN_R2F_INVALID,      /* endOfDNIS */
    IISDN_R2F_15,           /* endOfANI_Available */
    IISDN_R2F_12,           /* InfoNotAvailable */
    IISDN_R2F_INVALID,      /* spare1 */
    IISDN_R2F_INVALID,      /* spare2 */
    IISDN_R2F_01,           /* aniCategoryDefault */

    /* 
     * Backward R2 signals, use IISDN_R2B_XX's 
     */
    IISDN_R2B_01,           /* sendNextDigitDNIS */
    IISDN_R2B_09,           /* sendLastButOneDigitDNIS */
    IISDN_R2B_03,           /* sendCallCategoryAndSwitchToGroupB_DNIS */
    IISDN_R2B_04,           /* congestion */
    IISDN_R2B_05,           /* sendCallingPartyCategory */
    IISDN_R2B_INVALID,      /* callComplete_SetUpSpeechPath - Group A */
    IISDN_R2B_07,           /* sendLastButTwoDigitDNIS */
    IISDN_R2B_08,           /* sendLastButThreeDigitDNIS */
    IISDN_R2B_INVALID,      /* spare3 */
    IISDN_R2B_INVALID,      /* changeFrom_ANI_To_DNIS_SendNextDigit_F */
    IISDN_R2B_02,           /* sendFirstDigitDNIS */
    IISDN_R2B_05,           /* sendNextDigitANI */
    IISDN_R2B_INVALID,      /* changeFrom_ANI_To_DNIS_SendNextDigit */
    IISDN_R2B_INVALID,      /* changeFrom_ANI_To_DNIS_SendLastDigit */

        {   /* groupB_LineConditions */
            IISDN_R2B_01,           /* called Line Free, Charge */
            IISDN_R2B_05,           /* called Line Free, No Charge */
            IISDN_R2B_INVALID,      /* alternate called Line Free, Charge */ 
            IISDN_R2B_02,           /* called Line Busy */
            IISDN_R2B_INVALID,      /* called Line Out Of Order */
            IISDN_R2B_07,           /* called Line Unallocated */
            IISDN_R2B_INVALID,      /* spare */
            IISDN_R2B_INVALID,      /* spare */
            IISDN_R2B_INVALID,      /* spare */ 
            IISDN_R2B_INVALID,      /* spare */ 
        },      

    /*
     * Call Progress generation 
     */
    {   /* IISDN_CPGEN_MF_PARAMS */
        {   /* IISDN_CPGEN_MF_PARAMS for RING (cpSignals[0]) */
            425,    /* (unsigned short) freqTone1 */
            0,      /* (short) powerTone1, -10 dBm */
            0,      /* (unsigned short) freqTone2, NONE */
            0,      /* (short) powerTone2 */
            1,      /* (unsigned short) numCadences */
            1000,   /* (short) makeTime1 */
            4000,   /* (short) breakTime1 */
            0,      /* (short) makeTime2 */
            0,      /* (short) breakTime2 */
            0,      /* (short) makeTime3 */
            0,      /* (short) breakTime3 */    
            2,      /* (unsigned short) numCycles, finite duration */
        },
        {   /* IISDN_CPGEN_MF_PARAMS for BUSY (cpSignals[1]) */
            425,    /* (unsigned short) freqTone1 */
            0,      /* (short) powerTone1, -10 dBm */
            0,      /* (unsigned short) freqTone2, NONE */
            0,      /* (short) powerTone2 */
            1,      /* (unsigned short) numCadences */
            250,    /* (short) makeTime1 */
            250,    /* (short) breakTime1 */
            0,      /* (short) makeTime2 */
            0,      /* (short) breakTime2 */
            0,      /* (short) makeTime3 */
            0,      /* (short) breakTime3 */    
            0,      /* (unsigned short) numCycles, repeat indefinitely */
        },
    },

    //  July 2002

    0,              /* BillingPulse */
    0,              /* BillingQualificationTime */
    0,              /* R2Options */

    //  November 2002

    20000           /* reset timer */
};

#endif
