
/****************************************************************************
 *
 * Module:               ATModem.h
 * Date:                 Oct. 24, 2001
 * Purpose:              All definitions global to the AT Modem facility
 ****************************************************************************
 * (c) 1992-2001 Brooktrout Technology, Inc.                                  *
 *                                                                          *
 *                                                                          *
 *   All Rights Reserved.                                                   *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 * This software is furnished under a license and may only be               *
 * read, modified, copied, or otherwise used in accordance with the         *
 * terms of such license. This notice must never be removed from            *
 * this software and must be included with every copy. No other             *
 * rights to reproduce, use, or disseminate this software in any form,      *
 * in whole or in part, are granted.                                        *
 *                                                                          *
 * Brooktrout Technology, Inc. makes no representation or warranties        *
 * with respect to the performance of this software and specifically        *
 * disclaims any responsibility for any damages, special or                 *
 * consequential, connected with the use of this software.                  *
 *                                                                          *
 ****************************************************************************/
 
#ifndef ATMODEM_H
#define ATMODEM_H

/* Facility                   */
#define  MILL_FACILITY_AT_MODEM                0x03

/* Event                      */
#define  AT_MODEM_STATUS_EVENT                 0x02
#define  AT_MODEM_LINE_STATUS_EVENT            0x03
#define  AT_MODEM_FLOW_STATUS_EVENT            0x04
#define  AT_MODEM_RCV_FLUSH_COMPLETE_EVENT     0x05
#define  AT_MODEM_TRANSMITTER_EMPTY_EVENT      0x06
#define  AT_MODEM_STARTED_EVENT                0x07
#define  AT_MODEM_FINISHED_EVENT               0x08

/* Request                    */
#define  AT_MODEM_STATUS_REG_REQUEST           0x02
#define  AT_MODEM_FLOW_STATUS_REQUEST          0x03
#define  AT_MODEM_TRANSMIT_FLUSH_REQUEST       0x04
#define  AT_MODEM_RECEIVE_FLUSH_REQUEST        0x05
#define  AT_MODEM_TRANSMITTER_EMPTY_REQUEST    0x06

/* Start/Finish               */
#define  AT_MODEM_MODE                         0x02

/* Variable/Parameter         */
#define  AT_MODEM_BPS_RATE_VAR                 0x02
#define  AT_MODEM_LINE_CONTROL_VAR             0x03
#define  AT_MODEM_XON_CHAR_VAR                 0x04
#define  AT_MODEM_XOFF_CHAR_VAR                0x05
#define  AT_MODEM_CTRL_REG_VAR                 0x06
#define  AT_MODEM_HNDSHK_FLOW_CTRL_VAR         0x07
#define  AT_MODEM_XON_LIMIT_VAR                0x08
#define  AT_MODEM_XOFF_LIMIT_VAR               0x09
#define  AT_MODEM_STATUS_PAR                   0x0A
#define  AT_MODEM_LINE_STATUS_PAR              0x0B
#define  AT_MODEM_HOLD_REASONS_PAR             0x0C
#define  AT_MODEM_CHARS_RECEIVED_PAR           0x0D
#define  AT_MODEM_CHARS_TRANSMITTED_PAR        0x0E

/* Defines For Tagged Values  */
#define  AT_MODEM_DATA_5_DEF                   0x00
#define  AT_MODEM_DATA_6_DEF                   0x01
#define  AT_MODEM_DATA_7_DEF                   0x02
#define  AT_MODEM_DATA_8_DEF                   0x03
#define  AT_MODEM_DATA_MASK_DEF                0x03
#define  AT_MODEM_STOP_1_DEF                   0x00
#define  AT_MODEM_STOP_1_5_DEF                 0x04
#define  AT_MODEM_STOP_2_DEF                   0x04
#define  AT_MODEM_STOP_MASK_DEF                0x04
#define  AT_MODEM_PARITY_NONE_DEF              0x00
#define  AT_MODEM_PARITY_ODD_DEF               0x08
#define  AT_MODEM_PARITY_EVEN_DEF              0x18
#define  AT_MODEM_PARITY_MARK_DEF              0x28
#define  AT_MODEM_PARITY_SPACE_DEF             0x38
#define  AT_MODEM_PARITY_MASK_DEF              0x38
#define  AT_MODEM_BREAK_DEF                    0x40
#define  AT_MODEM_DTR_DEF                      0x01
#define  AT_MODEM_RTS_DEF                      0x02
#define  AT_MODEM_OUT1_DEF                     0x04
#define  AT_MODEM_OUT2_DEF                     0x08
#define  AT_MODEM_DTR_MASK_DEF                 0x03
#define  AT_MODEM_DTR_CTRL_DEF                 0x01
#define  AT_MODEM_DTR_HNKSHK_DEF               0x02
#define  AT_MODEM_CTS_HNKSHK_DEF               0x08
#define  AT_MODEM_DSR_HNDSHK_DEF               0x10
#define  AT_MODEM_DCD_HNDSHK_DEF               0x20
#define  AT_MODEM_HANDSHAKE_MASK_DEF           0x38
#define  AT_MODEM_DSR_SENSITIVITY_DEF          0x40
#define  AT_MODEM_ERROR_ABORT_DEF              0x8000
#define  AT_MODEM_CTRL_INVALID_DEF             0x7F84
#define  AT_MODEM_AUTO_TRANSMIT_DEF            0x10000
#define  AT_MODEM_AUTO_RECEIVE_DEF             0x20000
#define  AT_MODEM_ERROR_CHAR_DEF               0x40000
#define  AT_MODEM_NULL_STRIPPING_DEF           0x80000
#define  AT_MODEM_BREAK_CHAR_DEF               0x100000
#define  AT_MODEM_RTS_MASK_DEF                 0xC00000
#define  AT_MODEM_RTS_CTRL_DEF                 0x400000
#define  AT_MODEM_RTS_HNDSHK_DEF               0x800000
#define  AT_MODEM_TRANSMIT_TOGGLE_DEF          0xC00000
#define  AT_MODEM_XOFF_CONTINUE_DEF            0x80000000
#define  AT_MODEM_FLOW_INVALID_DEF             0x7F200000
#define  AT_MODEM_INBAND_FC_OUT_DEF            0x1000000
#define  AT_MODEM_INBAND_FC_IN_DEF             0x2000000
#define  AT_MODEM_MSR_DCTS_DEF                 0x01
#define  AT_MODEM_MSR_DDRS_DEF                 0x02
#define  AT_MODEM_MSR_TERI_DEF                 0x04
#define  AT_MODEM_MSR_DDCD_DEF                 0x08
#define  AT_MODEM_MSR_CTS_DEF                  0x10
#define  AT_MODEM_MSR_DSR_DEF                  0x20
#define  AT_MODEM_MSR_RI_DEF                   0x40
#define  AT_MODEM_MSR_DCD_DEF                  0x80
#define  AT_MODEM_ERROR_OVERRUN_DEF            0x02
#define  AT_MODEM_ERROR_PARITY_DEF             0x04
#define  AT_MODEM_ERROR_FRAMING_DEF            0x08
#define  AT_MODEM_ERROR_BREAK_DEF              0x10
#define  AT_MODEM_TX_WAITING_FOR_CTS_DEF       0x01
#define  AT_MODEM_TX_WAITING_FOR_DSR_DEF       0x02
#define  AT_MODEM_TX_WAITING_FOR_DCD_DEF       0x04
#define  AT_MODEM_TX_WAITING_FOR_XON_DEF       0x08
#define  AT_MODEM_TX_WAITING_ON_BREAK_DEF      0x10
#define  AT_MODEM_RX_WAITING_FOR_DSR_DEF       0x20
#define  AT_MODEM_TX_WAITING_XOFF_SENT_DEF     0x40


#endif
