
/****************************************************************************
 *
 * Module:               Sig_Det.h
 * Date:                 Mar. 25, 2005
 * Purpose:              All definitions global to the Signal Detect facility
 ****************************************************************************
 * (c) 1992-2005 Brooktrout Technology, Inc.                                  *
 *                                                                          *
 *                                                                          *
 *   All Rights Reserved.                                                   *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 * This software is furnished under a license and may only be               *
 * read, modified, copied, or otherwise used in accordance with the         *
 * terms of such license. This notice must never be removed from            *
 * this software and must be included with every copy. No other             *
 * rights to reproduce, use, or disseminate this software in any form,      *
 * in whole or in part, are granted.                                        *
 *                                                                          *
 * Brooktrout Technology, Inc. makes no representation or warranties        *
 * with respect to the performance of this software and specifically        *
 * disclaims any responsibility for any damages, special or                 *
 * consequential, connected with the use of this software.                  *
 *                                                                          *
 ****************************************************************************/
   
#ifndef SIG_DET_H
#define SIG_DET_H


/* Facility                   */

#define  MILL_FACILITY_SIGNAL_DETECTION                 0x05


/* Event                      */

#define  SIG_DET_CP_DETECTION_STARTED_EVENT             0x02
#define  SIG_DET_CP_DETECTION_FINISHED_EVENT            0x03
#define  SIG_DET_DTMF_DETECT_STARTED_EVENT              0x04
#define  SIG_DET_TONE_DETECT_STARTED_EVENT              0x04
#define  SIG_DET_DTMF_DETECT_FINISHED_EVENT             0x05
#define  SIG_DET_TONE_DETECT_FINISHED_EVENT             0x05
#define  SIG_DET_SIGNAL_DETECTION_EVENT                 0x06
#define  SIG_DET_DISS_STARTED_EVENT                     0x07
#define  SIG_DET_DISS_FINISHED_EVENT                    0x08
#define  SIG_DET_PCPM_TABLE_LOADED_EVENT               0x502
#define  SIG_DET_PCPM_TABLE_NOT_LOADED_EVENT           0x503
#define  SIG_DET_CALLER_ID_STARTED_EVENT                0x09
#define  SIG_DET_CALLER_ID_FINISHED_EVENT               0x0A
#define  SIG_DET_ERROR_REPORT_EVENT                     0x0B


/* Request                    */

#define  SIG_DET_CP_PROGRAM_FREQ_REQUEST                0x02
#define  SIG_DET_CP_PROGRAM_CADENCE_REQUEST             0x03
#define  SIG_DET_CP_DIS_PROGRAM_FREQ_REQUEST            0x04
#define  SIG_DET_CP_DIS_PROGRAM_CADENCE_REQUEST         0x05


/* Start/Finish               */

#define  SIG_DET_DISS_MODE                              0x02
#define  SIG_DET_CALL_PROGRESS_DETECT_MODE              0x03
#define  SIG_DET_TONE_DETECTION_MODE                    0x04
#define  SIG_DET_DTMF_DETECTION_MODE                    0x04
#define  SIG_DET_CI_DETECT_MODE                         0x05
#define  SIG_DET_LOAD_PCPM_TABLE_MODE                  0x502 
#define  SIG_DET_GET_PCPM_TABLE_MODE                   0x503
#define  SIG_DET_CC_TONE_DETECTION_MODE                 0x07
#define  SIG_DET_CC_DTMF_DETECTION_MODE                 0x07
#define  SIG_DET_CNG_DETECT_MODE                        0x08

/* Mode                       */

#define  SIG_DET_CALLER_ID_DETECT_MODE                  0x06


/* Variable/Parameter         */

#define  SIG_DET_CP_DETECTION_TYPE_PAR                  0x02
#define  SIG_DET_CP_DETECT_ORIG_ANSWER_PAR              0x03
#define  SIG_DET_TONE_VALUE_PAR                         0x04
#define  SIG_DET_DTMF_VALUE_PAR                         0x04
#define  SIG_DET_CP_VALUE_PAR                           0x05
#define  SIG_DET_SIGNAL_TYPE_PAR                        0x06
#define  SIG_DET_DTMF_DETECTION_TYPE_PAR                0x07
#define  SIG_DET_TONE_DETECTION_TYPE_PAR                0x07
#define  SIG_DET_TONE_END_LENGTH_PAR                    0x08
#define  SIG_DET_DTMF_END_LENGTH_PAR                    0x08
#define  SIG_DET_CP_RAW_VALUE_PAR                       0x0F
#define  SIG_DET_DTMF_DETECT_REPORT_END_PAR             0x10
#define  SIG_DET_TONE_DETECT_REPORT_END_PAR             0x10
#define  SIG_DET_RAW_HIGH_OR_LOW_PAR                    0x11
#define  SIG_DET_CP_DETECT_REPORT_RAW_VAR               0x12
#define  SIG_DET_RAW_DURATION_PAR                       0x13
#define  SIG_DET_DTMF_HARMONIC_VAR                      0x14
#define  SIG_DET_DTMF_PLAY_SNR_VAR                      0x15
#define  SIG_DET_DTMF_RECORD_SILTHR_VAR                 0x16
#define  SIG_DET_DTMF_THRESH_VAR                        0x17
#define  SIG_DET_DTMF_INTERDIGIT_VAR                    0x18
#define  SIG_DET_DTMF_DURATION_VAR                      0x19
#define  SIG_DET_DTMF_RDELTA_VAR                        0x1A
#define  SIG_DET_REPORT_RAW_PAR                         0x1B
#define  SIG_DET_REPORT_CADENCE_PAR                     0x1C
#define  SIG_DET_REPORT_FREQ_PAR                        0x1D
#define  SIG_DET_CP_CADENCE_PAR                         0x1E
#define  SIG_DET_CP_TONE_LIST_PAR                       0x1F
#define  SIG_DET_CP_TONE_DESC_PAR                       0x20
#define  SIG_DET_CP_FREQ_PAR                            0x21
#define  SIG_DET_CP_LEVEL_PAR                           0x22
#define  SIG_DET_CP_TEMPLATE_PAR                        0x23
#define  SIG_DET_CP_DURATION_PAR                        0x24
#define  SIG_DET_CED_TIMEOUT_VAR                        0x25
#define  SIG_DET_FREQ_REPORT_TIME_VAR                   0x26
#define  SIG_DET_MAX_RAW_DURATION_VAR                   0x27
#define  SIG_DET_DTMF_SKIP_INIT_VAR                     0x28
#define  SIG_DET_DTMF_BACK_NOISE_VAR                    0x29
#define  SIG_DET_DTMF_NOISE_TOLERANCE_VAR               0x2A
#define  SIG_DET_DTONE_MIN_VAR                          0x2B
#define  SIG_DET_DTONE_MIN_OVERRIDE_PAR                 0x2C
#define  SIG_DET_V32_AUTO_SWITCH_VAR                    0x2D
#define  SIG_DET_FAX_AUTO_SWITCH_VAR                    0x2E
#define  SIG_DET_DETECT_CTRL_VAR                        0x2F
#define  SIG_DET_PCPM_TONE_COUNT_VAR                   0x502
#define  SIG_DET_PCPM_STRUCTURE_SIZE_VAR               0x503
#define  SIG_DET_PCPM_LOAD_TABLE_NUM_VAR               0x504
#define  SIG_DET_PCPM_TABLE_VERSION_VAR                0x505
#define  SIG_DET_PCPM_MAX_START_SILENCE_VAR            0x506
#define  SIG_DET_PCPM_MAX_START_SOUND_VAR              0x507
#define  SIG_DET_PCPM_MIN_SPEECH_VAR                   0x508
#define  SIG_DET_PCPM_PCPM_DELAY_VAR                   0x509
#define  SIG_DET_PCPM_START_SEGMENT_VAR                0x50A
#define  SIG_DET_PCPM_MAX_RINGS_VAR                    0x50B
#define  SIG_DET_PCPM_MIN_CYCLES_VAR                   0x50C
#define  SIG_DET_PCPM_SOUND_SPIKE_VAR                  0x50D
#define  SIG_DET_PCPM_SOUND_DROPOUT_VAR                0x50E
#define  SIG_DET_PCPM_PVC_SPIKE_VAR                    0x50F
#define  SIG_DET_PCPM_PVC_DROPOUT_VAR                  0x510
#define  SIG_DET_PCPM_TERM_SPIKE_VAR                   0x511
#define  SIG_DET_PCPM_TERM_DROPOUT_VAR                 0x512
#define  SIG_DET_PCPM_FREQUENCY_LOCK_VAR               0x513
#define  SIG_DET_PCPM_MAX_FREQUENCY_SHIFTS_VAR         0x514
#define  SIG_DET_PCPM_MAX_FREQUENCY_DROPS_VAR          0x515
#define  SIG_DET_PCPM_MAX_UNMATCHED_SEGMENTS_VAR       0x516
#define  SIG_DET_PCPM_FREQUENCY_COVER_VAR              0x517
#define  SIG_DET_PCPM_PCPM_OPTIMIZE_VAR                0x518
#define  SIG_DET_PCPM_IGNORE_FREQUENCY_DEVIATION_VAR   0x519
#define  SIG_DET_PCPM_ENABLE_RINGBACK_TRACKING_VAR     0x51A
#define  SIG_DET_PCPM_ENABLE_PVC_VAR                   0x51B
#define  SIG_DET_PCPM_ENABLE_ANSWER_ANALYSIS_VAR       0x51C
#define  SIG_DET_PCPM_MAX_ANSWER_ANALYSIS_VAR          0x51D
#define  SIG_DET_PCPM_MAX_ANSWER_VOICE_VAR             0x51E
#define  SIG_DET_PCPM_MAX_ANSWER_SILENCE_VAR           0x51F
#define  SIG_DET_PCPM_WB_ENERGY_VAR                    0x520
#define  SIG_DET_PCPM_MAX_INITIAL_ANSWER_SILENCE_VAR   0x521
#define  SIG_DET_PCPM_ANSWER_SPIKE_VAR                 0x522
#define  SIG_DET_PCPM_ANSWER_DROPOUT_VAR               0x523
#define  SIG_DET_PCPM_MIN_ENERGY_VAR                   0x524
#define  SIG_DET_PCPM_TWIST_VAR                        0x525
#define  SIG_DET_PCPM_IN_TO_OUT_VAR                    0x526
#define  SIG_DET_PCPM_IN_TO_IN_VAR                     0x527
#define  SIG_DET_PCPM_FREQS_VAR                        0x528
#define  SIG_DET_PCPM_COEFFS_VAR                       0x529
#define  SIG_DET_PCPM_TONE_VAR                         0x52A
#define  SIG_DET_PCPM_MODE_PAR                         0x52B
#define  SIG_DET_PCPM_TABLE_NUM_PAR                    0x52C
#define  SIG_DET_PCPM_VALUE_PAR                        0x52D
#define  SIG_DET_PCPM_VALUE_INDEX_PAR                  0x52E
#define  SIG_DET_PCPM_DSP_FREQ_BUFFER_VAR              0x52F
#define  SIG_DET_DTMF_ALGORITHM_PAR                    0x530
#define  SIG_DET_RHET_DTMF_PLAY_MIN_DIGIT_ON_VAR       0x531
#define  SIG_DET_RHET_DTMF_PLAY_MIN_DIGIT_OFF_VAR      0x532
#define  SIG_DET_RHET_DTMF_PLAY_IN_TO_IN_VAR           0x533
#define  SIG_DET_RHET_DTMF_PLAY_IN_TO_OUT_VAR          0x534
#define  SIG_DET_RHET_DTMF_PLAY_HI_TO_LO_TWIST_VAR     0x535
#define  SIG_DET_RHET_DTMF_PLAY_LO_TO_HI_TWIST_VAR     0x536
#define  SIG_DET_RHET_DTMF_IDLE_MIN_DIGIT_ON_VAR       0x537
#define  SIG_DET_RHET_DTMF_IDLE_MIN_DIGIT_OFF_VAR      0x538
#define  SIG_DET_RHET_DTMF_IDLE_IN_TO_IN_VAR           0x539
#define  SIG_DET_RHET_DTMF_IDLE_IN_TO_OUT_VAR          0x53A
#define  SIG_DET_RHET_DTMF_IDLE_HI_TO_LO_TWIST_VAR     0x53B
#define  SIG_DET_RHET_DTMF_IDLE_LO_TO_HI_TWIST_VAR     0x53C
#define  SIG_DET_CALLER_ID_TYPE_PAR                     0x30
#define  SIG_DET_CALLER_ID_DATA_PAR                     0x31
#define  SIG_DET_CALLER_ID_DATA_LENGTH_PAR              0x32
#define  SIG_DET_CALLER_ID_KEEP_ENABLED_PAR             0x33
#define  SIG_DET_ERROR_VALUE_PAR                        0x34
#define  SIG_DET_CP_CNG_GEN_SUPPRESS_VAR                0x35
#define  SIG_DET_TONE_DETECT_REPORT_START_PAR           0x36
#define  SIG_DET_DISS_SILENCE_VAR                       0x37
#define  SIG_DET_CP_SILENCE_DURATION_PAR                0x38


/* Defines For Tagged Values  */

#define  SIG_DET_ORIG_DEF                               0x00
#define  SIG_DET_ANSWER_DEF                             0x01
#define  SIG_DET_VOICE_DEF                              0x00
#define  SIG_DET_FAX_DEF                                0x01
#define  SIG_DET_NO_HIGH_LEVEL_DEF                      0x02
#define  SIG_DET_DTMF_DEF                               0x01
#define  SIG_DET_RAW_DEF                                0x02
#define  SIG_DET_12_TONE_DEF                            0x0C
#define  SIG_DET_DTMF_12_TONE_DEF                       0x0C
#define  SIG_DET_DTMF_16_TONE_DEF                       0x10
#define  SIG_DET_16_TONE_DEF                            0x10
#define  SIG_DET_R1_TONE_DEF                            0x11
#define  SIG_DET_R2_FORWARD_TONE_DEF                    0x12
#define  SIG_DET_R2_BACKWARD_TONE_DEF                   0x13
#define  SIG_DET_CP_ANSWER_DEF                          0x1E
#define  SIG_DET_CP_SILENCE_DEF                         0x1F
#define  SIG_DET_CP_DIALTONE_DEF                        0x20
#define  SIG_DET_CP_RING1_DEF                           0x21
#define  SIG_DET_CP_BUSY1_DEF                           0x22
#define  SIG_DET_CP_RING_NO_ANS_DEF                     0x23
#define  SIG_DET_CP_BUSY_NO_ANS_DEF                     0x24
#define  SIG_DET_CP_HUMAN_DEF                           0x25
#define  SIG_DET_CP_CNG_DEF                             0x26
#define  SIG_DET_CP_CED_DEF                             0x27
#define  SIG_DET_CP_SITINTC_DEF                         0x28
#define  SIG_DET_CP_SITVACODE_DEF                       0x29
#define  SIG_DET_CP_SITREORD_DEF                        0x2A
#define  SIG_DET_CP_SITNOCIR_DEF                        0x2B
#define  SIG_DET_CP_RING2_DEF                           0x2C
#define  SIG_DET_CP_BUSY2_DEF                           0x2D
#define  SIG_DET_CP_RMTOFFHK_DEF                        0x2E
#define  SIG_DET_CP_SPECIALCP_DEF                       0x2F
#define  SIG_DET_CP_RECALL_DEF                          0x30
#define  SIG_DET_CP_CONFIRM_DEF                         0x31
#define  SIG_DET_CP_PULSE_DEF                           0x32
#define  SIG_DET_CP_ROBUSY_DEF                          0x33
#define  SIG_DET_CP_QUIET_DEF                           0x34
#define  SIG_DET_CP_DIS_SILENCE_DEF                     0x35
#define  SIG_DET_CP_DIS_DIALTONE_DEF                    0x36
#define  SIG_DET_CP_DIS_CNG_DEF                         0x37
#define  SIG_DET_CP_DIS_CED_DEF                         0x38
#define  SIG_DET_CP_V21_DEF                             0x39
#define  SIG_DET_CP_CUSTOM_FREQ0_DEF                    0x3A
#define  SIG_DET_CP_CUSTOM_FREQ1_DEF                    0x3B
#define  SIG_DET_CP_CUSTOM_FREQ2_DEF                    0x3C
#define  SIG_DET_CP_CUSTOM_CAD0_DEF                     0x3D
#define  SIG_DET_CP_CUSTOM_CAD1_DEF                     0x3E
#define  SIG_DET_CP_CUSTOM_CAD2_DEF                     0x3F
#define  SIG_DET_CP_DIS_CUSTOM_FREQ0_DEF                0x40
#define  SIG_DET_CP_DIS_CUSTOM_CAD0_DEF                 0x41
#define  SIG_DET_CP_CUSTOM_FREQ3_DEF                    0x42
#define  SIG_DET_CP_CUSTOM_FREQ4_DEF                    0x43
#define  SIG_DET_CP_CUSTOM_FREQ5_DEF                    0x44
#define  SIG_DET_CP_CUSTOM_FREQ6_DEF                    0x45
#define  SIG_DET_CP_CUSTOM_FREQ7_DEF                    0x46
#define  SIG_DET_CP_CUSTOM_CAD3_DEF                     0x47
#define  SIG_DET_CP_CUSTOM_CAD4_DEF                     0x48
#define  SIG_DET_CP_CUSTOM_CAD5_DEF                     0x49
#define  SIG_DET_CP_CUSTOM_CAD6_DEF                     0x4A
#define  SIG_DET_CP_CUSTOM_CAD7_DEF                     0x4B
#define  SIG_DET_CP_V32_DEF                             0x4C
#define  SIG_DET_CNG_ENABLE_DEF                          0x1
#define  SIG_DET_CED_ENABLE_DEF                          0x2
#define  SIG_DET_HDLC_ENABLE_DEF                         0x4
#define  SIG_DET_V32_ENABLE_DEF                          0x8
#define  SIG_DET_CAP_MF_DEF                             0x01
#define  SIG_DET_NEEDHAM_BUILD_DEF                      0x00
#define  SIG_DET_PCPM_IDLE_DEF                          0x01
#define  SIG_DET_PCPM_ONLY_DEF                          0x02
#define  SIG_DET_PCPM_RAW_RUN_DEF                       0x03
#define  SIG_DET_PCPM_RAW_AUTO_RUN_DEF                  0x04
#define  SIG_DET_ADAPTIVE_CP_DEF                        0x05
#define  SIG_DET_MIXED_MODE_DEF                         0x06
#define  SIG_DET_PCPM_VOICE_DEF                         0x80
#define  SIG_DET_NEEDHAM_DTMF_DEF                       0x00
#define  SIG_DET_RHET_DTMF_DEF                          0x01
#define  SIG_DET_CALLER_ID_TYPE_BELLCORE_DEF            0x0A
#define  SIG_DET_CALLER_ID_TYPE_NTT_DEF                 0x0B
#define  SIG_DET_CALLER_ID_TYPE_ETSI_DEF                0x0C
#define  SIG_DET_ERR_NONE_DEF                           0x00
#define  SIG_DET_ERR_CID_CHECKSUM_DEF                   0x01                  
#define  SIG_DET_ERR_CID_DATA_TRANSFER_DEF              0x02             
#define  SIG_DET_ERR_CID_DATA_PENDING_DEF               0x03 
#define  SIG_DET_ERR_CID_INVALID_TYPE_DEF               0x04 
#define  SIG_DET_CAP_PCPM_DEF                           0x2
 
#endif
