
/****************************************************************************
 *
 * Module:               Speech.h
 * Date:                 May 19, 2005
 * Purpose:              All definitions global to the Speech facility
 ****************************************************************************
 * (c) 1992-2001 Brooktrout Technology, Inc.                                  *
 *                                                                          *
 *                                                                          *
 *   All Rights Reserved.                                                   *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 * This software is furnished under a license and may only be               *
 * read, modified, copied, or otherwise used in accordance with the         *
 * terms of such license. This notice must never be removed from            *
 * this software and must be included with every copy. No other             *
 * rights to reproduce, use, or disseminate this software in any form,      *
 * in whole or in part, are granted.                                        *
 *                                                                          *
 * Brooktrout Technology, Inc. makes no representation or warranties        *
 * with respect to the performance of this software and specifically        *
 * disclaims any responsibility for any damages, special or                 *
 * consequential, connected with the use of this software.                  *
 *                                                                          *
 ****************************************************************************/
 
#ifndef SPEECH_H
#define SPEECH_H

/* Facility                   */ 
#define  MILL_FACILITY_SPEECH                           0x02

/* Event                      */ 
#define  SPEECH_RECORD_STARTED_EVENT                    0x02
#define  SPEECH_RECORD_FINISHED_EVENT                   0x03
#define  SPEECH_PLAYBACK_STARTED_EVENT                  0x04
#define  SPEECH_PLAYBACK_FINISHED_EVENT                 0x05
#define  SPEECH_CONF_PARTY_STARTED_EVENT                0x06
#define  SPEECH_CONF_PARTY_FINISHED_EVENT               0x07
#define  SPEECH_OUTBOUND_STARTED_EVENT                  0x08
#define  SPEECH_INBOUND_STARTED_EVENT                   0x09
#define  SPEECH_OUTBOUND_FINISHED_EVENT                 0x0A
#define  SPEECH_INBOUND_FINISHED_EVENT                  0x0B
#define  SPEECH_INBOUND_PARMS_QUERIED_EVENT             0x0C
#define  SPEECH_OUTBOUND_PARMS_QUERIED_EVENT            0x0D
#define  SPEECH_INBOUND_PARMS_MODIFIED_EVENT            0x0E
#define  SPEECH_OUTBOUND_PARMS_MODIFIED_EVENT           0x0F
#define  SPEECH_VAD_SPEECH_STARTED_EVENT                0x10
#define  SPEECH_VAD_SPEECH_ENDED_EVENT                  0x11
#define  SPEECH_RECORD_SILENCETIMEOUT_EVENT            0x502
#define  SPEECH_RECORD_SOUNDTERMINATE_EVENT            0x503
#define  SPEECH_SILENCE_STARTED_EVENT                   0x12
#define  SPEECH_SOUND_STARTED_EVENT                     0x13
#define  SPEECH_SUM_GROUP_INFO_EVENT                    0x14


/* Request                    */ 
#define  SPEECH_MODIFY_PLAY_CHAR_REQUEST                0x02
#define  SPEECH_RECORD_DATAFLOW_CTRL_REQUEST            0x03
#define  SPEECH_INBOUND_PARMS_MODIFY_REQUEST            0x04
#define  SPEECH_OUTBOUND_PARMS_MODIFY_REQUEST           0x05
#define  SPEECH_INBOUND_PARMS_QUERY_REQUEST             0x06
#define  SPEECH_OUTBOUND_PARMS_QUERY_REQUEST            0x07
#define  SPEECH_ECHOC_RESET_REQUEST                     0x08
#define  SPEECH_SUM_GROUP_QUERY_REQUEST                 0x09

/* Start/Finish               */ 
#define  SPEECH_RECORD_MODE                             0x02
#define  SPEECH_PLAY_MODE                               0x03
#define  SPEECH_CONF_PARTY_MODE                         0x04
#define  SPEECH_OUTBOUND_MODE                           0x05
#define  SPEECH_INBOUND_MODE                            0x06

/* Variable/Parameter         */ 
#define  SPEECH_STOP_PLAY_IMMEDIATE_PAR                 0x02
#define  SPEECH_FORMAT_VAR                              0x03
#define  SPEECH_RECORD_SILENCE_TIMEOUT_VAR              0x04
#define  SPEECH_RECORD_AGC_ENABLE_VAR                   0x05
#define  SPEECH_RECORD_SILENCE_COMPRESSION_VAR          0x06
#define  SPEECH_PLAY_SPEED_MODIFIER_VAR                 0x07
#define  SPEECH_PLAY_GAIN_MODIFIER_VAR                  0x08
#define  SPEECH_PLAY_GAIN_MODIFICATION_PAR              0x09
#define  SPEECH_PLAY_SPEED_MODIFICATION_PAR             0x0A
#define  SPEECH_BYTES_PROCESSED_PAR                     0x0B
#define  SPEECH_DURATION_PAR                            0x0C
#define  SPEECH_PLAY_DRC_MODE_VAR                       0x0D
#define  SPEECH_PLAY_PREEMPH_MODE_VAR                   0x0E
#define  SPEECH_PLAY_CDM_MODE_VAR                       0x0F
#define  SPEECH_CAP_FORMATS_PAR                         0x10
#define  SPEECH_CAP_FEATURES_PAR                        0x11
#define  SPEECH_SILCOMPR_START_VAR                      0x12
#define  SPEECH_SILCOMPR_MIDDLE_VAR                     0x13
#define  SPEECH_CONF_PARTY_ROLE_PAR                     0x14
#define  SPEECH_CONF_PARTY_TYPE_PAR                     0x15
#define  SPEECH_CONF_PARTY_CONFID_PAR                   0x16
#define  SPEECH_CONF_PARTY_CLAMP_PAR                    0x17
#define  SPEECH_CONF_PARTY_GAIN_PAR                     0x18
#define  SPEECH_BERT_INFO_PAR                           0x19
#define  SPEECH_BERT_DSP_COUNT_CTRL_PROC_PAR            0x1A
#define  SPEECH_BERT_DSP_COUNT_PHLINE_PAR               0x1B
#define  SPEECH_BERT_DSP_ERRORS_CTRL_PROC_PAR           0x1C
#define  SPEECH_BERT_DSP_ERRORS_PHLINE_PAR              0x1D
#define  SPEECH_PLAY_FORMAT_VAR                         0x1E
#define  SPEECH_RECORD_FORMAT_VAR                       0x1F
#define  SPEECH_CONF_PARTY_TYPE_VAR                     0x20
#define  SPEECH_CONF_PARTY_CLAMP_VAR                    0x21
#define  SPEECH_CONF_PARTY_GAIN_VAR                     0x22
#define  SPEECH_DATAFLOW_CTRL_PAR                       0x23
#define  SPEECH_VAD_CTRL_VAR                            0x24
#define  SPEECH_ECHOC_CTRL_VAR                          0x25
#define  SPEECH_ECHOC_TSLOT_ENABLE_VAR                  0x26
#define  SPEECH_DATAFLOW_CTRL_VAR                       0x27
#define  SPEECH_RECORD_SPEED_MODIFICATION_PAR           0x28
#define  SPEECH_RECORD_GAIN_MODIFICATION_PAR            0x29
#define  SPEECH_RECORD_SPEED_MODIFIER_VAR               0x2A
#define  SPEECH_RECORD_GAIN_MODIFIER_VAR                0x2B
#define  SPEECH_PLAY_FRAMEDURATION_PAR                  0x2C
#define  SPEECH_RECORD_FRAMEDURATION_PAR                0x2D
#define  SPEECH_PLAY_FRAMEDURATION_VAR                  0x2E
#define  SPEECH_RECORD_FRAMEDURATION_VAR                0x2F
#define  SPEECH_COMFORT_NOISE_VAR                       0x30
#define  SPEECH_SILENCE_CTRL_VAR                        0x31
#define  SPEECH_FRAME_REPLACEMENT_VAR                   0x32
#define  SPEECH_CONF_COMFORT_NOISE_PAR                  0x33
#define  SPEECH_CONF_COMFORT_NOISE_TYPE_PAR             0x34
#define  SPEECH_CONF_PARTY_MONITOR_PAR                  0x35
#define  SPEECH_SIG_PASS_CONTROL_VAR                    0x36
#define  SPEECH_ADPCM_EXP_VAR                           0x37
#define  SPEECH_VAD_THRESH_VAR                          0x38
#define  SPEECH_VAD_PASSTHRU_VAR                        0x39
#define  SPEECH_RECORD_VAR_ENABLE_VAR                  0x502
#define  SPEECH_RECORD_MIN_ENERGY_VAR                  0x503
#define  SPEECH_RECORD_SOUND_THRESH_VAR                0x504
#define  SPEECH_RECORD_SILENCE_TRUNC_VAR               0x505
#define  SPEECH_RECORD_SILENCE_TO_VAR                  0x506
#define  SPEECH_RECORD_SOUND_VAR                       0x507
#define  SPEECH_BUFF_SIZE_VAR                          0x508
#define  SPEECH_RECORD_DTMF_DIG_RMV_PAR                0x509
#define  SPEECH_RECORD_AGC_ATTACK_TIME_VAR             0x50A
#define  SPEECH_RECORD_AGC_DECAY_TIME_VAR              0x50B
#define  SPEECH_RECORD_AGC_TARGET_AMP_VAR              0x50C
#define  SPEECH_RECORD_AGC_SILENCE_AMP_VAR             0x50D
#define  SPEECH_RECORD_VOX_DEGLITCH_SPIKE_VAR          0x50E
#define  SPEECH_RECORD_VOX_DEGLITCH_DROPOUT_VAR        0x50F
#define  SPEECH_RECORD_DELAY_BLOCK_VAR                 0x510
#define  SPEECH_RECORD_SND_THRES_BACKCOMP_VAR          0x511
#define  SPEECH_RECORD_INPUT_GAIN_VAR                  0x512
#define  SPEECH_RECORD_CUE_TONE_PAR                    0x513
#define  SPEECH_RECORD_REC_CUE_START_THRESHOLD_VAR     0x514
#define  SPEECH_RECORD_REC_CUE_STOP_THRESHOLD_VAR      0x515
#define  SPEECH_RECORD_REC_CUE_TONE_ON_VAR             0x516
#define  SPEECH_RECORD_REC_CUE_TONE_OFF_VAR            0x517
#define  SPEECH_RECORD_REC_CUE_TONE_GAIN_VAR           0x518
#define  SPEECH_ADPCM_LOUDNESS_VAR                      0x3C
#define  SPEECH_CONF_VAD_SENS_VAR                       0x3D
#define  SPEECH_UNDERRUN_CTRL_VAR                       0x3E
#define  SPEECH_RECORD_ASR_VAR                          0x3F
#define  SPEECH_RECORD_BEEP_PAR                         0x40
#define  SPEECH_RECORD_BEEP_LENGTH_VAR                  0x41
#define  SPEECH_RECORD_BEEP_FREQ_VAR                    0x42
#define  SPEECH_SILENCE_SOUND_ENABLE_PAR                0x43
#define  SPEECH_SILENCE_DEGLITCH_PAR                    0x44
#define  SPEECH_SOUND_DEGLITCH_PAR                      0x45
#define  SPEECH_SILENCE_SOUND_THRESHOLD_PAR             0x46
#define  SPEECH_SUM_GROUP_PAR                           0x47
#define  SPEECH_SUM_CHANNEL_PAR                         0x48



/* Defines For Tagged Values  */ 
#define  SPEECH_PREEMPH_OFF_DEF                            0
#define  SPEECH_PREEMPH_V1_DEF                             1
#define  SPEECH_PREEMPH_V2_DEF                             2
#define  SPEECH_ULAW64_DEF                              0x00
#define  SPEECH_ADPCM32_DEF                             0x01
#define  SPEECH_ADPCM24_DEF                             0x02
#define  SPEECH_ULAW48_DEF                              0x03
#define  SPEECH_G723_1_DEF                              0x04
#define  SPEECH_G729_A_DEF                              0x05
#define  SPEECH_SX7300_DEF                              0x06
#define  SPEECH_SX9600_DEF                              0x07
#define  SPEECH_ALAW64_DEF                              0x08
#define  SPEECH_ALAW48_DEF                              0x09
#define  SPEECH_RHET_ADPCM32_DEF                        0x0A
#define  SPEECH_RHET_ADPCM24_DEF                        0x0B
#define  SPEECH_MS_GSM_DEF                              0x0C
#define  SPEECH_DVI_ADPCM32_DEF                         0x0D
#define  SPEECH_DVI_ADPCM24_DEF                         0x0E
#define  SPEECH_ULAW88_DEF                              0x0F
#define  SPEECH_ALAW88_DEF                              0x10
#define  SPEECH_G723_53_DEF                             0x11
#define  SPEECH_G723_63_DEF                             0x12
#define  SPEECH_FAX_MODEM_DEF                           0x13
#define  SPEECH_V32_MODEM_DEF                           0x14
#define  SPEECH_RAWDATA_DEF                             0x15
#define  SPEECH_GSM_610_DEF                             0x16
#define  SPEECH_GSM_660_DEF                             0x17
#define  SPEECH_LINEAR_128_DEF                          0x18
#define  SPEECH_CNID_FSK_DEF                            0x19
#define  SPEECH_CAP_PLAY_DEF                            0x01
#define  SPEECH_CAP_RECORD_DEF                          0x02
#define  SPEECH_CAP_FORMAT_INDEP_DEF                    0x04
#define  SPEECH_CAP_SIMUL_PLAY_RECORD_DEF               0x08
#define  SPEECH_CAP_OBP_DEF                             0x10
#define  SPEECH_CONF_PARTY_ROLE_NODE_DEF                0x00
#define  SPEECH_CONF_PARTY_ROLE_TERMINAL_DEF            0x01
#define  SPEECH_CONF_PARTY_TYPE_TALK_DEF                0x00
#define  SPEECH_CONF_PARTY_TYPE_LISTEN_DEF              0x01
#define  SPEECH_CONF_PARTY_TYPE_DUPLEX_DEF              0x02
#define  SPEECH_CONF_PARTY_TYPE_NONE_DEF                0x03
#define  SPEECH_CONF_PARTY_CLAMP_DISABLE_DEF            0x00
#define  SPEECH_CONF_PARTY_CLAMP_ENABLE_DEF             0x01
#define  SPEECH_CONF_PARTY_SUCCESS_DEF                  0x00
#define  SPEECH_CONF_PARTY_FAIL_DEF                     0x01
#define  SPEECH_ECHOC_OFF_DEF                           0x00
#define  SPEECH_ECHOC_ON_DEF                            0x01
#define  SPEECH_RECORD_TO_HOST_ONLY_DEF                 0x00
#define  SPEECH_RECORD_TO_HOST_TSLOT_DEF                0x01
#define  SPEECH_RECORD_TO_TSLOT_ONLY_DEF                0x02
#define  SPEECH_SILENCE_CTRL_LOW_RATE_DEF               0x00
#define  SPEECH_SILENCE_CTRL_SUPPRESS_ALL_DEF           0x01
#define  SPEECH_FRAME_REPLACE_STANDARD_DEF              0x00
#define  SPEECH_FRAME_REPLACE_SILENCE_DEF               0x01
#define  SPEECH_CAP_LINEAR_DEF                          0x04
#define  SPEECH_CAP_CVSD_DEF                            0x08
#define  SPEECH_CAP_ADPCM_DEF                           0x10
#define  SPEECH_CAP_UPCM_DEF                            0x20
#define  SPEECH_CAP_APCM_DEF                            0x40
#define  SPEECH_CAP_SUBBAND_DEF                         0x80
#define  SPEECH_CAP_G723_1_DEF                         0x100
#define  SPEECH_CAP_G729_A_DEF                         0x200
#define  SPEECH_CAP_SX7300_DEF                         0x400
#define  SPEECH_CAP_SX9600_DEF                         0x800
#define  SPEECH_CAP_ADPCM_RHET_DEF                    0x1000
#define  SPEECH_CAP_MS_GSM_DEF                        0x2000
#define  SPEECH_CAP_ADPCM_DVI_DEF                     0x4000
#define  SPEECH_CAP_FAX_MODEM_DEF                     0x8000
#define  SPEECH_CAP_V32_MODEM_DEF                    0x10000
#define  SPEECH_CAP_RAWDATA_DEF                      0x20000
#define  SPEECH_CAP_GSM_610_DEF                      0x40000
#define  SPEECH_CAP_GSM_660_DEF                      0x80000
#define  SPEECH_CAP_LINEAR_128_DEF                  0x100000
#define  SPEECH_CAP_CNID_FSK_DEF                    0x200000
#define  SPEECH_COMFORT_NOISE_DISABLE_DEF               0x00
#define  SPEECH_COMFORT_NOISE_BASIC_DEF                 0x01
#define  SPEECH_COMFORT_NOISE_ENABLE_DEF                0x01
#define  SPEECH_COMFORT_NOISE_NORMAL_DEF                0x01
#define  SPEECH_COMFORT_NOISE_UNIFORM_DEF               0x02
#define  SPEECH_CONF_PARTY_MONITOR_NORMAL_DEF           0x00
#define  SPEECH_CONF_PARTY_MONITOR_COACH_DEF            0x01
#define  SPEECH_CONF_PARTY_MONITOR_PUPIL_DEF            0x02
#define  SPEECH_SIG_BT_PROPRIETARY_DEF                  0x00
#define  SPEECH_SIG_SUPPRESS_DEF                        0x02
#define  SPEECH_SIG_PASSTHROUGH_DEF                     0x03
#define  SPEECH_ADPCM_EXP_BIAS_DEF                      0x00
#define  SPEECH_ADPCM_EXP_HPF_DEF                       0x01
#define  SPEECH_ADPCM_EXP_LEAK_DEF                      0x02
#define  SPEECH_VAD_OFF_DEF                             0x00
#define  SPEECH_VAD_ALL_DEF                             0x01
#define  SPEECH_VAD_DETECT_DEF                          0x02  
#define  SPEECH_VAD_DETECT_AUTO_DEF                     0x03
#define  SPEECH_VAD_THRESH_DEF                          0x04
#define  SPEECH_RECORD_VAR_DISABLED_DEF                 0x00
#define  SPEECH_RECORD_VAR_ENABLED_DEF                  0x01
#define  SPEECH_RECORD_VAR_NOTIFY_ONLY_DEF              0x02
#define  SPEECH_RECORD_LOUDNESS_NORMAL_DEF              0x00
#define  SPEECH_RECORD_LOUDNESS_LOW_DEF                 0x02 
#define  SPEECH_RECORD_NON_ASR_DEF                      0x00
#define  SPEECH_RECORD_ASR_DEF                          0x01                    
#define  SPEECH_G726_40_DEF                             0x1A
#define  SPEECH_G726_32_DEF                             0x1B
#define  SPEECH_G726_24_DEF                             0x1C
#define  SPEECH_G726_16_DEF                             0x1D
#define  SPEECH_CAP_G726_DEF                        0x400000
#define  SPEECH_CAP_SUMMATION_DEF                       0x20
#define  SPEECH_VAD_ALL_DTMF_DEF                        0x05 
#define  SPEECH_VAD_NO_START_SIL_DEF                    0x06
#define  SPEECH_VAD_NO_START_SIL_DTMF_DEF               0x07
#endif

