

/****************************************************************************
 *
 * Module:               SWITCH.h
 * Date:                 Jul. 2, 2004
 * Purpose:              All definitions global to the Call Switch facility
 ****************************************************************************
 * (c) 1992-2001 Brooktrout Technology, Inc.                                  *
 *                                                                          *
 *                                                                          *
 *   All Rights Reserved.                                                   *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 * This software is furnished under a license and may only be               *
 * read, modified, copied, or otherwise used in accordance with the         *
 * terms of such license. This notice must never be removed from            *
 * this software and must be included with every copy. No other             *
 * rights to reproduce, use, or disseminate this software in any form,      *
 * in whole or in part, are granted.                                        *
 *                                                                          *
 * Brooktrout Technology, Inc. makes no representation or warranties        *
 * with respect to the performance of this software and specifically        *
 * disclaims any responsibility for any damages, special or                 *
 * consequential, connected with the use of this software.                  *
 *                                                                          *
 ****************************************************************************/
 
#ifndef SWITCH_H
#define SWITCH_H

/* Facility                   */
#define  MILL_FACILITY_CALL_SW                       0x07

/* Event                      */
#define  CALL_SW_PCM_CONN_REPLY_EVENT                0x02
#define  CALL_SW_RESOURCE_LIST_EVENT                 0x03
#define  CALL_SW_CONN_LIST_EVENT                     0x04
#define  CALL_SW_SINGLE_CONN_EVENT                   0x05

/* Request                    */
#define  CALL_SW_PCM_CONN_REQUEST                    0x02
#define  CALL_SW_RESOURCE_LIST_REQUEST               0x03
#define  CALL_SW_CLEAR_CONNECTIONS_REQUEST           0x04
#define  CALL_SW_QUERY_CONN_REQUEST                  0x05
#define  CALL_SW_QUERY_SINGLE_CONN_REQUEST           0x06

/* Variable/Parameter         */
#define  CALL_SW_CONN_MODE_PAR                       0x03
#define  CALL_SW_PCM_CONN_SOURCE_PAR                 0x04
#define  CALL_SW_PORT_TYPE_PAR                       0x05
#define  CALL_SW_PORT_UNIT_PAR                       0x06
#define  CALL_SW_PORT_STREAM_PAR                     0x07
#define  CALL_SW_PORT_STREAM_SLOT_PAR                0x08
#define  CALL_SW_PCM_CONN_DEST_PAR                   0x09
#define  CALL_SW_PCM_CONN_REPLY_PAR                  0x0A
#define  CALL_SW_PORT_DESC_PAR                       0x0C
#define  CALL_SW_UNIT_PAR                            0x0D
#define  CALL_SW_STREAM_PAR                          0x0E
#define  CALL_SW_SLOT_PAR                            0x0F
#define  CALL_SW_PORT_CLASS_PAR                      0x10
#define  CALL_SW_CONN_NUMBER_PAR                     0x11
#define  CALL_SW_LOCAL_IP_PAR                        0x12
#define  CALL_SW_REMOTE_IP_PAR                       0x13
#define  CALL_SW_LOCAL_UDP_PORT_PAR                  0x14
#define  CALL_SW_REMOTE_UDP_PORT_PAR                 0x15
#define  CALL_SW_LOCAL_PORT_PAR                      0x16
#define  CALL_SW_REMOTE_PORT_PAR                     0x17
#define  CALL_SW_MAX_BIT_RATE_PAR                    0x18
#define  CALL_SW_PACKET_SIZE_PAR                     0x19
#define  CALL_SW_ERROR_CORRECTION_TYPE_PAR           0x1A
#define  CALL_SW_REDUNDANCY_CTRL_PAR                 0x1B
#define  CALL_SW_REDUNDANCY_IMAGE_PAR                0x1C
#define  CALL_SW_T38_VERSION_PAR                     0x1D
#define  CALL_SW_FILL_BIT_REMOVAL_PAR                0x1E
#define  CALL_SW_MMR_TRANSCODING_PAR                 0x1F
#define  CALL_SW_JBIG_TRANSCODING_PAR                0x20
#define  CALL_SW_DATA_RATE_MANAGEMENT_PAR            0x21
#define  CALL_SW_MAX_BUFFER_PAR                      0x22
#define  CALL_SW_MAX_DATAGRAM_PAR                    0x23
#define  CALL_SW_T38T30_FASTNOTIFY_PAR               0x24
#define  CALL_SW_DCHAN_VAR                           0x25
#define  CALL_SW_BCHAN_VAR                           0x26
#define  CALL_SW_MEDIA_TYPE_VAR                      0x27
#define  CALL_SW_LOCAL_RTP_IP_PAR                    0x28
#define  CALL_SW_REMOTE_RTP_IP_PAR                   0x29
#define  CALL_SW_LOCAL_RTP_PORT_PAR                  0x2A
#define  CALL_SW_REMOTE_RTP_PORT_PAR                 0x2B
#define  CALL_SW_LOCAL_RTCP_IP_PAR                   0x2C
#define  CALL_SW_REMOTE_RTCP_IP_PAR                  0x2D
#define  CALL_SW_LOCAL_RTCP_PORT_PAR                 0x2E
#define  CALL_SW_REMOTE_RTCP_PORT_PAR                0x2F
 

/* Defines For Tagged Values  */
#define  CALL_SW_DISCONNECT_DEF                      0x00
#define  CALL_SW_TRANSMIT_ONLY_DEF                   0x01
#define  CALL_SW_RECEIVE_ONLY_DEF                    0x02
#define  CALL_SW_FULL_DUPLEX_DEF                     0x03
#define  CALL_SW_DUPLEX_AND_SIGNALING_DEF            0x07
#define  CALL_SW_SDINV_TRANSMIT_ONLY_DEF             0x09
#define  CALL_SW_SDINV_RECEIVE_ONLY_DEF              0x0A
#define  CALL_SW_SDINV_FULL_DUPLEX_DEF               0x0B
#define  CALL_SW_SDINV_DUPLEX_AND_SIGNALING_DEF      0x0F
#define  CALL_SW_PORT_CHANNEL_DEF                    0x00
#define  CALL_SW_PORT_RESERVE_DEF                    0x01
#define  CALL_SW_PORT_MVIP_DEF                       0x02
#define  CALL_SW_PORT_PEB_DEF                        0x03
#define  CALL_SW_PORT_SCBUS_DEF                      0x04
#define  CALL_SW_PORT_H100_DEF                       0x05
#define  CALL_SW_PORT_T1_DEF                         0x06
#define  CALL_SW_PORT_E1_DEF                         0x07
#define  CALL_SW_PRI_T1_DEF                          0x08
#define  CALL_SW_PRI_E1_DEF                          0x09
#define  CALL_SW_AT_MODEM_DEF                        0x0A
#define  CALL_SW_PORT_BRI_DEF                        0x0B
#define  CALL_SW_PORT_ANALOG_LOOP_START_DEF          0x0C
#define  CALL_SW_PORT_ANALOG_DID_DEF                 0x0D
#define  CALL_SW_PORT_BUS_DEF                        0x0E
#define  CALL_SW_PORT_NETWORK_DEF                    0x0F
#define  CALL_SW_PORT_WILD_DEF                       0x10
#define  CALL_SW_PORT_PAT_GEN_DEF                    0x11
#define  CALL_SW_SUCCESS_DEF                         0x00
#define  CALL_SW_GENERIC_FAILURE_DEF                 0x01
#define  CALL_SW_UNKNOWN_CONN_MODE_DEF               0x02
#define  CALL_SW_UNKNOWN_PORT_TYPE_DEF               0x03
#define  CALL_SW_INVALID_PORT_TYPE_DEF               0x04
#define  CALL_SW_UNKNOWN_PORT_UNIT_DEF               0x05
#define  CALL_SW_INVALID_PORT_UNIT_DEF               0x06
#define  CALL_SW_INVALID_PORT_STREAM_DEF             0x07
#define  CALL_SW_INVALID_STREAM_SLOT                 0x08
#define  CALL_SW_INVALID_STREAM_SLOT_DEF             0x08
#define  CALL_SW_SLOT_ALREADY_ASSIGNED_DEF           0x09
#define  CALL_SW_SLOT_NOT_ASSIGNED_DEF               0x0A
#define  CALL_SW_UDP_TL_CLASS_DEF                    0x12
#define  CALL_SW_DISCONN_DEF                         0x80
#define  CALL_SW_BIT_RATE_2400_DEF                   0x00
#define  CALL_SW_BIT_RATE_4800_DEF                   0x01
#define  CALL_SW_BIT_RATE_7200_DEF                   0x02
#define  CALL_SW_BIT_RATE_9600_DEF                   0x03
#define  CALL_SW_BIT_RATE_12000_DEF                  0x04
#define  CALL_SW_BIT_RATE_14400_DEF                  0x05
#define  CALL_SW_REDUNDANCY_DEF                      0x06
#define  CALL_SW_FEC_DEF                             0x07
#define  CALL_SW_LOCAL_TCF_DEF                       0x08
#define  CALL_SW_TRANSFERRED_TCF_DEF                 0x09
#define  CALL_SW_PSTN_DEF                            0x0A
#define  CALL_SW_IP_DEF                              0x0B
#define  CALL_SW_PORT_UDPTL_DEF                      0x0C
#define  CALL_SW_DISCONNECT_MODE_DEF                 0x80
#define  CALL_SW_NO_T38_LICENSE_DEF                  0x0B
#define  CALL_SW_PORT_RTP_DEF                        0x13
#define  CALL_SW_IP_PORT_INUSE_DEF                   0x14
#define  CALL_SW_INVALID_CONN_MODE_DEF               0x15
#define  CALL_SW_IP_PORT_REUSE_DEF                   0x16
#endif



