
/****************************************************************************
 *
 * Module:               ecc.h
 * Date:                 Mar. 6, 2003
 * Author:               Simon Capper
 * Purpose:              All definitions global to the ECC facility
 ****************************************************************************
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. *
 *                                                                          *
 *                                                                          *
 *   All Rights Reserved.                                                   *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 * This software is furnished under a license and may only be               *
 * read, modified, copied, or otherwise used in accordance with the         *
 * terms of such license. This notice must never be removed from            *
 * this software and must be included with every copy. No other             *
 * rights to reproduce, use, or disseminate this software in any form,      *
 * in whole or in part, are granted.                                        *
 *                                                                          *
 * Brooktrout Technology, Inc. makes no representation or warranties        *
 * with respect to the performance of this software and specifically        *
 * disclaims any responsibility for any damages, special or                 *
 * consequential, connected with the use of this software.                  *
 *                                                                          *
 ****************************************************************************/
 
#ifndef ECC_H
#define ECC_H

/* Facility                   */
#define  MILL_FACILITY_ECC                      0x12

/* Event                      */
#define  ECC_BCHANNEL_STATE_CHANGE_EVENT        0x01
#define  ECC_COMMAND_ERROR_EVENT                0x02
#define  ECC_EVENT_MONITOR_STARTED_EVENT        0x03
#define  ECC_EVENT_MONITOR_FINISHED_EVENT       0x04
#define  ECC_MASTER_READY_EVENT                 0x05
#define  ECC_STATUS_MONITOR_STARTED_EVENT       0x06
#define  ECC_STATUS_MONITOR_FINISHED_EVENT      0x07
#define  ECC_STATUS_UPDATE_EVENT                0x08
#define  ECC_MODULE_INFO_EVENT                  0x09
#define  ECC_FLASH_HOOK_SENT_EVENT              0x0A
#define  ECC_CALL_ON_HOLD_EVENT                 0x0B
#define  ECC_CALL_RETRIEVED_EVENT               0x0C
#define  ECC_TRANSFER_COMPLETED_EVENT           0x0D
#define  ECC_CAPABILITIES_EVENT                 0x0E
#define  ECC_PORT_CONFIG_EVENT                  0x0F
#define  ECC_LOOP_CUR_LOST_EVENT                0x10
#define  ECC_CALL_DIVERTED_EVENT                0x11
#define  ECC_CALL_DIVERT_TIMEOUT_EVENT          0x12
#define  ECC_CALL_DIVERT_REJECTED_EVENT         0x13

/* Request                    */
#define  ECC_RING_DETECT_REQUEST                0x01
#define  ECC_MAKE_CALL_REQUEST                  0x02
#define  ECC_ANSWER_CALL_REQUEST                0x03
#define  ECC_DROP_CALL_REQUEST                  0x04
#define  ECC_BCHANNEL_STATE_REQUEST             0x05
#define  ECC_SET_ACTIVE_REQUEST                 0x06
#define  ECC_SLAVE_CONNECT_REQUEST              0x07
#define  ECC_MODULE_INFO_REQUEST                0x08
#define  ECC_FLASH_HOOK_REQUEST                 0x09
#define  ECC_HOLD_CALL_REQUEST                  0x0A
#define  ECC_RETRIEVE_CALL_REQUEST              0x0B
#define  ECC_COMPLETE_TRANSFER_REQUEST          0x0C
#define  ECC_GET_CAPABILITIES_REQUEST           0x0D
#define  ECC_GET_PROTOCOL_REQUEST               0x0E
#define  ECC_DIVERT_CALL_REQUEST                0x0F
#define  ECC_FORWARD_DIVERT_REQUEST             0x10
#define  ECC_SEND_ALERTING_REQUEST              0x11
#define  ECC_STATIC_RING_DETECT_REQUEST         0x12

/* Start/Finish               */
#define  ECC_EVENT_MONITOR_MODE                 0x01
#define  ECC_STATUS_MONITOR_MODE                0x02

/* Variable/Parameter         */
#define  ECC_BCHANNEL_PAR                       0x01
#define  ECC_TRUNK_ID_PAR                       0x02
#define  ECC_CALLED_PARTY_PAR                   0x03
#define  ECC_CALLING_PARTY_PAR                  0x04
#define  ECC_PRESENTATION_PAR                   0x05
#define  ECC_SCREENING_PAR                      0x06
#define  ECC_ENQUIRY_PAR                        0x07
#define  ECC_CALL_PROTOCOL_PAR                  0x08
#define  ECC_CALL_TYPE_PAR                      0x09
#define  ECC_ANSWER_CODE_PAR                    0x0A
#define  ECC_IDLE_CAUSE_PAR                     0x0B
#define  ECC_RING_MODE_PAR                      0x0C
#define  ECC_CHANNEL_ID_PAR                     0x0D
#define  ECC_COMMAND_ID_PAR                     0x0E
#define  ECC_ERROR_CODE_PAR                     0x0F
#define  ECC_BCHANNEL_STATE_PAR                 0x10
#define  ECC_BCHANNEL_ENABLED_PAR               0x11
#define  ECC_PORT_ID_PAR                        0x12
#define  ECC_BCHANNEL_ID_PAR                    0x13
#define  ECC_NEEDS_CALL_PROGRESS_PAR            0x14
#define  ECC_AOC_TYPE_PAR                       0x15
#define  ECC_AOC_CURRENCY_MULTI_PAR             0x16
#define  ECC_AOC_CHARGE_PAR                     0x17
#define  ECC_REDIRECT_PARTY_PAR                 0x18
#define  ECC_ORIGINAL_PARTY_PAR                 0x19
#define  ECC_CALLED_PARTY_SUB_ADDR_PAR          0x1A
#define  ECC_CALLING_PARTY_SUB_ADDR_PAR         0x1B
#define  ECC_AUTO_RESET_PAR                     0x1C
#define  ECC_REDIRECT_REASON_PAR                0x1D
#define  ECC_LAYER1_STATE_PAR                   0x1E
#define  ECC_LAYER2_STATE_PAR                   0x1F
#define  ECC_PORT_INFO_PAR                      0x20
#define  ECC_NUM_BCHANNEL_PAR                   0x21
#define  ECC_START_BCHANNEL_PAR                 0x22
#define  ECC_END_BCHANNEL_PAR                   0x23
#define  ECC_SECOND_CHANNEL_ID_PAR              0x24
#define  ECC_SECOND_MODULE_ID_PAR               0x25
#define  ECC_SECOND_PORT_ID_PAR                 0x26
#define  ECC_SECOND_BCHANNEL_ID_PAR             0x27
#define  ECC_TRANSFER_TYPE_PAR                  0x28
#define  ECC_TRANSFER_GROUP_PAR                 0x29
#define  ECC_CALLS_ON_HOLD_PAR                  0x2A
#define  ECC_IE_COUNT_PAR                       0x2B
#define  ECC_IE_DATA_PAR                        0x2C
#define  ECC_IDLE_CAUSE_LOCATION_PAR            0x2D
#define  ECC_PORT_CONFIG_PAR                    0x2E
#define  ECC_NETWORK_ALARM_PAR                  0x2F
#define  ECC_NETWORK_ERROR_PAR                  0x30
#define  ECC_SD_REMOTE_ANSWER_HACK_PAR          0x31
#define  ECC_LINE_ENCODING_PAR                  0x32
#define  ECC_NAME_IDENT_PAR                     0x33
#define  ECC_NAME_CHAR_SET_PAR                  0x34
#define  ECC_REDIRECT_COUNTER_PAR               0x35
#define  ECC_REDIRECT_ORIGINAL_REASON_PAR       0x36
#define  ECC_CONNECTED_NUM_PAR                  0x37

/* Defines For Tagged Values  */
#define  ECC_CALL_TYPE_DEFAULT_DEF              0x00
#define  ECC_CALL_TYPE_VOICE_DEF                0x01
#define  ECC_CALL_TYPE_MODEM_DEF                0x02
#define  ECC_CALL_TYPE_AUTO_DEF                 0x63
#define  ECC_ERROR_OK_DEF                       0x00
#define  ECC_ERROR_TRUNK_OOS_DEF                0x01
#define  ECC_ERROR_BAD_DIALSTRING_DEF           0x02
#define  ECC_ERROR_COMMS_DEF                    0x03
#define  ECC_ERROR_MODULE_NOT_FOUND_DEF         0x04
#define  ECC_ERROR_CHANNEL_NOT_FOUND_DEF        0x05
#define  ECC_ERROR_INVALID_STATE_DEF            0x06
#define  ECC_ERROR_SYSTEM_DEF                   0x07
#define  ECC_ERROR_GLARE_DEF                    0x08
#define  ECC_ERROR_NON_SUPPORTED_FEATURE_DEF    0x09
#define  ECC_CP_NONE_DEF                        0x00
#define  ECC_CP_FAIL_ONLY_DEF                   0x01
#define  ECC_CP_FULL_DEF                        0x02
#define  ECC_STATE_DOWN_DEF                     0x00
#define  ECC_STATE_RUNNING_DEF                  0x01
#define  ECC_XFER_NONE                          0x00
#define  ECC_XFER_SINGLE                        0x01
#define  ECC_XFER_TWO_CHAN                      0x02
#define  ECC_XFER_ALL                           0x03
#define  ECC_XFER_TWO_CHAN_NEEDS_NAILUP         0x04
#define  ECC_DIVERT_NONE                        0x00
#define  ECC_DIVERT_UNCONDITIONAL               0x01
#define  ECC_DIVERT_BUSY                        0x02
#define  ECC_DIVERT_NO_RESPONSE                 0x03
#define  ECC_LAYER_1_DEF                        0x00
#define  ECC_LAYER_2_DEF                        0x01
#define  ECC_PORT_DOWN_DEF                      0x00
#define  ECC_PORT_RUNNING_DEF                   0x01
#define  ECC_ALARM_NONE_DEF                     0x00
#define  ECC_ALARM_LOS_DEF                      0x01
#define  ECC_ALARM_RAI_DEF                      0x02
#define  ECC_ALARM_AIS_DEF                      0x04
#define  ECC_ERROR_NONE_DEF                     0x00
#define  ECC_ERROR_FRAMING_DEF                  0x01
#define  ECC_ERROR_CRC_DEF                      0x02
#define  ECC_ERROR_BPV_DEF                      0x04
#define  ECC_ERROR_SLIP_DEF                     0x08

#endif
