/*
 * milluser.h
 * Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved.
 */

/* Definitions used by applications and possibly the driver, too.
   Many are system dependent */

/* There should be no references to MSDOS, CDOS, or QNX 2, as we don't support
   these platforms */

#ifndef _milluser_h_
#define _milluser_h_

#define PKTV0_OR_V1    // Allow both packet versions 0 & 1

#include "millver.h"
#include "milladdr.h"

#ifndef M_KERNEL
/* Prototypes for driver standard interface function for use by
   Bfv API and low level applications. Functions are defined by
   sources in boston/driver/<platform>/common. */
extern int mill_open(char *path,int oflag);
extern int mill_close(int fd);
extern int mill_ioctl(int fd,int cmd,char *arg);
extern unsigned _mill_do_times(void);

/* If our own getopt functions are used */
#if (defined(WINNT) || defined(__OS2__) || defined(WIN95))
#define getopt _mill_getopt
#define optind _mill_optind
#define optarg _mill_optarg
#define opterr _mill_opterr
#endif

void *_mill_opendir(char *dirname);
char *_mill_readdir(void *dir_handle);
int _mill_closedir(void *dir_handle);
#endif

#if defined(unix) || defined(M_XENIX) || defined(_AIX)
/* XENIX doesn't include protection against multiple inclusions, of
   sys/types.h, nor does AT&T System V Rel 3.2 Ver 3.2.2/3. The last
   check is for a symbol found in sys/param.h. */
#if !(defined(APLIOCTL_C) || defined(PHYSISA_C) || defined(DMA_C) || defined(MAXPID) || defined(linux) && defined(M_KERNEL))
#include <sys/types.h>
#endif
#endif

/* The type mill_time_rep is the type of value used to store time supplied
   to applications. */
#define mill_time_rep unsigned

#define splfax spl6


#ifdef __QNX__
#ifndef M_KERNEL
/* #define FAX_CTL FAX_CTL_UNIT */
extern get_ticks();
/* #define times(buf) ((time_t)_mill_get_ticks()) */
#endif
#define hz HZ
#endif

#ifdef WINNT
#ifndef M_KERNEL
/* User only */
#include <windows.h>
#include <stdlib.h>
#define sleep(n) Sleep((n) * 1000)
struct tms {int xxx;};
#define HZ 1000
#define _mill_do_times() ((unsigned)GetTickCount())
#endif
/* Both kernel & user */
#undef mill_time_rep
typedef LARGE_INTEGER mill_time_rep;
/* #define FAX_CTL FAX_CTL_UNIT */
#define hz HZ
typedef char *caddr_t;
typedef char *faddr_t;
typedef void *paddr_t;
#define MILL_TYPE 33000
#define MILL_IOCTL_DIFF 0x800
#ifdef CTL_CODE
/* Actual ioctl doesn't matter. All stuff goes through one code. */
#define MILL_IOCTL_VAL ((unsigned)CTL_CODE(MILL_TYPE,MILL_IOCTL_DIFF, \
        METHOD_BUFFERED,FILE_ANY_ACCESS))
#endif
#ifndef EZERO
#define EZERO 0
#endif
#endif

#ifdef linux
#ifdef BFAX_THREADS
#include <pthread.h>
#endif
#ifndef M_KERNEL
/* Part of HZ kludge */
/* Macros and declarations for history to get time info from driver. */
#ifdef USER_APP
#define _mill_do_times() _mill_do_times_x(fd)
unsigned _mill_do_times_x(int fd);
#undef HZ
#define HZ HZ_linux
extern unsigned HZ_linux;
#endif
#endif
#endif

#ifdef __OS2__
#ifndef M_KERNEL
#ifdef USER_APP
#include "err_fix.h"
#endif
struct tms {int xxx;};
typedef char *faddr_t;
#ifndef __TURBOC__
/* The _Seg16 causes the 32-bit compiler to store the addr as 16:16 */
typedef char * _Seg16 faddr16_t;
#else
/* The __far16 causes the 32-bit compiler to store the addr as 16:16 */
typedef char __far16 *faddr16_t;
#endif
#define sleep(n) DosSleep((n) * 1000L)
#define INCL_BASE 1
#include <os2.h>
#endif /*M_KERNEL*/
/*#define FAX_CTL FAX_CTL_UNIT*/
#define HZ 1000
#define hz HZ
typedef char *caddr_t;
typedef unsigned long paddr_t;
/* We have to pack the structures, because the default alignment is different
   for the different compilers used for the driver and applications. */
#pragma pack(1)
#endif

#ifdef WIN95
#ifndef M_KERNEL
#include <windows.h>
#include <stdlib.h>
#define times(x) ((unsigned)GetTickCount())
#define HZ 1000
#define sleep(n) Sleep((n) * 1000)
struct tms {int xxx;};
#endif
/* Both kernel & user */
/* #define FAX_CTL FAX_CTL_UNIT */
#define hz HZ
typedef char *caddr_t;
typedef char *faddr_t;
typedef unsigned paddr_t;
#ifndef EZERO
#define EZERO 0
#endif
#endif


#ifdef _AIX
#ifndef M_KERNEL
#endif
struct mill_dds {
    int    bus_id;

    /* MCA stuff */
    uchar  slotno;           /* Slot number in MicroChannel */
    ushort adapter_id;       /* Adapter Id, replaces pos0 and pos1 */
    ulong  bus_io_addr;      /* Bus I/O Address */

    ushort bus_intr_lvl;     /* Bus Interrupt Level */
    int    dma_level;
    unsigned int nmill;
    unsigned int mill_buf_size;
    unsigned int mill_num_user_intrs;
    unsigned cmd_history_size;
    unsigned fifo_history_size;
};
#endif


/* Definitions of ioctl cmd codes and structures */

  
#if defined(unix) || defined(M_XENIX) || defined(_AIX)
#define MILL_BASE ('F'<<8)
#else
#define MILL_BASE 0
#endif
#undef _IO
#undef _IOR
#undef _IOW
#undef _IOWR
#define _IO(x,y) (MILL_BASE+(y))
#define _IOR(x,y,t) (MILL_BASE+(y))
#define _IOW(x,y,t) (MILL_BASE+(y))
#define _IOWR(x,y,t) (MILL_BASE+(y))
#define MILL_BASELESS(x) ((unsigned char)((x) - MILL_BASE))
#define MILL_REBASE(x) (MILL_BASE+(x))

#define MILL_GET_VERSION        _IOR (F, 0, struct mill_ver)
#define MILL_INW                _IOWR(F, 1, struct mill_io)
#define MILL_OUTW               _IOW (F, 2, struct mill_io)
#define MILL_INB                _IOWR(F, 3, struct mill_io)
#define MILL_OUTB               _IOW (F, 4, struct mill_io)
#define MILL_IND                _IOWR(F, 5, struct mill_io)
#define MILL_OUTD               _IOW (F, 6, struct mill_io)
#define MILL_GETHISTORY         _IOWR(F, 7, struct gethistory_buf)
#define MILL_CLEAR_HISTORY      _IOW (F, 8, unsigned short)
#define MILL_DEBUG              _IOWR(F, 9, struct mill_dbg)
#define MILL_SESSION_CREATE     _IOWR(F, 10, struct mill_session_create_destroy)
#define MILL_SESSION_DESTROY    _IOWR(F, 11, struct mill_session_create_destroy)
#define MILL_MSG_SEND           _IOWR(F, 12, struct mill_send_rcv)
#define MILL_MSG_RECEIVE        _IOWR(F, 13, struct mill_send_rcv)
#define MILL_ADDRESS_ASSOC      _IOWR(F, 14, struct mill_addr_assoc)
#define MILL_CLEAR_OUTPUT       _IOWR(F, 15, struct mill_clear_io)
#define MILL_DIRECT_XFER        _IOWR(F, 16, struct mill_direct_xfer)
#define MILL_ALLOW_UNLOAD       _IO  (F, 17)
#define MILL_MODULE_INFO        _IOWR(F, 18, struct mill_mod_info)
#define MILL_MODULE_RECONFIG    _IOWR(F, 19, struct mill_mod_reconfig)
#define MILL_ADDRESS_CHECK      _IOWR(F, 20, struct mill_addr_check)
#define MILL_MODULE_RESET       _IOWR(F, 21, struct mill_mod_reset)
#define MILL_MODULE_DEACTIVATE  _IOWR(F, 22, struct mill_mod_deactivate)
#define MILL_MODULE_CONFIG_SPECS _IOWR(F, 23, struct mill_mod_config_specs)
#define MILL_FLOW_CTRL_FWD      _IOW (F, 24, struct mill_flow_ctrl_forwarding)
#define MILL_MSG_SEND_ASYNC     _IOWR(F, 25, struct mill_send_rcv)
#define MILL_MSG_RECEIVE_ASYNC  _IOWR(F, 26, struct mill_send_rcv)
#define MILL_ASYNC_CANCEL       _IOWR(F, 27, struct mill_async_cancel)
#define MILL_CLEAR_INPUT        _IOWR(F, 28, struct mill_clear_io)
#define MILL_PARAMS_RECONFIG    _IO  (F, 29)
#define MILL_GET_TIME_INFO      _IOR (F, 30, struct mill_time_info)


/* Structures used by the ioctl cmds. */
/* Note that they must have a field named "err". */

/* NOTE: Any change to these structures, or whether members are used
   on input or output, may require changes to a Solaris-specific portion
   of the driver code. */

struct mill_io {
    unsigned port;
    unsigned val;
    unsigned err;
};

struct gethistory_buf {
    unsigned short module;
    unsigned short channel;
    unsigned historysize;
    unsigned char *history;
    unsigned use_start_pos;
    unsigned start_pos;
    unsigned no_running_time;
#ifdef PKTV0_OR_V1
#define HIST_MOD_IS_HW 0x1
    unsigned char mod_info[256];
#endif
    unsigned err;
};

#define HIST_OFF_COUNT 0
#define HIST_OFF_TIME (HIST_OFF_COUNT + 1)
#define HIST_OFF_SEQ (HIST_OFF_TIME + sizeof(mill_time_rep))
#define HIST_OFF_TYPE (HIST_OFF_SEQ + 4)
#define HIST_OFF_AUX (HIST_OFF_TYPE + 1)

struct mill_dbg {
    int ints[5];
};

#define MILL_SESS_OPTION_RESET_ON_DESTROY       0x1
#define MILL_SESS_OPTION_UNIQUE                 0x2
#define MILL_SESS_OPTION_VIRT_MOD               0x4

struct mill_session_create_destroy {
    unsigned hSession;
    struct m_mill_addr our_addr;
    unsigned options;
    unsigned err;
    unsigned buf_size;
    unsigned vol_mon_upper_thresh;
    unsigned vol_mon_lower_thresh;
};

#define MILL_MSSR_FLAG_PRIO             0x1
#define MILL_MSSR_FLAG_PKT_AVAIL        0x2
#define MILL_MSSR_FLAG_BULK_DATA        0x4
#define MILL_MSSR_FLAG_DEST_BUF         0x8
struct mill_send_rcv {
    unsigned hSession;
    unsigned char *msg;
    unsigned msg_len;
    unsigned timeout;
    unsigned flags;
    unsigned err;
    struct m_mill_addr notif_addr;
    struct m_mill_addr bulk_dest_addr;
    struct m_mill_addr bulk_src_addr;
    unsigned bulk_bits;
    unsigned bulk_stream;
};

#define MILL_HSESSION_INVALID 0xffffffff
struct mill_addr_assoc {
    unsigned hSession;
    struct m_mill_addr assoc_addr;
    unsigned err;
};

struct mill_addr_check {
    struct m_mill_addr search_addr;
    unsigned err;
};

struct mill_ver {
    unsigned driver_ver;
    unsigned build_num;
    unsigned pipr_ver;
    unsigned ioctl_interface_ver;
    unsigned pkt_ver;
    unsigned err;
};

struct mill_clear_io {
    unsigned hSession;
    struct m_mill_addr clear_addr;
    int clear_all;
    unsigned err;
};

#define DIRECT_XFER_WRITE 0
#define DIRECT_XFER_READ 1
struct mill_direct_xfer {
    unsigned hSession;
    unsigned mod_num;
    unsigned char *buf;
    unsigned buf_len;
    int direction;
    unsigned mod_addr;
    unsigned err;
};

#ifndef MILL_BUS_UNKNOWN
#define MILL_BUS_UNKNOWN        0
#define MILL_BUS_ISA            1
#define MILL_BUS_PCI            2
#define MILL_BUS_VIRT_MOD       3
#endif
struct mill_mod_info {
    unsigned mod_num;
    unsigned exists;
    unsigned num_channels;
    unsigned is_hardware;
    unsigned bus_type;
    unsigned hw_type;
    unsigned hw_id;
    unsigned ordinal_start;
    unsigned num_ordinals;
    unsigned num_ordinals_init;
    struct {
        unsigned short vend_id;
        unsigned short dev_id;
        unsigned short sub_vend_id;
        unsigned short sub_id;
        unsigned char interf_type;
    } pci_info;
    struct {
        unsigned ver_major;
        unsigned ver_middle;
        unsigned ver_minor;
        unsigned ver_build;
        unsigned ver_auto_num;
        char ver_date[20];
        char ver_comment[50];
    } boot_rom_ver_info;
    struct {
        unsigned runtime_start;
        unsigned runtime_size;
        unsigned mem_start;
        unsigned mem_size;
        unsigned modctrl_start;
        unsigned modctrl_size;
        unsigned io_start;
        unsigned io_size;
        unsigned irq;
    } hw_resource_info;
    unsigned hw_info;
    unsigned char private_info[128];
    unsigned err;
};

struct mill_mod_reconfig {
    unsigned mod_num;
    unsigned config_option;
    unsigned err;
};

struct mill_mod_reset {
    unsigned mod_num;
    unsigned err;
};

struct mill_mod_deactivate {
    unsigned mod_num;
    unsigned err;
};

struct mill_mod_config_specs {
    unsigned mod_num;
    unsigned err;
    int config_spec_num;
    unsigned config_spec_value;
    char config_spec_desc[50];
};

struct mill_flow_ctrl_forwarding {
    unsigned hSession;
    unsigned enable;
    unsigned err;
};

struct mill_async_cancel {
    unsigned hSession;
    unsigned err;
};

struct mill_time_info {
    unsigned hz_value;
    unsigned time_value;
    unsigned err;
};

#ifndef MILL_MAX_TIMEOUT
#define MILL_MAX_TIMEOUT 0xffffffff
#endif

#ifdef __OS2__
#pragma pack()
#endif

/* The version number of the ioctl interface.  Normally the number is updated
   when any structure in the user/driver interface is modified, which would be
   in this file above. If this value does not match the value returned by
   MILL_GET_VERSION, the application must abort.
*/

#define MILL_IOCTL_INTERFACE_VER_NUM 31

#endif
