/* pktdef.h */
/* Copyright (c) 1989-2004 Brooktrout Technology Inc.  All rights reserved. */

/* Include various packet definition header files from the
   boston/fw/cmndset directory. */


#ifndef PKTDEF_H
#define PKTDEF_H

/* Not a firmware facility. Reserved for control communication between
   drivers and host applications. */
#include "hostctrl.h"

/* Not a firmware facility. Reserved for control communication between
   user applications and the ECC/BSMI server applications. */
#include "ecc.h"

/* Not a firmware facility. Reserved for control communication between
   ECC and IP Call Control faciilties*/

#include "ip_cc.h"
#include "pstn_cc.h"

#include "../../driver/fw/cmndset/common.h"
#include "../../driver/fw/cmndset/admin.h"
#include "../../driver/fw/cmndset/sig_det.h"
#include "../../driver/fw/cmndset/sig_gen.h"
#include "../../driver/fw/cmndset/speech.h"
#include "../../driver/fw/cmndset/teleph.h"
#include "../../driver/fw/cmndset/switch.h"
#include "../../driver/fw/cmndset/fax.h"
#include "../../driver/fw/cmndset/atmodem.h"
#include "../../driver/fw/cmndset/ipv.h"
#include "../../driver/fw/cmndset/isdn.h"
#include "../../driver/fw/cmndset/ppi.h"
#include "../../driver/fw/cmndset/vtty.h"


#if 1
/* temp for speech caching */
#define SPEECH_CACHE_MODE 0x10
#define SPEECH_CAP_CACHE_DEF 0x10
#define SPEECH_CACHE_STARTED_EVENT 0x10
#define SPEECH_CACHE_FINISHED_EVENT 0x11
#define SPEECH_CACHE_CONFIG_DONE_EVENT 0x14
#define SPEECH_CACHE_LIST_EVENT 0x15
#define SPEECH_CACHE_CONFIG_REQUEST 0x10
#define SPEECH_CACHE_QUERY_REQUEST 0x11
#define SPEECH_CACHE_FLUSH_REQUEST 0x12
#define SPEECH_CACHE_PROMPT_ID_PAR 0x40
#define SPEECH_CACHE_PROMPT_SIZE_PAR 0x41
#define SPEECH_CACHE_STICKY_PAR 0x42
#define SPEECH_CACHE_BIN_ID_PAR 0x43
#define SPEECH_CACHE_BIN_SIZE_PAR 0x44
#define SPEECH_CACHE_BIN_COUNT_PAR 0x45
#define SPEECH_CACHE_NUMBER_PAR 0x46
#define SPEECH_CACHE_PROMPT_HITS_PAR 0x47
#define SPEECH_PLAY_CACHE_STATUS_PAR 0x48
#define SPEECH_PLAY_CACHE_PLAYING_DEF   0x1
#define SPEECH_PLAY_CACHE_EMPTY_DEF     0x2
#define SPEECH_PLAY_CACHE_BUSY_DEF      0x3
#define SPEECH_PLAY_CACHE_INSUFF_MEM_DEF        0x4
#define SPEECH_PLAY_CACHE_DISABLED_DEF  0x5
#define SPEECH_PLAY_CACHE_SUCCESS_DEF   0x0
#define SPEECH_PLAY_CACHE_INVALID_SIZE_DEF      0x1
#define SPEECH_PLAY_CACHE_ERROR_DEF     0x2
#define SPEECH_CACHE_CONFIG_SUCCESS_DEF 0x0
#define SPEECH_CACHE_CONFIG_INSUFF_MEM_DEF      0x1
#endif



/* Definitions for types based on basic components */
#define TYPE_UBYTE_FIXED_UNITLESS \
    (MILL_SIZE_BYTE | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_NONE)
#define TYPE_UBYTE_EXTENSIBLE_UNITLESS \
    (MILL_SIZE_BYTE | MILL_UNSIGNED| MILL_EXTENSIBLE | MILL_UNIT_NONE)
#define TYPE_LIST \
    (MILL_SIZE_LIST | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_NONE)
#define TYPE_UBYTE_FIXED_BOOLEAN \
    (MILL_SIZE_BYTE | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_BOOLEAN)
#define TYPE_USHORT_FIXED_UNITLESS \
    (MILL_SIZE_SHORT | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_NONE)
#define TYPE_ULONG_FIXED_UNITLESS \
    (MILL_SIZE_LONG | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_NONE)
#define TYPE_UBYTE_FIXED_CHAR \
    (MILL_SIZE_BYTE | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_CHARACTER)
#define TYPE_UBYTE_FIXED_MSEC \
    (MILL_SIZE_BYTE | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_MSEC)
#define TYPE_USHORT_FIXED_MSEC \
    (MILL_SIZE_SHORT | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_MSEC)
#define TYPE_ULONG_FIXED_MSEC \
    (MILL_SIZE_LONG | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_MSEC)
#define TYPE_LONG_FIXED_MSEC \
    (MILL_SIZE_LONG | MILL_SIGNED | MILL_FIXED | MILL_UNIT_MSEC)
#define TYPE_UBYTE_FIXED_HZ \
    (MILL_SIZE_BYTE | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_HZ)
#define TYPE_USHORT_FIXED_HZ \
    (MILL_SIZE_SHORT | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_HZ)
#define TYPE_ULONG_FIXED_HZ \
    (MILL_SIZE_LONG | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_HZ)
#define TYPE_UBYTE_FIXED_CBELLS \
    (MILL_SIZE_BYTE | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_CBELLS)
#define TYPE_USHORT_FIXED_CBELLS \
    (MILL_SIZE_SHORT | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_CBELLS)
#define TYPE_ULONG_FIXED_CBELLS \
    (MILL_SIZE_LONG | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_CBELLS)
#define TYPE_BYTE_FIXED_CBELLS \
    (MILL_SIZE_BYTE | MILL_SIGNED | MILL_FIXED | MILL_UNIT_CBELLS)
#define TYPE_SHORT_FIXED_CBELLS \
    (MILL_SIZE_SHORT | MILL_SIGNED | MILL_FIXED | MILL_UNIT_CBELLS)
#define TYPE_LONG_FIXED_CBELLS \
    (MILL_SIZE_LONG | MILL_SIGNED | MILL_FIXED | MILL_UNIT_CBELLS)
#define TYPE_UBYTE_FIXED_CBM \
    (MILL_SIZE_BYTE | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_CBM)
#define TYPE_USHORT_FIXED_CBM \
    (MILL_SIZE_SHORT | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_CBM)
#define TYPE_ULONG_FIXED_CBM \
    (MILL_SIZE_LONG | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_CBM)
#define TYPE_BYTE_FIXED_CBM \
    (MILL_SIZE_BYTE | MILL_SIGNED | MILL_FIXED | MILL_UNIT_CBM)
#define TYPE_SHORT_FIXED_CBM \
    (MILL_SIZE_SHORT | MILL_SIGNED | MILL_FIXED | MILL_UNIT_CBM)
#define TYPE_LONG_FIXED_CBM \
    (MILL_SIZE_LONG | MILL_SIGNED | MILL_FIXED | MILL_UNIT_CBM)
#define TYPE_BYTE_FIXED_UNITLESS \
    (MILL_SIZE_BYTE | MILL_SIGNED | MILL_FIXED | MILL_UNIT_NONE)
#define TYPE_SHORT_FIXED_UNITLESS \
    (MILL_SIZE_SHORT | MILL_SIGNED | MILL_FIXED | MILL_UNIT_NONE)
#define TYPE_LONG_FIXED_UNITLESS \
    (MILL_SIZE_LONG | MILL_SIGNED | MILL_FIXED | MILL_UNIT_NONE)
#define TYPE_ULONG_FIXED_BYTES \
    (MILL_SIZE_LONG | MILL_UNSIGNED | MILL_FIXED | MILL_UNIT_BYTES)


#define MILL_STD_HEADER_LEN 11

#endif
