
/****************************************************************************
 *
 * Module:               pstn_cc.h
 * Date:                 Jan 25, 2005
 * Author:               Ben Troxell
 * Purpose:              All definitions global to the PSTN_CC facility
 ****************************************************************************
 * Copyright (c) 1989-2005 Brooktrout Technology Inc.  All rights reserved. *
 *                                                                          *
 *                                                                          *
 *   All Rights Reserved.                                                   *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 * This software is furnished under a license and may only be               *
 * read, modified, copied, or otherwise used in accordance with the         *
 * terms of such license. This notice must never be removed from            *
 * this software and must be included with every copy. No other             *
 * rights to reproduce, use, or disseminate this software in any form,      *
 * in whole or in part, are granted.                                        *
 *                                                                          *
 * Brooktrout Technology, Inc. makes no representation or warranties        *
 * with respect to the performance of this software and specifically        *
 * disclaims any responsibility for any damages, special or                 *
 * consequential, connected with the use of this software.                  *
 *                                                                          *
 ****************************************************************************/
 
#ifndef PSTN_CC_H
#define PSTN_CC_H

/* Facility                   */
#define  MILL_FACILITY_PSTN_CC                  0x11

/* Event                      */
#define PSTN_CC_SETUP_IND_EVENT                 0x01
#define PSTN_CC_SETUP_ACK_EVENT                 0x02
#define PSTN_CC_INFO_EVENT                      0x03
#define PSTN_CC_CALL_PROC_EVENT                 0x04
#define PSTN_CC_ALERTING_EVENT                  0x05
#define PSTN_CC_STATUS_IND_EVENT                0x06
#define PSTN_CC_PROGRESS_EVENT                  0x07
#define PSTN_CC_CONNECT_EVENT                   0x08
#define PSTN_CC_DISCONNECT_EVENT                0x09
#define PSTN_CC_CLEAR_RQ_EVENT                  0x0a
#define PSTN_CC_FACILITY_EVENT                  0x0b
#define PSTN_CC_NOTIFY_EVENT                    0x0c
#define PSTN_CC_RESTART_EVENT                   0x0d
#define PSTN_CC_L2_UP_EVENT                     0x0e
#define PSTN_CC_L2_DOWN_EVENT                   0x0f
#define PSTN_CC_L2_DL_EST_CO_EVENT              0x10

/* Request                    */
#define PSTN_CC_SETUP_REQUEST                   0x01
#define PSTN_CC_SETUP_ACK_REQUEST               0x02
#define PSTN_CC_INFO_REQUEST                    0x03
#define PSTN_CC_CALL_PROC_REQUEST               0x04
#define PSTN_CC_ALERTING_REQUEST                0x05
#define PSTN_CC_STATUS_ENQ_REQUEST              0x06
#define PSTN_CC_PROGRESS_REQUEST                0x07
#define PSTN_CC_CONNECT_REQUEST                 0x08
#define PSTN_CC_DISCONNECT_REQUEST              0x09
#define PSTN_CC_RELEASE_REQUEST                 0x0a
#define PSTN_CC_FACILITY_REQUEST                0x0b
#define PSTN_CC_NOTIFY_REQUEST                  0x0c
#define PSTR_CC_RESTART_REQUEST                 0x0d
#define PSTN_CC_ENABLE_INTFC_REQUEST            0x0e
#define PSTN_CC_DISABLE_INTFC_REQUEST           0x0f
#define PSTN_CC_REQ_L2_STATUS_REQUEST           0x10
#define PSTN_CC_RESTART_INIT_REQUEST            0x11
#define PSTN_CC_RESTART_REQUEST                 0x12
#define PSTN_CC_RESTART_ACK_REQUEST             0x13
#define PSTN_CC_INIT_STACK_REQUEST              0x14


/* Data               */

/* Variable/Parameter         */
//Note: the first part of the name is the IE name ie PSTN_CC_BEARER_CAP_TYPE_PAR is from Bearer Cap IE
#define PSTN_CC_USER_REF_PAR                    0x01 //a 'extended' CR line access number:L3 CR
#define PSTN_CC_INTFC_SEND_CONNACK_PAR          0x02 //Used do enable/disable sending ConnAck
#define PSTN_CC_BEARER_CAP_XFER_CAP_PAR         0x03 //octet 3 bits 5-1
#define PSTN_CC_BEARER_CAP_XFER_RATE_PAR        0x04 //octet 4 bits 5-1
#define PSTN_CC_BEARER_CAP_L1_PROTO_PAR         0x05 //octet 5 bits 5-1
#define PSTN_CC_BEARER_CAP_MULTIPLIER_PAR       0x06 //octet 4.1 bits 7-1
#define PSTN_CC_CALL_STATE_VALUE_PAR            0x07 //octet 3 bits 6-1
#define PSTN_CC_CALLED_PTY_ADDR_PAR             0x08 //octet 4...
#define PSTN_CC_CALLED_PTY_NUM_TYPE_PAR         0x09 //octet 3 bits 7-5
#define PSTN_CC_CALLED_PTY_NUM_PLAN_PAR         0x0a //octet 3 bits 4-1
#define PSTN_CC_CALLED_PTY_SUBADDR_ADDR_PAR     0x0b //octet 4...
#define PSTN_CC_CALLED_PTY_SUBADDR_TYPE_PAR     0x0c //octet 3 bits 7-5
#define PSTN_CC_CALLED_PTY_SUBADDR_IND_PAR      0x0d //octet 3 bit 4
#define PSTN_CC_CALLING_PTY_ADDR_PAR            0x0e //octet 4...
#define PSTN_CC_CALLING_PTY_NUM_TYPE_PAR        0x0f //octet 3 bits 7-5
#define PSTN_CC_CALLING_PTY_NUM_PLAN_PAR        0x10 //octet 3 bits 4-1
#define PSTN_CC_CALLING_PTY_PRESENTATION_PAR    0x11 //octet 3a bits 7-6
#define PSTN_CC_CALLING_PTY_SCREENING_PAR       0x12 //octet 3a bits 2-1
#define PSTN_CC_CALLING_PTY_SUBADDR_ADDR_PAR    0x13 //octet 4...
#define PSTN_CC_CALLING_PTY_SUBADDR_TYPE_PAR    0x14 //octet 3 bits 7-5
#define PSTN_CC_CALLING_PTY_SUBADDR_IND_PAR     0x15 //octet 3 bit 4 
#define PSTN_CC_CAUSE_CODE_PAR                  0x16 //octet 4 bits 7-1
#define PSTN_CC_CAUSE_LOCATION_PAR              0x17 //octet 3 bits 4-1
#define PSTN_CC_CAUSE_DIAGS_PAR                 0x18 //octet 5
#define PSTN_CC_CHAN_IDENT_NUM_PAR              0x19 //octet 3.2 bit 5
#define PSTN_CC_CHAN_IDENT_BCHANNEL_PAR         0x1a //octet 3.3
#define PSTN_CC_CHAN_IDENT_BCHANNEL_MASK_PAR    0x1b //octet 3.3...
#define PSTN_CC_CHAN_IDENT_EXCLUSIVE_PAR        0x1c //octet 3 bit 4
#define PSTN_CC_CONNECTED_ADDR_PAR              0x1d //octet 4...
#define PSTN_CC_CONNECTED_NUM_TYPE_PAR          0x1e //octet 3 bits 7-5
#define PSTN_CC_CONNECTED_NUM_PLAN_PAR          0x1f //octet 3 bits 4-1
#define PSTN_CC_CONNECTED_PRESENTATION_PAR      0x20 //octet 3a bits 7-6
#define PSTN_CC_CONNECTED_SCREENING_PAR         0x21 //octet 3a bits 2-1
#define PSTN_CC_CONNECTED_SUBADDR_TYPE_PAR      0x22 //octet 3 bits 7-5
#define PSTN_CC_CONNECTED_SUBADDR_IND_PAR       0x23 //octet 3 bit 4
#define PSTN_CC_CONNECTED_SUBADDR_ADDR_PAR      0x24 //octet 4...
#define PSTN_CC_PARTY_CATAGORY_PAR              0x25 //octet 3 bits 3-1
#define PSTN_CC_PROGRESS_LOCATION_PAR           0x26 //octet 3 bits 4-1
#define PSTN_CC_PROGRESS_DESCR_PAR              0x27 //octet 4 bits 7-1
#define PSTN_CC_PROGRESS_CODE_STD_PAR           0x28 //octet 3 bits 7-6
#define PSTN_CC_RESTART_IND_CLASS_PAR           0x29 //octet 3 bits 3-1
#define PSTN_CC_SENDING_COMPLETE_PAR            0x2a //single octet IE 
#define PSTN_CC_TRANSIT_COUNTER_PAR             0x2b //octet 3 bits 5-1
#define PSTN_CC_NOTIFY_IND_PAR                  0x2c //a ASN.1 byte string
#define PSTN_CC_NOTIFY_DESCR_PAR                0x2d
#define PSTN_CC_FACILITY_IE_PAR                 0x2e
#define PSTN_CC_FACILITY_INVOKE_ID_PAR          0x2f //the invoke identifier from a FACILITY message
#define PSTN_CC_FACILITY_COMPONENT_PAR          0x30 //the component type of the FACILITY message
#define PSTN_CC_FACILITY_OP_VALUE_PAR           0x31 //the operation value of a FACILITY message
#define PSTN_CC_AOC_STATUS_PAR                  0x32 //AOC status from mill_api.h
#define PSTN_CC_AOC_CHARGE_INFO_PAR             0x33 //ISDN AOC currency from mill_api.h
#define PSTN_CC_AOC_CHARGE_PAR                  0x34 //'amount' returned in aocFinal.inv
#define PSTN_CC_AOC_MODE_COMBO_PAR              0x35 //AdviceModeCombination enum in chargeRequest
#define PSTN_CC_FACILITY_DATA_SIZE_PAR          0x36 //the operation data size of a FACILITY message
#define PSTN_CC_FACILITY_DATA_PTR_PAR           0x37 //the operation data pointer of a FACILITY message
#define PSTN_CC_NAME_IDENT_PAR                  0x38 //Name Identification- name
#define PSTN_CC_NAME_CHAR_SET_PAR               0x39 //Name Identification- charater set
#define PSTN_CC_INTFC_T302_TIMER_PAR            0x3a //Used do to set the T302 timer
#define PSTN_CC_QSIG_SUPPORT_PAR                0x3b //Used to pass the qsig_support callctrl.cfg parameter
#endif
