/* osilib.h */
/* OS independent function calls */
/* Copyright (c) 2001-2004 Brooktrout Technology Inc.  All rights reserved. */


#ifndef OSILIB_H
#define OSILIB_H

#include "../../../bfv.api/inc/sdk_version.h"

// define version
#define	OSI_API_MAJOR_VERSION    5
#define	OSI_API_MIDDLE_VERSION   1
#define	OSI_API_MINOR_VERSION    0
#define	OSI_API_BUILD_NUMBER     SDK_BUILD_NUMBER

#define OSI_API_COPYRIGHT_STR    "Copyright (c) 2001-2004 Brooktrout Technology"

/*  Create version string in the format "1.2.3B4"
    (major.middle.minorBbuild) */
#define OSI_API_VERSION2(a,b,c,d)   #a "." #b "." #c "B" #d
#define OSI_API_VERSION1(a,b,c,d)   OSI_API_VERSION2(a,b,c,d)
#define OSI_API_VERSION             OSI_API_VERSION1(OSI_API_MAJOR_VERSION, OSI_API_MIDDLE_VERSION, OSI_API_MINOR_VERSION, SDK_BUILD_NUMBER)


// These macros are used to rename the OSI functions. 
// These old names should no longer be in use.
/*
#define Thread_Create osiThread_Create
#define Thread_Wait osiThread_Wait
#define Thread_Destroy osiThread_Destroy
#define Semaphore_Init osiSemaphore_Init
#define Semaphore_Destroy osiSemaphore_Destroy
#define Semaphore_Increment osiSemaphore_Increment
#define Mutex_Init osiMutex_Init
#define Mutex_Release osiMutex_Release
#define Mutex_Destroy osiMutex_Destroy
#define Wait_For_Keyboard_Entry osiWait_For_Keyboard_Entry
#define Wait osiWait
#define Get_Time osiGet_Time
#define Get_Time_Stamp osiGet_Time_Stamp
*/

#ifdef WINNT
#undef _MT
#define _MT
#include <sys\types.h>
#include <sys\timeb.h>
#include <time.h>
#include <stdio.h>
#include <conio.h>
#include <windows.h>
#include <winbase.h>
#include <process.h>
#include <assert.h>
#include <winsock.h>

#define osiThread_End(param) _endthreadex(param)
#define time_x struct _timeb
#define bt_mutex HANDLE
#define bt_sem HANDLE
#define bt_thread HANDLE
#define START_SUSPENDED CREATE_SUSPEND
#define bt_socket SOCKET

#else

#define UNIX

/* NT defines for non-NT systems */
#define	STILL_ACTIVE	103
#define	INFINITE		-1L
#define osiThread_End(param)

/* NT function prototypes for non-NT systems */
int kbhit(void);

#endif		/* WINNT */



#ifdef UNIXWARE
#include <sys/types.h>
#include <sys/timeb.h>
#include <sys/filio.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <semaphore.h>
/*#include <synch.h> */
#include <poll.h>
#include <termios.h>
#include <string.h>
#include <curses.h>
#include <errno.h>

#define bt_mutex mutex_t
#define bt_sem sem_t
#define bt_thread thread_t
#define time_x struct timeb
#define START_SUSPENDED THR_SUSPENDED

#define socklen_t size_t

#endif		/* UNIXWARE */


#ifdef SOLARIS
#include <sys/types.h>
#include <sys/timeb.h>

#include <sys/filio.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <thread.h>
#include <semaphore.h>
#include <synch.h>

#include <termios.h>
#include <unistd.h>
#include <stropts.h>

#include <errno.h>

#define bt_mutex mutex_t
#define bt_sem sem_t
#define bt_thread thread_t
#define time_x struct timeb
#define START_SUSPENDED THR_SUSPENDED

#endif		/* SOLARIS */


#ifdef linux

#include <sys/types.h>
#include <sys/timeb.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/select.h>

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include <termios.h>
#include <signal.h>
#include <unistd.h>

#include <errno.h>

#define bt_mutex pthread_mutex_t
#define bt_sem sem_t
#define bt_thread pthread_t
#define time_x struct timeb
#define START_SUSPENDED	THR_SUSPENDED

#endif		/* linux */

#ifdef UNIX
/* Berkeley Socket Interface. */

#include <sys/socket.h>
#include <unistd.h>     /* For gethostname() */
#include <netdb.h>      /* For gethostbyname() */
#include <netinet/in.h> /* For htons() */

#define bt_socket int
#define SOCKET_ERROR -1
#define INVALID_SOCKET -1

#endif /* UNIX */

#ifdef SOLARIS
/* socklen_t not defined on Solaris 6 */
/*  these lines cause errors on linux with c++ */
/*  so moved from #ifdef UNIX to #ifdef SOLARIS */
#ifndef socklen_t
#define socklen_t int
#endif
#endif /* SOLARIS */

#define M_THREAD    0
#define M_PROCESS   1

#define S_THREAD    0
#define S_PROCESS   1

#define TIMEOUT_INCREMENT 100


/* Definitions for osiDelayf functions */

/* Error conditions */
#define OSI_DELAYF_ERR                 -1
#define OSI_DELAYF_MEM_ERR             -2
#define OSI_DELAYF_NOT_INITIALIZED     -3
#define OSI_DELAYF_ALREADY_INITIALIZED -4

/* Informational messages (not errors) */
#define OSI_DELAYF_NOT_FOUND            1

typedef struct bt_delayfStr
{
    struct bt_delayfStr *next;
    struct bt_delayfStr *prev;
    void (*fn) (struct bt_delayfStr *, void *);
    void *args;
    time_x triggerTime;
} bt_delayf;



/* Function Prototypes */

int osiThread_Create ( void*             stack_start,
				       int               stack_size, 
					   void*             (*start_routine)(void *),  
					   void*             arguments, 
					   int               flags, 
					   unsigned long*    thr_id);

int osiThread_Wait( bt_thread thr_id,  int *status);
int osiThread_Destroy( bt_thread thr_id);

int osiSemaphore_Init (bt_sem *Sem_ID, int resources, int type);
int osiSemaphore_Destroy(bt_sem *Sem_ID);
int osiSemaphore_Increment (bt_sem *Sem_ID);
int osiSemaphore_Decrement (bt_sem *Sem_ID, long timeout);

int osiMutex_Init (bt_mutex * Mutex_ID, int type);
int osiMutex_Acquire(bt_mutex * Mutex_ID, long timeout );
int osiMutex_Release(bt_mutex * Mutex_ID);
int osiMutex_Destroy(bt_mutex * Mutex_ID);

char osiWait_For_Keyboard_Entry (void);
void osiWait (unsigned long time);
void osiGet_Time ( time_x * timebuffer);
unsigned long osiGet_Millitime(void);
void osiGet_Time_Stamp(char * Time_Stamp);


/* Socket Function Prototypes */
int osiSockets_Init (void);
int osiSockets_Destroy (void);

bt_socket osiSocket_Create (int, int, int);
int	osiSocket_Bind (bt_socket, struct sockaddr *, int);
int	osiSocket_Connect (bt_socket, const struct sockaddr *, int);
int	osiSocket_Listen (bt_socket, int);
bt_socket osiSocket_Accept (bt_socket, struct sockaddr *, int *);
int	osiSocket_Close (bt_socket);

int osiSocket_Recv (bt_socket, void *, int, int);
int osiSocket_Send (bt_socket, const void *, int, int);

int osiSocket_SetOption (bt_socket, int, int, const void *, int);
int	osiSocket_GetHostName (char *, int);
struct hostent * osiSocket_GetHostByName (const char *);
unsigned short int osiSocket_htons (unsigned short int);

unsigned long osiLastError (void);

/* Delay function prototypes */
int osiDelayf_Init(void);
int osiDelayf_Create(void *func, void *arg, unsigned long delay, bt_delayf **id);
int osiDelayf_Cancel(bt_delayf **id);
int osiDelayf_Terminate(void);

#endif	// end if not defined OSILIB_H
