#include <stdio.h>

/* faxlist.c
 *
 * Faxlist is an UNSUPPORTED, CONTRIBUTED program for handling distribution
 * lists using enfax and sendfax.
 *
 * This program faxes a common message to everyone on a distribution list.
 *
 * It does this by calling a script name "faxmail" for each person on the
 * list, passing the name of the person as the first arg, and their fax number
 * as the second arg.  It can be easily extended to handle more args, for
 * instance, their company name.
 *
 * The shell script could invoke fax software on a remote machine.
 *
 * This program's standard input should contain the message (it can be 
 * PostScript), which is in turn passed to "faxmail" for each recipient.
 * 
 * Calling syntax:
 *	faxlist message distribution_list
 *
 *  The format of the distribution list file is
 *
 *		Name of first recip
 *		Fax phone of first recip.
 *
 *		Name of 2nd recip
 *		Fax phone of 2nd recip.
 *
 *		etc.
 *
 * Note: The phone number can contain blanks only if you have the latest
 * version of faxmail, which has double quotes around the $2 in the enfax and
 * sendfax calls.
 *
 */

char tmp_filename[200];

main(argc, argv)
char **argv;
{
	char s[200], name[100], phone[100];
	char *get_list_line();
	FILE *fp, *dlist_fp;

	if (argc != 2)
	{
		printf("syntax: faxlist message list\n");
		exit(1);
	}

	/* capture stdin in a temporary file, since we need to use
	   it for each person in the dist. list */

	sprintf(tmp_filename, "/tmp/isod%d", getpid());

	capture_stdin();

	/* open dist. list file */
	dlist_fp = fopen(argv[1], "r");
	if (dlist_fp == NULL)
	{
		perror("");
		exit(1);
	}
	/* for each recipient call the isofax shell script */
	for (;;)
	{
		/* get the name */
		if (get_list_line(name, sizeof(name), dlist_fp) == NULL)
			break;	/* end of list */

		/* ignore blank line when looking for name */
		if (strlen(name) == 0)
			continue;

		/* get the phone number */
		if (get_list_line(phone, sizeof(phone), dlist_fp) == NULL)
			break;	/* end of list */
	
		/* call the shell script; the first arg is the name, the
		   second arg is the phone number */
		sprintf(s,
			"faxmail '%s' '%s'",
			name,
			phone);

		call_shell_script(s);
	}
	unlink(tmp_filename);
}

/* get a line from the dist. list file */
char *
get_list_line(s, n, dlist_fp)
char *s;
FILE *dlist_fp;
{
	if (fgets(s, n, dlist_fp) == NULL)
		return NULL;

	if (strlen(s) == 0)
		return s;

	s[strlen(s) - 1] = '\0';	 /* get rid of newline */

	return s;
}

/* capture this pg's stdin in a temporary file */
capture_stdin()
{
	int c;
	FILE *fp;

	fp = fopen(tmp_filename, "w");
	if (fp == NULL)
	{
		perror("");
		exit(1);
	}

	for (;;)
	{
		 if ((c = getchar()) == EOF)
			break;
		 putc((char) c, fp);
	}

	fclose(fp);
}

/* call the shell script and pipe it this program's stdin */
call_shell_script(shell_command)
char *shell_command;
{
	int c;
	FILE *fp, *ifp;

	fp = popen(shell_command, "w");

	sprintf(tmp_filename, "/tmp/isod%d", getpid());

	ifp = fopen(tmp_filename, "r");
	if (ifp == NULL)
	{
		perror("");
		exit(1);
	}
	for (;;)
	{
		if ((c = getc(ifp)) == EOF)
			break;
		
		 putc((char) c, fp);
	}
	pclose(fp);
	pclose(ifp);
}
