#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

static char SCCS_ID[] = "@(#)|faxtolist.c|9|01|2|0|0|07/26/99|14:47:48";

/* faxtolist.c - send fax to everyone on an IsoFax distribution list
 *
 * ISOFAXHOME must be set in the environment to that this program can
 * execute enfax and sendfax. 
 *
 * If the env. var. DLIST_TEST is set, no actual faxing will be done, but
 * a list of the commands which would have been executed will instead be
 * be written to stdout.
 *
 * faxtolist -d doc -b book -l dlname [-p] [-c coversheet] [-e enfax_sws] [-s
 *                 sendfax_sws]
 *
 *     doc -- path to faxable document.  Must be in IsoFax 'fax' format
 *     book -- path to an IsoFax address book
 *     dlname -- name of an IsoFax dist list in <book.dlists>.  If -p sw. is
 *               included, than we'll look for an individual to match, if no
 *               dlist name matches
 *     coversheet -- path to an optional IsoFax cover sheet.
 *     enfax_sws -- optional arguments for enfax, when coversheeets are used'
 *                  e.g., '-s cancun_server'.  You will probably to need to put
 *                  quotes around these arguments.
 *     sendfax_sws -- optional arguments for sendfax; e.g. '-s cancun_server'
 */

char name[100];
char dept[100];
char company[100];
char fax_number[100];
char secondary_fax_number[100];
char email[100];
char voice_number[100];
char mail_stop[100];
char addr1[100];
char addr2[100];
char record_number[100];
char salutation[100];

FILE *fp;

char *cp, s[10000];
char local_sendfax_sws[100] = "";
char local_enfax_sws[100] = "";

void syntax();
void parse_line();
void get_field(char *);
void strip_trailing_spaces(char *s);

void
main(int argc, char **argv)
{
	int found_list, i = 0, opt, n_members = 0, execute = 1;
	int look_for_individual = 0, found_individual, dlist_indices[10000];
	char ss[100], exec[1000], dlist_filename[300], dlist_name[100];
	char *phonebook_path = NULL, *named_list = NULL, *coversheet_path = NULL;
	char *faxable_doc, *enfax_sws, *sendfax_sws;
	extern char *optarg;

	faxable_doc = enfax_sws = sendfax_sws = NULL;
	while ((opt = getopt(argc, argv, "d:b:l:pc:e:s:")) >= 0)
	{
		switch (opt)
		{
		  case 'd':
			faxable_doc = optarg;
			break;
		  case 'b':
			phonebook_path = optarg;
			break;
		  case 'l':
			named_list = optarg;
			break;
		  case 'c':
			coversheet_path = optarg;
			break;
		  case 'e':
			enfax_sws = optarg;
			break;
		  case 's':
			sendfax_sws = optarg;
			break;
		  case 'p':
			look_for_individual = 1;
			break;
		  default:
			syntax();
		}
	}

	if (faxable_doc == NULL ||
	    phonebook_path == NULL ||
	    named_list == NULL)
		syntax();

	if (sendfax_sws != NULL)
		strcpy(local_sendfax_sws, sendfax_sws);
	if (enfax_sws != NULL)
		strcpy(local_enfax_sws, enfax_sws);

    if (getenv("DLIST_TEST"))
    {
        printf("env var DLIST_TEST set, no faxing\n");
        execute = 0;
    }

	sprintf(dlist_filename, "%s.dlists", phonebook_path);
	fp = fopen(dlist_filename, "r");
	if (fp == NULL)
	{
        printf("(fatal) Couldn't find dlist '%s'\n", dlist_filename);
		exit(1);
	}
		
	strip_trailing_spaces(named_list);
	found_list = 0;
	for (;;)
	{
		if (fgets(dlist_name, 1000, fp) == NULL)
			break;		/* not found */

		dlist_name[strlen(dlist_name) - 1] = '\0';

		strip_trailing_spaces(dlist_name);

		if (!strcmp(named_list, dlist_name))
		{
			found_list = 1;
			break;		/* found */
		}
		
		if (fgets(s, 10000, fp) == NULL)		/* skip over member list */
			break; /* not found or bad dlist */
	}
	if (found_list)
		printf("found dlist '%s'\n", dlist_name);
	else
	{
		if (look_for_individual == 1)
		{
			dlist_name[0] = '\0';
			printf(
				"didn't find dlist '%s', presuming it's an individual name\n",
												 named_list);
			goto openbook;
		}
		else
		{
			printf("(fatal) didn't find dlist '%s'\n", named_list);
			exit(1);
		}
	}

	if (fgets(s, 1000, fp) == NULL)
	{
		printf("(fatal) problem with dlist file '%s'", named_list);
		exit(2);
	}
	cp = NULL;
	n_members = 0;
	for (;;)
	{
		if (cp == NULL)
			cp = strtok(s, " ");		/* first time */
		else
			cp = strtok(NULL, " ");		/* every other time */

		if (cp == NULL)
			break;

		dlist_indices[n_members++] = atoi(cp);
	}
	printf("dlist has %d members\n", n_members);

	fclose(fp);
	
	openbook:

	fp = fopen(phonebook_path, "r");
	if (fp == NULL)
	{
		printf("(fatal) Couldn't open phone book '%s'\n", phonebook_path);
		exit(3);		/* no phonebook */
	}

	found_individual = 0;
	for (;;)
	{
		if (fgets(s, 1000, fp) == NULL)	/* get next record */
			break;
		cp = s;
		parse_line();					/* extract the fields */

		/* special case processing to handle individuals rather than lists */
		if (found_list == 0)	
		{
			/* the following comparison is actually between the supplied
             * list name and the next individual in the phone book.  You
             * might want to make it more elaborate in terms of captialization
             * and closeness of fit.  Right now everything but be exact
             */
			if (!strcmp(named_list, name))
			{
				printf("found individual: '%s'\n", name);
				found_individual = 1;
				goto faxdude;	/* name match */
			}
			else
				continue;
		}
			
		for (i = 0; i < n_members; i++)
		{
			if (atoi(record_number) == dlist_indices[i])
			{
				faxdude:
				if (coversheet_path != NULL && strlen(coversheet_path))
				{
					sprintf(exec,
				       "enfax %s -b3 -2 -a1 '%s' -e '%s' -g '%s' -h '%s' -m '%s'"
					   " -o '%s' -p '%s' -r '%s' -v '%s' %s %s /tmp/dfax%d",
					   local_enfax_sws, addr1, email, dept, salutation,
					   mail_stop, company, fax_number, name, voice_number,
					   coversheet_path, faxable_doc, (int)getpid());
					printf("%s\n", exec);
					if (execute)
						system(exec);

					sprintf(exec, "sendfax %s -r '%s' /tmp/dfax%d '%s'",
							 local_sendfax_sws, name, (int)getpid(), fax_number);
					printf("%s\n", exec);
					if (execute)
						system(exec);

					sprintf(exec, "rm /tmp/dfax%d", (int)getpid());
					printf("%s\n", exec);
					if (execute)
						system(exec);
				}
				else
				{
					sprintf(exec, "sendfax %s -r '%s' '%s' '%s'",
						 local_sendfax_sws, name, faxable_doc, fax_number);
					printf("%s\n", exec);
					if (execute)
						system(exec);
				}
			}
		}
	}
	if (found_list || found_individual)
		exit(0);
	else
	{
		printf("(fatal) didn't find individual '%s' either\n", named_list);
		exit(1);		/* no list and no individual found */
	}
}


void
parse_line()
{
	get_field(name);
	get_field(dept);
	get_field(company);
	get_field(fax_number);
	get_field(secondary_fax_number);
	get_field(email);
	get_field(voice_number);
	get_field(mail_stop);
	get_field(addr1);
	get_field(addr2);
	get_field(record_number);
	get_field(salutation);
}

void
get_field(field)
char * field;
{
		char *field_start = field;
		int c;

		for (;;)
		{
			c = *cp++;
			if (c == '\n')
			{
				printf("(fatal) premature EOR in phone book\n");
				exit(1);
			}
			else if (c == ':')
				break;
			else if (c == '\\')
				c = *cp++;
			*field++ = c;
		}
		*field = '\0';
}

void
syntax()
{
	printf(
	"syntax: faxtolist -d doc -b book -l dlname [-c coversheet] [-e enfax_sws] "
		"[-s sendfax_sws]\n");
	exit(1);
}

void
strip_trailing_spaces(char *s)
{
	if (s[strlen(s) - 1] != ' ')
		return;

	s[strlen(s) - 1] = '\0';
	strip_trailing_spaces(s);
}
