#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>

#if defined(SUNOS)
int printf(char *,...);
unsigned long time(unsigned long *);
#endif

/* exit codes:
 *	0 -- server info file exists, has been mod. in last "old" seconds
 *	1 -- server info file doesn't exist
 *	2 -- server info file not mod. in last "old" seconds
 */

/* args:  1 - pathname for work directory
 *        2 - "old" criterion in seconds
 */
void main(int argc, char **argv)
{
	struct stat statbuf1, statbuf2;
	char time_filename[200], server_info_filename[200];
	int time_fd;
	time_t numsecs;

	sprintf(time_filename, "%s/tempfile", argv[1]);
	sprintf(server_info_filename, "%s/server_info", argv[1]);

	if (stat(server_info_filename, &statbuf1))
	{
		printf("server_info file '%s' doesn't exist\n",
		       server_info_filename);
		exit(1);	/* file doesn't exist */
	}

	time_fd = open(time_filename, O_WRONLY|O_CREAT, 0666);
	write(time_fd, "temp", 4);

	/* use old method for determining age of file */
	if (stat(time_filename, &statbuf2))
	{
	    printf("temp file '%s' wasn't created...\n", time_filename);

	    numsecs = time(NULL);  /* get unix time of day */
	    if (numsecs > statbuf1.st_mtime + atoi(argv[2]))
	      exit(2);	/* too old */
	    else 
	      exit(0);
	}
	/* new method: compare newly created file time with last update
	 * time of server_info file.  These times depend on the files'
	 * disk partition machine's time, not client machine's time
	 */	 
	else
	{
	    if (abs(statbuf1.st_mtime - statbuf2.st_mtime) > atoi(argv[2]))
	      {
		close(time_fd);
		exit(2);
	      }
	}

	close(time_fd);
	exit(0);
}
