/* transfer_phones.c
 *
 * Transfer_phones is an UNSUPPORTED, CONTRIBUTED program for merging
 * records from other databases into an existing IsoFax address book or
 * simply creating a new IsoFax address book from them.  To see an explanation
 * of its syntax and use, run it with the argument 'formathelp':
 *    % transfer_phones formathelp
 *
 */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>

#if defined(SUNOS)
int printf(char *,...);
int fprintf(FILE *,char *,...);
int fclose(FILE *);
int system(char *);
void perror(char *);
#endif
#include "fax.h"   /* holds phone_entry struct */

#define NUM_AB_FIELDS  10 /* could change some day */

/*
 * This function takes as input a non-IsoFax-format address book
 * with a special header (which explains which fields are represented
 * and in what order), and a destination filename (to either be created
 * or appended to) for the new IsoFax-format address_book.
 */

phone_entry trphones[10000];  /* struct for all entries */
int num_entries = 0;                /* how many there really are */

/* ANSI prototypes */
void printformat(void);
void syntax(void);
int parse_header(FILE *src_file, int *fo);
void set_up_phone_entry(phone_entry *cs_ptr);
void create_ab_struct(char *record_ptr, phone_entry *dest_struct, int which_field);
void add_entry(phone_entry *newentry, FILE *dest);
int last_name_compare(const void *void_i, const void *void_j);
void reorder_name(char *s, char *target);
unsigned int read_and_find_ident(FILE *given_file);

int
main(int argc, char **argv)
{
  char *dest_ab;
  char *src_ab;
  FILE *dest_file;
  FILE *src_file;
  int fields_order[NUM_AB_FIELDS];
  int num_src_fields = 0;
  char ab_record[300];
  struct stat s_buf;
  unsigned int current_ident = 1;
  int field_count = 0;
  int i;
  char s[MAXPATHLEN];  /* for system call */
  char s2[MAXPATHLEN];
  
  if ((argc == 2) && (strcmp(argv[1], "formathelp") == 0))
    {
      printformat();
      return 0;
    }
  else if (argc != 3)
    {
      syntax(); 
      return 1;
    }
  else
    {
      src_ab = argv[1];
      dest_ab = argv[2];
    }

  if ((src_file = fopen(src_ab, "r")) == NULL)
    {
      sprintf(s2,"transfer_phones, opening file '%s' for reading",src_ab);
      perror(s2);
      return 1;
    }
      
  if (!stat(dest_ab, &s_buf))  /* file exists already */
    {
      if ((dest_file = fopen(dest_ab, "r")) == NULL) /* read first */
	{
	  sprintf(s2,"transfer_phones, opening file '%s' for reading",dest_ab);
	  perror(s2);
	  return 1;
	}
      current_ident = read_and_find_ident(dest_file);

      if ((dest_file = fopen(dest_ab, "a")) == NULL) /* then append */
	{
	  sprintf(s2,"transfer_phones, opening file '%s' for appending",dest_ab);
	  perror(s2);
	  return 1;
	}
    }
  else if ((dest_file = fopen(dest_ab, "w")) == NULL)
    {
      sprintf(s2,"transfer_phones, opening file '%s' for writing",dest_ab);
      perror(s2);
      return 1;
    }

  if ((num_src_fields = parse_header(src_file, fields_order)) == -1) /*failed*/
    return 1;

  if (num_src_fields == 0)
    {
      printf("Error:  According to header of source address book, all entries have 0 fields.");
      return 1;
    }
  
  while (fgets(ab_record, 300, src_file) != NULL) /* cycle through records */
    {                                             /* and blank lines */
      phone_entry *current_struct = &trphones[num_entries];

      if (strcmp(ab_record,"\n") == 0) /* pass over blank lines */
	{
	  field_count = 0; /* resets this var. between records */
	  continue;
	}

      if (field_count > 0) /* means never got newline between records */
	{
	  printf("Error:  Number of fields per entry does not match the header of the source address book.\n");
	  return 1;
	}

      set_up_phone_entry(current_struct); /* set all strings to "" */

      for(field_count = 0; field_count<num_src_fields; field_count++) 
	{
	  int putnull;
	  
	  if (field_count != 0) /* if not first time thru, get next field */
	    if (fgets(ab_record, 300, src_file) == NULL) 
	      break;
	  if (strcmp(ab_record,"\n") == 0)
	    {
	      field_count = 0;
	      break;
	    }
	  putnull = strlen(ab_record) -1;	  
      	  ab_record[putnull] = '\0';  /* convert newline to null */

	  create_ab_struct(ab_record, current_struct, fields_order[field_count]);
	}
      if (field_count < num_src_fields)
	{
	  printf("Error: \n");
	  printf("       An entry in the source address book does not have as many\n");
	  printf("       fields as specified in the header.  It will not be included in the\n");
	  printf("       destination address book file.\n");
	  printf("       However, all other entries have been processed.\n");
	  printf("\n");
	}
      else  /* record had enough fields */
	{
	  num_entries++;  /* finally count the completed entry */
	  current_struct->ident = current_ident++;
	}
    }

  /* sort all according to last names first */
  qsort((char *)trphones, num_entries, sizeof(phone_entry),
	last_name_compare);
  fclose(dest_file);
  sprintf(s, "rm %s", dest_ab);
  system(s);               /* remove old dest file before write new one */
  if ((dest_file = fopen(dest_ab, "w")) == NULL)
    {
      sprintf(s2,"transfer_phones, opening file '%s' for writing",dest_ab);
      perror(s2);
      return 1;
    }
  for (i=0; i < num_entries; i++)
    {
      add_entry(&trphones[i], dest_file);
    }
  return 0;
}

int
last_name_compare(const void *void_i, const void *void_j)
{
  phone_entry *iphone = (phone_entry *) void_i;
  phone_entry *jphone = (phone_entry *) void_j;
  char *i = iphone->name;
  char *j = jphone->name;
  char targeti[200];
  char targetj[200];
  char *ti, *tj;

  reorder_name(i, targeti);  /* need to link with send.c */
  reorder_name(j, targetj);

  if (strcmp(targeti, targetj) == 0)  /* Last names same */
    {
      ti = (char *)strtok(targeti, ", "); 
      tj = (char *)strtok(targetj, ", ");

      return (strcmp(ti, tj));  /* so compare first names */
    }                           /* (stop bothering after that) */
  else
    return (strcmp(targeti, targetj));
}  
  
int
parse_header(FILE *src_file, int *fo) 
{
  char format_line[100];
  int found[NUM_AB_FIELDS];
  int doparse = 0;
  int done = 0;
  int fo_count = 0;
  int i = 0;

  for(; i< NUM_AB_FIELDS; i++)
    found[i] = 0;
  
  while (!done && (fgets(format_line, 100, src_file) != NULL))
    {
      format_line[strlen(format_line)] = '\0';  /* convert newline to null */

      /* repeated use of strncmp prevents spaces from causing trouble */
      if (!doparse)
	{
	  if (format_line[0] == '#')
	    {
	      if (strncmp(format_line, "#start", 6) == 0)
		doparse = 1;
	    }
	  continue;  /* keep 'fgets'ing in while loop until find #start */
	}

      if (format_line[0] == '#')
	{
	  if (strncmp(format_line, "#stop", 5) == 0)
	    done = 1;
	  continue;
	}

      if ((strncmp(format_line, "ORG", 3) == 0) && (!found[2]))
	{
	  found[2] = 1;
	  fo[fo_count++] = 2;
	}
      else if ((strncmp(format_line, "NAME", 4) == 0) && (!found[0]))
	{
	  found[0] = 1;
	  fo[fo_count++] = 0;
	}
      else if ((strncmp(format_line, "DEPT", 4) == 0) && (!found[1]))
	{
	  found[1] = 1;
	  fo[fo_count++] = 1;
	}
      else if ((strncmp(format_line, "EMAIL", 5) == 0) && (!found[5]))
	{
	  found[5] = 1;
	  fo[fo_count++] = 5;
	}
      else if ((strncmp(format_line, "FAXNUM1", 7) == 0) && (!found[3]))	
	{
	  found[3] = 1;
	  fo[fo_count++] = 3;
	}
      else if ((strncmp(format_line, "FAXNUM2", 7) == 0) && (!found[4]))	
	{
	  found[4] = 1;
	  fo[fo_count++] = 4;
	}
      else if ((strncmp(format_line, "VOICENUM", 8) == 0) && (!found[6]))
	{
	  found[6] = 1;
	  fo[fo_count++] = 6;
	}
      else if ((strncmp(format_line, "MAILSTOP", 8) == 0) && (!found[7]))
	{
	  found[7] = 1;
	  fo[fo_count++] = 7;
	}
      else if ((strncmp(format_line, "MAIL_ADDRESS1", 13) == 0) && (!found[8]))
	{
	  found[8] = 1;
	  fo[fo_count++] = 8;
	}
      else if ((strncmp(format_line, "MAIL_ADDRESS2", 13) == 0) && (!found[9]))
	{
	  found[9] = 1;
	  fo[fo_count++] = 9;
	}
      else
	{
	  printf ("ERROR in header format of source address book!  '%s' is not a recognized address book field.\n", format_line);
	  return -1;
	}

    }

  if (done)
    fo[fo_count] = -1;  /* to signify end of entries */
  else                  /* got out of while loop before seeing #stop */
    {                   /* OR never saw #start */
      if (doparse)
	printf("ERROR in header format of source address book!  Missing '#stop' line.\n");
      else
	printf("ERROR in header format of source address book!  Missing '#start' line.\n");
      return -1;
    }
  
  return fo_count;
}

void
create_ab_struct(char *record_ptr, phone_entry *dest_struct, int which_field)
{
  switch (which_field)
    {
    case 0:
      record_ptr = strcpy(dest_struct->name, record_ptr);
      break;
    case 1:
      record_ptr = strcpy(dest_struct->dept, record_ptr);
      break;
    case 2:
      record_ptr = strcpy(dest_struct->org, record_ptr);
      break;
    case 3:
      record_ptr = strcpy(dest_struct->faxnum, record_ptr);
      break;
    case 4:
      record_ptr = strcpy(dest_struct->faxnum2, record_ptr);
      break;
    case 5:
      record_ptr = strcpy(dest_struct->email, record_ptr);
      break;
    case 6:
      record_ptr = strcpy(dest_struct->voicenum, record_ptr);
      break;
    case 7:
      record_ptr = strcpy(dest_struct->mailstop, record_ptr);
      break;
    case 8:
      record_ptr = strcpy(dest_struct->mail_address, record_ptr);
      break;
    case 9:
      record_ptr = strcpy(dest_struct->mail_address2, record_ptr);
      break;
    }
}

void
add_entry(phone_entry *newentry, FILE *dest)
{
  fprintf(dest, "%s:", newentry->name);
  fprintf(dest, "%s:", newentry->dept);
  fprintf(dest, "%s:", newentry->org);
  fprintf(dest, "%s:", newentry->faxnum);
  fprintf(dest, "%s:", newentry->faxnum2);
  fprintf(dest, "%s:", newentry->email);
  fprintf(dest, "%s:", newentry->voicenum);
  fprintf(dest, "%s:", newentry->mailstop);
  fprintf(dest, "%s:", newentry->mail_address);
  fprintf(dest, "%s:", newentry->mail_address2);
  fprintf(dest, "%d:", newentry->ident);
  fprintf(dest, "\n");
}

void
set_up_phone_entry(phone_entry *cs_ptr)
{
  strcpy(cs_ptr->name, "");
  strcpy(cs_ptr->dept, "");
  strcpy(cs_ptr->org, "");
  strcpy(cs_ptr->faxnum, "");
  strcpy(cs_ptr->faxnum2, "");
  strcpy(cs_ptr->email, "");
  strcpy(cs_ptr->voicenum, "");
  strcpy(cs_ptr->mailstop, "");
  strcpy(cs_ptr->mail_address, "");
  strcpy(cs_ptr->mail_address2, "");
}

/* based on utils.c's 'next_field' */
static char
*copy_field(char *from, char *to)  
{
  while (*from != ':' && *from != '\0')
    *to++ = *from++;

  if (*from == ':')
    from++;

  *to = '\0';

  return from;
}	 

unsigned int
read_and_find_ident(FILE *given_file) /* based on utils.c's 'read_phone_dir' */
{
  unsigned int largest_ident_found = 1;
  char entry[300];
  char asc_ident[100];
  int found;

  while (fgets(entry, 300, given_file) != NULL)
    {
      phone_entry *current_struct = &trphones[num_entries++];
      char *ptr = entry;
      entry[(strlen(entry) - 1)] = '\0'; /* convert newline to null */

      set_up_phone_entry(current_struct); /* set all strings to "" */  

      ptr = copy_field(ptr,current_struct->name);
      ptr = copy_field(ptr,current_struct->dept);
      ptr = copy_field(ptr,current_struct->org);
      ptr = copy_field(ptr,current_struct->faxnum);
      ptr = copy_field(ptr,current_struct->faxnum2);
      ptr = copy_field(ptr,current_struct->email);
      ptr = copy_field(ptr,current_struct->voicenum);
      ptr = copy_field(ptr,current_struct->mailstop);
      ptr = copy_field(ptr,current_struct->mail_address);
      ptr = copy_field(ptr,current_struct->mail_address2);
      ptr = copy_field(ptr,asc_ident);
      
      found = atoi(asc_ident);
      current_struct->ident = found;
      
      if (found > (int) largest_ident_found)
	largest_ident_found = found;
    }
  return (largest_ident_found + 1);
}

void
test_array(int *arr)
{
  int i = 0;
  
  for(; arr[i] != -1; i++)
    printf("Array entry %d is %d\n", i, arr[i]);
}

void
printformat(void)
{
  printf("\n");
  printf("This is the format of a source address book's header.  \n");
  printf("Lines between #start and #stop should be in the order of\n");
  printf("fields in the source file. Fields not represented in the\n");
  printf("source file shouldn't be listed.  Records should begin \n");
  printf("after the #stop line, with fields separated by newlines and \n");
  printf("whole records separated by blank lines (As in this example).\n");
  printf("\n");
  printf("#start \n");
  printf("NAME \n");
  printf("DEPT \n");
  printf("ORG\n");
  printf("FAXNUM1\n");
  printf("FAXNUM2\n");
  printf("EMAIL\n");
  printf("VOICENUM\n");
  printf("MAILSTOP\n");
  printf("MAIL_ADDRESS1\n");
  printf("MAIL_ADDRESS2\n");
  printf("#stop\n");
  printf("\n");
  printf("Record1_field1\n");
  printf("Record1_field2\n");
  printf("...\n");
  printf("Record1_lastfield\n");
  printf("\n");
  printf("Record2_field1\n");
  printf("Record2_field2\n");
  printf("etc.\n");
  printf("\n");
}

void
print_phone_entry_struct(phone_entry *str)
{
  printf("From inside print_phone_entry_struct :\n");
  printf("Name is %s\n", str->name);
  printf("Dept is %s\n", str->dept);
  printf("Org is %s\n", str->org);
  printf("Faxnum is %s\n", str->faxnum);
  printf("Faxnum2 is %s\n", str->faxnum2);
  printf("Email is %s\n", str->email);
  printf("Voicenum is %s\n", str->voicenum);
  printf("Mailstop is %s\n", str->mailstop);
  printf("Mail_address is %s\n", str->mail_address);
  printf("Mail_address2 is %s\n", str->mail_address2);
}

void
syntax(void)
{
  printf("\nSyntax for calling transfer_phones: \n");
  printf("\n");
  printf(" 'transfer_phones source_address_book_file destination_address_book_file'\n");
  printf("\n");
  printf("\n OR 'transfer_phones formathelp' for a description of the header format\n");
  printf("of the source address book file.\n");
  printf("\n");
}
  
/* reorder "name0 name1 ... nameN" to "nameN, name0, name1..." */
void
reorder_name(char *s, char *target)
{
        char ls[200], *t;
        char last_name[200];
        char t0[100];
        char t1[100];
        char t2[100];
        char t3[100];
	char t4[100];
        char t5[100];

        target[0] = '\0';
        t0[0] = '\0';
        t1[0] = '\0';
        t2[0] = '\0';
        t3[0] = '\0';
        t4[0] = '\0';
        t5[0] = '\0';

        strcpy(ls, s);

	if ((t = (char *) strtok(ls, " \n")) == NULL)
	        return;

        strcpy(t0, t);   /* pick up first string */

        if ((t = (char *) strtok(NULL, " \n")) == NULL)
	  {
                strcpy(target, t0);
                return;
	  }
        strcpy(t1, t);  /* pick up second string */

        if ((t = (char *) strtok(NULL, " \n"))== NULL)
	  {
                strcpy(last_name, t1);
                t1[0] = '\0';
                goto common;
	      }
        strcpy(t2, t);

        if ((t = (char *) strtok(NULL, " \n")) == NULL)
        {
                strcpy(last_name, t2);
                t2[0] = '\0';
                goto common;
        }
        strcpy(t3, t);

        if ((t = (char *) strtok(NULL, " \n")) == NULL)
        {
                strcpy(last_name, t3);
                t3[0] = '\0';
                goto common;
        }
        strcpy(t4, t);

        if ((t = (char *) strtok(NULL, " \n")) == NULL)
        {
                strcpy(last_name, t4);
		t4[0] = '\0';
                goto common;
        }
        strcpy(t5, t);
        strcpy(last_name, t5);
        t5[0] = '\0';

        common:
      
	strcpy(target, last_name);
        strcat(target, ",");

        if (strlen(t0))
	  {
                strcat(target, " ");
		strcat(target, t0);
	  }
        if (strlen(t1))
	  {
                strcat(target, " ");
		strcat(target, t1);
	  }
        if (strlen(t2))
	  {
                strcat(target, " ");
                strcat(target, t2);
	  }
	if (strlen(t3))
	  {
                strcat(target, " ");
                strcat(target, t3);
	  }
        if (strlen(t4))
	  {
                strcat(target, " ");
                strcat(target, t4);
	  }
        return;
}
