
/* SCCS_ID = "@(#)group3.h	9.01.2.0.0	07/26/99	14:35:32" */

/*
 *
 * group3.h - common include file for group III sources.
 */

#if !defined(INCLUDE_GROUP3_H)
#define INCLUDE_GROUP3_H

#if defined(__cpluplus)
	extern "C" {
#endif

/*
 * T.30 session subparameter code enumerations.
 *
 * all translations between enumerations and the T30 session parameter
 * characters used by modems are handled by the conversion routines
 * in fax_header.c (see the prototypes below).
 *
 * if you add enums, update the functions in fax_header.c!
 *
 * -gm 8/24/95
 */

/*
 * T.30 vertical resolution
 *
 * for backward compatibility, VR_98 _must_ equal 0, and VR_196 _must_
 * equal 1!
 *  -gm 8/28/95
 */
enum VR_t {
	VR_ERROR	= -1,	/* keeps clever compilers quiet */
	VR_98		= 0,
	VR_196		= 1,
	VR_300,
	VR_400,
	VR_600,
	VR_NULL
};
typedef enum VR_t VR_t;

/*
 * T.30 bit rate
 */
enum BR_t {
	BR_ERROR = -1,	/* keeps clever compilers quiet */
	BR_2400	= 0,	/* 2400  bps V.27 ter */
	BR_4800,		/* 4800  bps V.27 ter */
	BR_7200,		/* 7200  bps V.29 or V.17 */
	BR_9600,		/* 9600  bps V.29 or V.17 */
	BR_12000,		/* 12000 bps V.33 or V.17 */
	BR_14400,		/* 14400 bps V.33 or V.17 */
	BR_NULL
};
typedef enum BR_t BR_t;

/*
 * T.30 page width
 */
enum WD_t {
	WD_ERROR = -1,	/* keeps clever compilers quiet */
	WD_1728	= 0,	/* 1728 pixels in 215 mm */
	WD_2048,		/* 2048 pixels in 255 mm */
	WD_2432,		/* 2432 pixels in 303 mm */
	WD_1216,		/* 1216 pixels in 151 mm */
	WD_864,			/* 864  pixels in 107 mm */
	WD_NULL
};
typedef enum WD_t WD_t;

/*
 * T.30 page length
 */
enum LN_t {
	LN_ERROR = -1,	/* keeps clever compilers quiet */
	LN_A4 = 0,		/* A4, 297 mm */
	LN_B4,			/* B4, 364 mm */
	LN_NONE,		/* unlimited length */
	LN_NULL
};
typedef enum LN_t LN_t;

/*
 * T.30 data compression format
 */
enum DF_t {
	DF_ERROR = -1,	/* keeps clever compilers quiet */
	DF_MH = 0,		/* 1-D modified huffman */
	DF_MR,			/* 2-D modified read */
	DF_UMMR,		/* 2-D uncompressed mode */
	DF_MMR,			/* 2-D modified modified huffman */
	DF_NULL
};
typedef enum DF_t DF_t;

/*
 * T.30 error correction
 */
enum EC_t {
	EC_ERROR = -1,	/* keeps clever compilers quiet */
	EC_OFF = 0,		/* disable ECM */
	EC_ECM_64,		/* enable ECM, 64 bytes/frame */
	EC_ECM_256,		/* enable ECM, 256 bytes/frame */
	EC_NULL
};
typedef enum EC_t EC_t;

/*
 * T.30 binary file transfer
 */
enum BF_t {
	BF_ERROR = -1,	/* keeps clever compilers quiet */
	BF_OFF = 0,		/* disable BFT */
	BF_ON,			/* enable BFT */
	BF_NULL
};
typedef enum BF_t BF_t;

/*
 * T.30 scan time/line
 * (i've no clue what to name these.. do we even use them?)
 */
enum ST_t {
	ST_ERROR = -1,	/* keeps clever compilers quiet */
	ST_0 = 0,		/* 0/0 ms (VR = normal/VR = fine) */
	ST_1,			/* 5/5 ms (VR = normal/VR = fine) */
	ST_2,			/* 10/5 ms (VR = normal/VR = fine) */
	ST_3,			/* 10/10 ms (VR = normal/VR = fine) */
	ST_4,			/* 20/10 ms (VR = normal/VR = fine) */
	ST_5,			/* 20/20 ms (VR = normal/VR = fine) */
	ST_6,			/* 40/20 ms (VR = normal/VR = fine) */
	ST_7,			/* 40/40 ms (VR = normal/VR = fine) */
	ST_NULL
};
typedef enum ST_t ST_t;

/*
 * T.30 session information structure
 */
struct fax_caps_t {
	VR_t vertical_res;
	BR_t bit_rate;
	WD_t page_width;
	LN_t page_length;
	DF_t compress_format;
	EC_t ecm_mode;
	BF_t bft_flag;
	ST_t scan_time;
};
typedef struct fax_caps_t fax_caps_t;

/*
 * T30 enum <-> modem session parameter conversion functions..
 */

int VRtoascii(VR_t vertical_res);
int BRtoascii(BR_t bit_rate);
int WDtoascii(WD_t page_width);
int LNtoascii(LN_t page_length);
int DFtoascii(DF_t compress_format);
int ECtoascii(EC_t ecm_mode);
int BFtoascii(BF_t bft_flag);
int STtoascii(ST_t scan_time);

VR_t asciitoVR(char vertical_res);
BR_t asciitoBR(char bit_rate);
WD_t asciitoWD(char page_width);
LN_t asciitoLN(char page_length);
DF_t asciitoDF(char compress_format);
EC_t asciitoEC(char ecm_mode);
BF_t asciitoBF(char bft_flag);
ST_t asciitoST(char scan_time);

char *DFtostring(DF_t compress_format);
char *VRtostring(VR_t vertical_resolution);

#if defined(__cplusplus)
	}
#endif

#endif
