#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include "Pplatform.h"

/*
 * This is a sample program demonstrating the viewfax API.
 * Documentation for this API is in READMES/README.viewfax.api.
 */

static char SCCS_ID[] = "@(#)|spawn_viewfax.c|9|01|2|0|0|07/26/99|14:49:08";

int pipe_io[3][2];				/* see diagram */

static void sigchldcatcher(int code);
void send_message_to_viewfax(char *buf);

/*
 *                 [0]                           [1]
 *    child reads:  |========= pipe_io[0] ========| <- parent writes
 *   pipe_io[0][0]                                     pipe_io[0][1]
 *
 *    parent reads: |========= pipe_io[1] ========| <- child writes
 *   pipe_io[1][0]                                     pipe_io[1][1]
 *
 * The parent process reads the output of the child process by reading
 * pipe_io[1][0] because the child is writing to pipe_io[1][1].
 * The child process gets its input from pipe_io[0][0] because the
 * parent writes to pipe_io[0][1].  Thus, one process is reading from
 * one end of the pipe while the other is writing at the other end.
 */
void
main(int argc, char **argv)
{
	char command;
	char buf[100];
	int pid, i;
	int ret;

	pipe(pipe_io[0]);			/* set up input pipe */
	pipe(pipe_io[1]);			/* set up output pipe */
	pipe(pipe_io[2]);			/* set up stderr pipe */
	switch (pid = fork())
	{
	case -1:
		close(pipe_io[0][0]);
		close(pipe_io[0][1]);
		close(pipe_io[1][0]);
		close(pipe_io[1][1]);
		close(pipe_io[2][0]);
		close(pipe_io[2][1]);
		perror("Viewfax fork failed");
		return;
	case 0:					/* child */
		/* redirect child's stdin (0) and stdout (1) */
		dup2(pipe_io[0][0], 0);

		/* THIS IS WHERE YOU WOULD CATCH STDOUT AND STDERR
		   This functionality is not implemented in this test
		   program.
		   dup2(pipe_io[1][1], 1);
		   dup2(pipe_io[2][1], 2);
		 */
#ifdef SYSV
		for (i = sysconf(_SC_OPEN_MAX); i > 2; i--)
#else
		for (i = getdtablesize(); i > 2; i--)
#endif /* SYSV */
			(void) close(i);
		for (i = 0; i < NSIG; i++)
			(void) signal(i, SIG_DFL);
		execvp("viewfax", argv);
		/* Here if exec failed */
		if (errno == ENOENT)
		{
			fprintf(stderr, "viewfax: command not found.\n");
			fflush(stderr);
		}
		else
		{
			perror("viewfax");
			fflush(stderr);
		}
		perror("execl");
		fflush(stderr);
		_exit(1);
		return;
	default:					/* parent */
		close(pipe_io[0][0]);	/* close unused portions of pipes */
		close(pipe_io[1][1]);
		close(pipe_io[2][1]);
	}

	signal(SIGCHLD, sigchldcatcher);

	/* IMPLEMENTATION NOTE:
	   Listen for data coming from viewfax (stdout or stderr) with
	   select() or XtAppAddInput() if you are calling viewfax from
	   an Xt (e.g. Motif) program.
	 */

	for (;;)
	{
		/* read input from stdin, forward it to viewfax */
		ret = read(0, buf, 100);
		buf[ret] = '\0';
		send_message_to_viewfax(buf);
	}
}

int Debug = 0;

void
send_message_to_viewfax(char *buf)
{
	if (Debug)
		printf("sending '%s' to viewfax\n", buf);
	write(pipe_io[0][1], buf, strlen(buf));
}

/* handle the death of the child by exiting ourselves.  */
static void
sigchldcatcher(int code)
{
	exit(0);
}
