
static char SCCS_ID[] = "@(#)|use_check_fax_status.c|9|01|2|0|0|07/26/99|14:32:26";

/*
 * The Bristol Group, 1996
 * Sample program which demonstrates use of the function check_fax_status().
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>

#ifdef WIN32
#include "winrpc.h"
#include "getopt.h"
#else
#include <pwd.h>
#include <unistd.h>
#endif

#include "queue_status.h"
#include "Pplatform.h"
#include "br_net.h"

#if defined(SOLARIS) || defined(AIX)
extern int gethostname(char *name, int namelen);
#endif

#if defined(SUNOS)
extern char *strerror(int);
#endif

extern int check_fax_status(char *server,
							int uid,
							char *username,
							char *passwd,
							char *cfilename,
							queue_status_t *fillme,
							int queue_number);

static void syntax(void);

static char **gargv;
static int gargc;

void 
main(int argc, char **argv)
{
	extern char *optarg;
	extern int optind;
	extern int opterr;

	int opt;
	int errval;
	int uid;
	int qnum;
	char *errmsg;
	char *cfile;
	char server[512];
	char passwd[512];
	char usrname[512];
	queue_status_t qbuf;

	if((errval = raise_rpc()))
	{
		fprintf(stderr,
			"Could not initialize the network layer, error=%d\n", errval);
		exit(1);
	}

	gargv = argv;
	gargc = argc;
	errmsg = (char *)0;
	*passwd = '\0';
	*server = '\0';
	*usrname = '\0';
	uid = -1;
	qnum = -1;
	optind = opterr = 1;
	while((opt = getopt(argc, argv, "hi:p:q:s:u:")) != -1)
	{
		switch(opt)
		{
			case 'h':
				syntax();
			case 'i':
				if(*optarg == '-')
					syntax();
				uid = atoi(optarg);
				break;
			case 'p':
				if(*optarg == '-')
					syntax();
				strcpy(passwd, optarg);
				break;
			case 'q':
				if(*optarg == '-')
					syntax();
				qnum = atoi(optarg);
				break;
			case 's':
				if(*optarg == '-')
					syntax();
				strcpy(server, optarg);
				break;
			case 'u':
				if(*optarg == '-')
					syntax();
				strcpy(usrname, optarg);
				break;
			default:
				fprintf(stderr, "Unsupported option '%c'!\n", opt);
				syntax();
		}
	}

	if(optind >= argc)					/* missing the job to check on? */
	{
		fputs("Control file not specified!\n", stderr);
		syntax();
	}
	cfile = argv[optind];

	/*
	 * Set values to default if not specified
	 * on the command line.
	 */
	if(qnum < 0)
		qnum = 1;
	if(uid < 0)
		uid = getuid();
	if(*server == '\0')
		gethostname(server, sizeof(server));
	if(*usrname == '\0')
	{
#ifdef WIN32
		*usrname = '\0';
		errval = sizeof(usrname);
		if(!GetUserName(usrname, &errval))
		{
			fprintf(stderr, 
				"Failed to get your username: %d\n", GetLastError());
			down_rpc();
			exit(1);
		}
#else
		struct passwd *pwent;

		pwent = getpwuid(uid);				/* returns static struct */
		if(pwent == (struct passwd *)0)
		{
			fprintf(stderr,
				"Failed to get your username: %s\n", strerror(errno));
			exit(1);
		}
		strcpy(usrname, pwent->pw_name);
#endif
	}

	/* This is where all the action is */
	errval = check_fax_status(server, uid, usrname, passwd,
								cfile, &qbuf, qnum);
	switch(errval)
	{
		case 0:		/* no problems */
			break;
		case 1:
			/* Could not connect to the host or the server */
			errmsg = "service not available";
			break;
		case 2:
			/* System call on server failed */
			errmsg = "system call error";
			break;
		case 4:
			/* Permission to access the queue is denied */
			errmsg = "queue access denied";
			break;
		case 5:
			/* Search for the specified control file failed */
			errmsg = "control file not found";
			break;
		case 6:
			/* You do not have permission to get status on specified file */
			errmsg = "not your job, permission denied";
			break;
		case 8:
			/* General failure on server (ie. memory allocation) */
			errmsg = "couldn't retrieve queue status";
			break;
		default:
			errmsg = "unhandled error return from server";
			break;
	}
	down_rpc();

	if(errmsg != (char *)0)
	{
		fprintf(stderr, "%s [%d].  Exiting...\n", errmsg, errval);
		exit(1);
	}

	/* Print out the interesting stuff */
	fprintf(stdout, "Fax file              : %s\n", qbuf.fax_name);
	fprintf(stdout, "Submitted             : %s",
			ctime((time_t *)&qbuf.time_submitted));
	fprintf(stdout, "Priority              : %d\n", qbuf.priority);
	if(qbuf.pos == 0)
		fputs("Queue Position        : \"Sending\"\n", stdout);
	else
		fprintf(stdout, "Queue Position        : %d\n", qbuf.pos);
	fprintf(stdout, "Sent                  : %d of %d\n", qbuf.nsent, qbuf.npages);
	fprintf(stdout, "Attempts              : %d of %d\n", qbuf.previous_attempts, qbuf.total_attempts);
	fprintf(stdout, "Time to send          : %s",
		ctime((time_t *)&qbuf.time_to_send));
	fprintf(stdout, "First attempt         : %s",
		ctime((time_t *)&qbuf.time_first_attempt));
	fprintf(stdout, "Last attempt          : %s",
		ctime((time_t *)&qbuf.time_last_attempt));
	if(qbuf.previous_attempts > 0 && *qbuf.fail_excuse)
		fprintf(stdout, "Failure Message       : %s\n", qbuf.fail_excuse);
	fprintf(stdout, "Tracking ID           : %s\n", qbuf.tracking_id);
		

	exit(0);
}

static void
syntax(void)
{
	fprintf(stderr,
		"  %s [options] <C_FILENAME>\n"
		"        [-help]                     Print this message.\n"
		"        [-i <user ID>]              User ID from host that submitted fax.\n"
		"        [-p <IsoFax password>]      Users's password in server's user list.\n"
		"        [-q <IsoFax queue number>]  Server's queue to query.\n"
		"        [-s <IsoFax server>]        Hostname for the server.\n"
		"        [-u <user name>]            User name from host that submitted fax.\n"
		, gargv[0]);
	exit(1);
}
