#ifdef RELEASE_8

#define IISDN_6_8 1

/* NOTE:  Should ALWAYS only have 11 chars in the following defines. */
/*                    "12345678901"                                  */
#define IISDN_VERSION           "r06.08.06a "
/* NOTE:  Starting with Release 7.x.x of IISDN, the IISDN 5.4 version */
/*        is NOT to be updated unless there are specific changes in   */
/*        the ISA/VME code stored in the "iisdn54" directory. The     */
/*        PRI.0 and PRI_LITE.0 files will continue to be built but    */
/*        will not be included in the driver release kits.            */
#define IISDN_VERSION_54        "r05.08.00  "

#else

#define IISDN_VERSION           "r07.00.62  "
/* NOTE:  Starting with Release 7.x.x of IISDN, the IISDN 5.4 version */
/*        is NOT to be updated unless there are specific changes in   */
/*        the ISA/VME code stored in the "iisdn54" directory. The     */
/*        PRI.0 and PRI_LITE.0 files will continue to be built but    */
/*        will not be included in the driver release kits.            */
#define IISDN_VERSION_54        "r05.07.70  "

#endif
/*******************************************************************************
 *      http://www.brooktrout.com
 *      iisdn.h - Interface definition for Netaccess IISDN T-1/E-1/BRI controllers
 *
 *      Description:
 *              This header file defines the interface between the host processor
 *      and the Netaccess family of T1/E1/BRI ISDN boards.  The ISDN controllers 
 *      implement layers 1 through 3 of the ISDN stack.  Layer 1 can be run
 *      in either HDLC packetized mode or 'Raw' mode. Raw mode disables any
 *      HDLC packetization and delivers the raw data stream to the host.  Layer 2
 *      can be LAP-D/F (ISDN), LAP-B (X.25), MTP2 (SS7) or it can be disabled.  
 *  Both the user side (DTE) and network side (DCE) are implemented.  
 *  Symmetric LAP-D, which is used by V.120, is also implemented.  Layer 3 can 
 *  be Q.931 (ISDN call control), X.25 Packet Layer, or disabled.  In addition,
 *  the interface supports AT&T Accunet certified alarm processing, extended
 *  superframe data link with physical layer performance monitoring in 
 *  ANSI T1.403 1995 mode, E&M Dialpulse Robbed Bit Signalling (D4 channel bank
 *      signalling), and the BONDING protocol for inverse video multiplexing.
 *
 *      The interface is split between control message functions and data passing
 *      functions.  All control messages are passed via two circular control
 *      message buffer queues on the Netaccess controller.  Data is passed over a set
 *      of up to 256 separate data channels.  Each data channel has its own set
 *      of user defined buffer pools.
 *
 *      Naming Conventions:
 *      L3L4*   Names which pertain to the interface from Layer 3 (the
 *                      Netaccess Controller board) to Layer 4 (the host)
 *      L4L3*   Names which pertain to the interface from Layer 4 (the host) to
 *                      Layer 3 (the Netaccess Controller board)
 *      IISDN*  General prefix meaning it came from this file.
 *
 *      Portability Considerations:
 *      - This header file assumes that byte ordering follows the Motorola
 *        convention (big endian).
 *  - For little-endian machines such as Intel processors, Instant
 *        ISDN performs control message byte swapping onboard when so directed.
 *  - It also assumes that a 'short' occupies 16 bits and a 'long' occupies
 *    32 bits.
 *  - There are defines supplied to allow for alternative bus architectures
 *    such as 64 bit buses.  The defines may be overridden to allow for other
 *    type definitions.
 *  - All structures are explicitly longword aligned (perhaps using pad bytes).
 *  - This file will compile under both ANSI 'C' and most older 'C' compilers.
 *
 *      (C) - Copyright Brooktrout, Inc., 1997, 1998, 1999, 2001, 2002, 2003.
 ******************************************************************************/

#ifndef IISDN_H
#define IISDN_H 1


/* Hardware/Software version string stored at offset x10 in shared RAM */
#define IISDN_VERSION_STRING_OFFSET 0x0010      /* Hardware/Software version */
#define IISDN_VERSION_STRING_LEN        0x20    /* 32 characters, null terminated */

/* DED -- Firmware version of IISDN.  This is used by BSMI API to make sure the */
/*                same version of iisdn.h was used to build the firmware, bsmi.lib and App */
/* NOTE:  Currently the file vtty_cli.c has its own copies of the version numbers */
#define IISDN_VER_MAJOR 7
#define IISDN_VER_MINOR 0
#define IISDN_VER_BUILD 80
/* Apollo 4710 - Created IISDN Interface Version. BSMI API will check
   only IISDN Interface Version for compatibility with firmware
   
   SDK 3.3 again changed the way this works since ECC checks version number
   SDK 4.1 adds interface changes necessary to accomodate 8-port boards.
     no interface version requested - stack reports 6.7.54 for SDK 3.0.3 and earlier
     interface version 1  requested  - stack reports 7.0.60 for SDK 3.1
     interface version 2  requested  - stack reports 7.0.62 for SDK 3.2
     interface version 3  requested  - stack reports 7.0.64 for SDK 3.3 and 4.0
     interface version 10 requested  - stack reports 7.0.80 for SDK 4.1
*/
#define IISDN_VER_INTERFACE   10

/* Signed type definitions. */
#ifndef IISDNs8bit
#define IISDNs8bit      char
#endif
#ifndef IISDNs16bit
#define IISDNs16bit     short
#endif
#ifndef IISDNs32bit

#ifdef _LP64    /* 64-bit model where a long is 64 bits and int is 32 bits */
#define IISDNs32bit int
#else
#define IISDNs32bit     long
#endif
#endif
/* Unsigned type definitions. */
#ifndef IISDNu8bit
#define IISDNu8bit      unsigned char
#endif
#ifndef IISDNu16bit
#define IISDNu16bit     unsigned short
#endif
#ifndef IISDNu32bit
#ifdef _LP64    /* 64-bit model where a long is 64 bits and int is 32 bits */
#define IISDNu32bit unsigned int
#else
#define IISDNu32bit     unsigned long
#endif
#endif
/* Pointer type definitions. */
#ifndef IISDNp8bit
#define IISDNp8bit      unsigned char *
#endif
#ifndef IISDNp16bit
#define IISDNp16bit     unsigned short *
#endif
#ifndef IISDNp32bit
#ifdef _LP64    /* 64-bit model where a long is 64 bits and int is 32 bits */
#define IISDNp32bit unsigned int  *
#else
#define IISDNp32bit     unsigned long *
#endif
#endif

/* 
 * Diagnostic order of operations:
 *   1) reset the Netaccess card and its processor
 *   2) run a host based memory test against all the card's shared memory
 *   3) 0 out the interrupt_reason byte, CMD-REG, STATUS-REG, GLOBAL-CTL-REG
 *   4) download Netaccess diagnostic image
 *   5) unhalt processor and wait 5 seconds for interrupt and interrupt-reason
 *   6) if the interrupt reason is not L3L4irsnREADY_TO_RUN then the card's DOA
 *   7) 0 out the interrupt-reason byte
 *   8) fill in valid host addresses for IISDN_HOST_RAM_TEST_START/END or 0/0
 *   9) set IISDN_DIAG_CMD_REG = IISDNdcmdPOST (big Endian!!)
 *  10) test output should be visible on TTY or VTTY interface
 *  11) wait up to 30 seconds for interrupt/interrupt-reason
 *  12) if interrupt_reason is not L3L4irsnPOWERUP_BOOT then the card's DOA
 *  13) IISDN_DIAG_STATUS_REG may contain a failing test number
 */
#define IISDN_DIAG_CMD_REG_OFFSET       0x0030  /* 16 bit register. IISDNdcmdXXX */
#define IISDN_BOOT_CMD_REG_OFFSET       0x0030  /* IISDN legacy cards */
#define IISDN_DIAG_STATUS_REG_OFFSET    0x0032  /* 16 bit test results (0 == pass) */

/* Diagnostic stuff.  use dcmdNULL before diag code load, others after running */
#define IISDNdcmdNULL                   0x0000  /* write this value before loading diags */
#define IISDNbcmdPOWERUP_BOOT           0x0002  /* IISDN legacy card code load after pup */
#define IISDNdcmdPOST                   0x0008  /* execute power on self tests */
#define IISDNdcmdBURNIN                 0x0009  /* execute burnin tests */
#define IISDNdcmdEXTENDED               0x000a  /* execute extended diagnostics */
#define IISDNdcmdDEBUG                  0x000c  /* start Diagnostic debugger */
#define IISDNdcmdOneTest                0x0010  /* execute a single test */
#define IISDNdcmdHALT_DIAGS             0x1000  /* command to halt PRI-ISA diagnostics (applied at least 50ms after uP reset) */
#define IISDN_HOST_RAM_TEST_START       0xFEF8  /* init before issuing dcmd */
#define IISDN_HOST_RAM_TEST_END         0xFEFC  /* START = END = 0 to bypass test */

#define IISDN_GLOBAL_CTL_REG_OFFSET     0x0034  /* 8 bit reg, IISDNgcXXXX bitmask */
#define IISDNgcBYTESWAP_CTLMSGS           0x01  /* this register is NEVER written */
#define IISDNgcVTTY_ENABLE                0x02  /* by the card, not even at boot! */
#define IISDNgcMISC_ISA_INTERRUPTS        0x04
#define IISDNgcOFFBOARD_L4_BUFFERS        0x08
#define IISDNgcI2O_SMI                    0x10  /* reserved for future use */
#define IISDNgcDMSX                       0x20  /* reserved for future use */
#define IISDNgcDATA_INTERFACE_CONFIG      0x40  /* Host configured data interface */
#define IISDNgcCONFIG_BUFFER_POOL         0x80  /* Requests reconfiguration of onboard data buffer pool (write desired amount to IISDN_BUFF_POOL_ADDR_OFFSET). */

/* 
 * The virtual tty port is a means of getting serial debug access over
 * the bus.  Every character sent out to the debug console will also
 * be sent to the IISDN_VTTY.l3l4_buffer if the IISDNgcVTTY_ENABLE flag is set.
 * Use of the VTTY interface does NOT preclude standard debug port operation.
 */
#define IISDN_VTTY_OFFSET               0x0080
typedef struct {
    IISDNu8bit l4l3_count;      /* written by host, cleared by controller */
    IISDNs8bit l4l3_buffer[7];  /* filled by host prior to writing l4l3-count */
    IISDNu8bit l3l4_count;      /* written by controller, cleared by host */
    IISDNs8bit l3l4_buffer[7];  /* read by host prior to clearing l3l4-count */
} IISDN_VTTY;

/*
 * Data Passing Queues for HDLC, transparent, LAP-B, LAP-D, and LAP-F frames.
 *
 *      Buffered data is handled via a set of data passing queues
 * and buffers which reside in either onboard shared memory or user provided 
 * space.  The buffers are managed by the host and may not exceed 16k bytes.
#ifdef RELEASE_8
 * The root of the data structure is an array of IISDN_DCHAN_DESCR data
 * channel descriptor structures.  The address of this structure starts is
 * programmed by the host in the DATA_INTERFACE_CONFIGURATION struture which
 * is located at SMI offset IISDN_DATA_INTERFACE_CONFIG_OFFSET.
#else
 * The root of the data structure is an array of 256 IISDN_DCHAN_DESCR data
 * channel descriptor structures.  This structure starts in the SMI at address
 * DATA_QUEUE_OFFSET.
#endif
 * Each one of these Data Channel Descriptors points to
 * two arrays of IISDN_BUF_DESCR buffer descriptors; one for data buffers from
 * L3->L4 and one for data buffers from L4->L3.  The algorithm for managing
 * the circular buffers is the same GIVE and TAKE scheme used for the control
 * buffers.  The host allocates all space for arrays of IISDN_BUF_DESCR and
 * must initialize all these structures prior to enabling the data channel or
 * with L4L3mENABLE-PROTOCOL.
 *
 *      To avoid the overhead of scanning a large number of data queues, there
 * is a separate common event queue for transmitting and receiving on
 * the data interface.  Each time a packet is transmitted, the data stream
 * id of the packet is placed on the event queue to ensure that the packet is
 * processed.  The rule of thumb is to place an event in the event queue
 * whenever advancing either a GIVE or a TAKE index.  Generating an event when
 * bumping TAKE allows flow control to work properly.  There is not always
 * a one-to-one correlation between events and give/take movement, however.
 * When buffer concatenation is used only one event is generated per packet and
 * a packet may span several buffers.
 */
#define IISDN_NUM_DS1_INTERFACES    20
#define IISDN_MAX_DATA_BUF          (16*1024)   /* 16 K data buffers */
#define IISDN_MAX_VC_PER_CHAN       8                   /* 8 virtual ckts per physical */

/* LLI is a two byte virtual circuit identifier used in LAP-B/D/F/Q.921.
 * InstantISDN uses Logical Link Identifier (LLI); Data Link Connection
 * Identifier (DLCI); and the pair Service Access Point Identifier 
 * (SAPI)/Terminal Endpoint Identifier (TEI) interchangably.  InstantISDN 
 * represents these values the way they appear from the network.  
 * The high 6 bits of the most significant byte are the SAPI.  
 * The high 7 bits of the least significant byte are the TEI.
 * The other bits are set to zero.
 * For V.120, the user might use the following macro to specify LLI in
 * an ENABLE_D_CHANNEL message:
 *      #define TO_IISDN_LLI(LLI) ((((LLI) & 0x1F8) << 3) | (((LLI) & 0x7F) << 1))
 *
 * For Q.921, the user might use the following macro to specify LLI in
 * an ENABLE_D_CHANNEL message:
 *      #define TO_IISDN_LLI(SAPI,TEI) (((SAPI) << 10) | ((TEI) << 1))
 *
 * For Frame Relay, the composition of DLCI or LLI is:
 *      #define TO_IISDN_LLI(DLCI) (((DLCI & 0x000F) << 4) | ((DLCI & 0x03F0) << 6))
 *
 * For X.25 Level 2 (LAP-B), the value IISDN_LLI_LAPB must be used.
 *
 *      X.25 Packet Layer passes the 3 byte Packet Layer header in the buffer
 *      pointed to by 'buffer_pointer'.  This is done to provide users explicit
 *      control and notification of the state of the 'M' bit (More Data bit) and 
 *      'Q' bit (Qualifier bit).  
 *       -  The 'Q' bit is located at bit position 0x80 in the 0th byte of the 3
 *              byte X.25 packet layer header.  
 *       -  The 'M' bit is located in buffer_pointer[2] at bit position 0x10.
 *      The data payload starts at buffer_pointer[3] and can be up to 256 bytes.
 *      (The total length of the buffer would be 259 bytes).
 *      X.25 packet layer logical channel group and logical channel number is
 *      passed in the call_ref field of control messages.  The most significant
 *      byte contains the logical channel group.  The least significant byte
 *      contains the logical channel number.  A value of zero is not allowed.
 *
 *      Note: The X.25 packet layer will pass a 4 byte header in buffers when
 *      virtual circuits are configure for modulo-128 operation.
 *
 */
#define IISDN_LLI_CNTL              0x0000  /* The LLI for call control (Q.931) */
#define IISDN_LLI_LAPB              0x4000  /* 'Fake' LLI value for X.25 Level 2 */
#define IISDN_LLI_256               0x0800  /* V.120 Default LLI 256 */

#define IISDN_X25_HEADER_SZ              3  /* 3 byte X.25 packet layer header */
#define IISDN_X25_QBIT_MASK           0x80  /* X.25 Packet Layer Q-Bit position */
#define IISDN_X25_QBIT_OFFSET            0  /* Q-bit is in buffer_pointer[0] */
#define IISDN_X25_MBIT_MASK           0x10  /* X.25 Packet Layer M-Bit position */
#define IISDN_X25_MBIT_OFFSET            2  /* M-bit is in buffer_pointer[2] */

/* IISDN_BUF_DESCR structure declaration for an array of pointers to 
 * transmit and receive buffers.  This data structure resides in user 
 * provided memory.  It can be put onboard if a IISDN_POOL_ENTRY of adequate
 * size is available.
 */
#define IISDN_CONCAT_BIT        0x80000000  /* OR into IISDN_BUF_DESCR.length */
/* Concat may be set in l4l3 direction to tell IISDN to chain multiple
 * buffers into a single xmit frame.  If level2.data_interface.fillANDspill
 * is true then IISDN will use the CONCAT bit to indicate that one or more
 * l3l4 buffers have been chained together to deliver a single frame to L4.
 * The length field ALWAYS represents the number of bytes in the buffer,
 * never grand totalling.  The concat-bit is NOT set in the final buffer
 * of a chained frame.
 */
typedef struct
        {
    IISDNp8bit  buffer_pointer; /* Buffer must be word aligned */
    IISDNu32bit length;         /* Not to exceed 16k bytes */
                                /* Minimum of 1 byte with 68mh360 */
                                /* Minimum of 2 byte with Lucent7115 */
        } IISDN_BUF_DESCR;

/* ROOT DATA TRANSFER STRUCTURE
 *      There is an array IISDN_MAX_DATA_BUF long of these structures located at
 *      IISDN_DATA_QUEUE_OFFSET in shared RAM on the IISDN Controller
 */
typedef struct
        {
    IISDNp32bit L3L4_queue_ptr; /* Point to start of array for L3->L4 (IISDN_BUF_DESCR) */
    IISDNp32bit L4L3_queue_ptr; /* Point to start of array for L4->L3 (IISDN_BUF_DESCR) */
    IISDNu16bit num_L3L4_bufs;  /* 3 <= num_bufs <= 256 */
    IISDNu16bit num_L4L3_bufs;  /* 3 <= num_bufs <= 256 */
    IISDNu8bit  L3L4_take;              /* Array index into L3L4_queue_ptr */
    IISDNu8bit  L3L4_give;
    IISDNu8bit  L4L3_take;              /* Array index into L4L3_queue_ptr */
    IISDNu8bit  L4L3_give;
        
    IISDNu8bit  L4L3_ddcmd;             /* not implemented */
    IISDNu8bit  L3L4_ddack;             /* not implemented */
#define IISDNddTX_HALT  0x01
#define IISDNddRX_HALT  0x02
        /* next 3 fields for INTERNAL USE */
        IISDNu16bit     hdlc_id;                /* data pump channel [0..63] */
        IISDNu16bit     dlci;                   /* DLCI or LLI in internal format */
        IISDNp8bit      state_mach_ptr; /* Point to internal data structure */
        IISDNu8bit      pad;
        /* EXTERNAL AGAIN: tx packet complete interrupt control */
        IISDNu16bit     low_water_mark; /* 0 => generate interrupt on all tx packets
                                                                   Nonzero => generate interrupt when
                                                                   L4->L3 queue has less than low_water_mark
                                                                   packets left to transmit */
        IISDNu8bit      irq_level;              /* multi cpu interrupts, 1 <= level <= 7 */
        IISDNu8bit      int_vector;     /* multi cpu interrupts, 0 <= vect <= 255 */
        } IISDN_DCHAN_DESCR;

#ifdef RELEASE_8

#define IISDN_DATA_INTERFACE_CONFIG_OFFSET      0x0040
                                /* Location of data interface configuration data */

/* Suggested defaults for data interface configuration */
#define IISDN_DATA_QUEUE_OFFSET                 (0x09000)       /* Add in SMI BASE address */
#define IISDN_NUM_DATA_QUEUES                           256             /* Max number of data streams */
#define IISDN_NUM_EVENT_Q_ENTRIES                       1024    /* 2 K tx and 2 K rx events */
#define IISDN_NUM_L3L4_EVENT_Q_ENTRIES          1024
#define IISDN_NUM_L4L3_EVENT_Q_ENTRIES          1024
#define IISDN_EVENT_QUEUE_OFFSET                (0x0E000)       /* Add in SMI BASE address */
#define IISDN_L3L4_EVENT_QUEUE_OFFSET   (0x0E000)
#define IISDN_L4L3_EVENT_QUEUE_OFFSET   (0x0E800)

#else

#define IISDN_DATA_QUEUE_OFFSET                 (0x09000)       /* Add in SMI BASE address */
#define IISDN_NUM_DATA_QUEUES                           256             /* Max number of data streams */
#define IISDN_NUM_EVENT_Q_ENTRIES                       1024    /* 2 K tx and 2 K rx events */
#define IISDN_NUM_L3L4_EVENT_Q_ENTRIES          1024
#define IISDN_NUM_L4L3_EVENT_Q_ENTRIES          1024
#define IISDN_EVENT_QUEUE_OFFSET                (0x0B000)       /* Add in SMI BASE address */
#define IISDN_L3L4_EVENT_QUEUE_OFFSET   (0x0B000)
#define IISDN_L4L3_EVENT_QUEUE_OFFSET   (0x0B400)

#endif

typedef struct
        {
        IISDNu16bit     L3L4_take;
        IISDNu16bit     L3L4_give;
        IISDNu16bit     L4L3_take;
        IISDNu16bit     L4L3_give;
        } IISDN_GIVE_TAKE_STRUCT;

/* Location of IISDN_GIVE_TAKE_STRUCT for event queues in shared RAM */
#define IISDN_EVENT_Q_GIVE_TAKE_OFFSET          (0x000008)

/* 16 bit array indexes of position in the above queues as #defines */
#define IISDN_EVENT_Q_L3L4_TAKE_OFFSET          (0x000008)
#define IISDN_EVENT_Q_L3L4_GIVE_OFFSET          (0x00000A)
#define IISDN_EVENT_Q_L4L3_TAKE_OFFSET          (0x00000C)
#define IISDN_EVENT_Q_L4L3_GIVE_OFFSET          (0x00000E)

typedef struct
{
        IISDNp32bit                     dchan_descr_addr; /* Physical (board based) address of DCHAN descriptors */
        IISDNu16bit             num_dchan_descr;        /* Number of data channels */
//      IISDNu16bit                     pad;
        IISDNp16bit                     dchan_event_queue_addr; /* Address of data channel event queues -
                                                                        L4L3 queue immediately follows L3L4 queue */
        IISDNu16bit             num_l3l4_dchan_events;  /* Board to host event queue size */
        IISDNu16bit             num_l4l3_dchan_events;  /* Host to board event queue size */

} IISDN_DATA_INTERFACE_CONFIGURATION;


/* Structure declaration for buffer pool descriptors.  These areas reside on the
 * Netaccess controller card, but are allocated and managed by the host.
 * The pool-address and length are in big endian onboard relative format and 
 * need to be swapped if on a little endian host.
 */
/* location of IISDN_POOL_DESCR in shared RAM */
#ifdef RELEASE_8

#define IISDN_BUFF_POOL_ADDR_OFFSET                     (0x000050)

#else

#define IISDN_BUFF_POOL_ADDR_OFFSET                     (0x000040)

#endif

typedef struct
        {
        IISDNp32bit     pool_address;   /* Beginning address of pool area */
        IISDNu32bit     pool_length;    /* length of pool area */
        } IISDN_POOL_ENTRY;
        
typedef struct
        {
        IISDNs8bit                      num_pools;              /* number of pool segments available */
        IISDNs8bit                      pad1;                   /* a pad byte */
        IISDNs8bit                      pad2;                   /* a pad byte */
        IISDNs8bit                      pad3;                   /* a pad byte */
        IISDN_POOL_ENTRY        pools[4];               /* pool descriptors */
        } IISDN_POOL_DESCR;

typedef struct
        {
#ifdef RELEASE_8
        IISDNu8bit      enable;                                 /* 1 to enable data passing interface */
        IISDNu8bit      fillANDspill;                   /* permits iisdn to chain L3L4 data across multiple buffers if needed indicated with IISDN_CONCAT_BIT */
        IISDNu16bit     data_channel;                   /* index to data_channel_dscr array */
#else
        IISDNu8bit      enable;                                 /* 1 to enable data passing interface */
        IISDNu8bit      data_channel;                   /* index to data_channel_dscr array */
        IISDNu8bit      fillANDspill;                   /* permits iisdn to chain L3L4 data across multiple buffers if needed indicated with IISDN_CONCAT_BIT */
        IISDNu8bit      allow_buffer_preload;   /* A non-zero allows transmit buffers to be preloaded with data. */
#endif
        } IISDN_DATA_INTERFACE;

/*
 * Control interface for board configuration and call control
 *
 *              The host communicates to the IISDN board via two circular buffers.
 * The host sends messages to the board via the L4L3_QUEUE.  The 
 * host receives messages from the board via the L3L4_QUEUE.  Each
 * buffer in the queue is 512 bytes long and there are 32 buffers in each
 * queue.  There are two 16 bit array indexes called the GIVE and TAKE index
 * which are used to insert and remove data from each queue.  
 *              To send a message, the transmitter fills out the buffer referenced
 * by the GIVE index and increments the GIVE index to reference
 * the next buffer (handling the wrap-around condition).  The receiver
 * determines that there is data in the queue by testing GIVE != TAKE.  If
 * the indexes are not equal, there are messages to take from the queue. 
 * The receiver should process the data in the buffer referenced by TAKE (or
 * copy it out of the buffer), and increment the TAKE index (handling the
 * wrap-around condition).  The receiver should continue processing buffers
 * until GIVE == TAKE.
 *
 * The IISDN board initializes the GIVE and TAKE indexes to 0 at boot time.
 */

#define L3L4_NUM_BUFS           32              /* Number of fixed size buffers L3->L4 */
#define L4L3_NUM_BUFS           32              /* Number of fixed size buffers L4->L3 */
#define L3L4_BUF_SIZE           512             /* Size of fixed size buffers L3->L4 */
#define L4L3_BUF_SIZE           512             /* Size of fixed size buffers L4->L3 */

#define L4L3_QUEUE_SIZE         (L4L3_NUM_BUFS * L4L3_BUF_SIZE)
#define L3L4_QUEUE_SIZE         (L3L4_NUM_BUFS * L3L4_BUF_SIZE)

/* These are the offsets for a generalized interface with several boards */
#define L4L3_QUEUE_OFFSET               (0x001000)
#define L3L4_QUEUE_OFFSET               (0x005000)

/* Location of IISDN_GIVE_TAKE_STRUCT for control queues in shared RAM */
#define L3L4_GIVE_TAKE_OFFSET   (0x000000)

/* 16 bit array indexes of position in the above queues as #defines */
#define L3L4_TAKE_OFFSET                (0x000000)
#define L3L4_GIVE_OFFSET                (0x000002)
#define L4L3_TAKE_OFFSET                (0x000004)
#define L4L3_GIVE_OFFSET                (0x000006)

/*
 * User -> IISDN Controller Control Messages (contained in L4L3_take->msgtype)
 */

#define FIRST_L4L3m_VALUE                   0x81    /* first value used for L4L3m messages */

#define L4L3mNULL                           0x00    /* no message */
/* Q.931 messages */
#define L4L3mCALL_REQUEST                   0x81    /* L4 initiates an outbound call */
#define L4L3mPROGRESS_REQUEST               0x82    /* L4 is processing call */
#define L4L3mALERTING_REQUEST               0x83    /* Local term is ringing */
#define L4L3mCONNECT_REQUEST                0x84    /* Local term has answered */
#define L4L3mCLEAR_REQUEST                  0x85    /* L4 wants to idle this call */
#define L4L3mFACILITY_REQUEST               0x86    /* L4 resp to alerting for RLT svc */
#define L4L3mUSER_INFO                      0x87    /* User->User Information */
#define L4L3mUNIVERSAL                      0x88    /* Generic Q.931 message */
#define L4L3mCALL_PROCEEDING_REQUEST        0x89    /* L4 responds to incoming call */
#define L4L3mSETUP_ACK_REQUEST              0x8A    /* L4 responds to incoming call */
#define L4L3mINFO_REQUEST                   0x8B    /* L4 sends INFO */
#define L4L3mRESEND_CALL_REQUEST            0x8C    /* L4 re-initiates an outbound call */
#define L4L3mCLEAR_CALL_REQUEST             0x8D    /* L4 clears an outbound call that never gets a response*/
#define L4L3mSPID                           0x8E    /* L4 accepts or rejects received SPID */

#define L4L3mCONTINUE                       0x8F    /* Continuation of previous L4L3m (for additional information elements) */

/* Supplemental ISDN Messages; Not supported by AT&T/Northern Telecom */
#define L4L3mSUSPEND_REQUEST                0x90    /* Suspend-call request (U->N) */
#define L4L3mSUSPEND_ACK                    0x91    /* Suspend Ack (Net->User) */
#define L4L3mSUSPEND_REJECT                 0x92    /* Suspend Reject (Net->User) */
#define L4L3mRESUME_REQUEST                 0x93    /* Resume (User->Network) */
#define L4L3mRESUME_ACK                     0x94    /* Resume Ack (Net->User) */
#define L4L3mRESUME_REJECT                  0x95    /* Resume Reject (Net->User) */

/* X.25 Extensions to Q.931 Connection Control messsages */
#define L4L3mX25_INTERRUPT                  0x96    /* X25 Packet Layer Interrupt Msg */
#define L4L3mX25_INT_CONF                   0x97    /* X25 Packet Layer Interrupt Confirm */
#define L4L3mX25_RESTART                    0x99    /* Restart with cause/diag fields */
#define L4L3mX25_RESET                      0x9A    /* Reset with cause/diag fields */
#define L4L3mX25_CONNECT                    0x9B    /* X.25 Call Accept */

/* GR_303 Messages */
#define L4L3mCONNECT_ACK                    0x98    /* RDT only: Acknowledge connect */

/* Management functions */
#define L4L3mDISABLE_PROTOCOL               0xA1    /* Disable a Data channel */
#define L4L3mENABLE_B_CHANNEL               0xA2    /* Q.931 Enable a B channel */
#define L4L3mDISABLE_B_CHANNEL              0xA3    /* Q.931 Disable a B channel */
#define L4L3mRESTART                        0xA4    /* Clear calls for a B channel */
#define L4L3mREQ_PROTOCOL_STATUS            0xA5    /* Request PROTOCOL_STATUS message */
#define L4L3mREQ_LINE_STATUS                0xA6    /* Request line (T1/E1) status msg */
#define L4L3mSET_HARDWARE                   0xA7    /* Set T1 and board level values */
#define L4L3mREQ_HW_STATUS                  0xA8    /* Get T1 and board level values */
#define L4L3mSET_TSI                        0xA9    /* PRI-48 only! Set time slot int. */
#define L4L3mPM_REQUEST                     0xAA    /* Performance Monitoring Request */
#define L4L3mREQ_L2_STATS                   0xAB    /* Request L2 Statistics (and zero) */
#define L4L3mREQ_LMI_STATUS                 0xAC    /* Request Frame Relay LMI Table */
#define L4L3mSET_LMI                        0xAD    /* Configure Frame Relay LMI */
#define L4L3mENABLE_BONDING                 0xAE    /* Enable bonding channels */
#define L4L3mADD_BONDING                    0xAF    /* Add bonding channels */
#define L4L3mCOMPARE                        0xB0    /* Compare blocks of memory */
#define L4L3mCLEAR_BONDING                  0xB1    /* Clear bonding channels */
#define L4L3mSET_MVIP_CAS                   0xB2    /* ABCD sig bits to MVIP bus */
#define L4L3mSET_EBUS_CAS                   0xB2    /* ABCD sig bits to expansion bus */
#define L4L3mFEATURE_REQUEST                0xB3    /* Billing changes and ANI on demand */
#define L4L3mENABLE_PROTOCOL                0xB6    /* New msg to replace enable d chan. */
#define L4L3mSUBMIT_DIRLIST                 0xB7    /* bonding msg to pass directory list */
#define L4L3mSEND_ALARM                     0xB8    /* Send an alarm out the line */
#define L4L3mSET_IDLE_PATTERN               0xB9    /* Set channel idle pattern */
#define L4L3mCLEAR_IDLE_PATTERN             0xBA    /* Disable channel idle pattern */
#define L4L3mREQ_ABCD_DATA                  0xBC    /* Get abcd signaling data - ISA only */
#define L4L3mTX_WINK                        0xBD    /* Transmit a wink (sw56) */
#define L4L3mEND_DIAL                       0xBE    /* MVIP component done dialing (sw56) */
#define L4L3mTX_HOOKFLASH                   0xC0    /* transmit hookflash (sw56) */
#define L4L3mREQ_BOARD_ID                   0xC1    /* request board identification */
#define L4L3mCAS_TO_MEMORY                  0xC2    /* Turn on/off cas to memory. 
                                                     * Use IISDNcasLINE in lli field */
#define L4L3mSET_D_CHANNEL_LAPDID           0xC3    /* ISA8D only */
#define L4L3mSET_MODEM_INIT_STRING          0xC5    /* for non-standard modem init */
#define L4L3mENABLE_CAS                     0xC6    /* initiate RBS processing */
#define L4L3mDISABLE_CAS                    0xC7    /* terminate RBS processing */
#define L4L3mSET_Q933A_PVC                  0xC8    /* notify board of Q.933 Annex A PVC change */
#define L4L3mCOLLECT_DIGITS                 0xC9    /* Collect dialed digits (sw56) */
#define L4L3mDIAL                           0xCA    /* Dial pulse digits (sw56) */
#define L4L3mDOWNLOAD                       0xCB    /* send additional module to firmware */
#define L4L3mRELAY_ADD_RULE                 0xCC    /* define additional pkt relay rule */
#define L4L3mRELAY_DEL_RULE                 0xCD    /* remove existing pkt relay rule */
#define L4L3mRELAY_CLEAR_RULES              0xCE    /* remove all pkt relay rules */
#define L4L3mREQ_RELAY_RULES                0xCF    /* generate L3L4mRELAY_RULES msg */
#define L4L3mREQ_RELAY_STATS                0xD0    /* generate L3L4mRELAY_STATS msg */
#define L4L3mUPLOAD                         0xD1    /* receive data from firmware */
#define L4L3mSET_HOST_WATCHDOG              0xD2    /* arm L3L4 queue timer, enable HA support */
#define L4L3mSET_HOST_DEAD                  0xD3    /* force HA failure declaration */
#define L4L3mREQ_TSI_STATUS                 0xD4    /* request L3L4mTSI_STATUS (CPCI only) */
#define L4L3mTONE_CTRL                      0xD5    /* start/stop/detect a tone, dsp cards only */
#define L4L3mCAS_CHAN_BLOCK                 0xD6    /* block a CAS channel */
#define L4L3mCAS_CHAN_UNBLOCK               0xD7    /* unblock a CAS channel */
#define L4L3mREQ_GEOGRAPHIC_ADDRESS         0xD8    /* request L3L4mGEOGRAPHIC_ADDRESS (CPCI only) */

#define L4L3mENABLE_MF_BACKWARD_DETECTOR    0xD9    /* debugging */
#define L4L3mDISABLE_MF_BACKWARD_DETECTOR   0xDA    /* debugging */
#define L4L3mENABLE_MF_FORWARD_DETECTOR     0xDB    /* debugging */
#define L4L3mDISABLE_MF_FORWARD_DETECTOR    0xDC    /* debugging */

#define L4L3mPLAY_MF_BACKWARD               0xDD    /* debugging */
#define L4L3mSTOPPLAY_MF_BACKWARD           0xDE    /* debugging */
#define L4L3mPLAY_MF_FORWARD                0xDF    /* debugging */
#define L4L3mSTOPPLAY_MF_FORWARD            0xE0    /* debugging */

#define L4L3mCONTROL_DATA                   0xE1    /* DED Direct to LAP message */
#define L4L3mSET_CAS_SIGNALING_BITS         0xE2    /* DED Direct to E1/T1 Slot 0 Spare Bits COntrol */
#define L4L3mSPARE_BITS_CONTROL             0xE2    /* DED Direct to E1/T1 Slot 0 Spare Bits COntrol */
                                                    /* defined for backward compatibility */
#define L4L3mR2HISTORY                      0xE3    /* debugging */

#define L4L3mCPU_UTILIZATION                0xE4    /* start, stop, or report CPU Utilization data
                                                       collection (PCI and CPCI only) */
#define L4L3mREQ_VERSION                    0xE5    /* request IISDN version information */
#define L4L3mSET_SOFTWARE                   0xE6    /* set software data */
#define L4L3mREQ_SW_STATUS                  0xE7    /* generate L3L4mSOFTWARE_STATUS msg */
#define L4L3mEXPLICIT_CALL_TRANSFER         0xE8    /* generate L3L4mSOFTWARE_STATUS msg */

#define L4L3mSET_CONFIGURATION              0xE9    /* Set hardware configuration. */
#define L4L3mREQ_CONFIGURATION              0xEA    /* Get hardware configuration status. */
#define L4L3mFORCE_CONNECTION_REQUEST       0xEB    /* CAS protocols: Inform hardware that host has detected a connection and host should update protocol state */
#define L4L3mJATE_REDIAL                    0xEC    /* Used to :                                      */
                                                    /* - specify max dialed nos to tracked for redial restriction */
                                                    /* - specify JAPAN emergency #'s              */
                                                    /* - clear redial restriction on a b-channel in 15X method    */
#define L4L3mCALL_INFORMATION_RECEIVED      0xED    /* Call Information received by host: (Caller ID - NTT) */

#define LAST_L4L3m_VALUE                    0xED    /* last value used for L4L3m messages */

/*
 * IISDN Controller -> User Control Messages (in L3L4_give->msgtype)
 */

#define FIRST_L3L4m_VALUE                   0x01    /* first value used for L3L4m messages */

#define L3L4mNULL                           0x00    /* no message */
/* Q.931 specific messages */
#define L3L4mSETUP_IND                      0x01    /* incoming call (CALL_REQ) */
#define L3L4mPROGRESS                       0x02    /* network is processing call */
#define L3L4mALERTING                       0x03    /* distant end is ringing */
#define L3L4mCONNECT                        0x04    /* distant end has answered */
#define L3L4mDISCONNECT                     0x05    /* distant end onhook */
#define L3L4mCLEAR_REQUEST                  0x06    /* interface is IDLE */
#define L3L4mSTATUS_IND                     0x07    /* remote state mismatch - used by both q931 and r2 */
                                                    /*     for q931 data union is IISDN_RAW_QDATA       */
                                                    /*     for r2 data union is IISDN_R2_CALL_STATUS    */
#define L3L4mCLEAR_WITH_RESTART_REQUEST     0x08    /* no reponse on clearing */
#define L3L4mUSER_INFO                      0x09    /* User->User Information */
#define L3L4mSETUP_ACK                      0x0A    /* distant end initial response */
#define L3L4mINFO                           0x0B    /* distant end information */
#define L3L4mCONN_ACK_IND                   0x0C    /* rcvd conn ack (optional) */
#define L3L4mRESTART                        0x0D    /* rcv'd restart */
#define L3L4mCONTINUE                       0x0F    /* Continuation of previous L3L4m (for additional information elements */

/* Supplemental ISDN Messages; Not supported by AT&T/Northern Telecom */
#define L3L4mSUSPEND_REQUEST                0x10    /* Suspend-call request (U->N) */
#define L3L4mSUSPEND_ACK                    0x11    /* Suspend Ack (Net->User) */
#define L3L4mSUSPEND_REJECT                 0x12    /* Suspend Reject (Net->User) */
#define L3L4mRESUME_REQUEST                 0x13    /* Resume (User->Network) */
#define L3L4mRESUME_ACK                     0x14    /* Resume Ack (Net->User) */
#define L3L4mRESUME_REJECT                  0x15    /* Resume Reject (Net->User) */

/* Raw Q.931 Packet Data for above messages if IISDNctAPEND_RAW_QMSG
   is set in conn_type of ENABLE_D_CHANNEL when enabling Q.931.  This
   message will immediately follow the above ISDN call control messages */
#define L3L4mRAW_QDATA                      0x16    /* For IISDNctAPEND_RAW_QMSG */
#define L3L4mUNIVERSAL                      0x17    /* Generic Q.931 message */
#define L3L4mSPID                           0x18    /* SPID received */

#define L3L4mX25_RESTART_IND                0x19    /* X25 Restart indication */

/* The following are management messages which do not pertain to phone calls.
   Call reference will be set to 0 and L4_ref to (-1) */

/* The error message.  Usually as a result of parsing an L4->L3 msg */
#define L3L4mERROR                          0x20    /* IISDN board detected an error */

/* The following are management messages which do not pertain to phone calls.
   Call reference will be set to 0 and L4_ref to (-1) */
#define L3L4mPROTOCOL_STATUS                0x21    /* Protocol channel status changed */
#define L3L4mB_CHANNEL_STATUS               0x22    /* B channel status changed */
#define L3L4mLINE_STATUS                    0x23    /* Line (T1/E1) status */
#define L3L4mHARDWARE_STATUS                0x24    /* Line and board level values */
#define L3L4mL2_STATS                       0x25    /* Level 2 Statistics data */
#define L3L4mLMI_STATUS                     0x26    /* Frame relay PVC table from network */

/* Utility Function Messages */
#define L3L4mCOMPARE_STATUS                 0x27    /* Result of L4L3mCOMPARE request */

/* Performance Monitoring Messages for Extended Superframe Data Link */
#define L3L4mPM_RESPONSE                    0x28    /* Respond to Perf Monitoring Reqst */
#define L3L4mPM_ALERT                       0x29    /* Unsolicited Perf Monitoring Msg */

/* Bonding Messages */
#define L3L4mBONDING_STATUS                 0x2A    /* Report bonding state */
#define L3L4mADD_BONDING                    0x2B    /* Add channel request */

/* X.25 Packet Layer Connection Control Message Extensions to Q.931 msgs */
#define L3L4mX25_INTERRUPT                  0x2C    /* X25 Packet Layer Interrupt Msg */
#define L3L4mX25_INT_CONF                   0x2D    /* X25 Packet Layer Interrupt Confirm */
#define L3L4mX25_RESET_IND                  0x2E    /* X25 Packet Layer Reset (pkts lost) */
#define L3L4mX25_CONN_ACK                   0x2F    /* X25 Packet Layer ack L4L3mCONNECT */

/* Feature Activation Messages */
#define L3L4mBILLING_STATUS                 0x30    /* Billing Rate Change Status */
#define L3L4mANI                            0x31    /* ANI from ANI on demand feature */
#define L3L4mPRE_SEIZE                      0x33    /* Report a preseize event */
#define L3L4mSEIZE_COMP                     0x34    /* Report seizure complete (sw56) */
#define L3L4mABCD_SIGNAL_DATA               0x35    /* Channel ABCD signaling data */
#define L3L4mRX_WINK                        0x36    /* Report wink reception (sw56) */
#define L3L4mTXWINK_END                     0x37    /* Report Tx wink completion (sw56) */
#define L3L4mTX_HOOKFLASH_END               0x38    /* Report hookflash completion (sw56) */
#define L3L4mCALL_PROC_SENT                 0x39    /* Report the transmission of a call proceeding (sw56) */
#define L3L4mCALL_PROCEEDING                0x3A    /* Report receipt of call proceeding */
#define L3L4mBOARD_ID                       0x3B    /* Report board identification */
#define L3L4mREJECT_JATE_REDIAL             0x3C    /* Jate Redial reject call after 3 tries*/
#define L3L4mCALL_REJECT                    0x3D    /* Incoming Call Rejected */
#define L3L4mHOOKFLASH                      0x3E    /* Report incoming hookflash */
#define L3L4mCAS_STATUS                     0x40    /* Bchannel status change */
#define L3L4mEND_DIAL                       0x42    /* Report end of pulse dialing (sw56) */
#define L3L4mREQ_DOWNLOAD                   0x43    /* request module from host */
#define L3L4mACK_DOWNLOAD                   0x44    /* acknowledge module part from host */
#define L3L4mRELAY_RULES                    0x45    /* response to L4L3mREQ_RELAY_RULES */
#define L3L4mRELAY_STATS                    0x46    /* response to L4L3mREQ_RELAY_STATS */
#define L3L4mACK_UPLOAD                     0x47    /* response to L4L3mUPLOAD */
#define L3L4mHOST_DEAD                      0x48    /* report HA host failure */
#define L3L4mHOST_ALIVE                     0x49    /* report return of HA host */
#define L3L4mTSI_STATUS                     0x4A    /* current SET_TSI mappings (CPCI only) */
#define L3L4mCAS_CHAN_BLOCKED               0x4B    /* CAS channel blocked (but protocol enabled) */
#define L3L4mCAS_CHAN_UNBLOCKED             0x4C    /* CAS channel unblocked */
#define L3L4mGEOGRAPHIC_ADDRESS             0x4D    /* report geographic address (CPCI only) */
#define L3L4mCONTROL_DATA                   0x4E    /* DED Direct to LAP message */
#define L3L4mSPARE_BITS_STATUS              0x4F    /* DED E1/T1 Slot 0 Spare Bits Status */
                                                    /* defined for backward compatibility */
#define L3L4mCAS_SIGNALING_BITS_STATUS      0x4F    /* CAS signaling data changed (spare bits in TS0 or ABCD bits) */

#define L3L4mCPU_UTILIZATION_DATA           0x50    /* CPU Utilization data (PCI and CPCI only) */
#define L3L4mVERSION                        0x51    /* IISDN version information */

#define L3L4mSOFTWARE_STATUS                0x52    /* response to L4L3mREQ_SW_STATUS */
#define L3L4mWARNING                        0x53    /* IISDN board detected a warning */
#define L3L4mTSI_ACK                        0x54    /* SET_TSI mappings finished */
#define L3L4mCONFIGURATION_STATUS           0x55    /* Hardware configuration status. */
#define L3L4mLOOP_ON                        0x56    /* Presence/absence of loop current */
#define L3L4mLOOP_REVERSAL                  0x57    /* Change in direction of loop current */
#define L3L4mRING_STATUS                    0x58    /* Ring signal on/off */
#define L3L4mCALLER_ID_DETECTED             0x59    /* Caller ID detected */
#define L3L4mACTIVATION_ACK_SIGNAL          0x5A    /* Activation Acknowledgment Signal sent to network (Caller ID - NTT) */
#define L3L4mINFORMATION_ACK_SIGNAL         0x5B    /* Information Acknowledgment Signal sent to network (Caller ID - NTT) */

#define LAST_L3L4m_VALUE                    0x5B    /* last value used for L3L4m messages */

/* !! IISDN does not generate the L3L4mFATAL_ERROR message.
      The     0x7F message code is reserved for device driver use. Consult your device
      driver manual to see if it utilizes this message and for what purpose.
      L3L4mFATAL_ERROR would typically be used to communicate to applications
      that the device driver has determined that IISDN is no longer operational.
*/
#define L3L4mFATAL_ERROR                    0x7F

/*
 * IISDN Controller -> User error codes (error_code for L3L4mERROR)
 */
/* These are responses to L4->L3 messages.  L4_ref is valid */
#define FIRST_L3L4err_VALUE              0 /* first error code used for L3L4mERROR messages */

#define L3L4errNO_ERROR                  0 /* No error occured */
#define L3L4errLAPDID_OUT_OF_RANGE       1 /* LAPD ID not [0..23] or [0..31] on XP, E1, or ISA */
#define L3L4errLAPDID_NOT_ESTABLISHED    2 /* LAPD is not up */
#define L3L4errINVALID_CALLED_NUMBER     3 /* Bad phone # or > 17 digits */
#define L3L4errNO_CRV_AVAILABLE          4 /* No more call ref. numbers */
#define L3L4errNO_CRSTRUCT_AVAILABLE     5 /* No Call Record structs avail */
#define L3L4errCALL_REF_ERROR            6 /* Unknown call reference */
#define L3L4errINVALID_B_CHANNEL         7 /* B channel in CALL REQ bad */
#define L3L4errB_CHANNEL_RESTARTING      8 /* B channel is being restarted */
#define L3L4errB_CHANNEL_OOS             9 /* B channel is out of service */
#define L3L4errINVALID_CALL_TYPE        10 /* Cannot place outgoing call */
#define L3L4errINVALID_CONN_TYPE        11 /* Invalid enable protocol cmd */
#define L3L4errD_CHAN_NOT_DISABLED      12 /* Invalid enable protocol cmd */
#define L3L4errPROTOCOL_NOT_DISABLED    12 /* Invalid enable protocol cmd */
#define L3L4errINVALID_HDLC_MAPPING     13 /* Invalid enable protocol cmd */
#define L3L4errINVALID_DATA_QUEUE       14 /* Host set up invalid buffers */
#define L3L4errINVALID_COMMAND_ARGS     15 /* Out of range arguments */
#define L3L4errINVALID_MSG_FOR_STATE    16 /* Invalid cntl message */
#define L3L4errDATA_PACKET_LOST         17 /* TX buffer too small */
#define L3L4errPM_NOT_ESF               18 /* Performance Monitoring Not ESF */
#define L3L4errINVALID_INTERFACE        19 /* NFAS invalid PRI interface */
#define L3L4errB_CHANNEL_INUSE          20 /* Rejected Call Request */
#define L3L4errINVALID_LLI              21 /* (LLI & x0301) is nonzero */
                                           /* or Q931 != 0 or LAPB != x4000 */
                                           /* or LAPF and (LLI & x030F) */
#define L3L4errVC_TABLE_FULL            22 /* Max LLIs per phys chan reached */
#define L3L4errLLI_NOT_FOUND            23 /* LLI not in data structs */
#define L3L4errBLOCKED                  24 /* tsi blocked from connecting */
#define L3L4errNO_HARDWARE              25 /* hardware required for request is not present on board type */
#define L3L4errINVALID_SPID_LEN         26 /* BRI Svc Prof Length not 3..20 */
#define L3L4errNON_NFAS                 27 /* NFAS used on non NFAS lapdid */
#define L3L4errINVALID_STATE            28 /* NFAS cannot switch states */
#define L3L4errSERVICE_NOT_OFFERED      29 /* Service not offered for call or feature */
#define L3L4errDCHAN_TEMP_UNAVAIL       30 /* NFAS D-channel is switching */
#define L3L4errTOO_MANY_Q931_STACKS     31 /* Exceeded max_L3_stacks */
#define L3L4errSERVICE_NOT_CONFIG       32 /* Use L4L3mCONFIG_IISDN */
#define L3L4errDATA_INTERFACE_REQUIRED  33 /* Must specify data interface */
#define L3L4errDATA_INTERFACE_INVALID   34 /* Q.931 has no data interface */
#define L3L4errSYMM_MODE_NOT_SUPPORTED  35 /* LAP-B must be DCE or DTE */
#define L3L4errINVALID_BUFFSZ           36 /* Invalid HDLC chip buffer size */
#define L3L4errBOND_CHAN_NOT_CNFG       37 /* Exceeded bonding_numchan */
#define L3L4errBOND_CHAN_BIT_CONFLICT   38 /* lapdid's channel bit not set */
#define L3L4errBOND_WRONG_SPYDER_CHIP   39 /* add-on channel on wrong chip */
#define L3L4errBOND_TOO_MANY_CHANNELS   40 /* Exceeded channels-per-call */
#define L3L4errBOND_DUP_ADDON_CHAN      41 /* Add-on channel already in call */
#define L3L4errDCHAN_ODD_POINTER_ERROR  42 /* Host-provided pointer ODD */
#define L3L4errDCHAN_TOO_FEW_BUFFERS    43 /* Must have 3 tx/3 rx bufs */
#define L3L4errDCHAN_TOO_MANY_BUFFERS   44 /* Max 255 tx or rx bufs */
#define L3L4errDCHAN_GIVE_TAKE_NONZERO  45 /* Host must init give/take = 0 */
#define L3L4errDCHAN_ZERO_RXBUF_LEN     46 /* rx bufs must be nonzero len */
#define L3L4errSYMM_MODE_REQUIRED       47 /* LAP-F must be Symmetric */
#define L3L4errDLCI_MANDITORY           48 /* DLCI field is manditory */
#define L3L4errCHAN_KBIT_RATE_BAD       49 /* channel kilobit rate invalid */
#define L3L4errINVALID_MEM_SIZE         50 /* device_xx_mem too small/large */
#define L3L4errNOT_ENOUGH_MEMORY        51 /* can't carve bufs from dev mem */
#define L3L4errTX_BUFFER_MISALIGNED     52 /* blk bfr not 256 byte aligned */
#define L3L4errRX_BUFFER_MISALIGNED     53 /* blk bfr not 256 byte aligned */
#define L3L4errTOO_MANY_DLCIS           54 /* exceeded IISDN_MAX_VC_PER_CHAN */
#define L3L4errBOND_BAD_STATE           55 /* error issued in illegal state */
#define L3L4errSPID_REJECTED            56 /* BRI SPID rejected by network */
#define L3L4errTEI_IDENT_REMOVE_REQ     57 /* received REMOVE for fixed TEI */
#define L3L4err2ND_SPID_REJECTED        58 /* BRI 2nd SPID rejected by netwk */
#define L3L4errINVALID_SMI_MSGID        59 /* SMI message type is invalid */
#define L3L4errINVALID_CLOCK_MODE       60 /* Invalid clocking type in L4L3mSET_HARDWARE message */
#define L3L4errNO_OVERFLOW_QUEUE        61 /* Overflow queue not initialized */
#define L3L4errTOO_MANY_CAS_DEST        62 /* Cannot run mvip_cas and cas_to_memory */
#define L3L4errSEGMENT_TOO_LARGE        63   /* Tramsmitted segment message contains more than 8 segments */
#define L3L4errSEGMENT_MESSAGE_EXPECTED 64   /* Received message not a segment message */
#define L3L4errSEGMENT_MESSAGE_INVALID  65   /* Received segment message content error */
#define L3L4errSEGMENT_TIMER_EXPIRED    66   /* Expected segment message not recieved */
#define L3L4errINVALID_DOWNLOAD_MSG     67   /* unrecognized L4L3mDOWNLOAD msg content*/
#define L3L4errPROTOCOL_DISABLED        68   /* protocol is disabled */
#define L3L4errINVALID_VARIANT          69   /* MTP variant out of range */
#define L3L4errTOO_MANY_LINKS           70   /* MTP can only handle 256 links */
#define L3L4errTOO_MANY_HEADERS         71   /* Relay module only supports 255 headers per lapdid */
#define L3L4errFEATURE_NOT_ACTIVE       72  /* R2 */
#define L3L4errCOMPELLING_ERROR         73  /* R2 */
#define L3L4errDCHAN_OUT_OF_RANGE       74  /* Data channel out of configured range */
#define L3L4errCOMMAND_DISABLED         75  /* Data channel out of configured range */
#define L3L4errPARAMETER_NOT_SUPPORTED  76  /* Parameter not supported by platform */
#define L3L4errPARAMETER_OUT_OF_RANGE   77  /* Parameter value out of accepted range */
#define L3L4errFATAL_ERROR              78  /* L3L4mFATAL_ERROR:  indicates a board panic. */
#define L3L4errHOT_SWAP_EXTRACTION      79  /* L3L4mFATAL_ERROR:  indicates a board was extracted. */
#define L3L4errINITIALIZATION_FAILED    80  /* CAS initialization failed */
#define L3L4errCID_CHECKSUM             81  /* CallerId error: Bad checksum */
#define L3L4errCID_DATA_TRANSFER        82  /* CallerId error: Data transfer failed */
#define L3L4errCID_DATA_PENDING         83  /* CallerId error: Stale data */
#define L3L4errCID_INVALID_FORMAT       84  /* CallerId error: Invalid parameter data or length */
#define L3L4errCID_INVALID_TYPE         85  /* CallerId error: Invalid caller ID type */
#define L3L4errCALL_INFORMATION_TIMEOUT 86  /* Timeout waiting for Call Information */

#define LAST_L3L4err_VALUE              86 /* last error code used for L3L4mERROR messages */

/*
 * IISDN Controller -> User warning codes (warning_code for L3L4mWARNING)
 */
/* These are responses to L4->L3 messages.  L4_ref is valid */

#define L3L4wrnNO_WARNING               0
#define L3L4wrnINVALID_FILL_COUNT       1
#define L3L4wrnTOO_MANY_COPY_DEST       2
#define L3L4wrnMAX_NUM_WARNINGS         2


/* Network Maintanence error codes (L4_ref set to FFFF) */
#define  L3L4errSTATUS_IGNORED                  100 /* Ignored a message from ATT Maintence */
#define  L3L4errBAD_CALL_REF                    101 /* Got bad call reference # in maint. msg */
#define  L3L4errGLARE                                   102 /* Glare detected */

/* Layer 1 -> application MPH (physical layer managment messages */
#define IISDN_mph_NONE                                  0x00    /* no mph message */
#define IISDN_mph_INFO_IND_CONN                 0x01    /* connected */
#define IISDN_mph_INFO_IND_DISC                 0x02    /* disconnected */
#define IISDN_mph_DEACT_IND_T3_EXP              0x04    /* line deactivated; Timer3 exprd */
#define IISDN_mph_DEACT_IND_I0_RCV              0x08    /* line deactivated; INFO0 rcvd */
#define IISDN_mph_ERROR_IND_FRAM                0x10    /* error ind; lost framing */
#define IISDN_mph_ERROR_IND_F7_I2               0x20    /* error ind; INFO2 rcvd in st 7 */
#define IISDN_mph_ACT_IND                               0x40    /* line activated */
#define IISDN_mph_STATUS_RESP                   0x80    /* resp to L4L3mREQ_LINE_STATUS  */

/* 
 * IISDN Controller <-> User   Interface data structures
 */

/* Data structures for passing digits when originating/receiving a call */
#define IISDN_MAX_DIGITS        24              /* AT&T parameter */

/* Calling party number "number type" for AT&T 4.5.10 */
#define IISDNnumtUNKNOWN                0       /* Unknown type of number */
#define IISDNnumtINTERNATIONAL  1       /* International number */
#define IISDNnumtNATIONAL               2       /* National number */
#define IISDNnumtSUBSCRIBER             4       /* Subsriber number */
#define IISDNnumtABBREVIATED    6       /* Abbreviated number */

/* Calling party "Numbering plan identification" 4.5.10 */
#define IISDNnumpUNKNOWN                0       /* Unknown numbering plan */
#define IISDNnumpISDN                   1       /* Recommendation E.164/E.163 */
#define IISDNnumpPRIVATE                9       /* Private numbering plan */
#define IISDNnumpTELEPHONY              2       /* Telephony numbering plan */

/* Default digit type - specifies the dialing mode: DTMF, MF, PULSE, etc. */
#define IISDNdigtypeNONE      0  /* not speficied */
#define IISDNdigtypeDTMF      1  /* send DTMF digits */
#define IISDNdigtypeMF        2  /* send MF digits */
#define IISDNdigtypePULSE     3  /* send PULSE (decadic) digits (analog or digital) */
#define IISDNdigtypeR2        4  /* R2 compelled interregister signaling */

/* Presentation indicator and screening indicator are optional.  The
 * network may not supply this information on incoming calls in which
 * case the values default to IISDNpresALLOWED and PREscrUSER_NOT_SCREENED.
 * If you don't care about the values when originating calls, use the
 * default values. (Zero's)
 */

#define IISDNpresALLOWED                0       /* Presentation is allowed */
#define IISDNpresRESTRICTED             1       /* Presentation is restricted */
#define IISDNpresNUM_NOT_AVAIL  2       /* Number not avail. due to interworking */

#define IISDNscrUSER_NOT_SCREENED       0       /* User provided, not screened */
#define IISDNscrUSER_VER_PASSED         1       /* User provided, verified and passed */
#define IISDNscrUSER_VER_FAILED         2       /* User provided, verified and failed */
#define IISDNscrNETWORK_PROVIDED        3       /* Network provided */

/* Redirection Reason */
#define IISDNrrsnUNKNOWN                0x0     /* Redirection reason not available */
#define IISDNrrsnCALL_FWD_BUSY  0x1     /* Call forward/busy */
#define IISDNrrsnCALL_FWD_NOANS 0x2     /* Call forward/no answer */
#define IISDNrrsnCALL_TRANSFER  0x4     /* Call transfer */
#define IISDNrrsnCALL_PICKUP    0x5     /* Call pickup */
#define IISDNrrsnOOS                    0x9     /* Called DTE out of order */
#define IISDNrrsnCALL_FWD_DTE   0xA     /* Call fwd by the called DTE */
#define IISDNrrsnCALL_FWD_ALL   0xF     /* Unconditional call forwarding */

/* Types of incoming and outgoing calls */
#define IISDNcalltypVOICE                       0x00000001L     /* Voice call (uLaw-North America)*/
#define IISDNcalltypMODEM                       0x00000002L     /* 3.1 kHz audio (uLaw-North Amer)*/
#define IISDNcalltyp56K                         0x00000004L     /* 56 K data call, unknown type */
#define IISDNcalltyp64K                         0x00000008L     /* 64 K data call, unknown type */
#define IISDNcalltyp64K_REST            0x00000010L     /* Restricted 64 K data call */
#define IISDNcalltyp384K                        0x00000020L     /* 384 K data call, unknown type */
#define IISDNcalltyp384K_REST           0x00000040L     /* Restricted 384 K data call */
#define IISDNcalltyp56K_V120            0x00000080L     /* 56 K V.120 data call */
#define IISDNcalltyp64K_V120            0x00000100L     /* 64 K V.120 data call */
#define IISDNcalltyp56K_X25                     0x00000200L     /* 56K X.25 call */
#define IISDNcalltyp64K_X25                     0x00000800L     /* 64K X.25 call */
#define IISDNcalltypQ922                        0x00001000L     /* Q.922 */
#define IISDNcalltyp64K_V110            0x00002000L     /* V.110 */
#define IISDNcalltypMULTIRATE_DATA      0x00004000L /* Unrestricted Multirate data */
#define IISDNcalltyp1536K                       0x00008000L     /* 1536 K data call, unrestricted */
#define IISDNcalltyp56K_UNREST          0x00010000L /* 56K Unrestricted data */
#define IISDNcalltypALAW_VOICE          0x00020000L     /* Voice call (aLaw-outside US) */
#define IISDNcalltypALAW_MODEM          0x00040000L     /* 3.1 kHz audio (aLaw-outside US) */
#define IISDNcalltypULAW_7KHZ           0x00080000L     /* 7 kHz audio (uLaw-North Amer) */
#define IISDNcalltypALAW_7KHZ           0x00100000L     /* 7 kHz audio (aLaw) */
#define IISDNcalltypFAX                         0x00200000L     /* facsimile */
#define IISDNcalltypPACKET                      0x00400000L     /* Packet Mode */
#define IISDNcalltyp32K_ADPCM           0x00800000L     /* G.721 */
#define IISDNcalltypAMF                         0x01000000L     /* Alternate Modem Function */
#define IISDNcalltypR2_GND                      0x02000000L     /* R2 terrestrial */
#define IISDNcalltypR2_SAT                      0x04000000L     /* R2 satellite */

/* Network-Specific Types of Calls (Call-by-Call feature) */
#define IISDNnsNULL                             0x00            /* The default case */
#define IISDNnsATT_SDN                  0x01            /* AT&T Software Defined Network */
#define IISDNnsATT_MEGACOM800   0x02            /* AT&T Megacom 800 service */
#define IISDNnsATT_MEGACOM              0x03            /* AT&T Megacom */
#define IISDNnsATT_ACCUNET              0x06            /* AT&T Accunet */
#define IISDNnsATT_I800                 0x08            /* AT&T International 800 */
#define IISDNnsATT_MULTIQUEST   0x10            /* AT&T MultiQuest */
#define IISDNnsNTI_PRIVATE              0x01            /* Northern Tel Private Net */
#define IISDNnsNTI_INWATS               0x02            /* Northern Tel InWats */
#define IISDNnsNTI_OUTWATS              0x03            /* Northern Tel OutWats */
#define IISDNnsNTI_FX                   0x04            /* Northern Tel Foreign Exchange */
#define IISDNnsNTI_TIE_TRUNK    0x05            /* Northern Tel Tie Trunk */
#define IISDNnsNTI_TRO                  0x10            /* Northern Tel TRO call */

/* ISDN Cause values (ATT Pub 41449 subset of ITU 1988) */
#define IISDNcausDEFAULT                0               /* Use the default cause or not avail */
#define IISDNcausUNALOC_NUM             1               /* Unallocated number */
#define IISDNcausNO_ROUTE               2               /* No route specified to transit net */
#define IISDNcausBAD_CHAN               6               /* Channel unacceptable */
#define IISDNcausNORML_CLR              16              /* Normal Clearing */
#define IISDNcausUSER_BUSY              17              /* User busy */
#define IISDNcausNO_USR_RSP             18              /* No user responding */
#define IISDNcausCALL_REJ               21              /* Call rejected */
#define IISDNcausNUM_CHANGED    22              /* Number has been changed */
#define IISDNcausDEST_OOO               27              /* Destination Out of Order */
#define IISDNcausINVALID_NUM    28              /* Invalid number format */
#define IISDNcausFACIL_REJ              29              /* Facility rejected */
#define IISDNcausSTAT_RESP              30              /* Response to status enquiry */
#define IISDNcausNRML_UNSPEC    31              /* Normal, Unspecified */
#define IISDNcausNO_CHAN_AVL    34              /* No ckt/channel avail */
#define IISDNcausNET_OOS                38              /* Network out of order */
#define IISDNcausTEMP_FAILURE   41              /* Temporary Failure */
#define IISDNcausSW_CONGEST             42              /* Switching Equip Congestion */
#define IISDNcausACCESS_DISC    43              /* Access info discarded */
#define IISDNcausCKT_NOT_AVL    44              /* Requested circuit channel not avail */
#define IISDNcausPREEMPT                45              /* Pre-empted (AT&T Special Defined) */
#define IISDNcausRES_UNAVAIL    47              /* Resource unavailable, unspecified */
#define IISDNcausFAC_NOT_SUSC   50              /* Requested facility not subscribed */
#define IISDNcausOUT_BARRED             52              /* Outgoing calls barred */
#define IISDNcausIN_BARRED              54              /* Incoming calls barred */
#define IISDNcausBR_CAP_AUTH    57              /* Bearer Capabilitiy not authorized */
#define IISDNcausBR_CAP_NA              58              /* Bearer Capibility not available */
#define IISDNcausSRVC_NA                63              /* Service/option not avail; unspecified */
#define IISDNcausBR_SVC_NIMP    65              /* Bearer service not implemented */
#define IISDNcausCHNTYP_NIMP    66              /* Channel type not implemented */
#define IISDNcausFAC_NIMP               69              /* Requested facility not implemented */
#define IISDNcausBR_REST_ONLY   70              /* Only restricted digitial available */
#define IISDNcausSVC_NIMP               79              /* Service not implemented, unspecified */
#define IISDNcausINV_CALL_REF   81              /* Invalid call reference value */
#define IISDNcausCHAN_DNE               82              /* Identified channel does not exist */
#define IISDNcausINCOMPAT_DST   88              /* Incompatible destination */
#define IISDNcausINVL_MSG               95              /* Invalid message, unspecified */
#define IISDNcausMAND_IE                96              /* Mandatory info element missing */
#define IISDNcausMSGTYP_BAD             97              /* Message type non-existant/not impl */
#define IISDNcausINCOMPAT_MSG   98              /* Msg no good for state of call */
#define IISDNcausIE_NOT_EXIST   99              /* Info element does not exist */
#define IISDNcausINVL_IE_CONT   100             /* Invalid info element contents */
#define IISDNcausMSG_COMPAT             101             /* Message not compatible w/call state */
#define IISDNcausTIMER_EXPIR    102             /* Recovery on timer expiration */
#define IISDNcausNO_ACK       103      /* Remote end not acknowledging call */
#define IISDNcausNO_DIAL_TONE 104      /* No dial tone - dialing cannot proceed */
#define IISDNcausPROTO_ERR              111             /* Protocol error, unspecified */
#define IISDNcausINTERWORK              127             /* Interworking, unspecified */

/* DPNSS specific cause codes */
#define IISDNcausDPNSS_ACK              128             /* Supplimentary service OK */

/* Coding Standard for Cause Information Element */
#define IISDNcodCCITT                   0               /* Follows CCITT Coding Standard */
#define IISDNcodITU                             0               /* Follows ITU Coding Standard */
#define IISDNcodNATIONAL_STD    2               /* National Standard Coding */
#define IISDNcodSTD_SPF_2_LOC   3               /* Standard specific to ident location */

/* Location for Cause Information Element */
#define IISDNlocUSER                    0               /* User */
#define IISDNlocPVT_LOCAL               1               /* Private net serving local user */
#define IISDNlocPUB_LOCAL               2               /* Public net serving local user */
#define IISDNlocTRANSIT_NET             3               /* Transit network */
#define IISDNlocPUB_REMOTE              4               /* Public net serving remote user */
#define IISDNlocPVT_REMOTE              5               /* Private net serving remote user */
#define IISDNlocINTERNATIONAL   7               /* International networks */
#define IISDNlocBEY_INTERWORK   10              /* Net Beyond interworking point */

/* Progress Indicators */
#define IISDNprogUNKNOWN                        0       /* Information not available */
#define IISDNprogNOT_ISDN_INBAND        1       /* Not End2End ISDN; Info may be in band */
#define IISDNprogDEST_NOT_ISDN          2       /* Destination address not ISDN */
#define IISDNprogORIG_NOT_ISDN          3       /* Origination address not ISDN */
#define IISDNprogRETURNED_ISDN          4       /* Call has returned to the ISDN */
#define IISDNprogINBAND_INFO_AVL        8       /* In-Band information available */


#define IISDNctQ931_FIELD               0x00FF0000L     /* Mask to access Q.931 enumerations */
#define IISDNctATT4_Q931                0x00800000L     /* Q.931 for an AT&T #4 ESS */
#define IISDNctATT5_Q931                0x00810000L     /* Q.931 for an AT&T #5 ESS */
#define IISDNctNTI100_Q931              0x00820000L     /* Q.931 for Northern Telecom DMS-100 */
#define IISDNctNTI250_Q931              0x00830000L     /* Q.931 for Northern DMS-250 */
#define IISDNctATT_NET_Q931             0x00840000L     /* Network side emulation (4ESS) */
#define IISDNctMD110_T1                 0x00850000L     /* Ericsson MD-110 (North America) */
#define IISDNctMD110_E1                 0x00860000L     /* Ericsson MD-110 (International) */
#define IISDNctSIEMENS                  0x00870000L     /* Siemens (North America) */
#define IISDNctNTT                              0x00880000L     /* NTT Switch Type (Japan) */
#define IISDNctSIEMENS_1TR6             0x008A0000L /* Siemens 1TR6 */
#define IISDNctE1_30_BCHAN              0x01000000L     /* 30 B channels rather than 23 */
#define IISDNctAPEND_RAW_QMSG   0x02000000L     /* Send Undecoded IFRAME msgs too */
#define IISDNctNFAS                             0x04000000L     /* Non-Facility Assoc Signalling */
#define IISDNctBASIC_RATE               0x08000000L     /* BRI Q.931 Signalling */
#define IISDNctCCITT                    0x10000000L     /* CCITT compatible mode */
#define IISDNctITU                              0x10000000L     /* ITU compatible mode */
#define IISDNctB_CHAN_NEGOT             0x20000000L     /* B-channel negotiation feature */
#define IISDNctPROC_ON_EXCLSV   0x40000000L     /* If exclusive CALL_PROC response */

/* Defines for backup_control field */
#define IISDNbcNO_CHANGE                0
#define IISDNbcSWITCH_ACTIVE    1

/* Defines for q931_config field */
#define IISDNcnfgUSE_SLOT_MAP                   0x00000001L     /* Channel ID IE coded with     slot map rather than channel number. */
#define IISDNcnfgSPRS_CHANID_CALLPROC   0x00000002L     /* Suppress Channel ID IE in the CALL PROCEEDING message. */
#define IISDNcnfgNO_CHANID_CALLPROC             0x00000004L     /* Accept no Channel ID IE in the CALL PROCEEDING message. */

/* X.25 Packet Layer Configuration (optional) */
#define IISDNx25cDCE                                            0x00000001L     /* Set DCE, Cleared if DTE */
#define IISDNx25cNEG_PACKETSZ                           0x00000002L     /* If set, negotiate packet size */
#define IISDNx25cNEG_WINDOWSZ                           0x00000004L     /* If set, negotiate window size */
#define IISDNx25cNET_USER_ID                            0x00000008L     /* If set, accept network user id */
#define IISDNx25cIGNORE_UNUSED_FACILITIES       0x00000010L     /* If set, ignore all facilities except packet and window size */
#define IISDNx25cDTE_DTE                                        0x00000020L     /* DTE-DTE mode */
#define IISDNx25cRESTART_CONFIRM                        0x00000040L     /* use L3L4mX25_RESTART_IND when restart confirm received */
typedef struct
        {
        IISDNu32bit     cfg_msk;                /* configuration bit mask IISDNx25cXXXX */
        /* L3 timer tics in seconds. 0 choses default. 0xFFFF disables timer */
        IISDNu16bit     T10;                    /* DCE( 60s)/DTE T20(180s) Restart Timer */
        IISDNu16bit     T11;                    /* DCE(180s)/DTE T21(200s) Call Req Timer */
        IISDNu16bit     T12;                    /* DCE( 60s)/DTE T22(180s) Reset Timer */
        IISDNu16bit     T13;                    /* DCE( 60s)/DTE T23(180s) Clear Timer */
        IISDNu16bit     T28;                    /* DTE (300s) Registration Timer (not impl) */
        IISDNu16bit     P;                              /* Packet Layer packet size (maximum) (default 256) 128, 256, 512, 1024, 2048 */
        IISDNu8bit      W;                              /* Packet layer window size (maxiumum), (default = 2) 1-7 */
        IISDNu8bit      max_clr_retry;  /* Allow n T13 expirations (default = 1) */
        IISDNu8bit      max_svcs;               /* Maximum number of Switched Virtual Circuits/Calls */
        IISDNu8bit      max_pvcs;               /* Maximum number of Permanent Virtual Circuits */
        } IISDN_X25_CONFIG;

/* Q.931 timer values */
typedef struct
        {
        /* All q931 level 3 timers are in 100 millisecond second tics */
        /* Setting a field to 0 will cause it to be set to the default */
        /* A value of 0xFFFF disables the timer */
        IISDNu16bit t302;       /* Overlap Receiving timer Default 150 (15 sec) */
        IISDNu16bit t305;       /* Disconnect request state timer Default 50(5 sec) or for ITU: 30 seconds (300 tics) */
        IISDNu16bit t308;       /* Release Request State timer Default 50 (5 sec) or for ITU: 4 seconds (40 tics) */
        IISDNu16bit t313;       /* Connect Request state timer Default 50 (5 sec) or for ITU: 4 seconds (40 tics) */
        IISDNu16bit t314;       /* Message Segment timer Default 4 seconds (40 tics) */
        IISDNu16bit t316;       /* Restart Request state timer Default 120 sec */
        IISDNu16bit t318;       /* Resume Request state timer Default 4 sec */
        IISDNu16bit t319;       /* Suspend Request state timer Default 120 sec */
        IISDNu16bit t3m1;       /* Maintence SERVICE ACK timer Default 120 sec */
        IISDNu16bit t321;       /* D channel backup timer Default 40 s (400 tics) */
        } IISDN_Q931_TIMERS;

#define IISDN_MAX_SPID_LEN 20   /* Basic Rate Service Profile ID String */
#define IISDN_MAX_DN_LEN    8 /* Basic Rate Directory Number length */



/* Following data structures are for the L4L3mENABLE_PROTOCOL message */

/* Level 1 mode */
#define IISDNl1modHDLC                  0       /* HDLC packetization */
#define IISDNl1modRAW                   1       /* No HDLC packetization(raw byte stream) */
#define IISDNl1modV110                  2       /* V110 mode - transparent mode 1 */
#define IISDNl1modRAW_SPECIAL1  4       /* aggregate Raw stream with header byte */
#define IISDNl1modISLP                  5       /* HDLC packetization without CRCs */
#define IISDNl1modMODEM                 6       /* Analog communications */
#define IISDNl1modSIGNALING             7       /* Channel Associated Signaling Bit Data */
#define IISDNl1modETHERNET              8       /* IEEE 802.3 / ethernet layer 1 MAC */
#define IISDNl1modBULK_RAW              9       /* Roughly syncronized raw data stream */

/* Netaccess buffered I/O devices */
enum IISDNdevs {
 IISDNdevDEFAULT = 0,   /* use default device associated with lapdid */
                                        /* ------------------------------------------------- */
                                        /* ISA       PCI       CPCI-V    CPCI-D    CPCI-D2   */
                                        /* --------- --------- --------- --------- --------- */
 IISDNdevHDLC_0,        /* LuT7115   68mh360   mpc860mh  Bt8474(1) Bt8478(1) */
 IISDNdevHDLC_1,        /* LuT7115   LuT7115   n/a       Bt8474(1) Bt8478(1) */
 IISDNdevHDLC_2,        /* n/a       n/a       n/a       Bt8474(1) Bt8478(1) */
 IISDNdevHDLC_3,        /* n/a       n/a       n/a       Bt8474(1) Bt8478(1) */
 IISDNdevHDLC_4,        /* n/a       n/a       n/a       Bt8474(2) Bt8478(1) */
 IISDNdevHDLC_5,        /* n/a       n/a       n/a       Bt8474(2) Bt8478(1) */
 IISDNdevHDLC_6,        /* n/a       n/a       n/a       Bt8474(2) Bt8478(1) */
 IISDNdevHDLC_7,        /* n/a       n/a       n/a       Bt8474(2) Bt8478(1) */
 IISDNdevHDLC_8,        /* n/a       n/a       n/a       mpc860    Bt8478(2) */
 IISDNdevHDLC_9,        /* n/a       n/a       n/a       n/a       Bt8478(2) */
 IISDNdevHDLC_10,       /* n/a       n/a       n/a       n/a       Bt8478(2) */
 IISDNdevHDLC_11,       /* n/a       n/a       n/a       n/a       Bt8478(2) */
 IISDNdevHDLC_12,       /* n/a       n/a       n/a       n/a       Bt8478(2) */
 IISDNdevHDLC_13,       /* n/a       n/a       n/a       n/a       Bt8478(2) */
 IISDNdevHDLC_14,       /* n/a       n/a       n/a       n/a       Bt8478(2) */
 IISDNdevHDLC_15,       /* n/a       n/a       n/a       n/a       Bt8478(2) */
 IISDNdevHDLC_16,       /* n/a       n/a       n/a       n/a       mpc860    */
                                        /* ------------------------------------------------- */
                                        /* Note: the CPCI-D2 column represents the higher    */
                                        /* density product.  The example shown lists the     */
                                        /* maximum density possible.  Population may include */
                                        /* any combination of Bt8478 (8 logical devices),    */
                                        /* Bt8474 (4 logical devices) or Bt8472 (2 logical   */
                                        /* devices); there will never be more than two of    */
                                        /* these chips.  Assume that if two chips with       */
                                        /* different densities are present, the chip with    */
                                        /* the higher density will be assigned the lower     */
                                        /* logical HDLC device numbers.  Also assume that    */
                                        /* the logical HDLC device number for the CPU HDLC   */
                                        /* controller will be the first available logical    */
                                        /* channel number after the Bt847x devices.          */
                                        /* ------------------------------------------------- */
 IISDNdevMODEM_0,       /* primary Netaccess integrated modem bank */
 IISDNdevMODEM_1,       /* secondary Netaccess integrated modem bank */
 IISDNdevSYNC_0,        /* synchronous comm controller */
 IISDNdevFDL,           /* T1 facility data link */
 IISDNdevHDLC_LINE,     /* hdlc resource assoc'd with line interface */
 IISDNdevSIGNALING,     /* signaling resource assoc'd with line interface */
 IISDNdevETHERNET       /* integrated Ethernet port */
};

#define IISDNdevHDLC(x) (IISDNdevHDLC_0 + (x))  /* 0 >= x <= 16 */

/* HDLC flag stuffing modes.  Transmit extra flags between packets */
#define IISDNflgmCONST                          0       /* Always insert 'value' extra flags */
#define IISDNflgmCONST_FLAG_ADJUST      1       /* Insert value decremented for hdlc bit stuffing */
#define IISDNflgmRTA                            2       /* Insert extra variable flags as: */
                                                                                /* flags = 1+size*(64000-value)/value */
                                                                                /* size is L2_her + data + crc + flag */
                                                                                /* stuff a maximum of 2047 flags */
#define IISDNflgmRTA_FLAG_ADJUST        3       /* decrement for hdlc bit stuffing */

typedef struct
        {
        IISDNu8bit      l1_mode;                        /* Level 1 mode, use IISDNl1modXXX */
        IISDNu8bit      invert_hdlc;            /* Invert the hdlc stream if nonzero */
        IISDNu16bit     num_txbuf;                      /* Number of layer 1 device tx buffers */
        IISDNu16bit     num_rxbuf;                      /* Number of layer 1 device rx buffers */
        IISDNu16bit     buffsz;                         /* Size of layer 1 device buffers */
        IISDNu8bit      chain;                          /* obsolete, ignored */
        IISDNu8bit      device;                         /* onboard device, use IISDNdevXXX */
        IISDNu8bit      bit_reverse;            /* tx/rx bytes MSbit first */
        IISDNu8bit      vme_lock;                       /* obsolete, ignored */
        IISDNu32bit     hdlc_channels;          /* Channel concatenaton control */
                                                                        /* Least significant bit is slot 0 */
                                                                        /* Most significant bit is slot 31 */
                                                                        /* Zero => (1 << (lapdid % 32)) */
        IISDNu16bit     chan_kbit_rate;         /* device chip bit rate  0 => 4096k */
        IISDNu8bit      crc_bytes;                      /* 0 default, 2 = 16bit, 4 = 32bit */
        IISDNu8bit      crc_ignore_errs;        /* count errors, and deliver *bad* packet */
        /* Rate adaption.  value x7f is standard 56K rate adaption */
        struct
                {
                IISDNu8bit enable;                      /* 1 to enable rate adaption */
                IISDNu8bit rate_adapt_value;/* 0 => 7F.  BRI D-channel, use 0x03 */
                IISDNu16bit pad;                        /* Longword alignment */
                } rate_adapt;
        /* Default fill/idle character in raw (or sca) mode is 0xFF */
        struct
                {
                IISDNu8bit enable;                      /* 1 to enable fill/idle char overide */
                IISDNu8bit fill_value;          /* Tx value when no data to send */
                IISDNu16bit pad;                        /* Longword alignment */
                } raw_fillchar;
        /* HDLC between-packet flag stuffing.  Default is 1 flag between packets */
        struct
                {
                IISDNu8bit      enable;                 /* 1 when enabling flag stuffing */
                IISDNu8bit      mode;                   /* IISDNflgmXXX */
                IISDNu16bit     value;                  /* # flags to stuff [1..2047] */
                } hdlc_flag_fill;                       /*   or tx bit rate [1..64000] */

        /* integrated Netacceess MODEM analog connection parameters */
        struct 
                {
                IISDNu8bit originate;           /* 0=answer, 1=dial, 2=L4 transparent access */
                IISDNu8bit faxClass;            /* 0=notFax, class 1 or 2 */
                IISDNu8bit encoding;            /* 0=uLaw, 1=Alaw */
                IISDNu8bit amf;                         /* Alternate Modem Function */
                IISDNu32bit amf_params[4];      /* Alternate Modem Function parameters */
                IISDNu32bit minBPS;                     /* slowest connection to accept, 0=don't care */
                IISDNu32bit maxBPS;                     /* fastest connection acceptable, 0=don't care */
                } modem;

        /* V.110 parameter settings */
        struct 
                {
                IISDNu32bit bit_rate;           /* preferred bit rate, default 9600 */
                IISDNu8bit auto_detect;         /* 1 => detect bit rate 0 => only use specified bit rate */
                IISDNu8bit pad[3];                      /* Longword alignment */
                } v110;
                
        } IISDN_LEVEL1_CNFG;

#define IISDNl2modLAP_D                 0       /* Q.921 data link layer */
#define IISDNl2modDISABLED              1       /* No layer 2 Protocol (Transparent/Raw Mode) */
#define IISDNl2modLAP_B                 2       /* X.25 data link layer */
#define IISDNl2modBONDING               3       /* Bandwidth-on-Demand Inverse Multiplexing */
#define IISDNl2modLAP_F                 4       /* Q.922 */
#define IISDNl2modDPNSS                 5       /* DPNSS 1, digital private network signalling */
#define IISDNl2modLAP_D_EFA             6       /* Q.921 data link layer w/Envelope Function Address (4 bytes) */
#define IISDNl2modPM                    7       /* ANSI T1.403-1995 */
#define IISDNl2modSS7                   8       /* ANSI T1.111 & ITU */
#define IISDNl2modUDP_IP                9       /* Internet Protocol over IISDNdevETHERNET */
#define IISDNl2modDASS                  10      /* DASS 2, digital access signalling system */
#define IISDNl2modV110          11  /* v110, software version, ie. not dsp */
#define IISDNl2modV120                  12      /* V120 mode - synchronous HDLC mode */
#define IISDNl2modLAP_F_CORE    13      /* Q.922 Annex A */

#define IISDNstatL2                             0       /* L3L4mL2_STATS response mode for Lap */
#define IISDNstatMTP2                   1       /* L3L4mL2_STATS response mode for SS7 */

#define IISDNdirUSER_SIDE               0       /* DTE (the customer premise equipment) */
#define IISDNdirNETWORK_SIDE    1       /* DCE (the network) */
#define IISDNdirSYMMETRIC               2       /* Symmetric/V.120/Q922 invalid on LAP-B */

typedef struct 
        {
        IISDNu8bit mode;                        /* IISDNl2modLAP_X */
        IISDNu8bit dce_dte;                     /* IISDNdirXXX */
        IISDNu8bit TEI_mode;            /* 1 => dynamic TEI assignment, for BRI ONLY */
        IISDNu8bit no_sabme;            /* 1 => TEI_ASSIGNED state/no SABM/SABME sent */
        IISDNu8bit L2_detail;           /* bit map */
                                                                /* 0x01 => Send status on all error indications */
                                                                /* 0x02 => Don't send status when SABME xmitted in TEI ASSIGNED state */
        IISDNu8bit timestamp;           /* 1 => prefix 32-bit timestamp hdr to rx pkt */
        IISDNu8bit ui_mode;                     /* 1 => Unnumbered information frame mode */
        IISDNu8bit priority;            /* 0 => lowest priority, 255 => highest prio */
        IISDNu8bit no_reestab;          /* 1 => go to TEI-UNASSIGNED on tx/rx SABME */
        IISDNu8bit mode_1tr6;           /* 1 => 1TR6 LAP-D error handling, not ITU (set automatically in Q.931 1TR6 modes) */
        IISDNu8bit mode_TEI_1;          /* 1 => use custom TEI mode for LAPD over FDL */
        IISDNu8bit no_piggyback;        /* 1 => Always use RR to ack. Do not pigyback (This is required for K1403 BRI tester) */
        } IISDN_L2_LAP_PARAMS;

typedef struct
        {
        IISDNu16bit     t200;   /* X.25 T1. Max retrans time. Default 1 sec */
        IISDNu16bit     t201;   /* X.25 T2. Min Retrans TEI Identity Check */
        IISDNu16bit     t202;   /* Min TEI Identity Request.  Default 2 sec. */
        IISDNu16bit     t203;   /* X.25 T3. Max link inactivity Default 30 sec */
        IISDNu16bit     n200;   /* X.25 N2. Max retrans.  Default 3 */
        IISDNu16bit     n201;   /* X.25 N1. Max bytes in Iframe  */ 
        IISDNu16bit n202;       /* The number of TEI assignment requests */ 
        IISDNu16bit     K;              /* Max Transmit Window.  Default 7.  Max 127. */
        } IISDN_L2_LAP_CONSTS;

typedef struct 
        {
        IISDNu8bit mode;                /* IISDNl2modSS7 */
        IISDNu8bit variant;     /* 0 =>ANSI, 1 => ITU */
        IISDNu16bit pad;
        IISDNu8bit pad2[8];
        } IISDN_L2_SS7_PARAMS;

typedef struct
        { /* ss7 mtp2 timer values are specified in 10 millisecond tics */
        IISDNu16bit t1;         /* aligned/ready timer */
        IISDNu16bit t2;         /* not aligned timer */
        IISDNu16bit t3;         /* aligned timer */
        IISDNu16bit t4n;        /* normal proving period timer */
        IISDNu16bit t4e;        /* emergency proving period timer */
        IISDNu16bit t5;         /* sending SIB timer */
        IISDNu16bit t6;         /* remote congestion timer */
        IISDNu16bit t7;         /* excessive delay of ack timer */
        } IISDN_L2_SS7_CONSTS;

typedef struct 
        {
        IISDNu8bit      mode;           /* IISDNl2modUDP_IP */
        IISDNu8bit      pad;
        IISDNu16bit     dstPort;        /* udp destination port */
        IISDNu32bit     dstIpAddr;      /* destination IP address */
        IISDNu8bit      pad2[4];
        } IISDN_L2_UDPIP_PARAMS;

typedef struct
        {                                                       /* IISDNl2modUDP_IP */
        IISDNu8bit      no_dhcp;                /* 1 => don't use dhcp to get local ip address */
        IISDNu8bit      pad[3];
        IISDNu32bit     ipAddr;                 /* IP address when no_dhcp == 1 */
        IISDNu32bit     gwAddr;                 /* default gateway IP address when no_dhcp == 1 */
        IISDNu32bit     subnet_mask;    /* IP subnet mask when no_dhcp == 1 */
        } IISDN_L2_IP_CONSTS;

typedef struct 
        {
        IISDNu8bit      mode;                   /* IISDNl2modDPNSS or IISDNl2modDASS */
        IISDNu8bit      pbx_B;                  /* for DPNSS: 0 => pbx-A, 1 => pbx-B, for DASS2: 0 => ET, 1 => PBX */
        IISDNu8bit  sabmr_as_ack;       /* 1 => accept rx sabmr as ack */
        IISDNu8bit  tie_line_mode;      /* 1 => DASS tie-line on UI msgs (field is ignored for DPNSS) */
        } IISDN_L2_DPNSS_PARAMS;

typedef struct 
        {                                                       /* IISDNl2modDPNSS and IISDNl2modDASS */
        IISDNu16bit NL;                         /* min retrans count, 0 => 64 */
        IISDNu16bit pad;
        /* DPNSS timer values are set in milliseconds */
        /* NT1 and NT2 should be set to 0 for IISDNl2modDASS */
        IISDNu32bit NT1;                        /* min retrans periot, 0 => 500 msecs */
        IISDNu32bit NT2;                        /* min post trans ack delay, 0 => 0 */
        } IISDN_L2_DPNSS_CONSTS;

typedef struct 
        {
        IISDNu8bit mode;                        /* IISDNl2modV110 */
        IISDNu8bit ebits;                       /* Ebit settings in idle frames:
                                                                        E7=msb,E1=lsb default=0xfd */ 

        IISDNu8bit flow_control;        /* flow control transmitter based on received
                                                                        Xbit (second X-bit).
                                                                        1=flow control enabled, 0=disabled */
                                                
        IISDNu8bit nine_byte_rx_frames; /* 0=10 byte receive frames data channel buffers
                                                                           1=9 byte receive frames with octet0 removed */
        IISDNu16bit num_tx_idle_frames;
                /* number of idle frames to xmit before checking for host transmit data */
        IISDNu16bit max_rx_frame_size;
                /* number of data frames to buffer before sending to host */

        IISDNu32bit stale_rx_data_timer;
            /* amount of time (in msecs) to wait for more receive data before sending
               accumulated receive data to host */
        } IISDN_L2_V110_PARAMS;


typedef struct
        {
        union {
                IISDN_L2_LAP_PARAMS     lap;
                IISDN_L2_UDPIP_PARAMS   udpip;
                IISDN_L2_DPNSS_PARAMS   dpnss;
                IISDN_L2_SS7_PARAMS             mtp2;
                IISDN_L2_V110_PARAMS    v110;
                } par;
#ifdef RELEASE_8
#else
        IISDN_DATA_INTERFACE data_interface;
#endif
        union {
                IISDN_L2_LAP_CONSTS     l2;
                IISDN_L2_IP_CONSTS              ip;
                IISDN_L2_DPNSS_CONSTS   dpnss;
                IISDN_L2_SS7_CONSTS     mtp2;
                } consts;
        } IISDN_LEVEL2_CNFG;

#define IISDNstATT_4ESS         0x00    /* AT&T #4 ESS (Default) */
#define IISDNstATT_5ESS         0x01    /* AT&T #5 ESS */
#define IISDNstNTI_DMS100       0x02    /* Northern Telecom DMS-100 */
#define IISDNstNTI_DMS250       0x03    /* Northern DMS-250 */
#define IISDNstMD110_T1         0x04    /* Ericsson MD-110 (North America) */
#define IISDNstMD110_E1         0x05    /* Ericsson MD-110 (International) */
#define IISDNstSIEMENS          0x06    /* Siemens (North America) */
#define IISDNstNTT              0x07    /* Japan */
#define IISDNstUNKNOWN          0x08    /* Unknown/ITU conformant */
#define IISDNstMAX_SWITCH_TYP   0x08

#define IISDNvarATT_CUSTOM      0x00    /* AT&T Pub 41449 (Default) */  
#define IISDNvarNTI_CUSTOM      0x01    /* Northern Telecom NIS A211-1 */
#define IISDNvarNATL_ISDN_1     0x02    /* BRI only, US National ISDN 1 */
#define IISDNvarNATL_ISDN_2     0x03    /* Bellcore TR 1268 */
#define IISDNvarJATE            0x04    /* Japan INS-1500 */
#define IISDNvarCTR3            0x05    /* BRI only, Euro-ISDN */
#define IISDNvarNET3            0x05    /* BRI only, Euro-ISDN, old name */
#define IISDNvarCTR4            0x06    /* PRI only, Euro-ISDN */
#define IISDNvarNET5            0x06    /* PRI only, Euro-ISDN, old name */
#define IISDNvar1TR6_BRI        0x07    /* BRI only, Germany (unsupported) */
#define IISDNvar1TR6_PRI        0x08    /* PRI only, Germany (unsupported) */
#define IISDNvarVN3             0x09    /* France, pre-Euro-ISDN */
#define IISDNvarITU             0x0A    /* Generic ITU Q.931 Specification */
#define IISDNvarCCITT           0x0A    /* Generic CCITT Q.931 Spec, old name */
#define IISDNvarQ933            0x0B    /* Q.933 Frame Relay Forum Impl Agree */
#define IISDNvarQ933_T123       0x0C    /* Q.933 variant for T.123 */
#define IISDNvarTS014           0x0D    /* Austrailian PRI (TS014) */
#define IISDNvarTS013           0x0E    /* Austrailian BRI (TS013) */
#define IISDNvarARINC_746       0x0F    /* ARINC 746 SatCom Specification */
#define IISDNvarGR_303          0x10    /* GR-303 Common Signaling Channel (CSC) */
#define IISDNvarGR_303_TMC      0x11    /* GR-303 Time Slot Management Channel (TMC) */
#define IISDNvarMAX_VARIANT     0x11

#define IISDN_E1_CH16_OK        2       /* For e1_30_bchan, can use bchannel 16 */

#define IISDNvarJATE_REDIAL_3IN3_MINS    0 /* JATE Redial method 3 in 3 minutes    */
#define IISDNvarJATE_REDIAL_15X          1 /* JATE Redial method 15 times          */
#define IISDNvarJATE_REDIAL_NO_RESTRICT  2 /* JATE no redial restrictions          */

/* For backward compatibility */
#define IISDN_JATE_REDIAL_3IN3_MINS      IISDNvarJATE_REDIAL_3IN3_MINS
#define IISDN_JATE_REDIAL_15X            IISDNvarJATE_REDIAL_15X
#define IISDN_JATE_REDIAL_NO_RESTRICT    IISDNvarJATE_REDIAL_NO_RESTRICT

typedef struct
        {
        IISDNu16bit     switch_type;                            /* Connected equipment.  Use IISDNstXXX   */
        IISDNu16bit     variant;                                        /* Q.931 Variant. Use IISDNvarXXX                 */
        IISDNu32bit     call_filtering;                         /* Call filtering on bearer capibility.   */
                                                                                        /* Reject incoming call if bit not set in */
                                                                                        /* IISDNcalltypXXX bit mask                               */
        IISDN_Q931_TIMERS q931_timers;
        IISDNu32bit     b_channel_service_state[IISDN_NUM_DS1_INTERFACES];
                                                                                        /* 0 => out-of-service 1 =>in-service     */
                                                                                        /* Bit position is (1 << bchannel)                */
                                                                                        /* Least significant bit not used.                */
                                                                                        /* T1/PRI 1..23 in-service use 0x00FFFFFE */
                                                                                        /* T1/PRI 1..24 (nfas) use 0x01FFFFFE     */
                                                                                        /* E1/PRI 1..31 in-service use 0xFFFEFFFE */
                                                                                        /* Single PRI interfaces (non-NFAS) use   */
                                                                                        /*  field b_channel_service_state[0]      */
        IISDNu8bit      nfas;                                           /* 1 => Non-Facilities Assoc Signaling    */
        IISDNu8bit      e1_30_bchan;                            /* 0 => Call control for 24 b channels    */
                                                                                        /* 1 => Call Control for 30 b channels    */
                                                                                        /* 2 => (use IISDN_E1_CH16_OK constant)   */
                                                                                        /*      31 b channels including chan 16   */
        IISDNu8bit      basic_rate;                                     /* 1 => ISDN Basic Rate                                   */
        IISDNu8bit      net_side_emul;                          /* 1 => Network Side Protocol Emulation   */
        IISDNu8bit      b_chan_negot;                           /* 1 => Incoming bchannel negotiation     */
        IISDNu8bit      proc_on_exclusv;                        /* 1 => Tx CALL_PROCEEDING if channel     */
                                                                                        /*              id on incoming call is exclusive  */
                                                                                        /*              and b_chan_negot selected                 */
        IISDNu8bit      chanid_slot_map;                        /* 1 => encode channel id using slot map  */
                                                                                        /*              rather than channel number                */
        IISDNu8bit      sprs_chanid_callproc;           /* 1 => supress Channel ID info element   */
                                                                                        /*              in the CALL_PROCEEDING message    */
        IISDNu8bit      no_chanid_callproc;                     /* 1 => Accept missing Channel ID ie      */
                                                                                        /*              in the CALL_PROCEEDING message    */
        IISDNu8bit      append_raw_qmsg;                        /* 1 => Append undecoded IFRAME messages  */
                                                                                        /*              to all call control messages      */
        IISDNu8bit      ccitt_mode;                                     /* 1 => ITU compatibility mode                    */
        IISDNu8bit      raw_qmsg;                                       /* 1 => all L3L4 Q.931 messages undecoded */
                                                                                        /*              and use data.raw_q931_data                */
        IISDNu8bit      no_ie_errcheck;                         /* 1 => No error checking on information  */
                                                                                        /*              elements from network                     */
        IISDNu8bit      user_ie_encode;                         /* 1 => User provides all ie encodings    */
        IISDNu8bit      overlap_rcv;                            /* 1 => Support Overlap Receive                   */
        IISDNu8bit      send_l3l4_callproc;                     /* 1 => Send an L3L4mCALL_PROC_SENT upon  */
                                                                                        /*              transimission of a call proceeding*/
        IISDNu8bit      sending_cmplt;
        IISDNu8bit      require_send_complete;          /* 1 => require sending complete IE       */
                                                                                        /*              to allow incoming call to proceed */
        IISDNu8bit      report_incoming_callproc;       /* Send L3L4mCALL_PROCEEDING on               */
                                                                                        /* receipt of call proceeding             */
        IISDNu8bit      no_tx_conn_ack;                         /* 1 => Do not send Connect Ack                   */
        IISDNu8bit      no_rx_conn_ack;                         /* 1 => Do not wait for Connect Ack               */
        IISDNu8bit      sprs_chanid_setupack;           /* 1 => supress Channel ID info element   */
                                                                                        /*              in the SETUP_ACK message                  */
        IISDNu8bit      no_chanid_setupack;                     /* 1 => Accept missing Channel ID ie      */
                                                                                        /*              in the SETUP_ACK message                  */
        IISDNu8bit      no_canned_spid_rej;                     /* 1 => Pass all 5ESS Custom spid MGMT    */
                                                                                        /*      INF messages up as UNIVERSAL's    */
                                                                                        /* 0 => Send a canned reject message to   */
                                                                                        /*      the net.                          */
        IISDNu8bit      call_reject_notify;                     /* 1 => Send L3L4mCALL_REJECT for         */
                                                                                        /*              incoming calls on OOS channels    */
        IISDNu8bit      advice_of_charge;                       /* 1 = > parse AOC information elevements */
        IISDNu8bit      message_segmentation;           /* 1 = > message segmentation enabled     */
        IISDNu8bit      no_bc_user_info;                        /* 1 => Don't check for missing user info octet (octet 5) */
        IISDNu8bit      incoming_call_slot_map;         /* 1 => Encode outgoing chan id in same format as */
                                                                                        /* that in SETUP for incoming calls */
        IISDNu8bit      release_complete_control;       /* 1 => Give application control of */
                                                                                        /* sending release complete when release is received */
        
        /* The following pertain to D Channel Backup */
#ifdef RELEASE_8
        IISDNu16bit     primary_lapdid;
        IISDNu16bit     backup_lapdid;                          /* 0xFFFF if no backup channel                    */
#else
        IISDNu8bit      pad1;
        IISDNu8bit      pad2;
        IISDNu8bit      primary_lapdid;
        IISDNu8bit      backup_lapdid;                          /* 0xFF if no backup channel                      */
#endif
        IISDNu8bit      primary_ifnum;                          /* Interface number [0..19] with Dchan    */
        IISDNu8bit      backup_ifnum;                           /* Int # [0..19] with Dchan or 0xFF               */
        IISDNu8bit      backup_control;                         /* Used for control of NFAS backup. Use   */
                                                                                        /*      IISDNbcXXXX.  Set to 0 for no change    */
                                                                                        /*      or for non-NFAS case. If set no other */
                                                                                        /*      fields are processed. */
        /* The following pertains to Basic Rate only.  DMS-100 needs 2 service
        profile identifiers because its implementation of National ISDN ONE only 
        allows a device to make one phone call at a time so we have to spoof
        the switch that we're two different telephones. If the user only needs to
        make one call at a time on Northern equipment, only 1 spid is needed */
        IISDNu8bit      spid_len;                                       /* Basic Rate Nat'l ISDN-1 SPID 3..20     */
        IISDNu8bit      spid_1_len;                                     /* Basic Rate Nat'l ISDN-1 SPID 3..20     */
        IISDNu8bit      dn_len;                                         /* Length of directory number */
        IISDNu8bit      dn_1_len;                                       /* Length of directory number for spid_1 */
        IISDNu8bit      spid[IISDN_MAX_SPID_LEN];       /* Service Profile ID (ASCII)     */
        IISDNu8bit      spid_1[IISDN_MAX_SPID_LEN]; /* DMS-100 for 2nd phone call */
        IISDNu8bit      dn[IISDN_MAX_DN_LEN];           /* ASCII Directory number for spid */
        IISDNu8bit      dn_1[IISDN_MAX_DN_LEN];         /* ASCII Dir number for spid_1 */
        
        IISDNu8bit      chan_id_high_bit;                       /* 1 => Set high bit of channel id to 0   */
                                                                                        /*      Necessary on older DMS-250's      */
        IISDNu8bit      att_cust_bri_ekts;                      /* 1 => basic rate supplimentary operation*/
                                                                                        /*              for at&t custom on basic rate */
        IISDNu8bit      subscribe_connack;                      /* 1 => receive L3L4mCONN_ACK_IND */
        IISDNu8bit      suppress_auto_spid;                     /* 1 => don't automatically register spid */
                                                                                        /* 0 => automatically register spid */
        IISDNu8bit      accept_all_bri_calls;           /* 1 => don't check SETUP msg for USID/DN */
                                                                                        /* 0 => check SETUP for proper USID/DN */
        IISDNu8bit      jate_redial_method;             /* specifies JATE redial restriction method */
                                                                                        /* IISDN_JATE_REDIAL_3IN3_MINS -3 redials in 3 mins method */
                                                                                        /* IISDN_JATE_REDIAL_3IN3_MINS -15 times redial  method    */
                                                                                        /* IISDN_JATE_REDIAL_NO_RESTRICT -No redial restriction    */
        IISDNu8bit      pad5;
        IISDNu8bit      pad6;
        } IISDN_Q931_CNFG;

/* Performance Monitoring L4L3mENABLE_PROTOCOL level3.cnfg.pm structure ...
 * The line number associated with an ENABLE_Protocol for PM is determined
 * from the least significant 3 bits of the lapdid used.
 * -- lapdid modulo IISDN_MAX_LINES equals IISDNline num --
 */
typedef struct {
#define IISDNpmmNEAREND         0               /* no FDL for far end, mode available for T1 and E1 */
#define IISDNpmmANSI            1               /* use FDL, follow ANSI T1.403-1995, T1 only */
        IISDNu8bit mode;                                /* use IISDNpmmXXX value */
        IISDNu8bit carrier;                             /* 0->customerInstallation, 1->carrier side */
        IISDNu8bit fdl_alert;                   /* 1->sends all non-stats rx'd fdl packets to L4 */
        IISDNu8bit pad[2];
        IISDNu8bit equipmentID[10];             /* ascii strings for ANSI T1.403 use */
        IISDNu8bit locationID[11];              /* general location, building */
        IISDNu8bit frameID[10];                 /* location within building */
        IISDNu8bit unitID[6];                   /* location within equip bay */
        IISDNu8bit facilityID[38];              /* specific DS1 path */
} IISDN_PM_CONFIG;

/* defines for onboard packet relay and routing service */
typedef struct {
#ifdef RELEASE_8
        IISDNu8bit      default_dest;           /* IISDN_RELAYdestXXXX */
        IISDNu8bit      default_root_idx;       /* offset into received packets for first rule comparison */
        IISDNu16bit     default_dest_id;        /* lid number or dhannel number depending upon destXXXX above */
#else
        IISDNu8bit  default_dest;               /* IISDN_RELAYdestXXXX */
        IISDNu8bit  default_dest_id;    /* lid number or dhannel number depending upon destXXXX above */
        IISDNu8bit  default_root_idx;   /* offset into received packets for first rule comparison */
        IISDNu8bit  pad[1];                             /* maintain long word alignment */
#endif
} IISDN_RELAY_CONFIG;

/* Defines for bonding mode */
#define IISDNbond56K            0x0010
#define IISDNbond64K            0x0000
#define IISDNbondMODE_0         0x0000
#define IISDNbondMODE_1         0x0001
#define IISDNbondINCOMING       0xffff

/* Bonding Destination */
#define IISDNbdestOFF_CARD      0       /* Back out Spyder-T to TSI/MVIP */
#define IISDNbdestTO_HOST       1       /* Data Interface/Raw Superchannel to host */

/* Data structure for Enable_Bonding */
#define IISDN_MAX_BOND_CHAN     30
typedef struct                                          /* for bonding circuits */
        {
        IISDNu16bit     mode;                   /* IISDNbondXX mask 56/64k, mode, incoming */
        IISDNu8bit      destination;    /* IISDNbdestXXX  */
        IISDNu8bit      num_tx_buf;             /* num internal 256-byte tx buf (D:4) */
        IISDNu8bit      num_rx_buf;             /* num internal 256-byte rx buf (D:24) */
#ifdef RELEASE_8
        IISDNu16bit     data_channel;   /* data channel dscr if IISDNbdestTO_HOST */
#else
        IISDNu8bit      data_channel;   /* data channel dscr if IISDNbdestTO_HOST */
#endif

        /* Bonding timers are in tics of 50 milliseconds.  0 uses defaults              */
        IISDNu16bit     txinit;                 /* initalization timer (Default:200)    */
        IISDNu16bit     txadd01;                /* add 1 channel timer (D: 20)                  */
        IISDNu16bit     txfa;                   /* frame alignment timer (D: 20)                */
        IISDNu16bit     txdisc;                 /* disconnect timer (D: 200)                    */
        IISDNu16bit     txdeq;                  /* delay equalization timer (D: 200)    */
        IISDNu16bit     tcid;                   /* calling id timer (D: 200)                    */
        IISDNu16bit     tanull;                 /* answering null timer (D:200)                 */
        IISDNu32bit     channels;               /* bonding channel bit map */
        IISDNu32bit     directory[IISDN_MAX_BOND_CHAN];
                                                                        /* additional channel number directory */
        } IISDN_BONDING_DATA;

/* Structure for L4L3mADD_BONDING.  Sent when each 64K call is answered */
typedef struct
        {
#ifdef RELEASE_8
        IISDNu16bit     addon_channel;  /* Spyder-T channel number */
   IISDNu16bit pad;
#else
        IISDNu8bit      addon_channel;  /* Spyder-T channel number */
   IISDNu8bit pad[3];
#endif
        } IISDN_ADD_BOND_DATA;

/* Defines for bonding state */
#define IISDNbonsENABLED                0               /* Acknowlege ENABLE_BONDING */
#define IISDNbonsDNX_EXCHANGE   1               /* Directory list exchange complete */
#define IISDNbonsIN_SYNC                2               /* Aggregate Call has cut thru */
#define IISDNbonsNO_SYNC                3               /* Aggregate Call has cleared */
#define IISDNbonsNEED_DIRLIST   4               /* Directory list is required */
#define IISDNbonsADDON_ENABLED  5               /* Channel has been added to port */

/* Clearing causes for Bonding Status state = IISDNbonsNO_SYNC */
#define IISDNboncNO_CAUSE                                       0       /* For state != IISDNbonsNO_SYNC */
#define IISDNboncHOST_CLEARING                          1       /* Host-induced clearing */
#define IISDNboncEXCESS_RELATIVE_DELAY          2       /* Exceed max relative delay */
#define IISDNboncLOST_SYNC                                      3       /* Lost synch with other end */
#define IISDNboncINCOMPATIBLE_DEST                      4       /* Ans unit attempted param negot */
#define IISDNboncDIRLIST_ERROR_1                        5       /* Directory list error, 1st num */
#define IISDNboncINCOMPATIBLE_SRC                       6       /* Calling unit incompatible */
#define IISDNboncCHANNEL_TIMER_EXPIRATION       7       /* Timer expiration */
#define IISDNboncCALL_TIMER_EXPIRATION          8       /* Timer expiration */
#define IISDNboncHOST_TX_Q_EMPTY                        9       /* Host->network tx queue empty */
#define IISDNboncHOST_RX_Q_FULL                         10      /* Host->network rx queue full */
#define IISDNboncHOST_BAD_BUFFER_LEN            11      /* Buffer len != 256 * num chan */

/* Data structure for L3L4mBONDING_STATUS and L3L4mADD_BONDING */
typedef struct
        {
        IISDNu8bit      state;                                          /* IISDNbonsXXX */
        IISDNu8bit      num_channels;                           /* number of aggr. channels */
        IISDNu8bit      no_sync_cause;                          /* IISDNboncXXX */
        IISDNu8bit      pad;                                            /* Longword alignment */
        IISDNu16bit     mode;                                           /* IISDNbondXX mask 56/64k, mode0/1 */
        IISDNu16bit     pad1;                                           /* longword alignment */
        IISDNu32bit     list[IISDN_MAX_BOND_CHAN];      /* directory list */
        } IISDN_BONDING_STATUS;

#define IISDNl3modDISABLED              0
#define IISDNl3modQ931                  1
#define IISDNl3modX25_PKT               2
#define IISDNl3modQ933                  3
#define IISDNl3modBONDING               4
#define IISDNl3modPM                    5       /* ANSI T1.403-1995 Performance Monitoring */
#define IISDNl3modRELAY                 6       /* onboard packet relay and routing service */
#define IISDNl3modDPNSS                 7       /* DPNSS 1, digital private network signalling */
#define IISDNl3modDASS                  8       /* DASS 2, digital access signalling system */
#define IISDNl3modQ933_ANNEX_A  9       /* Q.933 Annex A */


typedef struct {
        IISDNu32bit b_channel_service_state;    /* bit field, all 0s == 0xfffefffe */
                                                                                        /* 0 => chan out of service, 1 => chan in service */
        /* T_ timer values are in milliseconds */
        IISDNs32bit     T_digit_racking;        /* 0 default is 4 seconds */
        /* time to wait during overlapped receive processing */
        IISDNs8bit      N_clear_retries;        /* ET side only, 0 => 5 */
        IISDNu8bit      pad[3];
} IISDN_DASSCC_CONFIG;

typedef struct {
        IISDNs8bit pbx_Y;                               /* 0 => pbx-X, 1 => pbx-Y */
        IISDNs8bit no_virtual_channels; /* 0 => virtual channels on, 1 => no virts */
        IISDNu8bit dest_addr_len;               /* Sufficient address length.
                                                                        Number of destination address digits in
                                                                        ISRM_I messages before issuing incoming call
                                                                        message (L3L4mSETUP_IND).
                                                                        0=> IISDN waits for ISRM_C or T_I_MSG timeout
                                                                        !0 => IISDN waits for specifed number of destination
                                                                                address digits or T_I_MSG timeout.
                                                                        A non-zero value can be used for networks that do not issue
                                                                                a ISRM_C message. */
        IISDNu8bit  pad;

        IISDNu32bit b_channel_service_state;    /* bit field, all 0s == 0xfffefffe */
                                                                                        /* 0 => chan out of service, 1 => chan in service */
        IISDNu32bit v_channel_service_state;    /* bit field, all 0s == same as b_channel_service_state */
                                                                                        /* 0 => chan out of service, 1 => chan in service */
        /* T_ timer values are in milliseconds */
        IISDNs32bit     T_I_MSG;                        /* _I msg reconstruction timer, 0 => 10 seconds */
        IISDNs32bit     T_GUARD;                        /* release guard timer, 0 => 10 seconds */
} IISDN_DPNSSCC_CONFIG;

typedef struct {
        IISDNu8bit      network_side; /* Network (1) or User (0) side (FYI: Network side not fully tested) */
        /* Counters */
        IISDNu8bit      n391;   /* Full status polling counter.  Range: 1-255 Default: 6 */
        IISDNu8bit      n392;   /* Error threshold counter.  Range: 1-10 Default: 3 */
        IISDNu8bit      n393;   /* Monitored events counter.  Range: 1-10 Default: 4 */
        /* 50 ms timers */
        IISDNu16bit     t391;   /* Link integrity verification polling timer.  Default 200 (200 x 50ms = 10s) */
        IISDNu16bit     t392;   /* Polling verification polling timer.  Default 300 (300 x 50ms = 30s) */
} IISDN_Q933A_CONFIG;

typedef struct
        {
        IISDNu8bit      l3_mode;        /* IISDNl3modXXX */
        IISDNu8bit      pad;
        IISDNu16bit     pad1;           /* Longword alignment */
        union {
                IISDN_Q931_CNFG                 q931;
                IISDN_BONDING_DATA              bond;
                IISDN_X25_CONFIG                x25;
                IISDN_PM_CONFIG                 pm;
                IISDN_RELAY_CONFIG              relay;
                IISDN_DPNSSCC_CONFIG    dpnss;
                IISDN_DASSCC_CONFIG     dass;
                IISDN_Q933A_CONFIG      q933a;
                } cnfg;
        } IISDN_LEVEL3_CNFG;

/* Commands are used to control protocols which are already configured */
#define IISDNepcmdNO_COMMAND                    0       /* Initial stack configuration */
#define IISDNepcmdDL_ESTABLISH                  1       /* Layer2 DL-ESTABLISH (send SABME) */
#define IISDNepcmdSTART                             1  /* Layer2 DL-ESTABLISH (send SABME) */
#define IISDNepcmdDL_RELEASE                    2       /* Layer2 DL-RELEASE (send DISC) */
#define IISDNepcmdSTOP                      2 
#define IISDNepcmdNEW_HDLC_FLAG_FILL    3       /* Reconfig HDLC flag stuffing */
#define IISDNepcmdPERSISTENT_DEACT              4       /* ii6 INTERNAL msg Layer1->layer2 MPH_DI */
#define IISDNepcmdLINE_ACTIVE                   5       /* ii6 INTERNAL msg Layer1->layer2 MPH_AI */
#define IISDNepcmdTX_HALT                               6       /* NOT IMPLEMENTED */
#define IISDNepcmdTX_RESUME                             7       /* NOT IMPLEMENTED */
#define IISDNepcmdRX_HALT                               8       /* NOT IMPLEMENTED */
#define IISDNepcmdRX_RESUME                             9       /* NOT IMPLEMENTED */
#define IISDNepcmdBULK_ENABLE                   10      /* roughly syncronized raw data streams */
#define IISDNepcmdLOCAL_PROC_OUTAGE             11      /* SS7 mtp2 */
#define IISDNepcmdLOCAL_PROC_RESUME             12      /* SS7 mtp2 */
#define IISDNepcmdEMERGENCY                             13      /* SS7 mtp2 */
#define IISDNepcmdEMERGENCY_CEASES              14      /* SS7 mtp2 */
#define IISDNepcmdRETRIEVE_BSNT                 15      /* SS7 mtp2 */
#define IISDNepcmdRETRIEVAL_REQUEST_AND_FSNC    16      /* SS7 mtp2 */
#define IISDNepcmdFLUSH_BUFFERS                 17      /* SS7 mtp2 */
#define IISDNepcmdCONTINUE                              18      /* SS7 mtp2 */
#define IISDNepcmdCLEAR_BUFFERS             19  /* ss7 mtp2*/
#define IISDNepcmdCLEAR_RTB                     20  /* ss7 mtp2*/

typedef struct
        {
        IISDNu16bit     command;                         /* IISDNepcmdXXX */
        IISDNu16bit     command_parameter;              /* Optional command parameter*/
#ifdef RELEASE_8
        IISDN_DATA_INTERFACE data_interface;
#endif
        IISDN_LEVEL1_CNFG       level1;
        IISDN_LEVEL2_CNFG       level2;
        IISDN_LEVEL3_CNFG       level3;
        } IISDN_ENA_PROTO_DATA;

/* Current maximum hdlc channel is 17 HDLC devices at 32 channels each */
#ifdef RELEASE_8
#define IISDN_MAX_HDLC_CHANNEL          544
#else
#define IISDN_MAX_HDLC_CHANNEL          256
#endif
        
#define IISDN_MAX_LINES 8

/* Mask to select device address for mapping the TSI in the SET_TSI command */
#define IISDNtsiDSi0    0x0000          /* 32 slot MVIP bus highway */
#define IISDNtsiDSi1    0x0100          /* 32 slot MVIP bus highway */
#define IISDNtsiDSi2    0x0200          /* 32 slot MVIP bus highway */
#define IISDNtsiDSi3    0x0300          /* 32 slot MVIP bus highway */
#define IISDNtsiDSi4    0x0400          /* 32 slot MVIP bus highway */
#define IISDNtsiDSi5    0x0500          /* 32 slot MVIP bus highway */
#define IISDNtsiDSi6    0x0600          /* 32 slot MVIP bus highway */
#define IISDNtsiDSi7    0x0700          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo0    0x0800          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo1    0x0900          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo2    0x0a00          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo3    0x0b00          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo4    0x0c00          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo5    0x0d00          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo6    0x0e00          /* 32 slot MVIP bus highway */
#define IISDNtsiDSo7    0x0f00          /* 32 slot MVIP bus highway */
#define IISDNtsiLINE_A  0x1000          /* Network Interface ports */
#define IISDNtsiLINE_B  0x1100          
#define IISDNtsiLINE_C  0x1200
#define IISDNtsiLINE_D  0x1300
#define IISDNtsiLINE_E  0x1400
#define IISDNtsiLINE_F  0x1500
#define IISDNtsiLINE_G  0x1600
#define IISDNtsiLINE_H  0x1700
                               /* ISA        PCI        CPCI-V     CPCI-D     CPCI-D    */
                               /* ---------  ---------  ---------  ---------  --------- */
#define IISDNtsiHDLC_0  0x2000 /* LuT7115    68mh360    mpc860mh   Bt8474(1)  Bt8478(1) */
#define IISDNtsiHDLC_1  0x2100 /* LuT7115    LuT7115    n/a        Bt8474(1)  Bt8478(1) */
#define IISDNtsiHDLC_2  0x2200 /* n/a        n/a        n/a        Bt8474(1)  Bt8478(1) */
#define IISDNtsiHDLC_3  0x2300 /* n/a        n/a        n/a        Bt8474(1)  Bt8478(1) */
#define IISDNtsiHDLC_4  0x2400 /* n/a        n/a        n/a        Bt8474(2)  Bt8478(1) */
#define IISDNtsiHDLC_5  0x2500 /* n/a        n/a        n/a        Bt8474(2)  Bt8478(1) */
#define IISDNtsiHDLC_6  0x2600 /* n/a        n/a        n/a        Bt8474(2)  Bt8478(1) */
#define IISDNtsiHDLC_7  0x2700 /* n/a        n/a        n/a        Bt8474(2)  Bt8478(1) */
#define IISDNtsiHDLC_8  0x2800 /* n/a        n/a        n/a        mpc860     Bt8478(2) */
#define IISDNtsiHDLC_9  0x2900 /* n/a        n/a        n/a        n/a        Bt8478(2) */
#define IISDNtsiHDLC_10 0x2a00 /* n/a        n/a        n/a        n/a        Bt8478(2) */
#define IISDNtsiHDLC_11 0x2b00 /* n/a        n/a        n/a        n/a        Bt8478(2) */
#define IISDNtsiHDLC_12 0x2c00 /* n/a        n/a        n/a        n/a        Bt8478(2) */
#define IISDNtsiHDLC_13 0x2d00 /* n/a        n/a        n/a        n/a        Bt8478(2) */
#define IISDNtsiHDLC_14 0x2e00 /* n/a        n/a        n/a        n/a        Bt8478(2) */
#define IISDNtsiHDLC_15 0x2f00 /* n/a        n/a        n/a        n/a        Bt8478(2) */
#define IISDNtsiHDLC_16 0x3000 /* n/a        n/a        n/a        n/a        mpc860p   */
#define IISDNtsiMODEM_0 0x6000 /* primary integrated MODEM bank                         */
#define IISDNtsiMODEM_1 0x6100 /* secondary integrated MODEM bank                       */
#define IISDNtsiDISABLE 0x4000
#define IISDNtsiPATTERN 0x5000

#define IISDNtsiDSP_0   0x7000
#define IISDNtsiDSP_1   0x7100
#define IISDNtsiDSP_2   0x7200
#define IISDNtsiDSP_3   0x7300
#define IISDNtsiDSP_4   0x7400
#define IISDNtsiDSP_5   0x7500
#define IISDNtsiDSP_6   0x7600
#define IISDNtsiDSP_7   0x7700

/* Macros for programmatic assignment of various TSI highways. */
#define IISDNtsiDSi(x)          ((x) << 8)                                      /* 8 MVIP DSi highways:   0x0000-0x0700, x=0-7.  */
#define IISDNtsiDSo(x)          (((x)+8) << 8)                          /* 8 MVIP DSo highways:   0x0800-0x0F00, x=0-7.  */
#define IISDNtsiLINE(x)         (0x1000 | ((x) << 8))           /* 8 Line highways:       0x1000-0x1700, x=0-7.  */
#define IISDNtsiHDLC(x)         (0x2000 | ((x) << 8))           /* 17 HDLC highways:      0x2000-0x3000, x=0-16. */
#define IISDNtsiMODEM(x)        (0x6000 | ((x) << 8))           /* 2 Modem highways:      0x6000-0x6100, x=0-1.  */
#define IISDNtsiDSP(x)          (0x7000 | ((x) << 8))           /* 8 DSP highways:        0x7000-0x7700, x=0-7.  */
#define IISDNtsiCTd(x)          (0x8000 | ((x) << 10))          /* 32 H100 bus highways:  0x8000-0xfc00, x=0-31. */

/* Granularity selections. */
#define IISDNtsiGRANULARITY_64K 0x00
#define IISDNtsiGRANULARITY_32K 0x01
#define IISDNtsiGRANULARITY_16K 0x02
#define IISDNtsiGRANULARITY_8K  0x03

#define IISDN_MAX_SET_TSI               120     /* Up to 120 mappings in the SET_TSI cmd */
/* TSI is mapped as <destination> listens to <source> */
typedef struct
        {
        IISDNu16bit     destination;    /* IISDNtsiXXXX + time slot number */
        IISDNu16bit     source;                 /* IISDNtsiXXXX + time slot number */
        } IISDN_TSI_MAP;
/* following defines are for backward compatibility with previous 
 * definition of IISDN-TSI-MAP.  The port-addr and listen-addr fields
 * were harder to intuit.
 */
#define port_address destination
#define listen_address source
typedef struct
        {
        IISDNu16bit num_mappings;       /* Up to IISDN_MAX_SET_TSI */
        IISDNu8bit granularity;         /* IISDNtsiGRANULARITY_XXX */
        IISDNu8bit last;                                /* only used in L3L4mTSI_STATUS */
        IISDN_TSI_MAP   tsi_map[IISDN_MAX_SET_TSI];
        } IISDN_TSI_DATA;
        
/*
 * Definitions for E1 Multiframe Channel Associated Signalling (CAS) over the
 * MVIP bus for PRI-ISA64.  Read/write ABCD bits in timeslots [1..15],[16..31] 
 * of MVIP line DS*_* where channel 1 ABCD bits go into the low 4 bits of
 * timeslot 1, etc.  Timeslots 0 and 16 are not touched.
 *
 * For T1, ABCD signalling goes in low 4 bits of MVIP timeslots [0..23].  
 * For D4 framing (Superframe), C and D bits are not used.
 */
/* Mappings for PRI-ISA64 MVIP Bus */
#define IISDN_DSi_0             0x00    /* MVIP Stream DSi0 */
#define IISDN_DSi_1             0x01    /* MVIP Stream DSi1 */
#define IISDN_DSi_2             0x02    /* MVIP Stream DSi2 */
#define IISDN_DSi_3             0x03    /* MVIP Stream DSi3 */
#define IISDN_DSi_4             0x04    /* MVIP Stream DSi4 */
#define IISDN_DSi_5             0x05    /* MVIP Stream DSi5 */
#define IISDN_DSi_6             0x06    /* MVIP Stream DSi6 */
#define IISDN_DSi_7             0x07    /* MVIP Stream DSi7 */
#define IISDN_DSo_0             0x08    /* MVIP Stream DSo0 */
#define IISDN_DSo_1             0x09    /* MVIP Stream DSo1 */
#define IISDN_DSo_2             0x0A    /* MVIP Stream DSo2 */
#define IISDN_DSo_3             0x0B    /* MVIP Stream DSo3 */
#define IISDN_DSo_4             0x0C    /* MVIP Stream DSo4 */
#define IISDN_DSo_5             0x0D    /* MVIP Stream DSo5 */
#define IISDN_DSo_6             0x0E    /* MVIP Stream DSo6 */
#define IISDN_DSo_7             0x0F    /* MVIP Stream DSo7 */

/* The following two structures are NOT exclusive to MVIP and are used 
        to view and change the tx/rx of cas data in shared memory */
typedef struct
        {
        IISDNu8bit tx[32];              /* as 0000ABCD */
        IISDNu32bit rx[32];     /* Low byte has new sample, high byte has oldest */
        } IISDN_CAS_LINE_DATA;

typedef struct
        {
        IISDN_CAS_LINE_DATA line[4];
        } IISDN_CAS_DATA;
/* Bit mask line defines for cas to memory (use lli field in common header). 
        Bit set to 1 is on, bit set to 0 turns it off.  Use L4L3mCAS_TO_MEMORY. */
#define IISDNcasLINE0   0x01
#define IISDNcasLINE1   0x02
#define IISDNcasLINE2   0x04
#define IISDNcasLINE3   0x08

/* Starting address for cas to memory data structs */
#define IISDN_CAS_DATA_OFFSET   0X00FC80

typedef struct
        {
        IISDNu8bit      timeslot;
        IISDNu8bit      stream;
//   IISDNu16bit pad; not nessary; aligned in union below
        } IISDN_CAS_STREAM;
#define cas_upper_byte timeslot

typedef struct
        {
        IISDNu8bit      state;          /* 0 => Disabled, 1 => Enabled */
        IISDNu8bit      pad1;           /* Longword boundary. */
        IISDNu8bit      pad2;           /* Longword boundary. */
        IISDNu8bit      pad3;           /* Longword boundary. */
        IISDNu32bit     ts_mask;        /* timeslot mask. Bit 0 is timeslot 0 */
                                                        /* 0 => use all appropriate timeslots */
                                                        /* (Default 0xFFFEFFFE for E1, 0xFFFFFF for T1) */
        union
        {
                IISDN_CAS_STREAM        address1;
                IISDNu16bit     address;
        } destination;
        union
        {
                IISDN_CAS_STREAM        address1;
                IISDNu16bit     address;
        } source;
        } IISDN_CAS_CONTROL;

typedef struct
        {
        IISDN_CAS_CONTROL       line[IISDN_MAX_LINES];  /* Indexed by IISDNlineA or IISDNlineB */
        } IISDN_EBUS_CAS;

/* For backwards compatibility. */
#define CSTo destination.address1.stream
#define CSTi1 source.address1.stream
#define IISDN_MVIP_CAS IISDN_EBUS_CAS
#define mvip_cas_data ebus_cas_data

/*
 * Definitions for E1 Multiframe Channel Associated Signalling (CAS) read at
 * the user's request using L4L3mREQ_ABCD_DATA.
 * Timeslots 1 to 15, 17 to 31  ABCD bits are read from the framer and copied
 * in chars [1..15],[17..31] in the abcd[] array. Where channel 1 ABCD
 * bits go into the low 4 bits of timeslot 1, etc.  Timeslots 0 and 16 are
 * reserved.
 *
 * For T1, ABCD signalling goes in low 4 bits of the abcd[0..23] struct.  
 * For D4 framing (Superframe), C and D bits are not used.
 */
typedef struct
        {
        IISDNu8bit      abcd[32];       /* channel signalling bits lower byte */
        } IISDN_ABCD_SIGNALS;

/* Data structure for L3L4mABCD_SIGNAL_DATA */
typedef struct
        {
        IISDN_ABCD_SIGNALS line[IISDN_MAX_LINES];       /* Indexed by IISDNlineA or IISDNlineB */
        } IISDN_ABCD_DATA;

/* Data structure for L3L4mCAS_SIGNALING_BITS_STATUS */
#define CAS_EVENT_RX_CHANGE                         0x00
#define CAS_EVENT_TX_CHANGE                         0x01
#define CAS_EVENT_SPARE_BITS_CHANGE                 0x02

#define CAS_COMMAND_SET_TX_BITS                     0x00
#define CAS_COMMAND_DISABLE_RX_NOTIFICATION         0x01
#define CAS_COMMAND_DISABLE_SET_TX_BITS             0x02
#define CAS_COMMAND_DISABLE_TX_NOTIFICATION         0x03
#define CAS_COMMAND_ENABLE_RX_NOTIFICATION          0x05
#define CAS_COMMAND_ENABLE_SET_TX_BITS              0x06
#define CAS_COMMAND_ENABLE_TX_NOTIFICATION          0x07

#define CAS_SPARE_BITS_CHANNEL                      0xFF

typedef struct
        {
        IISDNu8bit    command;
        IISDNu8bit    bchannel;
        IISDNu8bit    tx_bits;
        IISDNu8bit    rx_bits;
        } IISDN_CAS_SIGNALING_BITS;

#define IISDN_MAX_CAS_SIGNALING_BITS_ENTRIES       50

typedef struct
        {
        IISDNu8bit                  n_entries;
        IISDN_CAS_SIGNALING_BITS    entry[IISDN_MAX_CAS_SIGNALING_BITS_ENTRIES];
        } IISDN_CAS_SIGNALING_BITS_DATA;

typedef struct 
    {
    IISDNu16bit     duration;
    IISDNu16bit     pad;                
    } IISDN_SIGNAL_DURATION_DATA;

/* 
 * Useful macros to set bit fields in IISDN_ABCD_SIGNALS
 *
 * The format used to store the signaling bits is 
 *     TTTTRRRR
 * where TTTT are the 4 transmitted bits (upper 4 bits of the byte)
 * and   RRRR are the 4 received bits    (lower 4 bits of the byte)
 *
 */
#define SET_RXTX_BITS(rx,tx)   ((unsigned char) (((tx) << 4) | (rx)))
#define GET_RX_BITS(rxtx)      ((unsigned char) ((rxtx) & 0x0F))
#define GET_TX_BITS(rxtx)      ((unsigned char) (((rxtx) >> 4) & 0x0F))


/*
 * Definitions for encoded alarm (fault) bits for MVIP_CAS application
 *
 * These bits will be set in the MVIP CAS data stream
 *
 * Bits 0-3 are for ABCD signalling
 * Bits 4-7 are for these faults
 *
 */
#define IISDNcasF1              0x10    /* First CAS Fault bit - OOF/LOS/Red */
#define IISDNcasF2              0x20    /* Second CAS Fault bit - AIS/CRC */
#define IISDNcasF3              0x40    /* Third CAS Fault bit - Yellow/RFA/RMFA */
#define IISDNcasF4              0x80    /* Fourth CAS Fault bit - unassigned */


/* Data structure for configuring line and hardware parameters */
/* Framing types for T1 boards */
#define IISDNfrmESF                                     0               /* The Default: Extended Superframe */
#define IISDNfrmRESRVD                          1               /* Reserved for future use */
#define IISDNfrmSF_SLC96                        2               /* Superframe SLC 96 */
#define IISDNfrmSF                                      3               /* Superframe */

/* Framing types for E1 boards */
#define IISDNfrmFEBE                            0               /* CRC4 multiframing with Si=FEBE */
#define IISDNfrmCRC4                            0               /* CRC4 multiframing with Si=FEBE*/
#define IISDNfrmBASIC                           2               /* basic framing with no CRC4, Si=1 */
#define IISDNfrmFEBE_CAS                        3               /* FEBE w/ TS16 channel associated signaling */
#define IISDNfrmCRC4_CAS                        3               /* CRC4 w/ TS16 channel associated signaling */
#define IISDNfrmBASIC_CAS                       5               /* basic w/ TS16 channel associated signaling */
#define IISDNfrmDISABLED                        6               /* Framing on timeslot 0 is disabled */
#define IISDNfrmTS014_FEBE                      7               /* Si=FEBE=0 when not in frame alignment */

/* Line coding types for T1 boards */
#define IISDNcodB8ZS                            0               /* The Default: Binary 8 zero substitution */
#define IISDNcodB7ZS                            1               /* Bit 7 zero supression (not supported on Bt8370 framer) */

/* Line coding types for E1 boards */
#define IISDNcodHDB3                            0               /* The Default: Binary 3 zero substitution */

/* Line coding types for both T1+E1 boards */
#define IISDNcodTRANSPARENT                     2               /* AMI coding */

/* DSX-1 Pre-equalization: IISDNlen133ft is default */
#define IISDNlen133ft                           0               /*   0-133 ft */
#define IISDNlen266ft                           1               /* 133-266 ft */
#define IISDNlen399ft                           2               /* 266-399 ft */
#define IISDNlen533ft                           3               /* 399-533 ft */
#define IISDNlen655ft                           4               /* 533-655 ft */
#define IISDNlenLOCAL_LOOPBACK          0x80    /* Local Loopback */
#define IISDNlenREMOTE_LOOPBACK         0x40    /* Remote Loopback */
#define IISDNlenCLEAR_LOOPBACK          0x20    /* Clear Loopback */
#define IISDNlenPAYLOAD_LOOPBACK        0x10    /* Payload Loopback */

/* DS1 line build out: IISDNlbo15_0db is default */
#define IISDNlbo15_0db                          0               /*   -15.0 db */
#define IISDNlbo07_5db                          1               /*    -7.5 db */
#define IISDNlbo00_0db                          2               /*     0.0 db */
#define IISDNlbo22_5db                          4               /*   -22.5 db */
#define IISDNlboLOCAL_LOOPBACK          0x80    /* Local Loopback */
#define IISDNlboREMOTE_LOOPBACK         0x40    /* Remote Loopback */
#define IISDNlboCLEAR_LOOPBACK          0x20    /* Clear Loopback */
#define IISDNlboPAYLOAD_LOOPBACK        0x10    /* Payload Loopback */

#define IISDNlineA                      0       /* default BRI/cPCI recovery line */
#define IISDNlineB                      1       /* default PRI recovery line */
#define IISDNlineC                      2
#define IISDNlineD                      3
#define IISDNlineE                      4
#define IISDNlineF                      5
#define IISDNlineG                      6
#define IISDNlineH                      7

/* TX Clocking */
#define IISDNclkRECOVER_A               1       /* use clock of 1st line, BRI & CPCI default */
#define IISDNclkRECOVER_B               0       /* use clock of 2nd line, PCI-PRI default */
#define IISDNclkRECOVER_C               4   /* use clock of 3rd line */
#define IISDNclkRECOVER_D               5   /* use clock of 4th line */
#define IISDNclkRECOVER_E               6       /* */
#define IISDNclkRECOVER_F               7       /* */
#define IISDNclkRECOVER_G               8   /* */
#define IISDNclkRECOVER_H               9   /* */
#define IISDNclkINTERNAL                2       /* use internal clocking */
#define IISDNclkEXTERNAL                3       /* use external highway clocking (MVIP, CTBus)*/

/* Masks to modify clocking choices for MVIP, SCSA, or H.110 Bus */
#define IISDNclkTDM_MASTER              0x80    /* Provide tdm highway clocks */
#define IISDNclkTDM_REFERENCE   0x40    /* Provide tdm highway reference clock */
#define IISDNclkCT_C8_A                 0x80    /* provide H.110 "A" clock for system */ 
#define IISDNclkCT_C8_B                 0x20    /* provide H.110 "B" clock for system */ 
#define IISDNclkNETREF_1                0x40    /* provide H.110 NETREF_1 for system */ 
#define IISDNclkNETREF_2                0x10    /* provide H.110 NETREF_2 for system */ 

/* Terminations for E1 */
#define IISDNtrmOHM_120                         0x00    /* Default: 120 ohm termination (twisted pair) */
#define IISDNtrmOHM_75                          0x01    /* 75 ohm termination (coax) */
#define IISDNtrmLOCAL_LOOPBACK          0x80    /* Local Loopback */
#define IISDNtrmREMOTE_LOOPBACK         0x40    /* Remote Loopback */
#define IISDNtrmCLEAR_LOOPBACK          0x20    /* Clear Loopback */
#define IISDNtrmPAYLOAD_LOOPBACK        0x10    /* Payload Loopback */

/* BRI level 1 timer 3 values in seconds */
#define IISDN_l1t3_DEFAULT              15
#define IISDN_l1t3_MAX                  30

/* BRI level 1 timer 4 values in milliseconds */
/* note that the designation T4 has been assigned arbitrarily.  ETS 300 012     */
/* (table A.1 page 36) and TBR3 (table 9.9 page 51) use NOTE 4 to specify       */
/* this timer.                                                                                                                          */
#define IISDN_l1t4_MIN                   500
#define IISDN_l1t4_DEFAULT               550
#define IISDN_l1t4_MAX                  1000

/* BRI l1mode values for set hardware command */
#define IISDN_l1mode_Terminal                           0       /* BRI establish user/terminal node */
#define IISDN_l1mode_Network                            1       /* BRI establish network node */
#define IISDN_l1mode_Terminal_multipoint        2       /* BRI S/T only */
        /* not all Netaccess S/T cards and lines support multipoint mode
        * On the PCI BRI 4 port S/T card only 1 port may be enabled as
        * as multipoint port and that port must be the timing source
        * for the card.
        */


/* BRI L1cmds for activation state machine control */
#define IISDNcmdL1_NULL                         0       /* leave BRI layer 1 just the way it is */
#define IISDNcmdL1_ACTIVATE                     1       /* force activation of BRI layer1 state mach */
#define IISDNcmdL1_DEACTIVATE           2       /* deactivate layer1 BRI */
#define IISDNcmdL1_RESET                        3       /* reset layer1 BRI state machine, and hold */
#define IISDNcmdL1_SSZ                          4       /* send single zeros test pattern */
#define IISDNcmdL1_SCZ                          5       /* send continuous zeros test pattern */
#define IISDNcmdL1_LOOP                         6       /* setup loopback according to IISDNloopBRIxxx */
#define IISDNcmdL1_AUTO_ACTIVATE        7       /* (re)try S/T interface activation */

/* BRI test loopbacks */
#define IISDNloopBRI_NORMAL     0       /* straight through, no loopback */
#define IISDNloopBRI_REMOTE     1       /* remote loopback activated */
#define IISDNloopBRI_LOCAL      2       /* local loopback activated */

/* line types */
#define IISDNline_typeUNKNOWN           0
#define IISDNline_typePRI_T1            1
#define IISDNline_typePRI_T1CSU         2
#define IISDNline_typePRI_E1            3
#define IISDNline_typeBRI_U                     4
#define IISDNline_typeBRI_ST            5

/* T1 Integrated Alarm Modes. */
#define IISDNiamDEFAULT                         0       /* NOTE:  CPCI default is Fast Alarms, PCI default is Accunet. */
#define IISDNiamACCUNET                         1       /* NOTE:  For CPCI, all OTHER values act as default. */
#define IISDNiamFAST_ALARMS                     2       /* NOTE:  For PCI, all OTHER values act as default. */

typedef struct
        {
        IISDNu8bit framing;                             /* IISDNfrmXXXX */
        IISDNu8bit line_code;                   /* IISDNcodXXXX */
        IISDNu8bit pm_mode;                             /* IISDN 5.x ONLY - iisdn 6.x uses L4L3mENABLE_PROTOCOL */
        IISDNu8bit line_length;                 /* IISDNlenXXXX or IISDNlboXXXX*/
        IISDNu8bit term;                                /* IISDNtrmXXXX E1 only */
        IISDNu8bit line_type;                   /* IISDNline_typeXXXX */
        IISDNu8bit integrate_alarms;    /* IISDNiamXXXX - T1 only. */
        IISDNu8bit filter_unsolicited;  /* Allows supression of unsolicited alarms when non-zero */
        IISDNu8bit swap_bri_chann;      /* 0=chip default 1=B1 and B2 swapped on S/T interface
        /* the following pertains only to T1 Superframe (D4) */
        IISDNu8bit filter_yellow;               /* Do not report yellow alarm (T1/SF) */
         /* the following are only applicable to BRI cards */
        IISDNu8bit      bri_l1mode;                     /* BRI: IISDN_l1mode_XXXXX */
        IISDNu8bit      briL1_cmd;                      /* BRI: IISDNcmdL1XXXX layer1 state request     */
        IISDNu8bit      bri_loop;                       /* BRI: remote loop enable, IISDNloopBRIXXXX    */
        IISDNu8bit      briL1_T3;                       /* BRI: activation timer, 0 => default (sec) */
        IISDNu16bit     briL1_T4;                       /* BRI: survive spurious effect timer (ms)      */
        } IISDN_LINE_DATA;

typedef struct
        {
        IISDNu8bit clocking;                    /* IISDNclkXXXX source for onboard timing */
        IISDNu8bit clocking2;                   /* IISDNclkXXXX to use when clocking fails */
        IISDNu8bit enable_clocking2;    /* permit use of clocking2 when clocking fails */
        IISDNu8bit netref_clocking;             /* generate H.1x0 NetRef clock from IISDNclkXXXX */
        IISDNu8bit netref_rate;                 /* 8kHz, 1.544, 2.048MHz enter 0, 1, 2 */
        /*
         * H.1x0 specification allows configuration for other bus standards.
         * This is currently limited to MVIP-90 on PRI-PCI. 
         * Mixed bus systems are not currently supported! 
         */
#define IISDNctbusDEFAULT           0x00                /* generic support (pre IISDN 6.7.59) */
#define IISDNctbusH100_MVIP         0x01                /* H.100 with MVIP-90 support (PCI with HDM only) */
#define IISDNctbusPRIMARY_MASTER    0x02                /* H.110/H.100 primary master support */
#define IISDNctbusSECONDARY_MASTER  0x04                /* H.110/H.100 secondary master support */
#define IISDNctbusSLAVE             0x08                /* H.110/H.100 slave support */
        IISDNu8bit ctbus_mode;                                          /* use single IISDNctbusXXX value - ORing is not allowed */
        IISDNu8bit force_framer_init;                           /* 0=no or 1=yes (set to yes for pre-IISDN 6.7.59 functionality) */
        IISDNu8bit TDM_rate;                                            /* iisdn5.4 only 2,4,8,16 Mb/s enter 0,1,2,3 */
        IISDN_LINE_DATA line_data[IISDN_MAX_LINES];     /* Indexed by IISDNlineX - only applied if changed */
        IISDNu8bit CSU[IISDN_MAX_LINES];                        /* boolean, valid on L3L4mLINE_STATUS */
        IISDNu8bit enable_8370_rliu_monitor;            /* 0=disable, 1=enable software to monitor and adjust receive LIU sensitivity on the 8370 framer. */
        } IISDN_HARDWARE_DATA;

#define IISDN_DS1_CONFIG_DATA IISDN_LINE_DATA

typedef struct    /* Clocking related parameters. */
{
    IISDNu8bit clock_config;                    /* (!!! MUST ALWAYS APPEAR FIRST!!!) Indicates the start of the clock parameters (reserved for internal use only). */
    IISDNu8bit clocking;                        /* IISDNclkXXXX source for onboard timing */
    IISDNu8bit clocking2;                       /* IISDNclkXXXX to use when clocking fails */
    IISDNu8bit enable_clocking2;                /* permit use of clocking2 when clocking fails */
    IISDNu8bit netref_clocking;                 /* generate H.1x0 NetRef clock from IISDNclkXXXX */
    IISDNu8bit netref_rate;                     /* 8kHz, 1.544, 2.048MHz enter 0, 1, 2 */
    IISDNu8bit ctbus_mode;                      /* use single IISDNctbusXXX value - ORing is not allowed */
    IISDNu8bit CLK_status;                      /* CT_C8_A/B status only for T8105, set to 0xff to indicate not supported*/
                                                /* 0x80 indicates CT_C8_A failure while 0x40 indicates CT_C8_B failure */
} IISDN_CLOCK_CONFIG_DATA;

#define IISDN_MAX_SMI_LINES_FOR_CONFIG_DATA     28
/* NOTE The array dimensions for this macro needs to be the largest size   */
/*      needed by any use of the macro.                                    */
typedef struct    /* Miscellaneous parameters. */
{
    IISDNu8bit misc_config;                     /* (!!! MUST ALWAYS APPEAR FIRST!!!) Indicates the start of the miscellaneous parameters (reserved for internal use only). */
    IISDNu8bit force_framer_init;               /* 0=no or 1=yes (set to yes for pre-IISDN 6.7.59 functionality) */
    IISDNu8bit TDM_rate;                        /* iisdn5.4 only 2,4,8,16 Mb/s enter 0,1,2,3 */
    IISDNu8bit enable_8370_rliu_monitor;        /* 0=disable, 1=enable software to monitor and adjust receive LIU sensitivity on the 8370 framer. */
    IISDNu8bit dbcount;                         /* debounce counter for signaling protocol */
    IISDNu8bit enable_t810x_snap_mode;          /* 0=disable, 1=enable T810X snap mode (slide mode is default). */
    IISDNu8bit pad2[2];                         /* Longword alignment. */
    IISDNu8bit CSU[IISDN_MAX_SMI_LINES_FOR_CONFIG_DATA];    /* boolean, valid on L3L4mLINE_STATUS */
} IISDN_MISC_CONFIG_DATA;

/* DS3 defines. */
/* Modes. */
#define IISDNds3modeT3              0   /* T3 mode. */
#define IISDNds3modeE3              1   /* E3 mode. */
#define IISDNds3modeSTS1            2   /* STS-1 mode. */
#define IISDNds3modeAIS             3   /* Unframed AIS. */
#define IISDNds3modeAIS_ANSI        4   /* Unframed AIS per ANSI T1.107. */
#define IISDNds3mode101010          5   /* Unframed 101010... pattern. */
#define IISDNds3modePRBS_2_23       6   /* 2^23 - 1 PRBS pattern. */
#define IISDNds3modePRBS_2_15       7   /* 2^15 - 1 PRBS pattern. */

/* Line Build Out. */
#define IISDNds3lenGE255FT          0   /* Line build out for cable lengths >= 255 feet. */
#define IISDNds3lenLT255FT          1   /* Line build out for cable lengths < 255 feet. */

/* Loopback. */
#define IISDNds3loopCLEAR_LOOPBACK  0   /* Disables all loopbacks. */
#define IISDNds3loopANALOG_LOOPBACK 1   /* Enables analog loopback. */
#define IISDNds3loopREMOTE_LOOPBACK 2   /* Enables remote loopback. */

/* Line Coding. */
#define IISDNds3codeNRZ             0   /* Non-return to zero line coding. */
#define IISDNds3codeB3ZS            0   /* Non-return to zero line coding. */
#define IISDNds3codeHDB3            0   /* Non-return to zero line coding. */
#define IISDNds3codeBIPOLAR         1   /* Bipolar line coding. */

/* Signal Characteristics (Gain and Jitter Attenuation). */
#define IISDNds3condNOJA_NORXGAIN   0   /* No jitter attentuation, no rx gain. */
#define IISDNds3condTXJA_NORXGAIN   1   /* Tx jitter attentuation, no rx gain. */
#define IISDNds3condNOJA_20RXGAIN   2   /* No jitter attentuation, 20dB rx gain. */
#define IISDNds3condTXJA_20RXGAIN   3   /* Tx jitter attentuation, 20dB gain. */
#define IISDNds3condRXJA_NORXGAIN   4   /* Rx jitter attentuation, no rx gain. */

/* DS3 configuration structure. */
typedef struct
{
    IISDNu8bit  mode;                   /* IISDNds3modeXXXX */
    IISDNu8bit  line_length;            /* IISDNds3lenXXXX */
    IISDNu8bit  loopback;               /* IISDNds3loopXXXX */
    IISDNu8bit  coding;                 /* IISDNds3codeXXXX */
    IISDNu8bit  conditioning;           /* IISDNds3condXXXX */
} IISDN_DS3_CONFIG_DATA;

/* Structure for hardware configuration (L4L3mSET_CONFIGURATION/L4L3mGET_CONFIGURATION/L3L4mCONFIGURATION_STATUS) */
#define IISDNcfgtypeCLOCK                  0   /* Used to select clocking level configuration. */
#define IISDNcfgtypeMISC                   1   /* Used to select miscellaneous level configuration. */
#define IISDNcfgtypeDS1                    2   /* Used to select DS1 level configuration. */
#define IISDNcfgtypeLINE                   2   /* Used to select DS1 level configuration. */
#define IISDNcfgtypeDS3                    3   /* Used to select DS3 level configuration. */
#define IISDNcfgtypeLINE_INPUT_GAIN        4   /* Input gain on CODECs */
#define IISDNcfgtypeLINE_OUTPUT_GAIN       5   /* Output gain on CODECs */
#define IISDNcfgtypeSETUP_IND_RING_COUNT   6   /* #of ring cycles to wait for before sending L3L4mSETUP_IND */

typedef struct
{
    IISDNu8bit type;                    /* IISDNcfgtypeXXXX */
    IISDNu8bit line_index;              /* starting line number for the configuration data */
    IISDNu8bit pad[2];                  /* Longword alignment. */
    union
    {
        IISDN_CLOCK_CONFIG_DATA clock;
        IISDN_MISC_CONFIG_DATA  misc;
        IISDN_DS1_CONFIG_DATA   ds1[IISDN_MAX_SMI_LINES_FOR_CONFIG_DATA];
//      IISDN_DS3_CONFIG_DATA   ds3;
        IISDNs32bit             gain;
        IISDNu8bit              setup_ind_ring_count;
    } par;
} IISDN_CONFIG_DATA;

typedef struct
    {
    IISDNu8bit enable_warnings;
    IISDNu8bit pad[3];
    } IISDN_SOFTWARE_DATA;

/* Idle pattern control information for T1.
 * Normally, channels should be transmitting x7F when not in use.
 * Warning: T1 ISDN bchannel 1 is in channel zero (timeslot zero).
 * User may enable/disable pattern on multiple channels via ds0_mask.
 */
typedef struct
        {
        IISDNu8bit      channel;        /* ds0 number.  ignored if ds0_mask nonzero */
        IISDNu8bit      line;           /* IISDNlineA or IISDNlineB */
        IISDNu8bit      pattern;        /* Normally, value on T1 is 0x7F */
        IISDNu8bit      pad;            /* Longword alignment */
        IISDNu32bit     ds0_mask;       /* T1: bit 0 == ds0[0] == bchannel 1 */
        } IISDN_IDLE_PATTERN;

/* Line alarm/status data structures and definitions */
typedef struct
        {
        /* Flags for alarms.  0 => alarm off, not0 => alarm on */
        IISDNu8bit rcv_yellow;  /* Rec remote alarm indication (yellow alarm) */
        IISDNu8bit rcv_blue;    /* Rec alarm indication signal (blue alarm) */
        IISDNu8bit rcv_red;             /* no frame sync (red alarm) */
        IISDNu8bit snd_yellow;  /* 1 => sending RAI (remote alm ind, yellow alm) */
                                                        /* 2 => sending AIS (alm ind sig, blue alm) */
        } IISDN_ALARM_STATUS;

typedef struct
        {
        /* Flags for alarms.  0 => alarm off, 1 => alarm on */
        IISDNu8bit snd_ais;     /* Send alarm indicator signal */
        IISDNu8bit snd_rai;     /* Send remote alarm indication  */
        IISDNu8bit spare1;      /* spare - pad to longword boundary */
        IISDNu8bit spare2;      /* spare - pad to longword boundary */
        } IISDN_ALARM_SENDING;

/* Data structure used to send alarm messages from User to IISDN Controller */
typedef struct
        {
        IISDN_ALARM_SENDING alarm_sending[IISDN_MAX_LINES];     /* Indexed by IISDNlineX */
        } IISDN_ALARM_DATA;

/* l1 line state types */
#define IISDNl1state_F1     0xf1   /* Green */
#define IISDNl1state_F2     0xf2   /* RAI(Remote Alarm Indicator) */
#define IISDNl1state_F3     0xf3   /* LOS(Loss OF Signal) */
#define IISDNl1state_F4     0xf4   /* AIS(Alarm Indicator Signal, All 1's) */
#define IISDNl1state_F5     0xf5   /* RAI & CRC-4 Alarm */ 
#define IISDNl1state_OOF    0x0f   /* OOF(Out Of Frame) */
#define IISDNl1state_FAE    0xae   /* Frame Alignment Error */

typedef struct
        {
        IISDN_ALARM_STATUS      alarm_status[IISDN_MAX_LINES];  /* Index by IISDNlineX */
        IISDNu8bit      l1_state[IISDN_MAX_LINES];                              /* layer 1 line state */
        IISDNs8bit      line_type[IISDN_MAX_LINES];                             /* IISDNline_typeXXXX */
        IISDNs8bit      mph_msg[IISDN_MAX_LINES];                               /* use IISDN_mph_XXX BIT MASK */
        IISDNu8bit error_point;                                                         /* Unique IISDN identifier for location in source code that generated the message */
        } IISDN_LINE_STATUS;

/* L4L3mCOMPARE request (a.k.a. UNIX diff) */
typedef struct
        {
        IISDNu16bit     offset_a;               /* Offset to 64K IISDN Shared Ram */
        IISDNu16bit     offset_b;               /* Offset to 64K IISDN Shared Ram */
        IISDNu16bit     length;                 /* Number of bytes to compare */
        IISDNu16bit                     pad;
        } IISDN_COMPARE_DATA;

/* IISDN 6.x E&M Robbed Bit Signalling uses L4L3mENABLE_CAS, L4L3mDISABLE_CAS
 * with any lapdid value.  The desired IISDNlineX number is determined by
 * line = (lapdid % IISDN_MAX_LINES).
 * Note!!! L4L3mENABLE_B_CHANNEL will no longer start RBS processing under 
 * IISDN 6.x.
 */

/* E&M Trunk Types */
#define IISDNttWINK_START               0 /* Generate/wait-for WINK pulse before dialing */
#define IISDNttDELAY_DIAL               1 /* DELAY_DIAL pulse must clear before dialing */
#define IISDNttIMMEDIATE_DIAL           2 /* Incoming Call; Delay before accepting digts */
#define IISDNttFIXED_PAUSE              2 /* Outgoing Call; Delay before dialing */
/* 4ESS 5ESS Types */
#define IISDNtt4ESS_LPSTART_WINK        3 /* 4ESS Loop Start with wink */
#define IISDNtt4ESS_LPSTART_NOWINK      4 /* 4ESS Loop Start with no wink */
#define IISDNtt4ESS_GDSTART_WINK        5 /* 4ESS Ground Start with wink */
#define IISDNtt4ESS_GDSTART_NOWINK      6 /* 4ESS Ground Start with no wink */
#define IISDNttFXS_LPSTART              7 /* FXS Loop Start, Equiv to 5EES Loop STart */
#define IISDNttFXS_GDSTART              8 /* FXS Ground Start, Equiv to 5EES Ground Start */
#define IISDNttFXO_LPSTART              9 /* FXO Loop Start */
#define IISDNttFXO_GDSTART             10 /* FXO Ground Start */
#define IISDNttEND_OF_TYPES            11 /* End of l4l3menable_b_channel.  MUST stay at bottom of this list */

#define IISDNtt5ESS_LPSTART             IISDNttFXS_LPSTART
#define IISDNtt5ESS_GDSTART             IISDNttFXS_GDSTART

/* Method of transmission of Call Setup Information */
#define IISDNcsmodeNONE                   0x00   /* not sent */
#define IISDNcsmodePULSE                  0x01   /* pulse digits */
#define IISDNcsmodeDTMF                   0x02   /* DTMF digits */
#define IISDNcsmodeR1_MF                  0x04   /* R1 MF digits */
#define IISDNcsmodeFSK                    0x08   /* FSK information */

/* Specification describing Call Setup Information Characteristics */                                          
#define IISDNcidtypeNONE                     0   /* Not defined */
#define IISDNcidtypeBELLCORE              0x0A   /* BellCode (US) standard */
#define IISDNcidtypeNTT                   0x0B   /* NTT (Japan) standard */
#define IISDNcidtypeETSI                  0x0C   /* ETSI 300 778-1 standard */
                                                 
#define IISDNringflgNONE                  0x00   /* do not report ring present/absent transitions */
#define IISDNringflgRAW_CADENCE           0x01   /* report ring present/absent transitions as they happen */
#define IISDNringflgDEGLITCHED_CADENCE    0x02   /* report ring present/absent transitions after deglitched using IISDN_ROBBED_BIT_DATA fields hooktimer_min_ring_on and hooktimer_min_ring_off */

/* Method used to notify terminal CallerID information is about to follow */
#define IISDNcidalertsigNONE                 0   /* Not defined */
#define IISDNcidalertsigFIRST_RING           1   /* First Ring Pattern */
#define IISDNcidalertsigDUAL_TONE            2   /* DualTone */
#define IISDNcidalertsigRINGING_PULSE        3   /* Ringing Pulse */
#define IISDNcidalertsigPOLARITY_REVERSAL    4   /* Polarity */
                                                 
#define IISDNringstatRING_OFF                0   /* No ring present */
#define IISDNringstatRING_ON                 1   /* No ring present */

/* Identifies the possible types of acknowledgment expected  */
/* when initiating outbound calls or receiving inbound calls */
#define IISDNacktypeDEFAULT                  0   /* Use default for protocol type */
#define IISDNacktypeNONE                     1   /* No acknowledgment */
#define IISDNacktypeDELAY                    2   /* Delay */
#define IISDNacktypeWINK                     3   /* Wink */
#define IISDNacktypeOFFHOOK                  4   /* Offhoook */
#define IISDNacktypeLOOP_CURRENT             5   /* Loop Current on the line */
#define IISDNacktypeACTIVATION_ACK_SIGNAL    6   /* Activation Acknowledgment Signal (NTT lines) */

typedef struct {
    IISDNu8bit      Event;            // one of IISDNringflg.
    IISDNu8bit      RingStatus;       // IISDNringstatXxx
    IISDNu8bit      RingCount;        // number of ring cycles (raw cycles if reporting a RAW_CADENCE event, deglitched cycles otherwise)
    IISDNu8bit      pad;              // long word alignment
    IISDNu32bit     pad1;             // long word alignment
} IISDN_RING_STATUS_DATA;

// The following defines are used to parse the Caller ID raw data.
//
#define IISDN_MAX_CALLER_ID_LENGTH           257    // Includes Message Type and Message Length

#define IISDNcidmsgtypeSDMF                 0x04    // Single Data Message Format
#define IISDNcidmsgtypeMDMF                 0x80    // Multiple Data Message Format
#define IISDNcidmsgtypeNTT                  0x40    // Pre-communication Data Notification Service (NTT)

typedef struct {
    IISDNu8bit       MessageType;        // Message type: see IISDNcidmsgtypeXxx  
    IISDNu8bit       MessageLength;      // Message length, excluding MessageType and MessageLength
                                         
    IISDNu8bit       CidNumOffset;       // number byte offset in Caller ID raw data
    IISDNu8bit       CidNumSize;         // number of bytes in Caller ID number
    IISDNu8bit       CidDateOffset;      // date byte offset in Caller ID raw data
    IISDNu8bit       CidDateSize;        // number of bytes in Caller ID date
    IISDNu8bit       CidTimeOffset;      // time byte offset in Caller ID raw data
    IISDNu8bit       CidTimeSize;        // number of bytes in Caller ID time
    IISDNu8bit       CidNameOffset;      // name byte offset in Caller ID raw data
    IISDNu8bit       CidNameSize;        // number of bytes in Caller ID name
    IISDNu8bit       CalledNumberOffset; // number byte offset in DID raw data
    IISDNu8bit       CalledNumberSize;   // number of bytes in DID number
    IISDNu8bit       Reserved[95];       // reserved for future expansion
    IISDNu8bit       CidRawSize;         // number of bytes in Caller ID raw data
    IISDNu8bit       CidRawData[IISDN_MAX_CALLER_ID_LENGTH];  // Caller ID raw data, including Message Type and Message Length
    IISDNu8bit       pad[3];             // used for data alignment only  
} IISDN_CALLER_ID_DATA;

/*
 * definition of DTMF parameters for T1 and E1 trunk configurations 
 */
typedef struct
{
        /* DTMF detection paramaters, comments are defaults */
        IISDNs16bit     minDigitPower;          /* -45 dBm per dtmf frequency */     
        IISDNu16bit     minDigitDuration;       /* 5 blocks (40ms) min on duration */  
        IISDNu16bit     minSilence;             /* 3 blocks (24ms) min off duration */
        IISDNu16bit     hiTwistCutoff;          /* 8 dB max high twist */
        IISDNu16bit     loTwistCutoff;          /* 8 dB max low twist */
        IISDNu16bit     freqTolerance;          /* 1 (+/-)3.5% accept/reject deviation */
        IISDNu16bit     maxFailedSampleBlocks;  /* 6 max number of bad blocks */
        IISDNu16bit     pad;                        /* pad to 32-bit word alignment */    

        /* outbound tone parameters, power relative to system 0dBm level, units are .5dB  */
        IISDNs16bit     tone1Power;   /*  low freq power:  default -14 (-7 dBm) , range [3...-25] */  
        IISDNs16bit     tone2Power;   /*  high freq power: default -14 (-7 dBm) , range [3...-25] */
        IISDNu16bit     makeTime;         /*  default 64 ms, 48-1000 ms */
        IISDNu16bit     breakTime;    /*  default 48 ms, 48-1000 ms */

        /*
         *   Signaling velocity for default make and break time is:
         *              (1000ms/((64ms + 48ms)/digit)) = 8.9 digits/sec
         */
} IISDN_DTMF_DATA;

// Identifies the possible meanings of a loop current reversal signal
// during a call. Notice that the signal can have different meanings
// in inbound and outbound calls
#define IISDNlooprevNONE                    0x00
#define IISDNlooprevINBOUND_CONNECT         0x01   // Inbound:  Audio path connected from board to calling party
#define IISDNlooprevINBOUND_CHARGE          0x02   // Inbound:  Charge (metering) signal
#define IISDNlooprevINBOUND_DISCONNECT      0x04   // Inbound:  Disconnect signal
#define IISDNlooprevOUTBOUND_CONNECT        0x08   // Outbound: Audio path connected from board to calling party
#define IISDNlooprevOUTBOUND_CHARGE         0x10   // Outbound: Charge (metering) signal
#define IISDNlooprevOUTBOUND_DISCONNECT     0x20   // Outbound: Disconnect signal

/* Per DS-0 configuration for robbed bit E&M Signalling via ENABLE_CAS
 * Zeroing this out will configure for a standard Wink Start Trunk with
 * 10 pulse-per-second dialpulse dialing.
 */
typedef struct
  {
  /*
   * 50 mSec Timer Values.  Values in 50 mSec Tics
   */
#define IISDN56_OUTS_ACK_TIMER_DEFAULT               60
  IISDNu16bit       outseize_ack_timer;         /* Out: Waiting for wink from C.O. */
#define IISDN56_FIXP_TIMER_DEFAULT                    2
  IISDNu16bit       fixed_pause_timer;          /* Out: Pause before dialing */
#define IISDN56_FIRST_DGT_TIMER_DEFAULT              60
  IISDNu16bit       first_indigit_timer;        /* In: Max wait for first digit */
#define IISDN56_CRIT_TIMER_DEFAULT                   20
  IISDNu16bit       critical_timer;             /* In: Max wait between digits */
#define IISDN56_GUARD_TIMER_DEFAULT                  20
  IISDNu16bit       guard_interval_timer;       /* InOut: Pause before send CLR msg */
#define IISDN56_MOD_INRLS_TIMER_DEFAULT              10
  IISDNu16bit       modified_in_rls_timer;      /* In: Use hooktimer_onhk_mod_in_rls */
#define IISDN56_MOD_OUTRLS_TIMER_DEFAULT             10
  IISDNu16bit       modified_out_rls_timer;     /* Out:Use hooktimer_onhk_mod_out_rls*/
#define IISDN56_ANSW_TIMER_DEFAULT                   60
  IISDNu16bit       answer_timer;               /* Out:Snd CONNECT if timed_answer_supervision*/
#define IISDN56_RINGING_TIMER_DEFAULT                40
  IISDNu16bit       ringing_timer;              /* Ringing timer for FXS Loop Start */
#define IISDN56_NO_RINGING_TIMER_DEFAULT             80
  IISDNu16bit       no_ringing_timer;           /* Silence period during ring cycle */
#define IISDN56_DELAYED_WINK_TIMER_DEFAULT           20
  IISDNu16bit       delayed_wink_timer;         /* Max delay before winking if delayed wink enabled */
#define IISDN56_DELAYED_ONHOOK_TIMER_DEFAULT        120
  IISDNu16bit       delayed_onhook_timer;       /* Delay hang up on far end hang up */
  /*
   *10 mSec Timer Values.  Values in 10 mSec Tics
   */
#define IISDN56_GLARE_TIMER_DEFAULT                   5
  IISDNu16bit       glare_detection_timer;      /* Out:Fail if rcv OFHK while running*/ 
#define IISDN56_IGNINSZ_TIMER_DEFAULT                 5
  IISDNu16bit       ign_insz_post_rls_timer;    /* Ignore onhook reflection */
#define IISDN56_IGNRLS_TIMER_DEFAULT                  5
  IISDNu16bit       ign_rls_post_ans_timer;     /* Ignore offhook reflection */
  /*
   *10 mSec Hookswitch Analysis Timer Values.  Values in 10 mSec Tics
   */
#define IISDN56_ONHK_TIMER_DEFAULT                   50
  IISDNu16bit       hooktimer_onhook_rls;       /* Normal onhk detect release filter */
#define IISDN56_MOD_ONHK_IN_TIMER_DEFAULT            80
  IISDNu16bit       hooktimer_onhk_mod_in_rls;  /* In: ignore reflections */
#define IISDN56_MOD_ONHK_OUT_TIMER_DEFAULT           80
  IISDNu16bit       hooktimer_onhk_mod_out_rls; /* Out: ignore reflections */
#define IISDN56_OFHK_INSZ_TIMER_DEFAULT               5
  IISDNu16bit       hooktimer_offhook_inseize;  /* In: # tics before Offhk */
#define IISDN56_OFHK_ANS_TIMER_DEFAULT               10
  IISDNu16bit       hooktimer_offhook_answer;   /* Out: Answer filtering time */
#define IISDN56_MIN_WINK_TIMER_DEFAULT               10
  IISDNu16bit       hooktimer_min_wink;         /* Out:Min # tics to declare wink */
#define IISDN56_MAXMAKE_TIMER_DEFAULT                 8
  IISDNu16bit       hooktimer_maxmake;          /* In:Done w/digit if it expires */ 
#define IISDN56_MAXBREAK_TIMER_DEFAULT                8
  IISDNu16bit       hooktimer_maxbreak;         /* In: Hung up if it expires */
#define IISDN56_MINMAKE_TIMER_DEFAULT                 1
  IISDNu16bit       hooktimer_minmake;          /* In: min make pulse time */
#define IISDN56_MINBREAK_TIMER_DEFAULT                1
  IISDNu16bit       hooktimer_minbreak;         /* In: min break pulse time */
  /*
   *10 mSec Outpulsing, Hookflash and Wink Timer Values.  Values in 10 mSec Tics
   */
#define IISDN56_DPMAKE_TIMER_DEFAULT                  6
  IISDNu16bit       dptimer_make;               /* Out: OFFHOOK time for dialpulse */
#define IISDN56_DPBREAK_TIMER_DEFAULT                 4
  IISDNu16bit       dptimer_break;              /* Out: ONHOOK time for dialpulse */
#define IISDN56_DPINTERDGT_TIMER_DEFAULT             30
  IISDNu16bit       dptimer_interdigit;         /* Out: Time between digits */
#define IISDN56_DPEND_DIAL_TIMER_DEFAULT              5
  IISDNu16bit       dptimer_end_dial_pause;     /* Out: Ignore reflections */
#define IISDN56_DPPREWINK_TIMER_DEFAULT               5
  IISDNu16bit       dptimer_pre_wink;           /* In: Pause before Wink */
#define IISDN56_DPWINK_TIMER_DEFAULT                 20
  IISDNu16bit       dptimer_wink;               /* In: Length of Wink */
#define IISDN56_DPPOSTWINK_TIMER_DEFAULT              5
  IISDNu16bit       dptimer_post_wink;          /* In: Pause After Wink */
#define IISDN56_HOOKFLASH_TIMER_DEFAULT              80
  IISDNu16bit       hookflash_timer;            /* Out: Length of Hookflash */
#define IISDN56_IMMED_DELAY_TIMER_DEFAULT            10
  IISDNu16bit       dptimer_immed_delay;        /* In:Ignore digits IISDNttIMMEDIATE_DIAL*/

  IISDNu8bit        send_seize_comp_event;      /* 0=>none 1=>rx L3L4mSEIZE_COMP */
  IISDNu8bit        send_preseize_event;        /* 0=>no event, 1=>receive L3L4mPRE_SEIZE */
  IISDNu8bit        send_glare_err_event;       /* tx L3L4mERROR if glare detected*/
  IISDNu8bit        send_hookflash_event;       /* send L3L4mHOOKFLASH if detected */
  IISDNu8bit        in_trunk_type;              /* IISDNttXXX for incoming calls */
  IISDNu8bit        out_trunk_type;             /* IISDNttXXX for outgoing calls */
  IISDNu8bit        max_incoming_digit_count;   /* Default: 17 (critical timed) */
  IISDNu8bit        timed_answer_supervision;   /* 0=>trunk can answer 1=>timed */
  IISDNu8bit        fgb_fgd_mode;               /* Feature Group B&D 0=>off 1=>on */
  IISDNu8bit        delayed_wink_mode;          /* Wink Start Delayed Wink Feature 0=>off 1=>on */
  IISDNu8bit        delayed_onhook;             /* Delayed Onhook Feature 0=>off 1=>on */
  IISDNu8bit        loop_current_reversal_flag; /* Flags describing meaning of loop current reversal. See IISDNlooprevXXX */
  IISDN_DTMF_DATA   dtmf_data;                  /* NOT CURRENTLY USED */

  /*
   *More 10 mSec Hookswitch Analysis Timer Values.  Values in 10 mSec Tics
   */
#define IISDN56_WINK_END_TIMER_DEFAULT                0
  IISDNu16bit       hooktimer_wink_end;         /* Out: detection of end of wink signal */
#define IISDN56_MIN_RING_ON_TIMER_DEFAULT            30
  IISDNu16bit       hooktimer_min_ring_on;      /* In:  Minimum ring-on duration to be recognized */
#define IISDN56_MIN_RING_OFF_TIMER_DEFAULT           50
  IISDNu16bit       hooktimer_min_ring_off;     /* In:  Minimum ring-off duration to be recognized */
#define IISDN56_CURRENT_FEED_OPEN_DEFAULT            70
  IISDNu16bit       dptimer_current_feed_open;  /* Out: Duration of current feed open when disconnecting call */
  IISDNu8bit        send_loop_reversal_event;   /* Out: Send L3L4mLOOP_REVERSAL if detected loop current reversal */
  IISDNu8bit        send_caller_id_event;       /* In:  Send L3L4mCALLER_ID_DETECTED if detected caller ID information */
  IISDNu8bit        ring_status_reporting_flag; /* In:  Send L3L4mRING_STATUS to report ring events - see explanation of IISDNringflgXxx */
  IISDNu8bit        call_setup_message_mode;    /* In:  How call setup information will be sent (MF, DTMF, FSK) */
  IISDNu8bit        caller_id_standard;         /* In:  Caller ID standard used */
  IISDNu8bit        caller_id_alerting_signal;  /* In:  Type of CallerID alerting signal used - see IISDNcidalertsigXxx */
  IISDNu8bit        ignore_loop_current;        /* Don't wait for loop current ON before completing L4L3mCALL_REQUEST or L4L3mCONNECT_REQUEST*/
  IISDNu8bit        send_loop_on_event;         /* Out: Send L3L4mLOOP_ON if detected */
#define IISDN56_TONE_DIAL_DIGIT_DURATION_DEFAULT      9
  IISDNu16bit       tone_dial_digit_duration;   /* Out: Duration of tone (DTMF/MF) digits */
#define IISDN56_TONE_DIAL_KP_DURATION_DEFAULT        10
  IISDNu16bit       tone_dial_KP_duration;      /* Out: Duration of MF KP digit (usually longer than other digits) */
#define IISDN56_TONE_DIAL_INTERDIGIT_DEFAULT          6
  IISDNu16bit       tone_dial_interdigit;       /* Out: Time between tone (DTMF/MF) digits */
#define IISDN56_DIAL_PAUSE_TIMER_DEFAULT            100
  IISDNu16bit       dial_pause_timer;           /* Out: Duration of pause (represented by token ',') when dialing PULSE, MF or DTMF digits */
#define IISDN56_DIAL_LONG_PAUSE_TIMER_DEFAULT       500
  IISDNu16bit       dial_long_pause_timer;      /* Out: Duration of long pause (represented by token ';') when dialing PULSE, MF or DTMF digits */
#define IISDN56_LOOP_CURRENT_TIMER_DEFAULT          100
  IISDNu16bit       loop_current_timer;         /* Out: Waiting for loop current before completing L4L3mCALL_REQUEST and L4L3mCONNECT_REQUEST */
#define IISDN56_MIN_RINGING_PULSE_TIMER_DEFAULT      20
  IISDNu16bit       hooktimer_min_ringing_pulse;/* In:  Minimum valid duration of a ringing pulse alerting signal (CallerID - ETSI) */
#define IISDN56_MAX_RINGING_PULSE_TIMER_DEFAULT      50
  IISDNu16bit       hooktimer_max_ringing_pulse;/* In:  Maximum valid duration of a ringing pulse alerting signal (CallerID - ETSI) */
#define IISDN56_FIRST_RING_TIMER_DEFAULT            400
  IISDNu16bit       hooktimer_first_ring_timer; /* In:  Maximum interval between FSK data and first ring pattern (CallerID - ETSI) */
#define IISDN56_ACTIVATION_ACK_TIMER_DEFAULT         30
  IISDNu16bit       activation_ack_timer;       /* In:  Interval between Activation Signal and Activation Ack Signal (CallerID - NTT) */
#define IISDN56_INFORMATION_ACK_TIMER_DEFAULT       650
  IISDNu16bit       information_ack_timer;      /* In:  Maximum interval between Activation Ack Signal and Information Ack Signal (CallerID - NTT) */
#define IISDN56_ANSWER_DELAY_TIMER_DEFAULT           50
  IISDNu16bit       answer_delay_timer;         /* In:  Minimum interval between Information Ack Signal and Answer (Connect) Signal (CallerID - NTT) */
  IISDNu8bit        send_activation_ack_event;  /* In:  Send L3L4mACTIVATION_ACK_SIGNAL if detected Activation Ring (CallerID - NTT) */
  IISDNu8bit        send_information_ack_event; /* In:  Send L3L4mINFORMATION_ACK_SIGNAL if detected Call Information (CallerID - NTT) */
  IISDNu8bit        transmitted_call_ack_type;  /* In:  Type of acknowledgment signal sent when incoming call is detected */
  IISDNu8bit        expected_call_ack_type;     /* Out: Type of acknowledgment signal expected when initiating outbound call */
#define IISDN56_INFORMATION_ACK_DELAY_TIMER_DEFAULT  30
  IISDNu16bit       information_ack_delay_timer;/* In:  Minimum interval between Call Setup Information and Information Ack Signal (CallerID - NTT) */
#define IISDN56_RING_RESET_TIMER_DEFAULT            650
#define IISDN56_RING_RESET_TIMER_JAPAN_DEFAULT      450
  IISDNu16bit       ring_reset_timer;           /* In: */
  IISDNu16bit       reserved[5];                /* reserved for future expansion */
  IISDNu16bit       signature;                  /* reserved; do not use */
} IISDN_ROBBED_BIT_DATA;

typedef enum {
    IISDN_R2MF_INVALID = 0,
    /*
     * forward tone definitions
     */ 
    IISDN_R2F_INVALID   = 0,
    IISDN_R2F_01        = 1,    
    IISDN_R2F_02        = 2,
    IISDN_R2F_03        = 3,
    IISDN_R2F_04        = 4,
    IISDN_R2F_05        = 5,
    IISDN_R2F_06        = 6,
    IISDN_R2F_07        = 7,
    IISDN_R2F_08        = 8,
    IISDN_R2F_09        = 9,
    IISDN_R2F_10        = 10,
    IISDN_R2F_11        = 11,
    IISDN_R2F_12        = 12,
    IISDN_R2F_13        = 13,
    IISDN_R2F_14        = 14,
    IISDN_R2F_15        = 15,
    /*
     * backward tone definitions
     */ 
    IISDN_R2B_INVALID   = 0,
    IISDN_R2B_01        = 1,    
    IISDN_R2B_02        = 2,
    IISDN_R2B_03        = 3,
    IISDN_R2B_04        = 4,
    IISDN_R2B_05        = 5,
    IISDN_R2B_06        = 6,
    IISDN_R2B_07        = 7,
    IISDN_R2B_08        = 8,
    IISDN_R2B_09        = 9,
    IISDN_R2B_10        = 10,
    IISDN_R2B_11        = 11,
    IISDN_R2B_12        = 12,
    IISDN_R2B_13        = 13,
    IISDN_R2B_14        = 14,
    IISDN_R2B_15        = 15,
    IISDN_NUM_R2MF_SIGNAL_CODES = (IISDN_R2B_15+1)
} IISDN_R2MF_SIGNAL_CODES;

typedef enum {
        IISDN_R2MFCP_LINE_FREE_CHARGE           = 0,            /* call accepted */
        IISDN_R2MFCP_LINE_FREE_NO_CHARGE        = 1,            /* call accepted */
        IISDN_R2MFCP_LINE_ALTERNATE_ANSWER      = 2,            /* call accepted */
        IISDN_R2MFCP_LINE_BUSY                          = 3,            /* call rejected */
        IISDN_R2MFCP_LINE_OUT_OF_ORDER          = 4,            /* call rejected */
        IISDN_R2MFCP_LINE_UNALLOCATED           = 5,            /* call rejected */
        IISDN_R2MFCP_LINE_CONGESTION            = 6,            /* call rejected */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_1     = 7,            /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_2     = 8,            /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_3     = 9,            /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_4     = 10,           /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_5     = 11,           /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_6     = 12,           /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_7     = 13,           /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_8     = 14,           /* spare */
        IISDN_R2MFCP_LINE_SPARE_CONDITION_9     = 15,           /* spare */
        IISDN_NUM_R2MFCP_GROUP_B_LINE_CONDITIONS = 16   /* number of conditions */
} IISDN_R2MFCP_GROUP_B_CALLED_LINE_CONDITIONS;

/* 
 * set of forward channel actions...these actions correspond to functions in the 
 * the DSP.
 */
typedef enum {
        IISDN_PROCESS_INVALID_BACKWARD_SIGNAL = 0,              /* forward channel stops operation */
        /*
         * DNIS or ANI state actions
         */
        IISDN_PROCESS_NEXT_DNIS_DIGIT_REQUEST,                  /* request to send next DNIS digit */
        IISDN_PROCESS_LAST_BUT_1_DNIS_DIGIT_REQUEST,    /* request to send last (n-1) DNIS digit */
        IISDN_PROCESS_LAST_BUT_2_DNIS_DIGIT_REQUEST,    /* request to send (n-2) DNIS digit */
        IISDN_PROCESS_LAST_BUT_3_DNIS_DIGIT_REQUEST,    /* request to send (n-3) DNIS digit */
        IISDN_PROCESS_RESTART_DNIS_REQUEST,                             /* request to restart DNIS */
        IISDN_PROCESS_CALL_COMPLETE_CHANGE_TO_GROUP_B,  /* call complete, wait for line condition */
        IISDN_PROCESS_CONGESTION_SIGNAL,                                /* congestion signal before group B */

        /*
         * ANI state only actions
         */
        IISDN_PROCESS_NEXT_ANI_DIGIT_REQUEST,
        /*
         * DNIS state only actions
         */
        IISDN_PROCESS_CALLING_PARTY_CATEGORY_REQUEST,   /* (ITU: response to A5) */
        IISDN_PROCESS_CALL_ACCEPTED_NO_GROUP_B,                 /* call complete without need for Group B */
        IISDN_PROCESS_NATURE_OF_CIRCUIT_QUERY,                  /* satellite link included or not */
        IISDN_PROCESS_ECHO_SUPPRESSOR_QUERY,                    /* NOT IMPLEMENTED !!! */
        /*
         * Called line condition state only action
         */
        IISDN_PROCESS_GROUP_B_LINE_CONDITION,

        IISDN_NUM_R2MFCP_FORWARD_ACTIONS                                /* number of actions defined */ 
} IISDN_R2MFC_FORWARD_ACTIONS;

typedef enum {

        IISDN_PROCESS_INVALID_FORWARD_SIGNAL = 0,               /* pulse congestion signal if available */
        /*
         * DNIS state actions
         */     
        IISDN_PROCESS_DNIS_DIGIT,                                               /* process DNIS digit */
        IISDN_PROCESS_DNIS_END_OF_PULSING,                              /* process end of DNIS signal */
        /*
         * Call (DNIS) category state actions
         */
        IISDN_PROCESS_CALL_CATEGORY_AND_SEND_LINE_STATE,        /* process the call category */
        IISDN_PROCESS_CALL_CATEGORY_AND_REJECT_CALL,            /* invalid call category, reject */
        /*
         * Calling (ANI) category state actions
         */
        IISDN_PROCESS_CALLING_CATEGORY_AND_DONT_REQUEST_ANI,/* process calling cat.; no ANI needed */
        IISDN_PROCESS_CALLING_CATEGORY_AND_REQUEST_ANI,         /* process calling category, get ANI */
        IISDN_PROCESS_CALLING_CATEGORY_AND_REJECT_CALL,         /* process calling cat.; reject call */
        IISDN_PROCESS_CALLING_CATEGORY_REQUEST_DENIED,          /* process denied request for category */
        /*
         * ANI state actions
         */
        IISDN_PROCESS_ANI_DIGIT,                                /* process received ANI digit */
        IISDN_PROCESS_ANI_END_OF_ID,                    /* process end of ANI signal when digits are available */
        IISDN_PROCESS_ANI_NOT_AVAILABLE,                /* process ANI not available signal */
        IISDN_PROCESS_ANI_RESTRICTED,                   /* process ANI restricted signal */
        
        IISDN_NUM_R2MFCP_BACKWARD_ACTIONS               /* number of actions defined */ 
} IISDN_R2MFCP_BACKWARD_ACTIONS;

/*
 * The following structure defines the required parameters for Multifrequency (MF)
 * Tone Generation.
 *
 * All parameters for Call Progress MF tones that can be generated
 * by a particular CPE are identified in the application parameter table that must
 * be written prior to the Controller issuing the CONFIGURE_DSP_CMD. Support for 
 * descriptions for individual channels has not been identified as a requirement
 * at this time.
 * 
 * Any single or dual tone can be generated that incorporates 1-3 distinct
 * cadences, and an arbitrary cycle count.
 *
 * The parameters are as follows:
 *
 *              (1) freqTone1:    frequency of tone #1;
 *                                                units: Hz; range: [0-4000]
 *              (2) powerTone1:   power of tone #1;
 *                                                units: 0.5 dB, relative to system reference output level
 *              (3) freqTone2:    frequency of tone #2;
 *                                                units: Hz; range: [0-4000]
 *              (4) powerTone2:   power of tone #2;
 *                                                units: 0.5 dB, relative to system reference output level.
 *              (5)     numCadences:  the number of distinct cadences to generate (on/off pairs);
 *                                                range [1,2,3]
 *              (6) makeTime1:    duration of first ON cadence;
 *                                                units: milliseconds
 *                                                range: if [1 - 8191] ===> tone plays for this duration
 *                                                               if = 0, ===> duration is 0
 *                                                               if < 0, ===> tone played continuously
 *
 *              (7) breakTime1:   duration of first OFF cadence;
 *                                                units: milliseconds
 *                                                range: if [1 - 8191] ===> tone is off for this duration
 *                                                               if = 0, ===> silence not generated between ON cadences
 *                                                               if < 0 ==> silence is played continuously
 *
 *              (8) makeTime2:    See makeTime1; not used if numCadences = 1.
 *          (9) breakTime2:   See breakTime1; not used if numCadences = 1.
 *         (10) makeTime3:        See makeTime1; not used if numCadences = 2.
 *         (11) breakTime3:       See breakTime1; not used if numCadences = 2.
 *
 *         (12) numCycles:        Number of cadence cycles to generate. If set to 0, the pattern
 *                                                repeats indefinitely.  
 */
typedef struct {
        IISDNu16bit     freqTone1;              /* tone #1 frequency */
        IISDNs16bit     powerTone1;             /* tone #1 power */
        IISDNu16bit     freqTone2;              /* tone #2 frequency */
        IISDNs16bit     powerTone2;             /* tone #2 power */
        IISDNu16bit     numCadences;    /* number of distinct cadences to generate [1-3] */
        IISDNs16bit     makeTime1;              /* cadence #1 on time (ms) */
        IISDNs16bit     breakTime1;             /* cadence #1 off time (ms) */
        IISDNs16bit     makeTime2;              /* cadence #2 on time (ms) */
        IISDNs16bit     breakTime2;             /* cadence #2 off time (ms) */
        IISDNs16bit     makeTime3;              /* cadence #3 on time (ms) */
        IISDNs16bit     breakTime3;             /* cadence #3 off time (ms) */  
        IISDNu16bit     numCycles;              /* number of tone cycles to generate */
} IISDN_CPGEN_MF_PARAMS;

/*
 * Indicate the Call Progress signal "class." As of now, only MF is supported (if SIT
 * is ever needed, it would be added here as another signal class).
 */
typedef enum {
        IISDN_CP_MF_SIGNAL = 0
} IISDN_CP_SIGNAL_CLASS;

/*
 * Declare type for arbitrary Call Progress signal description. Currently, only 
 * MF tone types are supported.  
 */
typedef struct {
        IISDNu16bit     signalClass;    /* cast as IISDN_CP_SIGNAL_CLASS */
        IISDNu16bit     pad;
        union {
                /*
                 * add descriptions here for other signal classes
                 */
                IISDN_CPGEN_MF_PARAMS   mfSignal;       
        } sigDesc;
} IISDN_CP_SIGNAL_DESCRIPTION; 

/*
 * R2 inter-register (MF) parameters
 */
typedef struct _IISDN_R2_INTERREGISTER_PARAMS
{
        IISDNu16bit     dnisMaxNumDigits;               /* maximum number of DNIS digits to collect */  
        IISDNu16bit     aniMaxNumDigits;                /* maximum number of ANI digits to collect */   
        IISDNu16bit     dnisNumDigitsBeforeANI; /* number of DNIS digits before ANI request */

//  R2 Inbound Timers

        IISDNs16bit     interForwardToneTimeOut;                /* units: sec or DISABLED */
        IISDNs16bit     prePulseToneDelay;                              /* units: sec */
        IISDNs16bit     pulseToneDuration;                              /* units: sec */

#define IISDN_INBOUND_PULSE_MIN_DELAY_ITU_MSEC                  (100) /* ITU default: 100 ms */
#define IISDN_INBOUND_PULSE_DURATION_ITU_MSEC                   (150) /* ITU default: 150 ms */

//  R2 Oubound Timers

        IISDNs16bit     forwardToneMaxOnTime;           /* units: sec or DISABLED */    
        IISDNs16bit     forwardToneMaxOffTime;          /* units: sec or DISABLED */
        IISDNs16bit forwardGroup2MaxOnTime;         /* not used */

//  R2 Backward Protocol Params

        IISDNu16bit  dnisTimeOutAction;         /* not used */
#define IISDN_INDIAL_DNIS_TIME_OUT_PULSE_CONGESTION       0 
#define IISDN_INDIAL_DNIS_TIME_OUT_PULSE_GROUPII_REQUEST  1

//  R2 signal definitions

        IISDNu16bit     endOfDNIS;                                              /* end of DNIS signaling */
        IISDNu16bit     endOfANI_Available;                             /* end of ANI signaling when available */               
   IISDNu16bit  InfoNotAvailable;             /* for category - was halfEchoSuppressorRequired */
   IISDNu16bit  spare1;                       /* not used - was noSatelliteLinkInCircuit */
   IISDNu16bit  spare2;                       /* not used - was satelliteLinkInCircuit */
   IISDNu16bit  aniCategoryDefault;

        IISDNu16bit     sendNextDigitDNIS;                                                                      /* ITU A-1 (1) */
        IISDNu16bit     sendLastButOneDigitDNIS;                                                        /* ITU A-2 (2) */
        IISDNu16bit     sendCallCategoryAndSwitchToGroupB_DNIS;                         /* ITU A-3 (3) */
        IISDNu16bit     congestion;                                                                             /* ITU A-4 (4) */
        IISDNu16bit     sendCallingPartyCategory;                                                       /* ITU A-5 (5) */
        IISDNu16bit     callComplete_SetUpSpeechPath;                                           /* ITU A-6 (6) */
        IISDNu16bit     sendLastButTwoDigitDNIS;                                                        /* ITU A-7 (7) */
        IISDNu16bit     sendLastButThreeDigitDNIS;                                                      /* ITU A-8 (8) */
   IISDNu16bit  spare3;                                                                         /* not used */
        IISDNu16bit     changeFrom_ANI_To_DNIS_SendNextDigit_F;                         /* when next DNIS = next ANI */
        IISDNu16bit     sendFirstDigitDNIS;                                                                     /* non-ITU (11) */
        IISDNu16bit     sendNextDigitANI;                                                                       /* non-ITU (12) */
        IISDNu16bit     changeFrom_ANI_To_DNIS_SendNextDigit;                           /* (15) */
        IISDNu16bit     changeFrom_ANI_To_DNIS_SendLastDigit;                           /* (16) */      
        IISDNu16bit     groupB_LineConditions[IISDN_NUM_R2MFCP_GROUP_B_LINE_CONDITIONS]; /* (21-36) */

#define IISDN_R2_MAX_NUM_CALL_PROGRESS_SIGNALS  (2)   /* RING and BUSY */
        IISDN_CPGEN_MF_PARAMS   cpSignals[IISDN_R2_MAX_NUM_CALL_PROGRESS_SIGNALS];

} IISDN_R2_INTERREGISTER_PARAMS;

#define IISDN_TONE_START 1
#define IISDN_TONE_STOP 0

typedef struct
{
        IISDNu8bit command;  /* use IISDN_TONE_xxxx */
        IISDNu8bit channel;  /* 0 through 31 */
        IISDNu16bit pad;
        /* IISDN_R2MF_SIGNAL_CODES*/ IISDNu32bit  code;
} IISDN_TONE_CTRL_DATA;

#define IISDN_START_OF_TX_CTRL_DATA_BUF 10      /* L4L3 Common Message header (8) + 2 bytes for Length */
#define IISDN_START_OF_RX_CTRL_DATA_BUF 18      /* L3L4 Common Message header (16) + 2 bytes for Length */
#define IISDN_CTRL_DATA_TX_BUF_SIZE     (L4L3_BUF_SIZE - IISDN_START_OF_TX_CTRL_DATA_BUF)
#define IISDN_CTRL_DATA_RX_BUF_SIZE     (L3L4_BUF_SIZE - IISDN_START_OF_RX_CTRL_DATA_BUF)

typedef struct
{
        IISDNu16bit     len;    /* Length of the control data buffer */
        IISDNu8bit      data[IISDN_CTRL_DATA_TX_BUF_SIZE];
} IISDN_CONTROL_DATA_TX;

typedef struct
{
        IISDNu16bit     len;    /* Length of the control data buffer */
        IISDNu8bit      data[IISDN_CTRL_DATA_RX_BUF_SIZE];
} IISDN_CONTROL_DATA_RX;

// Do not use.
// These flags were made obsolete and may be removed in the future.
#define R2_SHOW_EVENTS                          0x01
#define R2_SHOW_COMMANDS                        0x02
#define R2_SHOW_STATUS_INFORMATION              0x04
#define R2_SHOW_STATUS_EXTRA_INFORMATION        0x08
#define R2_SHOW_DISPATCHER                      0x10

#define IISDN_CONTROL_DATA_BUF_SIZE 
typedef struct 
{
        unsigned short r2OutSeizeTimer;            // outbound: time to wait for seize ack
        unsigned short r2OutAnswerTimeOut;         // outbound: time waiting for answer in seconds
        unsigned short inboundReleaseGuardTime;    // inbound:  release guard time
        unsigned short inboundLineQualTimerIdle;   // inbound:  qualification time for seizure

        unsigned short DebugBitMask;               // Reserved

        unsigned char c_d_cas_bits;                // value of C & D bits -- D must be set
        unsigned char clearbackControl;            // 1 => outbound recognizes forced release
} IISDN_R2_DIGITAL_LINE_SIG_PARAMS;

typedef struct
{ /* WARNING: size of this structure is pushing the limit of L4L3_BUF_SIZE. Add new parameters with care! */
   IISDN_R2_DIGITAL_LINE_SIG_PARAMS     r2_digital_line_sig;            
   IISDN_R2_INTERREGISTER_PARAMS                r2_interregister_sig;   /* so far, all shorts */

   unsigned short BillingPulse;

#define A_BIT_METERING                      1
#define B_BIT_METERING                      2

   unsigned short BillingQualificationTime;
   unsigned short R2Options;
   unsigned long  R2ResetTimer;

#define R2_COLLECT_DIGITS                   1

} IISDN_E1_CAS_R2_DATA;

#define R2_BLOCKED_REMOTE_BLOCKING              1
#define R2_BLOCKED_NEAR_END_BLOCKING            2
#define R2_BLOCKED_NO_RESOURCES                 3
#define R2_BLOCKED_NO_RESPONSE                  4
#define R2_BLOCKED_IDLE_TIMEOUT                 5

/* Definitions for L4L3mJATE_REDIAL  */
// JATE emergency array size comes from a customer accepted value. It is not specified
// in any specs
#define IISDN_MAX_EMERG_NUMS                    10
#define IISDN_JATE_CLEAR_RESTRICTION            1
#define IISDN_JATE_LOAD_EMERG_NUMS              2
#define IISDN_JATE_NUMBERS_TRACKED              3

typedef struct
{
    IISDNu8bit num_digits;
    IISDNu8bit digits[IISDN_MAX_DIGITS]; /* contains 1 emergency number                        */
} IISDN_EMERG_DIGITS;

typedef struct
{
    IISDNu8bit            mode;                     /* IISDN_JATE_CLEAR_RESTRICTION -
                                                       clears a restricted channel, 15X method            */
                                                    /* IISDN_JATE_LOAD_EMERG_NUMS -
                                                       loads emergency numbers to FW,                     */
                                                    /* IISDN_JATE_NUMBERS_TRACKED -
                                                       specify the max no to be tracked for restrictions  */
    IISDNu8bit            lapdid;                   /* lapdid where restricted channel resides            */
    IISDNu8bit            bchannel;                 /* b-channel # that is currently restricted           */
    IISDNu8bit            max_numbers_tracked;      /* max numbers to be tracked for redial restrictions  */
    IISDN_EMERG_DIGITS    emergNums[IISDN_MAX_EMERG_NUMS];  /* holds 10 emergency #'s that     */
                                                            /* can be loaded using             */
                                                            /* IISDN_JATE_LOAD_EMERG_NUMS mode */
} IISDN_JATE_REDIAL;



/* Dial information for L4L3mDIAL */
typedef struct
        {
        IISDNu8bit num_digits;                  /* 0 => off board dialing (e.g. DTMF) */
        IISDNu8bit wait_for_ack_wink;   /* 0 => no acknowledgement wink expected, 1=> wait for wink */
        IISDNu8bit report_dial_completion;      /* 0 => don't report 1 => send L3L4mEND_DIAL when done dialing */
        IISDNu8bit bchannel;
        IISDNu8bit digits[IISDN_MAX_DIGITS]; /* digits to dial */
        IISDNu8bit default_digit_type;       /* IISDNdigtypeXXXX : audio-based call setup only; not used for CCS protocols */
        } IISDN_DIAL_DATA;
        

/* Defines for billing_change_type */
#define IISDNvbfNEW_RATE                        0x10
#define IISDNvbfFLAT_RATE                       0x11
#define IISDNvbfPREMIUM_CHARGE          0x12
#define IISDNvbfPREMIUM_CREDIT          0x13
#define IISDNvbfFREE_CALL                       0x18

typedef struct
        {
        IISDNu8bit billing_change_type; /* Use IISDNvbfXXXX */
        /* The following fields must be populated regardless of whether they
                apply.  For 0 the ASCII 0x30 ('0') must be used. */
        IISDNs8bit      hundreds_of_dollars;    /* ASCII character for hundreds digit */
        IISDNs8bit      tens_of_dollars;                /* ASCII character for tens digit */
        IISDNs8bit      dollars;                                /* ASCII character for ones digit */
        IISDNs8bit      tenths_of_dollars;              /* ASCII character for tenths digit */
        IISDNs8bit      hundredths_of_dollars;  /* ASCII character for hundredths digit */
        IISDNu16bit pad;                                        /* Longword alignment */
        } IISDN_BILLING_FEATURE;
        
typedef struct
        {
        IISDNu8bit      coding_standard;        /* IISDNcodXXXX */
        IISDNu8bit      location;                       /* IISDNlocXXXX */
        IISDNu8bit      description;
        IISDNu8bit      pad;
        } IISDN_PROGRESS_IND;


/* IISDN-BOARD-ID struct returned in response to L4L3mREQ_BOARD_ID
 * can be used to determine physical hardware configuration
 */
#define IISDN_MAX_BFIO_DEVICES 16
typedef struct
        {
        IISDNs8bit iisdn_ver[32];               /* InstantISDN 6.x.x */
        IISDNs8bit banner[32];          /* NAIXXX (C)1997 */
        IISDNs8bit date[16];                    /* DDMMYYYY */
        IISDNs8bit model[16];                   /* model number */
        IISDNs8bit rev[12];                     /* revision of this board */
        IISDNu8bit      board_type;
        IISDNu8bit      num_lines;
        IISDNu8bit      pad1[2];
        IISDNu16bit     num_hdlc_chan;
        IISDNu16bit     num_modem_chan;
        IISDNu8bit      line_type[IISDN_MAX_LINES];     /* IISDNline_typeXXX */
        IISDNu32bit     kernel_ram_size;
        IISDNu32bit     mezz_ram_size;
        IISDNu8bit      num_bfio_devices;
        IISDNu8bit      pad2[3];
        struct 
                {
                IISDNu8bit devType;             /* IISDNdevXXXX */
                IISDNu8bit numLapdids;  /* number of channels */
                IISDNu16bit firstLapdid;  /* default id of first channel */
                IISDNu8bit ethernet_address[6]; /* IISDNdevETHERNET only, 0050-91xx-xxxx */
                IISDNu8bit pad[2];
                } bfio_device[IISDN_MAX_BFIO_DEVICES];
        } IISDN_BOARD_ID;       

/* IISDN_BOARD_VERSION struct returned in response to L4L3mREQ_VERSION
 */
typedef struct
        {
        IISDNs8bit firmware_ver[32];            /* InstantISDN 7.x.x */
        IISDNs8bit iisdn_header_ver[32];        /* NAIXXX (C)1997 */
        IISDNs8bit driver_ver[32];                      /* DDMMYYYY */
        } IISDN_BOARD_VERSION;

/* Special NOTE: when BOARD_ID.devType == IISDNdevHDLC_LINE
 * The numLapdids are evenly distributed across BOARD_ID.num_lines.
 * IISDNlineX = (your_lapdid_here - firstLapdid) / (numLapdids / num_lines)
 * Suppose
 *   devType = IISDNdevHDLC_LINE
 *   num_lines = 8
 *   firstLapdid = 32
 *   numLapdids = 16
 * then
 *   lapdid 32, 33 are associated with IISDNlineA
 *   lapdid 34, 35 are associated with IISDNlineB
 *   lapdid 36, 37 are associated with IISDNlineC
 *   lapdid 38, 39 are associated with IISDNlineD
 *   lapdid 40, 41 are associated with IISDNlineE
 *   lapdid 42, 43 are associated with IISDNlineF
 *   lapdid 44, 45 are associated with IISDNlineG
 *   lapdid 46, 47 are associated with IISDNlineH
 * and 
 *   SET-TSI cannot be used to map IISDNdevHDLC_LINE devices.
 *   They are directly part of a line interface chip.  
 *   Use enable_protocol.level1.hdlc_channels
 *   to designate a specific line DS0 for tx/rx
 */

/* Defines for feature_type */
#define IISDNftANI_ON_DEMAND            1       /* Automatic Number Identification */
#define IISDNftBILLING_CHANGE           2       /* Billing rate change request */

/* Data structure for Feature Requests */
typedef struct
        {
        IISDNu8bit      feature_type;   /* IISDNftXXXX */
        IISDNu8bit      pad0;
        IISDNu16bit     pad1;                   /* longword alignment */
        struct
                {
                IISDN_BILLING_FEATURE   billing;/* change request if IISDNftBILLING_CHANGE */
                }feature;                                       /* Data corresponding to feature request */
        } IISDN_FEATURE_REQUEST;

/* Defines for billing_change_status in L3L4mBILLING_STATUS response */
#define IISDNvbsBCHG_ACCEPTED           1
#define IISDNvbsBCHG_DENIED             2
#define IISDNvbsBCHG_REJECTED           3

/* Universal IE Structure Definition--a string of IE's may be specified.  The
        last IE must be zero terminated.  The total of all IE's (host specified
        and Instant ISDN supplied) must be 30 or less. */
typedef struct
        {
        IISDNu8bit ie_id;                               /* Information Element Identifier */
        IISDNu8bit ie_length;                   /* Length of data bytes */
        IISDNu8bit ie_data[1];                  /* IE data bytes */
        } IISDN_IE_STRUCT;

/* For L4L3mCONTINUE and L3L4mCONTINUE messages */
typedef struct
        {
        IISDNu8bit      ie_count;                       /* Count of IE's in this message */
        IISDNu8bit      ie_more;                        /* More IEs in next message (0 = no) */
        IISDNu16bit     pad1;                           /* Preserve long work boundary */
        IISDN_IE_STRUCT ie;                                     /* String of IE's of ie_count */
        } IISDN_CONTINUE_DATA;
        
/* For L4L3mUNIVERSAL and L3L4mUNIVERSAL messages */
typedef struct
        {
        IISDNu8bit      msg_id;                         /* Q.931/932 message id */
        IISDNu8bit      null_call_ref;          /* 1 => Use null call reference */
        IISDNu8bit      q931_escape;            /* 1 => Use q931 escape message */
        IISDNu8bit      pad;                            /* Preserve long work boundary */
        IISDNu8bit      aoc[16];                        /* "Parsed" Advice of Charge (AOC) info */
        IISDNu8bit      ie_count;                       /* Count of IE's in this message */
        IISDNu8bit      ie_more;                        /* More IEs in next message (0 = no) */
        IISDNu16bit     pad1;                           /* Preserve long work boundary */
        IISDN_IE_STRUCT ie;                                     /* String of IE's of ie_count */
        } IISDN_UNIVERSAL;

/* For L4L3mSPID and L3L4mSPID messages */
typedef struct
        {
        IISDNu8bit      spid_len;                               /* Basic Rate Nat'l ISDN-1 SPID 3..20 */
        IISDNu8bit      spid[IISDN_MAX_SPID_LEN];       /* Service Profile ID (ASCII)     */
        IISDNu8bit      tei;                                    /* TEI of SPID */
        IISDNu8bit      reject_spid;                    /* L4L3m only to accept/reject SPID 1=>reject 0=>accept */
        IISDNu8bit      usid;                                   /* L4L3m only to accept a SPID */
        IISDNu8bit      tid;                                    /* L4L3m only to accept a SPID */
   IISDNu8bit  pad[3];
        } IISDN_SPID;

typedef struct
        {
        IISDNu16bit     num_digits;
        IISDNu8bit      num_type;                       /* IISDNnumtXXXX */
        IISDNu8bit      num_plan;                       /* IISDNnumpXXXX */
        IISDNu8bit      presentation_ind;       /* Presentation indicator (optional) use IISDNpresXXXX, or IISDN_R2_ANI_XXXX for R2 */
        IISDNu8bit      screening_ind;          /* Screening indicator (optional)  use IISDNscrXXXX */
        IISDNu16bit ani_category;       /* R2 only, use IISDN_R2MF_SIGNAL_CODES */
        IISDNu16bit pad;                /* For backward compatibility, don't need this pad */
        IISDNu8bit      digits[IISDN_MAX_DIGITS+2];     /* NOTE: Digits are ASCII! */
        } IISDN_CALLING_PARTY;

/* User-User Information Element */
#define IISDN_MAX_USER_INFO 130
typedef struct
        {
        IISDNu16bit     len; /* 0 means no information to be passed */
        IISDNu8bit      info[IISDN_MAX_USER_INFO];      /* Octets 3,... */
        } IISDN_USER_INFO;

/*
 * For R2 ANI (Automatic Number Identification) category status 
 */
#define  IISDN_R2_ANI_AVAILABLE         0       /* ANI is available     */
#define  IISDN_R2_ANI_NOT_AVAILABLE             1       /* ANI is NOT available */
#define  IISDN_R2_ANI_RESTRICTED                2       /* ANI is RESTRICTED    */

typedef struct
        {
        IISDNu16bit     num_digits;
        IISDNu8bit      num_type;               /* IISDNnumtXXXX */
        IISDNu8bit      num_plan;               /* IISDNnumpXXXX */
        IISDNu16bit dnis_category;  /* R2 only, one of IISDN_R2MF_SIGNAL_CODES */
        IISDNu8bit  default_digit_type;   /* IISDNdigtypeXXXX : audio-based call setup only; not used for CCS protocols */
        IISDNu8bit  pad;
        IISDNu8bit      digits[IISDN_MAX_DIGITS];       /* NOTE: Digits are ASCII! */
        } IISDN_CALLED_PARTY;

/* IISDN_CONNECTED_ADDRESS is defined the same as IISDN_CALLED_PARTY because */
/* IISDN source code expects the structures to be identical when encoding digits. */
#define IISDN_CONNECTED_ADDRESS IISDN_CALLED_PARTY

/* Call ID field (primarily used by Release Link Trunk) */
#define IISDN_MAX_CALL_ID 31
typedef struct
        {
        IISDNu8bit      len;            /* 0 means no information to be passed */
        IISDNu8bit      call_id[IISDN_MAX_CALL_ID];     /* Octets 3, ... */
        } IISDN_CALL_ID;
        
typedef struct
        {
        IISDNu16bit     num_digits;
        IISDNu8bit      num_type;                       /* IISDNnumtXXXX */
        IISDNu8bit      num_plan;                       /* IISDNnumpXXXX */
        IISDNu8bit      presentation_ind;       /* Presentation indicator (optional) use IISDNpresXXXX */
        IISDNu8bit      screening_ind;          /* Screening indicator (optional) use IISDNscrXXXX */
        IISDNu8bit      redir_reason;           /* Redirection Reason IISDNrrsnXXX */
        IISDNu8bit      cfw_no_reply_ind;       /* Call forwarding no reply indicator 0 => False 1 => True */
        IISDNu8bit      redirection_counter;/* use 0-5 */
        IISDNu8bit      spare;                          /* For alignment */
        IISDNu16bit     spare1;                         /* For backward compatibility, otherwise don't need this */
        IISDNu8bit      digits[IISDN_MAX_DIGITS];       /* NOTE: Digits are ASCII! */
        } IISDN_ORIG_CALLED_NUM;
        
typedef struct
        {
        IISDNu16bit     num_digits;
        IISDNu8bit      num_type;                       /* IISDNnumtXXXX */
        IISDNu8bit      num_plan;                       /* IISDNnumpXXXX */
        IISDNu8bit      presentation_ind;       /* Presentation indicator (optional) use IISDNpresXXXX */
        IISDNu8bit      screening_ind;          /* Screening indicator (optional) use IISDNscrXXXX */
        IISDNu8bit      redir_reason;           /* Redirection Reason IISDNrrsnXXX */
        IISDNu8bit      spare;                          /* For word alignment */
        IISDNu8bit      digits[IISDN_MAX_DIGITS];       /* NOTE: Digits are ASCII! */
        } IISDN_REDIRECT_NUM;

#define IISDN_MAX_DISPLAY 82
typedef struct
        {
        IISDNu8bit      num_digits;                     /* Number of valid IA5 characters */
        IISDNu8bit      spare;                          /* For word alignment */
        IISDNu8bit      digits[IISDN_MAX_DISPLAY];
        } IISDN_DISPLAY;

/* Q.933 Frame Relay DLCI negotiation field (similar to Q.931 channel id) */
typedef struct
        {
        IISDNu8bit      dlci_present;   /* 0 => DLCI information not available */
        IISDNu8bit      preferred;              /* 0 => Exclusive, 1=> Preferred */
        IISDNu16bit     dlci;                   /* Q.922 DLCI [0..1023] */
        } IISDN_Q922_DLCI;

/* Q.933 End-to-End Transit Delay */
typedef struct
        {
        IISDNu16bit     cumulative;             /* Cumulative transit delay or 0 */
        IISDNu16bit     requested;              /* Requested transit delay or 0 */
        IISDNu16bit     maximum;                /* Maximum transit delay or 0 */
        IISDNu16bit     pad;
        } IISDN_TRANS_DELAY;

/* Q.933 Link Layer Core Parameters */
typedef struct
        {
        IISDNu16bit max_outgoing_framesz;
        IISDNu16bit max_incoming_framesz;
        IISDNu8bit      outgoing_thruput_magnitude;
        IISDNu8bit      incoming_thruput_magnitude;
        IISDNu16bit outgoing_thruput_multiplier;
        IISDNu16bit incoming_thruput_multiplier;
        IISDNu8bit      outgoing_min_thruput_magnitude;
        IISDNu8bit      incoming_min_thruput_magnitude;
        IISDNu16bit outgoing_min_thruput_multiplier;
        IISDNu16bit incoming_min_thruput_multiplier;
        } IISDN_LNK_CORE_PARAM;

/* Q.933 Link Layer Protocol Parameters */
typedef struct
        {
        IISDNu8bit      k;
        IISDNu8bit      pad;
        IISDNu16bit t200;
        } IISDN_LNK_PROTO_PARAM;

/* Q.933 X.213 Priority */
typedef struct
        {
        IISDNu8bit      valid;                  /* 0 => data not available */
        IISDNu8bit      priority;               /* X.213 priority */
        IISDNu8bit      lowest_valid;   /* Lowest X.213 priority valid */
        IISDNu8bit      low_priority;   /* Lowest acceptable priority */
        } IISDN_X213_PRIORITY;

/* Q.933 Network Service Access Point (NSAP) for subaddress */
#define IISDN_NSAP_LEN                          22
#define IISDNnsapX213                           0x80    /* X.213/ISD 8348 AD2 */
#define IISDNnsapUSER_SPECIFIED         0xA0    /* User defined */
#define IISDNnsapUSER_SPECIFIED_ODD     0xA8    /* User defined odd BCD */
typedef struct
        {
        IISDNu8bit      length;                                 /* 0 => not available */
        IISDNu8bit      encoding;                               /* IISDNnsapXXX */
        IISDNu8bit      subaddress[IISDN_NSAP_LEN];
        } IISDN_NSAP;

#define IISDN_MAX_BC    12
typedef struct
        {
        IISDNu32bit             bchannel_mask;  /* Bit mask used for multi-rate
                                                                                1 => In-Service; 0 => Out-of-Service
                                                                                B channels numbered [1..24]
                                                                                bit => (1 << channel_num). This is the
                                                                                slot map for multi-rate calls 
                                                                                Normally set to 0x00FFFFFF for all 24
                                                                                channels.  Channel 1 being the lsb and 
                                                                                24 the msb. WARNING:field only valid
                                                                                if call_type=IISDNcalltypMULTIRATE_DATA */
        IISDNu8bit              bchannel;       /* Bchannel (or H0/H11/1st multirate chan)*/
        IISDNu8bit              preferred;      /* 0 => must use the above channel
                                                                        1=>network can chose a different channel */
        IISDNu8bit              net_spfc;       /* Call-by-call feature or IISDNnsNULL(0) */
        IISDNu8bit              iface;          /* Non-Facility Signalling.  Set to 0xFF */
        IISDNu32bit             call_type;              /* Usually IISDNcalltypVOICE */
        IISDN_CALLING_PARTY     calling_party;  /* num_digits=0 means not available */
        IISDN_CALLED_PARTY      called_party;   /* num_dig=0 is valid in some cases */
        IISDN_REDIRECT_NUM      redirect_num;   /* num_digits=0 means not available */
        IISDNu16bit             overide_bc_len; /* 0 or Override bearer cap len */
        IISDNu8bit              bearer_cap[IISDN_MAX_BC];
                                                                                /* Override bc data (octets 3,4...) */
        IISDNu8bit              pad;
        IISDNu8bit              reserved;       /* 1 => allowed to use channel for outgoing call 
                                       used for Call Transfer */
        IISDN_USER_INFO         user_info;              /* User-User Information or .len = 0 */
        IISDN_Q922_DLCI         fr_dlci;                /* Q.933 DLCI negotiation */
        IISDN_TRANS_DELAY               fr_e2e_trans_dly;
                                                                                /* Q.933 cumulative delay */
        IISDN_LNK_CORE_PARAM    fr_lnk_core;    /* Q.933 Link Layer Core Params */
        IISDN_LNK_PROTO_PARAM   fr_lnk_proto;   /* Q.933 Link Layer Protocol Params */
        IISDN_X213_PRIORITY     fr_x213_priority; /* Q.933 X.213 priority */
        IISDN_NSAP                      fr_calling_subaddress;
                                                                                /* Q.933 Calling Party Subaddress */
        IISDN_NSAP                      fr_called_subaddress;
                                                                                /* Q.933 Called Party Subaddress */
        IISDNu8bit              rlt_service;    /* 1 =>DMS-250 Release Link Trunk Svc */
                                                                                /* insert facility ie in the setup msg*/
        
        IISDNu8bit              jate_redial;    /* jate redial */
        IISDNu8bit              ie_count;               /* Count of IE's in this message */
        IISDNu8bit              ie_more;                /* More IEs in next message (0 = no) */
        IISDN_ORIG_CALLED_NUM   orig_called_num;/* num_digits=0 means not available */
        IISDN_IE_STRUCT         ie;                     /* String of IE's of ie_count */
        } IISDN_CALL_REQ_DATA;

/* Defines for features_availability */
#define IISDNfaNONE                                     0
#define IISDNfaFLEXIBLE_BILLING         1

/* Defines for bearer_selection */
#define IISDNbsNONE             0       /* Bearer negotiation not configured */
#define IISDNbsPREFERRED        1       /* Bearer is preferred.  Host must respond */
                                                        /* with channel and interface number */
#define IISDNbsEXCLUSIVE        2       /* Bearer is exculsive.  Host must respond */
                                                        /* with channel and interface number */
#define IISDNbsCHAN_RESERVED 0x03 //chan id IE, octet 3 has 'info chan select'
                     // set for 'RESERVED'

typedef struct
        {
        IISDNu32bit             call_type;              /* Usually IISDNcalltypVOICE */
        IISDN_CALLING_PARTY     calling_party;  /* num_digits=0 means not available */
        // IISDN_CALLING_PARTY  calling_party2; /* possible second calling party */
                                                /* Removed for backward compatibility */
        IISDN_CALLED_PARTY      called_party;   /* num_dig=0 is valid in some cases */
        IISDN_REDIRECT_NUM      redirect_num;   /* num_digits=0 means not available */
        IISDNu16bit             net_spfc;               /* Call-by-call feature. IISDNnsXXXX */
        IISDNu16bit             bc_len;                 /* bearer cap len */
        IISDNu8bit              bearer_cap[IISDN_MAX_BC];       
                                                                                /* bc data (octets 3,4...) */
        IISDNu16bit             x25_tx_pktsize; /* Negotiated tx pkt size for X.25 */
        IISDNu16bit             x25_rx_pktsize; /* Negotiated rx pkt size for X.25 */
        IISDN_USER_INFO         user_info;              /* User-User Information or .len = 0 */
        IISDN_Q922_DLCI         fr_dlci;                /* Q.933 DLCI negotiation */
        IISDN_TRANS_DELAY       fr_e2e_trans_dly; /* Q.933 cumulative delay */
        IISDN_LNK_CORE_PARAM    fr_lnk_core;    /* Q.933 Link Layer Core Params */
        IISDN_LNK_PROTO_PARAM   fr_lnk_proto;   /* Q.933 Link Layer Protocol Params */
        IISDN_X213_PRIORITY     fr_x213_priority; /* Q.933 X.213 priority */
        IISDN_NSAP                      fr_calling_subaddress;
                                                                                /* Q.933 Calling Party Subaddress */
        IISDN_NSAP                      fr_called_subaddress;
                                                                                /* Q.933 Called Party Subaddress */
        IISDNu8bit              feature_availability; /* Features selected. IISDNfaXXX */
        IISDNu8bit              bearer_selection;       /* Bearer selection.  IISDNbsXXX */
        IISDNu8bit              sending_complete;       /* Sending complete ie present */
        IISDNu8bit              x25_modulo128;          /* 0 -> modulo 8 1 -> modulo 128 for X.25 */
        IISDN_PROGRESS_IND      progress_ind;
        IISDN_ORIG_CALLED_NUM   orig_called_num;        /* num_digits=0 means not available */
        IISDN_DISPLAY                   display;                /* Display IE */
        } IISDN_SETUP_DATA;

typedef enum {
        IISDNR2statDEFAULT      = 0,                            /* Outbound/Inbound: default */
        IISDNR2statNETWORK_DISCONNECT,                  /* Outbound/Inbound: from connected state */
        IISDNR2statNETWORK_CLEAR_DURING_INSEIZE,/* Inbound: during inbound ack state */
        IISDNR2statL4_CLEAR_REQUEST,                    /* Outbound/Inbound: L4 requested when connected */
        IISDNR2statGROUP_B_AVAILABLE,                   /* Outbound; Group B available */
        IISDNR2statGROUP_A_CALL_ACCEPTED,               /* Outbound; ITU A6 */
        IISDNR2stat_CONGESTION,                         /* Outbound */
        IISDNR2statLINE_ABNORMAL_ALARM,                 /* Outbound abnormal bits received */           
        IISDNR2statSEIZE_ACK_TIMEOUT,           
        IISDNR2statFAULT_LINE_SIGNAL,                   /* Inbound fault condition */   
        IISDNR2statINTERREG_PCOL_ERR,                   /* Outbound/Inbound: interreg protocol error */
        IISDNR2statINTERREG_PCOL_REJECTED_CALL, /* Inbound; protocol rejected address info */
        IISDNR2statINTERREG_T1_TIMEOUT,                 /* Outbound; interregister T1 time-out */
        IISDNR2statINTERREG_T2_TIMEOUT,                 /* Outbound; interregister T2 time-out */
        IISDNR2statINTERREG_T3_TIMEOUT,                 /* Inbound; interregister T3 time-out */
        IISDNR2statMISC_MF_ERROR,                               /* Outbound/Inbound miscellaneous error */
        IISDNR2statOUTSEIZE_TIMEOUT,                    /* Outbound */
        IISDNR2statANSWER_TIMEOUT,                              /* Outbound */
        IISDNR2statPREMATURE_ANSWER,                    /* Outbound; before MF signaling is complete */
        IISDNR2statDOUBLE_SEIZURE,                              /* Outbound; double seizure */
        IISDNR2statDIGITSTUCK,
        IISDNR2statNOFIRSTDIGIT,
        IISDNR2statUNEXPECTEDGROUPI,
        IISDNR2statUNEXPECTEDGROUPII,
        IISDNR2statUNEXPECTEDGROUPA,
        IISDNR2statUNEXPECTEDGROUPB,
        IISDNR2statLINEBUSY,
        IISDNR2statLINEOUTOFORDER,
        IISDNR2statNUMBERUNALLOCATED,
        IISDNR2statFALSESEIZURE,
        IISDNR2statHISTORY,
   IISDNR2statTONEERROR,
   IISDNR2statPACKETERROR,
   IISDNR2statCPERROR,
   IISDNR2statAPPLICATIONTIMEOUT,
   IISDNR2statCLEARDURINGCOLLECTION,
   IISDNR2statLOOPBACK,
   IISDNR2statCOMPELLINGERROR,
   IISDNR2statPACKETSTATSBEGIN = 40,
   IISDNR2statPACKETSTATSEND   = 49,
   IISDNR2statNULL                                                      /* null, no reason */
} IISDN_R2_CALL_STATUS_CODES;

typedef struct
{
        IISDNu8bit       call_status;           /* IISDN_R2_CALL_STATUS_CODES */     
        IISDNu8bit              group_B;                        /* IISDN_R2MFCP_GROUP_B_CALLED_LINE_CONDITIONS */ 
        IISDNu8bit              numberRings;            /* number of RING cycles before accepting call */
   IISDNu8bit     BChannel;
} IISDN_R2_CALL_STATUS;

typedef struct
{
   IISDNu8bit     error_code;       // Error associated with command
   IISDNu8bit     CurrentCommand;
   IISDNu16bit    state;            // State information
} IISDN_R2_ERROR;

/* Alerting/connect message data for L4L3mALERTING_REQUEST,  
   L4L3mCONNECT_REQUEST and L3L4mALERTING, L3L4mCONNECT */
typedef struct
        {
        IISDN_REDIRECT_NUM      redirect_num;   /* num_digits=0 means not available */
        IISDNu16bit             x25_tx_pktsize; /* Negotiated tx pkt size for X.25 */
        IISDNu16bit             x25_rx_pktsize; /* Negotiated rx pkt size for X.25 */
        /* The following two fields are used for B-channel negotiation.
                The first response to a SETUP_IND must contain the B-channel
                that has been selected by the application.  If B-channel
                negotiation is not used or for the L3L4mALERTING or L3L4mCONNECT
                messages the next two fields will be ignored. */
    IISDNu32bit         bchannel_mask;  /* Bit mask used for multi-rate
                                                                                1 => In-Service; 0 => Out-of-Service
                                                                                B channels numbered [1..24]
                                                                                bit => (1 << channel_num). This is the
                                                                                slot map for multi-rate calls 
                                                                                Normally set to 0x00FFFFFF for all 24
                                                                                channels.  Channel 1 being the lsb and 
                                                                                24 the msb */
        IISDNu8bit              bchannel;               /* B-channel for this call */
        IISDNu8bit              iface;                  /* Interface for this call */
        IISDNu16bit             pad;
        IISDN_USER_INFO         user_info;              /* User-User Information or .len = 0 */
        IISDN_Q922_DLCI         fr_dlci;                /* Q.933 DLCI negotiation */
        IISDN_TRANS_DELAY               fr_e2e_trans_dly;
                                                                                /* Q.933 cumulative delay */
        IISDN_LNK_CORE_PARAM    fr_lnk_core;    /* Q.933 Link Layer Core Params */
        IISDN_LNK_PROTO_PARAM   fr_lnk_proto;   /* Q.933 Link Layer Protocol Params */
        IISDN_X213_PRIORITY     fr_x213_priority; /* Q.933 X.213 priority */
        IISDN_NSAP                      fr_connected_subaddress;
                                                                                /* Q.933 Connected Party Subaddress */
        IISDN_CONNECTED_ADDRESS connected_address;/* Connected Address (1TR6) */
        IISDN_PROGRESS_IND      progress_ind;
        IISDN_DISPLAY                   display;                /* Display IE - L3L4 Only */
        IISDN_CALL_ID                   call_id;                /* Call ID for RLT */
        IISDN_R2_CALL_STATUS    r2_call_status; /* use is dependent on call direction */

        IISDNu8bit              ie_count;               /* Count of IE's in this message */
        IISDNu8bit              ie_more;                /* More IEs in next message (0 = no) */
        IISDNu8bit     no_send_channid; /*=1 do not send Channel ID IE while in state 25 */
        IISDNu8bit              pad2;                   /* preserve 32-bit alignment */
        IISDN_IE_STRUCT         ie;                             /* String of IE's of ie_count */
        } IISDN_AL_CON_DATA;

/* SETUP ACK message data for L4L3mSETUP_ACK_REQUEST */
typedef struct
        {
    IISDNu32bit         bchannel_mask;  /* Bit mask used for multi-rate
                                                                                1 => In-Service; 0 => Out-of-Service
                                                                                B channels numbered [1..24]
                                                                                bit => (1 << channel_num). This is the
                                                                                slot map for multi-rate calls 
                                                                                Normally set to 0x00FFFFFF for all 24
                                                                                channels.  Channel 1 being the lsb and 
                                                                                24 the msb */
        IISDN_PROGRESS_IND      progress_ind;
        IISDNu8bit              bchannel;               /* B-channel for this call */
        IISDNu8bit              iface;                  /* Interface for this call */
        IISDNu8bit              ie_count;               /* Count of IE's in this message */
        IISDNu8bit              ie_more;                /* More IEs in next message (0 = no) */
        IISDN_IE_STRUCT         ie;                             /* String of IE's of ie_count */
        } IISDN_SETUP_ACK;

/* Cause and user-info in clearing messages */
#define IISDN_DIAG_LEN 20
typedef struct
        {
        IISDNu8bit      coding_standard;        /* IISDNcodXXXX */
        IISDNu8bit      location;                       /* IISDNlocXXXX */
        IISDNu8bit      cause_val;                      /* 0 means use default or not avail
                                                                                                or IISDNcausXXXX */
        IISDNu8bit      diag_len;                       /* 0 if not available */
        IISDNu8bit      diags[IISDN_DIAG_LEN];
        } IISDN_CAUSE;

/* Keypad IE */
typedef struct
        {
        IISDNu16bit     num_digits;                                     /* NOTE: Digits are ASCII! */
        IISDNu16bit     pad;                                            /* Longword alignment */
        IISDNu8bit      digits[IISDN_MAX_DIGITS];
        } IISDN_KEYPAD;

/* INFO message data for L4L3mINFO_REQUEST and L3L4mINFO */
typedef struct
        {
        IISDNu8bit              sending_complete; /* Flag for the
                                                                                                sending complete IE.
                                                                                                0 - no sending complete IE
                                                                                                non-zero - sending
                                                                                                complete IE included */
        IISDNu8bit              ie_count;               /* Count of IE's in this message */
        IISDNu8bit              ie_more;                /* More IEs in next message (0 = no) */
        IISDNu8bit              pad;                    /* Forces long word alignment */
        IISDN_CALLED_PARTY      called_party;   /* num_dig=0 is valid in some cases */
        IISDN_KEYPAD                    keypad;                 /* keypad digits */
        IISDN_CAUSE                     cause;                  /* Cause or .cause_val = 0 */
        IISDN_IE_STRUCT         ie;                             /* String of IE's of ie_count */
        } IISDN_INFO_DATA;

/* Call proceeding message data for L4L3mCALL_PROCEEDING_REQUEST 
        and for L3L4mCALL_PROCEEDING 
 */
typedef struct
        {
        IISDN_Q922_DLCI         fr_dlci;                /* Q.933 DLCI negotiation */
    IISDNu32bit         bchannel_mask;  /* Bit mask used for multi-rate
                                                                                1 => In-Service; 0 => Out-of-Service
                                                                                B channels numbered [1..24]
                                                                                bit => (1 << channel_num). This is the
                                                                                slot map for multi-rate calls 
                                                                                Normally set to 0x00FFFFFF for all 24
                                                                                channels.  Channel 1 being the lsb and 
                                                                                24 the msb */
        IISDNu8bit              bchannel;               /* B-channel for this call */
        IISDNu8bit              iface;                  /* Interface for this call */
        IISDNu8bit              ie_count;               /* Count of IE's in this message */
        IISDNu8bit              ie_more;                /* More IEs in next message (0 = no) */
        IISDN_PROGRESS_IND      progress_ind;
        IISDN_IE_STRUCT         ie;                             /* String of IE's of ie_count */
        } IISDN_CALL_PROC_DATA;


/* Progress indicator definition for L4L3mPROGRESS_REQUEST, L3L4mPROGRESS */
typedef struct
        {
        IISDNu8bit      coding_standard;        /* IISDNcodXXXX */
        IISDNu8bit      location;                       /* IISDNlocXXXX */
        IISDNu8bit      progress_dscr;          /* IISDNprogXXXX or 0 to use default */
        IISDNu8bit      spare;                          /* Unused */
        IISDN_CAUSE             cause;                          /* Cause or .diag_len = 0 */ 
        IISDN_USER_INFO user_info;                      /* User-User Information or .len = 0 */
        IISDNu8bit      ie_count;                       /* Count of IE's in this message */
        IISDNu8bit      ie_more;                        /* More IEs in next message (0 = no) */
        IISDNu16bit     pad;
        IISDN_CALL_ID   call_id;                /* Call ID for RLT */
        IISDN_IE_STRUCT ie;                                     /* String of IE's of ie_count */
        } IISDN_PROGRESS;

/* Data for L3L4mDISCONNECT, L3L4mCLEAR_REQUEST, L4L3mCLEAR_REQUEST */
typedef struct
        {
        IISDN_CAUSE                     cause;                  /* Cause or .diag_len = 0 */ 
        IISDN_USER_INFO         user_info;              /* User-User Information or .len = 0 */
        IISDN_PROGRESS_IND      progress_ind;
        IISDN_CONNECTED_ADDRESS connected_address;/* Connected Address (1TR6) */
        IISDN_R2_CALL_STATUS    r2_call_status; /* use is dependent on call direction */
        IISDNu8bit      aoc[16];                /* "Parsed" Advice of Charge (AOC) info */
        IISDNu8bit      ie_count;               /* Count of IE's in this message */
        IISDNu8bit      ie_more;                /* More IEs in next message (0 = no) */
        IISDNu16bit     pad;
        IISDN_IE_STRUCT         ie;                             /* String of IE's of ie_count */
        } IISDN_CLR_DATA;

/* Suspend/Resume Data (Not supported by AT&T or Northern Telecom) */
#define IISDN_CALLID_LEN                10
typedef struct 
        {
        IISDNu8bit      bchannel;                               /* For L4L3mRESUME_ACK */
        /* Call id is optional for suspend request/resume request messages */
        IISDNu8bit      id_len;                                 /* 0 means not used */
        IISDNu8bit      call_id[IISDN_CALLID_LEN];/* IA5 Text string to ID call */
        IISDN_CAUSE             cause;                                  /* Cause or .diag_len = 0 
                                                                                                For _REJECT messages */
        } IISDN_SUSP_RES_DATA;

/* Q931 B channel identifier for L4L3mENABLE_B_CHANNEL, L4L3mDISABLE_B_CHANNEL
 * and L4L3mRESTART.  Also identifies B channel for L4L3mENABLE_CAS and 
 * L4L3mDISABLE_CAS msgs.
 */
typedef struct
        {
    IISDNu32bit bchannel_mask;  /* Bit mask used for multi-rate
                                                                                1 => In-Service; 0 => Out-of-Service
                                                                                B channels numbered [1..24]
                                                                                bit => (1 << channel_num). This is the
                                                                                slot map for multi-rate calls 
                                                                                Normally set to 0x00FFFFFF for all 24
                                                                                channels.  Channel 1 being the lsb and 
                                                                                24 the msb */
        IISDNu8bit      bchannel;               /* Bchannel number 1..X . If use_bit_mask
                                                                                field is non zero then
                                                                                this field is ignored and should
                                                                                be set to 0. */
        IISDNu8bit      iface;                  /* Interface 0..19 or 0xFF if unknown */
        IISDNu8bit      busy_out_chan;  /* 1 => Go offhook when disabling b channel (sw56 only) */
        IISDNu8bit      use_bit_mask;   /* If non-zero then n_bchannel is used,
                                                                                else bchannel is used.  Allows
                                                                                specification of a single or an
                                                                                entire interface worth of B-channels.
                                                                                For L4L3mRESTART must be set to 0. */
        IISDNu32bit     n_bchannel;             /* Bit mask used for Q.931 NFAS modes only.
                                                                                Used in generating SERVICE_ACK messages
                                                                                1 => In-Service; 0 => Out-of-Service
                                                                                B channels numbered [1..23]
                                                                                        or [1..30] for E1
                                                                                bit => (1 << channel_num) 
                                                                                bit 0 and high bits are ignored
                                                                                bit 24 is the D-channel
                                                                                Normally set to 0x00FFFFFE. */
        IISDNu8bit   signalling_type;/* Use IISDNsigtypeXXXX */ 
        IISDNu8bit   jate_redial_method;    /* specifies JATE redial restriction mode in 
                                                                                L4L3mENABLE_CAS. Use values IISDNvarJATE_REDIAL_XXX */
        IISDNu16bit  pad1;
        /* for L4L3mENABLE_CAS.....(Note: the first enable_cas on a bchannel
         * inits the entire span containing that bchannel)
         */
        union { 
                IISDN_ROBBED_BIT_DATA   robbed_bit_data;
                IISDN_E1_CAS_R2_DATA    e1_cas_r2_data;  /* for IISDNsigtypeR2_CAS,_MF */
        } cas_params;
} IISDN_BCHANNEL_ID;

/* IISDNsigtypeSIGNALLING_TYPE_START and IISDNsigtypeSIGNALLING_TYPE_END
 * are used to indicate the valid range of field IISDN_E1_CAS_R2_DATA.signalling_type.
 * 
 * IISDNsigtypeLEC_NETWORK indicates the whole range of CAS protocols used in the US.
 * The actual trunk type is specified in fields IISDN_ROBBED_BIT_DATA.in_trunk_type and
 * IISDN_ROBBED_BIT_DATA.out_trunk_type (see constants IISDNttXXXX).
 *
 * If signalling types are added or removed, don't forget to adjust these constants.
 */
#define IISDNsigtypeSIGNALLING_TYPE_START       0

#define IISDNsigtypeRBS_PULSE                   0
#define IISDNsigtypeRBS_DTMF                    1 /* For backward compatibility */
#define IISDNsigtypeE1_CAS                      2 /* For backward compatibility */
#define IISDNsigtypeR2_CAS                      3
#define IISDNsigtypeQ931                        4 /* for q931 with call progress tones etc */
                                                  /* for backward compatibility */
#define IISDNsigtypeR2_MF                       5

#define IISDNsigtypeLEC_NETWORK                 0 /* extension of as RBS_PULSE  */
#define IISDNsigtypeTRANSPARENT                 6 /* host handles protocol      */
#define IISDNsigtypeANALOG                      7

#define IISDNsigtypeSIGNALLING_TYPE_END         8

#define IISDNsigtypeENTRIES   (IISDNsigtypeSIGNALLING_TYPE_END - IISDNsigtypeSIGNALLING_TYPE_START)

/* For L4L3mX25_RESTART and L3L4mX25_RESTART_IND messages */
typedef struct
        {
        IISDN_CAUSE     cause;
        IISDNu8bit modulo128;   /* 1 -> module 128 0 -> module 8 */

#define IISDN_X25_RESTART_REQUEST               0x00
#define IISDN_X25_RESTART_CONFIRM               0x01

        IISDNu8bit request_confirm;     /* L3L4m only, request or confirm received, see macros above */
        IISDNu16bit pad1;
        } IISDN_X25_RESTART;

/* For L4L3mX25_RESET message */
typedef struct
        {
        IISDN_CAUSE     cause;
        } IISDN_X25_RESET;

/* For L3L4mX25_RESET_IND message */
typedef struct
        {
#define IISDN_X25_RESET_REQUEST         0x00
#define IISDN_X25_RESET_CONFIRM         0x01
#define IISDN_X25_RESET_COLLISION       0x02
        IISDNu8bit      request_confirm;
                                /* See macros above: reset request or confirm received, or reset collision */ 
        IISDNu8bit      pad;
        IISDNu16bit     pad1;
        } IISDN_X25_RESET_IND;

/* For L4L3mX25_CONNECT message */
typedef struct
        {
        IISDN_CALLED_PARTY      called_party;   /* called DTE address */
        IISDN_CALLING_PARTY     calling_party;  /* calling DTE address */
        } IISDN_X25_CONNECT;

/* Raw/Undecoded Q.931 Data when IISDNctAPEND_RAW_QMSG is set */
/* This mode of operation sends a raw Q.931 message after the decoded one */

#ifdef RELEASE_8
#define IISDN_MAX_RAWQ  (L3L4_BUF_SIZE - 24)
#else
#define IISDN_MAX_RAWQ  (L3L4_BUF_SIZE - 20)
#endif

typedef struct
        {
        IISDNu16bit     len;    /* Length of the raw buffer */
        IISDNu8bit      more;   /* 0 => end of message, 1=> another to follow */
        IISDNu8bit      pad;    /* preserve word alignment */
        IISDNu8bit      qdata[IISDN_MAX_RAWQ];
        } IISDN_RAW_QDATA;

/* Facility message data for L4L3mFACILITY_REQUEST only DMS-250 Release Link 
   Trunk supported at this time.  This message causes the IISDN controller to 
   transmit a FACILITY message with an INVOKE, RLT THIRD PARTY msg 
   The IE struct is for any additional ies the user may want to send. */
typedef struct
        {
        IISDN_CALL_ID           call_id;                /* received from the net in ALERTING msg */
        IISDNu8bit      ie_count;               /* Count of IE's in this message */
        IISDNu8bit      ie_more;                /* More IEs in next message (0 = no) */
        IISDNu16bit     rlt_disable;            /* if 1 do not send the DMS-250 specific RLT IE  */
        IISDN_IE_STRUCT ie;                             /* String of IE's of ie_count */
        } IISDN_FACILITY_DATA;

/* Defines for status */
#define IISDNaniREJECTED                        1       /* ANI unavailable, FACILITY rejected, */
                                                                        /*      facility not subscribed, or */
                                                                        /*  facility not implemented. */
#define IISDNaniACCEPTED                        2       /* ANI information included */
#define IISDNaniERROR                           3       /* Error in request */

typedef struct
        {
        IISDNu8bit              status;                 /* IISDNaniXXX */
        IISDNu8bit              filler;                 /* Reserved for future use */
        IISDNu8bit              pad1;
        IISDNu8bit              pad2;                   /* Longword alignment */
        IISDN_CALLING_PARTY     calling_party;  /* num_digits=0 means not available */
        } IISDN_ANI_DATA;

/* Data sent in a L3L4mPROTOCOL_STATUS message */
/* Data sent in a L3L4mD_CHANNEL_STATUS message */
/* LAP-D channel status */
#define IISDNdsNOT_ESTABLISHED  0       /* D channel not enabled */
#define IISDNdsESTABLISHING             1       /* In process of enabling D chan */
#define IISDNdsESTABLISHED              2       /* Channel is enabled */

/* Level 2 detail in L3L4mD_CHANNEL_STATUS message */
#define IISDNdsmskNULL                          0x0000  /* No detail available */
#define IISDNdsmskDM_RCVD                       0x0001  /* DM received with F bit set */
#define IISDNdsmskSABME_RCVD            0x0002  /* SABME/SABM received */
#define IISDNdsmskSABME_SENT            0x0004  /* SABME/SABM sent */
#define IISDNdsmskFRAME_MOD_8           0x0008  /* Rcvd frame w/ non-integral octets */
#define IISDNdsmskBAD_CRC                       0x0010  /* Rcvd frame w/ bad CRC */
#define IISDNdsmskBAD_LEN                       0x0020  /* Rcvd frame w/ bad length */
#define IISDNdsmskUNKN_CTRL                     0x0040  /* Rcvd frame w/ unknown cntl field */
#define IISDNdsmskUNKN_DLCI                     0x0080  /* Rcvd frame w/ unknown address */
#define IISDNdsmskUNEXPECTED            0x0100  /* Rcvd valid msgtype in bad state */
#define IISDNdsmskDISC_RCVD                     0x0200  /* Received disconnect message */
#define IISDNdsmskT200                          0x0400  /* T200/N200 Timeout */
#define IISDNdsmskXID_RCVD                      0x0800  /* XID received (V.120 UI mode) */
#define IISDNdsmskUA_RCVD                       0x1000  /* UA received */
#define IISDNdsmskONR_ERROR                     0x2000  /* Bt8474/8 ONR Error */

/* Level 2 detail in L3L4mPROTOCOL_STATUS message */
#define  IISDNdsmskOUT_OF_SERVICE             0x0000
#define  IISDNdsmskREMOTE_PROCESSOR_OUTAGE    0x0001
#define  IISDNdsmskREMOTE_PROCESSOR_RECOVERED 0x0002
#define  IISDNdsmskIN_SERVICE                             0x0003
#define  IISDNdsmskRTB_CLEARED                            0x0004
#define  IISDNdsmskRB_CLEARED                             0x0005
#define  IISDNdsmskBSNT                                   0x0006
#define  IISDNdsmskRETRIEVAL_COMPLETE             0x0007

/* Defines for the all_calls_dropped field (Q.931 channels only) */
#define IISDNacdNO_CHANGE                       0               /* No calls were dropped */
#define IISDNacdRESTART                 1               /* All calls dropped */
#define IISDNacdDCHAN_SWITCHING 2               /* NFAS, all non-stable calls dropped */
                                                                                /*      The D-channel is in the process of
                                                                                        switching. */
#define IISDNacdDCHAN_FAULT             3               /* For NFAS, the backup D-channel did */
                                                                                /*      not come into service.  No D-channel
                                                                                        is available for this NFAS group.
                                                                                        All calls have been dropped. */

/* NFAS D primary/backup D channel status in L3L4mD_CHANNEL_STATUS message */
#define IISDNdcsNA                              0       /* Not availiable */
#define IISDNdcsSTBY                            1       /* Standby */
#define IISDNdcsIS                              2       /* In-Service */
#define IISDNdcsOOS                             3       /* Out-of-Service */
#define IISDNdcsMOOS                            4       /* Not currently implemented */
#define IISDNdcsMB                              5       /* Maint. Busy */
#define IISDNdcsWAIT                            6       /* Changing states.  Waiting for net. */

/* LAP-D/LAP-B/LAP-F States.  Per ITU specifications. */
#define IISDNlpdsTEI_UNASSIGNED                 0
#define IISDNlpdsTEI_ASSIGNED                           4
#define IISDNlpdsAWAITING_ESTABLISHMENT 5
#define IISDNlpdsAWAITING_RELEASE                       6
#define IISDNlpdsMULTIFRAME_ESTABLISHED 7
#define IISDNlpdsTIMER_RECOVERY                 8
#define IISDNlpdsINVALID                                        0xFF    /* Internal use only */

/* MDL-ERROR codes for LAP-D/LAP-B/LAP-F errors per ITU */
#define IISDNl2errNO_ERROR      0       /* If no MDL-ERROR is reported */
                                        /* Receipt of Unsolicited responses */
#define IISDNl2errA                     1       /* Error Code A: Supervisory (F=1) */
#define IISDNl2errB                     2       /* Error Code B: Rcv DM(F=1)*/
#define IISDNl2errC                     3       /* Error Code C: Rcv UA(F=1) */
#define IISDNl2errD                     4       /* Error Code D: Rcv UA(F=0) */
#define IISDNl2errE                     5       /* Error Code E: Rcv DM(F=0) */
                                        /* Peer initiated re-establishment */
#define IISDNl2errF                     6       /* Error Code F: SABME rcvd */
                                        /* Unsuccessful retransmission (N200 times) */
#define IISDNl2errG                     7       /* Error Code G: SABME */
#define IISDNl2errH                     8       /* Error Code H: DISC */
#define IISDNl2errI                     9       /* Error Code I: Status Enquiry */
                                        /* Other */
#define IISDNl2errJ                     10      /* Error Code J: N(R) error*/
#define IISDNl2errK                     11      /* Error Code K: Rcv FRMR response */
#define IISDNl2errL                     12      /* Error Code L: Rcv non-implemented frame */
#define IISDNl2errM                     13      /* Error Code M: Rcv I field not permitted */
#define IISDNl2errN                     14      /* Error Code N: Rcv Frame w/wrong size */
#define IISDNl2errO                     15      /* Error Code O: N201 error */
#define IISDNl2errN202     16 /* Ident Req has been sent N202 times */

/* Ethernet interface speed. */
#define IISDNespeedUNKNOWN              0
#define IISDNespeed10Mbps               1
#define IISDNespeed100Mbps              2

/* Ethernet interface mode. */
#define IISDNemodeUNKNOWN               0
#define IISDNemodeHALF_DUPLEX   1
#define IISDNemodeFULL_DUPLEX   2

typedef struct
        {
        IISDNu16bit     lli;

/* Macros for status field */
#define IISDN_Q933A_PVC_ACTIVE  0x02
#define IISDN_Q933A_PVC_DELETE  0x04
#define IISDN_Q933A_PVC_NEW             0x08

        IISDNu8bit      status;
        IISDNu8bit      pad;
        
        } IISDN_Q933A_PVC_STATUS;

typedef struct
        {
                IISDNu8bit              num_pvcs;       /* number of pvcs in this message */
                IISDNu8bit              pad[3];         /* preserve longword alignment */
                IISDN_Q933A_PVC_STATUS q933a_pvcs[IISDN_MAX_VC_PER_CHAN-1];     /* for IISDNl3modQ933A_ANNEX_A */
        } IISDN_Q933A_PVC;

typedef struct
        {
        IISDNu8bit              status;         /* IISDNdsXXXX protocol channel status */
        IISDNu8bit              l2_state;       /* LAP-D state IISDNlpdsXXX (if L2 enabled) */
        IISDNu8bit              l2_error;       /* MDL-ERROR code or IISDNl2errNO_ERROR */
        IISDNu8bit              l2_errpt;       /* For MDL-ERROR, unique error point */
        /* The next 2 fields are bit masks.  B channel bit is (1 << channel) */
        IISDNu32bit             b_channels;     /* Report which B channels are up.
                                                                                For NFAS this field is not used.
                                                                                Instead, status is given in
                                                                                n_b_channels. */
        IISDNu32bit             b_chan_req;     /* B channels L4 wants to be up.
                                                                                For NFAS this field is not used.
                                                                                Instead, status is given in
                                                                                n_b_chan_req. */
        IISDNu32bit             txcount;        /* Number of packets transmitted */
        IISDNu32bit             rxcount;        /* Number of packets received */
        IISDNu16bit             l2_detail;      /* IISDNdsmsk* if enabled w/ ena_l2_detail */
        IISDNu16bit             l2_detail_data; /* this is introduced to send data*/
        IISDNu16bit             pad;           
        IISDNu8bit              all_calls_dropped;      /* IISDNacdXXX */
        IISDNu8bit              num_q933a_pvcs; /* for IISDNl3modQ933A_ANNEX_A */
        IISDNu32bit             n_b_channels[IISDN_NUM_DS1_INTERFACES]; /* NFAS, Report which B channels are up */
        IISDNu32bit             n_b_chan_req[IISDN_NUM_DS1_INTERFACES]; /* NFAS, B channels L4 want to be up */
        IISDNu8bit              nfas_primary_dchan_status;      /* IISDNdcsXXX. State of NFAS primary lapdid or 0xff if not NFAS. */
        IISDNu8bit              nfas_backup_dchan_status;       /* IISDNdcsXXX. State of NFAS backup lapdid or 0xff if not defined or not NFAS. */
        IISDNu8bit              ethernet_speed; /* IISDNespeedXXXX (currently read only). */
        IISDNu8bit              ethernet_mode;  /* IISDNemodeXXXX (currently read only). */
        IISDNu32bit             connectBPS;     /* final connect bps rate for modem and V.110 */
        IISDNu32bit             connectTyp;     /* IISDNcalltypXXXX */
        IISDN_L2_IP_CONSTS      ip;                     /* for IISDNl2modUDP_IP */
        IISDN_Q933A_PVC_STATUS q933a_pvcs[IISDN_MAX_VC_PER_CHAN-1];     /* for IISDNl3modQ933A_ANNEX_A */
        IISDNu32bit             amf_status[4];/* AMF status for IISDNcalltypAMF */
        } IISDN_PROTOCOL_STAT;

/* B channel status in L3L4mB_CHANNEL_STATUS message */
#define IISDNbcsOUT_OF_SERVICE  0               /* Channel is out of service */
#define IISDNbcsRESTARTING              1               /* Channel is restarting */
#define IISDNbcsMAINTENANCE             2               /* Channel is in maint. mode */
#define IISDNbcsIN_SERVICE              3               /* Channel is in service */

/*
 * Level 2 Statistics Counts in L3L4mL2_STATS message
 *      Response to an L4L3mREQ_L2_STATS message
 *      -       Counts are Zeroed out after generating this message
 *      -       Only one set of counts is maintained per lapdid so if more than
 *              one virtual ckt is run on a channel, the counts are an aggregate
 */     
typedef struct
        {
        IISDNu16bit     iframe_tx;              /* Count of IFRAME commands sent */
        IISDNu16bit     rr_cmd_tx;              /* Count of RR commands sent */
        IISDNu16bit     rnr_cmd_tx;             /* Count of RNR commands sent */
        IISDNu16bit     rej_cmd_tx;             /* Count of REJ commands sent */
        IISDNu16bit     sabm_tx;                /* Count of SABM commands sent */
        IISDNu16bit     sabme_tx;               /* Count of SABME commands sent */
        IISDNu16bit     disc_tx;                /* Count of DISC commands sent */
        IISDNu16bit     rr_rsp_tx;              /* Count of RR responses sent */
        IISDNu16bit     rnr_rsp_tx;             /* Count of RNR responses sent */
        IISDNu16bit     rej_rsp_tx;             /* Count of REJ responses sent */
        IISDNu16bit     dm_tx;                  /* Count of DM responses sent */
        IISDNu16bit     ua_tx;                  /* Count of UA responses sent */
        IISDNu16bit     frmr_tx;                /* Count of FRMR responses sent */
        IISDNu16bit     iframe_rx;              /* Count of IFRAME commands rcvd */
        IISDNu16bit     rr_cmd_rx;              /* Count of RR commands rcvd */
        IISDNu16bit     rnr_cmd_rx;             /* Count of RNR commands rcvd */
        IISDNu16bit     rej_cmd_rx;             /* Count of REJ commands rcvd */
        IISDNu16bit     sabm_rx;                /* Count of SABM commands rcvd */
        IISDNu16bit     sabme_rx;               /* Count of SABME commands rcvd */
        IISDNu16bit     disc_rx;                /* Count of DISC commands rcvd */
        IISDNu16bit     rr_rsp_rx;              /* Count of RR responses rcvd */
        IISDNu16bit     rnr_rsp_rx;             /* Count of RNR responses rcvd */
        IISDNu16bit     rej_rsp_rx;             /* Count of REJ responses rcvd */
        IISDNu16bit     dm_rx;                  /* Count of DM responses rcvd */
        IISDNu16bit     ua_rx;                  /* Count of UA responses rcvd */
        IISDNu16bit     frmr_rx;                /* Count of FRMR responses rcvd */
        IISDNu16bit     crc_errors;             /* Count of CRC errors (RX) */
        IISDNu16bit     rcv_errors;             /* Count of non-CRC errors (RX) */
        IISDNu16bit     retrans_cnt;    /* Count of retransmitted iframes */
        IISDNu16bit     poll_errors;    /* Count of polls not responded to */
        IISDNu16bit     ui_tx;                  /* Count of UI frames transmitted */
        IISDNu16bit     ui_rx;                  /* Count of UI frames received */
        /* remember that link.a expects this structure to be 64 bytes long. */
        } IISDN_L2_STATS;

        typedef struct{
        IISDNu32bit  MSUrcvedCount;
        IISDNu32bit  MSUtxmittedCount;
        IISDNu32bit  MSUrcvErrsCount;
        IISDNu32bit  MSUreTxmittedCount;
        IISDNu32bit  NegAckCount;
        IISDNu32bit  OctetsRcvedCount;
        IISDNu32bit  OctetsTxmittedCount;
        } IISDN_MTP2_STATS;

        typedef struct
        {
                IISDNu8bit mode;
                IISDNu8bit pad[3];
                union 
                {
                        IISDN_L2_STATS lap;
                        IISDN_MTP2_STATS mtp2;
                }stats;
        } IISDN_L2_MTP2_STATS;

/* solaris x86 #defines CS & ES because it is a register in the Intel process, this causes the following
   structure to fail, the workaround is to undefine it*/
#ifdef CS
#undef CS
#endif
#ifdef ES
#undef ES
#endif

/* ---- Performance Monitoring ---- */
/* RFC 1406 information available for PRI and BRI */
/* Error Events, Performance Defects, Performance Parameters */
/* not all gauges are relevant to all interface types T1/E1/BRI-U/BRI-ST */
typedef struct
        {
        /* Error Events */
        IISDNu32bit     BPV;    /* Bipolar Violations */
        IISDNu32bit     EXZ;    /* Excessive Zeroes */
        IISDNu32bit     LCV;    /* Line Coding Violations */
        IISDNu32bit     PCV;    /* Path Coding Violations */
        IISDNu32bit     CS;     /* Controlled Slips */
        /* Performance Defects */
        IISDNu32bit     OOFS;   /* Out Of Frame Seconds */
        IISDNu32bit     AISS;   /* Alarm Indication Signal Seconds */
        /* Performance Parameters */
        IISDNu32bit     LES;    /* Line Errored Seconds */
        IISDNu32bit     CSS;    /* Controlled Slip Seconds */
        IISDNu32bit     ES;     /* Errored Seconds */
        IISDNu32bit     BES;    /* Bursty Errored Seconds */
        IISDNu32bit     SES;    /* Severely Errored Seconds */
        IISDNu32bit     SEFS;   /* Severely Errored Framing Seconds */
        IISDNu32bit     DM;     /* Degraded Minutes */
        IISDNu32bit     UAS;    /* Unavailable seconds */
        } IISDN_PP;


/* Performance Monitoring Request/Response Commands */

#define IISDNpmcGET_THRSHLDS_15MIN                                      0
#define IISDNpmcSET_THRSHLDS_15MIN                                      1
#define IISDNpmcGET_THRSHLDS_24HR                                       2
#define IISDNpmcSET_THRSHLDS_24HR                                       3
#define IISDNpmcRESET_PM_COUNTERS                                       4
#define IISDNpmcGET_15MIN_DATA                                          5
#define IISDNpmcGET_24HR_DATA                                           6
#define IISDNpmcPLB_ACTIVATE                                            7
#define IISDNpmcPLB_DEACTIVATE                                          8
#define IISDNpmcLLB_ACTIVATE                                            9
#define IISDNpmcLLB_DEACTIVATE                                          10
#define IISDNpmcSEND_FDL_REQUEST                                        11

#define IISDN_PP_NUM_INTERVALS_PER_PACKET                       6


typedef struct
        {
        IISDNu8bit      line;                           /* IISDNlineX */
        IISDNu8bit      endx;                           /* IISDNpmeXXXX */
        IISDNu8bit      pm_cmd;                         /* IISDNpmcXXXX */
        IISDNu8bit      fdl_request;            /* see ATT TR 54016 Table B */
        IISDNu8bit      interval_id;            /* 0-95 or 255 for current interval */
        IISDNu8bit      num_intervals;          /* 0-IISDN_PP_NUM_INTERVALS_PER_PACKET */
        IISDNu16bit     pad;                            /* preserve long word alignment */
        IISDN_PP        threshold;                      /* 0 entries are disabled */
        } IISDN_PM_REQ_DATA;                    /* L4 => L3 only */

/* Performance Monitoring Status */
#define IISDNpmsSUCCESS                                 0
#define IISDNpmsFAIL_GENERAL                    1
#define IISDNpmsFAIL_UNKNOWN_REQUEST    2

typedef struct
        {
        IISDNu8bit      line;                                                   /* IISDNlineX */
        IISDNu8bit      endx;                                                   /* IISDNpmeXXXX */
        IISDNu8bit      pm_cmd;                                                 /* IISDNpmcXXXX */
        IISDNu8bit      requested_interval_id;                  /* 0-95 or 255 ival in progress */
        IISDNu8bit      pm_status;                                              /* IISDNpmsXXXX */
        IISDNu8bit      current_interval_id;                    /* 0-95 */
        IISDNu16bit     seconds_in_current_interval;    /* 0-899 (15 mins) */
        IISDNu8bit      num_valid_intervals;                    /* 0-96 */
        IISDNu8bit      num_intervals;                                  /* 0-IISDN_PP_NUM_INTERVALS_PER_PACKET */       
        IISDNu16bit  pad;                                                       /* preserve longword alignment */
        union {
                IISDN_PP                interval[IISDN_PP_NUM_INTERVALS_PER_PACKET];    /* num_intervals idx */
                IISDN_PP                threshold;
                IISDN_PP                last24hours;
                } pp;
        } IISDN_PM_RSP_DATA;                            /* L3 => L4 only */

/* Performance Monitor Alert Types */
#define         IISDNpmaTHRESHOLD_XING          0
#define         IISDNpmaFDL_RX_MOP                      1
#define         IISDNpmaFDL_RX_BOP                      2

/* Performance Monitor end identifier codes */
#define         IISDNpmeNEAR_END                        0
#define         IISDNpmeFAR_END                         1

/* Performance Monitor Interval Sizes */
#define         IISDNpmi15MIN                           0
#define         IISDNpmi24HR                            1

/* Performance Monitor threshold crossing masks */
#define IISDNpmtcmBPV           0x0001  /* Bipolar Violations exceeded threshold */
#define IISDNpmtcmEXZ           0x0002  /* Excessive Zeroes exceeded threshold */
#define IISDNpmtcmLCV           0x0004  /* Line Coding Violations exceeded threshold */
#define IISDNpmtcmPCV           0x0008  /* Path Coding Violations exceeded threshold */
#define IISDNpmtcmCS            0x0010  /* Controlled Slips exceeded threshold */
#define IISDNpmtcmOOFS          0x0020  /* Out Of Frame Seconds exceeded threshold */
#define IISDNpmtcmAISS          0x0040  /* Alarm Indication Signal Seconds exceeded...*/
#define IISDNpmtcmLES           0x0080  /* Line Errored Seconds exceeded threshold */
#define IISDNpmtcmCSS           0x0100  /* Controlled Slip Seconds exceeded threshold */
#define IISDNpmtcmES            0x0200  /* Errored Seconds exceeded threshold */
#define IISDNpmtcmBES           0x0400  /* Bursty Errored Seconds exceeded threshold */
#define IISDNpmtcmSES           0x0800  /* Severely Errored Seconds exceeded... */
#define IISDNpmtcmSEFS          0x1000  /* Severely Errored Framing Seconds exceed... */
#define IISDNpmtcmDM            0x2000  /* Degraded Minutes exceeded threshold */
#define IISDNpmtcmUAS           0x4000  /* Unavailable seconds exceeded threshold */

typedef struct
        {
        IISDNu8bit      line;                                           /* IISDNlineXXXX */
        IISDNu8bit      endx;                                           /* IISDNpmeXXXX */
        IISDNu8bit      interval_size;                          /* IISDNpmiXXXX */
        IISDNu8bit      interval_id;                            /* 0-95 or 0 for 24hr */
        IISDNu32bit     threshold_crossing_mask;        /* IISDNpmtcmXXXX */
        IISDN_PP        interval;
        IISDN_PP        threshold;
        } IISDN_PM_THRESHOLD_XING_DATA;

typedef struct
        {
        IISDNu8bit      line;                                   /* IISDNlineXXXX */
        IISDNu8bit      msg_code;
        IISDNs16bit     len;
        IISDNu8bit      buffer[256];
        } IISDN_PM_FDL_MSG_DATA;

typedef struct
        {
        IISDNu8bit      alert_type;                             /* IISDNpmaXXXX */
        IISDNu8bit      pad[3];
        union
                {
                IISDN_PM_THRESHOLD_XING_DATA    x;
                IISDN_PM_FDL_MSG_DATA                   f;
                } a;
        } IISDN_PM_ALERT_DATA;                          /* L3 => L4 only */

/* End of Performance Monitoring Definitions */

/* Start of Onboard Packet Relay and Routing Service Definitions */
enum IISDN_RELAYdestinations {
        IISDN_RELAYdestDCHAN = 0,
        IISDN_RELAYdestLID,
        IISDN_RELAYdestLID_DEFERRED_PRIORITY,
        IISDN_RELAYdestDROP, 
        IISDN_RELAYdestLIST,
        IISDN_RELAYdestBLOCK,   /* internal use only */
        IISDN_RELAYdestDCHAN_OF_LID
};
#define IISDN_RELAY_MAX_INSERT 16
#define IISDN_RELAY_MAX_COPIES 16
#define IISDN_RELAY_MAX_MATCHES 8
typedef struct {
        struct {
                IISDNu8bit idx;                         /* offset into packet at which comparison is performed */
                IISDNu8bit msk;                         /* bit mask to apply on octet at the idx offset, 0 ends list */
                IISDNu8bit val;                         /* value to expect in octet at idx offset */
                IISDNu8bit pad; 
        } match[IISDN_RELAY_MAX_MATCHES];
        IISDNu8bit  dest;                               /* IISDN_RELAYdestXXXX value */
#ifdef RELEASE_8
        IISDNu8bit  pad;                                /* IISDN_RELAYdestXXXX value */
        IISDNu16bit dest_id;                    /* lapdid or data_interface.data_channel number */
#else
        IISDNu8bit      dest_id;                        /* lapdid or data_interface.data_channel number */
        IISDNu16bit     pad;
#endif
        IISDNu8bit  delBytes;                   /* number of bytes to delete from head of packet */
        IISDNu8bit  insBytes;                   /* number of bytes to insert at head of packet 0..16 */
        IISDNu8bit  insHeader[IISDN_RELAY_MAX_INSERT];          /* bytes to insert when insBytes > 0 */
        IISDNu8bit  insTimeStamp;               /* 0 for no timestamp, 1, 2, 4 bytes wide */
        IISDNu8bit  insTimeStampOffset; /* offset into final packet for timestamp insertion */
} IISDN_RELAY_RULE;

typedef struct {
    IISDNs32bit recv_cnt;       /* number of packets received on lapdid */
    IISDNs32bit recv_bytes; /* number of bytes received on lapdid */
        IISDNs32bit drop_cnt;   /* num of recv'd packets sent to IISDN_RELAYdestDROP */
        IISDNs32bit xmit_cnt;   /* num of packets transmitted on lapdid */
    IISDNs32bit xmit_bytes; /* number of bytes transmitted on lapdid */
        IISDNs32bit xmit_deferred_cnt;  /* num of pkts sent from deferred queue */
        IISDNs32bit xmit_overflow_cnt;  /* num of pkts discarded because tx queue full */
        IISDNs32bit current_deferred_queue_depth; /* num of packets awaiting deferred tx */
} IISDN_RELAY_STATS;

/* End of Onboard Packet Relay and Routing Service Definitions */

/* Start of image download commands and responses */
typedef struct {                                /* L3L4mREQ_DOWNLOAD */
        IISDNu8bit module_id;           /* unique IISDN internal handle */
        IISDNu8bit pad[3];                      /* maintain long word alignment */
        IISDNs8bit filename[16];        /* expected name of module image */
        IISDNs8bit version[64];         /* latest version known to work with IISDN */
} IISDN_REQ_DOWNLOAD_DATA;              /* L3L4mREQ_DOWNLOAD */

/* Macros for module ID */
#define IISDNmodidMEMORY        0xFF

typedef struct {                                /* L4L3mDOWNLOAD, L3L4mDOWNLOAD_ACK */
        IISDNu8bit module_id;           /* module_id from original L3L4mREQ_DOWNLOAD */
        IISDNu8bit last;                        /* signals final L4L3mDOWNLOAD for image */
        IISDNu16bit seq_num;            /* when one L4L3mDOWNLOAD msg isn't enough */
        IISDNp8bit bptr;                        /* 32bit phys addr of image */
   IISDNu8bit   pad[3];
        IISDNu32bit length;                     /* num bytes of image at this location */
        IISDNu32bit total_length;       /* complete image size */
        IISDNu8bit buffer[384];         /* used when above bptr is NULL */
} IISDN_DOWNLOAD_DATA;                  /* L4L3mDOWNLOAD, L3L4mDOWNLOAD_ACK */
/* End of image download commands and responses */

/* Start of high availability commands and responses */
typedef struct {                                /* L4L3mSET_HOST_WATCHDOG */
        IISDNu32bit timer;                      /* L3L4 queue watch timer in milliseconds */
                                                                /* 0 default => 5000 */
} IISDN_HOST_WATCHDOG;

typedef struct {                                                /* L3L4mHOST_DEAD, L3L4mHOST_ALIVE */
        IISDNu32bit timestamp;                          /* msecs since IISDN boot when message posted */
        IISDNu8bit death_cause;                         /* IISDNhdeadXXXX value */
#define IISDNhdeadBUSERROR              1               /* death by bus error */
#define IISDNhdeadL3L4TIMEOUT   2               /* death by L3L4 watchdog timeout */
#define IISDNhdeadL3L4FULL              3               /* death by L3L4 queue full */
#define IISDNhdeadL4L3KILL              4               /* death by L4L3mSET_HOST_DEAD */
        IISDNu8bit pad[3];
        IISDNu32bit dropped_L3L4m_count;        /* valid for L3L4mHOST_ALIVE */
} IISDN_HOST_STATUS;
/* End of high availability commands and responses */

/* Start of geographic addressing commands and responses */
typedef struct {
        IISDNu8bit slot;
        IISDNu8bit shelf;
   IISDNu16bit  pad;
} IISDN_GEOGRAPHIC_ADDRESS;
/* End of geographic addressing commands and responses */

/* Start CPU Utilization structures */
#define CPU_UTILIZATION_COMMAND_STOP    0       /* Stop collecting data */
#define CPU_UTILIZATION_COMMAND_START   1       /* Start collecting data */
#define CPU_UTILIZATION_COMMAND_REPORT  2       /* Report data via L3L4mCPU_UTILIZATION_DATA */

#define IISDN_CPU_UTIL_MIN_TIME                         100     /* Reasonable minimum timer granularity. */
#define IISDN_CPU_UTIL_MAX_TIME                         50000   /* Reasonable maximum timer granularity. */

typedef struct {
        IISDNu8bit      command;

        IISDNu8bit      pad[3];
        IISDNu32bit interval_us;        /* CPU Utilization sampling interval: Default 20000 (20ms) */
} IISDN_CPU_UTIL;

#define IISDN_CPU_UTIL_POLLING                              0   /* Polling queues/HW for work */
#define IISDN_CPU_UTIL_SMI_CONTROL                          1   /* Processing L4L3m/L3L4m/HA messages */
#define IISDN_CPU_UTIL_TTY                                  2   /* Debug port/VTTY processing */
#define IISDN_CPU_UTIL_TIMERS                               3   /* Timer expired processing */
#define IISDN_CPU_UTIL_TX_DATA                              4   /* Process data buffers from host */
#define IISDN_CPU_UTIL_TX_DATA_COMPLETE                     5   /* Process tx data buffers complete from HW */
#define IISDN_CPU_UTIL_TX_DATA_MALLOC                       6   /* Allocating tx data buffers */
#define IISDN_CPU_UTIL_TX_DATA_MISC                         7   /* Transmit data buffers (misc. protocol) */
#define IISDN_CPU_UTIL_TX_DATA_IP                           8   /* Transmit data buffers (IP) */
#define IISDN_CPU_UTIL_TX_DATA_LAP                          9   /* Transmit data buffers (LAP) */
#define IISDN_CPU_UTIL_RX_DATA                              10  /* Process rx data buffers from HW */
#define IISDN_CPU_UTIL_RX_DATA_INDICATION_MISC              11  /* Misc. process rx data buffers from HW */
#define IISDN_CPU_UTIL_RX_DATA_MALLOC                       12  /* Allocating rx data buffers */
#define IISDN_CPU_UTIL_RX_DATA_INDICATION_LAP               13  /* LAP process rx data buffers from HW */
#define IISDN_CPU_UTIL_RX_DATA_INDICATION_IP                14  /* IP process rx data buffers from HW */
#define IISDN_CPU_UTIL_RX_DATA_INDICATION_RELAY             15  /* Packet relay process rx data buffers from HW */
#define IISDN_CPU_UTIL_RX_DATA_INDICATION_RELAY_COPY        16  /* Packet relay process rx data buffers from HW with copy */
#define IISDN_CPU_UTIL_RX_DATA_RELAY_DEL                    17  /* Packet relay process rx data buffers - deleting bytes (not implemented) */
#define IISDN_CPU_UTIL_RX_DATA_RELAY_INS                    18  /* Packet relay process rx data buffers - inserting bytes */
#define IISDN_CPU_UTIL_PLX_DMA_CALLBACK_0                   19  /* Polling PLX for DMA completion */
#define IISDN_CPU_UTIL_PLX_DMA_CALLBACK_1                   20  /* Polling PLX for DMA completion */
#define IISDN_CPU_UTIL_CAS                                  21  /* CAS/RBS processing */
#define IISDN_CPU_UTIL_RX_DATA_INDICATION_RELAY_DEFERRED    22  /* Packet relay process rx data buffers - packets with deferred transmission */
#define IISDN_CPU_UTIL_NUM_FUNCTIONAL_AREAS                 23
#define IISDN_MAX_CPU_UTIL_FA_COUNT IISDN_CPU_UTIL_NUM_FUNCTIONAL_AREAS

typedef struct {
        IISDNu8bit      num_functional_areas;
        IISDNu8bit      pad[3];

        IISDNu32bit     total_count;
        IISDNu32bit fa_count[20];

} IISDN_CPU_UTIL_DATA;
/* End CPU Utilization structures */


/* begin DEPRECATE DEPRECATE DEPRECATE DEPRECATE DEPRECATE DEPRECATE */
/* DEPRECATED structures and #defines */
/* The following stuff may be obsoleted in the future */
#define L4L3mCONFIG_SPYDER_T    0xB4    /* Repartition HDLC chip RAM */
#define L4L3mCONFIG_IISDN               0xB5    /* Instant ISDN Memory Configuration */
#define L4L3mENABLE_SCATTER             0xBB    /* enable stream spray channel */
#define L4L3mSET_TDM                    0xBF    /* tdm switching on SCSA mezz boards */
#define L3L4mCFG_ISDN_ACK               0x32    /* Acknowlege L4L3mCONFIG_IISDN */

#define IISDNtsiLINE_0  IISDNtsiLINE_A /* old definitions for compatibility */
#define IISDNtsiLINE_1  IISDNtsiLINE_B
#define IISDNtsiLINE_2  IISDNtsiLINE_C
#define IISDNtsiLINE_3  IISDNtsiLINE_D
#define IISDNtsiLINE_4  IISDNtsiLINE_E
#define IISDNtsiLINE_5  IISDNtsiLINE_F
#define IISDNtsiLINE_6  IISDNtsiLINE_G
#define IISDNtsiLINE_7  IISDNtsiLINE_H

/* before IISDN_LEVEL3_CNFG structure was a union */
#define q931_cnfg cnfg.q931
#define x25_cnfg cnfg.x25
#define l2_mode par.lap.mode

typedef struct
        {
        IISDN_LEVEL1_CNFG       level1;
        IISDNu32bit stream;
        IISDNu8bit hdlc_scatter_map[256];
        } IISDN_ENA_SCATTER;


/* Data structure for reconfiguring HDLC processor buffer pool partitions.
 * !! IISDN 5 ONLY !!
 * The PRI-24 and PRI-48 have a 64K RAM area dedicated to 24 channels
 * The PRI-48XP/PRI-64/PRI-ISA have a 256K RAM area dedicated to 32 channels
 * There are x350 bytes which are not available as buffer pools.
 * The optional Spyder mezzanine card adds an extra 32 channels w/256K RAM
 * Each tx or rx buffer has a x0e (14) byte internal header prefixed to it
 * enable_d_data.level_2_cnfg.n201 specifies buffer size (override)
 * enable_d_data.level_2_cnfg.num_hdlc_txbuf specifies # tx bufs (override)
 * enable_d_data.level_2_cnfg.num_hdlc_rxbuf specifies # rx bufs (override)
 *  for pri-24/pri-48, default is 256 + 6 (L2 header) + 2 (crc) = 264
 *              4 tx buffers, 5 rx buffers
 *              Default partition size 2694 bytes (must be even)
 *  for pri-48XP/pri-64/pri-ISA, default is 512 + 6 (L2 header) + 2 (crc) = 520
 *              4 tx buffers, 5 rx buffers
 *              Default partition size is 8164 bytes (must be even)
 */
typedef struct
        {
        IISDNu32bit     max_rcv_frame;  
        IISDNu32bit     partition[64];  /* Default x1010 (Header + Data + 2-byte CRC) */
        } IISDN_SPYDER_CNFG;


/* This section contains items to support L4L3mSET_TDM and the SCSA bus */
#define IISDNtdmSD_0            0x3000
#define IISDNtdmSD_1            0x3100
#define IISDNtdmSD_2            0x3200  
#define IISDNtdmSD_3            0x3300
#define IISDNtdmSD_4            0x3400
#define IISDNtdmSD_5            0x3500
#define IISDNtdmSD_6            0x3600
#define IISDNtdmSD_7            0x3700
#define IISDNtdmSD_8            0x3800
#define IISDNtdmSD_9            0x3900
#define IISDNtdmSD_10           0x3a00
#define IISDNtdmSD_11           0x3b00
#define IISDNtdmSD_12           0x3c00
#define IISDNtdmSD_13           0x3d00
#define IISDNtdmSD_14           0x3e00
#define IISDNtdmSD_15           0x3f00

#define  IISDN_MAX_SET_TDM 60

typedef struct
        {
        IISDNu16bit port;
        IISDNu16bit listen;
        IISDNu8bit port_mask;
        IISDNu8bit listen_mask;
        IISDNu16bit pad; /* reserved */
        } IISDN_TDM_MAP;
        
typedef struct
        {
        IISDNu16bit num_mappings;
        IISDNu16bit pad;
        IISDN_TDM_MAP tdm_map[IISDN_MAX_SET_TDM];
        } IISDN_TDM_DATA;

#define IISDNtsiTDM_0           0x0000          /* VME PRI-64 TDM Bus, deprecated */

/* IISDN 5.x E&M Robbed Bit Signalling uses lapdid values 0x80 and 0x81
 * with L4L3mENABLE_B_CHANNEL and L4L3mDISABLE_B_CHANNEL messages.
 */
#define IISDN_E_AND_M_T1A_LAPDID 0x80           /* Dual-Channel Controllers only */
#define IISDN_E_AND_M_T1B_LAPDID 0x81           /* Any T1 Netaccess Controller */

/* IISDN 5.x only, for use with L4L3mCONFIG_IISDN */
typedef struct
        {
        IISDNu16bit     max_q931_calls;                         /* Default 32 call records */
        IISDNu8bit      max_q931_restarts;                      /* Default 24 bchannel restarts */
        IISDNu8bit      max_L3_stacks;                          /* Default 1 Q.931 stacks [1..32]  */
        IISDNu8bit      dlci_per_chan;                          /* Default 1 DLCI per hdlc stream */
        IISDNu8bit      max_L3L2_bufs;                          /* Default 20 max 62 internal msg buf */
        IISDNu8bit      max_x25_stacks;                         /* Default 0 X.25 stacks */
        IISDNu8bit      x25_circuits_per_chan;          /* Default 4 X.25 max virtual circuits per X.25 stack */
        IISDNu8bit      config_sw56;                            /* Default 0 AB signalling disabled */
        IISDNu8bit      config_pm;                                      /* Default 0 T1 ESF perf monitoring */
        IISDNu8bit      bonding_numchan;                        /* Default 0 BONDING 64K channels */
        IISDNu8bit      bonding_numcall;                        /* Default 0 BOND aggregate calls */
        IISDNu32bit     device_tx_mem;                          /* default carve 64k for sca txbufs */
        IISDNu32bit     device_rx_mem;                          /* default carve 64k for sca rxbufs */
        IISDNu8bit  smi_overflow_size;                  /* Default 0 overflow SMI entries */
        IISDNu8bit      max_q931_message_segments;      /* Default 0 Q.931 Receive Message Segments */
        IISDNu8bit      pad1;
        IISDNu8bit      pad2;
        IISDNu32bit     tty_buffer_size;                        /* Default 0x1000 (4k) for V/TTY output buffer size */
        } IISDN_SW_CONFIG;
/* end DEPRECATE DEPRECATE DEPRECATE DEPRECATE DEPRECATE DEPRECATE DEPRECATE */

/* begin OBSOLETE OBSOLETE OBSOLETE OBSOLETE OBSOLETE OBSOLETE OBSOLETE */
/* OBSOLETE structures and #defines */
/* One day these structs and defines will simply disappear from iisdn.h */

#define L4L3mOBSOLETE_ENABLE_D_CHANNEL  0xA0    /* Enable a Data channel */
#define L4L3mOBSOLETE_DISABLE_D_CHANNEL 0xA1    /* Disable a Data channel */
#define L4L3mREQ_D_CHAN_STATUS                  0xA5    /* Request D_CHANNEL_STATUS message */
#define IISDN_D_CHAN_STAT                               IISDN_PROTOCOL_STAT
#define L3L4mD_CHANNEL_STATUS                   0x21    /* Data channel status changed */
/* Types of data channel connections supported (obsolete) */
#define IISDNconnINVALID                0x00000000L     /* For internal use only */
#define IISDNconnATT4_Q931              0x00800400L     /* Q.931/Q.921 for AT&T 4ESS Offices */
#define IISDNconnATT5_Q931              0x00810400L     /* Q.931/Q.921 for AT&T 5ESS Offices */
#define IISDNconnNTI_Q931               0x00820400L     /* Q.931 for Northern Telecom DMS-100*/
#define IISDNconnNTI250_Q931    0x00830400L     /* Q.931 for Northern Telecom DMS-250*/
#define IISDNconnATT_NET_Q931   0x00840800L     /* Network side of AT&T Q.931 (4ESS) */
#define IISDNconnMD110_T1               0x00850400L     /* Q.931 for Ericsson MD-110 (U.S.) */
#define IISDNconnMD110_E1               0x01860400L     /* Q.931 for Ericsson MD-110 (Int'l) */
#define IISDNconnSIEMENS                0x00870400L     /* Q.931 for Siemens (U.S.) */
#define IISDNconnLAPD_SYMM              0x00000100L     /* V.120 (symmetric LAP-D) */
#define IISDNconnLAPD_SYMM_R    0x00001100L     /* 7 bits of data, high bit set to 1 */
#define IISDNconnX25_LAPB               0x00002500L     /* X.25 Level 2 (DTE) */
#define IISDNconnX25_LAPB_R             0x00003500L     /* 7 bits of data, high bit set to 1 */
#define IISDNconnDCE_LAPB               0x00002900L     /* X.25 Level 2 (DCE) */
#define IISDNconnDCE_LAPB_R             0x00003900L     /* 7 bits of data, high bit set to 1 */
#define IISDNconnLAPD_USER              0x00000500L     /* Level 2 only.  User side */
#define IISDNconnLAPD_USER_R    0x00001500L     /* 7 bit data, high bit set to 1 */
#define IISDNconnLAPD_NET               0x00000900L     /* Level 2 only.  Net side */
#define IISDNconnLAPD_NET_R             0x00001900L     /* 7 bit data, high bit set to 1 */
#define IISDNconnRAW_HDLC               0x00004100L     /* Unprocessed HDLC frames */
#define IISDNconnRAW_HDLC_R             0x00005100L     /* Unprocessed HDLC frames 7 bit data */
#define IISDNconnRAW_T1                 0x0000C100L     /* Raw T1; no HDLC processing */
#define IISDNconnRAW_T1_VLOCK   0x0000C300L     /* Raw T1; VMEBus locked for xfers */
#define IISDNconnX25_L3_DTE             0x00002540L     /* X.25 Packet Layer over LAP-B - DTE */
#define IISDNconnX25_L3_DCE             0x00002940L     /* X.25 Packet Layer over LAP-B - DCE */

/* Bit settings derivations for the above values (obsolete) */
#define IISDNctFULL_A24                 0x00000001L     /* VME Buffer ptrs access full A24 (PRI-24/PRI-48 only) */
#define IISDNctINVERT_HDLC              0x00000002L     /* Run inverted HDLC */
#define IISDNctL2_DETAIL                0x00000004L     /* L2 errors delivered to control i/f */
#define IISDNctUI_MODE                  0x00000008L     /* LAPD Unnumberd Info Mode for V.120 */
#define IISDNctTIMESTAMP                0x00000010L     /* prefix 1 ms timestamp to rx pkts */
#define IISDNctNO_SABME                 0x00000020L     /* Don't send SABME-TEI Assigned State*/
#define IISDNctX25_PKT_LAYER    0x00000040L     /* Run X.25 Packet Layer over L2 */
#define IISDNctBONDING                  0x00000080L     /* Bonding */
#define IISDNctDATA_INTERFACE   0x00000100L     /* Packets delivered to VME */
#define IISDNctVME_LOCK                 0x00000200L     /* VMEbus held for 128 byte xfer */
#define IISDNctUSER_SIDE_L2             0x00000400L     /* Run User Side L2 (0 for symmetric) */
#define IISDNctNET_SIDE_L2              0x00000800L     /* Run Network Side L2 (0 for symm) */
#define IISDNct56K_RATE_ADAPT   0x00001000L     /* N x 56 Operation */
#define IISDNctLAP_B                    0x00002000L     /* Run X.25 Level 2 instead of Q.921 */
#define IISDNctDISABLE_L2               0x00004000L     /* Disable all L2 packet processing */
#define IISDNctDISABLE_HDLC             0x00008000L     /* Disable HDLC Processing */

/* Lap-D/Lap-B and Q.931 configuration parameters for obsolete Enable_D */
typedef struct
        {       /* All Level 2 timers are in 50 milliSecond tics (1/20 second) */
                /* Setting a field to 0 will cause it to be set to the default */
        IISDNu16bit     t200;   /* X.25 T1. Max retrans time. Default 1 sec */
        IISDNu16bit     t201;   /* X.25 T2. Min Retransmission TEI Identity Check */
        IISDNu16bit     t202;   /* Min TEI Identity Request.  Default Inactive. */
        IISDNu16bit     t203;   /* X.25 T3. Max link inactivity.  Default 30 sec */
        IISDNu16bit     n200;   /* X.25 N2. Max retransmissions.  Default 3 */
        IISDNu16bit     n201;   /* X.25 N1. Max bytes in Iframe including HDR/CRC (See IISDN_SPYDER_CNFG for defaults) */
        IISDNu16bit     n202;   /* LAP D # of TEI assignment */
        IISDNu16bit     K;              /* Max Transmit Window.  Default 7.  Max 127. */

        /* Settings to override internal buffer queues
         * Default internal tx and rx buffer size is 264 with 4 tx bufs/5 rx bufs
         * for channelized (64K/56K) HDLC modes.
         * Default internal tx and rx buffer size is 256 with 4 tx bufs/5 rx bufs
         *  for channelized (64K/56K) RAW_T1 mode.
         * Default internal tx and rx buffer size is 4106 with 7 tx bufs/7 rx bufs
         *  for an HDLC mode superchannel (more than one bit set in hdlc_channels) 
         * Default internal tx and rx buffer size is 128 with 40 tx bufs/40 rx bufs
         *  for a RAW_T1 mode superchannel (more than one bit set in hdlc_channels)
         */
        IISDNu16bit     num_hdlc_txbuf; /* # buffers in internal tx queue or 0 */
        IISDNu16bit     num_hdlc_rxbuf; /* # buffers in internal rx queue or 0 */
        IISDN_Q931_TIMERS q931; /* Q.931 timer values, if any */
        } IISDN_L2_CONFG;

/* Data structure for Enable_Data_Channel (obsolete) */
typedef struct
        {
        IISDNu32bit     conn_type;              /* Use IISDNconnXXX or bits from IISDNctXXX */ 
        IISDNu32bit     b_channels;             /* Bit mask used for Q.931 modes only
                                                                                Used in generating SERVICE_ACK messages 
                                                                                1 => In-Service; 0 => Out-of-Service
                                                                                B channels numbered [1..23]
                                                                                        or [1..30] for E1
                                                                                bit => (1 << channel_num) 
                                                                                bit 0 and high bits are ignored
                                                                                bit 24 is the D-channel
                                                                                Normally set to 0x00FFFFFE. 
                                                                                For NFAS this field is ignored and
                                                                                should be set to 0.  For NFAS use
                                                                                the n_b_channels field. */
        IISDNu32bit     call_type;              /* Bit mask for Q.931 of calls accepted from network.  Use IISDNcalltypXXX */
        IISDNu32bit     hdlc_channels;  /* For data connections with conn_type bit
                                                                                IISDNctDATA_INTERFACE set
                                                                                usually (1 << lapdid)
                                                                                1 => Use this HDLC channel
                                                                                bit 0 <-> HDLC channel 0
                                                                                any combination of 0..23 is valid 
                                                                                 or [0..31] on XP, E1, or ISA */
        IISDNu16bit     data_channel;   /* Data channel when IISDNctDATA_INTERFACE set index [0..255] to IISDN_DCHAN_DESCR array */
        IISDNu16bit     level_2_override;/* 0 => use default values, 1 => use next structure */
        IISDN_L2_CONFG  level_2_cnfg;   /* Timers, etc if level_2_override == 1 */
        IISDN_X25_CONFIG        x25_level_3_cnfg;       /* Valid if IISDNctX25_PKT_LAYER set */
        /* The following pertain to Q931 non-facility associated signalling when IISDNctNFAS bit is set in conn_type */
        IISDNu32bit     n_b_channels[IISDN_NUM_DS1_INTERFACES]; /* NFAS B channels service state */
#ifdef RELEASE_8
        IISDNu16bit     primary_lapdid;
        IISDNu16bit     backup_lapdid;  /* 0xFFFF if no backup channel */
#else
        IISDNu8bit      primary_lapdid;
        IISDNu8bit      backup_lapdid;  /* 0xFF if no backup channel */
#endif
        IISDNu8bit      primary_ifnum;  /* Interface number [0..19] with Dchan */
        IISDNu8bit      backup_ifnum;   /* Int # [0..19] with Dchan or 0xFF */
        IISDNu8bit      backup_control; /* Used for control of NFAS backup. Use
                                                                                IISDNbcXXXX.  Set to 0 for no change
                                                                                or for non-NFAS case. If set no other
                                                                                fields are processed. */
        IISDNs8bit      pad;                    /* Preserve longword alignment */
        IISDNu16bit     pad1;
        IISDNu32bit     q931_config;    /* Additional configuration for layer 3.  Use IISDNcnfgXXXX */
        IISDNu16bit     spid_len;               /* Basic Rate Nat'l ISDN-1 SPID 3..20 */
        IISDNu8bit      spid[IISDN_MAX_SPID_LEN];       /* Service Profile ID (ASCII) */
        } IISDN_ENA_D_DATA;

/* for L3L4mD_CHANNEL_STATUS */
#define d_chan_data pcol_stat_data

/* end OBSOLETE OBSOLETE OBSOLETE OBSOLETE OBSOLETE OBSOLETE OBSOLETE */



/* Data structure used to send control messages from User to Netaccess Controller */
typedef struct
{
    /* lapdid:  main handle with several overloaded definitions.
     * Originally referred to LAPD channel, but applies regardless
     * of LAPD actually being enabled.  Basic channel identifier when 
     * L4L3mENABLE_PROTOCOL is issued (actual physical timeslot(s)
     * associated with the channel are determined by level1.hdlc_channels)
     * lapdid:  Interpreted as IISDNlineNUMBER when L4L3mENABLE_CAS is issued.
     * There are up to 256 of these handles available depending on 
     * board model.  Use L4L3mREQ_BOARD_ID to learn maximum number of 
     * lapdids for a card as well as the default device associated with
     * a lapdid.  The level1.device field may be used to modify default
     * device association.
     */

#ifdef RELEASE_8
    IISDNu16bit   lapdid;
    IISDNu8bit    msgtype;    /* L4L3mXXXXX */
    IISDNu8bit    pad[3];
#else
    IISDNu8bit    lapdid;
    IISDNu8bit    msgtype;    /* L4L3mXXXXX */
#endif

    IISDNu16bit L4_ref;   /* provided by L4 for L3 ind. */
    IISDNu16bit call_ref; /* Must be valid, except for L4L3mCALL_REQUEST */
    IISDNu16bit lli;      /* Logical Link Id or DLCI.  See IISDN_LLI_*** */
    union
    {
        IISDN_CALL_REQ_DATA              call_req_data;        /* For L4L3mCALL_REQUEST */
        IISDN_PROGRESS                   progress_data;        /* For L4L3mPROGRESS_REQUEST */
        IISDN_AL_CON_DATA                al_con_data;          /* For L4L3mALERTING; L4L3mCONNECT, L4L3mCONNECT_ACK */
        IISDN_CALL_PROC_DATA             proc_data;            /* For L4L3mCALL_PROCEEDING_REQUEST */
        IISDN_CLR_DATA                   clr_data;             /* For L4L3mCLEAR_REQUEST */ 
        IISDN_USER_INFO                  user_data;            /* L4L3mUSER_INFO/L4L3mX25_INTERRUPT */
        IISDN_CONTINUE_DATA              continue_data;        /* L4L3mCONTINUE for additional IEs*/
        IISDN_BONDING_DATA               bonding_data;         /* For L4L3mENABLE_BONDING */
        IISDN_ADD_BOND_DATA              add_bond_data;        /* For L4L3mADD_BONDING */
        IISDN_ENA_PROTO_DATA             enable_protocol;      /* For L4L3mENABLE_PROTOCOL */
        IISDN_TSI_DATA                   tsi_data;             /* For L4L3mSET_TSI */
        IISDN_HARDWARE_DATA              hardware_data;        /* For L4L3mSET_HARDWARE */
        IISDN_CONFIG_DATA                config_data;          /* For L4L3mSET_CONFIGURATION */
        IISDN_SOFTWARE_DATA              software_data;        /* For L4L3mSET_SOFTWARE */
        IISDN_COMPARE_DATA               compare_data;         /* For L4L3mCOMPARE */
        IISDN_BCHANNEL_ID                channel;              /* For L4L3mENABLE_B_CHANNEL, L4L3mDISABLE_B_CHANNEL, L4L3mRESTART */
        IISDN_BCHANNEL_ID                cas_data;             /* For L4L3mENABLE_CAS, L4L3mDISABLE_CAS, L4L3mCAS_CHANNEL_BLOCK, L4L3mCAS_CHANNEL_UNBLOCK */
        IISDN_PM_REQ_DATA                pm_req_data;          /* For L4L3mPM_REQUEST */
        IISDN_SUSP_RES_DATA              sus_res_data;         /* For Suspend/Resume messages */
        IISDN_EBUS_CAS                   ebus_cas_data;        /* For L4L3mSET_EBUS_CAS */
        IISDN_FEATURE_REQUEST            feature_req;          /* For L4L3mFEATURE_REQUEST */
        IISDN_UNIVERSAL                  universal;            /* For L4L3mUNIVERSAL */
        IISDN_SPID                       spid_data;            /* For L4L3mSPID */
        IISDN_X25_RESTART                x25_restart_data;     /* For L4L3mX25_RESTART */
        IISDN_X25_RESET                  x25_reset_data;       /* For L4L3mX25_RESET */
        IISDN_X25_CONNECT                x25_connect_data;     /* For L4L3mX25_CONNECT */
        IISDN_SETUP_ACK                  setup_ack_data;       /* For L4L3mSETUP_ACK_REQUEST */
        IISDN_INFO_DATA                  info_data;            /* For L4L3mINFO_REQUEST */
        IISDN_ALARM_DATA                 alarm_data;           /* For L4L3mSEND_ALARM */
        IISDN_IDLE_PATTERN               pattern_data;         /* For L4L3mSET_IDLE_PATTERN, L4L3mCLEAR_IDLE_PATTERN */
        IISDN_FACILITY_DATA              facility_data;        /* For L4L3mFACILITY_REQUEST */
        IISDN_DIAL_DATA                  dial_data;            /* For L4L3mDIAL */
        IISDN_DOWNLOAD_DATA              download_data;        /* for L4L3mDOWNLOAD,L4L3mUPLOAD */
        IISDN_RELAY_RULE                 relay_rule[8];        /* L4L3mRELAY_ADD_RULE, L4L3mRELAY_DEL_RULE */
        IISDN_HOST_WATCHDOG              host_watchdog;        /* L4L3mSET_HOST_WATCHDOG */
        IISDNu8bit                       line;                 /* For L4L3mSET_D_CHANNEL_LAPDID */
        IISDN_CAS_SIGNALING_BITS_DATA    cas_signaling_bits;   /* For L4L3mSET_CAS_SIGNALING_BITS */
        IISDN_SIGNAL_DURATION_DATA       signal_duration_data; /* For L4L3mTX_HOOKFLASH, L4L3mTX_WINK */
        IISDN_CALLER_ID_DATA             caller_id_data;       /* For L4L3mCALL_INFORMATION_RECEIVED */
#ifdef RELEASE_8                     
        IISDNu8bit                       databuf[L4L3_BUF_SIZE - 12];    /* Round up to 512 bytes */
#else                                
        IISDNu8bit                       databuf[L4L3_BUF_SIZE - 8];     /* Round up to 512 bytes */
#endif                               
        IISDN_SW_CONFIG                  sw_config;            /* For L4L3mCONFIG_IISDN */
        IISDN_TONE_CTRL_DATA             tone_data;            /* For L4L3mTONE_CTRL, R2 tones only so far, set signalling_type to IISDNsigtypeR2_MF */
        IISDN_CPU_UTIL                   cpu_util;             /* For L4L3mCPU_UTILIZATION */
        IISDN_BOARD_VERSION              board_version;        /* board version ejl */
        IISDN_Q933A_PVC                  q933a_pvc_data;       /* For L4L3mSET_Q933A_PVC */
        IISDN_CONTROL_DATA_TX            control_data_tx;      /* For L4L3mCONTROL_DATA; for direct to LAP Call Control */
        IISDN_JATE_REDIAL                jate_redial;          /* For L4L3mJATE_REDIAL  */
        /******* Deprecated and obsoleted sub-structures *******/
        IISDN_ENA_D_DATA                 enable_d_data;        /* For L4L3mENABLE_D_CHANNEL */
        IISDN_TDM_DATA                   tdm_data;             /* For L4L3mSET_TDM */
        IISDN_SPYDER_CNFG                spyder_t_data;        /* For L4L3mCONFIG_SPYDER_T */
        IISDN_ENA_SCATTER                enable_scatter;       /* For L4L3mENABLE_SCATTER */
        /******* End of deprecated and obsoleted sub-structures *******/
    } data;
} L4_to_L3_struct;

/* Data structure used to send control messages from IISDN Controller to User */
typedef struct
{
#ifdef RELEASE_8
    IISDNu16bit   lapdid;
    IISDNu8bit    msgtype;            /* L3L4mXXXXXX */
    IISDNu8bit    iface;              /* Non-Facility Signalling */
    IISDNu16bit   L4_ref;             /* Selected by L4. 0xFFFF if unknown */
    IISDNu16bit   call_ref;           /* Selected by L3. 0 if not for call */
    IISDNu8bit    bchannel;           /* 0 => not valid */
    IISDNu8bit    pad[3];
#else
    IISDNu8bit    lapdid;
    IISDNu8bit    msgtype;            /* L3L4mXXXXXX */
    IISDNu16bit   L4_ref;             /* Selected by L4. 0xFFFF if unknown */
    IISDNu16bit   call_ref;           /* Selected by L3. 0 if not for call */
    IISDNu8bit    bchannel;           /* 0 => not valid */
    IISDNu8bit    iface;              /* Non-Facility Signalling */
#endif
    IISDNu32bit   bchannel_mask;      /* Bit mask used for multi-rate
                                          1 => In-Service; 0 => Out-of-Service
                                          B channels numbered [1..24]
                                          bit => (1 << channel_num). This is the
                                          slot map for multi-rate calls 
                                          Normally set to 0x00FFFFFF for all 24
                                          channels.  Channel 1 being the lsb and 
                                          24 the msb */
    IISDNu16bit   lli;                /* Logical link or DLCI See IISDN_LLI_ */
    IISDNu16bit   data_channel;       /* Data Stream [0..255] or xFFFF */
    union
    {
        IISDN_SETUP_DATA                 setup_data;         /* For L3L4mSETUP_IND */
        IISDN_PROGRESS                   progress_data;      /* For L3L4mPROGRESS */
        IISDN_AL_CON_DATA                al_con_data;        /* For L3L4mALERTING; L3L4mCONNECT */
        IISDN_CLR_DATA                   clr_data;           /* L3L4mDISCONNECT,L3L4mCLEAR_REQUEST, L3L4mCLEAR_WITH_RESTART_REQUEST */
        IISDN_USER_INFO                  user_data;          /* L3L4mUSER_INFO/L3L4mX25_INTERRUPT */
        IISDN_CONTINUE_DATA              continue_data;      /* L3L4mCONTINUE for additional IEs*/
        IISDN_PROTOCOL_STAT              pcol_stat_data;     /* For L3L4mPROTOCOL_STATUS */
        IISDNu8bit                       error_code;         /* For L3L4mERROR; L3L4errXXXX */
        IISDNu8bit                       error_info[2];      /* For L3L4mERROR; L3L4errXXXX, L4L3mXXX */
        IISDNu8bit                       b_channel_status;   /* For L3L4mB_CHANNEL_STATUS, L3L4mCAS_STATUS (Use IISDNbcsXXXX to interpret) */
        IISDN_LINE_STATUS                line_status;        /* For L3L4mLINE_STATUS */
        IISDNu16bit                      compare_status;     /* For L3L4mCOMPARE_STATUS, Length of compare if it worked, Index of first different byte if fail */
        IISDN_HARDWARE_DATA              hardware_data;      /* For L3L4mHARDWARE_STATUS */
        IISDN_CONFIG_DATA                config_data;        /* For L3L4mCONFIGURATION_STATUS */
        IISDN_SOFTWARE_DATA              software_data;      /* For L3L4mSOFTWARE_STATUS */
        IISDN_PM_RSP_DATA                pm_rsp_data;        /* For L3L4mPM_RESPONSE */
        IISDN_PM_ALERT_DATA              pm_alert_data;      /* For L3L4mPM_ALERT  */
        IISDN_L2_MTP2_STATS              stat_data;          /* For L3L4mL2_STATS */
        IISDN_SUSP_RES_DATA              sus_res_data;       /* For Suspend/Resume messages */
        IISDN_BONDING_STATUS             bonding_status;     /* For L3L4mBONDING_STATUS, L3L4mADD_BONDING */
        IISDN_RAW_QDATA                  raw_q931_data;      /* Appended Undecoded Q.931 msg */
        IISDNu16bit                      billing_change_status; /* L3L4mBILLING_STATUS, IISDNvbsXXX */
        IISDN_ANI_DATA                   ani_data;           /* For L3L4mANI */
        IISDN_UNIVERSAL                  universal;          /* For L3L4mUNIVERSAL */
        IISDN_SETUP_ACK                  setup_ack_data;     /* For L3L4mSETUP_ACK */
        IISDN_INFO_DATA                  info_data;          /* For L3L4mINFO */
        IISDN_ABCD_DATA                  abcd_data;          /* For L3L4mABCD_SIGNAL_DATA */
        IISDN_CALL_PROC_DATA             proc_data;          /* For L3L4mCALL_PROCEEDING */
        IISDN_BOARD_ID                   board_id;           /* For L3L4mBOARD_ID info on board */
        IISDN_SPID                       spid_data;          /* For L3L4mSPID */
        IISDN_REQ_DOWNLOAD_DATA          req_download_data;  /* For L3L4mREQ_DOWNLOAD */
        IISDN_DOWNLOAD_DATA              ack_download_data;  /* For L3L4mACK_DOWNLOAD, L3L3mACK_UPLOAD */
        IISDN_RELAY_STATS                relay_stats;        /* for L3L4mRELAY_STATS */
        IISDN_TSI_DATA                   tsi_data;           /* For L3L4mTSI_STATUS */
        IISDN_HOST_STATUS                host_data;          /* For L3L4mHOST_DEAD, L3L4mHOST_ALIVE */
        IISDN_X25_RESTART                x25_restart_data;   /* For L3L4mX25_RESTART_IND */
        IISDN_X25_RESET_IND              x25_reset_data;     /* For L3L4mX25_RESTART_IND */
        IISDN_GEOGRAPHIC_ADDRESS         geographic_address; /* For L3L4mGEOGRAPHIC_ADDRESS */
        IISDNu8bit                       reject_cause;       /* For L3L4mCALL_REJECT */
        IISDNu16bit                      hf_duration;        /* For L3L4mHOOKFLASH (in 10ms increments) */
        IISDN_R2_CALL_STATUS             r2_call_status;     /* For L3L4mSTATUS_IND in r2 only; for failed inbound R2 calls */
        IISDN_R2_ERROR                   r2_error;           /* For L3L4mERROR */
        IISDN_CPU_UTIL_DATA              cpu_util_data;      /* For L3L4mCPU_UTILIZATION_DATA */
        IISDN_BOARD_VERSION              board_version;      /* board version ejl */
        IISDN_CONTROL_DATA_RX            control_data_rx;    /* For L3L4mCONTROL_DATA; for direct to LAP Call Control */
        IISDN_CAS_SIGNALING_BITS_DATA    cas_signaling_bits; /* For L3L4mCAS_SIGNALING_BITS_STATUS */
        IISDNu8bit                       warning_code;       /* For L3L4mWARNING; L3L4wrnXXXX */
        IISDN_CALLER_ID_DATA             caller_id_data;     /* For L3L4mCALLER_ID_DETECTED */
        IISDN_RING_STATUS_DATA           ring_status_data;   /* For L3L4mRING_STATUS */
           
#ifdef RELEASE_8
        IISDNu8bit                       pad[L3L4_BUF_SIZE - 20];   /* Round up to 512 bytes */
#else                                
        IISDNu8bit                       pad[L3L4_BUF_SIZE - 16];   /* Round up to 512 bytes */
#endif
    } data;
} L3_to_L4_struct;

 
/*
 *      PCI, ISA, VME bus Interrupts
 *              The Netaccess Controller uses shared memory to pass interrupt status.
 * In normal operation, the Netaccess board will generate an interrupt whenever
 * it places a message in the L3->L4 queue.  It will generate an interrupt
 * reason of L3L4irsnNORMAL in this case.  This interrupt arrives at a
 * level/vector specified by specific hardware technical description and
 * register settings that are unique to a Netaccess controller bus type.
 *              On rare occasions, the Netaccess card may panic and generate an interrupt
 * with a L3L4irsnFATAL_ERROR interrupt reason.  Panics usually indicate
 * a fatal internal software error (a.k.a. a bug).  It would be VERY helpful
 * if the programmer could save the error_code, error_point, and error_arg
 * fields so the problem can be fixed.  
 *              Before the Netaccess card will generate another interrupt, the host
 * MUST write L3L4irsnNULL to clear the reason field.
 */
/* Interrupt reason codes */
#define L3L4irsnNULL                            0       /* Written by host to ack interrupt */
#define L3L4irsnPOWERUP_BOOT            1       /* Request for download of code.  Occurs at Powerup or VME Reset */
#define L3L4irsnREADY_TO_RUN            2       /* Used at successful boot time */
#define L3L4irsnFATAL_ERROR                     3       /* Fatal error occured (reload code) */
#define L3L4irsnNORMAL                          4       /* Normal processing interrupt; check the message queues */
#define L3L4irsnMAILBOX_REBOOT          5       /* Request for download of code.  Occurs when host writes to mailbox */
#define L3L4irsnDIAG_FAILURE            6       /* Diagnostics failed */
#define L3L4irsnCHECKSUM_FAILED         7       /* Alternate form of Checksum failed */
#define L3L4irsnMORE_CODE_NEEDED        8       /* Download next chunk of code */

/* ISA/VME Diagnostic failure codes */
#define IISDNerrLOCAL_RAM                       0x01    /* Local Ram Test Failed */
#define IISDNerrMFP                                     0x02    /* MFP Chip Test Failed */
#define IISDNerrTIMER_INT                       0x03    /* Timer/Interrupt Test Failed */
#define IISDNerrSHARED_RAM                      0x04    /* Shared Ram Test Failed */
#define IISDNerrMAILBOX                         0x05    /* Mailbox Interrupt Test Failed */
#define IISDNerrVME_INTR                        0x06    /* VME Interruptor Test Failed */
#define IISDNerrHDLC_RAM                        0x09    /* HDLC Ram Test Failed */
#define IISDNerrHDLC_CHIP_LOOP          0x0A    /* HDLC Loopback Failed */
#define IISDNerrHDLC_LOOP_RAM           0x0B    /* combo of last 2 tests */
#define IISDNerrFRAMER_RAM                      0x0C    /* Framer Ram Test Failed */
#define IISDNerrTSI                                     0x0E    /* TSI Test Failed (PRI-48 only) */
#define IISDNerrTSI_HDLC                        0x0F    /* TSI/HDLC Test (PRI-48 only) */
#define IISDNerrBAD_CHECKSUM            0x10    /* Code downloaded is bad */
#define IISDNerrDATA_INTERFACE_NOT_CONFIGURED           0x11    /* Host has not configured data interface */
#define IISDNerrUNKNOWN_DEVICEID        0x12    /* Unknown PCI/vendor ID */
#define IISDNerrDATA_INTERFACE_CONFIGURATION_ERROR  0x13    /* Unknown PCI/vendor ID */

/* Panic error codes.  The board panics and needs to be re-loaded. */
#define IISDNerrOUT_OF_TX_BUFFERS                               0xFF    /* L3->L4 queue is full */
#define IISDNerrERROR_LAPDID_OUT_OF_RANGE               0xFE    /* L2->L3 LAPD ID bogus */
#define IISDNerrERROR_LAPDID_NOT_ESTABLISHED    0xFD    /* L2->L3 interface bug */
#define IISDNerrINTERNAL_ERROR                                  0xFC    /* Internal error (bug) */
#define IISDNerrUNKNOWN_INTERRUPT                               0xFB    /* Bad interrupt vector */

/* interrupt status block at 0xS0FFF0 in shared RAM */
typedef struct
        {
        IISDNu8bit      interrupt_reason;       /* In L3L4irsnXXXX */
        IISDNu8bit      error_code;                     /* IISDNerrXXX. For L3L4irsnFATAL_ERROR */
        IISDNu16bit     error_point;            /* Internal diagnostic */
        IISDNu32bit     error_arg;                      /* Internal diagnostic */
        IISDNu32bit     timestamp;                      /* Incremented on each clock tic */
        IISDNu16bit     spare1;                         /* Unused.  Round up to 16 bytes. */
        IISDNu8bit      spare2;
        IISDNu8bit      vme_irq_level;          /* This is the level the VME card uses
                                                                                when it generates VMEbus interrupts.
                                                                                Default is level 3 interrupts.
                                                                                Host may change this before initializing
                                                                                IISDN_INT_VECTOR_REG_OFFSET
                                                                                0 => level 3, otherwise use 1 thru 7 */
        } IISDN_INTERRUPT_STATUS_BLOCK;

/* Start of interrupt status block at 0xS0FFF0 */
#define IISDN_INT_STATUS_BLOCK_OFFSET   (0x00FFF0)

/* IISDN 68k Logout data recorded if controller crashes. */
#define IISDN_LOG_STACKSZ       172
typedef struct
        {
        IISDNu16bit     vector_number;  /* MC68000 exception vector num */
        IISDNu16bit     pad;                    /* preserve longword alignment */
        IISDNu32bit     d0;
        IISDNu32bit     d1;
        IISDNu32bit     d2;
        IISDNu32bit     d3;
        IISDNu32bit     d4;                             /* Data Registers D0 thru D7 */
        IISDNu32bit     d5;
        IISDNu32bit     d6;
        IISDNu32bit     d7;
        IISDNu32bit     a0;
        IISDNu32bit     a1;
        IISDNu32bit     a2;
        IISDNu32bit     a3;                             /* Address Registers A0 thru A7 */
        IISDNu32bit     a4;
        IISDNu32bit     a5;
        IISDNu32bit     a6;
        IISDNu32bit     a7;
        IISDNu8bit      stack[IISDN_LOG_STACKSZ];       /* Exception Stack */
        } IISDN_LOGOUT;

/* IISDN PowerPC logout data */
typedef struct {
        IISDNu32bit r[32];                              /* SMI+0xff00 */
        IISDNu32bit exception_number;   /* SMI+0xff80 */
        IISDNu32bit msr;                                /* SMI+0xff84 */
        IISDNu32bit srr0;                               /* SMI+0xff88 */
        IISDNu32bit srr1;                               /* SMI+0xff8c */
        IISDNu32bit lr;                                 /* SMI+0xff90 */
        IISDNu32bit dsisr;                              /* SMI+0xff94 */
        IISDNu32bit bar;                                /* SMI+0xff98 */
        IISDNu32bit dar;                                /* SMI+0xff9c */
        IISDNu32bit stack[20];                  /* SMI+0xffa0 */
} IISDN_PPC_LOGOUT;

#define IISDN_LOGOUT_OFFSET     0X00FF00

#define NOT_FIRST_TIME 0x00
#define FIRST_TIME 0x01

/* Protocol Files are composed of a header followed by data (the protocol itself).
 * 
 * The header format is defined below; in the future it should be made part of the Boston 
 * Command Set internal data structures.
 * 
 * The main reason why we need a header is to add support to both R2 protocols and 
 * LEC protocols (SignallingType = IISDNsigtypeR2_CAS or IISDNsigtypeLEC_NETWORK).
 * These two families of CAS protocols used different data structures (IISDN_E1_CAS_R2_DATA or
 * IISDN_ROBBED_BIT_DATA), which have diffent formats. It's safer to have a common header
 * that identifies the type of protocol to be used then to rely on the size of the data file.
 * 
 * LEC protocols are sometimes referred to as Robbed-Bit protocols or T1 protocols,
 * even though these names refer to the TDM transmission structure used in the US, Canada
 * and Japan. 
 * 
 * The old constant defined in IISDN.H for LEC protocols is IISDNsigtypeRBS_PULSE
 * and can still be used.
 */

/* FileDescriptor: 
 * Upper byte specifies the type of file (protocol, coefficients, etc),
 * while lower byte contains type-specific data.
 */

#define IISDNfiletypePROTOCOL               0x00 // lower byte = signaling type (IISDNsigtypeXXX)
#define IISDNfiletypeQSLAC_COEFFICIENTS     0x01 // lower byte not yet used

#define MakeFileDescriptor(FileType, SubType)    (((FileType) << 8) | (SubType))
#define GetFileType(FileDescriptor)              (((FileDescriptor) >> 8) & 0xFF)
#define GetFileSubType(FileDescriptor)           (((FileDescriptor) >> 0) & 0xFF)

#define IISDN_PROTOCOL_HEADER_SIGNATURE          ((unsigned short) 0x3C4D)
#define IISDN_PROTOCOL_FILE_SIGNATURE            ((unsigned short) 0x1A2B)

typedef struct {
    unsigned short    Signature;        // a fixed value to idenfy protocol files
    unsigned short    HeaderSize;       // in case this structure grows in the future
    unsigned short    ProtocolVersion;  // helps identify the data structure layout; MSB=Major number; LSB=Minor number
    unsigned short    ProtocolSize;     // size of the data following the header
    unsigned short    FileDescriptor;   // Upper byte = file type; lower byte = signaling type.
    unsigned short    CheckSum;         // add the ASCII value of each data byte (skip the header)
} IISDN_PROTOCOL_HEADER_S;

#endif
